##########################################################################
#                                                                        # #
# Licensed Materials - Property of IBM                                   #   #
# Restricted Material of IBM                                             #######
# IBM Rational Test RealTime                                                   #
#  Copyright IBM Corp. 1999,2004 All Rights Reserved.                         #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.            #
#                                                                              #
################################################################################
##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Application Node -*-                             #
#                                                                              #
#                                                                              #
#  Script for Application Test Node.                                           #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   @OBJECTS       : object files                                              #
#   $TARGETDIR     : directory of the Target Package                           #
#   $TESTPROGRAM   : executable extension                                      #
#   $LDFLAGS       : options for the linker                                    #
#   $LIBRARIES     : libraries for the linker                                  #
#   $EXECMODE      : execution mode ('NORMAL' or 'DEBUG')                      #
#   $ARGUMENTS     : argument for the generated program execution.             #
#   $BINDIR        : directory of the studio/tracer binaries.                  #
#   $ATTOLHOME     : attol studio intallation directory.                       #
#   $ATTLOOPDETECT : activate loop detection utility for TDF generated file.   #
#   $ATTLOOPDETECTFLAGS: options for attloop                                   #
#   $ATTSPLIT      : active de decoupage des traces.                           #
#   $ATTSPLITSIZE  : taille (Kb) des fichier decoupes.                         #
#   $ATTSPLITPREFIX: prefix des fichiers generes.                              #
#                                                                              #
################################################################################
#$Id: Application.pl,v 1.102 2007/10/03 17:40:03 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] unless($main::TestRTCC); #environment and global variables.

# Miscellaneous functions 
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";

# Target package related functions
require $SCRIPTDIR . "SplitJvmpi.pl" unless($main::TestRTCC);
require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);

# Target-dependant functions
require $TARGETDIR . "/cmd/atl_link.pl";
require $TARGETDIR . "/cmd/atl_exec.pl";

################################################################################
sub Application
{
 my %mode=('default',0,'fifo',1,'file',2,'user',3,'ignore',4);
 my $ada_mode = 0;
 my $status = 0;
 # Here is the common to all components traces file name
 if ( $TDP_LANGUAGE=~/^ADA/ )
 {
  $ada_mode = 1;
  $OUTFILE = $ATL_OUT_FILE;

  if ( $ATL_MAIN_PROCEDURE eq "" )
  {
    die &PerlTranslate("Application.pl", "Error: please update Execution settings with the name of the main procedure.\n");
  }
 } else {
  $OUTFILE = "$ATL_OUT_FILE";
 }
 $OUTFILE =~ s/\\/\\\\/;

 my $java_mode=0;
 if ( $TDP_LANGUAGE eq "JAVA" )
 {
  $java_mode = 1 ;
 }

 if($MISRA_MODE && @XOB) {
   my $res;
   if ($STAGE =~ /e/ ) {
      $res=codeChecker_link() if($STAGE =~ /l/ );
   }else{
      $res=codeChecker_link();
   }
 }

#----------------------------------------------------------------------------
## Link pass
 if ( $STAGE =~ /l/ )
 {
  $ATKMODE=1 if (@OTIFILES);
  #--------------------------------------------------------------------------
  ## do not forget object file of target package.
  if ( ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || $ATKMODE || $ATS_PROBE_CUSTOM) &&
       ($TDP_LANGUAGE eq "C" || $TDP_LANGUAGE eq "CPP") &&
       ( $ATL_RUNTIME_FORMAT ne "NONE"))
  {
    if (MakeTargetPackageObject()) {
      return 1;
    }
    if($ATL_RUNTIME_FORMAT eq "OBJ") {
       push( @OBJECTS, GetTargetPackageLib($OBJEXT) );
    } else {
       push( @ADDITIONAL_LIBRARIES, GetTargetPackageLib($LIBEXT) );
    }
  }

#link with atlprobe.o for probe
 
  if ($ATS_PROBE_CUSTOM)
  {

      if ($ATS_PROBE_CUSTOM_TYPE==$mode{ignore})
      {
	  #none additional object files needed      
      }    
      else
      {
          ## Do not forget the OBJ file in output log
          open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Application.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
	  
          push( @OBJECTS, "$INTDIR" . "$PATH_SEPARATOR" . "atlprobe.$OBJEXT");
          print LOG "OBJ $INTDIR/atlprobe.$OBJEXT\n";

	  if (($ATS_PROBE_CUSTOM_TYPE==$mode{fifo}) || ($ATS_PROBE_CUSTOM_TYPE==$mode{file}) || ($ATS_PROBE_CUSTOM_TYPE==$mode{user}))
	  {
	      push( @OBJECTS, "$INTDIR" . "$PATH_SEPARATOR" . "probecst.$OBJEXT");
              print LOG "OBJ $INTDIR/probecst.$OBJEXT\n";
	  }
      
          close ( LOG );
      }
  }


  if ($ada_mode)
  {
      print &PerlTranslate("Application.pl", "Linking %1...\n", "%1 is file to be linked", $TESTPROGRAM_ABS);
      $status = atl_link ( $TESTPROGRAM_ABS, \@OBJECTS, $LDFLAGS,
			   \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES );      
  }
  elsif ($java_mode)
  {
      # @OBJECTS is not used (not filled by the Source.pl because it is an
      # automatic compilation process
      if ($JAVA_BUILDJARFILE) {
        CheckOrCreateManifestFile();
        print &PerlTranslate("Application.pl", "Creating %1...\n", "%1 is file to be created", $JAVA_JARFILENAME);
        $status = atl_link ( $JAVA_JARFILENAME, \@OBJECTS, "\"$JAVA_MANIFEST\"",
                             \@JAVA_JAR_OTHER_DIR, $INTDIR );
      } else {
        return 0;
      }
  }
  else
  {
      print &PerlTranslate("Application.pl", "Linking %1...\n", "%1 is file to be linked", $TESTPROGRAM);
      $status = atl_link ( $TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                           \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
  }
  EndStep($status);
  return($status) if ( $status );

  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Application.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "EXE $TESTPROGRAM\n";
  close( LOG );
  
 }

#----------------------------------------------------------------------------
## Execution
 if ( $STAGE =~ /e/ )
 {
    if ( $java_mode ) {
      ($JAVA_MAIN_CLASS) or die &PerlTranslate("Application.pl", "Error: please update Execution settings with the name of the main class to be run.\n");
      print &PerlTranslate("Application.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $JAVA_MAIN_CLASS, $ARGUMENTS);
      $status = AtlExec ($JAVA_JARFILENAME, $JAVA_MAIN_CLASS, $ARGUMENTS);
    } elsif ($ada_mode)
    {
      print &PerlTranslate("Application.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $TESTPROGRAM_ABS, $ARGUMENTS);
      $status = AtlExec ($TESTPROGRAM_ABS, $OUTFILE, $ARGUMENTS);
    }
    else
    {
      print &PerlTranslate("Application.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $TESTPROGRAM, $ARGUMENTS);
      $status = AtlExec ($TESTPROGRAM, $OUTFILE, $ARGUMENTS);
    }
    EndStep($status);
    $status=0; #status is shown but not used by studio

    $ATKMODE=1 if(@OTIFILES);

    if ( ( $ATCMODE || $ATTMODE || ( $ATPMODE && !$java_mode ) ||
           $ATQMODE || $ATKMODE ) )
    {
       # when socket uploader is used SplitTraces check created files and make the log 
          #in Multi-process the spt file name has changed as xxxxx.pid.spt instead of xxxxx.spt
          if($ATL_MULTI_PROCESS){
          	 my ($dir,$name,$ext) = SplitFileName($OUTFILE);
          	 opendir(DIR,"$dir") || die &PerlTranslate("Application.pl", "Error: unable to open directory %1.\n","%1 directory name",$dir);
          	 while($f=readdir(DIR)) {
          	 	SplitTraces ($dir.$f) if($f=~ /$name\..*\.$ext/);
          	 }
          	 closedir(DIR);
          } else {
	  	SplitTraces ($OUTFILE); # LOG are made inside SplitTraces
	  }
	  #---------------------------------------------------------------------
	  ## Coverage check ...
	  if ( $ATCMODE && ( ! $ATTMODE || ! $ATT_ON_THE_FLY ) )
	  {
	      # TBD: A verifier pour Ada
	      if ( ! $used[$tio] )
	      {
		  print &PerlTranslate("Application.pl", "No Coverage results found.\n");
		  #exit(1);
	      }
	      
	      # Postpro Coverage (TPM...)
	      atc_postpro($ATC_TIO_NAME,""); # no CIO in this node
	  } 
	  
	  #---------------------------------------------------------------------
	  ## Manage TDF Traces ...
	  if ( $ATTMODE && ! $ATT_ON_THE_FLY )
	  { # No attsplit nor attloop on the fly
	      if ( ! $used[$tdf] )
	      {
		  print &PerlTranslate("Application.pl", "No Tracing results found.\n");
		  #exit(1);
	      }
	      PostAttManage ($TDFFILE);
	  } #ATTMODE==1
	  
	  #---------------------------------------------------------------------
	  ## Memory Profiling check ...
	  if ( $ATPMODE && ! $java_mode )
	  {
	      if ( ! $used[$tpf] )
	      {
		  print &PerlTranslate("Application.pl", "No Mempro results found.\n");
		  #exit(1);
	      }
	  } #ATPMODE==1
	  
	  #---------------------------------------------------------------------------
	  ## Performance Profiling check ...
	  if ( $ATQMODE )
	  {
	      if ( ! $used[$tqf] )
	      {
		  print &PerlTranslate("Application.pl", "No Perfpro results found.\n");
		  #exit(1);
	      }
	  } #ATQMODE==1
	  
	  #---------------------------------------------------------------------------
	  ## TBD: Contract Check check ...
	  if ( $ATKMODE )
	  {
	      # TBD: PostAtkManage ($TDFFILE);
	  } #ATKMODE==1
	
    } #ATCMODE==1 || ATTMODE==1 || ATPMODE==1 || ATQMODE==1 || $ATKMODE==1
    
 } #STAGE==execution

 if ($STAGE =~ /r/ )
 {

    if ( $ATPMODE && $TDP_LANGUAGE eq "JAVA" ) 
    {
	SplitJvmpiTraces ();
    }
   
    if (($ATS_PROBE_CUSTOM) && ($ATS_PROBE_CUSTOM_TYPE!=$mode{ignore}))
    {
	#First, get Riofilenames 
	my $tmp="$INTDIR" . "$PATH_SEPARATOR" . "atlprobe.pts";
	
	my @listrio="";
	open ( LOG, "$tmp" ) or die &PerlTranslate("Application.pl", "Cannot open log file.\n");
	while (<LOG>)
	{
	    if (/atl_dump_inst/)
	    {
		my ($tmp,$temp,$rioname) = ($_ =~ /atl_dump_inst\(([^,]+),([^,]+),\s*\"([^,]+?)\s*\"/ );
		#Rio files are generated in the WOKSPACE DIR
		push(@listrio,$rioname);
	    }
	}
	close (LOG);	
	
	$ENV{'PROBE'}= "1";
	$ENV{'ATS_DIR'}=$BINDIR;
	my $status=SystemP("$BINDIR"."$PATH_SEPARATOR"."atsmerge ". join(" ",@listrio) . "  \"$INTDIR". "$PATH_SEPARATOR" . "atlprobe.tdc\" " . "\"-RDD=$OUTDIR". "$PATH_SEPARATOR" . "$NAME"."_p.rdd\"");
	if ($status>3)
	{
	    print &PerlTranslate("Application.pl", "Failed.\n");
	    exit (1);
	}
	open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Application.pl", "Cannot open log file.\n");
	while(@listrio)
	{
	    my $rio = pop(@listrio);
	    print LOG "RIO $rio\n";
	}	
	print  LOG "XRD $OUTDIR"."$PATH_SEPARATOR"."$NAME"."_p.xrd\n";
# no event report when instrumentation. tsf and tdf disturb the report. So suppress them
	if($ATTMODE || $ATPMODE || $ATQMODE)
	{
	   unlink($OUTDIR."/$NAME"."_p.tsf");
	   unlink($OUTDIR."/$NAME"."_p.tdf");
	}
	else
	{
	   print  LOG "TSF $OUTDIR"."$PATH_SEPARATOR"."$NAME"."_p.tsf\n";
	   print  LOG "TDF $OUTDIR"."$PATH_SEPARATOR"."$NAME"."_p.tdf\n";
	}
	close( LOG );
        my $filename = $WORKSPACE_PATH.$PATH_SEPARATOR.$OUTDIR.$PATH_SEPARATOR.$NAME."_p.xdf";
        my $filename2 = $WORKSPACE_PATH.$PATH_SEPARATOR.$NAME."_p.pts";
	SystemP("$BINDIR"."$PATH_SEPARATOR"."TestRTperl -I$ATTOLHOME"."$PATH_SEPARATOR"."lib"."$PATH_SEPARATOR"."perl"." $SCRIPTDIR"."ats2pts.pl"." \"$ARGV[0]\" \"$filename\""." \"$filename2\" ".$ATS_REPLAY_CMDLINE_ARGS);
    }
    
    if(@OTIFILES) 
    {
	  my $wait_time = 2;
	  if ($SOCKET_UPLOAD)
	    {
	      my $t=5;
	      while($t && !(-f "trtpd.log")){
		sleep($wait_time);
		$t--;
	      }	
	    }
	  # not used since studio generate atlendtest.txt and trtpd generate trtpd.log allowing to SplitTraces to wait for the end of the test. thanks to "tdp end" token generated by the runtime
	  #wait_close() if (($ATTMODE && $ATT_ON_THE_FLY) || ($SOCKET_UPLOAD)); #still there for required special conf

	  print &PerlTranslate("Application.pl", "Test Report Generation...\n");

	  $otslist = "";
	  foreach $ots ( @OTSFILES )
	  {
	     $otslist .= "\"$ots\" ";
	  }
	  $status = app_postpro ($TDFFILE, $otslist, $XRDFILE);
	  if ($status != 0 ) { print &PerlTranslate("Application.pl", "Failed.\n"); exit(1); }
	  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("ObjecTest.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
	  print  LOG "XRD $XRDFILE\n";
	  close( LOG );
   	
    }
 }#STAGE==Report Generation
 print &PerlTranslate("Application.pl", "Done.\n");
 return $status;
}
################################################################################
# Attol ObjecTest postprocessor execution
# Arguments:
#    $tdf    : traces file
#    $ots    : static correspondence files (space-separated list)
#    $xrd    : generated AML report
# Return value: atopostpro return value
# Required global variables:
#    $BINDIR
# Used global variables:
################################################################################
sub app_postpro #($$$)
{
   my ($tdf,$ots,$xrd) = @_;

   # command
   my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "atopospro -ots $ots -tdf \"$tdf\" -xrd \"$xrd\"";

   # execution
   my $Status = SystemP ($Command);
   return $Status;
}

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

CheckIntDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
exit (Application());
}
1;

# That's all Folks.
################################################################################








