#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                  -*-  Batch TDP Library preparation -*-                      #
#                                                                              #
#                                                                              #
#  Script used by javic (batch compilation launcher)                           #
#                                                                              #
#                                                                              #
################################################################################
#$Id: BatchJavaDefaults.pl,v 1.17 2007/06/06 16:23:05 rfrance Exp $
################################################################################

################################################################################
#
# Using this settings outside of javic:
# After a copy of this file, just select and uncomment the following variables
#
################################################################################

# Select the components you want to use:
# $ATJMODE=1; # Component Testing for Java
# $ATCMODE=1; # Coverage
# $ATTMODE=1; # Trace
# $ATQMODE=1; # Performance Profiling

################################################################################
#
# These defaults settings are used by the attolcc launcher in order to compile
# the Target Deployment Port Library.
#
# To Override these defaults settings, copy this file in a directory,
# modify whatever you need and then the ATL_OVER_SET environment variable to
# your modified copy of the file.
#
################################################################################

# General default settings:
$ATL_THREADS_MAX=64;
$ATL_BUFFER_SIZE=16384;
$ADDRESS="127.0.0.1";
$UPLOADER_PORT=7777;
$ATL_RUN_GC_AT_EXIT=1;

# Component Testing for Java flags:
$ATJ_CHECK_STUB=0;
$ATJ_DISPLAY_STUB=0;

# Trace default settings:
$ATT_ON_THE_FLY=1;
$ATT_PARTIAL_DUMP=0;
$ATT_TIMESTAMP=1;
$ATT_HEAP_SIZE=1;
$ATT_THREAD_INFO=1;
$ATT_RECORD_MAX_STACK=1;

sub ShowSettingsList
{
  print &PerlTranslate("BatchJavaDefaults.pl",
    "List of Products.java settings:\n" .
    "  --atl_threads_max=([1-9][0-9]*) (default 64)\n" .
    "      Maximum number of threads at the same time\n" .
    "  --atl_buffer_size=([1-9][0-9]*) (default 16384)\n" .
    "      Size of the Dump Buffer in bytes\n" .
    "  --address=(.*) (default 127.0.0.1)\n" .
    "      Address of the Socket Trace Receiver Host\n" .
    "  --uploader_port=([1-9][0-9]*) (default 7777)\n" .
    "      Port listened by the Socket Trace Receiver Host\n" .
    "  --atl_run_gc_at_exit=([0-1]) (default 1)\n" .
    "      Try to run finalizers invoking the Garbagge Collector at exit\n" .
    "  --atj_check_stub=([0-1]) (default 0)\n" .
    "      Check Component Testing for Java stubs\n" .
    "  --atj_display_stub=([0-1]) (default 0)\n" .
    "      Display Component Testing for Java stubs in Runtime Tracing\n" .
    "  --att_on_the_fly=([0-1]) (default 1)\n" .
    "      If set to 1, implies that each tdf lines are flushed immediatly\n" .
    "      in order to be read on-the-fly by Runtime Tracing\n" .
    "  --att_partial_dump=([0-1]) (default 0)\n" .
    "      Partial Message Dump is on if set to 1 in Runtime Tracing\n" .
    "  --att_timestamp=([0-1]) (default 1)\n" .
    "      If 1 record and display Time Stamp in Runtime Tracing\n" .
    "  --att_heap_size=([0-1]) (default 1)\n" .
    "      Record and Display Current Heap Size in Runtime Tracing\n" .
    "  --att_thread_info=([0-1]) (default 1)\n" .
    "      Record and Display Thread Information in Runtime Tracing\n" .
    "  --att_record_max_stack=([0-1]) (default 1)\n" .
    "      Record and Display Max Stack in a note in Runtime Tracing\n" .
    "  --help\n" .
    "      Print this help\n");
}

sub IsASetting #($)
{
  my $arg = shift;
  $_ = uc($arg);
  SWITCH: {
    /^--HELP$/ &&
      do { ShowSettingsList();
           return 1; } ;
    /^--ATL_THREADS_MAX=([1-9][0-9]*)$/ &&
      do { $ATL_THREADS_MAX=$1;
           return 1; } ;
    /^--ATL_BUFFER_SIZE=([1-9][0-9]*)$/ &&
      do { $ATL_BUFFER_SIZE=$1;
           return 1; } ;
    /^--ADDRESS=(.*)$/ &&
      do { $ADDRESS="$1";
           return 1; } ;
    /^--UPLOADER_PORT=([1-9][0-9]*)$/ &&
      do { $UPLOADER_PORT=$1;
           return 1; } ;
    /^--ATL_RUN_GC_AT_EXIT=([0-1])$/ &&
      do { $ATL_RUN_GC_AT_EXIT=$1;
           return 1; } ;

    /^--ATJ_CHECK_STUB=([0-1])$/ &&
      do { $ATJ_CHECK_STUB=$1;
           return 1; } ;
    /^--ATJ_DISPLAY_STUB=([0-1])$/ &&
      do { $ATJ_DISPLAY_STUB=$1;
           return 1; } ;

    /^--ATT_ON_THE_FLY=([0-1])$/ &&
      do { $ATT_ON_THE_FLY=$1;
           return 1; } ;
    /^--ATT_PARTIAL_DUMP=([0-1])$/ &&
      do { $ATT_PARTIAL_DUMP=$1;
           return 1; } ;
    /^--ATT_TIMESTAMP=([0-1])$/ &&
      do { $ATT_TIMESTAMP=$1;
           return 1; } ;
    /^--ATT_HEAP_SIZE=([0-1])$/ &&
      do { $ATT_HEAP_SIZE=$1;
           return 1; } ;
    /^--ATT_THREAD_INFO=([0-1])$/ &&
      do { $ATT_THREAD_INFO=$1;
           return 1; } ;
    /^--ATT_RECORD_MAX_STACK=([0-1])$/ &&
      do { $ATT_RECORD_MAX_STACK=$1;
           return 1; } ;
    /^--.*$/ &&
      do { print &PerlTranslate("BatchJavaDefaults.pl", "Unknown setting: %1\n", "%1 is the unknown setting", $arg);
           ShowSettingsList();
           exit 1; } ;
  } #SWITCH
  return 0;
}

sub ParseOtherSettings #($)
{
  my $i = shift;

  while ($ARGV[$i]) {
    if ( ! IsASetting($ARGV[$i]) ) {
      do { print &PerlTranslate("BatchJavaDefaults.pl", "Unknown option: %1\n", "%1 is the unknown option", $ARGV[$i]);
           ShowSettingsList();
           exit 1; } ;
    }
    $i++;
  }
}

1;

