#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                  -*-  Batch TDP Library compilation -*-                      #
#                                                                              #
#                                                                              #
#  Script used by attolcc (batch compilation launcher), at least               #
#                                                                              #
#                                                                              #
# Required environment variables:                                              #
#   ATTOLHOME, ATLTGT
#                                                                              #
################################################################################
#$Id: BatchSplit.pl,v 1.22 2007/10/25 12:39:47 rfrance Exp $
################################################################################
# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
# This script depends on the followings (both studio'n Target Package)
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Split.pl";

$ATUMODE=1;
$ATSMODE=1;
$ATOMODE=1;
$ATKMODE=1;
$ATCMODE=1;
$ATTMODE=1;
$ATPMODE=1;
$ATQMODE=1;

my @SPTS = ();
my $filter;
my $log_index = 0, $i = 0;
foreach (@ARGV) {
  if (/-studio_log/) {
    $log_index = $i + 1;
  } else {
    if(/\.tio$/){
       $ENV{'ATC_TRACES'}=$_;
    } elsif (/\.tpf$/){
       $ENV{'ATP_TRACES'}=$_;
    } elsif (/\.tqf$/){
       $ENV{'ATQ_TRACES'}=$_;
    } elsif (/\.tdf$/){
       $ENV{'ATT_TRACES'}=$_;
    } elsif (/\.rio$/){
       if(-f $_){
          push(@SPTS, $_);
       }else{
          $ENV{'ATU_TRACES'}=$_;
          $used[$rio]=1;
       }
    } elsif (/\+filter$/){
       $filter=1;
    } else {
       push(@SPTS, $_);
    }
  }
  $i += 1;
}

if($filter){
  if (($ENV{'ATLTGT'}) &&  (-r "$ENV{'ATLTGT'}/cmd/decode.pl")){
  	require "$ENV{'ATLTGT'}/cmd/decode.pl";
  }else{
        die &PerlTranslate("BatchSplit.pl", "decode option require decode.pl in the TDP. Not found\n","");
  }
}

if ($log_index) {
  if ($ARGV[$log_index] ne "") {
    $LOGFILENAME = $ARGV[$log_index];
    $log_file_to_be_removed = 0;
  } else {
    $LOGFILENAME = "batchsplit.log";
    $log_file_to_be_removed = 1;
  }
} else {
  $LOGFILENAME = "batchsplit.log";
  $log_file_to_be_removed = 1;
}

my $multi_spt = 0;
if (scalar(@SPTS) == 0) {
  if (! -f "$ATL_OUT_FILE") {
    die &PerlTranslate("BatchSplit.pl", "BatchSplit.pl needs an <atlout.spt> file on the command line in order to work...\n");
  }
  push(@SPTS, $ATL_OUT_FILE);
} elsif (scalar(@SPTS) > 1) {
  $multi_spt = 1;
}

#----------------------------------------------------------------------------

@existed = (0, 0, 0, 0, 0, 0);

if (! $multi_spt) {
  # Unit Testing/System Testing default settings:
  $RIOFILE="atltest.rio";
  if ($ENV{'ATU_TRACES'} ne "") {
    $RIOFILE=$ENV{'ATU_TRACES'};
  } elsif ($ENV{'ATS_TRACES'} ne "") {
    $RIOFILE=$ENV{'ATS_TRACES'};
  }
  if ( -f $RIOFILE ) { $existed[$rio] = 1; }
}

# Coverage default settings:
$ATC_TIO_NAME="attolcov.tio";
if ($ENV{'ATC_TRACES'} ne "") {
  $ATC_TIO_NAME=$ENV{'ATC_TRACES'};
}
# Memory Profiling default settings:
$ATP_TPF_NAME="purifylt.tpf";
if ($ENV{'ATP_TRACES'} ne "") {
  $ATP_TPF_NAME=$ENV{'ATP_TRACES'};
}
# Performance Profiling default settings:
$ATQ_TQF_NAME="quantifylt.tqf";
if ($ENV{'ATQ_TRACES'} ne "") {
  $ATQ_TQF_NAME=$ENV{'ATQ_TRACES'};
}
# ObjectTesting/Trace default settings:
$TDFFILE="atltrace.tdf";
if ($ENV{'ATO_TRACES'} ne "") {
  $TDFFILE=$ENV{'ATO_TRACES'};
} elsif ($ENV{'ATT_TRACES'} ne "") {
  $TDFFILE=$ENV{'ATT_TRACES'};
}

if ( -f $ATC_TIO_NAME ) { $existed[$tio] = 1; }
if ( -f $ATP_TPF_NAME ) { $existed[$tpf] = 1; }
if ( -f $ATQ_TQF_NAME ) { $existed[$tqf] = 1; }
if ( -f $TDFFILE      ) { $existed[$tdf] = 1; }

#----------------------------------------------------------------------------

my $first_spt = 1;

foreach $spt (@SPTS) {
  if (! -f $spt) {
    die &PerlTranslate("BatchSplit.pl", "%1 does not exist\n", "%1 is what does not exist", $spt);
  }
  my ($INTDIR,$file,$ext) = SplitFileName ($spt);
  if ( $INTDIR eq "" ) {
    $INTDIR = ".";
  }
  if ($multi_spt) {
    $RIOFILE="$file.rio";
    if ( -f $RIOFILE ) { $existed[$rio] = 1; }
    $tdf_file_mode = "a" if (! $first_spt);
  }
  # Let's do it
  decode($spt) if($filter);
  if (SplitTraces($spt) != 0) {
    exit(1);
  }
  if ( $multi_spt && ! $used[$rio] && ! $existed[$rio] ) { unlink ($output_files[$rio]); }
  $first_spt = 0;
}

if ( ! $multi_spt && ! $used[$rio] && ! $existed[$rio] ) { unlink ($output_files[$rio]); }
if ( ! $used[$tio] && ! $existed[$tio] ) { unlink ($output_files[$tio]); }
if ( ! $used[$tpf] && ! $existed[$tpf] ) { unlink ($output_files[$tpf]); }
if ( ! $used[$tqf] && ! $existed[$tqf] ) { unlink ($output_files[$tqf]); }
if ( ! $used[$tdf] && ! $existed[$tdf] ) { unlink ($output_files[$tdf]); }

if ($log_file_to_be_removed) {
  unlink ("$LOGFILENAME");
}

0;
# That's all Folks.
################################################################################
