#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2008 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                                -*-  Basics -*-                               #
#                                                                              #
#                                                                              #
################################################################################
require $SCRIPTDIR . "Translate.pl";
require $SCRIPTDIR . "version.pl";
use File::Copy;
use File::Spec;

################################################################################
# Clean file names from exotic char to be able to generate package/function names
#
# Arguments:
#   input name
# Return value:
#   output name
################################################################################
sub clean_name_from_exotic_char #($)
{
    my $instr = shift;
    my $outstr = "";
    my $i;
    my $j;
    my $first;
    my $l;
    my $insize;    

    $outstr = $instr;
    $outstr =~s/((?![\w_])~)+/_/g;
    $outstr =~s/\s+/_/g;
    $outstr =~s/^([_\d]*)//g;
    $outstr =~s/_+/_/g;
    $outstr =~s/(_+)$//o;
    return $outstr;
}

################################################################################
# build link file name "adalink.alk" using COMPLETE_INCLUDE_DIR_LIST
# Arguments:
#   none
# Return value:
#   0 if ok, 1 if COMPLETE_INCLUDE_DIR_LIST isn't set or if attolalk isn't happy
################################################################################
sub BuildAdaLinkFile
{
  #prefer to use ALK_FILE_NAME link file rather than COMPLETE_INCLUDE_DIR_LIST
  if ( $USER_ALK_FILE )
  {
    if ( ! -f $USER_ALK_FILE )
    {
      print &PerlTranslate("Fcts.pl", "Error: User Link File not found (%1)\n      Update your Compiler options settings.\n", "%1 is the expected link file name", $USER_ALK_FILE);
      return 1;
    }
    open ( ALK,">adalink.alk" ) or die &PerlTranslate("Fcts.pl", "Cannot create adalink.alk file: %1\n", "%1 is the reason given by the Operating System", $!);
    print  ALK "\"$USER_ALK_FILE\" LINK\n";
    close( ALK ); 
  }
  else
  {
    my $directories = "";
    my $has_directory = 0;
    foreach $D ( @COMPLETE_INCLUDE_DIR_LIST )
    {
      $has_directory = 1;
      $directories .= "\"$D\" ";
    }     
    if ( ! $has_directory )
    {
      print &PerlTranslate("Fcts.pl", "Error: missing include source directory, please update settings.\n");
      return 1;
    }   
    my $status;
    if($Ini{'ATTOLALK'}) {
     print "using custom ATTOLALK\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLALK'}";
     my @all_includes = @COMPLETE_INCLUDE_DIR_LIST;
     $status = attolalk("adalink.alk", \@GENERATION_CONDITION, \@all_includes);
    } else {
      $status = SystemP( $BINDIR . $PATH_SEPARATOR . "attolalk -s adalink.alk $directories" );     
    }
    if ( $status != 0 ) 
    { 
      return 1; 
    }
  }
  return 0;
}

################################################################################
# Extension of a file name
# Arguments:
#   $filename : name of the file to get the extension of
# Return value:
#   "" if no extension was found, extension (without dot) otherwise
################################################################################
sub GetExtension #($)
{
   my $filename = $_[0];
   if ($pos = rindex $filename,".")
   {
      return substr ($filename,$pos+1);
   }
   else
   {
      return "";
   }
}

################################################################################
# Returns the short path name of a long path name
# Arguments:
#   $filename : name of the file in a long path format
# Return value:
#   short path name
################################################################################
sub ShortPathName #($)
{
   my $filename = shift;
   TestRTUtil::ShortPathName($filename);
}
 
################################################################################
# Arguments:
#   $str : string which contains some patterns to be replaced
#   $regexp : regular expression defining the pattern to replace
#   $by : string to replace the regular expression above
# Return value:
#   The new string corresponding to $str where all the paterns $regexp have
#   been replaced by $by
################################################################################
sub LocaleStringReplace #($$$)
{
   my ($str, $regexp, $by) = @_;
   TestRTUtil::LocaleStringReplace($str, $regexp, $by);
}
 

################################################################################
# (Directory,Name,Extension) splitted from a file name
# Arguments:
#   $filename : name of the file to get the extension of
# Return value:
#   (Directory,Name,Extension)
################################################################################
sub SplitFileName #($)
{
   my $filename = shift;
   TestRTUtil::SplitFileName($filename);
}
 

##  my ($filename) = @_;

##  if (!$OSNAME) { $OSNAME = ( $^O=~/win32/i) ? "win32" : "unix"; }
##  if ( $OSNAME eq "win32" ) { $PATH_SEPARATOR = '\\'; }
## else { $PATH_SEPARATOR = '/'; }
    
##  my @file;
##  if ((($dotPos = rindex $filename,".") < 0) || ((($slashPos = rindex $filename,"$PATH_SEPARATOR") >= $dotPos)))
##  {
##    @file = ($filename =~ /^((?:.*[:\\\/])?)(.*)/);
##  } 
##  else
##  {  
##    @file = ($filename =~ /^((?:.*[:\\\/])?)(.*)\.(.*)/);
##  } 
##
##  return @file;
##}

###########################################################################
# splitArgs split a line on blanc space without splitting on space found between cot
sub splitArgs #($)
{
   my $text=shift;
   my @new = ();
   $text.=" "; # to have a \s+ at the end to match the last one.
   push(@new,$+) while $text =~ m{([^\s]*"[^\"\\]*(?:\\.[^\"\\]*)*")\s+|([^\s]+)\s+}g;
   return @new;
}

################################################################################
# Create given directory if it didn't exist.
# Arguments:
#   $dirname : name of the file to get the extension of
# Return value: 
#   none
################################################################################
sub CreateDirectory #($)
{
  my $dirname = shift ;

  use Cwd ;

  my @subdir = split( /\//, $dirname );

  if (!$#subdir)
  {
	@subdir = split( /\\\\/, $dirname );
	if (!$#subdir)
  	{	
        	@subdir = split( /\\/, $dirname );

  	}
  }
  
#TBD execution PC ???
# my $save_dir=getcwd();   becarefull with getcwd in japanese locale
#                          has strange behaviour

    use File::Path ;
    #!! Seems to not works on dirname="\\attila\c\Program Files\myDirectory"
    mkpath( $dirname, 0, 0777 );

}

################################################################################
# Clear log file
# Arguments:
#   $logfilename : name of the file to clear
# Return value: 
#   none
################################################################################
sub ClearLogFile #($)
{
   my $logfile  = shift ;

   unlink( $logfile );
}

################################################################################
# GetModificationTime
# Arguments:
#   $file1 : first file
# Return value: 
#   0 if file does not exist
#   else last modification time
################################################################################
sub GetModificationTime #($)
{
   my $file1 = shift;

   # This is the way to get last modification time : stat
   my ( $t ) = (stat($file1))[9];
   return ($t);
}

################################################################################
# IsOlderThan
# Arguments:
#   $file1 : first file
#   $file2 : second file
# Return value: 
#   0 if file2 is older (or same modification time) than file1
#   1 if file1 is older than file2
################################################################################
sub IsOlderThan #($$)
{
  my ( $file1, $file2 ) = @_;

  # Use GetModificationTime sub
  my ( $t1 ) = GetModificationTime ( $file1 );
  my ( $t2 ) = GetModificationTime ( $file2 );

  # Compare and decide what is to be returned
  return (1) if ( $t1 < $t2 );
  return (0);
}

################################################################################
# Files renaming 
################################################################################
sub RenameFile
{
  my ($old, $new) = @_;
  if (rename ($old, $new))
  {
    return 1;
  }
  else
  {
    if (copy ($old, $new) && unlink ($old))
    {
      return 1;
    }
  }
  return 0;
}

################################################################################
# Files concatenation 
################################################################################
sub Cat #($$$)
{
   my ($one, $two, $result) = @_;

   open (RESULT, ">$result") or die &PerlTranslate("Fcts.pl", "Cannot open %1 in write mode", "%1 is the file that cannot be opened", $result);
   open (ONE, "<$one") or die &PerlTranslate("Fcts.pl", "Cannot open %1", "%1 is the file that cannot be opened", $one);

   while (<ONE>)
   {
      print RESULT;
   }

   close (ONE);
   open (TWO, "<$two") or die &PerlTranslate("Fcts.pl", "Cannot open %1", "%1 is the file that cannot be opened", $two);

   while (<TWO>)
   {
      print RESULT;
   }
   close (TWO);
   close (RESULT);
}

################################################################################
# FileDiff
# Arguments:
#   $file1 : first file
#   $file2 : second file
# Return value: 
#   0 if file1 and file2 contents are the same
#   1 if they differ or do not exist
################################################################################
sub FileDiff #($$)
{
  my ( $file1, $file2 ) = @_;
  
  # Open first file
  open ( FILE1, "<$file1" ) || return (1);
  # Open second one
  open ( FILE2, "<$file2" ) || close ( FILE1 ) && return (1);

  # Compare each line
  while ( $l1 = <FILE1> ) {
    $l2 = <FILE2>;
    if ( $l1 ne $l2 ) {
      # One line is different : close files and return 1
      close ( FILE1 );
      close ( FILE2 );
      return (1);
    }
  }
  # No lines differ : close files and return 0
  close ( FILE1 );
  close ( FILE2 );
  return (0);
}

################################################################################
# GetDate
# Return value:
#   Date in a Perl string and the C ctime() format
################################################################################
my $TrDays = "";
my $TrMonths = "";
sub GetDate
{
  local($time) = time;
  local($[) = 0;
  local($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);

  $TrDays = &PerlTranslate("Fcts.pl", "Sun,Mon,Tue,Wed,Thu,Fri,Sat", "List of Days of the Week; please keep order and separating commas without spaces") if ($TrDays eq "");
  $TrMonths = &PerlTranslate("Fcts.pl", "Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec", "List of Months in a Year; please keep order and separating commas without spaces") if ($TrMonths eq "");
 
  @DoW = split(/,/,$TrDays);
  @MoY = split(/,/,$TrMonths);

  # Determine what time zone is in effect.
  # Use GMT if TZ is defined as null, local time if TZ undefined.
  # There's no portable way to find the system default timezone.

  $TZ = defined($ENV{'TZ'}) ? ( $ENV{'TZ'} ? $ENV{'TZ'} : 'GMT' ) : '';
  ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
      ($TZ eq 'GMT') ? gmtime($time) : localtime($time);

  # Hack to deal with 'PST8PDT' format of TZ
  # Note that this cannot deal with all the esoteric forms, but it
  # does recognize the most common: [:]STDoff[DST[off][,rule]]

  if($TZ=~/^([^:\d+\-,]{3,})([+-]?\d{1,2}(:\d{1,2}){0,2})([^\d+\-,]{3,})?/){
      $TZ = $isdst ? $4 : $1;
  }
  $TZ .= ' ' unless $TZ eq '';

  $year += 1900;
  sprintf("%s %s %2d %2d:%02d:%02d %s%4d",
    $DoW[$wday], $MoY[$mon], $mday, $hour, $min, $sec, $TZ, $year);
  # This is the implied return value
}

################################################################################
# LanguageOf
# return the language corresponding to the given extension.
# Arguments:
#   $ext : extension to check.
# Environment variables:
#   ATL_C_EXT        : more C extension
#   ATL_CPLUSPLUS_EXT: more C++ extension
#   ATL_FORCE_CPLUSPLUS: force C++ source for C/C++ common extensions
# Return value: 
#   "C"   : language C
#   "CPP" : language C++
#   "ADA" : language ADA(95)
#   "ADA83": language ADA83 (value from TDP).
#   "JAVA": language JAVA
#   "ASM" : language ASM
#   ""    : if not recognized
################################################################################
sub LanguageOf #($$)
{
  my $ext=shift;
  my $default_lang = shift ;
  my $LANG = "";
  if ( $ext[0] ne '.' ) 
  {
    $ext = "." . $ext ;
  }
  my $atl_c_ext=$ENV{'ATL_C_EXT'};
  my $atl_cplusplus_ext=$ENV{'ATL_CPLUSPLUS_EXT'};
  my $src_ext=$Ini{'SRCEXT'};
  
  if ( $OSNAME eq "win32" ) {
    $src_ext=lc($src_ext);
    $atl_c_ext=lc($atl_c_ext);
    $atl_cplusplus_ext=lc($atl_cplusplus_ext);
    $ext = lc ($ext);
  }
  
  $_=$ext;
  SWITCH: {
      /\.$atl_c_ext$/ && do { $LANG="C"; last SWITCH; }; #must be first
      /\.$atl_cplusplus_ext$/ && do { $LANG="CPP"; last SWITCH; };#must be second
      /\.c$/   && do { if ( lc($ENV{'ATL_FORCE_CPLUSPLUS'}) eq "on" )
		       { $LANG="CPP"; } else { $LANG="C"; }
		       last SWITCH; };
      /\.i$/   && do { if ( lc($ENV{'ATL_FORCE_CPLUSPLUS'}) eq "on" )
		       { $LANG="CPP"; } else { $LANG="C"; }
		       last SWITCH; };
      /\.cpp$/ && do { $LANG="CPP"; last SWITCH; };
      /\.hpp$/ && do { $LANG="CPP"; last SWITCH; };
      /\.cxx$/ && do { $LANG="CPP"; last SWITCH; };
      /\.cc$/  && do { $LANG="CPP"; last SWITCH; };
      /\.rc$/  && do { $LANG="RC"; last SWITCH; };


      /\.ada$/  && do { $LANG=uc($Ini{'LANGUAGE'}); last SWITCH; };
      /\.ads$/  && do { $LANG=uc($Ini{'LANGUAGE'}); last SWITCH; };
      /\.adb$/  && do { $LANG=uc($Ini{'LANGUAGE'}); last SWITCH; };

      /\.java$/ && do { $LANG="JAVA"; last SWITCH; };
      /\.jav$/  && do { $LANG="JAVA"; last SWITCH; };
      
      /\.ptu$/  && do { $LANG="PTU"; last SWITCH; };
      (/\.otd$/ || /\.dcl$/ || /\.stb$/)
                && do { $LANG="OTD"; last SWITCH; };
      /\.otc$/  && do { $LANG="OTC"; last SWITCH; };

      /\.mdl$/  && do { $LANG="MDL"; last SWITCH; };
      /\.mat$/  && do { $LANG="MAT"; last SWITCH; };
      /\.csv$/  && do { $LANG="CSV"; last SWITCH; };
      /\.$src_ext$/  && do { $LANG=uc($Ini{'LANGUAGE'}); last SWITCH; };
      do {
	
	foreach $moreext (split(/,/,$Ini{'SRCEXTLIST'})) {
	  my $re = qr/\.$moreext/i;
	  if ($ext =~ $re) { $LANG=$Ini{'LANGUAGE'}; }
	}
		
	foreach $ASMEXT (split(/,/,$Ini{'ASMEXT'})) {
	  my $re = qr/\.$ASMEXT/i;
	  if ($ext =~ $re) { $LANG="ASM"; }
	}
	
      }
  } 
  if ( !$LANG )
  {
    $LANG = $default_lang;
  }
  if ( !$LANG )
  {
    $LANG="";
  }

  return $LANG;
}

################################################################################
# LanguageOf
# Arguments:
#   $source : le nom du fichier source
#   $default_lang : le langage par defaut si on ne peut pas conclure
# Return value:
#   "C"   : langage C
#   "CPP" : langage C++
#   "ADA" : langage ADA
#   "JAVA": langage JAVA
#   ""    : if not recognized
################################################################################
sub LanguageOfSource #($$)
{
  my ($source, $default_lang) = @_ ;

  my ($dir,$name,$ext) =SplitFileName($source);
  $lang = LanguageOf($ext,$default_lang);
  return $lang;
}

################################################################################
# DecodeIniFile
#   This routine will read the ini configuration file
# Arguments:
#   none
# Return value:
#   Ini associative table
################################################################################
sub DecodeIniFile
{
	open(INIFILE,"<$ENV{'ATLTGT'}/$INIFILE") || die &PerlTranslate("Fcts.pl", "Cannot open %1 ini file.\n", "%1 is the file that cannot be opened", $INIFILE);
      $EXPATH = $ENV{'PATH'};
	while(<INIFILE>)
	{
		chomp;
		next if(/^\s*$/);
		next if(/^\s*#/);
		my ($Name,$Value)=split(/=/,$_,2);
		$Value=~s/\&#22\;/\"/g;
		$Value=~s/^\"//;
		$Value=~s/\"$//;
		$Value=~s/\"\r$//;
		$Value=~s/\$ENV{\'(\w+)\'}/$ENV{$1}/g;
		$Value=~s/\$Ini{\'(\w+)\'}/$Ini{$1}/g;
		$Name=~s/^\s+//;
		$Name=~s/\s+$//;
		if($Name=~/^ENV_/i)
		{
			my $Key=$Name;
			$Key=~s/^ENV_//i;
			if($Key=~/^SET_IF_NOT_SET_/i)
			{
				my $Key2=$Key;
				$Key2=~s/^SET_IF_NOT_SET_//i;
				if(!defined($ENV{$Key2}))
				{
					$ENV{$Key2}=$Value;
				}
			}
			else
			{
				$ENV{$Key}=$Value;
			}
		}
		$Ini{$Name}=$Value;
	}
	close(INIFILE);

	# Convert to uppercase so that Ada can be written ADA or aDA etc.
	$Ini{'LANGUAGE'} = ucfirst(lc($Ini{'LANGUAGE'}));
	die &PerlTranslate("Fcts.pl", "ERROR: wrong TDP version($Ini{'TESTRTVER'} instead of $VERSION). Re-save the TDP using the tdpeditor.\n") if($Ini{TESTRTVER} ne $VERSION);
	$ATL_BUFFER_SIZE=$Ini{RTRT_BUFFER_SIZE} if(($Ini{RTRT_BUFFER_SIZE}) && ($Ini{RTRT_BUFFER_SIZE}<$ATL_BUFFER_SIZE));

	return(1);
}

###########################################################################
# ReturnAnalysis
# This routine will print the eventual message corresponding to signal
# received if any
###########################################################################
sub ReturnAnalysis #($$)
{
        my ($Ret,$Err) = @_ ;

        if ( $OSNAME eq "win32" ) {
	  $Ret >>= 8;
          return $Ret; # Windows
        }
        if ( $Ret == 0 ) {
          return $Ret;
        } elsif ( $Ret == 0xff00 ) {
          print "$Err\n"; # NO TRANSLATION
          return $Ret;
        } elsif ( $Ret > 0xff ) {
          $Ret >>= 8;
          return $Ret;
        } else {
          $Ret &= ~0x80;
          system("$BINDIR/strsignal $Ret");
          return $Ret;
        }
}

###########################################################################
# SystemP
# This routine will print and execute a given command, and 
# implicitly return the result
###########################################################################
sub SystemP
{
	my @Command=@_;
        my $Ret;

        print join (' ',@Command),"\n" if ($ENV{'ATTOLSTUDIO_VERBOSE'});
        $Ret=system(@Command) unless($showCmd);
        return (ReturnAnalysis($Ret,$Err));
}

###########################################################################
# SystemPiped
# This routine will print and execute a given command, in a pipe.
# To be used if the executed commands runs a DosExtender (Keil, Texas,...).
###########################################################################
sub SystemPiped
{
	my @Command=@_;
	my $Ret;

	print join (' ',@Command),"\n" if ($ENV{'ATTOLSTUDIO_VERBOSE'});
	open(OLDOUT,">&STDOUT");
	open(OLDERR,">&STDERR");
	open(STDOUT,">Out.txt");
	open(STDERR,">Err.txt");
	$Ret=system(@Command);
	close(STDOUT);open(STDOUT,">&OLDOUT");
	close(STDERR);open(STDERR,">&OLDERR");
	close(OLDOUT); close(OLDERR);
	open(File,"<Out.txt"); while(<File>) {print;} close(File);
	open(File,"<Err.txt"); while(<File>) {print;} close(File);	
        return (ReturnAnalysis($Ret,$Err));
}

###########################################################################
# SystemSilent
# This routine will print and execute a given command, in a pipe.
# To be used if the output of the run is optionaly to print
###########################################################################
sub SystemSilent
{
        my @Command=@_;
        my $Ret;
        my $Err;

        print join (' ',@Command),"\n" if ($ENV{'ATTOLSTUDIO_VERBOSE'});
        open(OLDOUT,">&STDOUT");
        open(OLDERR,">&STDERR");
        open(STDOUT,">Out.txt");
        open(STDERR,">Err.txt");
        $Ret=system(@Command);
        $Err=$!;
        close(STDOUT);open(STDOUT,">&OLDOUT");
        close(STDERR);open(STDERR,">&OLDERR");
        close(OLDOUT); close(OLDERR);
        my $status = ReturnAnalysis($Ret,$Err);
	print "\n"; # NO TRANSLATION
        if ($status>3) {
	  open(File,"<Err.txt"); while(<File>) {print;} close(File);	
        }
        return $status;
}

###########################################################################
# This routine will execute a given command, redirect STDOUT to the second
# argument and return the result
###########################################################################
sub SystemOut
{
        my $OutFile=shift;
        my @Command=@_;
        my $Ret;

        print &PerlTranslate("Fcts.pl", "%1 - Redirected stdout to %2\n","%1 is a command to be executed and %2 is the file name where the command output is to be redirected", join (' ',@Command), $OutFile) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
        open(OLDOUT,">&STDOUT");
        close(STDOUT);
        open(STDOUT,">$OutFile");
        $Ret=system(@Command);
        close(STDOUT);
        open(STDOUT,">&OLDOUT");
        close(OLDOUT);
        return (ReturnAnalysis($Ret,$Err));
}

###########################################################################
# This routine will execute a given command, redirect STDIN to the first
# argument and STDOUT to the second
# argument and return the result
###########################################################################
sub SystemInOut
{
        my $InFile=shift;
        my $OutFile=shift;
        my @Command=@_;
        my $Ret;

        print &PerlTranslate("Fcts.pl", "%1 - Redirected stdin from %2 and stdout to %3\n","%1 is a command to be executed, %2 is the file name from where the command input is to be read and %3 is the file name where the command output is to be redirected", join (' ',@Command), $InFile, $OutFile) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
        open(OLDOUT,">&STDOUT") || die &PerlTranslate("Fcts.pl", "Cannot save STDOUT\n");
        close(STDOUT);
        open(STDOUT,">$OutFile") || die &PerlTranslate("Fcts.pl", "Cannot redirect STDOUT to %1\n", "%1 is the file to redirect STDOUT in", $OutFile);
        open(IN,"<$InFile") || die &PerlTranslate("Fcts.pl", "Cannot open %1 for reading\n", "%1 is the file to read in", $InFile);
        open(CMD,"|@Command") || die &PerlTranslate("Fcts.pl", "Cannot fork %1\n", "%1 is the command that cannot be lauched", @Command);
        while(<IN>) { print CMD; }
        close(IN);
        close(CMD) || die &PerlTranslate("Fcts.pl", "Cannot exec %1\n", "%1 is the command that cannot be lauched", @Command);
        close(STDOUT);
        open(STDOUT,">&OLDOUT");
        close(OLDOUT);
        return $Ret;
}
###########################################################################
# each step call this function
###########################################################################
sub EndStep
{
   my $status=shift;
   	print STDERR "status $status\n" if($status);
	print STDERR "---\n" if($ENV{'ATTOLSTUDIO_VERBOSE'});
}
###########################################################################
# For scripts needing $INTDIR, check that this line does not contain empty
# string
###########################################################################
sub CheckIntDir
{
  # Check that $INTDIR is not an empty string
  if ($INTDIR eq "")
  {
     die &PerlTranslate("Fcts.pl", "No directory for intermediate files defined\n");
  }
  CreateDirectory ($INTDIR);
}

###########################################################################
# For scripts needing $OUTDIR, check that this line does not contain empty
# string
###########################################################################
sub CheckOutDir
{
  # Check that $OUTDIR is not an empty string
  if ($OUTDIR eq "")
  {
     die &PerlTranslate("Fcts.pl", "No directory for report files defined\n");
  }
  CreateDirectory ($OUTDIR);
}

###########################################################################
# It seems interesting to grep a line in a file and to know if it is in
###########################################################################
sub FindOutOptionInFile #($$)
{
  my ($expr,$file) = @_;
  my $ret = 0;

  open(THIS_FILE, "<$file") || die &PerlTranslate("Fcts.pl", "Cannot open %1\n", "%1 is the file that cannot be opened", $file);
  while(<THIS_FILE>) {
    $ret = 1 if(/$expr/);
  }
  close(THIS_FILE);
  return($ret);
}

###########################################################################
# This function replace a regular expr to a string in a file
# usage: my %stringReplace=( 
# before =>"after", # to  replace before by after
# up =>"down");     # and replace up     by down
# ReplaceInFile(\%stringReplace,"origineFile","destFile");
###########################################################################
sub ReplaceInFile #(\%$$)
{
  my $string=shift;
  my $fileIn=shift;
  my $fileOut=shift;
  my $ret = 0;
  
  $fileOut=$fileIn if($fileOut eq '');
  if(open(OUT_FILE, ">$fileOut.tmp")) {
	if(open(THIS_FILE, "<$fileIn")) {
		while(<THIS_FILE>) {
		    foreach my $exprToReplace (keys %$string){
			$ret=1 if(s,$exprToReplace,$string->{$exprToReplace},g);
		    }
		    print OUT_FILE;
		}
		close(THIS_FILE);
		close(OUT_FILE);
		rename("$fileOut.tmp",$fileOut);
	} else {
		print &PerlTranslate("Fcts.pl", "Cannot open %1\n", "%1 is the file that cannot be opened", $fileIn);
		close(OUT_FILE);
	}
  } else {
	print &PerlTranslate("Fcts.pl", "Cannot write file %1.tmp\n", "%1 is the file that cannot be opened", $fileOut);
  }
  return($ret);
}

###########################################################################
# Which product please?
###########################################################################
sub ReadProductName
{
#  my $mark = "$BINDIR" . $PATH_SEPARATOR . "delivery.mark";

  $SMALL_PRODUCT_NAME = "TestRT";
#  if (! open (MARK, "<$mark"))
#  {
#    return;
#  }
#  my $line = "";
#  if ($line = <MARK>)
#  {
#    if ($line =~ /^P\+RT.*$/)
#    {
#      $SMALL_PRODUCT_NAME = "P+RT";
#    }
#    elsif ($line =~ /^P\+LT.*$/)
#    {
#      $SMALL_PRODUCT_NAME = "P+Linux";
#    }
#  }
#  close (MARK);
}

###########################################################################
# This routine is called by the current top script
###########################################################################
sub EnterInScript
{
  #print "script $0\n"; copy($ARGV[0],"envNode.pl") if(-f $ARGV[0]);
  # No bufferisation of stdout
  #
  $|=1;
  # Remove the double / which may appear in $TARGETDIR
  $TARGETDIR =~ s|//|/|g;
  # First, set ATLTGT...
  $ENV{"ATLTGT"} = $TARGETDIR;
  # set RTRT_STUDIO_HEADER_FILE...
  $ENV{"RTRT_STUDIO_HEADER_FILE"} = $ARGV[0] if(-f $ARGV[0]);
  $OSNAME = ($^O=~/win32/i) ? "win32" : "unix" unless($OSNAME);
  # Try to add the path to cmd.exe in the PATH
  if ( $OSNAME eq "win32" ) {
    if ( $ENV{'ComSpec'} ) {
      ($dir, $name, $ext) = SplitFileName ($ENV{'ComSpec'});
      $dir =~ s/(.*)\\$/$1/;
      $ENV{'PATH'} = $dir . ";" . $ENV{'PATH'};
    } else {
      # Warn end-user
      die &PerlTranslate("Fcts.pl", "Fatal: COMSPEC environment variable not set, cannot find the shell command\n");
    }
    $PATH_SEPARATOR = '\\';
    $CLASSPATH_SEPARATOR = ';';
  } else {
    $PATH_SEPARATOR = '/';
    $CLASSPATH_SEPARATOR = ':';
  }
  # Then decode the .ini file
  DecodeIniFile() unless($TDPGEN);
  if($Ini{'ENTERINSCRIPT'}) {
       print &PerlTranslate("Fcts.pl","ENTERINSCRIPT specified as:%1\n","%1 is the file","$TARGETDIR/cmd/$Ini{'ENTERINSCRIPT'}");
       do "$TARGETDIR/cmd/$Ini{'ENTERINSCRIPT'}";
  }
  # Add Preferences Include Directories after Settings Include Directories
  @COMPLETE_INCLUDE_DIR_LIST = @INCLUDE_DIR_LIST;
  unshift(@COMPLETE_INCLUDE_DIR_LIST,@USER_INCLUDE_DIR_LIST);
  # Rule the RTRT world
  if ($ATL_OUT_FILE eq "" ) {
    $ATL_OUT_FILE = "atlout.spt";
  }
  # For GUI ATOMODE=ATOMODE||ATKMODE
  $ATKMODE=1 if ($ATOMODE);
  # Which product please?
  ReadProductName();
}
################################################################################
# register the tmp files generated by the TDP (used by atl_cc,atl_lnk,atl_exec)
################################################################################
sub RegisterTmpFiles
{
    #log files
    if(open ( LOG, ">>$LOGFILENAME" )) {
	    foreach(@_) {
		    print LOG "DUM $_\n";
    	}
    	close (LOG);
    }
}

################################################################################
# Put and Get Probe settings
################################################################################
sub PutProbeSetting #()
{
    my $tmp="$INTDIR". $PATH_SEPARATOR . "probe.log";
    open ( LOG, ">$tmp" ) or die &PerlTranslate("Fcts.pl", "Cannot open %1 file: %2\n", "%1 is the file,%2 is the reason given by the Operating System",$tmp, $!);
    print LOG "$ATS_PROBE_CUSTOM,$ATS_PROBE_CUSTOM_TYPE\n";
    close (LOG);
  
    #log opied files
    open ( LOG2, ">>$LOGFILENAME" ) or die &PerlTranslate("Fcts.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
    print LOG2 "DUM $tmp\n";
    close (LOG2);
}

sub GetProbeSetting #()
{
    my $tmp="$INTDIR". "$PATH_SEPARATOR" . "probe.log";
    my $result="";

    open ( LOG, "$tmp" ) or die &PerlTranslate("Fcts.pl", "Cannot open/read % file: %2\n", "%1 is the file,%2 is the reason given by the Operating System",$tmp, $!);
    
    while(<LOG>)
    {
	$result=$_;
    }
    close (LOG);
    return $result;
}

################################################################################
# TestSocket
# This function tests if a socket exists at the given remote address and port
# Returns 1 if socket doesn't exist, otherwise 0
################################################################################
sub TestSocket #($$)
{
   my ($remote_host, $remote_port) = @_;

   use IO::Socket;
   my $socket = IO::Socket::INET->new("$remote_host:$remote_port");
   return 1 unless $socket;
   return 0;
}
################################################################################
# hostinet
# This function return the inet address
# Returns string 
################################################################################
sub hostinet #()
{
   use Sys::Hostname;
   use IO::Socket;
   my $host = gethostbyname(hostname());
   my $ipad = "127.0.0.1";
   $ipad = inet_ntoa($host) if($host);
   return $ipad;
}

################################################################################
# launchLocalAgent
# This function launches a System Testing agent locally if it's not running yet
################################################################################
sub launchLocalAgent
{
   if (TestSocket($ADDRESS,$PORT))
   {
	if ($^O =~/Win32/i){
		$cmd = "$ENV{'COMSPEC'} /c start \"StudioHost Agent\" /MIN atsagtd.exe $PORT" ;
	}else{
		my $env=':'.$ENV{PATH}.':';
		unless($env =~/\:\.\:/){
			print &PerlTranslate("Fcts.pl", "Adding local path(.) to the \$PATH to help Agent to find binaries\n");
			$ENV{PATH}='.:'.$ENV{PATH};
		}
		$cmd = "atsagtd.bin $PORT &";
	}
	SystemP($cmd);
   }
   else
   {
	print &PerlTranslate("Fcts.pl", "Agent is already running on %1:%2\n", "%1 is target address %2 is port number", $ADDRESS, $PORT) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
   }
}


################################################################################
# CopyFile
# This function copies the filename's content to the file target_name
################################################################################
#sub CopyFile($$)
#{
#  my $filename = shift;
#  my $target_name = shift;
#  
#  open(IN_FILE, "< $filename") or return; # die "Couldn't open log file $filename\n";
#  open(OUT_FILE, "> $target_name") or return; # or die "Couldn't open log file $out_filename\n";
#  select(OUT_FILE);

#  while( <IN_FILE> )
#  {
#    print OUT_FILE $_;
#  }
  
#  close(IN_FILE);
#  close(OUT_FILE);
#}

################################################################################
# GetCurrentDir
# This function returns the current directory
################################################################################
sub GetCurrentDir
{
#   return TestRTUtil::GetCurrentDir();

    my $CurrentDir;
    if ( $OSNAME eq "win32" ) {
    	$CurrentDir=`cd`;
    }
    else
    {
    	$CurrentDir=`pwd`;
    }
    chomp($CurrentDir);
    return $CurrentDir;
}

###########################################################################
# absolutePath recompute the absolute path for a file
sub absolutePath
{
 my $f=shift;
 my $pwd=shift;
    $pwd=GetCurrentDir() unless($pwd);
 return File::Spec->rel2abs( $f , $pwd ) ;

# my $abs;
# if($f =~ /:|^[\\\/]/) {
#    $abs=$f;
#    }
# else {
#    $pwd=GetCurrentDir() unless($pwd);
#    $abs=$pwd.$PATH_SEPARATOR.$f;
#    }
# $abs =~ s/([\\\/])\.[\\\/]/$1/g
# while ($abs =~ s/[\\\/][^\\\/]*[\\\/]\.\.//);
 #keep path separator format as is
# return ($abs);
 }

################################################################################
# GetBinDir
# This function returns the binary directory
################################################################################
sub GetBinDir
{
    my $BinDir=$ENV{'ATTOLBIN'};
    if ($BinDir eq "") {
        $BinDir = "$ENV{'TESTRTDIR'}/bin/";
        my $RTRT_OS;
        $RTRT_OS = "rs6000/aix" if ($^O =~ /aix/i);
        $RTRT_OS = "hp700/hpux" if ($^O =~ /hpux/i);
        $RTRT_OS = "sun4/sunOS5" if ($^O =~ /solaris/i);
        $RTRT_OS = "intel/win32" if ($^O =~ /win32/i);
        if ($^O =~ /linux/i) {
                if ( -d "/etc/SuSEconfig" ) {
                        $RTRT_OS = "intel/linux_suse";
                }
                else {
                        $RTRT_OS = "intel/linux_redhat";
                }
        }
        $BinDir .= $RTRT_OS;
    }
    return $BinDir;
}


1;
#
################################################################################
