
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

my $error;
my @del;

sub CheckGcc
{
	my $Status;
	my $tmp = "$INTDIR/tmp_gcc_v";

        $back_lang = $ENV{'LANG'};
        $ENV{'LANG'} = "C";
	$Status=system("$compiler_name -v > $tmp 2>&1");
        $ENV{'LANG'} = $back_lang;
	if ($Status != 0)
	{
		die &PerlTranslate("GccUpdate.pl", "The compiler does not work properly. TDP autoconfiguration is not complete. The status of %1 -v is %2\n", "%1 is the compiler command and %2 is the status returned", $compiler_name, $Status);
	}
	my $found = 0;
	open(GCC,"<$tmp");
	while(<GCC>)
	{
		if (/^(.*) version (?:gcc-)?(?:egcs-)?(?:cygnus-)?((\d+\.)+\d+).*$/)
		{
			$gcc_name = $1;
			$gcc_version = $2;
			$found = 1;
			 break;
		}
	}
	close(GCC);
	push (@del,$tmp);
	return $found;
}

sub MakeDefinesFile #($$)
{
   my ($gcc_defines,$source_extension) = @_;
   my $empty = "$INTDIR/empty.$source_extension";

   # Make an empty source file
   open(EMPTY, ">$empty") || die &PerlTranslate("GccUpdate.pl", "Cannot create %1: %2\n", "%1 is the name of the file and %2 the reason given by the Operating System", $empty, $!);
   close(EMPTY);

   # Produce the defines list
   $status = system ("$compiler_name -E -dM $empty > $gcc_defines");
   unlink ($empty);
   if ($status != 0 )
   {
      $error = &PerlTranslate("GccUpdate.pl", "could not make preprocessor predefined macros list for $source_extension");
      print $error,"\n";
      return 0;
   }
   return 1;
}

sub MakeHpp #($$)
{
   my ($hpp, $defines) = @_;
   my $tmp = "$INTDIR/newatl.hpp";

   open(OLD, "<$hpp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $hpp, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   my $phase = 0;
   my $depth = 0;

   while (<OLD>)
   {
      if ($phase != 1)
      {
	 print NEW;
	 if (/^\s*#\s*ifdef\s*NEED_PREPRO_DEFS(\s.*)?$/ && $phase == 0)
	 {
	    $phase = 1;
	    open(DEF, "<$defines") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $defines, $!), return 0);
	    print NEW while (<DEF>);
	    close(DEF);
	 }
      }
      else
      {
	 if (/^\s*#\s*(if|ifdef|ifndef)\s.*$/)
	 {
	    $depth++;
	 }
	 elsif (/^\s*#\s*endif.*$/)
	 {
	    if ($depth > 0)
	    {
	       $depth--;
	    }
	    else
	    {
	       $phase = 2;
	       print NEW;
	    }
	 }
      }
   }

   close(OLD);
   close(NEW);
   print $error,"\n" if($error);
   unlink ($hpp);
   rename ($tmp, $hpp);
   return 1;
}

sub GetIncludesDirList 
{
   my $tmp = "$INTDIR/gcc_act";
   my $empty = "$INTDIR/empty.cc";
   my $tmp_o = "$INTDIR/empty.o";

   open(EMPTY, ">$empty") || ($error =  &PerlTranslate("GccUpdate.pl", "Cannot create %1: %2\n", "%1 is the name of the file and %2 the reason given by the Operating System", $empty, $!), return 0);
   close(EMPTY);

   $back_lang = $ENV{'LANG'};
   $ENV{'LANG'} = "C";
   system ("$compiler_name -c -v $empty -o $tmp_o 2> $tmp");

   $ENV{'LANG'} = $back_lang;
   unlink ($empty);
   unlink ($tmp_o) if (-f $tmp_o);
   if (! -f $tmp)
   {
      print $error,"\n" if($error);
      $error = &PerlTranslate("GccUpdate.pl", "could not execute %1 in verbose mode", "%1 is the name of the compiler", $compiler_name);
      print $error,"\n";
      return 0;
   }

   open(TMP, "<$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);
   $go = 0;
   my $index = 0;
   my $includes = ();
   my $complete=0;
   while (<TMP>)
   {
      if ($go)
      {
	 if (/^End of search list\.$/)
	 {
	    $go = 0;
	    $complete=1;
	 }
	 elsif (/^ (.*)$/)
	 {
	 	my $IncPath=$1;
		$includes[$index] = $IncPath;
		$includes[$index]= "\"$IncPath\"" if($IncPath=~/\s+/);
                $includes[$index]=~s/\//\\/g if($^O=~/Win32/i);
		$index++;
	 }
      }
      elsif (/^#include <\.\.\.> search starts here:$/)
      {
	 $go = 1;
      }
   }
   close (TMP);
   $error= &PerlTranslate("GccUpdate.pl", "Include path not complete\n","") unless($complete);
   print $error,"\n" if($error);
   print &PerlTranslate("GccUpdate.pl", "No include path found in %1 file\n","%1 is the GNU generated file",$tmp) unless($index);
   push (@del,$tmp) if($index && ($error ne ''));
   return 0 if($error);
   return 1;
}

# This subroutine removes the back-references in a path
# e.g. /home/../gcc becomes /gcc
sub canonicalize #($)
{
   my $new = shift;
   $new=~s/\\/\//g if($^O=~/Win32/i);
   my $old;
   do {
      $old = $new;
      $new =~ s|(.*)/((?!\.\.)[^/]+)/\.\./(.*)|$1/$3|;
   } until ($new eq $old);
   $new=~s/\//\\/g if($^O=~/Win32/i);
   return $new;
}

sub MakeOpp #($)
{
   my $opp = shift;
   my $tmp = "$INTDIR/newatl.opp";

   open(OLD, "<$opp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $opp, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   while (<OLD>) { print NEW; }
   close(OLD);
   if($gcc_version =~ /^(\d+)\.(\d+)/)
   {
   	my $v1=$1;
   	my $v2=$2;
   	my $v3=0;
   	if($gcc_version =~ /^(\d+)\.(\d+)\.(\d+)/)
   	{
   		$v3=$3;
   	}
	printf NEW "--gnu_version=%d%02d%02d\n",$v1,$v2,$v3 if(($v1 > 3) || (($v1==3)&&($v2>2)));
	print NEW "--gen_new_specialization\n" if ($v1 >=4);
   }
   foreach $include (@includes)
   {
	print NEW "--sys_include=$include\n";
	my $resolved = canonicalize ($include);
	if ($include ne $resolved)
	{
		# We also use the canonical version of the path (the #line directives
		# may appear in one way or the other)
		print NEW "--sys_include=$resolved\n";
	}
   }
   close(NEW);
   print $error,"\n" if($error);
   unlink ($opp);
   rename ($tmp, $opp);
   return 1;
}

sub MakeDef #($$)
{
   my ($deffile, $defines) = @_;
   my $tmp = "$INTDIR/newatl.def";
   my $in_macros = 0;

   open(OLD, "<$deffile") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $deffile, $!), return 0);
   open(NEW, ">$tmp") || ($error = &PerlTranslate("GccUpdate.pl", "cannot create %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $tmp, $!), return 0);

   while (<OLD>)
   {
      if (/Macros/ and /start/ and /do not remove this line/)
      {
	 $in_macros = 1;
	 print NEW;
      }
      elsif ($in_macros and /end/ and /do not remove this line/)
      {
	 my $line = $_;
         open(DEF, "<$defines") || ($error = &PerlTranslate("GccUpdate.pl", "cannot open %1: %2", "%1 is the name of the file and %2 the reason given by the Operating System", $defines, $!), return 0);
	 print NEW while (<DEF>);
	 close(DEF);
	 $in_macros = 0;
	 print NEW $line;
	 if($deffile=~ m/(atct|atus_c)\.def/){
	   if($gcc_version =~ /^(\d+)\.(\d+)/)
	   {
		my $v=$1*100+$2;
		printf NEW "typedef int _Bool;\n#define true 1\n#define false 0\n" if ($v >=304);
	   }
	 }
      }
      elsif ($in_macros)
      {
	 # Nothing
      }
      else
      {
	 print NEW;
      }
   }
   print NEW "\n";
   foreach $include (@includes)
   {
      print NEW "#pragma attol incl_std=$include\n";
      my $resolved = canonicalize ($include);
      print NEW "#pragma attol incl_std=$resolved\n" if ($include ne $resolved);
   }

   close (OLD);
   close (NEW);
   print $error,"\n" if($error);
   unlink ($deffile);
   rename ($tmp, $deffile);
   return 1;
}

sub GccUpdate #(@)
{
	my @argv=@_;

	$|=1;
	if (!$argv[0])
	{
   		print &PerlTranslate("GccUpdate.pl", "TestRT hpp Maker for GNU C++\n");
   		print &PerlTranslate("GccUpdate.pl", "Usage: TestRTperl GccUpdate.pl [-compiler=<compiler>] [<hpp file>] [<opp file>] [<def file>]\n");
   		print &PerlTranslate("GccUpdate.pl", "This tool updates <hpp file>, <opp file>, or <def file> according to the GNU C/C++ compiler installed on the system (if no compiler is specified, g++ will be used and must be in the search path).\n");
   		return 1;# K o
	}
	else
	{
   		@extensions = ("def", "hpp", "opp");
   		%make = ();
   		$compiler_name = "";

		foreach(@argv)
   		{
      			if (/^-compiler=(.*)$/)
      			{
	 			$compiler_name = $1;
      			}
      			elsif (/^-forceVersion=(.*)$/)
      			{
      				print $_,"\n";
      				$gcc_version=$1;
      				$gcc_name=$compiler_name;
      			}
      			elsif (/\.?(?:.*\.)+(.*)/)
      			{
	 			my $found = 0;
	 			$ext = $1;
	 			if (-f $_)
	 			{
					for ($i = 0; $i <= $#extensions; $i++)
					{
						if (lc($ext) eq $extensions[$i])
						{
							push @{$make{lc($ext)}}, $_;
							$found = 1;
							next;
						}
					}
					if (!$found)
					{
						print &PerlTranslate("GccUpdate.pl", "GccUpdate.pl - warning: file %1 not recognized.\n", "%1 is the name of the file", $_);
					}
				}
	 			else
	 			{
 	    				print &PerlTranslate("GccUpdate.pl", "File %1 not found\n", "%1 is the name of the file", $_);
	 			}
     			}
   		}
   		if ($compiler_name eq "")
   		{
      			print &PerlTranslate("GccUpdate.pl", "GccUpdate.pl - error: no compiler specified. Using g++...\n");
      			$compiler_name = "g++";
   		}
   		$INTDIR=".";
   		unless($gcc_version)
   		{
			die &PerlTranslate("GccUpdate.pl", "CheckGcc error\n") unless(CheckGcc ());
		}
      		print &PerlTranslate("GccUpdate.pl", "Compiler name:") . " $gcc_name\n";
      		print &PerlTranslate("GccUpdate.pl", "Version:") . " $gcc_version\n";
 
   		if ($make{def})
   		{
      			$def_c = "defines_c.hpp";
      			die &PerlTranslate("GccUpdate.pl", "Error MakeDefinesFile\n") unless(MakeDefinesFile($def_c,"c"));
   		}
   		if ($make{hpp})
   		{
      			$def_cpp = "defines_cpp.hpp";
      			die &PerlTranslate("GccUpdate.pl", "Error MakeDefinesFile\n") unless(MakeDefinesFile($def_cpp,"cpp"));
   		}
   		if ($make{def} or $make{opp})
   		{
			die &PerlTranslate("GccUpdate.pl", "Error GetIncludeDirList\n") unless(GetIncludesDirList());
   		}
   		if ($make{hpp})
   		{
      			foreach $hppfile (@{$make{hpp}})
      			{
				die &PerlTranslate("GccUpdate.pl", "Error MakeHpp\n") unless(MakeHpp($hppfile,$def_cpp));
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $hppfile);
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$hppfile.bak");
      			}
   		}
   		if ($make{opp})
   		{
      			foreach $oppfile (@{$make{opp}})
      			{
	    			die &PerlTranslate("GccUpdate.pl", "Error MakeOpp\n") unless(MakeOpp($oppfile));
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $oppfile);
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$oppfile.bak");
      			}
   		}
   		if ($make{def})
   		{
      			foreach $deffile (@{$make{def}})
      			{
				die &PerlTranslate("GccUpdate.pl", "Error MakeDef\n") unless(MakeDef($deffile,$def_c));
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "%1 successfully updated.\n", "%1 is the name of the updated file", $deffile);
# Silent Mode Preferred	    			print &PerlTranslate("GccUpdate.pl", "Original file was saved to %1.\n", "%1 is the name of the backup file", "$deffile.bak");
      			}
   		}
   		push (@del,$def_c) if ($def_c);
   		push (@del,$def_cpp) if ($def_cpp);
	}
	if($error){
		print $error,"\n" ;
		print "TDP unusable\n";
		return 1; # K o
	}
	unlink(@del);
	return 0; #ok
}

1;
