#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#          -*-  Import Microsoft Visual Studio Project .NET                    #
#                                                                              #
#                                                                              #
#  Script for importing a MVS .NET project.                                    #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $BINDIR		 : binary directory required for the PerlTranslate     #
#   $SCRIPTDIR           : directory of the scripts                            #
#   $IMPORTFILE          : name of the project to be imported (absolute path)  #
#   $MVSDIR              : directory of the project to be imported             #
#                                                                              #
################################################################################
#$Id: ImportMVSNETProject.pl,v 1.23 2007/10/12 08:19:20 rfrance Exp $
################################################################################
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings
require $ARGV[0]; #environment and global variables.

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

$DEBUG=0;

$INTDIR='cvisual7';
$confName='Unmanaged Visual .NET';
$lang='C++';
#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################

#----------------------------------------------------------------------------
## Start generation
print &PerlTranslate("ImportMVSNETProject.pl", "Importing files from Microsoft Visual Studio .NET project file: \"%1\"...\n", "%1 is the Microsoft Visual Studio .NET project file name", $IMPORTFILE);

open (MVSFILE,"$IMPORTFILE") || die &PerlTranslate("ImportMVSNETProject.pl", "Cannot open %1", "%1 is the Microsoft Visual Studio .NET project file name", $IMPORTFILE);

my $source = "";
my $key = "";
my ($dir,$name,$ext) = SplitFileName($IMPORTFILE);
my $EXT = uc($ext);

$MVSDIR = $dir;

ClearLogFile( "$name.rtp" );
ClearLogFile( "$name.inf" );

$TARGETDIR=$ENV{TESTRTDIR}.'/targets/'.$INTDIR;
$ENV{'ATLTGT'}=$TARGETDIR;
$INIFILE='tpcpp.ini';
DecodeIniFile();

open (IMPORTMVS_RTP,">>$name.rtp") || die &PerlTranslate("ImportProject.pl", "Error: Cannot write in %1\n", "%1 is the file that cannot be written", $name.rtp);
open (IMPORTMVS_INF,">>$name.inf") || die &PerlTranslate("ImportProject.pl", "Error: Cannot write in %1\n", "%1 is the file that cannot be written", $name.inf);

################################################################################
# Header of RTP file
################################################################################
printf IMPORTMVS_RTP "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
printf IMPORTMVS_RTP "<!DOCTYPE project SYSTEM \n\"$ENV{'ATTOLHOME'}"."/lib/project.dtd\" >\n";
printf IMPORTMVS_RTP "<project>\n";
printf IMPORTMVS_RTP "  <group>\n";
printf IMPORTMVS_RTP "    <node>\n";
printf IMPORTMVS_RTP "      <data>\n";
printf IMPORTMVS_RTP "        <name>$name</name>\n";
printf IMPORTMVS_RTP "        <renameable>false</renameable>\n";
printf IMPORTMVS_RTP "        <deletable>false</deletable>\n";
printf IMPORTMVS_RTP "        <open>true</open>\n";
printf IMPORTMVS_RTP "      </data>\n";
printf IMPORTMVS_RTP "      <current_configuration>$confName</current_configuration>\n";
printf IMPORTMVS_RTP "      <checked>false</checked>\n";
printf IMPORTMVS_RTP "      <background>false</background>\n";
printf IMPORTMVS_RTP "      <configuration_list>\n";
printf IMPORTMVS_RTP "        <configuration>\n";
printf IMPORTMVS_RTP "          <data>\n";
printf IMPORTMVS_RTP "            <name>$confName</name>\n";
printf IMPORTMVS_RTP "            <renameable>true</renameable>\n";
printf IMPORTMVS_RTP "            <deletable>true</deletable>\n";
printf IMPORTMVS_RTP "            <open>false</open>\n";
printf IMPORTMVS_RTP "          </data>\n";
printf IMPORTMVS_RTP "        </configuration>\n";
printf IMPORTMVS_RTP "      </configuration_list>\n";
printf IMPORTMVS_RTP "    </node>\n";

printf IMPORTMVS_RTP "    <group_child>\n";
printf IMPORTMVS_RTP "      <application>\n";
printf IMPORTMVS_RTP "        <test>\n";
printf IMPORTMVS_RTP "          <node>\n";
printf IMPORTMVS_RTP "            <data>\n";
printf IMPORTMVS_RTP "              <name>$name</name>\n";
printf IMPORTMVS_RTP "              <renameable>true</renameable>\n";
printf IMPORTMVS_RTP "              <deletable>true</deletable>\n";
printf IMPORTMVS_RTP "              <open>true</open>\n";
printf IMPORTMVS_RTP "            </data>\n";
printf IMPORTMVS_RTP "            <current_configuration>$confName</current_configuration>\n";
printf IMPORTMVS_RTP "            <checked>false</checked>\n";
printf IMPORTMVS_RTP "            <background>false</background>\n";
printf IMPORTMVS_RTP "            <perlscript_after_child>Application.pl</perlscript_after_child>\n";
printf IMPORTMVS_RTP "            <configuration_list>\n";
printf IMPORTMVS_RTP "              <configuration>\n";
printf IMPORTMVS_RTP "                <data>\n";
printf IMPORTMVS_RTP "                  <name>$confName</name>\n";
printf IMPORTMVS_RTP "                  <renameable>true</renameable>\n";
printf IMPORTMVS_RTP "                  <deletable>true</deletable>\n";
printf IMPORTMVS_RTP "                  <open>false</open>\n";
printf IMPORTMVS_RTP "                </data>\n";
printf IMPORTMVS_RTP "              </configuration>\n";
printf IMPORTMVS_RTP "            </configuration_list>\n";
printf IMPORTMVS_RTP "          </node>\n";
printf IMPORTMVS_RTP "          <test_child>\n";


################################################################################
# Header of INF file
################################################################################
printf IMPORTMVS_INF "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
printf IMPORTMVS_INF "<!DOCTYPE project SYSTEM \n\"$ENV{'ATTOLHOME'}"."/lib/project.dtd\" >\n";
printf IMPORTMVS_INF "<objects>\n";
printf IMPORTMVS_INF "  <folder>\n";
printf IMPORTMVS_INF "    <data>\n";
printf IMPORTMVS_INF "      <name>$name</name>\n";
printf IMPORTMVS_INF "      <renameable>false</renameable>\n";
printf IMPORTMVS_INF "      <deletable>true</deletable>\n";
printf IMPORTMVS_INF "      <open>true</open>\n";
printf IMPORTMVS_INF "    </data>\n";
printf IMPORTMVS_INF "    <filters></filters>\n";
printf IMPORTMVS_INF "    <folder_child>\n";


my @tabFile;
while ($line = <MVSFILE>)
{
    $finded = 0;

    if ($line =~ /RelativePath=(.*)$/) {
	$finded = 1;

	# $source = &removeSpecialChar($1);
      $source = $1;
      $source =~ s/[\"\>]//g;

      $DEBUG && print "File = $source\n";

	($dir,$name,$ext) = SplitFileName($source);
	push @tabFile, $source;
    }
    $_ = uc $ext;
    if( $finded == 1 and (/^CPP$/  or /^CXX$/ or /^CC$/ or /^C$/ or /^RC$/) ) {
	printf IMPORTMVS_RTP "            <source>\n";
	printf IMPORTMVS_RTP "              <nodefile_ref>\n";
	printf IMPORTMVS_RTP "                <node>\n";
	printf IMPORTMVS_RTP "                  <data>\n";
	printf IMPORTMVS_RTP "                    <name>$name.$ext</name>\n";
	printf IMPORTMVS_RTP "                    <renameable>false</renameable>\n";
	printf IMPORTMVS_RTP "                    <deletable>true</deletable>\n";
	printf IMPORTMVS_RTP "                    <open>false</open>\n";
	printf IMPORTMVS_RTP "                  </data>\n";
	printf IMPORTMVS_RTP "                  <current_configuration>$confName</current_configuration>\n";
	printf IMPORTMVS_RTP "                  <checked>false</checked>\n";
	printf IMPORTMVS_RTP "                  <background>false</background>\n";
	printf IMPORTMVS_RTP "                  <perlscript_after_child>Source.pl</perlscript_after_child>\n";
	printf IMPORTMVS_RTP "                  <configuration_list>\n";
	printf IMPORTMVS_RTP "                    <configuration>\n";
	printf IMPORTMVS_RTP "                      <data>\n";
	printf IMPORTMVS_RTP "                        <name>$confName</name>\n";
	printf IMPORTMVS_RTP "                        <renameable>true</renameable>\n";
	printf IMPORTMVS_RTP "                        <deletable>true</deletable>\n";
	printf IMPORTMVS_RTP "                        <open>true</open>\n";
	printf IMPORTMVS_RTP "                      </data>\n";
	printf IMPORTMVS_RTP "                    </configuration>\n";
	printf IMPORTMVS_RTP "                  </configuration_list>\n";
	printf IMPORTMVS_RTP "                </node>\n";
	printf IMPORTMVS_RTP "                <filepath>$source</filepath>\n";
	printf IMPORTMVS_RTP "              </nodefile_ref>\n";
	printf IMPORTMVS_RTP "              <integrated>false</integrated>\n";
	printf IMPORTMVS_RTP "              <source_child>\n";
	printf IMPORTMVS_RTP "              </source_child>\n";
	printf IMPORTMVS_RTP "            </source>\n";
    }
}


###############################################################################
#  Suite du fichier INF
###############################################################################
printf IMPORTMVS_INF "      <folder>\n";
printf IMPORTMVS_INF "        <data>\n";
printf IMPORTMVS_INF "          <name>Scripts</name>\n";
printf IMPORTMVS_INF "          <renameable>true</renameable>\n";
printf IMPORTMVS_INF "          <deletable>true</deletable>\n";
printf IMPORTMVS_INF "          <open>true</open>\n";
printf IMPORTMVS_INF "        </data>\n";
printf IMPORTMVS_INF "        <filters>*.ptu;*.pts;*.otd;*.otc;*.dcl;*.stb</filters>\n";
printf IMPORTMVS_INF "        <folder_child>\n";
$i = $#tabFile;
$j = 0;
while($i != -1) {
    $i--;
    ($dir,$name,$ext) = SplitFileName($tabFile[j]);
    if($ext =~ "ptu" or $ext =~ "pts" or $ext =~ "otd" or $ext =~ "otc"
       or $ext =~ "dcl" or $ext =~ "stb") {
	($dir,$name,$ext) = SplitFileName(splice(@tabFile,$j,1));
	printf IMPORTMVS_INF "          <file>\n";
	printf IMPORTMVS_INF "            <tested_tag>\n";
	printf IMPORTMVS_INF "              <tag>\n";
	printf IMPORTMVS_INF "                <data>\n";
	printf IMPORTMVS_INF "                  <name>$name.$ext</name>\n";
	printf IMPORTMVS_INF "                  <renameable>true</renameable>\n";
	printf IMPORTMVS_INF "                  <deletable>true</deletable>\n";
	printf IMPORTMVS_INF "                  <open>true</open>\n";
	printf IMPORTMVS_INF "                </data>\n";
	printf IMPORTMVS_INF "                <filepath>$MVSDIR$name.$ext</filepath>\n";
	printf IMPORTMVS_INF "                <line>1</line>\n";
	printf IMPORTMVS_INF "                <qualifier>0</qualifier>\n";
	printf IMPORTMVS_INF "                <tag_child>\n";
	printf IMPORTMVS_INF "                </tag_child>\n";
	printf IMPORTMVS_INF "              </tag>\n";
	printf IMPORTMVS_INF "            </tested_tag>\n";
	printf IMPORTMVS_INF "            <filepath>$dir$name.$ext</filepath>\n";
	printf IMPORTMVS_INF "            <last_modification></last_modification>\n";
	printf IMPORTMVS_INF "          </file>\n";
    }
    else {
	$j++;
    }
}
printf IMPORTMVS_INF "        </folder_child>\n";
printf IMPORTMVS_INF "      </folder>\n";

printf IMPORTMVS_INF "      <folder>\n";
printf IMPORTMVS_INF "        <data>\n";
printf IMPORTMVS_INF "          <name>Sources</name>\n";
printf IMPORTMVS_INF "          <renameable>true</renameable>\n";
printf IMPORTMVS_INF "          <deletable>true</deletable>\n";
printf IMPORTMVS_INF "          <open>true</open>\n";
printf IMPORTMVS_INF "        </data>\n";
printf IMPORTMVS_INF "        <filters>*.h;*.hxx;*.c;*.cpp;*.cxx;*.cc;*.java;*.ada;*.adb;*.ads;*.spv;*.mat;*.mdl;*.rc;*</filters>\n";
printf IMPORTMVS_INF "        <folder_child>\n";
$i = $#tabFile;
$j = 0;
while($i != -1) {
    $i--;
    ($dir,$name,$ext) = SplitFileName($tabFile[$j]);
     $_ = uc $ext;
    if(/^CPP$/ or /^C$/ or /^CXX$/ or /^CC$/ or /^RC$/) {
      ($dir,$name,$ext) = SplitFileName(splice(@tabFile,$j,1));
      printf IMPORTMVS_INF "          <file>\n";
      printf IMPORTMVS_INF "            <tested_tag>\n";
      printf IMPORTMVS_INF "              <tag>\n";
      printf IMPORTMVS_INF "                <data>\n";
      printf IMPORTMVS_INF "                  <name>$name.$ext</name>\n";
      printf IMPORTMVS_INF "                  <renameable>true</renameable>\n";
      printf IMPORTMVS_INF "                  <deletable>true</deletable>\n";
      printf IMPORTMVS_INF "                  <open>true</open>\n";
      printf IMPORTMVS_INF "                </data>\n";
      printf IMPORTMVS_INF "                <filepath>$MVSDIR$name.$ext</filepath>\n";
      printf IMPORTMVS_INF "                <line>1</line>\n";
      printf IMPORTMVS_INF "                <qualifier>0</qualifier>\n";
      printf IMPORTMVS_INF "                <tag_child>\n";
      printf IMPORTMVS_INF "                </tag_child>\n";
      printf IMPORTMVS_INF "              </tag>\n";
      printf IMPORTMVS_INF "            </tested_tag>\n";
      printf IMPORTMVS_INF "            <filepath>$dir$name.$ext</filepath>\n";
      printf IMPORTMVS_INF "            <last_modification></last_modification>\n";
      printf IMPORTMVS_INF "          </file>\n";
    }
    else {
	$j++;
    }
}
printf IMPORTMVS_INF "        </folder_child>\n";
printf IMPORTMVS_INF "      </folder>\n";

printf IMPORTMVS_INF "      <folder>\n";
printf IMPORTMVS_INF "        <data>\n";
printf IMPORTMVS_INF "          <name>Dependencies</name>\n";
printf IMPORTMVS_INF "          <renameable>true</renameable>\n";
printf IMPORTMVS_INF "          <deletable>true</deletable>\n";
printf IMPORTMVS_INF "          <open>true</open>\n";
printf IMPORTMVS_INF "        </data>\n";
printf IMPORTMVS_INF "        <filters></filters>\n";
printf IMPORTMVS_INF "        <folder_child>\n";
$i=$#tabFile;
while($i!=-1) {
    $i--;
    ($dir,$name,$ext) = SplitFileName(splice(@tabFile,0,1));
    printf IMPORTMVS_INF "          <file>\n";
    printf IMPORTMVS_INF "            <tested_tag>\n";
    printf IMPORTMVS_INF "              <tag>\n";
    printf IMPORTMVS_INF "                <data>\n";
    printf IMPORTMVS_INF "                  <name>$name.$ext</name>\n";
    printf IMPORTMVS_INF "                  <renameable>true</renameable>\n";
    printf IMPORTMVS_INF "                  <deletable>true</deletable>\n";
    printf IMPORTMVS_INF "                  <open>true</open>\n";
    printf IMPORTMVS_INF "                </data>\n";
    printf IMPORTMVS_INF "                <filepath>$MVSDIR$name.$ext</filepath>\n";
    printf IMPORTMVS_INF "                <line>1</line>\n";
    printf IMPORTMVS_INF "                <qualifier>0</qualifier>\n";
    printf IMPORTMVS_INF "                <tag_child>\n";
    printf IMPORTMVS_INF "                </tag_child>\n";
    printf IMPORTMVS_INF "              </tag>\n";
    printf IMPORTMVS_INF "            </tested_tag>\n";
    printf IMPORTMVS_INF "            <filepath>$dir$name.$ext</filepath>\n";
    printf IMPORTMVS_INF "            <last_modification></last_modification>\n";
    printf IMPORTMVS_INF "          </file>\n";
}
printf IMPORTMVS_INF "        </folder_child>\n";
printf IMPORTMVS_INF "      </folder>\n";


###############################################################################
#  Suite du fichier RTP
###############################################################################
printf IMPORTMVS_RTP "          </test_child>\n";
printf IMPORTMVS_RTP "        </test>\n";
printf IMPORTMVS_RTP "      </application>\n";
printf IMPORTMVS_RTP "    </group_child>\n";
printf IMPORTMVS_RTP "  </group>\n";

printf IMPORTMVS_RTP "  <object_file>$name.inf</object_file>\n";
printf IMPORTMVS_RTP "  <root_configuration_list>\n";
printf IMPORTMVS_RTP "    <configuration>\n";
printf IMPORTMVS_RTP "      <data>\n";
printf IMPORTMVS_RTP "        <name>$confName</name>\n";
printf IMPORTMVS_RTP "        <renameable>true</renameable>\n";
printf IMPORTMVS_RTP "        <deletable>true</deletable>\n";
printf IMPORTMVS_RTP "        <open>true</open>\n";
printf IMPORTMVS_RTP "      </data>\n";
printf IMPORTMVS_RTP "      <computerName>localhost</computerName>\n";
printf IMPORTMVS_RTP "      <computerAddress>127.0.0.1</computerAddress>\n";
printf IMPORTMVS_RTP "      <computerPort>10000</computerPort>\n";
printf IMPORTMVS_RTP "      <uploaderPort>7777</uploaderPort>\n";
printf IMPORTMVS_RTP "      <targetName>$lang $confName</targetName>\n";
printf IMPORTMVS_RTP "      <targetPath>$INTDIR</targetPath>\n";
printf IMPORTMVS_RTP "      <iniFileName>tpcpp.ini</iniFileName>\n";
printf IMPORTMVS_RTP "      <sourceLanguage>CPP</sourceLanguage>\n";
printf IMPORTMVS_RTP "      <sourceExt>$Ini{SRCEXT}</sourceExt>\n";
printf IMPORTMVS_RTP "      <objectExt>$Ini{OBJEXT}</objectExt>\n";
printf IMPORTMVS_RTP "      <binaryExt>$Ini{EXEEXT}</binaryExt>\n";
printf IMPORTMVS_RTP "      <libraryStaticExt>$Ini{LIBEXT}</libraryStaticExt>\n";
printf IMPORTMVS_RTP "      <libraryDynamicExt>$Ini{DLLEXT}</libraryDynamicExt>\n";
printf IMPORTMVS_RTP "      <ansiMode>true</ansiMode>\n";
printf IMPORTMVS_RTP "      <version></version>\n";
printf IMPORTMVS_RTP "      <target></target>\n";
printf IMPORTMVS_RTP "      <exception>true</exception>\n";
printf IMPORTMVS_RTP "      <definitionFile></definitionFile>\n";
printf IMPORTMVS_RTP "      <intdir>$INTDIR</intdir>\n";
printf IMPORTMVS_RTP "      <outdir>$INTDIR</outdir>\n";
printf IMPORTMVS_RTP "      <ge_incdir>$Ini{DEFAULT_INCLUDE_PATHS}</ge_incdir>\n";
printf IMPORTMVS_RTP "      <testProgram>&lt;default&gt;</testProgram>\n";
printf IMPORTMVS_RTP "      <arguments></arguments>\n";
printf IMPORTMVS_RTP "      <ge_host_workdir>tmp</ge_host_workdir>\n";
printf IMPORTMVS_RTP "      <cppFlags>$Ini{DEFAULT_CPPFLAGS}</cppFlags>\n";
printf IMPORTMVS_RTP "      <cflags>$Ini{DEFAULT_CFLAGS}</cflags>\n";
printf IMPORTMVS_RTP "      <tcflags>$Ini{DEFAULT_DEFINES}</tcflags>\n";
printf IMPORTMVS_RTP "      <ldflags>$Ini{DEFAULT_LDFLAGS}</ldflags>\n";
printf IMPORTMVS_RTP "      <additionnalObjects>$Ini{DEFAULT_LIBS}</additionnalObjects>\n";
printf IMPORTMVS_RTP "      <ge_libdir>$Ini{DEFAULT_LIBRARY_PATHS}</ge_libdir>\n";
printf IMPORTMVS_RTP "      <debug>false</debug>\n";
printf IMPORTMVS_RTP "      <multiThreaded>false</multiThreaded>\n";
printf IMPORTMVS_RTP "      <bufferSize>1024</bufferSize>\n";
printf IMPORTMVS_RTP "      <maxThreads>64</maxThreads>\n";
printf IMPORTMVS_RTP "      <atc_rungcatexit>true</atc_rungcatexit>\n";
printf IMPORTMVS_RTP "      <mainProc></mainProc>\n";
printf IMPORTMVS_RTP "      <clockKind>0</clockKind>\n";
printf IMPORTMVS_RTP "      <sourceFile>&lt;default&gt;</sourceFile>\n";
printf IMPORTMVS_RTP "      <otiFile>&lt;default&gt;</otiFile>\n";
printf IMPORTMVS_RTP "      <xrdFile>&lt;default&gt;</xrdFile>\n";
printf IMPORTMVS_RTP "      <nuerror>30</nuerror>\n";
printf IMPORTMVS_RTP "      <nodline>true</nodline>\n";
printf IMPORTMVS_RTP "      <atobreakonerror>false</atobreakonerror>\n";
printf IMPORTMVS_RTP "      <atonopassdump>false</atonopassdump>\n";
printf IMPORTMVS_RTP "      <atomaxinstances>64</atomaxinstances>\n";
printf IMPORTMVS_RTP "      <printbuffer>false</printbuffer>\n";
printf IMPORTMVS_RTP "      <printbuffersize>1024</printbuffersize>\n";
printf IMPORTMVS_RTP "      <atkbreakonerror>false</atkbreakonerror>\n";
printf IMPORTMVS_RTP "      <atknopassdump>false</atknopassdump>\n";
printf IMPORTMVS_RTP "      <atkreflexivestate>true</atkreflexivestate>\n";
printf IMPORTMVS_RTP "      <atkconstmethods>false</atkconstmethods>\n";
printf IMPORTMVS_RTP "      <atkreentrant>true</atkreentrant>\n";
printf IMPORTMVS_RTP "      <atkstrictcheck>false</atkstrictcheck>\n";
printf IMPORTMVS_RTP "      <st_interface_files></st_interface_files>\n";
printf IMPORTMVS_RTP "      <st_allocation></st_allocation>\n";
printf IMPORTMVS_RTP "      <st_notShared>false</st_notShared>\n";
printf IMPORTMVS_RTP "      <st_thread>false</st_thread>\n";
printf IMPORTMVS_RTP "      <st_thread_function></st_thread_function>\n";
printf IMPORTMVS_RTP "      <st_trace_circular>false</st_trace_circular>\n";
printf IMPORTMVS_RTP "      <st_trace></st_trace>\n";
printf IMPORTMVS_RTP "      <st_bufsize>10</st_bufsize>\n";
printf IMPORTMVS_RTP "      <st_values></st_values>\n";
printf IMPORTMVS_RTP "      <st_time>false</st_time>\n";
printf IMPORTMVS_RTP "      <st_dynfol>false</st_dynfol>\n";
printf IMPORTMVS_RTP "      <st_item_buf>false</st_item_buf>\n";
printf IMPORTMVS_RTP "      <st_item_buf_size>8192</st_item_buf_size>\n";
printf IMPORTMVS_RTP "      <st_dtrack>false</st_dtrack>\n";
printf IMPORTMVS_RTP "      <st_dtrack_buf_size>4096</st_dtrack_buf_size>\n";
printf IMPORTMVS_RTP "      <st_interrecv_timeout>300</st_interrecv_timeout>\n";
printf IMPORTMVS_RTP "      <st_custom>false</st_custom>\n";
printf IMPORTMVS_RTP "      <st_custom_type>0</st_custom_type>\n";
printf IMPORTMVS_RTP "      <fdc_name_opt>0</fdc_name_opt>\n";
printf IMPORTMVS_RTP "      <fdc_value></fdc_value>\n";
printf IMPORTMVS_RTP "      <tsf_name_opt>0</tsf_name_opt>\n";
printf IMPORTMVS_RTP "      <tsf_value></tsf_value>\n";
printf IMPORTMVS_RTP "      <tsf_selective_file></tsf_selective_file>\n";
printf IMPORTMVS_RTP "      <tsf_selective_class></tsf_selective_class>\n";
printf IMPORTMVS_RTP "      <exclude_dir></exclude_dir>\n";
printf IMPORTMVS_RTP "      <dump_incoming_list></dump_incoming_list>\n";
printf IMPORTMVS_RTP "      <dump_returning_list></dump_returning_list>\n";
printf IMPORTMVS_RTP "      <dump_calling_list></dump_calling_list>\n";
printf IMPORTMVS_RTP "      <no_header_instr>true</no_header_instr>\n";
printf IMPORTMVS_RTP "      <instantiate_template>false</instantiate_template>\n";
printf IMPORTMVS_RTP "      <misc_options></misc_options>\n";
printf IMPORTMVS_RTP "      <atc_instr_functions>1</atc_instr_functions>\n";
printf IMPORTMVS_RTP "      <atc_instr_calls>1</atc_instr_calls>\n";
printf IMPORTMVS_RTP "      <atc_instr_blocks>2</atc_instr_blocks>\n";
printf IMPORTMVS_RTP "      <atc_instr_conditions>3</atc_instr_conditions>\n";
printf IMPORTMVS_RTP "      <atc_tio_name>&lt;default&gt;</atc_tio_name>\n";
printf IMPORTMVS_RTP "      <atc_selective_no_ternary>true</atc_selective_no_ternary>\n";
printf IMPORTMVS_RTP "      <atc_selective_no_inline>true</atc_selective_no_inline>\n";
printf IMPORTMVS_RTP "      <atc_selective_no_included>true</atc_selective_no_included>\n";
printf IMPORTMVS_RTP "      <atc_selective_functions_list></atc_selective_functions_list>\n";
printf IMPORTMVS_RTP "      <atc_selective_spec>false</atc_selective_spec>\n";
printf IMPORTMVS_RTP "      <atc_information>0</atc_information>\n";
printf IMPORTMVS_RTP "      <atc_instr_ctrl_prefixe></atc_instr_ctrl_prefixe>\n";
printf IMPORTMVS_RTP "      <atc_instr_ctrl_suffixe>0</atc_instr_ctrl_suffixe>\n";
printf IMPORTMVS_RTP "      <atc_no_metrics>true</atc_no_metrics>\n";
printf IMPORTMVS_RTP "      <atc_user_comment></atc_user_comment>\n";
printf IMPORTMVS_RTP "      <atc_dump_driver>false</atc_dump_driver>\n";
printf IMPORTMVS_RTP "      <att_tdf_name>&lt;default&gt;</att_tdf_name>\n";
printf IMPORTMVS_RTP "      <att_calling_sequenced>false</att_calling_sequenced>\n";
printf IMPORTMVS_RTP "      <att_aggregate_traces>true</att_aggregate_traces>\n";
printf IMPORTMVS_RTP "      <att_template_notes>true</att_template_notes>\n";
printf IMPORTMVS_RTP "      <att_split>true</att_split>\n";
printf IMPORTMVS_RTP "      <att_split_size>5000</att_split_size>\n";
printf IMPORTMVS_RTP "      <att_split_prefixe>&lt;default&gt;</att_split_prefixe>\n";
printf IMPORTMVS_RTP "      <att_loop>false</att_loop>\n";
printf IMPORTMVS_RTP "      <att_loop_options></att_loop_options>\n";
printf IMPORTMVS_RTP "      <att_record_max_stack>true</att_record_max_stack>\n";
printf IMPORTMVS_RTP "      <att_on_fly>false</att_on_fly>\n";
printf IMPORTMVS_RTP "      <att_item_buffer>false</att_item_buffer>\n";
printf IMPORTMVS_RTP "      <att_item_buffer_size>64</att_item_buffer_size>\n";
printf IMPORTMVS_RTP "      <att_partial_dump>false</att_partial_dump>\n";
printf IMPORTMVS_RTP "      <att_signal_action>0</att_signal_action>\n";
printf IMPORTMVS_RTP "      <att_time_stamp>true</att_time_stamp>\n";
printf IMPORTMVS_RTP "      <att_heap_size>true</att_heap_size>\n";
printf IMPORTMVS_RTP "      <att_thread_info>true</att_thread_info>\n";
printf IMPORTMVS_RTP "      <atp_tpf_name>&lt;default&gt;</atp_tpf_name>\n";
printf IMPORTMVS_RTP "      <atp_instr_ctrl></atp_instr_ctrl>\n";
printf IMPORTMVS_RTP "      <atp_errors_fiu>true</atp_errors_fiu>\n";
printf IMPORTMVS_RTP "      <atp_errors_miu>false</atp_errors_miu>\n";
printf IMPORTMVS_RTP "      <atp_errors_sig>true</atp_errors_sig>\n";
printf IMPORTMVS_RTP "      <atp_fmwl>true</atp_fmwl>\n";
printf IMPORTMVS_RTP "      <atp_fmwl_queue_len>100</atp_fmwl_queue_len>\n";
printf IMPORTMVS_RTP "      <atp_fmwl_queue_size>1024</atp_fmwl_queue_size>\n";
printf IMPORTMVS_RTP "      <atp_abwl>true</atp_abwl>\n";
printf IMPORTMVS_RTP "      <atp_abwl_zone_size>16</atp_abwl_zone_size>\n";
printf IMPORTMVS_RTP "      <atp_call_stack>6</atp_call_stack>\n";
printf IMPORTMVS_RTP "      <atq_tqf_name>&lt;default&gt;</atq_tqf_name>\n";
printf IMPORTMVS_RTP "      <atq_dump_driver>false</atq_dump_driver>\n";
printf IMPORTMVS_RTP "      <attolClassName></attolClassName>\n";
printf IMPORTMVS_RTP "      <intermediateFile>&lt;default&gt;</intermediateFile>\n";
printf IMPORTMVS_RTP "      <simulation>false</simulation>\n";
printf IMPORTMVS_RTP "      <preproerrorlevel>true</preproerrorlevel>\n";
printf IMPORTMVS_RTP "      <breakontest>false</breakontest>\n";
printf IMPORTMVS_RTP "      <preparationOptions></preparationOptions>\n";
printf IMPORTMVS_RTP "      <moreoptions>false</moreoptions>\n";
printf IMPORTMVS_RTP "      <filter>0</filter>\n";
printf IMPORTMVS_RTP "      <strategy>0</strategy>\n";
printf IMPORTMVS_RTP "      <reportOutlay>0</reportOutlay>\n";
printf IMPORTMVS_RTP "      <moreRoptions>false</moreRoptions>\n";
printf IMPORTMVS_RTP "      <reportoptions></reportoptions>\n";
printf IMPORTMVS_RTP "      <noComparaison>true</noComparaison>\n";
printf IMPORTMVS_RTP "      <instrumentedComparaison>false</instrumentedComparaison>\n";
printf IMPORTMVS_RTP "      <secondContext>false</secondContext>\n";
printf IMPORTMVS_RTP "      <secondTarget>true</secondTarget>\n";
printf IMPORTMVS_RTP "      <secondIntermediateFile>false</secondIntermediateFile>\n";
printf IMPORTMVS_RTP "      <strictComparaison>false</strictComparaison>\n";
printf IMPORTMVS_RTP "      <comparaison>false</comparaison>\n";
printf IMPORTMVS_RTP "      <userCommandLine></userCommandLine>\n";

printf IMPORTMVS_RTP "    </configuration>\n";
printf IMPORTMVS_RTP "  </root_configuration_list>\n";
printf IMPORTMVS_RTP "  <version_file>1</version_file>\n";
printf IMPORTMVS_RTP "</project>\n";
###############################################################################
#  Fin d'ecriture du fichier RTP
###############################################################################


printf IMPORTMVS_INF "    </folder_child>\n";
printf IMPORTMVS_INF "  </folder>\n";
printf IMPORTMVS_INF "  <version_file>1</version_file>\n";
printf IMPORTMVS_INF "</objects>\n";
###############################################################################
#  Fin d'ecriture du fichier INF
###############################################################################


close( MVSFILE );
close( IMPORTMVS_RTP );
close( IMPORTMVS_INF );


0;

# on supprime les " et >  on recoit une chain type : "test">
sub removeSpecialChar {
    my($s) = $_[0];
    chomp($s);
    $s = substr($s,1,length($s)-4);  # ?4  normalement 3 !! mais marche avec 4, pas 3
    return $s;
}


