#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                     -*-  Import Makefile -*-                                 #
#                                                                              #
#                                                                              #
#  Script for importing a Makefile.                                            #
#                                                                              #
#  usage:                                                                      #
#       ImportMakefile.pl <environment file>                                   # 
#                                                                              #
# Required global variables:                                                   #
#   $BINDIR		 : binary directory required for the PerlTranslate     #
#   $SCRIPTDIR           : directory of the scripts                            #
#   $IMPORTFILE          : key of the Makefile to be imported                  #
#   $PROJECTNAME         : name for the created project                        #
#   $TDPNAME
#   $TARGETDIR
#   $INIFILE
#   $OBJEXT
#   $EXEEXT
#   $SRCEXT
#   $CPPFLAGS
#   $CFLAGS
#   $TCFLAGS
#   $LDFLAGS
#   $LIBRARIES
#   $TDP_LANGUAGE
#   $INTDIR
#   $OUTDIR
#   $INCLUDE_DIR_LIST
#   $LIBRARY_DIR_LIST
#   $JAVA_BOOTCLASSPATH
#   $J2ME_RESTRICTION
#                                                                              #
#                                                                              #
################################################################################
#$Id: ImportMakefile.pl,v 1.36 2007/09/21 21:54:38 rfrance Exp $
################################################################################

use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

$RTRT_REPLACE_ENV_VAR=0;
# This script depends on the followings
require $ARGV[0]; #environment and global variables.

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

$DEBUG=1;

################################################################################
# Here is the beginning
################################################################################
&EnterInScript;

#----------------------------------------------------------------------------
## 
print &PerlTranslate("ImportMakefile.pl", "Importing files from the Makefile: \"%1\"...\n", "%1 is the Makefile name", $IMPORTFILE);

my ($dir,$n,$ext1) = SplitFileName($IMPORTFILE);
$MKFDIR = $dir;

$projectFileName = $PROJECTNAME . ".rtp";
$infoFileName = $PROJECTNAME . ".inf";
$log=$PROJECTNAME."_Import.log";
print &PerlTranslate("ImportMakefile.pl", "Generating analysis in: \"%1\"...\n", "%1 is the Makefile name", $log);

ClearLogFile( $log );
ClearLogFile( $projectFileName );
ClearLogFile( $infoFileName );
open (LOG,">$log") || die &PerlTranslate("ImportMakefile.pl", "Error: Cannot write in %1\n", "%1 is the file that cannot be written", $log) ;
open (IMPORTMVS_RTP,">>$projectFileName") || die &PerlTranslate("ImportMakefile.pl", "Error: Cannot write in %1\n", "%1 is the file that cannot be written", $projectFileName) ;
open (IMPORTMVS_INF,">>$infoFileName") || die &PerlTranslate("ImportMakefile.pl", "Error: Cannot write in %1\n", "%1 is the file that cannot be written", $infoFileName);

print LOG <<"END_LOG";
#This is the generated import log file for the project: $PROJECTNAME 
#If the generated project is not as expected, you can modify this file as you want 
#and save it as .mk file 
#restart the importMakeFile from studio then choose this new file as makeFile
#to take into account your modifications in the generated project.
#
#Since studio is able to manage Environment variables in file paths,
#by default importMakeFile does not substitute environment variables by their values
#in order to get a more flexible/relocatable project.
#So, if you want to get these environment variables replaced then uncomment the following line
#RTRT_REPLACE_ENV_VAR=1
#
END_LOG
################################################################################
# Header of RTP file
################################################################################
printf IMPORTMVS_RTP <<"RTP1";
<?xml version=\"1.0\" encoding=\"utf-8\"?>
<!DOCTYPE project SYSTEM  \"$ATTOLHOME/lib/project.dtd\" >
<project>
  <group>
    <node>
      <data>
        <name>$PROJECTNAME</name>
        <renameable>false</renameable>
        <deletable>false</deletable>
        <open>true</open>
      </data>
      <current_configuration>$TDPNAME</current_configuration>
      <checked>false</checked>
      <background>false</background>
      <configuration_list>
        <configuration>
          <data>
            <name>$TDPNAME</name>
            <renameable>true</renameable>
            <deletable>true</deletable>
            <open>false</open>
          </data>
        <executionFlag>7743</executionFlag>
        </configuration>
      </configuration_list>
    </node>
    <group_child>
      <application>
        <test>
          <node>
            <data>
              <name>$PROJECTNAME</name>
              <renameable>true</renameable>
              <deletable>true</deletable>
              <open>true</open>
            </data>
            <current_configuration>$TDPNAME</current_configuration>
            <checked>false</checked>
            <background>false</background>
            <perlscript_after_child>Application.pl</perlscript_after_child>
            <configuration_list>
              <configuration>
                <data>
                  <name>$TDPNAME</name>
                  <renameable>true</renameable>
                  <deletable>true</deletable>
                  <open>false</open>
                </data>
              </configuration>
            </configuration_list>
          </node>
          <test_child>
RTP1

################################################################################
# Header of INF file
################################################################################
printf IMPORTMVS_INF <<"INF1";
<?xml version=\"1.0\" encoding=\"utf-8\"?>
<!DOCTYPE project SYSTEM \"$ATTOLHOME/lib/project.dtd\" >
<objects>
  <folder>
    <data>
      <name>$PROJECTNAME</name>
      <renameable>false</renameable>
      <deletable>true</deletable>
      <open>true</open>
    </data>
    <filters></filters>
    <folder_child>
INF1

sub Subsitute
{
    my $Value0=shift;
    $Value0=~ s/\%(\w+)\%/\$$1/g;
    while ($Value0=~/\$\(?(\w+)\)?/) # $(<mot>)
#    while ($Value0=~/\$\(\w+\)/) # $(<mot>)
    {
	my $word = $1;
#	print LOG "#Have to substitute a value \"$Value0\"\n";
#	print LOG "#The word to substitute is \"$word\"\n";
	if (exists($ENV{$word}))
	{
	    if($RTRT_REPLACE_ENV_VAR){
		    print LOG "#Substituting \"",$ENV{$word},"\" for \"$word\"\n";
		    $Value0=~s/\$\(?\Q$word\E\)?/$ENV{$word}/; 
	#	    print LOG "#Line is now $Value0\n";
	    }else{
		    $Value0=~s/\$\(?\Q$word\E\)?/\[\$\]$word/; #remove () only 
	    }
	}
	else 
	{
	    if (exists($Macros{$word}))
	    {
		print LOG "#Substituting macro \"",$Macros{$word},"\" for \"$word\"\n";
		$value = $Macros{$word};
		$Value0=~s/\$\(?\Q$word\E\)?/$value/;
#		print LOG "#Line is now $Value0\n";
	    }
	    else 
	    {
		print LOG &PerlTranslate("ImportMakefile.pl", "#Warning: not found macro: \"%1\" assuming env var\n", "%1 is the name of the variable", $word);
		$Value0=~s/\$\(?(\Q$word\E)\)?/\[\$\]$1/;
	    }
	}
    }
    $Value0=~s/\[\$\]/\$/g;
    return $Value0;
}

sub MyFormat
{
    my $Value0 = $_;
    my ($dir,$name,$ext) = SplitFileName($Value0);

    # suppression des 'parasites' en debut ou fin de chaine... 
    $Value0 =~s/[:|+|=|\\]$//;
    $Value0 =~s/^[:|+|=]//;
    # absolute path...
    if ( !$dir || $dir=~/^\.[a-zA-Z0-9_]/ || $dir=~/^\.\.\\/ || $dir=~/^\.\.\// ){
	$Value0 = "$MKFDIR" . "$Value0";
    }
    if ( $dir=~/^\.[\/|\\]/ ) 
    {
	$Value0=~s/^\.[\/|\\]//;
	$Value0 = "$MKFDIR" . "$Value0";
    }
    # pb des "/" et "\"
    $Value0 =~ s/[\\|\/]/$PATH_SEPARATOR/g;
    # suppression des ".." dans le path
    my $Count = 0;
    while ($Value0 =~ /\.\./) {
	$Value0 =~ s/[\\\/][^\.\\\/]*[\\\/]\.\.//g;
	$Count++;
	last if ($Count > 50); # Make sure we never loop forever
    }
    return ($Value0);
}

my %LaListe;
my %Includes;
my %Defines;
my $compilerOptions;
my $linkerOptions;
my $ld;


sub readTheFile
{
my $mkFile=shift;
my @line_items;
my $line;
my @LineOptions;
my $cmdType;#1=look like a compiler command, 0 unknown: maybe linker cmd, 2 linker cmd

  print LOG "#reading the file $mkFile\n";
  open (MAKFILE,"$mkFile") || die &PerlTranslate("ImportMakefile.pl", "Cannot open %1", "%1 is the Makefile name", $mkFile);

  while(<MAKFILE>)
  {
    chomp;
    next if(/^\s*$/);
    next if(/^\s*\#/);
    s/^\s+/ /;
    $line.=$_;
    next if($line=~s/\\$//); #cumulate multiline as only one
    print LOG "\n#Processing line: ",$line,"\n";
    $line=~ s/^\s*set\s//i;
    if($line=~ m/^\s*(\w+)\s*=\s*(.+)\s*$/i) #look for macros \s*(set)
    {
	my $Name=$1;my $Value=$2;
#	my $Name=$2;my $Value=$3;
	if($Name eq "RTRT_REPLACE_ENV_VAR"){$RTRT_REPLACE_ENV_VAR=1;next;}
	$DEBUG && print LOG "#Found \"$Name\" and value \"$Value\"\n";
	$Value=Subsitute $Value; #pour cas d'1 macro dependante d'autre(s) 
	print LOG "#Defining \"$Name\" = \"$Value\"\n";
	$Value=~ s/\s+/ /g;
	$Macros{$Name}= $Value;
	$compilerOptions.=$Value if($Name=~ /^CFLAG/);
	$linkerOptions.=$Value if($Name=~ /^LD?FLAG/);
    }
    else
    {
	$Value=Subsitute $line; #retourne le chemin apres execution des MACROS
    }
    $line='';
    @line_items=splitArgs($Value);
    if(@line_items[0] eq "include"){
    	my $incFile=$MKFDIR.'/'.@line_items[1];
    	readTheFile($incFile) if(-f $incFile);
    	next;
    }
    $Value='';
    @LineOptions=();
    $cmdType=0;
    my $cmd;
    ($d,$cmd,$e)=SplitFileName(@line_items[0]);
    $cmdType=2 if($cmd eq $ld);
    #print LOG "#>>>cmd='$cmd' and linker is '$ld' ($cmdType)\n"; 
    foreach (@line_items){
    	$Value.=$_; 
	# Extract all the include directories if there are any.
	next if($Value=~ m,^\"?[/-][ID]$,); #concat next item
	next if($Value=~ m,=$,); #concat next item
	next if($Value=~ m,=\\?\"[^\"]*$,); # try to complete when string start but does not end
	print LOG "#Processing item: ",$Value,"\n";
	if ($Value =~ m,^(\"?)([/-]I(.+))\1,g) {
		my $flg=$2;
		my $I=$3;
		$I=~ s,[\\\/](\")?$,$1,;
		$I=~ s,[\(\)],,g;
		print LOG "#Include dir: ",$I,"\n";
		$Includes{$I}++;
		$compilerOptions=~s,\Q$flg\E,,;
		$cmdType=1;
	}elsif($Value =~ m,^(\"?)([/-]D(.+))\1,g) {
		my $flg=$2;
		$Defines{$3}++;
		print LOG "#Define: ",$3,"\n";
		$compilerOptions=~s,\Q$flg\E,,;
		$cmdType=1;
	}elsif($Value =~ m,^(\"?)([/-](.+))\1,) {
		#is it an option
		push(@LineOptions,$2) unless ($3=~ /^[oc]$/);
	}else{
		# if not an option then it could be a file
		my @sources=split /:(?=[^\\])/,$Value;#split on the : which is not a C:\.....

		foreach (@sources)
		{
			s,^\",,;s,\"$,,;
			$Value=$_;
			next unless($Value);
			next if($Value=~ m/\$[@\<\>\*]/);
			if(/\$\w*\.\w{1,3}$/){
			    	# do not take fileName as environment variable
			    	print LOG "#Unavailable file \"$_\"\n";
			}else{

				$DEBUG && print LOG "#Look for Source: $Value\n";
				if ($Value=~/[-a-zA-Z0-9_]\.(cpp|c|C|ada|ads|adb|java)$/)
				{
				    my $ext=$1;
				    $Value=~s/.*[^\$\~ -\.a-zA-Z0-9\\\/_:]([\$\~ -\.a-zA-Z0-9\\\/_:]+\.$ext)$/$2/;
				    !$LaListe{$Value} && print LOG "#Adding \"$Value\" to the list of files\n";
				    $LaListe{$Value}=1;
				}
			}
		}
	}
	$Value='';
    }
    if($cmdType == 1){
        foreach my $opt (@LineOptions){
		$compilerOptions.=' '.$opt unless($compilerOptions=~ m/\Q$opt\E/);
	}
	print LOG "#found compiler options as $compilerOptions\n";
    } elsif($cmdType == 2){
        foreach my $opt (@LineOptions){
		$linkerOptions.=' '.$opt unless($linkerOptions=~ m/\Q$opt\E/);
	}
	print LOG "#found Linker options as $linkerOptions\n";
    } elsif(@LineOptions) {
	print LOG "#found options for unknow cmd $cmd",join(' ',@LineOptions),"\n";
    }
  }
close( MAKFILE );
}


my $d,$e;
$Ini{LINKER}=$Ini{LINK} unless($Ini{LINKER});
($d,$ld,$e)=SplitFileName($Ini{LINKER});
print LOG "#TDP ",$TDPNAME,"\n";

readTheFile($IMPORTFILE);

#Recover the includes that have been found:
$INCLUDE_USR_LIST = join (',', keys %Includes);
my $DEFINES_MACRO=join (',', keys %Defines);
$TCFLAGS.=',' if($DEFINES_MACRO ne '' && $TCFLAGS ne '');
$TCFLAGS.=$DEFINES_MACRO;
print LOG "-I",join("\n-I",keys %Includes),"\n";
print LOG "-D".join("\n-D",keys %Defines),"\n";
print &PerlTranslate("ImportMakefile.pl", "found macro define as: \"%1\"\n", "%1 is the Makefile name", $DEFINES_MACRO);
print &PerlTranslate("ImportMakefile.pl", "found include paths as: \"%1\"\n", "%1 is the Makefile name", $INCLUDE_USR_LIST);
if($compilerOptions){
	$CFLAGS=$compilerOptions ;
	print &PerlTranslate("ImportMakefile.pl", "found compilation flags as: \"%1\"\n", "%1 is the Makefile name", $compilerOptions); 
}
if($linkerOptions){
	$LDFLAGS=$linkerOptions ;
	print &PerlTranslate("ImportMakefile.pl", "found link flags as: \"%1\"\n", "%1 is the Makefile name", $linkerOptions); 
}
$CFLAGS=~s/\s\s+/ /g;
$LDFLAGS=~s/\s\s+/ /g;
print LOG "#compilation flags:\nCFLAGS=$CFLAGS\n";
print LOG "#link flags:\nLDFLAGS=$LDFLAGS\n";

###############################################################################
#  Suite du fichier INF
###############################################################################
printf IMPORTMVS_INF <<"INF2" ;
      <folder>
        <data>
          <name>Scripts</name>
          <renameable>true</renameable>
          <deletable>true</deletable>
          <open>true</open>
        </data>
        <filters>*.ptu;*.pts;*.otd;*.otc;*.dcl;*.stb</filters>
        <folder_child>
INF2

foreach (sort keys %LaListe) {
    $DEBUG && print LOG "#Treating the file \"$_\"\n";
    $DEBUG && print LOG "#Value before MyFormat = $_\n";
    $Value = MyFormat $_;

    print LOG "$Value\n";
    #if ( stat($Value) ) {
	my ($dir1,$name1,$ext1) = SplitFileName($Value);
	#if($ext1 =~ "cpp" or $ext1 =~ "rc") {
	if($ext1 =~ "h" or $ext1 =~ "hxx" or $ext1 =~ "c" or $ext1 =~ "cpp" or
	   $ext1 =~ "cxx" or $ext1 =~ "cc" or $ext1 =~ "java" or $ext1 =~ "ada" or
	   $ext1 =~ "adb" or $ext1 =~ "ads" or $ext1 =~ "spv" or $ext1 =~ "mat" or
	   $ext1 =~ "mdl" or $ext1 =~ "rc")
	{
	    $source = $Value;
	    printf IMPORTMVS_RTP "            <source>\n";
	    printf IMPORTMVS_RTP "              <nodefile_ref>\n";
	    printf IMPORTMVS_RTP "                <node>\n";
	    printf IMPORTMVS_RTP "                  <data>\n";
	    printf IMPORTMVS_RTP "                    <name>$name1.$ext1</name>\n";
	    printf IMPORTMVS_RTP "                    <renameable>false</renameable>\n";
	    printf IMPORTMVS_RTP "                    <deletable>true</deletable>\n";
	    printf IMPORTMVS_RTP "                    <open>false</open>\n";
	    printf IMPORTMVS_RTP "                  </data>\n";
	    printf IMPORTMVS_RTP "                  <current_configuration>$TDPNAME</current_configuration>\n";
	    printf IMPORTMVS_RTP "                  <checked>false</checked>\n";
	    printf IMPORTMVS_RTP "                  <background>false</background>\n";
	    printf IMPORTMVS_RTP "                  <perlscript_after_child>Source.pl</perlscript_after_child>\n";
	    printf IMPORTMVS_RTP "                  <configuration_list>\n";
	    printf IMPORTMVS_RTP "                    <configuration>\n";
	    printf IMPORTMVS_RTP "                      <data>\n";
	    printf IMPORTMVS_RTP "                        <name>$TDPNAME</name>\n";
	    printf IMPORTMVS_RTP "                        <renameable>true</renameable>\n";
	    printf IMPORTMVS_RTP "                        <deletable>true</deletable>\n";
	    printf IMPORTMVS_RTP "                        <open>true</open>\n";
	    printf IMPORTMVS_RTP "                      </data>\n";
	    printf IMPORTMVS_RTP "                    </configuration>\n";
	    printf IMPORTMVS_RTP "                  </configuration_list>\n";
	    printf IMPORTMVS_RTP "                </node>\n";
	    printf IMPORTMVS_RTP "                <filepath>$source</filepath>\n";
	    printf IMPORTMVS_RTP "              </nodefile_ref>\n";
	    printf IMPORTMVS_RTP "              <integrated>false</integrated>\n";
	    printf IMPORTMVS_RTP "              <source_child>\n";
	    printf IMPORTMVS_RTP "              </source_child>\n";
	    printf IMPORTMVS_RTP "            </source>\n";
	}
	if($ext1 =~ "ptu" or $ext1 =~ "pts" or $ext1 =~ "otd" or $ext1 =~ "otc"
	   or $ext1 =~ "dcl" or $ext1 =~ "stb") {
	    printf IMPORTMVS_INF "          <file>\n";
	    printf IMPORTMVS_INF "            <tested_tag>\n";
	    printf IMPORTMVS_INF "              <tag>\n";
	    printf IMPORTMVS_INF "                <data>\n";
	    printf IMPORTMVS_INF "                  <name>$name1.$ext1</name>\n";
	    printf IMPORTMVS_INF "                  <renameable>true</renameable>\n";
	    printf IMPORTMVS_INF "                  <deletable>true</deletable>\n";
	    printf IMPORTMVS_INF "                  <open>true</open>\n";
	    printf IMPORTMVS_INF "                </data>\n";
	    printf IMPORTMVS_INF "                <filepath>$MKFDIR$name1.$ext1</filepath>\n";
	    printf IMPORTMVS_INF "                <line>1</line>\n";
	    printf IMPORTMVS_INF "                <qualifier>0</qualifier>\n";
	    printf IMPORTMVS_INF "                <tag_child>\n";
	    printf IMPORTMVS_INF "                </tag_child>\n";
	    printf IMPORTMVS_INF "              </tag>\n";
	    printf IMPORTMVS_INF "            </tested_tag>\n";
	    printf IMPORTMVS_INF "            <filepath>$dir1$name1.$ext1</filepath>\n";
	    printf IMPORTMVS_INF "            <last_modification></last_modification>\n";
	    printf IMPORTMVS_INF "          </file>\n";

	}
    #}
}

printf IMPORTMVS_INF <<"INF4";
        </folder_child>
      </folder>
      <folder>
        <data>
          <name>Sources</name>
          <renameable>true</renameable>
          <deletable>true</deletable>
          <open>true</open>
        </data>
        <filters>*.h;*.hxx;*.c;*.cpp;*.cxx;*.cc;*.java;*.ada;*.adb;*.ads;*.spv;*.mat;*.mdl;*.rc;*</filters>
        <folder_child>
INF4

foreach (sort keys %LaListe) {
    $Value = MyFormat $_;
    $DEBUG && print LOG "#File for .inf file = $Value\n";
	my ($dir1,$name1,$ext1) = SplitFileName($Value);
	$DEBUG && print LOG "#split dir = $dir1 , name = $name1 , ext = $ext1\n";
	#if($ext1 =~ "h" or $ext1 =~ "hxx" or $ext1 =~ "c" or $ext1 =~ "cpp" or
	#   $ext1 =~ "cxx" or $ext1 =~ "cc" or $ext1 =~ "java" or $ext1 =~ "ada" or
	#   $ext1 =~ "adb" or $ext1 =~ "ads" or $ext1 =~ "spv" or $ext1 =~ "mat" or
	#   $ext1 =~ "mdl" or $ext1 =~ "rc" or $ext1 =~ "" )
	#   {
	       printf IMPORTMVS_INF "          <file>\n";
	       printf IMPORTMVS_INF "            <tested_tag>\n";
	       printf IMPORTMVS_INF "              <tag>\n";
	       printf IMPORTMVS_INF "                <data>\n";
	       printf IMPORTMVS_INF "                  <name>$name1.$ext1</name>\n";
	       printf IMPORTMVS_INF "                  <renameable>true</renameable>\n";
	       printf IMPORTMVS_INF "                  <deletable>true</deletable>\n";
	       printf IMPORTMVS_INF "                  <open>true</open>\n";
	       printf IMPORTMVS_INF "                </data>\n";
	       printf IMPORTMVS_INF "                <filepath>$MKFDIR$name1.$ext1</filepath>\n";
	       printf IMPORTMVS_INF "                <line>1</line>\n";
	       printf IMPORTMVS_INF "                <qualifier>0</qualifier>\n";
	       printf IMPORTMVS_INF "                <tag_child>\n";
	       printf IMPORTMVS_INF "                </tag_child>\n";
	       printf IMPORTMVS_INF "              </tag>\n";
	       printf IMPORTMVS_INF "            </tested_tag>\n";
	       printf IMPORTMVS_INF "            <filepath>$dir1$name1.$ext1</filepath>\n";
	       printf IMPORTMVS_INF "            <last_modification></last_modification>\n";
	       printf IMPORTMVS_INF "          </file>\n";

	   #}
    #}
}

printf IMPORTMVS_INF <<"INF6";
        </folder_child>
      </folder>
      <folder>
        <data>
          <name>Dependencies</name>
          <renameable>true</renameable>
          <deletable>true</deletable>
          <open>true</open>
        </data>
        <filters></filters>
        <folder_child>
        </folder_child>
      </folder>
INF6

###############################################################################
#  Suite du fichier RTP
###############################################################################
printf IMPORTMVS_RTP <<"RTP3";
          </test_child>
        </test>
      </application>
    </group_child>
  </group>
  <object_file>$PROJECTNAME.inf</object_file>
  <root_configuration_list>
    <configuration>
      <data>
        <name>$TDPNAME</name>
        <renameable>true</renameable>
        <deletable>true</deletable>
        <open>true</open>
      </data>
      <computerName>local host</computerName>
      <computerAddress>127.0.0.1</computerAddress>
      <computerPort>10000</computerPort>
      <uploaderPort>7777</uploaderPort>
      <targetName>$TDPNAME</targetName>
      <targetPath>$TARGETPATH</targetPath>
      <targetFullPath>$TARGETDIR</targetFullPath>
      <iniFileName>$INIFILE</iniFileName>
      <sourceLanguage>$TDP_LANGUAGE</sourceLanguage>
      <sourceExt>$SRCEXT</sourceExt>
      <objectExt>$OBJEXT</objectExt>
      <binaryExt>$EXEEXT</binaryExt>
      <ansiMode>true</ansiMode>
      <version></version>
      <target></target>
      <exception>true</exception>
      <definitionFile></definitionFile>
      <intdir>$INTDIR</intdir>
      <outdir>$OUTDIR</outdir>
      <ge_user_incdir>$INCLUDE_USR_LIST</ge_user_incdir>
      <ge_incdir>$INCLUDE_DIR_LIST</ge_incdir>
      <ge_alk_file_name></ge_alk_file_name>
      <testProgram>&lt;default&gt;</testProgram>
      <arguments></arguments>
      <javaMainClass></javaMainClass>
      <ge_host_workdir>tmp</ge_host_workdir>
      <javaBuildJarFile>false</javaBuildJarFile>
      <javaJarFileName></javaJarFileName>
      <javaManifest></javaManifest>
      <javaJarOtherDir></javaJarOtherDir>
      <cppFlags>$CPPFLAGS</cppFlags>
      <cflags>$CFLAGS</cflags>
      <tcflags>$TCFLAGS</tcflags>
      <ldflags>$LDFLAGS</ldflags>
      <additionnalObjects>$LIBRARIES</additionnalObjects>
      <debug>false</debug>
      <ge_libdir>$LIBRARY_DIR_LIST</ge_libdir>
      <javaBootClasspath>$JAVA_BOOTCLASSPATH</javaBootClasspath>
      <javaClasspath></javaClasspath>
      <multiThreaded>false</multiThreaded>
      <bufferSize>1024</bufferSize>
      <maxThreads>64</maxThreads>
      <atc_rungcatexit>true</atc_rungcatexit>
      <mainProc></mainProc>
      <clockKind>0</clockKind>
      <sourceFile>&lt;default&gt;</sourceFile>
      <otiFile>&lt;default&gt;</otiFile>
      <xrdFile>&lt;default&gt;</xrdFile>
      <nuerror>30</nuerror>
      <nodline>true</nodline>
      <atobreakonerror>false</atobreakonerror>
      <atonopassdump>false</atonopassdump>
      <atomaxinstances>64</atomaxinstances>
      <printbuffer>false</printbuffer>
      <printbuffersize>1024</printbuffersize>
      <atkbreakonerror>false</atkbreakonerror>
      <atknopassdump>false</atknopassdump>
      <atkreflexivestate>true</atkreflexivestate>
      <atkconstmethods>false</atkconstmethods>
      <atkreentrant>true</atkreentrant>
      <atkstrictcheck>false</atkstrictcheck>
      <st_interface_files></st_interface_files>
      <st_allocation></st_allocation>
      <st_notShared>false</st_notShared>
      <st_thread>false</st_thread>
      <st_thread_function></st_thread_function>
      <st_trace_circular>false</st_trace_circular>
      <st_trace></st_trace>
      <st_bufsize>10</st_bufsize>
      <st_values></st_values>
      <st_time>false</st_time>
      <st_dynfol>false</st_dynfol>
      <st_item_buf>false</st_item_buf>
      <st_item_buf_size>8192</st_item_buf_size>
      <st_dtrack>false</st_dtrack>
      <st_dtrack_buf_size>4096</st_dtrack_buf_size>
      <st_interrecv_timeout>300</st_interrecv_timeout>
      <st_custom>false</st_custom>
      <st_custom_type>0</st_custom_type>
      <fdc_name_opt>0</fdc_name_opt>
      <fdc_value></fdc_value>
      <tsf_name_opt>0</tsf_name_opt>
      <tsf_value></tsf_value>
      <tsf_selective_file></tsf_selective_file>
      <tsf_selective_class></tsf_selective_class>
      <exclude_dir></exclude_dir>
      <dump_incoming_list></dump_incoming_list>
      <dump_returning_list></dump_returning_list>
      <dump_calling_list></dump_calling_list>
      <no_header_instr>true</no_header_instr>
      <instantiate_template>false</instantiate_template>
      <misc_options></misc_options>
      <atc_instr_functions>1</atc_instr_functions>
      <atc_instr_calls>1</atc_instr_calls>
      <atc_instr_blocks>2</atc_instr_blocks>
      <atc_instr_conditions>3</atc_instr_conditions>
      <atc_tio_name>&lt;default&gt;</atc_tio_name>
      <atc_selective_no_ternary>true</atc_selective_no_ternary>
      <atc_selective_no_inline>true</atc_selective_no_inline>
      <atc_selective_no_included>true</atc_selective_no_included>
      <atc_selective_functions_list></atc_selective_functions_list>
      <atc_selective_spec>false</atc_selective_spec>
      <atc_information>0</atc_information>
      <atc_instr_ctrl_prefixe></atc_instr_ctrl_prefixe>
      <atc_instr_ctrl_suffixe>0</atc_instr_ctrl_suffixe>
      <atc_no_metrics>true</atc_no_metrics>
      <atc_user_comment></atc_user_comment>
      <atc_dump_driver>false</atc_dump_driver>
      <att_tdf_name>&lt;default&gt;</att_tdf_name>
      <att_calling_sequenced>false</att_calling_sequenced>
      <att_aggregate_traces>true</att_aggregate_traces>
      <att_template_notes>true</att_template_notes>
      <att_split>true</att_split>
      <att_split_size>5000</att_split_size>
      <att_split_prefixe>&lt;default&gt;</att_split_prefixe>
      <att_loop>false</att_loop>
      <att_loop_options></att_loop_options>
      <att_record_max_stack>true</att_record_max_stack>
      <att_on_fly>false</att_on_fly>
      <att_item_buffer>false</att_item_buffer>
      <att_item_buffer_size>64</att_item_buffer_size>
      <att_partial_dump>false</att_partial_dump>
      <att_signal_action>0</att_signal_action>
      <att_time_stamp>true</att_time_stamp>
      <att_heap_size>true</att_heap_size>
      <att_thread_info>true</att_thread_info>
      <atp_tpf_name>&lt;default&gt;</atp_tpf_name>
      <atp_instr_ctrl></atp_instr_ctrl>
      <atp_errors_fiu>true</atp_errors_fiu>
      <atp_errors_miu>false</atp_errors_miu>
      <atp_errors_sig>true</atp_errors_sig>
      <atp_fmwl>true</atp_fmwl>
      <atp_fmwl_queue_len>100</atp_fmwl_queue_len>
      <atp_fmwl_queue_size>1024</atp_fmwl_queue_size>
      <atp_abwl>true</atp_abwl>
      <atp_abwl_zone_size>16</atp_abwl_zone_size>
      <atp_call_stack>6</atp_call_stack>
      <javaObjectHashSize>1</javaObjectHashSize>
      <javaClassHashSize>1</javaClassHashSize>
      <javaTrigger>METHOD</javaTrigger>
      <javaHitKey>true</javaHitKey>
      <javaNextGC>true</javaNextGC>
      <javaStudioIPAdress>localhost</javaStudioIPAdress>
      <javaStudioPortNb></javaStudioPortNb>
      <javaTpfFileName></javaTpfFileName>
      <javaTsfFileName></javaTsfFileName>
      <javaAnalyze></javaAnalyze>
      <javaAnalyzeAlloc></javaAnalyzeAlloc>
      <javaFilterOption>WITH</javaFilterOption>
      <atq_tqf_name>&lt;default&gt;</atq_tqf_name>
      <atq_dump_driver>false</atq_dump_driver>
      <attolClassName></attolClassName>
      <intermediateFile>&lt;default&gt;</intermediateFile>
      <simulation>false</simulation>
      <preproerrorlevel>true</preproerrorlevel>
      <breakontest>false</breakontest>
      <preparationOptions></preparationOptions>
      <moreoptions>false</moreoptions>
      <filter>0</filter>
      <strategy>0</strategy>
      <reportOutlay>0</reportOutlay>
      <moreRoptions>false</moreRoptions>
      <reportoptions></reportoptions>
      <noComparaison>true</noComparaison>
      <instrumentedComparaison>false</instrumentedComparaison>
      <secondContext>false</secondContext>
      <secondTarget>true</secondTarget>
      <secondIntermediateFile>false</secondIntermediateFile>
      <strictComparaison>false</strictComparaison>
      <comparaison>false</comparaison>
      <userCommandLine></userCommandLine>
    </configuration>
    <javaStub>true</javaStub>
    <javaDisplay>false</javaDisplay>
    <javaReport></javaReport>
    <j2meRestriction>$J2ME_RESTRICTION</j2meRestriction>
  </root_configuration_list>
  <version_file>1</version_file>
</project>
RTP3

###############################################################################
#  Fin d'ecriture du fichier INF
###############################################################################

printf IMPORTMVS_INF <<"INF7";
    </folder_child>
  </folder>
  <version_file>1</version_file>
</objects>
INF7


close( IMPORTMVS_RTP );
close( IMPORTMVS_INF );
close( LOG );

#print &PerlTranslate("ImportMakefile.pl", "\nDone.\n");
0;
