#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2004 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#               -*-  Import other project files -*-                            #
#                                                                              #
#                                                                              #
#  Script for importing a other projects into a super project.                 #
#                                                                              #
#  usage:                                                                      #
#       ImportSubProject.pl <environment file>                                 #
#                                                                              #
# Required global variables:                                                   #
#   $BINDIR		 : binary directory required for the PerlTranslate     #
#   $SCRIPTDIR           : directory of the scripts                            #
#   $IMPORTFILE          : key of the Makefile to be imported                  #
#   $PROJECTNAME         : name for the created project                        #
#   $TDPNAME
#   $TARGETDIR
#   $INIFILE
#   $OBJEXT
#   $EXEEXT
#   $SRCEXT
#   $CPPFLAGS
#   $CFLAGS
#   $TCFLAGS
#   $LDFLAGS
#   $LIBRARIES
#   $TDP_LANGUAGE
#   $INTDIR
#   $OUTDIR
#   $INCLUDE_DIR_LIST
#   $LIBRARY_DIR_LIST
#   $JAVA_BOOTCLASSPATH
#   $J2ME_RESTRICTION
#                                                                              #
#                                                                              #
################################################################################
#$Id: ImportToSuperProject.pl,v 1.6 2007/07/23 18:41:58 rfrance Exp $
################################################################################

use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings
require $ARGV[0]; #environment and global variables.

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

$DEBUG=1;
my %options;

###########################################################################

sub getProjects {
  my $rootDir = shift;
  my $projectsRef = shift;
  my $dir = shift;
      print ("Checking        \"$dir\", ");
  return if (! -d $dir);
      print ("Got, ");
  return if (! opendir (DIR, $dir));
      print ("Opened\n");
  my $baseDir = $dir;

  # Convert paths to unix format
#  $baseDir =~ s,\\,/,g;
  my @dirContents = readdir(DIR);
  closedir(DIR);

  foreach my $file (@dirContents) {
    next
      if (($file eq  ".") || ($file eq  ".."));
    push (@$projectsRef, "$baseDir/$file")
      if ($file =~ m/\.rtp$/ && $file ne $PROJECTNAME.".rtp");
    getProjects($rootDir, $projectsRef, "$baseDir/$file")
      if (-d "$baseDir/$file");
  }
}

###########################################################################

sub extractConfigurations {
  my @projects = @_;

  my %projectConfigurations;
  foreach (@projects) {
    # Open the project file and extract the configurations
    open (PRJ, $_) || die "Can not open the project file \"$_\" for reading.\n";
    my $projfile=$_;
    my $inRootConfiguration = 0;
    my $inConfiguration = 0;
    my $currentConfig;
    while (<PRJ>) {
      if (m,\<root_configuration_list\>,) {
	$inRootConfiguration = 1;
      } elsif ($inRootConfiguration && m,\<configuration\>,) {
	$inConfiguration = 1;
      } elsif ($inConfiguration &&  m,\</configuration\>,) {
	$inConfiguration = 0;
      } elsif ($inRootConfiguration && m,\</configuration\>,) {
	$inConfiguration = 0;

      } elsif ($inConfiguration && m,\<name\>,) {
	# Get the configuration name
	m,\<name\>([^\<]*)\</name\>,;
	$currentConfig = $1;
	$projectConfigurations{$currentConfig} = {};
	$projectConfigurations{$currentConfig}{name} = $currentConfig;
	$options{config} = $1 unless ($options{config});

      } elsif ($inConfiguration && m,\<targetName\>,) {
	# Get the TDP name in the configuration
	m,\<targetName\>([^\<]*)\</targetName\>,;
	$projectConfigurations{$currentConfig}{targetName} = $1;

      }  elsif ($inConfiguration && m,\<targetPath\>,) {
	# Get the relative path to the target in the configuration
	m,\<targetPath\>([^\<]*)\</targetPath\>,;
	$projectConfigurations{$currentConfig}{targetPath} = $1;
	$options{tdpdir} = $1 unless ($options{tdpdir});

      }  elsif ($inConfiguration && m,\<targetFullPath\>,) {
	# Get the full path to the target in the configuration
	m,\<targetFullPath\>([^\<]*)\</targetFullPath\>,;
	$projectConfigurations{$currentConfig}{targetFullPath} = $1;

      }  elsif ($inConfiguration && m,\<iniFileName\>,) {
	# Get the full path to the target in the configuration
	m,\<iniFileName\>([^\<]*)\</iniFileName\>,;
	$projectConfigurations{$currentConfig}{iniFileName} = $1;

      }
      
    }
    close PRJ;
	# The tdp directory comes from the TARGETDIR

	# The Config comes from the TDPNAME variable

  }
  return %projectConfigurations;
}

###########################################################################

sub generateProject {
  my $optionsRef = shift;
  my $projectList = shift;
  my $configsRef = shift;

  # get the information from the TDP
  $ENV{ATLTGT}=$optionsRef->{tdpdir};
#  print "ATLTGT=$ENV{ATLTGT}\n";
#  DecodeIniFile();
#  $Ini{tp_filename} = $ENV{'ATLTGT'}.'/'.$INIFILE;
  my $tdpbaseDir = $optionsRef->{tdpdir};
  # Convert path to unix format path
  $tdpbaseDir =~ s,\\,/,g;
  # Remove trailing slash if there is one
  $tdpbaseDir =~ s,/$,,g;
  # Remove everything upto the last slash, what is left is the directory of the tdp
  $tdpbaseDir =~ s,(.*)/,,;
  my $tdpBaseDir = $1;

  my $projectFile = "$optionsRef->{rootdir}/$optionsRef->{project}";
  print "projectFile = $projectFile\n";
  if (-f $projectFile) {
    # rename existing project file
    rename ($projectFile, $projectFile.".bak") || die "\"$projectFile\" already exists and can not be renamed.\n";
  }
  open (PRJ, ">$projectFile") || die "Can not open the project file \"$projectFile\"\n";
  $ENV{TESTRTDIR} = 'C:\Program Files\Rational\TestRealTime' unless ($ENV{TESTRTDIR});
  $projectFile =~ m,.*/(.*)\.rtp$,;
  my $projectName = $1;
  print PRJ <<"END";
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE project SYSTEM "$ENV{TESTRTDIR}\\lib\\project.dtd" >
<project>
  <group>
    <node>
      <data>
        <name>$projectName</name>
        <renameable>false</renameable>
        <deletable>false</deletable>
        <open>true</open>
        <sorted>false</sorted>
      </data>
      <current_configuration>$optionsRef->{config}</current_configuration>
      <state>0</state>
      <background>false</background>
      <configuration_list>
        <configuration>
          <name>$optionsRef->{config}</name>
          <executionFlag>7743</executionFlag>
        </configuration>
END
  foreach my $c (sort keys %$configsRef) {
    %Ini={};
#    print "configName = $c\n";
    my $config = $configsRef->{$c};
    print PRJ <<"END";
        <configuration>
          <name>$config->{name}</name>
          <executionFlag>7743</executionFlag>
        </configuration>
END
  }
  print PRJ <<"END";
      </configuration_list>
    </node>
    <group_child>
END
  foreach (@$projectList) {
    my $projectFile = $_;
    $projectFile =~ m,.*[/\\](.*)\.rtp$,;
    my $projectName = $1;
    my $rootdir =$optionsRef->{rootdir};
#    print "Substituting \"$rootdir\" for \".\"\n";
    print "Substituting ", $projectFile;
    $projectFile =~ s/\Q$rootdir\E/./;
    print " by ", $projectFile, "\n";
    print PRJ <<"END";
      <sub_project>
        <data>
          <name>$projectName</name>
          <renameable>false</renameable>
          <deletable>true</deletable>
          <open>true</open>
          <sorted>false</sorted>
        </data>
        <project_file>$projectFile</project_file>
        <unload>false</unload>
      </sub_project>
END
  }
  print PRJ <<"END";
    </group_child>
  </group>
  <object_file></object_file>
  <sort_method>1</sort_method>
  <root_configuration_list>
END
#  print "configsRef = $configsRef\n";
#  print "keys\t: ", join (', ', keys %$configsRef),"\n";
  foreach my $c (sort keys %$configsRef) {
    %Ini={};
    print "configName = $c\t";
    my $config = $configsRef->{$c};
#    print "config = $config\n";
    $ENV{ATLTGT}=$ENV{TESTRTDIR}."/targets/".$config->{targetPath};
    if (-d $config->{targetFullPath}) {
      $ENV{ATLTGT}=$config->{targetFullPath};
    } else {
    	foreach my $d (@TDPDIRS) {
#	    	print "looking $d\n";
      		if(-d $d."/".$config->{targetPath}) {
      			$ENV{ATLTGT}=$d."/".$config->{targetPath};
      			last;
      		}
      	}
    }
    $INIFILE = $config->{iniFileName};
    print "Cannot find Ini file \"", $ENV{'ATLTGT'}.'/'.$INIFILE, "\"\n" unless (-f $ENV{'ATLTGT'}.'/'.$INIFILE);
    next unless (-f $ENV{'ATLTGT'}.'/'.$INIFILE);
    print "TDP found in ",$ENV{'ATLTGT'}."\n";
    DecodeIniFile();
    $Ini{tp_filename} = $ENV{'ATLTGT'}.'/'.$INIFILE;
    $Ini{DEFAULT_BOOTCLASSPATH} =~ s/\</\&lt;/;
    $Ini{DEFAULT_BOOTCLASSPATH} =~ s/\>/\&gt;/;
    print PRJ <<"END";
    <configuration>
      <name>$config->{name}</name>
      <targetName>$Ini{NAME}</targetName>
      <executionFlag>63</executionFlag>
      <envVariables></envVariables>
      <ASMFLAGS></ASMFLAGS>
      <cppFlags>$Ini{DEFAULT_CPPFLAGS}</cppFlags>
      <cflags>$Ini{DEFAULT_CFLAGS}</cflags>
      <tcflags></tcflags>
      <ge_incdir></ge_incdir>
      <ge_alk_file_name></ge_alk_file_name>
      <javaBootClasspath>$Ini{DEFAULT_BOOTCLASSPATH}</javaBootClasspath>
      <javaClasspath></javaClasspath>
      <ldflags>$Ini{DEFAULT_LDFLAGS}</ldflags>
      <additionnalObjects>$Ini{DEFAULT_LIBS}</additionnalObjects>
      <ge_libdir>$Ini{DEFAULT_LIBRARY_PATHS}</ge_libdir>
      <testProgram>&amp;lt;default&amp;gt;</testProgram>
      <javaBuildJarFile>false</javaBuildJarFile>
      <javaJarFileName>&amp;lt;default&amp;gt;</javaJarFileName>
      <javaManifest></javaManifest>
      <javaJarOtherDir></javaJarOtherDir>
      <generationFormat>DLL</generationFormat>
      <arguments></arguments>
      <virtualMachineArguments></virtualMachineArguments>
      <javaMainClass></javaMainClass>
      <mainProc></mainProc>
      <bufferSize>1024</bufferSize>
      <clockKind>0</clockKind>
      <multiThreaded>false</multiThreaded>
      <maxThreads>64</maxThreads>
      <TPLIB_CCFLAGS></TPLIB_CCFLAGS>
      <TPLIB_LDFLAGS></TPLIB_LDFLAGS>
      <runtimeFormat>OBJ</runtimeFormat>
      <UseDll>NONE</UseDll>
      <atc_rungcatexit>true</atc_rungcatexit>
      <computerName>localhost</computerName>
      <computerAddress>127.0.0.1</computerAddress>
      <computerPort>10000</computerPort>
      <uploaderPort>7777</uploaderPort>
      <intdir>$config->{targetPath}</intdir>
      <outdir>$config->{targetPath}</outdir>
      <targetName>$Ini{NAME}</targetName>
      <targetPath>$config->{targetPath}</targetPath>
      <targetFullPath>$config->{targetFullPath}</targetFullPath>
      <iniFileName>$INIFILE</iniFileName>
      <sourceLanguage>$Ini{LANGUAGE}</sourceLanguage>
      <objectExt>obj</objectExt>
      <libraryStaticExt>$Ini{LIBEXT}</libraryStaticExt>
      <libraryDynamicExt>$Ini{DLLEXT}</libraryDynamicExt>
      <binaryExt>$Ini{EXEEXT}</binaryExt>
      <sourceExt>$Ini{SRCEXT}</sourceExt>
      <j2meRestriction>false</j2meRestriction>
      <sourceDirectories></sourceDirectories>
      <structDefinition>true</structDefinition>
      <csvSeparator>;</csvSeparator>
      <csvDecimalSeparator>#comma#</csvDecimalSeparator>
      <csvThousandSeparator></csvThousandSeparator>
      <atc_selective_no_inline>true</atc_selective_no_inline>
      <atc_selective_no_included>true</atc_selective_no_included>
      <tsf_selective_class></tsf_selective_class>
      <tsf_selective_file></tsf_selective_file>
      <exclude_dir></exclude_dir>
      <dump_incoming_list></dump_incoming_list>
      <dump_returning_list></dump_returning_list>
      <dump_calling_list></dump_calling_list>
      <fdc_name_opt>0</fdc_name_opt>
      <fdc_value></fdc_value>
      <tsf_name_opt>0</tsf_name_opt>
      <tsf_value></tsf_value>
      <no_header_instr>true</no_header_instr>
      <instantiate_template>false</instantiate_template>
      <misc_options></misc_options>
      <atc_instr_functions>1</atc_instr_functions>
      <atc_instr_calls>1</atc_instr_calls>
      <atc_instr_blocks>2</atc_instr_blocks>
      <atc_instr_conditions>3</atc_instr_conditions>
      <atc_selective_no_ternary>true</atc_selective_no_ternary>
      <atc_information>0</atc_information>
      <atc_instr_ctrl_prefixe></atc_instr_ctrl_prefixe>
      <atc_instr_ctrl_suffixe>0</atc_instr_ctrl_suffixe>
      <atc_selective_spec>false</atc_selective_spec>
      <atc_selective_functions_list></atc_selective_functions_list>
      <atc_tio_name>&amp;lt;default&amp;gt;</atc_tio_name>
      <atc_no_metrics>false</atc_no_metrics>
      <atc_user_comment></atc_user_comment>
      <atp_errors_fiu>true</atp_errors_fiu>
      <atp_errors_miu>false</atp_errors_miu>
      <atp_errors_sig>true</atp_errors_sig>
      <atp_fmwl>true</atp_fmwl>
      <atp_fmwl_queue_len>100</atp_fmwl_queue_len>
      <atp_fmwl_queue_size>1024</atp_fmwl_queue_size>
      <atp_abwl>true</atp_abwl>
      <atp_abwl_zone_size>16</atp_abwl_zone_size>
      <atp_call_stack>6</atp_call_stack>
      <javaObjectHashSize>1</javaObjectHashSize>
      <javaClassHashSize>1</javaClassHashSize>
      <javaTrigger>METHOD</javaTrigger>
      <javaHitKey>false</javaHitKey>
      <javaNextGC>false</javaNextGC>
      <javaStudioIPAdress>localhost</javaStudioIPAdress>
      <javaStudioPortNb></javaStudioPortNb>
      <javaTpfFileName>&amp;lt;default&amp;gt;</javaTpfFileName>
      <javaTsfFileName>&amp;lt;default&amp;gt;</javaTsfFileName>
      <javaPackageAnalyze></javaPackageAnalyze>
      <javaAnalyze></javaAnalyze>
      <javaAnalyzeAlloc></javaAnalyzeAlloc>
      <javaFilterOption>WITHOUT</javaFilterOption>
      <atp_tpf_name>&amp;lt;default&amp;gt;</atp_tpf_name>
      <atp_instr_ctrl></atp_instr_ctrl>
      <checkfrequency>NONE</checkfrequency>
      <atq_tqf_name>&amp;lt;default&amp;gt;</atq_tqf_name>
      <att_tdf_name>&amp;lt;default&amp;gt;</att_tdf_name>
      <att_calling_sequenced>false</att_calling_sequenced>
      <att_aggregate_traces>true</att_aggregate_traces>
      <att_template_notes>true</att_template_notes>
      <att_split>true</att_split>
      <att_split_size>5000</att_split_size>
      <att_split_prefixe>&amp;lt;default&amp;gt;</att_split_prefixe>
      <att_loop>false</att_loop>
      <att_loop_options></att_loop_options>
      <att_record_max_stack>true</att_record_max_stack>
      <att_on_fly>false</att_on_fly>
      <att_item_buffer>false</att_item_buffer>
      <att_item_buffer_size>64</att_item_buffer_size>
      <att_partial_dump>false</att_partial_dump>
      <att_signal_action>0</att_signal_action>
      <att_time_stamp>true</att_time_stamp>
      <att_heap_size>true</att_heap_size>
      <att_thread_info>true</att_thread_info>
      <intermediateFile>&amp;lt;default&amp;gt;</intermediateFile>
      <preproerrorlevel>true</preproerrorlevel>
      <breakontest>false</breakontest>
      <simulation>false</simulation>
      <moreoptions>false</moreoptions>
      <preparationOptions></preparationOptions>
      <filter>0</filter>
      <strategy>0</strategy>
      <reportOutlay>0</reportOutlay>
      <comparaison>false</comparaison>
      <moreRoptions>false</moreRoptions>
      <reportoptions></reportoptions>
      <atkbreakonerror>false</atkbreakonerror>
      <atknopassdump>false</atknopassdump>
      <atkreflexivestate>true</atkreflexivestate>
      <atkconstmethods>false</atkconstmethods>
      <atkreentrant>true</atkreentrant>
      <atkstrictcheck>false</atkstrictcheck>
      <nodline>true</nodline>
      <atobreakonerror>false</atobreakonerror>
      <atomaxinstances>64</atomaxinstances>
      <atonopassdump>false</atonopassdump>
      <printbuffer>false</printbuffer>
      <printbuffersize>1024</printbuffersize>
      <sourceFile>&amp;lt;default&amp;gt;</sourceFile>
      <otiFile>&amp;lt;default&amp;gt;</otiFile>
      <xrdFile>&amp;lt;default&amp;gt;</xrdFile>
      <nuerror>30</nuerror>
      <javaStub>true</javaStub>
      <javaDisplay>true</javaDisplay>
      <st_interface_files></st_interface_files>
      <st_thread>false</st_thread>
      <st_thread_function></st_thread_function>
      <st_allocation>NONE</st_allocation>
      <st_notShared>false</st_notShared>
      <st_addOptions></st_addOptions>
      <st_trace>NONE</st_trace>
      <st_trace_circular>false</st_trace_circular>
      <st_bufsize>10</st_bufsize>
      <st_values>NONE</st_values>
      <st_repform>NONE</st_repform>
      <st_time>false</st_time>
      <st_rdv_timeout>300</st_rdv_timeout>
      <st_interrecv_timeout>300</st_interrecv_timeout>
      <ge_host_workdir>tmp</ge_host_workdir>
      <st_item_buf>false</st_item_buf>
      <st_item_buf_size>8192</st_item_buf_size>
      <st_dynfol>false</st_dynfol>
      <st_dtrack_buf_size>4096</st_dtrack_buf_size>
      <st_custom>false</st_custom>
      <st_custom_type>0</st_custom_type>
      <st_interface_files></st_interface_files>
      <st_replay_cmdline_args></st_replay_cmdline_args>
      <userCommandLine></userCommandLine>
      <metOneLevel>false</metOneLevel>
      <metRestrictDir></metRestrictDir>
    </configuration>
END
  }
    print PRJ <<"END";
  </root_configuration_list>
  <version_file>1</version_file>
</project>
END
  close (PRJ);
}

###########################################################################

sub main {
  # Parse the environment file to set up the options
  open (DEBUG, ">import.log") || die "Cannot open log file\n";
  open(STDOUT, ">&DEBUG");
  open(STDERR, ">&DEBUG");
  print "Reading from $ARGV[0]\n";
  open (ENV, $ARGV[0]);  while (<ENV>) { print $_; } close ENV;

  # For the moment the root directory is that of the project to create.
  $options{rootdir} = $IMPORTFILE;
  print "rootdir(raw) = $options{rootdir}\n";
  unless (-d $options{rootdir}) {
    # rootdir is a file so choose it's parent directory
    print "rootdir is a file so choose it's parent directory\n";
    $options{rootdir} =~ m,(.*)[/\\],;
    $options{rootdir} = $1;
  }
  print "rootdir = $options{rootdir}\n";
  die "$0: ERRROR: The root directory to search does not exist.\n".$options{rootdir}."\n" unless (-d $options{rootdir});

  # The project file to create comes from $PROJECTNAME
  $options{project} = $PROJECTNAME.".rtp";
  die "Missing project output name\n" unless ($options{project});

  foreach (sort keys %options) {
    print "$_\t = ", $options{$_}, "\n";
  }

  my @projects;
  getProjects($options{rootdir}, \@projects, $options{rootdir});
  print "\nFound projects:\n";
  foreach (@projects) {
    print "\t$_\n";
  }
  my %configs = extractConfigurations(@projects);
  print "\nFound Configurations:\n";
  foreach (sort keys %configs) {
    print $_, ":\n";
    print "\t", $configs{$_}->{name}, "\n";
    print "\t", $configs{$_}->{targetName}, "\n";
    print "\t", $configs{$_}->{targetPath}, "\n";
    print "\t", $configs{$_}->{iniFileName}, "\n";
  }

  print "\n Generate the project file\n";
  generateProject (\%options, \@projects, \%configs);
}

main();
