#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                       -*-  Java Testing Node -*-                             #
#                                                                              #
#                                                                              #
#  Script for Java Test Node.                                           #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   @OBJECTS       : object files                                              #
#   $TARGETDIR     : directory of the Target Package                           #
#   $TESTPROGRAM   : executable extension                                      #
#   $LDFLAGS       : options for the linker                                    #
#   $LIBRARIES     : libraries for the linker                                  #
#   $EXECMODE      : execution mode ('NORMAL' or 'DEBUG')                      #
#   $ARGUMENTS     : argument for the generated program execution.             #
#   $BINDIR        : directory of the studio/tracer binaries.                  #
#   $ATTOLHOME     : attol studio intallation directory.                       #
#   $ATTLOOPDETECT : activate loop detection utility for TDF generated file.   #
#   $ATTLOOPDETECTFLAGS: options for attloop                                   #
#   $ATTSPLIT      : active de decoupage des traces.                           #
#   $ATTSPLITSIZE  : taille (Kb) des fichier decoupes.                         #
#   $ATTSPLITPREFIX: prefix des fichiers generes.                              #
#                                                                              #
################################################################################
#$Id: JavaTesting.pl,v 1.31 2007/07/23 18:41:58 rfrance Exp $
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] ; #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";
require $SCRIPTDIR . "SplitJvmpi.pl";
require $SCRIPTDIR . "Products.pl";
require $TARGETDIR . "/cmd/atl_link.pl";
require $TARGETDIR . "/cmd/atl_exec.pl";

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
CheckIntDir();
EnterInScript();
ClearLogFile( $LOGFILENAME );

################################################################################
# Attol ObjecTest postprocessor execution
# Arguments:
#    $tdf    : traces file
#    $ots    : static correspondence files (space-separated list)
#    $xrd    : generated AML report
# Return value: atopostpro return value
# Required global variables:
#    $BINDIR
# Used global variables:
################################################################################
sub atj_postpro #($$$)
{
   my ($tsf,$tdf,$xrd) = @_;

   # command
   my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "javapospro -tsf $tsf -tdf \"$tdf\" -xrd \"$xrd\"";

   # execution
   my $Status = SystemP ($Command);
   return $Status;
}

$OUTFILE = "$ATL_OUT_FILE";
$OUTFILE =~ s/\\/\\\\/;

#----------------------------------------------------------------------------
## Link pass
if ( $STAGE =~ /l/ )
{

  # @OBJECTS is not used (not filled by the Source.pl because it is an
  # automatic compilation process
  if ($JAVA_BUILDJARFILE) {
    CheckOrCreateManifestFile();
    print &PerlTranslate("JavaTesting.pl", "Creating %1...\n", "%1 is ", $JAVA_JARFILENAME);
    $status = atl_link ( $JAVA_JARFILENAME, \@OBJECTS, $JAVA_MANIFEST,
                     \@JAVA_JAR_OTHER_DIR, $INTDIR );
    if ( $status != 0 ) { exit(1); }
  } else {
    exit 0;
  }
}

#----------------------------------------------------------------------------
## Execution
if ( $STAGE =~ /e/ )
{
    ($JAVA_MAIN_CLASS) or die &PerlTranslate("JavaTesting.pl", "Error: please update Execution settings with the name of the main class to be run.\n");
    unlink ($OUTFILE);
    print &PerlTranslate("JavaTesting.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $JAVA_MAIN_CLASS, $ARGUMENTS);
    $status = AtlExec ($JAVA_JARFILENAME, $JAVA_MAIN_CLASS, $ARGUMENTS);
    
    if ( $SOCKET_UPLOAD ) {
      # Split has been done by the Socket Uploader
      if ( $ATCMODE ) {
        if ( -f $ATC_TIO_NAME ) {
          open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("JavaTesting.pl", "Cannot open log file.\n");
          print  LOG "TIO $ATC_TIO_NAME\n";
          close( LOG );
        }
        # Postpro Coverage (TPM...)
        # atc_postpro($ATC_TIO_NAME,""); # no CIO in this node
      }
      if ( $ATTMODE && !$ATT_ON_THE_FLY )
      { # No attsplit nor attloop on the fly
        if ( -f $TDFFILE ) {
          open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("JavaTesting.pl", "Cannot open log file.\n");
          print  LOG "TDF $TDFFILE\n";
          close( LOG );
        }
        PostAttManage ($TDFFILE);
      }

      if ( $ATQMODE ) {
        if ( -f $ATQ_TQF_NAME ) {
          open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("JavaTesting.pl", "Cannot open log file.\n");
          print  LOG "TQF $ATQ_TQF_NAME\n";
          close( LOG );
        }
      }

    } else {

      SplitTraces ($OUTFILE); # LOG are made inside SplitTraces
          
      #---------------------------------------------------------------------
      ## Coverage check ...
      if ( $ATCMODE && ( ! $ATTMODE || ! $ATT_ON_THE_FLY ) )
      {
          if ( ! $used[$tio] )
          {
              print &PerlTranslate("JavaTesting.pl", "No Coverage results found.\n");
              #exit(1);
          }
          
          # Postpro Coverage (TPM...)
          atc_postpro($ATC_TIO_NAME,""); # no CIO in this node
      } 
      
      #---------------------------------------------------------------------
      ## Manage TDF Traces ...
      if ( $ATTMODE && ! $ATT_ON_THE_FLY )
      { # attsplit and/or attloop if and only if not on the fly
          if ( ! $used[$tdf] )
          {
              print &PerlTranslate("JavaTesting.pl", "No Tracing results found.\n");
              #exit(1);
          }
          PostAttManage ($TDFFILE);
      } #ATTMODE==1
      
      #---------------------------------------------------------------------
      ## Performance Profiling check ...
      if ( $ATQMODE )
      {
          if ( ! $used[$tqf] )
          {
              print &PerlTranslate("JavaTesting.pl", "No Prefpro results found.\n");
              #exit(1);
          }
      } #ATQMODE==1
    }
    
} #STAGE==execution

#STAGE end of execution.
if ($STAGE =~ /r/ )
{  
    # GD: modifcation of tempo
    my $wait_time = 2;
    if ($SOCKET_UPLOAD)
    {
	sleep(5 * $wait_time);
    }

    if (($ATTMODE && $ATT_ON_THE_FLY) || ($SOCKET_UPLOAD) )
    {

	# Initialization
	my $count_lines="0";
	my $start_count_line="0";
	my $start_time = time();
	my $current_time = time();
	my $end="0";

	# While the file grows do nothing.
	# when the file does not grow anymore, run the split (time out part).

	while($end!="1")
	{
	    if ( open(INPUTFL,$TDFFILE))
	    {
		$count_lines="0";
		while(<INPUTFL>)
		{
		    $count_lines++;
		}

		$current_time = time();
		
		#if has grown.
		if (($count_lines-$start_count_line) > 0)
		{
		    $start_count_line = $count_lines;
		    $count_lines="0";
		    $start_time = $current_time;
		} else {
		    #same file.
		    if (($current_time - $start_time) > $wait_time)
		    {
			$end="1";
		    }
		}
		if ($end != 1)
		{
		  sleep($wait_time);
		}
	    }
	    else
	    {
		#secure if there is no TDF file.
		sleep($wait_time);
		if ( open(INPUTFL,$TDFFILE))
	    	{
		} else {
		    $current_time = time();
		    if ($current_time - $start_time > $wait_time)
		    {
			$end="1";
		    }
		}
	    }
    	}
    }


    if ( $ATPMODE && $TDP_LANGUAGE eq "JAVA" ) 
    {
	SplitJvmpiTraces ();
    }
    
    print &PerlTranslate("JavaTesting.pl", "Test Report Generation...\n");
    my $tsflist = "";
    foreach $tsf ( @TSFFILES )
    {
        $tsflist .= " \"$tsf\"";
    }
    $tsflist .= " \"$TDFFILE\""; 
   
    $status = atj_postpro ($tsflist, $TDFFILE, $XRDFILE);
    
    if ($status != 0 ) { die &PerlTranslate("JavaTesting.pl", "Failed.\n"); }
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("JavaTesting.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
    print  LOG "XRD $XRDFILE\n";
    close( LOG );
}#STAGE==Report Generation
#    print &PerlTranslate("JavaTesting.pl", "Done.\n");

0;
# That's all Folks.
################################################################################













