##########################################################################
#                                                                        # #
# Licensed Materials - Property of IBM                                   #   #
# Restricted Material of IBM                                             #######
# IBM Rational Test RealTime                                                   #
#  Copyright IBM Corp. 1999,2004 All Rights Reserved.                         #
#                                                                              #
# US Government Users Restricted Rights - Use, duplication or                  #
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.            #
#                                                                              #
################################################################################
##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Library Node -*-                             #
#                                                                              #
#                                                                              #
#  Script for Library Test Node.                                           #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   @OBJECTS       : object files                                              #
#   $TARGETDIR     : directory of the Target Package                           #
#   $TESTPROGRAM   : executable extension                                      #
#   $LDFLAGS       : options for the linker                                    #
#   $LIBRARIES     : libraries for the linker                                  #
#   $EXECMODE      : execution mode ('NORMAL' or 'DEBUG')                      #
#   $ARGUMENTS     : argument for the generated program execution.             #
#   $BINDIR        : directory of the studio/tracer binaries.                  #
#   $ATTOLHOME     : attol studio intallation directory.                       #
#   $ATTLOOPDETECT : activate loop detection utility for TDF generated file.   #
#   $ATTLOOPDETECTFLAGS: options for attloop                                   #
#   $ATTSPLIT      : active de decoupage des traces.                           #
#   $ATTSPLITSIZE  : taille (Kb) des fichier decoupes.                         #
#   $ATTSPLITPREFIX: prefix des fichiers generes.                              #
#                                                                              #
################################################################################
#$Id: Library.pl,v 1.14 2007/07/23 18:41:58 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] ; #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Products.pl";
require $TARGETDIR . "/cmd/atl_link.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
CheckIntDir();
EnterInScript();
ClearLogFile( $LOGFILENAME );

my $ada_mode = 0;
# Here is the common to all components traces file name
if ( $TDP_LANGUAGE=~/^ADA/ )
{
  $ada_mode = 1;
  $OUTFILE = $ATL_OUT_FILE;

  if ( $ATL_MAIN_PROCEDURE eq "" )
  {
    die &PerlTranslate("Library.pl", "Error: please update Execution settings with the name of the main procedure.\n");
  }
} else {
  $OUTFILE = "$ATL_OUT_FILE";
}
$OUTFILE =~ s/\\/\\\\/;

my $java_mode=0;
if ( $TDP_LANGUAGE eq "JAVA" )
{
  $java_mode = 1 ;
}
#--------------------------------------------------------------------------
## Misra
if($MISRA_MODE && @XOB){
   my $res=codeChecker_link();
}

#----------------------------------------------------------------------------
## Link pass
if ( $STAGE =~ /l/ )
{
  $ATKMODE=1 if(@OTIFILES);

  #--------------------------------------------------------------------------
  ## do not forget object file of target package.
  if ( ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || $ATKMODE || $ATS_PROBE_CUSTOM) &&
       ($TDP_LANGUAGE eq "C" || $TDP_LANGUAGE eq "CPP") &&
       ( $ATL_RUNTIME_FORMAT ne "NONE"))
  {
    if (MakeTargetPackageObject()) {
      exit 1;
    }
    if($ATL_RUNTIME_FORMAT eq "OBJ") {
       push( @OBJECTS, GetTargetPackageLib($OBJEXT) );
    } else {
       push( @ADDITIONAL_LIBRARIES, GetTargetPackageLib($LIBEXT) );
    }
  }

#link with atlprobe.o for probe
 
  if ($ATS_PROBE_CUSTOM)
  {

      if ($ATS_PROBE_CUSTOM_TYPE==$mode{ignore})
      {
	  #none additional object files needed      
      }    
      else
      {
          ## Do not forget the OBJ file in output log
          open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Application.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
	  
          push( @OBJECTS, "$INTDIR" . "$PATH_SEPARATOR" . "atlprobe.$OBJEXT");
          print LOG "OBJ $INTDIR/atlprobe.$OBJEXT\n";

	  if (($ATS_PROBE_CUSTOM_TYPE==$mode{fifo}) || ($ATS_PROBE_CUSTOM_TYPE==$mode{file}) || ($ATS_PROBE_CUSTOM_TYPE==$mode{user}))
	  {
	      push( @OBJECTS, "$INTDIR" . "$PATH_SEPARATOR" . "probecst.$OBJEXT");
              print LOG "OBJ $INTDIR/probecst.$OBJEXT\n";
	  }
      
          close ( LOG );
      }
  }

  my $status = 0;
  my $result;
  if ($ada_mode)
  {
      print &PerlTranslate("Library.pl", "No link for Ada Library\n", "");
      exit 0;
      
  }
  elsif ($java_mode)
  {
      # @OBJECTS is not used (not filled by the Source.pl because it is an
      # automatic compilation process
      if ($JAVA_BUILDJARFILE) {
        CheckOrCreateManifestFile();
        print &PerlTranslate("Library.pl", "Creating %1...\n", "%1 is file to be created", $JAVA_JARFILENAME);
        $status = atl_link ( $JAVA_JARFILENAME, \@OBJECTS, "\"$JAVA_MANIFEST\"",
                             \@JAVA_JAR_OTHER_DIR, $INTDIR );
        $result=$JAVA_JARFILENAME;
      } else {
        exit 0;
      }
  }
  else
  {
      print &PerlTranslate("Library.pl", "Linking %1...\n", "%1 is file to be linked", $TESTPROGRAM);
      $status = atl_lib ( $TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                           \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES, $GENERATION_FORMAT );
        $result=$TESTPROGRAM;
  }
  EndStep($status);
  if ( $status != 0 ) { exit(1); }

  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Library.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "LIB $result\n";
  print  LOG "OTI ".join(',',@OTIFILES)."\n" if(@OTIFILES);
  close( LOG );
  
}



0;
# That's all Folks.
################################################################################








