#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                 -*-  Metrics Analysis  -*-                                   #
#                                                                              #
#                                                                              #
#  Script for Tracer Test Node.                                                #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME2        : name of the log file to generate.                   #
#   $TARGETDIR           : directory of the Target Package                     #
#   $BINDIR              : directory of the studio/tracer binaries.            #
#   $ATTOLHOME           : attol studio intallation directory.                 #
#   $FILENAME            : filename of the analysed file                       #
#   $METDIR              : output directory for the genreated metric files     #
#   $MET_ONE_LEVEL       : true if the one_level_metrics option is activated   #
#   $MET_RESTRICT_DIR    : analysed directories				       #
#                                                                              #
################################################################################
#$Id: Metrics.pl,v 1.29 2007/07/23 18:41:58 rfrance Exp $
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] ; #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "metricC.pl";
require $SCRIPTDIR . "metricA.pl";

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
EnterInScript();

ClearLogFile( $LOGFILENAME2 );


#----------------------------------------------------------------------------
## Metrics generation

# Defaults options
my $Command = "$BINDIR" . "$PATH_SEPARATOR";

  my ($file_dir, $file_name, $file_ext) =  SplitFileName($FILENAME);

#  my $keep = $Ini{'LANGUAGE'};
#  $Ini{'LANGUAGE'} = "DUMMY"; #ADA only : sinon LanguageOf renvoie le TDP_LANGUAGE ...
  my $language=LanguageOf( $file_ext , "");

  my $header_mode;
  if (uc($file_ext) eq "H")
  {
     $header_mode = 1;
     $language = $TDP_LANGUAGE; #by default
  }

  if (($language eq "PTU") || ($language eq "OTC") || ($language eq "OTD") || ($language eq "CSV") || (($language eq "") && !$header_mode))
  {
    #Nothing to do
    exit(0);  
  }

print &PerlTranslate("Metrics.pl", "Static metrics generation ...\n");


  if ($file_ext =~ /AD[ABS]/i) #on est donc en ADA
  {
     if (($TDP_LANGUAGE ne "C") && ($TDP_LANGUAGE ne "CPP") && ($TDP_LANGUAGE ne "JAVA"))
     {
       $language = $TDP_LANGUAGE;
#       $Ini{'LANGUAGE'} = $keep;
     }
     else #TDP non compatible
     {
       die &PerlTranslate("Metrics.pl", "No metrics analysis of %1. Please change your current configuration to ADA TDP.\n", "%1 is the name of the source file", $FILENAME);
     }
  }

  my $ada_mode = 0;
  if (($language eq "ADA") || ($language eq "ADA83"))
  {
     $ada_mode = 1;
  }

  # Control compatiblity with the TDP language
  if (!$ada_mode)
  {
     if (($TDP_LANGUAGE eq "CPP") && ($language ne "C") && ($language ne "CPP"))
     {
        die &PerlTranslate("Metrics.pl", "No metrics analysis of %1. Please change your current configuration to C++ TDP.\n", "%1 is the name of the source file", $FILENAME);
     }
  }

  if ($ada_mode && (($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP") || ($TDP_LANGUAGE eq "JAVA"))) 
  {
     die &PerlTranslate("Metrics.pl", "No metrics analysis of %1. Please change your current configuration to Ada TDP.\n", "%1 is the name of the source file", $FILENAME);
  }

  if ($header_mode && (($TDP_LANGUAGE ne "C") && ($TDP_LANGUAGE ne "CPP")))
  {
     die &PerlTranslate("Metrics.pl", "No metrics analysis of %1. Please change your current configuration to C/C++ TDP.\n", "%1 is the name of the source file", $FILENAME);
  }

  if(($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP")) {
	$status=metricsC ($TDP_LANGUAGE, $FILENAME, $LOGFILENAME2, $METDIR);
  } else {
	$status=metricsA ($TDP_LANGUAGE, $FILENAME, $LOGFILENAME2, $METDIR);
  }
0;
# That's all Folks.
################################################################################
