#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                   -*-  Naming Rule checker example -*-                       #
#                                                                              #
#                                                                              #
################################################################################
#$Id: NamingRules1.pl,v 1.4 2007/06/05 19:44:19 rfrance Exp $
################################################################################
#look at "Declaration and Access of Arrays of Arrays in perl" on the web to get hints
# on How to access and use fields.

############################ database array description ############################
# the database is in perl array format located in arrays @vars,@types,@fields,@enums.
# these arrays have the same structure and have one record per name. for each record:
#	field 0 is the name (string)
#	field 1 is the source fileNumber where it is declared (@files index)
#	field 2 is the source line where it is declared
#	field 3 is the scope for witch the declaration is visible (global,local,global static,local static,parameter) 
#		constante $scopeXXXXX must be used for comparison, to keep compliance with futur versions
#	field 4 is the based type. the type name used to define it. If the base type is negativ that means the C/C++ base type is used.
#		In this case the C/C++ base type name is provided in the @basetypes and can be used as 
#		$basetypes[-$type] to get the complete name, where $type is the content of the field 4
#	field 5 is the indirection level (string): * means it is a pointer for that type,  [ means this is an array of that type.
#	
#	few constant variables are set as well in the database file to access to these fields. 
#		There are ($namefield,$filefield,$linefield,$scopefield,$typefield,$indirectfield) respectively

# the database provide the list of all parsed file in the @files array
#	This allows to parse again those files in order to check your own rules, 
#	like the rate of comments per line of code, the file versions, the #define name and so on.
#	this array is made of 2 fields:
#	field 0 is the pathname file (string)
#	field 1 is set to 1 when the file is a system/compiler include. it is set to 0 otherwise.
#	


################################ Entry point #######################################
# this function is called at link time after generation of the Naming database.
# the Naming database is generated in perl format and initialize some variables and arrays
# on the "require" instruction of this file. (see database array description)
sub namingRuleCheck
{
my $names=shift;
require "$names"; # read the database

#generate the error result in the XOB file having the same basename as the perl database
my $outFile=$names;$outFile=~s/\.pl$/.xob/;
openXOB($outFile);

############################### Types Name checking ###############################
print "types name checking....\n";
#this example check the following name requirements
#All normal types schould start by t_. pointer types should prefix the normal type by p 
#All struct names schould start by s_.
#the char l should be used after the t_ when the based type is a long. 
#	u for unsigned int, i for signed int, f for float, d for double. 
#
foreach my $def (@types)
    {
	#read 1 record from the database and set local variables for each field
	my ($name,$fileIdx,$line,$scope,$type,$indirect)=@$def;
	
	next if($name eq "");# do not check unnamed typedef
	next if($files[$fileIdx][1]);# do not check system includes
	my ($deftype,$btype);
	if($type <0) {
		#this is a C/C++ base type
		$deftype=$basetypes[-$type];
		$btype=$deftype;
	} else {
		#this is a type based on an other type
		$deftype=$types[$type][$namefield];
		#look for the C/C++ base type of this type
		my $bt=$type;
		while($bt >=0) {
			$bt=$types[$bt][$typefield];
		}
		$btype=$basetypes[-$bt];
	}
#	print "#!info on type $name,found at $files[$fileIdx] line $line,scope $scope, type ($btype) $deftype $indirect\n";
	my $sl="";
	$sl.="p" if(($indirect =~/\*/) || ($deftype=~/^p/));#pointer type
	if($btype=~/int/){
		if($btype=~/unsigned/) {
			$sl.="u";
		} else {
			$sl.="i";
		}
		if($btype=~/long long/) {$sl.="64";}
		elsif($btype=~/long/)   {$sl.="32";}
		elsif($btype=~/short/)  {$sl.="16";}
		else			{$sl.="32";}
	} else {
		$sl.="l" if($btype=~/long/);
		$sl.="f" if($btype=~/float/);
		$sl.="d" if($btype=~/double/);	
	}
	
	
	if(($indirect =~/\*/) || ($deftype=~/^p/)){#pointer type
		if($name ne "p$deftype"){
			nameError("the type $name does not respect the naming rules. should be written as p$deftype",$fileIdx,$line,0);
		}
	} elsif($deftype eq "struct") {
		nameError("the type $name does not respect the naming rules. should start as s_",$fileIdx,$line,0 ) unless ($name=~/^s_/ || $name eq "");
	} else	{
		nameError("the type $name does not respect the naming rules. should start as t_$sl",$fileIdx,$line,0) unless($name=~/^\Qt_$sl\E/);
	}
    }

############################### enums Name checking ###############################
print "enums names checking...\n";
#this example check the following name requirements
#All enums 
#should be written using capital letters
#should begin with E_
foreach my $def (@enums)
    {
	#read 1 record from the database and set local variables for each field
	my ($name,$fileIdx,$line,$scope,$type,$indirect)=@$def;
	
	next if($files[$fileIdx][1]);# do not check system includes
	
	if($name=~/[a-z]/){
		nameError("the enum $name does not respect the naming rules. should be written using capital letters",$fileIdx,$line,0);		
	}else{
		unless($name=~/^E_/){
			nameError("the enum $name does not respect the naming rules. should begin with E_",$fileIdx,$line,0);
		}
	}
    }

############################### fields Name checking ###############################
print "fields names checking...\n";
#this example check the following name requirements
#All struct members
#followed by p for pointer. 
#followed by l for long. 
#followed by u for unsigned int, i for signed int, f for float, d for double. 

foreach my $def (@fields)
     {
	#read 1 record from the database and set local variables for each field
	my ($name,$fileIdx,$line,$scope,$type,$indirect)=@$def;
	
	next if($files[$fileIdx][1]);# do not check system includes
	my ($deftype,$btype);
	if($type <0) {
		$deftype=$basetypes[-$type];
		$btype=$deftype;
	} else {
		$deftype=$types[$type][$namefield];
		#look for the base type of this var
		my $bt=$type;
		while($bt >=0) {
			$bt=$types[$bt][$typefield];
		}
		$btype=$basetypes[-$bt];
	}
#	print "#!info on members $name,found at $files[$fileIdx] line $line,scope $scope, type ($btype) $deftype $indirect\n";
	my $sl="";
	$sl.="p" if(($indirect =~/\*/) || ($deftype=~/^p/));#pointer type
	$sl.="l" if($btype=~/long/);
	$sl.="u" if($btype=~/unsigned/);
	$sl.="f" if($btype=~/float/);
	$sl.="d" if($btype=~/double/);
	$sl.="i" if($btype=~/int/ && !($btype=~/unsigned/));

	unless($name=~/^\Q$sl\E/){
		nameError("the members $name does not respect the naming rules. should begin with $sl",$fileIdx,$line,0);
	}
    }

############################### vars Name checking ###############################
print "vars names checking...\n";
#this example check the following name requirements
#All var 
#schould start by g for globals s for static globals l for localstatic no prefix for locals or parameters. 
#followed by p for pointer. 
#followed by l for long. 
#followed by u for unsigned int, i for signed int, f for float, d for double. 
#should end by _p for all parameters 

my @vsprefix;
$vsprefix[$scopeglobal]="g";
$vsprefix[$scopeglobalstatic]="s";
$vsprefix[$scopelocalstatic]="l";
#print "\tvsprefix=( @vsprefix )\n";
foreach my $def (@vars)
    {
	#read 1 record from the database and set local variables for each field
	my ($name,$fileIdx,$line,$scope,$type,$indirect)=@$def;
	
	next if($files[$fileIdx][1]);# do not check system includes
	my ($deftype,$btype);
	if($type <0) {
		$deftype=$basetypes[-$type];
		$btype=$deftype;
	} else {
		$deftype=$types[$type][$namefield];
		#look for the base type of this var
		my $bt=$type;
		while($bt >=0) {
			$bt=$types[$bt][$typefield];
		}
		$btype=$basetypes[-$bt];
	}
#	print "#!info on var $name,found at $files[$fileIdx] line $line,scope $scope, type ($btype) $deftype $indirect\n";
	my $sl="";
	$sl=$vsprefix[$scope] if($scope <= $scopelast);
	$sl.="p" if(($indirect =~/\*/) || ($deftype=~/^p/));#pointer type
	$sl.="l" if($btype=~/long/);
	$sl.="u" if($btype=~/unsigned/);
	$sl.="f" if($btype=~/float/);
	$sl.="d" if($btype=~/double/);
	$sl.="i" if($btype=~/int/ && !($btype=~/unsigned/));
	
	unless($name=~/^\Q$sl\E/){
		nameError("the var $name does not respect the naming rules. should begin with $sl",$fileIdx,$line,0);
	}
	if($scope == $scopeparameter){#parameter
		unless($name=~/_p$/){
			nameError("the var $name does not respect the naming rules. should be written with _p for parameter",$fileIdx,$line,0);
		}
	}
    }

print "Name checking done\n\n";
closeXOB();
return $outFile;
}

############################### XOB generation API ###############################
# these 3 functions allow to write the XOB result file of your Rules checking.
#
#######################################################################
# this function must be called for each detected error
# parameters: 	the error string as shown in the final report.
#		the file number in accordance with the @files index
#		the source line number where the error is detected (see @vars,@types,@fields,@enums definition) 
#		the row in the line where the error is detected. if 0 the row information is not shown in the report.
sub nameError 
{
my $str=shift;
my $filenum=shift;
my $line=shift;
my $row=shift;
print OU "<trap name=\"CRC_990_10\" param=\"$str\">\n";
print OU "<position filenum=\"$filenum\" line=\"$line\" row=\"$row\"/>\n";
print OU "</trap>\n";
print "$files[$filenum][0]:$line $str\n";
}

#######################################################################
# this function must be called at the beginning to open the result file
# only one parameter: the name of the XOB file
sub openXOB
{
my $file=shift;# xob file as foo.xob
my $date=time();
open OU,">$file" || print &PerlTranslate("rtrtNamingRules.pl", "unable to create the file %1\n", "%1 is the name of the file to be created", $file);
print OU "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
print OU "<XOBJ filename=\"$file\" date=\"$date\" >\n";
my $Idx=0;
foreach my $f (@files) {
	my ($name,$fileSys)=@$f;
	my $sys= "sysinclude=\"1\"" if($fileSys);
	print OU "<filename fullname=\"$name\" $sys filenum=\"$Idx\" />\n";
	$Idx++;
   }
}

#######################################################################
# this function must be called at the end to close the result file
# no parameter
sub closeXOB
{
print OU "</XOBJ>";
close OU;
}

#the file must ends with 1
1;