#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                                                                              #
#                                                                              #
#  Produce products.h from $ATLTGT/lib/products_model.h                        #
#  Compile Target Package object in $INTDIR                                    #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $TARGETDIR    : directory of the Target Package                            #
#   $SCRIPTDIR    : directory of scripts                                       #
#   $INTDIR       : intermediate files directory                               #
#   $OBJEXT       : object extension                                           #
#   $ATUMODE      : 1 if use ATU                                               #
#   $ATSMODE      : 1 if use ATS                                               #
#   $ATOMODE      : 1 if use ATO                                               #
#   $ATCMODE      : 1 if use ATC                                               #
#   $ATTMODE      : 1 if use ATT                                               #
#                                                                              #
################################################################################
#$Id: Products.pl,v 1.150 2007/10/22 14:16:58 rfrance Exp $
################################################################################

use File::Copy;

require $SCRIPTDIR . "Fcts.pl";
die &PerlTranslate("Products.pl", "Unknown TDP... check configuration\n") unless (-d $TARGETDIR);
require $TARGETDIR . "/cmd/atl_cc.pl";
require $TARGETDIR . "/cmd/atl_link.pl";

# Debug mode:
$PRODUCTS_DEBUG = 0;

################################################################################
# TargetPackageIsToBeCompiled
# Arguments:
#   $tp_obj : TP.o(bj)
#   $tmp_header : temporary products header file
#   $header : last generated products header file
# Return value: 
#   0 if TP.o(bj) is up to date
#   1 if TP.o(bj) is to be recompiled
################################################################################
sub TargetPackageIsToBeCompiled #($$$)
{
  my ($tp_obj,$tmp_header,$header) = @_;

  # TP.o(bj) exists ?
  if (! -f $tp_obj) {
    print "$tp_obj does not exist\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    return (1);
  }
  print "$tp_obj already exists\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Has customizable file been modified ?
  my $custom = "$TARGETDIR/lib/custom.h";
  if (IsOlderThan($tp_obj,$custom)) {
    print "$custom has been modified\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    return (1);
  }
  print "$custom has not been modified\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Has last generated products header file been erased ?
  if (! -f $header) {
    print "$header does not exist\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    return (1);
  }
  print "$header already exists\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Has products switched been modified ?
  if (FileDiff($tmp_header,$header)) {
    print "Settings changed\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    return (1);
  }
  print "Settings did not change\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Else: TP.o(bj) is up to date
  return (0);
}

################################################################################
# MakeCProductsHeaderFile
# Arguments:
#   $model = model for products header file
#   $tmp_header = temporary products header file
# Return value: 
#   none
################################################################################
sub MakeCProductsHeaderFile #($$)
{
  my ($model,$tmp_header) = @_;

  #open(  LOG, ">>$LOGFILENAME") or die &PerlTranslate("Products.pl", "cannot open log file.\n");
  #print LOG "DUM $tmp_header\n";
  #close ( LOG );

  # Let's open in and out files
  open(PRODUCTS_MODEL, "<$model") || die &PerlTranslate("Products.pl", "cannot open %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $model, $!);
  open(PRODUCTS_TMP, ">$tmp_header") || die &PerlTranslate("Products.pl", "cannot create %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $tmp_header, $!);

  print (PRODUCTS_TMP "/* TDP:\n  " . $Ini{'NAME'} . " */\n\n");
  print ("/* TDP:\n  " . $Ini{'NAME'} . " */\n\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
  print (PRODUCTS_TMP "/* Ini File used:\n  " . $INIFILE . " */\n\n");
  print ("/* Ini File used:\n  " . $INIFILE . " */\n\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
  print (PRODUCTS_TMP "/* Preprocessor macro definitions:\n  ");
  foreach (@GENERATION_CONDITION) {
    print (PRODUCTS_TMP $_ . " ");
  }
  print (PRODUCTS_TMP "*/\n\n");
  if ($PRODUCTS_DEBUG) {
    print ("/* Preprocessor macro definitions:\n  "); # NO TRANSLATION
    foreach (@GENERATION_CONDITION) {
      print ($_ . " "); # NO TRANSLATION
    }
    print ("*/\n\n"); # NO TRANSLATION
  }
  print (PRODUCTS_TMP "/* Preprocessor options:\n  " . $CPPFLAGS . " */\n\n");
  print ("/* Preprocessing flags:\n  " . $CPPFLAGS . " */\n\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
  print (PRODUCTS_TMP "/* Compiler flags:\n  " . $CFLAGS . " */\n\n");
  print ("/* Compilation flags:\n  " . $CFLAGS . " */\n\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Parse in-file to change @<keyword>@ lines with Studio node settings
  while (<PRODUCTS_MODEL>) {

    if (/^\s*\@PRODUCT\@\s\s*(USE_AT[USOKCTPQ]).*$/) {
      my $product=$1;
      my $answer="";
      if ( $product eq "USE_ATU" ) {
          $answer = $ATUMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATU ",$answer,"\n");
          print ("#define USE_ATU ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATS" ) {
          $answer = ( $ATSMODE || $ATS_PROBE_CUSTOM ) && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATS ",$answer,"\n");
          print ("#define USE_ATS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATO" ) {
          $answer = $ATOMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATO ",$answer,"\n");
          print ("#define USE_ATO ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATK" ) {
          $answer = $ATKMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATK ",$answer,"\n");
          print ("#define USE_ATK ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATC" ) {
          $answer = $ATCMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATC ",$answer,"\n");
          print ("#define USE_ATC ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATT" ) {
          $answer = $ATTMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATT ",$answer,"\n");
          print ("#define USE_ATT ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATP" ) {
          $answer = $ATPMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATP ",$answer,"\n");
          print ("#define USE_ATP ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $product eq "USE_ATQ" ) {
          $answer = $ATQMODE && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define USE_ATQ ",$answer,"\n");
          print ("#define USE_ATQ ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATL\@\s\s*(ATL_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATL_MULTI_THREADS" ) {
          $answer = $ATL_MULTI_THREADS && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATL_MULTI_THREADS ",$answer,"\n");
          print ("#define ATL_MULTI_THREADS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_MULTI_PROCESS" ) {
          $answer = $ATL_MULTI_PROCESS && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATL_MULTI_PROCESS ",$answer,"\n");
          print ("#define ATL_MULTI_PROCESS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_THREADS_MAX" ) {
          $answer = $ATL_THREADS_MAX;
          print (PRODUCTS_TMP "#define ATL_THREADS_MAX ",$answer,"\n");
          print ("#define ATL_THREADS_MAX ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_BUFFER_SIZE" ) {
          $answer = $ATL_BUFFER_SIZE;
          print (PRODUCTS_TMP "#define ATL_BUFFER_SIZE ",$answer,"\n");
          print ("#define ATL_BUFFER_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_CHECK_STACK" ) {
          $answer = $ATL_CHECK_STACK;
          print (PRODUCTS_TMP "#define ATL_CHECK_STACK ",$answer,"\n");
          print ("#define ATL_CHECK_STACK ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_DYN_TABLE" ) {
          $answer = $DYNTABLES && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATL_DYN_TABLE ",$answer,"\n");
          print ("#define ATL_DYN_TABLE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_DUMP_ON_RETURNING" ) {
          $answer = "";
          $answer = "_ATP_CHK" if($ATL_CHECK_FREQUENCY=~ /R/);
          print (PRODUCTS_TMP "#define ATL_DUMP_ON_RETURNING ",$answer,"\n");
          print ("#define ATL_DUMP_ON_RETURNING ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_RUNTIME_FORMAT" ) {
          $answer = "ATL_OBJ";
          $answer = "ATL_DLL" if($ATL_RUNTIME_FORMAT eq "DLL");
          $answer = "ATL_LIB" if($ATL_RUNTIME_FORMAT eq "LIB");
          print (PRODUCTS_TMP "#define ATL_RUNTIME_FORMAT ",$answer,"\n");
          print ("#define ATL_RUNTIME_FORMAT ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_USE_VOLATIL_DLL" ) {
          if($USE_UNLOAD_DLL ne "" && $USE_UNLOAD_DLL ne "NONE") {
   	       $answer = "ATL_YES";
   	  } else {
   	       $answer = "ATL_NO";  	  
   	  }
          print (PRODUCTS_TMP "#define ATL_USE_VOLATIL_DLL ",$answer,"\n");
          print ("#define ATL_USE_VOLATIL_DLL ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_NODE_NAME" ) {
          $answer =$2;
          $answer =~ s/<string>/$NAME \/ $PROJECT_NAME/g;
          print (PRODUCTS_TMP "#define ATL_NODE_NAME ",$answer,"\n");
          print ("#define ATL_NODE_NAME ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_MIME_NAME" ) {
          $answer = $MIME_NAME;
          print (PRODUCTS_TMP "#define ATL_MIME_NAME \"",$answer,"\"\n");
          print ("#define ATL_MIME_NAME \"",$answer,"\"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_TRACES_FILE" ) {
          if ($ATL_TRACES_FILE ne "") {
            $answer = "$ATL_TRACES_FILE";
          } else {
            $answer = "$ATL_OUT_FILE"; # Always setted by Studio or scripts
          }
          if ( $Ini{'NOPATH'} || $Ini{'NOPATHOUT'} ) {
            my ($dir, $base, $ext) = SplitFileName ($answer);
            $answer = "$base.$ext";
	  } elsif ($Ini{'ABSPATHOUT'} ) {
		my $pwd=GetCurrentDir();
		$answer=$pwd.$PATH_SEPARATOR.$answer;
          }
          my $answerModel=$2;
          $answerModel =~ s/<string>/$answer/g;
          $answer = $answerModel;
          $answer =~ /.*\"(.*)\"/;
          print (PRODUCTS_TMP "#define ATL_TRACES_FILE_LEN ", length($1), "\n");
          print ("#define ATL_TRACES_FILE_LEN ", length($1), "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
          #$answer =~ s/\\/\\\\/g;
          $answer = TestRTUtil::LocaleStringReplace($answer,"\\\\","\\\\");
          print (PRODUCTS_TMP "#define ATL_TRACES_FILE ", $answer, "\n");
          print ("#define ATL_TRACES_FILE ", $answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
	  if (( lc($Ini{'SOCKET_UPLOAD'}) eq "true" ) || ($SOCKET_UPLOAD)){
            $ADDRESS=hostinet() if(($ADDRESS eq "127.0.0.1") || ($ADDRESS eq ""));
            $answer=$ADDRESS;
            $answer=~ s/\./,/g;
            print (PRODUCTS_TMP "#define ATL_TRACES_FILE_ADDRESS \"", $ADDRESS, "\"\n");
            print (PRODUCTS_TMP "#define ATL_TRACES_FILE_ADDRESS_NS ", $answer, "\n");
	    my $uport=$UPLOADER_PORT;
	    if($uport ==0) {
	 	$uport=7777;
	    }
            print (PRODUCTS_TMP "#define ATL_TRACES_FILE_PORT ", $uport, "\n");

            print ( "#define ATL_TRACES_FILE_ADDRESS \"", $ADDRESS, "\"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
            print ( "#define ATL_TRACES_FILE_ADDRESS_NS ", $answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
            print ( "#define ATL_TRACES_FILE_PORT ", $uport, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
         }
      } elsif ( $setting eq "ATL_CLOCK_KIND" ) {
	 $answer = $ATL_CLOCK_KIND && "ATL_PROCESS_CLOCK" || "ATL_OS_CLOCK";
	 print (PRODUCTS_TMP "#define ATL_CLOCK_KIND ",$answer,"\n");
	 print ("#define ATL_CLOCK_KIND ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATU\@\s\s*(ATU_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATU_BREAK_ON_TEST_FAILED" )
      {
	$answer = $ATU_BREAK_ON_TEST_FAILED;
	print (PRODUCTS_TMP "#define ATU_BREAK_ON_TEST_FAILED ",$answer, "\n");
	print ("#define ATU_BREAK_ON_TEST_FAILED ",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATS\@\s\s*(ATS_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATS_BUFFER_SIZE" ) {
	$answer = $ATS_BUFFER_SIZE;
	print (PRODUCTS_TMP "#define ATS_BUFFER_SIZE ",$answer, "\n");
	print ("#define ATS_BUFFER_SIZE ",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATS_TAGTBUF_SIZE" ) {
	$answer = $ATS_TAGTBUF_SIZE;
	print (PRODUCTS_TMP "#define ATS_TAGTBUF_SIZE ",$answer, "\n");
	print ("#define ATS_TAGTBUF_SIZE ",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATS_RT_BUFFER" ) {
	$answer = $ATS_RT_BUFFER;
	print (PRODUCTS_TMP "#define ATS_RT_BUFFER",$answer, "\n");
	print ("#define ATS_RT_BUFFER",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATS_DYNFLW" ) {
        my $a=$ATS_DYN_FOLLOW;
        $a =0 if($ATS_NO_SPV);
	$answer = $a;
	print (PRODUCTS_TMP "#define ATS_DYNFLW ",$answer, "\n");
	print ("#define ATS_DYNFLW ",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATS_INTERRECV_TIMEOUT" ) {
	$answer = $ATS_INTERRECV_TIMEOUT;
	print (PRODUCTS_TMP "#define ATS_INTERRECV_TIMEOUT ",$answer, "\n");
	print ("#define ATS_INTERRECV_TIMEOUT ",$answer, "\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATO\@\s\s*(ATO_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATO_BREAK_ON_CHECK_FAILED" ) {
	$answer = $ATO_STOP_ON_ERROR && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATO_BREAK_ON_CHECK_FAILED ", $answer, "\n");
	print ("#define ATO_BREAK_ON_CHECK_FAILED ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATO_DUMP_SUCCESS" ) {
	$answer = $ATO_DUMP_SUCCESS && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATO_DUMP_SUCCESS ", $answer, "\n");
	print ("#define ATO_DUMP_SUCCESS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATO_MAX_INSTANCES" ) {
	$answer = $ATO_MAX_INSTANCES;
	print (PRODUCTS_TMP "#define ATO_MAX_INSTANCES ", $answer, "\n");
	print ("#define ATO_MAX_INSTANCES ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATO_PRINT_BUFFER_SIZE" ) {
	$answer = $ATO_PRINT_BUFFER_SIZE;
	print (PRODUCTS_TMP "#define ATO_PRINT_BUFFER_SIZE ", $answer, "\n");
	print ("#define ATO_PRINT_BUFFER_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATK\@\s\s*(ATK_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATK_BREAK_ON_ASSERT_FAILED" ) {
	$answer = $ATK_STOP_ON_ERROR && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATK_BREAK_ON_ASSERT_FAILED ", $answer, "\n");
	print ("#define ATK_BREAK_ON_ASSERT_FAILED ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATK_DUMP_SUCCESS" ) {
	$answer = $ATK_DUMP_SUCCESS && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATK_DUMP_SUCCESS ", $answer, "\n");
	print ("#define ATK_DUMP_SUCCESS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATK_REPORT_REFLEXIVE_STATES" ) {
	$answer = $ATK_REPORT_REFLEXIVE_STATES && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATK_REPORT_REFLEXIVE_STATES ", $answer, "\n");
	print ("#define ATK_REPORT_REFLEXIVE_STATES ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATK_CONST_ASSERTIONS" ) {
	$answer = $ATK_CONST_ASSERTIONS && "ATL_YES" || "ATL_NO";
	print (PRODUCTS_TMP "#define ATK_CONST_ASSERTIONS ", $answer, "\n");
	print ("#define ATK_CONST_ASSERTIONS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATC\@\s\s*(ATC_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATC_INFORMATION" ) {
          $answer = $ATC_INFORMATION;
          print (PRODUCTS_TMP "#define ATC_INFORMATION ",$answer,"\n");
          print ("#define ATC_INFORMATION ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATC_DUMP_DRIVER" ) {
          $answer = $ATC_DUMP_DRIVER && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATC_DUMP_DRIVER ",$answer,"\n");
          print ("#define ATC_DUMP_DRIVER ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATT\@\s\s*(ATT_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATT_ON_THE_FLY" ) {
          $answer = $ATT_ON_THE_FLY && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATT_ON_THE_FLY ",$answer,"\n");
          print ("#define ATT_ON_THE_FLY ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_ITEMS_MAX" ) {
          $answer = $ATT_ITEM_BUFFER && $ATT_ITEM_BUFFER_SIZE || 0;
          print (PRODUCTS_TMP "#define ATT_ITEMS_MAX ",$answer,"\n");
          print ("#define ATT_ITEMS_MAX ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_DUMP_FROM_BEGINNING" ) {
          $answer = $ATT_PARTIAL_DUMP && "ATL_NO" || "ATL_YES";
          print (PRODUCTS_TMP "#define ATT_DUMP_FROM_BEGINNING ",$answer,"\n");
          print ("#define ATT_DUMP_FROM_BEGINNING ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_SIGNAL" ) {
          $answer = $ATT_SIGNAL_ACTION;
          print (PRODUCTS_TMP "#define ATT_SIGNAL ",$answer,"\n");
          print ("#define ATT_SIGNAL ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_COMPUTE_MAX_CALLING_LEVEL" ) {
          $answer = $ATT_RECORD_MAX_STACK && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATT_COMPUTE_MAX_CALLING_LEVEL ",$answer,"\n");
          print ("#define ATT_COMPUTE_MAX_CALLING_LEVEL ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_TIMESTAMP" ) {
          $answer = $ATT_TIMESTAMP && "ATL_YES" || "ATL_NO";
          if($ATT_TIMESTAMP && defined $Ini{ATQMODE})
	  {
		  $answer = "ATL_NO" ;
		  PerlTranslate("Products.pl", "Runtime Tracing Timestamp is not available for this TDP\n");
	  }
          print (PRODUCTS_TMP "#define ATT_TIMESTAMP ",$answer,"\n");
          print ("#define ATT_TIMESTAMP ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_THREAD_INFO" ) {
          $answer = $ATT_THREAD_INFO && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATT_THREAD_INFO ",$answer,"\n");
          print ("#define ATT_THREAD_INFO ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATP\@\s\s*(ATP_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATP_CALL_STACK_SIZE" ) {
          $answer = $ATP_CALL_STACK_SIZE;
          print (PRODUCTS_TMP "#define ATP_CALL_STACK_SIZE ",$answer,"\n");
          print ("#define ATP_CALL_STACK_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_FIU" ) {
          $answer = $ATP_REPORTS_FIU && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_FIU ",$answer,"\n");
          print ("#define ATP_REPORTS_FIU ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_FIM" ) {
          $answer = $ATP_REPORTS_FIM && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_FIM ",$answer,"\n");
          print ("#define ATP_REPORTS_FIM ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_MIU" ) {
          $answer = $ATP_REPORTS_MIU && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_MIU ",$answer,"\n");
          print ("#define ATP_REPORTS_MIU ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_SIG" ) {
          $answer = $ATP_REPORTS_SIG && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_SIG ",$answer,"\n");
          print ("#define ATP_REPORTS_SIG ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_FFM_FMWL" ) {
          $answer = $ATP_REPORTS_FFM_FMWL && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_FFM_FMWL ",$answer,"\n");
          print ("#define ATP_REPORTS_FFM_FMWL ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_MAX_FREEQ_LENGTH" ) {
          $answer = $ATP_MAX_FREEQ_LENGTH;
          print (PRODUCTS_TMP "#define ATP_MAX_FREEQ_LENGTH ",$answer,"\n");
          print ("#define ATP_MAX_FREEQ_LENGTH ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_MAX_FREEQ_SIZE" ) {
          $answer = $ATP_MAX_FREEQ_SIZE || 0;
          print (PRODUCTS_TMP "#define ATP_MAX_FREEQ_SIZE ",$answer,"\n");
          print ("#define ATP_MAX_FREEQ_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_MAX_FREEQ_BLOCK_SIZE" ) {
	  $answer = $ATP_MAX_FREEQ_SIZE || 0;
          $answer = $ATP_MAX_FREEQ_BLOCK_SIZE if($ATP_MAX_FREEQ_BLOCK_SIZE);
          print (PRODUCTS_TMP "#define ATP_MAX_FREEQ_BLOCK_SIZE ",$answer,"\n");
          print ("#define ATP_MAX_FREEQ_BLOCK_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_REPORTS_ABWL" ) {
          $answer = $ATP_REPORTS_ABWL && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_REPORTS_ABWL ",$answer,"\n");
          print ("#define ATP_REPORTS_ABWL ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_CHECK_SELECTED_BLOCKS" ) {
          $answer = "ATL_NO";
          $answer = "ATL_YES" if($ATL_CHECK_FREQUENCY =~ /S/);
          print (PRODUCTS_TMP "#define ATP_CHECK_SELECTED_BLOCKS ",$answer,"\n");
          print ("#define ATP_CHECK_SELECTED_BLOCKS ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_BREAK_ON_ERROR" ) {
          $answer = $ATP_BREAK_ON_ERROR && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATP_BREAK_ON_ERROR ",$answer,"\n");
          print ("#define ATP_BREAK_ON_ERROR ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_RED_ZONE_SIZE" ) {
          $answer = $ATP_RED_ZONE_SIZE;
          print (PRODUCTS_TMP "#define ATP_RED_ZONE_SIZE ",$answer,"\n");
          print ("#define ATP_RED_ZONE_SIZE ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_KEEP_CONTENT" ) {
          $answer = $ATP_KEEP_CONTENT;
          print (PRODUCTS_TMP "#define ATP_KEEP_CONTENT ",$answer,"\n");
          print ("#define ATP_KEEP_CONTENT ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATP_NO_TRACK_ALLOCS_BEFORE_INIT" ) {
          $answer = $ATP_NO_TRACK_ALLOCS_BEFORE_INIT;
          print (PRODUCTS_TMP "#define ATP_NO_TRACK_ALLOCS_BEFORE_INIT ",$answer,"\n");
          print ("#define ATP_NO_TRACK_ALLOCS_BEFORE_INIT ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATQ\@\s\s*(ATQ_[A-Z_0-9][A-Z_0-9]*)\s(.*)$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATQ_COMPUTE_OVERHEAD" ) {
          $answer = ATL_NO; # TBD: $ATQ_COMPUTE_OVERHEAD && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATQ_COMPUTE_OVERHEAD ",$answer,"\n");
          print ("#define ATQ_COMPUTE_OVERHEAD ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATQ_DUMP_DRIVER" ) {
          $answer = $ATQ_DUMP_DRIVER && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATQ_DUMP_DRIVER ",$answer,"\n");
          print ("#define ATQ_DUMP_DRIVER ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATQ_COMPUTE_MINMAX" ) {
          $answer = $ATQ_COMPUTE_MINMAX && "ATL_YES" || "ATL_NO";
          print (PRODUCTS_TMP "#define ATQ_COMPUTE_MINMAX ",$answer,"\n");
          print ("#define ATQ_COMPUTE_MINMAX ",$answer,"\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } else {
          print (PRODUCTS_TMP "/* #define $1 $2 */\n");
          print ("/* #define $1 $2 */\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } else {
      print PRODUCTS_TMP;
    }

  }

  # That's all folks
  close(PRODUCTS_TMP);
  close(PRODUCTS_MODEL);
}

################################################################################
# GetTargetPackageObject
# Arguments:
#   none
# Return value: 
#   Path and filename of the object file
################################################################################
#sub GetTargetPackageObject
#{
#  my $d=$INTDIR;
#  if($ENV{'ATTOLOBJ'}) {
#     $d=$ENV{'ATTOLOBJ'};
#  }
#
#  if ( $OSNAME eq "win32" )
#    {
#      return "$d\\TP.$OBJEXT";
#    }
#  else
#    {
#      return "$d/TP.$OBJEXT";
#    }
#}
################################################################################
# GetTargetPackagesLib
# Arguments:
#   none
# Return value:
#   Path and filename of the Lib file 
################################################################################
sub GetTargetPackageLib #($)
{
  my $TPext = shift;
  my $d=$INTDIR;
  if($ENV{'ATTOLOBJ'}) {
     $d=$ENV{'ATTOLOBJ'};
  }

  if ( $OSNAME eq "win32" )
    {
      return "$d\\TP.$TPext";
    }
  else
    {
      return "$d/TP.$TPext";
    }
}

################################################################################
# MakeTargetPackageObject
# Arguments:
#   none
# Return value: 
#   none
################################################################################
sub MakeTargetPackageObject
{
  my $model = "$TARGETDIR/lib/products_model.h";
  my $tmp_header = "$INTDIR/products_tmp.h";
  my $header = "$INTDIR/products.h";
  my $tp_obj = GetTargetPackageLib($OBJEXT);
  my $tp_lib = $tp_obj;
  my $tp_ldflags= $TP_LDFLAGS;
  my $tp_cflags = $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS;
     $tp_cflags = $TP_CCFLAGS if($TP_CCFLAGS ne "");
  $ATL_RUNTIME_FORMAT="OBJ" if($ATL_RUNTIME_FORMAT eq "");
     $tp_lib = GetTargetPackageLib ($LIBEXT) if($ATL_RUNTIME_FORMAT eq "LIB");
     $tp_lib = GetTargetPackageLib ($DLLEXT) if($ATL_RUNTIME_FORMAT eq "DLL");

  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Products.pl", "cannot open log file.\n");
  ## just do delete it from Studio
  print  LOG "DUM $tp_obj\n";
  print  LOG "DUM $header\n";
  print  LOG "DUM $tp_lib\n" if($ATL_RUNTIME_FORMAT ne "OBJ");
  close( LOG );
  
  # First make the temporary products header file
  MakeCProductsHeaderFile($model,$tmp_header);

  # Now decide if Target Package object is to be compiled, if so: do it !
#  if ((TargetPackageIsToBeCompiled($tp_obj,$tmp_header,$header))||($PRODUCTS_SPV)) {
  if (TargetPackageIsToBeCompiled($tp_lib,$tmp_header,$header)) {

    print "Need to compile TP\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    if ( -f $header ) {
      unlink ($header) || die &PerlTranslate("Products.pl", "unlink %1 failed\n", "%1 is the file to be unlinked", $header);
    }
    rename ($tmp_header,$header) || die &PerlTranslate("Products.pl", "rename %1 to %2 failed\n", "%1 is the original name of the file and %2 is the new name", $tmp_header, $header);

    # Target Package compilation:
    my $tp_src_ext = ".c";
    my $language = "C";
    if ( $TDP_LANGUAGE eq "CPP" ) {
      $tp_src_ext = ".cpp";
      $language = "CPP";
    }
    my $tp_src = $TARGETDIR . $PATH_SEPARATOR . "lib" . $PATH_SEPARATOR . "TP$tp_src_ext";
    my @tp_includes = ( "$INTDIR", $TARGETDIR . $PATH_SEPARATOR . "lib", @COMPLETE_INCLUDE_DIR_LIST );
    print &PerlTranslate("Products.pl", "Compiling %1 using %2 containing C/C++ TDP Library settings...\n", "%1 is the file to be compiled and %2 is the file to be used", $tp_src, $header) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
    $status = atl_cc ( $language, $tp_src, $tp_obj, $tp_cflags,
                       \@GENERATION_CONDITION, \@tp_includes );
    EndStep($status);
    if($status != 0) {
      print &PerlTranslate("Products.pl", "Target Package compilation failed\n");
      unlink ($tp_obj);
      return (1);
    }
    if($ATL_RUNTIME_FORMAT ne "OBJ") {
      my @OBJs = ($tp_obj);
      my @ADDlibs = ();
      $status = atl_lib ( $tp_lib, \@OBJs, $tp_ldflags,
                           \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDlibs, $ATL_RUNTIME_FORMAT );
      EndStep($status);
      if($status != 0) {
         print &PerlTranslate("Products.pl", "Target Package Library compilation failed\n");
         unlink ($tp_lib);
         return (1);
       }

    }
    #make startup if required
    if($Ini{STARTUP_CODE}){
	my $startup_src=$Ini{STARTUP_CODE};
	my @inc_dir_list=@COMPLETE_INCLUDE_DIR_LIST;
	my ($dir,$base,$ext) = SplitFileName ($startup_src);
	my $startup_object="$INTDIR/$base.$OBJEXT";
	if($dir){		
		unshift(@inc_dir_list,$dir) ;
	} else {
		$startup_src="$TARGETDIR/lib/$base.$ext";
	}
	#put this object file on the first position
	unless(grep(/[\\\/]$base\.$OBJEXT$/,@OBJECTS)) {
		unshift(@OBJECTS, $startup_object);
		open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Products.pl", "cannot open log file.\n");
		print  LOG "DUM $startup_object\n";
		close( LOG );
		unless(-f $startup_object){
		print &PerlTranslate("Products.pl", "Compiling startup file %1\n", "%1 is the file to be compiled", $startup_src);
			my $lang=LanguageOf($ext);
			if ( $lang eq "C" ) {
				$status = atl_cc ($lang, $startup_src, $startup_object, $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS,
					\@GENERATION_CONDITION, \@inc_dir_list);
			} elsif ( $lang eq "ASM" ) {
				$status = atl_asm ($lang, $startup_src, $startup_object, $ASMFLAGS,
					\@GENERATION_CONDITION, \@inc_dir_list);
			} else {
				print "Cannot handle startup file $startup_src (unknown extension: $ext) !..\n";
				$status = 1;
			}
			EndStep($status);
			if($status) {
				print &PerlTranslate("Products.pl", "Target Package startup compilation failed\n");
				unlink ($startup_object);
				return (1);
			}
		}
	}
    }
  } else {

    print "No need to compile TP\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
    # Remove temporary products header file
    unlink ($tmp_header);

  }
  
  return (0);
}

################################################################################
# MakeJavaProductsClassFile
# Arguments:
#   $model = model for products header file
#   $tmp_header = temporary products header file
# Return value: 
#   none
################################################################################
sub MakeJavaProductsClassFile #($$)
{
  my ($model,$products_class) = @_;

  #TBD: en attendant
  # Let's open in and out files
  open(PRODUCTS_MODEL, "<$model") || die &PerlTranslate("Products.pl", "cannot open %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $model, $!);
  print "Reading from $model\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION
  open(PRODUCTS_TMP, ">$products_class") || die &PerlTranslate("Products.pl", "cannot create %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $products_class, $!);
  print "Writting to $products_class\n" if ($PRODUCTS_DEBUG); # NO TRANSLATION

  # Parse in-file to change @<keyword>@ lines with Studio node settings
  while (<PRODUCTS_MODEL>) {

    if (/^\s*\@ATL\@\s\s*(ATL_[A-Z_0-9][A-Z_0-9]*)\s.*$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATL_THREADS_MAX" ) {
          $answer = $ATL_THREADS_MAX;
          print (PRODUCTS_TMP "    public static final int max_threads = ",$answer,";\n");
          print ("    public static final int max_threads = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_BUFFER_SIZE" ) {
          $answer = $ATL_BUFFER_SIZE;
          print (PRODUCTS_TMP "    public static final int buffer_size = ",$answer,";\n");
          print ("    public static final int buffer_size = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_NODE_NAME" ) {
          $answer = $NAME;
          print (PRODUCTS_TMP "    public static final String node_name = \"",$answer,"\";\n");
          print ("    public static final String node_name = \"",$answer,"\";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_TRACES_FILE" ) {
          if ($ATL_TRACES_FILE ne "") {
            $answer = "$ATL_TRACES_FILE";
          } else {
            $answer = "$ATL_OUT_FILE"; # Always setted by Studio or scripts
          }
          if ( $Ini{'NOPATH'} || $Ini{'NOPATHOUT'} ) {
             my ($dir, $base, $ext) = SplitFileName ($answer);
             $answer = "$base.$ext";
	  } elsif ($Ini{'ABSPATHOUT'} ) {
		my $pwd=GetCurrentDir();
		$answer=$pwd.$PATH_SEPARATOR.$answer;
          }
          #$answer =~ s/\\/\\\\/g;
          $answer = TestRTUtil::LocaleStringReplace($answer,"\\\\","\\\\");
          print (PRODUCTS_TMP "    public static final String out_file = \"", $answer, "\";\n");
          print ("    public static final String out_file = \"", $answer, "\";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_OUT_URL" ) {
	 $answer = "http://$ADDRESS:$UPLOADER_PORT";
	 print (PRODUCTS_TMP "    public static final String out_url = \"",$answer,"\";\n");
	 print ("    public static final String out_url = \"",$answer,"\";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_RUN_GC_AT_EXIT" ) {
          $answer = $ATL_RUN_GC_AT_EXIT && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean should_run_gc = ",$answer,";\n");
          print ("    public static final boolean should_run_gc = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATL_CLOSE_IO" ) {
          $answer = $ATTMODE && "false" || "true";
          print (PRODUCTS_TMP "    public static final boolean close_io = ",$answer,";\n");
          print ("    public static final boolean close_io = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATJ\@\s\s*(ATJ_[A-Z_0-9][A-Z_0-9]*)\s.*$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATJ_MODE" ) {
	$answer = $ATJMODE && "true" || "false";
	print (PRODUCTS_TMP "    public static final boolean mode_JTesting = ", $answer, ";\n");
	print ("    public static final boolean mode_JTesting = ", $answer, ";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATJ_CHECK_STUB" ) {
	$answer = $JAVA_STUB && "true" || "false";
	print (PRODUCTS_TMP "    public static final boolean mode_JTesting_STUB = ", $answer, ";\n");
	print ("    public static final boolean mode_JTesting_STUB = ", $answer, ";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATJ_DISPLAY_STUB" ) {
	$answer = $JAVA_DISPLAY && "true" || "false";
	print (PRODUCTS_TMP "    public static final boolean mode_JTesting_DISPLAY = ", $answer, ";\n");
	print ("    public static final boolean mode_JTesting_DISPLAY = ", $answer, ";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } elsif (/^\s*\@ATT\@\s\s*(ATT_[A-Z_0-9][A-Z_0-9]*)\s.*$/) {
      my $setting=$1;
      my $answer="";
      if ( $setting eq "ATT_ON_THE_FLY" ) {
          $answer = $ATT_ON_THE_FLY && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean trace_on_the_fly = ",$answer,";\n");
          print ("    public static final boolean trace_on_the_fly = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_DUMP_FROM_BEGINNING" ) {
          $answer = $ATT_PARTIAL_DUMP && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean trace_partial_message_dump = ",$answer,";\n");
          print ("    public static final boolean trace_partial_message_dump = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_TIMESTAMP" ) {
          $answer = $ATT_TIMESTAMP && "true" || "false";
           if($ATT_TIMESTAMP && defined $Ini{ATQMODE})
	  {
		  $answer = "false" ;
		  PerlTranslate("Products.pl", "Runtime Tracing Timestamp is not available for this TDP\n");
	  }
          print (PRODUCTS_TMP "    public static final boolean trace_time_stamp = ",$answer,";\n");
          print ("    public static final boolean trace_time_stamp = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_HEAP_SIZE" ) {
          $answer = $ATT_HEAP_SIZE && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean trace_heap_size = ",$answer,";\n");
          print ("    public static final boolean trace_heap_size = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_THREAD_INFO" ) {
          $answer = $ATT_THREAD_INFO && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean trace_thread_info = ",$answer,";\n");
          print ("    public static final boolean trace_thread_info = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      } elsif ( $setting eq "ATT_COMPUTE_MAX_CALLING_LEVEL" ) {
          $answer = $ATT_RECORD_MAX_STACK && "true" || "false";
          print (PRODUCTS_TMP "    public static final boolean trace_max_call_stack = ",$answer,";\n");
          print ("    public static final boolean trace_max_call_stack = ",$answer,";\n") if ($PRODUCTS_DEBUG); # NO TRANSLATION
      }
    } else {
      print PRODUCTS_TMP;
    }

  }

  # That's all folks
  close(PRODUCTS_TMP);
  close(PRODUCTS_MODEL);
}

################################################################################
# GetJavaProductFile
# Arguments:
#   none
# Return value: 
#   Products.java with the right file name
################################################################################
sub GetJavaProductFile
{
  if ( $OSNAME eq "win32" )
    {
      return "$INTDIR\\com\\rational\\testrealtime\\Products.java";
    }
  else
    {
      return "$INTDIR/com/rational/testrealtime/Products.java";
    }
}

################################################################################
# GetJavaProductClassFile
# Arguments:
#   none
# Return value: 
#   Products.java with the right file name
################################################################################
sub GetJavaProductClassFile
{
  if ( $OSNAME eq "win32" )
    {
      return "$INTDIR\\com\\rational\\testrealtime\\Products.class";
    }
  else
    {
      return "$INTDIR/com/rational/testrealtime/Products.class";
    }
}

################################################################################
# PrepareJavaTargetPackage
# Arguments:
#   none
# Return value: 
#   none
################################################################################
sub PrepareJavaTargetPackage
{
  my $model = "$TARGETDIR/lib/com/rational/testrealtime/ProductsModel.java.txt";
  my $products_class = GetJavaProductFile();
  my ($srcdir, $srcname, $srcext) = SplitFileName( $products_class );

  print &PerlTranslate("Products.pl", "Generating %1 containing Java TDP Library settings...\n", "%1 is the name of the file to be generated", $products_class) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
  CreateDirectory ($srcdir);
  if (-f GetJavaProductClassFile()) {
    unlink (GetJavaProductClassFile());
  }
  MakeJavaProductsClassFile($model, $products_class);
}


1;
##
################################################################################

