#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                            -*-  Source node -*-                              #
#                                                                              #
#                                                                              #
################################################################################
#$Id: Source.pl,v 1.157 2007/10/17 09:37:59 rfrance Exp $
################################################################################

use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0] unless($main::TestRTCC);

require $SCRIPTDIR . "csv2header.pl";

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

# Attol tools functions
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);
require $SCRIPTDIR . "metricC.pl" unless($main::TestRTCC);
require $SCRIPTDIR . "metricA.pl" unless($main::TestRTCC); 

# Target-dependant functions
die &PerlTranslate("Source.pl", "Unknown TDP... check configuration\n") unless (-d $TARGETDIR);
require $TARGETDIR . "/cmd/atl_cc.pl";

################################################################################

sub Source
{
	my ($srcdir, $srcname, $srcext) = SplitFileName( $FILENAME );
	my $srcfile = $srcname.'.'.$srcext;
	my $prepro_file = "$INTDIR".$PATH_SEPARATOR."$srcname.i";
	my $obj = "$INTDIR".$PATH_SEPARATOR."$srcname.$OBJEXT";
	my $language = LanguageOf( $srcext,"" );#unknown ext have to be assumed as linker file, So dont set language in this case 
	my $already_generated = 0;
	my $somethingDone;
	my $is_ada=0;
	my $is_java=0;
	# Ada Mode.
	if ($TDP_LANGUAGE=~/^ADA/)
	{
	  $is_ada = 1 ;
	#------------------------------------------------------------------------
	# EM: no TDP Ada generate process for each file, but the Ada TDP is
	# compiled one time before the compilation of all ada files =>
	# AdaTDP.pl  (RA:2227)
	#------------------------------------------------------------------------
	#  TDPAdaGenerate ( $TDP_LANGUAGE );
	}
	# Java Mode.
	elsif ( $TDP_LANGUAGE eq "JAVA" )
	{
	  $is_java = 1 ;
	}

	#------------------------------------------------------------------------
	# Resource Compilation for Microsoft
	elsif ($language eq "RC")
	{
	  # No instrumentation
	  $ATTMODE=0;
	  $ATCMODE=0;
	  $ATPMODE=0;
	  $ATQMODE=0;
	  $ATSMODE=0;
	  $ATUMODE=0;
	  $ATOMODE=0;
	  $ATKMODE=0;
	  $ATJMODE=0;
	}
	#------------------------------------------------------------------------
	# System Testing mode
	elsif ($ATSMODE)
	{
	  # No instrumentation
	  $ATTMODE=0;
	  $ATCMODE=0;
	  $ATPMODE=0;
	  $ATQMODE=0;
	  $ATKMODE=0;
	}

	# Temporary work-around to deal with GUI
	elsif ($ATOMODE)
	{
	   $ATKMODE=1;
	}

	if (@OTIFILES)
	{
	   $ATKMODE=1;
	}
	else
	{
	   $ATKMODE=0; # If no OTI file is provided, just skip Object Testing instrumentation
	}

	#-----------------------------------------------------------------------
	# Check if file should be instrumented
	if ((!$CODE_INSERTION) && !($is_java))
	{ 
	   # No instrumentation
	   $ATTMODE=0;
	   $ATCMODE=0;
	   $ATPMODE=0;
	   $ATQMODE=0;
	   $ATKMODE=0;
	   $ATOMODE=0;
	}


	if($STAGE =~ /c/){
		$somethingDone=1;
		if($language eq "") {
		    if($srcext=~ /h(..)?$/){
			    print &PerlTranslate("Source.pl", "header file assumed. Nothing to do\n","");
			    return 0;
		    }

		    print &PerlTranslate("Source.pl", "unrecognized source file type '%1', object file assumed\n","%1 is the ext",$srcext);
		    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		    print  LOG "OBJ $FILENAME\n";
		    close( LOG );
		    return(0);
		}

		if ($ATS_PROBE_CUSTOM)
		{
		    if ( -f "$INTDIR"."$PATH_SEPARATOR"."atlprobe.h" )
		    {
		      $already_generated = 1;
		    }
		    ($ATS_PROBE_CUSTOM,$ATS_PROBE_CUSTOM_TYPE) = (GetProbeSetting()=~ /(\d),(\d)/);
		    my %mode=('default',0,'fifo',1,'file',2,'user',3,'ignore',4);
		    my $probedir="$ATTOLHOME"."$PATH_SEPARATOR"."lib"."$PATH_SEPARATOR"."probe";

		    if (($ATS_PROBE_CUSTOM_TYPE== $mode{ignore}) 
			||
			(($ATS_PROBE_CUSTOM_TYPE== $mode{default}) && ! $already_generated)
		       )
		    {
			#contains atlprobe.h with null denfined macros	
			push (@COMPLETE_INCLUDE_DIR_LIST,"$probedir"."$PATH_SEPARATOR"."noprobe");
		    }
		    else
		    {
			#path of the generated file atlprobe.h and TP.h 	
			push (@COMPLETE_INCLUDE_DIR_LIST,"$INTDIR");
			push (@COMPLETE_INCLUDE_DIR_LIST,"$TARGETDIR" . "$PATH_SEPARATOR" . "lib");

			if ($ATS_PROBE_CUSTOM_TYPE== $mode{fifo})
			{
			    #custom fifo
			    push (@COMPLETE_INCLUDE_DIR_LIST,"$probedir"."$PATH_SEPARATOR"."probecst"."$PATH_SEPARATOR"."fifo");

			}
			elsif ($ATS_PROBE_CUSTOM_TYPE== $mode{file})
			{
			    #custom file
			    push (@COMPLETE_INCLUDE_DIR_LIST,"$probedir"."$PATH_SEPARATOR"."probecst"."$PATH_SEPARATOR"."file");

			}
			elsif  ($ATS_PROBE_CUSTOM_TYPE== $mode{user})  
			{
			    #the user has to set its own custom files path.
			    push (@COMPLETE_INCLUDE_DIR_LIST,$PROBE_CUSTOM_DIR) if($PROBE_CUSTOM_DIR ne "");
			}
		    }
		}
		if ($language eq "MDL")
		{
		    # MATLAB mode: Unit Test Script must be generated during wizard phase in StartUnitTesting.pl
		  print &PerlTranslate("Source.pl", "No action on %1.\n", "%1 is the file with no action on it", $FILENAME);
		  return(0);
		}

		if ($language eq "MAT")
		{
		  # MATLAB mode: data header file must be generated before Test Script Compilation in TestScript.pl
		  print &PerlTranslate("Source.pl", "MATLAB Simulink data generation from %1.\n", "%1 is the file from which the generation is to be done", $FILENAME);
		  $atu_command = $BINDIR . $PATH_SEPARATOR . "mat2data \"$FILENAME\" $MATLAB_DATATYPE";
		  my $status = SystemP($atu_command);
		  if ($status)
		  {
		    die &PerlTranslate("Source.pl", "Generation failed.\n");
		  }

		  #Log the file generated.
		  open ( LOGMATLAB, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		  #Change the name of the file to ...\matval.h.
		  #find the last \ or last /
		  my $last_index_trail = rindex($FILENAME,"\\");
		  if ($last_index_trail == -1)
		  {
		      my $last_index_trail = rindex($FILENAME,"/");
		  }
		  my $FILENAME_INCLUDE;
		  if ( $last_index_trail != -1)
		  {
		      $FILENAME_INCLUDE = substr($FILENAME,0,($last_index_trail+1));
		      $FILENAME_INCLUDE = $FILENAME_INCLUDE . "matval.h";
		  }
		  else
		  {
		      $FILENAME_INCLUDE="matval.h";
		  }
		  print  LOGMATLAB "INC $FILENAME_INCLUDE\n";
		  close( LOGMATLAB );

		#  print &PerlTranslate("Source.pl", "Done.\n");

		  return(0);
		}

		if ($language eq "CSV")
		{
		  # CSV file: C header file must be generated
		  print &PerlTranslate("Source.pl", "C header file generation from %1.\n", "%1 is the file from which the generation is to be done", $FILENAME);
		  my $csvSep         = $CSV_SEPARATOR;
		  my $cvsDecimalSep  = $CSV_DECIMAL_SEPARATOR;
		  my $cvsThousandSep = $CSV_THOUSAND_SEPARATOR;
		  if ($csvSep eq "") {
		    $csvSep = ";";
		  }
		  if ($csvDecimalSep eq "") {
		    $csvDecimalSep = ",";
		  }

		  genHeaderFileFromCSV($FILENAME, $INTDIR, $csvSep, $cvsDecimalSep, $cvsThousandSep);
		#  print &PerlTranslate("Source.pl", "Done.\n");
		  return(0);
		}

		# instrumented file has $FILENAME extension
		my $instr_file = "$INTDIR".$PATH_SEPARATOR."$srcname"."_aug.$srcext";




		#------------------------------------------------------------------------
		# Coverage or Tracer or Object Testing mode
		if ( $ATCMODE || $ATTMODE || ($ATPMODE && !$is_java) || $ATQMODE || $ATOMODE || $ATKMODE || $ATJMODE )
		{
		  my $current_src=$FILENAME;

		  if( $is_ada || $is_java || ($language eq "ASM"))
		  {
		    if($Ini{'PREPROCESSING'})
		    {

			## preprocessing
			$prepro_file = "$INTDIR".$PATH_SEPARATOR.$FILENAME;
			print &PerlTranslate("Source.pl", "%1: Preprocessing...\n", "%1 is the file to be preprocessed", $FILENAME);
			$status=atl_cpp ($language, $current_src, $prepro_file, $CPPFLAGS.' '.$PPFLAGS, 
				       \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
			EndStep($status);
			return $status if ($status);
			open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
			print  LOG "DUM $prepro_file\n";
			close( LOG );

		    } else {
			$prepro_file = $current_src ;
		    }
		  }
		  else
		  {
		      #-------------------------------------------------------------------------
		      ## preprocessing
		      print &PerlTranslate("Source.pl", "%1: Preprocessing...\n", "%1 is the file to be preprocessed", $srcfile);
		      $status=atl_cpp ($language, $current_src, $prepro_file, $CPPFLAGS.' '.$PPFLAGS, 
				       \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
		      EndStep($status);
		      return $status if ($status);
		      open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		      print  LOG "DUM $prepro_file\n";
		      close( LOG );
		  }

		  #------------------------------------------------------------------------
		  ## Instrumentation
		  print &PerlTranslate("Source.pl", "%1: Instrumenting ", "%1 is the file to be instrumented", $srcfile);
		  print &PerlTranslate("Source.pl", "using TDP \"%1\" in \"%2\"","%1 is the TDP name %2 is its path",$Ini{NAME},$ENV{ATLTGT}) if ($ENV{ATTOLSTUDIO_VERBOSE});
		  print "...\n";

		  my $instr_opts = "";

		  if ( ! $is_ada && ! $is_java ) {
		    $instr_opts = BuildCCppInstrOptions($language,"","","");  # opts for C/C++ only
		  } elsif ( $is_java ) {
		    $instr_opts = BuildJavaInstrOptions();
		  } else {
		    $instr_opts = BuildAdaInstrOptions();

		    my $status = BuildAdaLinkFile();
		    if ( $status != 0 ) { return(1); }

		    $instr_opts .= " -link=adalink.alk" ;

		    #LOG
		    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		    print  LOG "ALK adalink.alk\n";
		    close( LOG );
		  }

		  if ( $is_java ) {
		    $instr_file = "$INTDIR"; # It's a directory

		    ## Do not forget the JIR file in output log
		    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		    print  LOG "DUM $INTDIR/javi.jir\n";
		    close( LOG );
		  }
		  else
		  {
		    ## Do not forget the AUG file in output log
		    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		    print  LOG "DUM $instr_file\n";

		    if ( $is_ada )
		    {
		       print  LOG "DUM ".lc("$INTDIR/$ATC_INSTR_CTRL_PREFIX"."atc_"."$srcname"."_adb.ali\n");  
		       print  LOG "OBJ ".lc("$INTDIR/$ATC_INSTR_CTRL_PREFIX"."atc_"."$srcname"."_adb.$OBJEXT\n");  
		    }
		    close( LOG );
		  }

		  $status = atl_instr ($language, $prepro_file, $instr_file, $instr_opts);
		  $status && die &PerlTranslate("Source.pl", "Failed.\n");

		  if ($ATCMODE) {# log the tio file for the studio file tracking
			open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
			print  LOG "TIO $ATC_TIO_NAME\n" ;  
			close( LOG );
		  }

		  ## ATK compilation file
		  if ($ATKMODE)
		  {
		    $status = MakeCompilationOptionsFile ();
		    if ($status != 0) { return $status; }
		  }

		  my @all_includes = @COMPLETE_INCLUDE_DIR_LIST;

		  unshift(@all_includes,$INTDIR) if ($ATKMODE);
		  push(@all_includes,$TARGETDIR.$PATH_SEPARATOR."lib") if ($Ini{NOPATH});

		  if ( $is_java ) {
		    my $line = "";
		    my @instr_list = ();

		    (-f GetJavaProductFile()) or
		      ( $ATJMODE and die &PerlTranslate("Source.pl", "%1 has not been generated yet\n -> Build first the Java Testing node\n", "%1 is name of the file that needs to be generated", &GetJavaProductFile) ) or
		      die &PerlTranslate("Source.pl", "%1 has not been generated yet\n -> Build first the Application node\n", "%1 is name of the file that needs to be generated", &GetJavaProductFile);

		    open ( LOG, "<$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
		    while ($line = <LOG>)
		    {
		      if ( $line =~ /^SRC /)
		      {
			my ($src) = ($line =~ /^SRC (.*)/);
			push( @instr_list, $src );
		      }
		    }
		    close (LOG);
		    push( @instr_list, GetJavaProductFile() );
		    $instr_file = join("\" \"", @instr_list);
		    $obj = "$INTDIR";
		    @JAVA_CLASSPATH = ( "$TARGETDIR/lib", @JAVA_CLASSPATH );
		  }

		  #--------------------------------------------------------------------------
		  ## Compilation
		  print &PerlTranslate("Source.pl", "%1: Compiling...\n", "%1 is file to be compiled", $srcfile);

		  if ($language eq "ASM") {
			$status=atl_asm ( $language, $instr_file, $obj, $ASMFLAGS,
				   \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
		  }
		  else {
			$status=atl_cc ( $language, $instr_file, $obj, $CPPFLAGS . " " . $CFLAGS,
				   \@GENERATION_CONDITION, \@all_includes);
		  }
		  EndStep($status);
		  return $status if($status);

		  #--------------------------------------------------------------------------
		  ## Intermediate files remove
		  if ( ! $is_ada && ! $is_java && ($language ne "ASM"))
		  {
		    unlink($prepro_file) if (!$ENV{'ATTOLSTUDIO_VERBOSE'});
		    unlink($instr_file) if (!$ENV{'ATTOLSTUDIO_VERBOSE'});
		  }
		}
		else
		{
		  #----------------------------------------------------------------------------
		  ## Compilation only

		  # Launch Ada instrumentor to add Test Program Entry Point if any
		  if (($language eq "ADA") and (! $PACKAGES_TESTED eq "")) {
			my $status = BuildAdaLinkFile();
			if ( $status != 0 ) { return(1); }
			$instrOpts .= " -link=adalink.alk" ;

			$instrOpts .= " -INJECT=$PACKAGES_TESTED";
			if ( ! $TEST_ENTRY_POINT_NAME eq "" ) { $instrOpts .= " -NAMEINJECT=\"$TEST_ENTRY_POINT_NAME\""; }
			$status = atl_instr ($language, $FILENAME, $instr_file, $instrOpts);
			$status && die &PerlTranslate("Source.pl", "Failed.\n");
			print "PACKAGES_TESTED: $instrOpts\n";
		  }

		  if ($language eq "RC")
		  {
		     #---- resource compilation
		     my $res="res"; #default ext
		     $res = $Ini{'RESEXT'} if ($Ini{'RESEXT'});
		     $obj =  "$INTDIR".$PATH_SEPARATOR."$srcname.$res";
			print &PerlTranslate("Source.pl", "%1: Compiling Resource File...\n", "%1 is the name of the resource file", $srcfile);
			$status=atl_cc ( $language, $FILENAME, $obj, $CPPFLAGS,
					 \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
		  }
		  else
		  {
			print &PerlTranslate("Source.pl", "%1: Compiling...\n", "%1 is the name of the file to be compiled", $srcfile);
			if ( $is_java ) {
			  $obj = $INTDIR;
			}

			if ($language eq "ASM") {
				$status=atl_asm ( $language, $FILENAME, $obj, $ASMFLAGS,
					 \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
			}
			elsif (($language eq "ADA") and (! ($PACKAGES_TESTED eq ""))) {
				$status=atl_cc ( $language, $instr_file, $obj, $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS,
					 \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );	
			}
			else {
				push(@COMPLETE_INCLUDE_DIR_LIST,$TARGETDIR.$PATH_SEPARATOR."lib") if ($Ini{NOPATH});
				$status=atl_cc ( $language, $FILENAME, $obj, $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS,
					 \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST );
			}
			EndStep($status);
			return $status if($status);
		  }
		}
	}
	unless($ATSMODE) {
		#--------------------------------------------------------------------------
		## Misra
		if ( $MISRA_MODE && ($language eq "C"))
		{
		  $somethingDone=1;
		  my $res=codeChecker_cc($FILENAME);
		  if ($res eq "")
		  {
		      die &PerlTranslate("Source.pl", "Generation failed.\n");
		  }

		}

		#--------------------------------------------------------------------------
		## Metrics
		if ( $ATMMODE ) {
		      if (($language eq "C") || ($language eq "CPP"))
			{
			  $somethingDone=1;
			  print &PerlTranslate("Source.pl", "%1: Static Metrics Computing...\n", "%1 is file to be compiled", $srcfile);
			  $status=metricsC ($language, $FILENAME, $LOGFILENAME, $OUTDIR);
			}
		      elsif (($is_ada) || ($is_java))
			{
			  $somethingDone=1;
			  print &PerlTranslate("Source.pl", "%1: Static Metrics Computing...\n", "%1 is file to be compiled", $srcfile);
			  $status=metricsA ($language, $FILENAME, $LOGFILENAME, $OUTDIR);
			}
		      if (($status!=0) && (($status!=3)||(!$CONTONWARN)))
			{
			    die &PerlTranslate("Source.pl", "Generation  failed. Status $status\n");
			}
		}
	}

	if(( ! $is_java ) && ($STAGE =~ /c/)){
	  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
	  print  LOG "OBJ $obj\n";

	  if ( $is_ada ) {
	     print LOG "DUM $INTDIR/$srcname.ali\n";
	  }

	  close( LOG );
	}
print &PerlTranslate("Source.pl", "No steps selected.\n") unless($somethingDone);
return 0;
}
#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

CheckIntDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
exit (Source());
}
1;
#
################################################################################
