#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Traces Splitter Script  -*-                        #
#                                                                              #
#                                                                              #
#  This script splits the traces file produced by tester / instrumented        #
#  application into components-specific traces files.                          #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   $NAME          : name of test node (used as a prefix for traces files)     #
#   $ATxMODE       : use x component                                           #
#                                                                              #
################################################################################
#$Id: Split.pl,v 1.93 2007/10/15 13:16:39 rfrance Exp $
################################################################################

# Kinds of output files 
$tdf = 1;
$tio = 2;
$rio = 3;
$tpf = 4;
$tqf = 5;
$undefined = 7;

@log_codes = ("ERR", "TDF", "TIO", "RIO", "TPF", "TQF");
@ext = ("err", "tdf", "tio", "rio", "tpf", "tqf");
@used = (0, 0, 0, 0, 0, 0);

$tdf_file_mode = "w";

# Return the file type matching an opcode
sub GetOpcodeProduct #($)
{
  my $opcode = shift;

  if ($opcode eq "SF" ||
      $opcode eq "CS" ||
      $opcode eq "CI" ||
      $opcode eq "LK" ||
      $opcode eq "ST" ||
      $opcode eq "NO" ||
      $opcode eq "NI" ||
      $opcode eq "DI" ||
      $opcode eq "ME" ||
      $opcode eq "MR" ||
      $opcode eq "MC" ||
      $opcode eq "SY" ||
      $opcode eq "SA" ||
      $opcode eq "AC" ||
      $opcode eq "LS" ||
      $opcode eq "LE" ||
      $opcode eq "TM" ||
      $opcode eq "TS" ||
      $opcode eq "TU" ||
      $opcode eq "OT" ||
      $opcode eq "US" ||
      $opcode eq "IN" ||
      $opcode eq "MS" ||
      $opcode eq "TX" ||
      $opcode eq "SO" ||
      $opcode eq "SI" ||
      $opcode eq "HS" ||
      $opcode eq "TI" ||
      $opcode eq "ID" ||
      $opcode eq "DB" ||
      $opcode eq "ED" ||
      $opcode eq "BA" ||
      $opcode eq "EA" ||
      $opcode eq "GL")
  {
    # we do not differenciate TSF/TDF opcode
    $used[$tdf] = 1;
    return $tdf;
  }
  elsif ($opcode eq "NT" ||
         $opcode eq "DT" ||
         $opcode eq "CO" ||
         $opcode eq "FT" ||
         $opcode eq "DC" ||
         $opcode eq "TP" ||
         $opcode eq "TA" ||
         $opcode eq "TB" ||
         $opcode eq "TE" ||
         $opcode eq "EI")
  {
    $used[$tio] = 1;
    return $tio;
  }
  elsif ($opcode eq "PU" ||
         $opcode eq "PT" ||
         $opcode eq "PS" ||
         $opcode eq "PY" ||
         $opcode eq "PE" ||
         $opcode eq "PB" ||
         $opcode eq "PF" ||
         $opcode eq "PO" ||
         $opcode eq "PX")
  {
    $used[$tpf] = 1;
    return $tpf;
  }
  elsif ($opcode eq "QU" ||
         $opcode eq "QT" ||
         $opcode eq "QD" ||
         $opcode eq "QS" ||
         $opcode eq "QF")
  {
    $used[$tqf] = 1;
    return $tqf;
  }
  else
  {
    return 0;
  }
}

# Substitute Time and Date on some opcodes
sub SubstituteTimeAndDate #($$)
{
  my ($product,$text) = @_;
  my $result = $text;

  $_ = $text;
  if (/^DT 0$/ ||
      /^DT \"\"$/)
  {
    $result = "DT \"" . GetDate . "\"\n";
  }
  elsif (/^PT \"(.*)\"$/)
  {
    $result = "PT \"$1\" \"" . GetDate . "\"\n";
  }
  elsif (/^QD 0$/ ||
      /^QD \"\"$/)
  {
    $result = "QD \"" . GetDate . "\"\n";
  }
  elsif (/^H0$/)
  {
    $result = "H0 " . GetDate . "\n";
  }
  elsif (/^h 0/)
  {
     $result = "h " . GetDate . "\n";
  }
  return $result;
}

# Outputs into traces file for $product the line $text
sub WriteForProduct #($$)
{
  my ($product,$text) = @_;
  if ($opened[$product])
  {
    my $text_ = SubstituteTimeAndDate($product,$text);
    print { $handles[$product] } $text_;
  }
}

# Split $traces_file
sub ExecSplit #($$)
{
  my ($traces_file, $print_warnings) = @_;

  my $product = 0;
  my $line = 0;
  # Always a warning:
  $print_warnings = 1;
  my $warning_printed = 0;
  my $rejected = "atlout.rej";
  my $text = "";

  open (TRACES, "<$traces_file") || return 1;
  open (REJECTED, ">$rejected") || ($warning_printed = 0);
 
  while(<TRACES>)
  {
    $text = $_;
    $line++;
    if (/^([A-Z][A-Z])( .*)?/ &&
        ($product = GetOpcodeProduct ($1)))
    {
      WriteForProduct ($product, $_);
    }
    elsif ($ATUMODE &&
           /^[A-FGHJ-ORTV-Z][0-9]+.*$/)
    {
      if(/^D[0-4] -/) {print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored\n", "");next;}
      $used[$rio] = 1;
      WriteForProduct ($rio, $_);
    }
    elsif ($ATSMODE &&
           (/^[a-df-z][0-9]+.*$/ ||
            /^f[bt][0-9]+.*$/ ||
            /^i[c-e][0-9]+.*$/ ||
            /^sd[0-9]+.*$/ ||
            /^t[fn][0-9]+.*$/ ||
            /^ATTOL SystemTest .*$/))
    {
      $used[$rio] = 1;
      WriteForProduct ($rio, $_);
    }
    elsif ($print_warnings)
    {
      if(/^tdp/) {next;} #opcode used by TDP, line suppressed
      print &PerlTranslate("Split.pl", "%1:%2: unrecognized line (file %3 contains all the rejected lines)\n", "%1 is the file being split, %2 is the line number and %3 is name of the file to look into", $traces_file, $line, $rejected) if (!$warning_printed);
      $warning_printed = 1;
      print REJECTED "$line:$text";
    }
  }
  close (REJECTED);
  if (!$warning_printed) {
    unlink ($rejected);
  } else {
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
    print  LOG "DUM $rejected\n";
    close( LOG );
  }

  close (TRACES);
  return 0;
}

sub OpenDestFile #($$$)
{
   my ($type, $name, $mode) = @_;

   $output_files[$type] = $name;
   $handles[$type] = FileHandle->new ();
   if ($mode eq "a") {
     if(($warningTooBigFile) && ((-s $name) >4500000)){ #4.5MB
        $warningTooBigFile=0;
        print &PerlTranslate("Split.pl", "\nThe result files are growing and might be difficult to show. we suggest to reset the results before the next execution (see the report viewer menu)\n\n", "");
     }
     open ($handles[$type], ">>$name") 
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   } else {
     open ($handles[$type], ">$name") 
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   }
   $opened[$type] = 1;
}

sub CloseDestFile #($)
{
   my $type = shift;

   if ($used[$type]) {
     close( $handles[$type] );
   }
}

sub SplitTraces #($)
{
   my $tracesfile = shift;
   
   print &PerlTranslate("Split.pl", "Splitting '%1' traces file...\n", "%1 is the name of the file to be split", $tracesfile);

   my $warnings = 0;
   my $filen = 0;
   my $product = 0;
   my $one_file = "";

   if ($ATOMODE || $ATKMODE || $ATTMODE && !$ATJMODE)
   {
     $filen++;
     $product = $tdf;
     $TDFFILE = $NAME.'.'.$ext[$tdf] unless($TDFFILE);
     $one_file = $TDFFILE;
   }
   if ($ATCMODE)
   {
     $filen++;
     $product = $tio;
     $ATC_TIO_NAME=$NAME.'.'.$ext[$tio] unless($ATC_TIO_NAME);
   }
   if ($ATUMODE || $ATSMODE)
   {
     $filen++;
     $product = $rio;
     if ($ATUMODE)
     {
	$RIOFILE = $NAME.'.'.$ext[$rio] unless ($RIOFILE);
     }
     elsif ($ATSMODE)
     {
	$RIOFILE = "$tracesfile.date";
     }
     $one_file = $RIOFILE;
   }
   if ($ATPMODE)
   {
     $filen++;
     $product = $tpf;
     $ATP_TPF_NAME=$NAME.'.'.$ext[$tpf] unless($ATP_TPF_NAME);
   }
   if ($ATQMODE)
   {
     $filen++;
     $product = $tqf;
     $ATQ_TQF_NAME=$NAME.'.'.$ext[$tqf] unless($ATQ_TQF_NAME);
   }
   
   #### generate LOG 
   if($SOCKET_UPLOAD)
   {
      # assuming trtpd is currently splitting
      my $waitlong;
      while (!(-f "trtpd.log") && $waitlong < 5){sleep(2);$waitlong +=2;}
      print &PerlTranslate("Split.pl", "Split unneeded\n") if ( $ENV{'ATTOLSTUDIO_VERBOSE'} );
      $used[$tdf]=(-f $TDFFILE);
      $used[$rio]=(-f $RIOFILE);
      $used[$tio]=$ATCMODE;
      $used[$tpf]=$ATPMODE;
      $used[$tqf]=$ATQMODE;

      my $memsum=MemproSummary($ATP_TPF_NAME) if($used[$tpf]);
      if ($LOGFILENAME) {
        open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
        print  LOG "TDF $TDFFILE\n"      if ($used[$tdf]);
        print  LOG "TIO $ATC_TIO_NAME\n" if ($used[$tio]);
        print  LOG "RIO $RIOFILE\n"      if ($used[$rio]);
        print  LOG "TPF $ATP_TPF_NAME\n" if ($used[$tpf]);
        print  LOG "TQF $ATQ_TQF_NAME\n" if ($used[$tqf]);
        print  LOG "DUM $memsum\n" if ($memsum);
        close ( LOG );
      }
      return 0;
   }  
   elsif ( ! $ATSMODE && !$ATL_MULTI_PROCESS && ($ATTMODE || $ATJMODE) && $ATT_ON_THE_FLY )
   {
      # assuming gui is currently splitting
      $UIriofile=$INTDIR."/atlout.rio"; #all rio line are written in this file
      
      sleep (3) ;
      #wait End Mark
      my $waitlong;
      while (!(-f $INTDIR."/atlendtest.txt") && $waitlong < 15){sleep(2);$waitlong +=2;}
      unlink($INTDIR."/atlendtest.txt");
      $used[$rio]=$ATUMODE;
      if($ATUMODE && (-f $UIriofile)) {
          # unknown line are rejected by the UI; So now split those rejected lines
          RenameFile($UIriofile,$RIOFILE);
      }else {
      	  print &PerlTranslate("Split.pl", "Split unneeded\n") if ( $ENV{'ATTOLSTUDIO_VERBOSE'} );
          $used[$rio]=0;
      }
      $used[$tio]=1 if($ATCMODE && (-f $ATC_TIO_NAME));

      $used[$tdf]=(-f $TDFFILE);# this file is written at the beginning of the application
      #The split for TPF related information is done by SplitJvmpi for Java.
      $used[$tpf]=1 if($ATPMODE && ($TDP_LANGUAGE ne "JAVA"));
      $used[$tqf]=$ATQMODE;
      my $memsum=MemproSummary($ATP_TPF_NAME) if($used[$tpf]);
      if ($LOGFILENAME) {
        open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
        print  LOG "TDF $TDFFILE\n";
        print  LOG "TIO $ATC_TIO_NAME\n" if ($used[$tio]);
        print  LOG "TPF $ATP_TPF_NAME\n" if ($used[$tpf]);
        print  LOG "RIO $RIOFILE\n"      if ($used[$rio]);
        print  LOG "TQF $ATQ_TQF_NAME\n" if ($used[$tqf]);
        print  LOG "SPT $tracesfile\n";
        print  LOG "DUM $memsum\n" if ($memsum);
        close ( LOG );
      }
      return 0; ## if studio split

   }
   elsif ($filen == 1 && ($ATOMODE || $ATKMODE))
   # Optimize for these two ones
   {
     # No use to split file : rename only
     $output_files[$product] = $one_file;
     $used[$product] = 1;
     RenameFile ($tracesfile, $output_files[$product]);
     open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
     print  LOG "$log_codes[$product] $output_files[$product]\n";
     close( LOG );
     return 0;
   }
   
   ### now Do it
   
   if ($ATOMODE || $ATKMODE || $ATJMODE || $ATTMODE)
   {
     OpenDestFile ($tdf, $TDFFILE, $tdf_file_mode);
   }

   if ($ATCMODE)
   {
     OpenDestFile ($tio, $ATC_TIO_NAME, "a");
   }

   if ($ATPMODE)
   {
     OpenDestFile ($tpf, $ATP_TPF_NAME, "a");
   }

   if ($ATQMODE)
   {
     OpenDestFile ($tqf, $ATQ_TQF_NAME, "a");
   }
   if ($ATUMODE || $ATSMODE)
   {
     OpenDestFile ($rio, $RIOFILE, "w");
   }

   if (ExecSplit ($tracesfile, $warnings) != 0)
   {
     print &PerlTranslate("Split.pl", "Error while splitting traces file '%1'.\n", "%1 is the name of the file to be split", $tracesfile);
     return 1;
   }
   for ($i = $tdf; $i < $undefined; $i++)
   {
     if ($opened[$i])
     {
       close ($handles[$i]);
     }
   }
   if ($warnings)
   {
     if (!$opened[$tdf] && $used[$tdf])
     {
       print &PerlTranslate("Split.pl", "Warning: TDF traces were ignored.\n");
     }
     if (!$opened[$tio] && $used[$tio])
     {
       print &PerlTranslate("Split.pl", "Warning: TIO traces were ignored.\n");
     }
     if (!$opened[$rio] && $used[$rio])
     {
       print &PerlTranslate("Split.pl", "Warning: RIO traces were ignored.\n");
     }
     if (!$opened[$tpf] && $used[$tpf])
     {
       print &PerlTranslate("Split.pl", "Warning: TPF traces were ignored.\n");
     }
     if (!$opened[$tqf] && $used[$tqf])
     {
       print &PerlTranslate("Split.pl", "Warning: TQF traces were ignored.\n");
     }
   }

   open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n") unless($ATSMODE);

   if ($ATUMODE || $ATSMODE)
   {
     CloseDestFile ($rio);
     print  LOG "$log_codes[$rio] $output_files[$rio]\n" unless($ATSMODE);
   }

   if ($ATOMODE || $ATKMODE || $ATJMODE || $ATTMODE)
   {
     CloseDestFile ($tdf);
     print  LOG "$log_codes[$tdf] $output_files[$tdf]\n" unless($ATSMODE);
   }

   if ($ATCMODE)
   {
     CloseDestFile ($tio);
     print  LOG "$log_codes[$tio] $output_files[$tio]\n" unless($ATSMODE);
   }

   if ($ATPMODE)
   {
     CloseDestFile ($tpf);
     unless($ATSMODE){
        print  LOG "$log_codes[$tpf] $output_files[$tpf]\n" ;
        my $memsum=MemproSummary($output_files[$tpf]);
        print  LOG "DUM $memsum\n" if($memsum);
     }
   }

   if ($ATQMODE)
   {
     CloseDestFile ($tqf);
     print  LOG "$log_codes[$tqf] $output_files[$tqf]\n" unless($ATSMODE);
   }
   if($ENV{'ATTOLSTUDIO_VERBOSE'})
   {
      unlink ("$tracesfile.bak");
      rename ($tracesfile,"$tracesfile.bak");
      print  LOG "DUM $tracesfile.bak\n";
   }
   else
   {
      unlink ($tracesfile);
   }
   close( LOG ) unless($ATSMODE);
   print &PerlTranslate("Split.pl", "Traces file successfully split.\n");
   return 0;
}

sub MemproSummary
{
   my $file=shift;
   my $out=$file; $out=~s/\.tpf$//;
   my %ERR;
   my $x;
   open IN, "$file" || return 0;
   while(<IN>){ 
   	if(/^PE (\w+)/){
   	   my $e=$1;
   	   unless($e=~ m/CHK|FIU/){
   	      $ERR{$e}++ ;
   	      $x++;
   	   }
   	   #print "Mempro $e, $_, $ERR{$e}\n";
   	}
   }
   close IN;
   if($x) {
      open OUT, ">$out.atp.txt";
      foreach (sort keys %ERR){
   	print OUT "$_ found $ERR{$_} times\n";
      }
      print OUT "Total of $x Errors found\n";
      close OUT;
      print "$x Memory Errors found\n";
      return "$out.atp.txt";
   }
   return 0;
}

sub ParseAtsRioFile #($)
{
   my $riofile = shift;
   SplitTraces ($riofile) and return 1;
   rename ("$riofile.date", $riofile) or return 1;
   return 0;
}
1;
