#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Jvmpi Traces Splitter Script  -*-                  #
#                                                                              #
#                                                                              #
#  This script splits the traces file produced by tester / instrumented        #
#  application into components-specific traces files.                          #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   $NAME          : name of test node (used as a prefix for traces files)     #
#   $ATxMODE       : use x component                                           #
#                                                                              #
################################################################################
#$Id: SplitJvmpi.pl,v 1.21 2007/06/26 14:03:25 rfrance Exp $
################################################################################

# Configuration file
#already loaded #require $ARGV[0];

require $SCRIPTDIR . "Fcts.pl";

#
# Kinds of output files 
#
$jtdf = 1;
$jtpf = 2;
$jundefined = 3;
#
#
# Note: Change the extension for the TSF generated by Java ( DR: 1552 ).
@jlog_codes = ("ERR", "TXF", "TPF", "TXF");
@jused = (0, 0, 0);
#
#Global var for the JVMPI Tsf.
#
@JvmpiTSFBuffer;
$JvmpiCurrentClassName;
$JvmpiCurrentFileName;
$JvmpiClassHasBeenFound;
$JvmpiFileHasBeenFound;
$JvmpiExistTheCurrentTsfFile;
#
# Set the files string for the gui part.
#
sub ResetPathForGuiPart #($)
{

    my $string = shift;
    return TestRTUtil::LocaleStringReplace( $string, "\\\\", "\\\\" );
}

#
# Enable the return to the beginning of file or the line.
#
sub IsOtherInstrTool
{
    if ( $ATUMODE || $ATTMODE || $ATJMODE || $ATQMODE || $ATCMODE )
    {
	return 1;
    }
    return 0;

}
#
# Global vars to load the name of class and so on.
#
sub ResetGlobalJvmpi
{
    @JvmpiTSFBuffer=();
    $JvmpiCurrentClassName="";
    $JvmpiCurrentFileName="";
    $JvmpiClassHasBeenFound=0;
    $JvmpiFileHasBeenFound=0;
    $JvmpiExistTheCurrentTsfFile=0;
}
#
# In order to know if the LK has to be set to 0 or not.
#
sub CheckThatFileExist
{
    my $full_name;
    
    foreach $path (@JAVA_CLASSPATH)
    {
	$full_name = "$path/$JvmpiCurrentFileName";

	if ( open(TEST, "<$full_name") )
	{
	    $JvmpiCurrentFileName = $full_name;
	    $JvmpiExistTheCurrentTsfFile =  1;
	    return;
	}
	else
	{
	    $JvmpiExistTheCurrentTsfFile =  0;
	} 
    }
}
#
# For evolution.
#
sub ResetPathSeparator #($)
{
    my $in = shift;
    return $in;
}
#
# UpdateFileNameFrom Jvmpi mangle.
#
sub UpdateFileName #($)
{
    my $class_name = shift;
    my $position_end_dot;

    my $position_dot;
    $position_dot = index($class_name,".");

    if ($position_dot == -1)
    {
	$position_dot = index($class_name,"/");
	if ($position_dot == -1)
	{
	    return;
	}
	else
	{
	    $position_last = rindex(class_name,"/");
	    if ($position_last == -1)
	    {
		return;
	    }
	    
	    my $extracted;
	    $extracted = substr($class_name,0,($position_last+1));
	    $full_name = "$extracted/$JvmpiCurrentFileName";
	    $JvmpiCurrentFileName = resetpathseparator( $JvmpiCurrentFileName );
	    return;
	}
    }

    #Replace now the dots.
    $position_end_dot = rindex($class_name,".");
    if ($position_end_dot != -1)
    {
	$temp = substr($class_name,0,($position_end_dot+1));
	$pattern = "/";
	$searched = ".";
	$temp=~s/\./$pattern/g;
	$JvmpiCurrentFileName = "$temp/$JvmpiCurrentFileName";
    }

}
#
# FlushTheBuffer when the class and so on is known.
#
sub FlushJvmpiTsfBuffer #($)
{
  my $product_ = shift;
  my $line_;

  if ($opened[$product_])
  {
      foreach $line_ (@JvmpiTSFBuffer)
      {
	  my $opcode = $line_;
	  $opcode = substr($opcode,0,2);
	  if ($opcode eq "SO")
	  {
	      $line_ = "SO \"$JvmpiCurrentFileName\" 0 \"txt\"\n";
	      $line_ = ResetPathForGuiPart($line_);
	  }
	  print { $handles[$product_] } $line_;
      }
      
      @JvmpiTSFBuffer=();
  }
}
#
#Append a line in the TSF Buffer for JVMPI.
#
sub AppendLineInBuffer #($$$)
{
  my ($opcode,$line,$product) = @_;

  if ($opcode eq "DB")
  {
      ResetGlobalJvmpi();
  }
  elsif ($opcode eq "ED")
  {
      push @JvmpiTSFBuffer, $line;
      FlushJvmpiTsfBuffer($product);
      return;
  }
  elsif ($opcode eq "IN")
  {
      if ($JvmpiFileHasBeenFound==0)
      {
	  $index =  index($line,"IN \"File\"");
	  if ($index != -1)
	  {
	      $position_start_name = index($line,"\"",9);
	      $position_end_name = index($line,"\"",($position_start_name+1));
	      $JvmpiCurrentFileName = substr($line,($position_start_name+1),($position_end_name - ($position_start_name+1)));
	      $JvmpiFileHasBeenFound=1;
	  }
      }
      elsif ($JvmpiFileHasBeenFound==1)
      {
	  $index =  index($line,"IN \"Class\"");
	  if ($index != -1)
	  {
	      $position_start_name = index($line,"\"",10);
	      $position_end_name = index($line,"\"",($position_start_name+1));
	      $JvmpiCurrentClassName = substr($line,($position_start_name+1),($position_end_name - ($position_start_name+1)));
	      $JvmpiClassHasBeenFound=1;
	      #For IBM purpose.
	      $JvmpiCurrentClassName=~s/\//\./g;
	      UpdateFileName($JvmpiCurrentClassName);
	      CheckThatFileExist();
	  }
      }
  }
  elsif ($opcode eq "LK")
  {
      if ( $JvmpiClassHasBeenFound && !$JvmpiExistTheCurrentTsfFile)
      {
	  $line = "LK 0\n";
      }
      elsif ( $JvmpiClassHasBeenFound && $JvmpiExistTheCurrentTsfFile)
      {
	  if (IsOtherInstrTool())
	  {
	      $line = "LK 1\n";
	  }
      }
  }
	  
  push @JvmpiTSFBuffer, $line;
  return;


}
#
# Return the file type matching an opcode
#
sub GetJvmpiOpcodeProduct #($)
{
    my $opcode = shift; 
    
    if ($opcode eq "SF" ||
      $opcode eq "CS" ||
      $opcode eq "CI" ||
      $opcode eq "LK" ||
      $opcode eq "ST" ||
      $opcode eq "NO" ||
      $opcode eq "NI" ||
      $opcode eq "DI" ||
      $opcode eq "ME" ||
      $opcode eq "MR" ||
      $opcode eq "MC" ||
      $opcode eq "SY" ||
      $opcode eq "SA" ||
      $opcode eq "AC" ||
      $opcode eq "LS" ||
      $opcode eq "LE" ||
      $opcode eq "TM" ||
      $opcode eq "TS" ||
      $opcode eq "TU" ||
      $opcode eq "OT" ||
      $opcode eq "US" ||
      $opcode eq "IN" ||
      $opcode eq "MS" ||
      $opcode eq "TX" ||
      $opcode eq "SO" ||
      $opcode eq "SI" ||
      $opcode eq "HS" ||
      $opcode eq "TI" ||
      $opcode eq "ID" ||
      $opcode eq "DB" ||
      $opcode eq "ED" ||
      $opcode eq "BA" ||
      $opcode eq "EA" ||
      $opcode eq "GL")
  {
    # we do not differenciate TSF/TDF opcode
    $jused[$jtdf] = 1;
    return $jtdf;
  }
  elsif ($opcode eq "PU" ||
         $opcode eq "PV" ||
         $opcode eq "PP" ||
         $opcode eq "PC" ||
         $opcode eq "PD" ||
         $opcode eq "PG" ||
         $opcode eq "PX")
  {
    $jused[$jtpf] = 1;
    return $jtpf;
  }
  elsif ($opcode eq "XX" ||
         $opcode eq "YY")
  {
    return $jundefined;
  }
  else
  {
    return 0;
  }
}
#
# Substitute Time and Date on some opcodes
#
sub JvmpiSubstituteTimeAndDate #($$)
{
  my ($product,$text) = @_;
  my $result = $text;

  $_ = $text;
  if (substr($text, 0, 2) eq "DT")
  {
    $result = "DT \"" . GetDate . "\"\n";
  }
  elsif (substr($text, 0, 2) eq "PT")
  {
    if (length($text) == 3) {
      $result = "PT \"\" \"" . GetDate . "\"\n";
    } else {
      my $text2 = substr($text, 0, length($text)-1);
      $result = "$text2 \"" . GetDate . "\"\n";
    }
  }
  elsif (substr($text, 0, 2) eq "QD")
  {
    $result = "QD \"" . GetDate . "\"\n";
  }
  elsif (substr($text, 0, 2) eq "H0")
  {
    $result = "H0 " . GetDate . "\n";
  }
  elsif (substr($text, 0, 3) eq "h 0")
  {
     $result = "h " . GetDate . "\n";
  }
  return $result;
}
#
# Outputs into traces file for $product the line $text
#
sub JvmpiWriteForProduct #($$$)
{
  my ($opcode,$product,$text) = @_;
  if ($product == $jtdf)
  {
      AppendLineInBuffer($opcode,$text,$jtdf);
  }
  else
  {
      if ($opened[$product])
      {
	  my $text_ = JvmpiSubstituteTimeAndDate($product,$text);
	  print { $handles[$product] } $text_;
      }
  }
}
#
# Split $traces_file
#
sub ExecJvmpiSplit #($$)
{
  my ($traces_file, $print_warnings) = @_;

  my $product = 0;
  my $line = 0;

  open (TRACES, "<$traces_file") || return 1;
 
  while(<TRACES>)
  {
    $line++;
    if (/^([A-Z][A-Z])( .*)?$/)
    {

	if (($product = GetJvmpiOpcodeProduct ($1)))
	{
	    if ($product != $jundefined)
	    {
		JvmpiWriteForProduct ($1,$product, $_);
	    }
	}
	elsif ($print_warnings)
	{
	    print &PerlTranslate("SplitJvmpi.pl", "%1:%2: MINE unrecognized op-code `%3'\n", "%1 is the file being split, %2 is the line number and %3 is unrecognized opcode", $traces_file, $line, $1);
	}
    }
    elsif (/^[A-DGHJ-ORTV-Z][0-9]+.*$/)
    {
	$jused[$rio] = 1;
	JvmpiWriteForProduct ($1,$rio, $_);
    }
    elsif ($ATSMODE)
    {
	$jused[$rio] = 1;
	JvmpiWriteForProduct ($1,$rio, $_);
    }
    elsif ($print_warnings)
    {
	print &PerlTranslate("SplitJvmpi.pl", "%1:%2: non TestRT generated line ignored\n", "%1 is the file being split and %2 is the line number", $traces_file, $line);
    }
}
  
  close (TRACES);
  return 0;
}
#
#
#
sub JvmpiOpenDestFile #($$$)
{
    my ($type, $name, $mode) = @_;
    
    $output_files[$type] = $name;

    $handles[$type] = FileHandle->new ();
    if ($mode eq "a") {
	open ($handles[$type], ">>$name") 
	    or die &PerlTranslate("SplitJvmpi.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the file", $name);
    } else {
	open ($handles[$type], ">$name") 
	    or die &PerlTranslate("SplitJvmpi.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the file", $name);
    }
    $opened[$type] = 1;
}
#
#
#
sub JvmpiCloseDestFile #($)
{
    my $type = shift;
    
    if ($jused[$type]) {
	close( $handles[$type] );
	if (!$ATSMODE)
	{
	    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("SplitJvmpi.pl", "Cannot open log file.\n");
	    print  LOG "$jlog_codes[$type] $output_files[$type]\n";

	    close( LOG );
	}
    }
}
#
#
#
sub SplitJvmpiTraces
{
    $JPTNAME = "$INTDIR/$NAME.jpt";
    $warnings = "1";
    
    my $TPFNAME;
    my $TSFNAME; 
    
    if ($JAVA_TPF_FILENAME eq "<default>")
    {
	$TPFNAME = "$INTDIR/$NAME.tpf";
    }
    else
    {
	$TPFNAME = "$INTDIR/$JAVA_TPF_FILENAME.tpf";
    }
    
    if ($JAVA_TSF_FILENAME eq "<default>")
    {
	$TSFNAME = "$INTDIR/$NAME.txf";#DR 1552 to avoid mix between trace and purify.
    }
    else
    {
	$TSFNAME = "$INTDIR/$JAVA_TSF_FILENAME.txf";#DR 1552 to avoid mix between trace and purify.
    }
    
    if ( $ATPMODE && $TDP_LANGUAGE eq "JAVA" ) 
    {
	JvmpiOpenDestFile ($jtdf, $TSFNAME, "a");
	JvmpiOpenDestFile ($jtpf, $TPFNAME, "a");
    }
   
    if ( $JAVA_HIT_KEY )
    {
	return 0;
    }
    
    my $product = 0;
    
    $product = $jtpf;
   
    print &PerlTranslate("SplitJvmpi.pl", "\nSplitting %1 jvmpi traces file...\n", "%1 is the name of the file to be split", $JPTNAME);

    if (ExecJvmpiSplit ($JPTNAME, $warnings) != 0)
    {
	print &PerlTranslate("SplitJvmpi.pl", "Error while processing traces file '%1'.\n", "%1 is the name of the file to be split", $JPTNAME);
        unlink $TSFNAME;
        unlink $TPFNAME;
	return 1;
    }

   for ($i = $jtdf; $i < $jundefined; $i++)
   {
     if ($opened[$i])
     {
       close ($handles[$i]);
     }
   }

    JvmpiCloseDestFile ($jtdf);
    JvmpiCloseDestFile ($jtpf);
    JvmpiCloseDestFile ($TSFNAME);

    #Then last modification to change to utf8.


    my $post = "utfizer" ;
    my $utf_command = $BINDIR . $PATH_SEPARATOR . $post . " \"$TSFNAME\" " ;
    my $postpro_status = SystemP($utf_command) ;
    
    print &PerlTranslate("SplitJvmpi.pl", "Traces file successfully split.\n");
    return 0;
}

1;







