#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                 -*-  Start JavaTesting Node -*-                              #
#                                                                              #
#                                                                              #
#  Script for Tracer Test Node.                                                #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME2         : name of the log file to generate.                  #
#   $TARGETDIR           : directory of the Target Package                     #
#   $BINDIR              : directory of the studio/tracer binaries.            #
#   $ATTOLHOME           : attol studio intallation directory.                 #
#   $SCRIPTFILENAME      : filename of the test script should be generated     #
#   @TESTEDFILENAMES     : list of filename should be tested                   #
#   @COMPONENTS          : list of classname under test                        #
#   @METHODS             : list of methods under test                          #
#   @INSERTEDFILENAMES   : list of filename inserted into test script          #
#   @INTEGRATEDFILENAMES : list of filename integrated to the test             #
#   $OPTIONS	         : options for test driver generation (-typical, ...)  #
#   $OTCDIR	         : directory of generated otc files                    #
#                                                                              #
################################################################################
#$Id: StartJavaTesting.pl,v 1.20 2007/07/23 18:41:58 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] ; #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
EnterInScript();

ClearLogFile( $LOGFILENAME2 );


#----------------------------------------------------------------------------
## Start generation
print &PerlTranslate("StartJavaTesting.pl", "Test Script generation %1...\n", "%1 is the name of the Test Script file to be generated", $SCRIPTFILENAME);

my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "startjava"; 

foreach $file (@INTEGRATEDFILENAMES) {
    $Command .= " \"$file\"";
}

my $aux = 0;
foreach $class (@COMPONENTS) {
    $aux = $aux +1;
    if ($aux==1) { $Command .= " -test_class \"$class\""; }
    else { $Command .= " \"$class\""; }
}

$aux = 0;
foreach $method (@METHODS) {
    
    $aux = $aux +1;
    
    my $jstartrealmethodline ="";
    my $jstartrealmethodname ="";
    my $jargumentstarts = "";
    
    my $position_wsp = index($method," ");
    if ($position_wsp == -1)
    {
	$jmyargumentstarts = $method;
    }
    else
    {
	$jstartrealmethodname = substr($method,0,($position_wsp));
	$jstartrealmethodline = substr($method,($position_wsp+1),($position_wsp+1));
	$jmyargumentstarts = " \"$jstartrealmethodname\" \"$jstartrealmethodline\" ";
    }
    
    
    if ($aux==1) { $Command .= " -test_method $jmyargumentstarts "; }
    else { $Command .= " $jmyargumentstarts "; }
}


if ($TESTDRIVERNAME) {
    $Command .= " -o \"$TESTDRIVERNAME\"";
}

if ($JPREFIX) {
    $Command .= " -pref \"$JPREFIX\"";
}

$aux = 0;
foreach $file (@SIMULATEDFILES) {
    $aux = $aux +1;
    if ($aux==1) { $Command .= " -sc \"$file\""; }
    else { $Command .= " \"$file\""; }
}

if ( $ODIR )
{
   if ($OSNAME eq "win32")
   {
      # The remaining character may be an anti-slash and this is not allowed
      # by the MS shell when followed by double-quote
      $ODIR =~ s|^(.*)(\\)+$|\1|;
   }
   $Command .= " -odir \"$ODIR\"";
}

    # see below, but don't dream about this, because up to now studio
    # never set ATJMODE when it compute metrics .... bast....!
    $ATJMODE = 1;
      #please add to classpath to <TDP>/test and <PROJECT_PATH>
      # in order to analyze correctly test generated files.
      # this is needed only et metric time (not a test generation time).
$Command .= BuildCompleteJavaClassPath(" -classpath ",$CLASSPATH_SEPARATOR,"$TARGETDIR/test".$CLASSPATH_SEPARATOR."$PROJECT_PATH");
$Command .= $OPTIONS if ($OPTIONS);

$Command .= " -studiolog \"$LOGFILENAME2\"";
$Command .= " ".$Ini{'START_OPT'} if($Ini{'START_OPT'});

  my $status;
  if($Ini{'ATTOLSTART'}) {
     print "ATTOLSTART specified as:$TARGETDIR/cmd/$Ini{'ATTOLSTART'}\n";
     print "sub attolStart() executed with $Command\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLSTART'}";
     my $status = attolStart($Command,'JAVA','*',$TESTDRIVERNAME);
  }
  else
  {
     my $status = SystemP($Command);
  }

if($status!=0) { print &PerlTranslate("StartJavaTesting.pl", "Generation failed.\n"); exit($status); }

#print &PerlTranslate("StartJavaTesting.pl", "Done.\n");

0;
# That's all Folks.
################################################################################
