#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                 -*-  Start ObjectTest Node -*-                               #
#                                                                              #
#                                                                              #
#  Script for Tracer Test Node.                                                #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME2         : name of the log file to generate.                  #
#   $TARGETDIR           : directory of the Target Package                     #
#   $BINDIR              : directory of the studio/tracer binaries.            #
#   $ATTOLHOME           : attol studio intallation directory.                 #
#   $OTDNAME             : filename of the test script should be generated     #
#   $TESTCLASSPREFIX     : prefix for the name of the test class               #
#   @TESTEDFILENAMES     : list of filename should be tested                   #
#   @CLASSES             : list of classname under test                        #
#   @METHODS             : list of methodname under test                       #
#   @INSERTEDFILENAMES   : list of filename inserted into test script          #
#   @INTEGRATEDFILENAMES : list of filename integrated to the test             #
#   $OPTIONS	         : additionnal options for test script generation      #
#   $OTCNAME	         : path and name for the otc file                      #
#                                                                              #
################################################################################
#$Id: StartObjecTest.pl,v 1.41 2007/07/23 18:41:58 rfrance Exp $
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] unless($main::TestRTCC); #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";

#----------------------------------------------------------------------------
## Start generation
sub StartObjecTest {

print &PerlTranslate("StartObjecTest.pl", "Test Script generation %1...\n", "%1 is the name of Test Script file to be generated", $SCRIPTFILENAME);

my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "atostart"; 

foreach $file (@TESTEDFILENAMES) {
    $Command .= " \"$file\"";
}

foreach $file (@INSERTEDFILENAMES) {
    $Command .= " -i \"$file\"";
}

foreach $file (@INTEGRATEDFILENAMES) {
    $Command .= " -integrate \"$file\"";
}

foreach $I ( @COMPLETE_INCLUDE_DIR_LIST )
{
	$Command .= " -I\"$I\"" ;
}
$Command .= " -opp atl.opp -hpp atl.hpp";

foreach $D ( @GENERATION_CONDITION )
{
	$Command .= " \"-D$D\"";
}
$Command .= $OPTIONS if ($OPTIONS);

foreach $class (@CLASSES) {
    $Command .= " -test_class \"$class\"";
}

foreach $idmethod (@METHODS) {
    $Command .= " -test_method \"$idmethod\"";
}

    $Command .= " -test_private_methods" if($ATO_FRIEND);

#By default, in GUI mode, always overwrite files (Studio avoids name collisions
#but this option is useful when the user goes back and forth between 
#test generation options and test generation log page in the wizard).
$Command .= " -overwrite";

$Command .= " -otd \"$OTDNAME\" -test_class_prefix \"$TESTCLASSPREFIX\" -otc \"$OTCNAME\" -studio_log \"$LOGFILENAME2\"";
$Command .= " ".$Ini{'START_OPT'} if($Ini{'START_OPT'});

  my $status;
  if($Ini{'ATTOLSTART'}) {
     print "ATTOLSTART specified as:$TARGETDIR/cmd/$Ini{'ATTOLSTART'}\n";
     print "sub attolStart() executed with $Command\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLSTART'}";
     $status = attolStart($Command,'CPP','*',$OTDNAME);
  }
  else
  {
     $status = SystemP($Command);
  }

 print &PerlTranslate("StartObjecTest.pl", "Generation failed.\n") if($status);
 return($status);
}

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

	EnterInScript();

	ClearLogFile( $LOGFILENAME2 );
	exit (StartObjecTest());
}

1;
# That's all Folks.
################################################################################
