#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                -*-  Start Unit Testing Node -*-                              #
#                                                                              #
#                                                                              #
#  Script for Unit Testing Node.                                               #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME2         : name of the log file to generate.                  #
#   $TARGETDIR           : directory of the Target Package                     #
#   $BINDIR              : directory of the studio/tracer binaries.            #
#   $ATTOLHOME           : attol studio intallation directory.                 #
#   $SCRIPTFILE          : filename of the test script should be generated     #
#   @TESTEDFILE          : filename should be tested                           #
#   @SIMULATEDFILES      : list of simulated filenames                         #
#   @INTEGRATEDFILES     : list of integrated filename                         #
#   @INCLDIRECTORIES     : list of includes directories                        #
#   @INCLUDES            : list of includes directories (Preferences)          #
#   @INCLUDE_DIR_LIST    : list of includes directories (Settings)             #
#    (the 2 above lists are merged into @COMPLETE_INCLUDE_DIR_LIST in Fcts.pl) #
#   $TESTEDCOMPONENT     : name of the tested function, for C only.            #
#   @GENERATIONCONDITION : list of -define                                     #
#                                                                              #
#   $OPTIONS	         : additionnal options for test script generation      #
#   @SERVICES	         : list of methods selected in the wizard	       #
#                                                                              #
################################################################################
#$Id: StartUnitTesting.pl,v 1.53 2007/09/05 17:41:42 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0] unless($main::TestRTCC); #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";

#----------------------------------------------------------------------------
## Start generation
sub StartUnitTesting {

print &PerlTranslate("StartUnitTesting.pl", "Test Script generation %1...\n", "%1 is the name of the Test Script", $SCRIPTFILE);

  my ($tested_dir, $tested_name, $tested_ext) = SplitFileName($TESTEDFILE);
  my $tested_lang = LanguageOf( $tested_ext,$TDP_LANGUAGE );
  my $files=" \"$TESTEDFILE\" \"$SCRIPTFILE\"";
  my $Command = $BINDIR.$PATH_SEPARATOR."attolstartC".$files;
  my $ada_mode;
  if ($tested_lang eq "PTU")
  {
    open ( LOG, ">>$LOGFILENAME2" ) or die &PerlTranslate("StartUnitTesting.pl", "Cannot open log file.\n");
    print  LOG "PTU $TESTEDFILE\n";
    close( LOG );
    print &PerlTranslate("StartUnitTesting.pl", "Use of existing Unit Testing script %1.\n", "%1 is the name of the Test Script", $TESTEDFILE);
    return(0);
  }
  elsif ($tested_lang eq "MDL")
  {
     # Execution of MATLAB to Unit Testing Translation
     $Command .= $BINDIR.$PATH_SEPARATOR."mdl2ptu \"-l$LOGFILENAME2\" \"$TESTEDFILE\"";
     $status = SystemP($Command);
     return($status);
  }
  elsif($TDP_LANGUAGE=~/^ADA/)
  {
     $ada_mode = 1;
     $Command = $BINDIR.$PATH_SEPARATOR."attolstartADA".$files;
     if ($TDP_LANGUAGE eq "ADA83_V34")
     {
		$Command = $BINDIR.$PATH_SEPARATOR."attolstartADA83".$files;
     }else{
		my $adalink="adalink.alk";
		$adalink = "\"$USER_ALK_FILE\"" if (($USER_ALK_FILE) && (-f $USER_ALK_FILE));
		$Command .= " -ADALIEN=$adalink";
     }
     my $status = BuildAdaLinkFile();
     if ( $status != 0 ) { return(1); }

     my $ana= "ada95.alk" ;
     $ana="ada83.alk" if ($TDP_LANGUAGE =~/^ADA83/);
     $Command .= " -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR.$ana."\"";
  }
  elsif ($TDP_LANGUAGE eq "C")
  {
      $Command .= " -STD_DEFINE=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."atus_c.def\"";
      $Command.=" -define=\"".join("\",\"",@GENERATION_CONDITION)."\"" if(@GENERATION_CONDITION);
	if(@COMPLETE_INCLUDE_DIR_LIST){
		my $opt=" -incl=\"".join("\",\"",@COMPLETE_INCLUDE_DIR_LIST)."\"" ;
		$opt=~s,\\,/,g;
		$Command.=$opt ;
	}
      my $probedir="$ATTOLHOME"."$PATH_SEPARATOR"."lib"."$PATH_SEPARATOR"."probe"."$PATH_SEPARATOR"."noprobe";
      if ($ATS_PROBE_CUSTOM) {
	      if (@COMPLETE_INCLUDE_DIR_LIST) { $Command .= "\,\"$probedir\""; }
	      else { $Command .= " -incl=\"$probedir\""; }
      }
      $Command .= " -service=\"$TESTEDCOMPONENT\"" if ($TESTEDCOMPONENT);
  }

  if ($LOGFILENAME2) 
  {
     $Command .= " -studio_log=\"$LOGFILENAME2\"";
  }




    # Update options (same as attolprepro)
    $Command .= " -RESTRICTION=".$Ini{'RESTR'} if ($Ini{'RESTR'});
    # Suppress old features
    delete $ENV{'ATTOLLAN'} ;
    delete $ENV{'ATTOLCINC'} ;
    delete $ENV{'ATTOLTMPK'} ;
    delete $ENV{'ATTOLCPPINC'} ;
    delete $ENV{'ATW_NO_USR_INCLUDE'} ;

    # Let's environment
    $ENV{'ATTOLHOME'} = $ATTOLHOME;
    $ENV{'ATUDIR'} = $ATTOLHOME . $PATH_SEPARATOR."lib";
    $ENV{'ATUTGT'} = "$TARGETDIR";

  # Settings
  $Command .= " ".$OPTIONS if ($OPTIONS);
  $Command .= " ".$Ini{'START_OPT'} if($Ini{'START_OPT'});

 # Update options (specific attolstart)
 $Command.=" -simulate=\"".join("\",\"",@SIMULATEDFILES)."\"" if(@SIMULATEDFILES);
 $Command.=" -integrate=\"".join("\",\"",@INTEGRATEDFILES)."\"" if(@INTEGRATEDFILES);

# Execution


  my $status;
  if($Ini{'ATTOLSTART'}) {
     print "ATTOLSTART specified as:$TARGETDIR/cmd/$Ini{'ATTOLSTART'}\n";
     print "sub attolStart() executed with $atu_command\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLSTART'}";
     $status = attolStart($Command,'C','*',$SCRIPTFILE);
  }
  else
  {
     $status = SystemP($Command);
  }

  $status=0 if($status==3 && $CONTONWARN);
  if ($status)
  {
     print &PerlTranslate("StartUnitTesting.pl", "Generation failed.\n");
  }
  elsif(scalar(@SERVICES)  && !($ada_mode))
  {
  	my $on=1;
  	open(OUT,">$SCRIPTFILE.tmp") or die &PerlTranslate("StartUnitTesting.pl", "Cannot open tmp file.\n");
  	open(IN,"$SCRIPTFILE") or die &PerlTranslate("StartUnitTesting.pl", "Cannot open ptu file.\n");
  	while(<IN>){
  		if(/^-- Declaration of the service (.*)$/){
  			my $svc=$1;
  			if(grep(/$svc/,@SERVICES)) {$on=1;}
  			else {$on=0;}
  		}
  		print OUT if($on);
  	}
  	close(OUT);close(IN);
  	rename("$SCRIPTFILE.tmp",$SCRIPTFILE);
  }
  return ($status);
}

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

	EnterInScript();

	ClearLogFile( $LOGFILENAME2 );
	exit (StartUnitTesting());
}

1;
# That's all Folks.
################################################################################
