#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  System Testing node -*-                          #
#                                                                              #
#                                                                              #
################################################################################
#$Id: SystemTest.pl,v 1.98 2007/10/23 11:40:09 rfrance Exp $
################################################################################

use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0];

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

# RTRT tools functions
require $SCRIPTDIR . "Tools.pl";

# Target-dependant functions
require $TARGETDIR . "/cmd/atl_exec.pl";

require $SCRIPTDIR . "Split.pl";

#print "SystemTest.pl\n";
$ADDRESS = hostinet() if($ADDRESS eq "");
################################################################################
# SystemTest deployment script (Execution and postprocessing) 
# Arguments:
#	None
# Return value: 0 if OK
# Required global variables:
#   @VT_DEP_CONF=(['VT','instance',delay,NbOcc,firstOcc,ip@,port,'tmp','cvisual6','cvisual6','entrypoint'],[...],...)
#   $BINDIR
#   $STUDIO_PORT
# Used subroutines:
#   ExportTsBinary
#   ImportTsBinary
#   CreateDirectory
################################################################################
sub generateTheSpvFile #($)
{
   my $spvName = shift;
   my $ERROR=1; #ON
   my $count=0;
   my %HostList={''};
   my %HostDir={''};
   my %HostBin={''};
   open SPV, ">$spvName" or die &PerlTranslate("SystemTest.pl", "Cannot create file %1\n", "%1 is the name of the file", $spvName);
	print SPV "---------------------------------- HOSTS IDENTIFICATION ---------------------------------\n";
	print SPV "HOST StudioHost IS $ADDRESS($PORT)\n";
	$HostList{$ADDRESS.$PORT}="StudioHost";
	foreach (@VT_DEP_CONF)
        {
		my $VTADDR=$VT_DEP_CONF[$count][5];
		   $VTADDR=$ADDRESS if($VTADDR =~ /^\s*$/);
		my $VT_PORT=$VT_DEP_CONF[$count][6];
		if (!$HostList{$VTADDR.$VT_PORT})
                {
                        $VTHOST="HostNumber$count";
                        $HostList{$VTADDR.$VT_PORT}=$VTHOST;
                        print SPV "HOST $VTHOST IS $VTADDR($VT_PORT)\n";
                }
		$count++;
	}
	print SPV "\n------------------------ SET the StudioHost to the right directory ----------------------\n";
	### change the agent curent dir to $INTDIR
	my $dir=$WORKING_DIR."/$INTDIR";
	if (-d $dir){
		$HostDir{'StudioHost'}=$dir;
		print SPV "SET   StudioHost : INTDIR = $dir\n";
		print SPV "CHDIR StudioHost : \$INTDIR\n";
		print SPV "SET SPV_DIR = $WORKING_DIR\n";
		print SPV "SET INTDIR = $INTDIR\n";
	}
	print SPV "\n------------------------------- SET the RENDEZVOUS MEMBERS ------------------------------\n";
	print SPV "\n";
	print SPV "TIMEOUT $ATS_RDV_TIMEOUT\n" if ($ATS_RDV_TIMEOUT);
        $count=0;
        foreach (@VT_RDV_CONF)
        {
                print SPV "MEMBERS $VT_RDV_CONF[$count][0] $VT_RDV_CONF[$count][1]\n";
                $count++;
        }
	print SPV "\n--------------- MAKE SURE DIRECTORIES EXIST ON REMOTE HOSTS AND SEND FILES --------------\n";
        $count=0;
	foreach (@VT_DEP_CONF)
	{
		my $BINARY=$VT_DEP_CONF[$count][0];
		my $VTADDR=$VT_DEP_CONF[$count][5];
		my $VT_PORT=$VT_DEP_CONF[$count][6];
		my $VT_WORKING_DIR=$VT_DEP_CONF[$count][7];
		my $LOCAL_DIR=$VT_DEP_CONF[$count][8];
		my $VTHOST=$HostList{$VTADDR.$VT_PORT};
		
		if($VTHOST ne "StudioHost")
		{
			if (!$VT_WORKING_DIR)
			{
				$VT_WORKING_DIR=".";
			}
			if($HostDir{$VTHOST} ne $VT_WORKING_DIR)
			{
	#			### MAKE SURE THE REMOTE DIRECTORY EXISTS
	#			CreateDirectory($VT_WORKING_DIR);
				my @subdir = split( /\//, $VT_WORKING_DIR );
	#			my $ATS_REMOTE_DIR_IS_RELATIVE=0;
				#is there a drive specified ?
				if (!($VT_WORKING_DIR =~/[a-zA-Z]:/) && !($VT_WORKING_DIR =~/^\//))
				{
	#				$ATS_REMOTE_DIR_IS_RELATIVE=1;
					print SPV "ERROR OFF\n" if($ERROR);
					$ERROR=0;
					print SPV "-- ATS_AGT_BASEPWD is the directory where the agent has been started\n";
					print SPV "CHDIR $VTHOST :".'$ATS_AGT_BASEPWD'."\n";
				}
				for $d (@subdir)
				{
					if(!$d)
					{# absolute path in unix format
						$d='/'.pop(@subdir)
					}
					if ($d=~/[a-zA-Z]:/)
					{
						$d.="\\";
					}

					print SPV "SET $VTHOST : DIR = $d\n";
					print SPV "ERROR OFF\n" if($ERROR);
					print SPV "MKDIR $VTHOST : ".'$DIR'."\n";
					print SPV "ERROR ON\n";
					$ERROR=1;
					print SPV "CHDIR $VTHOST : ".'$DIR'."\n";
				}
	#			if ($ATS_REMOTE_DIR_IS_RELATIVE)
	#                        {
	#                                $VT_WORKING_DIR='$ATS_AGT_BASEPWD'."/$VT_WORKING_DIR";
	#                        }
				$HostDir{$VTHOST}=$VT_WORKING_DIR;
			}

			### NEED TO EXPORT THIS BINARY->WRITE IT IN THE SPV SCRIPT

			if(!$HostBin{$VTHOST.$BINARY})
			{
	#			if ($VT_WORKING_DIR  ne ".")
	#			{
	#				print SPV "ERROR OFF\n" if($ERROR);
	#				$ERROR=0;
	#				print SPV "SET $VTHOST : REMOTE_DIR = $VT_WORKING_DIR\n";
	#				print SPV "CHDIR $VTHOST : ".'$REMOTE_DIR'."\n";
	#			}
				print SPV "ERROR ON\n" if(!$ERROR);
				$ERROR=1;
				print SPV "COPY \"$INTDIR$PATH_SEPARATOR$BINARY.$EXEEXT\" $VTHOST : $BINARY.$EXEEXT\n";
				$HostBin{$VTHOST.$BINARY}=1;
			}
		}
        	$count++;
	}
	print SPV "\n----------------------------------- START DEPLOYMENT ------------------------------------\n";
	$count=0;
	my $prevVTHOST;
	my $command="-- Multi-thread mode";
	foreach (@VT_DEP_CONF)
	{
		my $BINARY=$VT_DEP_CONF[$count][0];
		my $INSTANCE=$VT_DEP_CONF[$count][1];
		my $DELAY=$VT_DEP_CONF[$count][2];
		my $VTADDR=$VT_DEP_CONF[$count][5];
		my $VT_PORT=$VT_DEP_CONF[$count][6];
		my $VT_WORKING_DIR="$VT_DEP_CONF[$count][7]";
		my $LOCAL_DIR=$VT_DEP_CONF[$count][8];
		my $VTHOST=$HostList{$VTADDR.$VT_PORT};
		my $OccId;
		$INSTANCE="" if ($INSTANCE eq "<default>");
		### BUILD AND DUMP THE EXECUTION COMMAND IN THE HOST.SPV SCRIPT
		for ($OccId=$VT_DEP_CONF[$count][4];
			$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
			$OccId++)
		{
			if (!$VT_WORKING_DIR)
		        {
		                $VT_WORKING_DIR=".";
			}
			if(($VTHOST ne 'StudioHost') && ($HostDir{$VTHOST} ne $VT_WORKING_DIR))
			{
				print SPV "ERROR OFF\n" if($ERROR);
				print SPV "SET $VTHOST : EXEC_DIR = $VT_WORKING_DIR\n";
				print SPV "CHDIR $VTHOST : ".'$EXEC_DIR'."\n";
				print SPV "ERROR ON\n";
				$ERROR=1;
				$HostDir{$VTHOST} = $VT_WORKING_DIR;
			}
			print SPV "PAUSE $DELAY\n" if($DELAY);
			if($ATS_THREAD){
				if($prevVTHOST ne $VTHOST){
					print SPV "$command\n";
					$command="EXECUTE Id$count$OccId IS $VTHOST : $BINARY.exe";
				}
				$command.=" -THREAD=$BINARY\_$count$OccId.rio,$INSTANCE,$OccId";
			} else {
				$command="EXECUTE Id$count$OccId IS $VTHOST : $BINARY.exe";
				if ($INSTANCE ne "")
				{
					$command.=" -INSTANCE=".$INSTANCE;
				}
				my $rio_name=$VT_DEP_CONF[0][10];
				   $rio_name="$BINARY\_$count$OccId.rio" if($rio_name eq '');
				print SPV "PRINTLN \"execute for RIO $BINARY\_$count$OccId.rio on $VTHOST\"\n";
				print SPV "$command -OCCID=$OccId -RIO=$rio_name\n";
			}
		}
		$prevVTHOST=$VTHOST;
		$count++;
	}
	print SPV "$command\n"if($ATS_THREAD);
	
	###	MAKE SURE THE SUPERVISOR DOES NOT LEAVE BEFORE ALL PROCESS HAS FINISED
	$count=0;
	$prevVTHOST="";
	foreach (@VT_DEP_CONF)
        {
		my $VTADDR=$VT_DEP_CONF[$count][5];
		my $VT_PORT=$VT_DEP_CONF[$count][6];
		my $VTHOST=$HostList{$VTADDR.$VT_PORT};
		my $OccId;
		
		if(!$ATS_KILL_VT && $ERROR && (@VT_RDV_CONF)) {
			print SPV "ERROR OFF -- do  not stop processes on RDV timeout\n" ;
			$ERROR=0;
		}
		for ($OccId=$VT_DEP_CONF[$count][4];
                        $OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
                        $OccId++)
                {
			if($prevVTHOST ne $VTHOST){
				print SPV "WHILE !ENDOF (Id$count$OccId)\n";
				print SPV "PAUSE 1\n";
				print SPV "END WHILE\n";
			}
		}
		$prevVTHOST=$VTHOST if($ATS_THREAD);
		$count++;
	}
	print SPV "\n--------------------------- GET RESULT(S) FROM REMOTE HOST(S) ---------------------------\n";
	$count=0;
	foreach (@VT_DEP_CONF)
	{
		my $BINARY=$VT_DEP_CONF[$count][0];
		my $INSTANCE=$VT_DEP_CONF[$count][1];
		my $DELAY=$VT_DEP_CONF[$count][2];
		my $VTADDR=$VT_DEP_CONF[$count][5];
		my $VT_PORT=$VT_DEP_CONF[$count][6];
		my $VT_WORKING_DIR="$VT_DEP_CONF[$count][7]";
		my $LOCAL_DIR=$VT_DEP_CONF[$count][8];
		my $VTHOST=$HostList{$VTADDR.$VT_PORT};
		my $OccId;
		if($VTHOST ne "StudioHost")
		{
			if($HostDir{$VTHOST} ne $VT_WORKING_DIR)
			{
				print SPV "ERROR ON\n" unless($ERROR);
				print SPV "SET $VTHOST : REMOTE_DIR = $VT_WORKING_DIR\n";
				print SPV "ERROR OFF\n";
				$ERROR=0;
				print SPV "CHDIR $VTHOST : ".'$REMOTE_DIR'."\n";
				$HostDir{$VTHOST} = $VT_WORKING_DIR;
			}
			### BUILD AND DUMP THE EXECUTION COMMAND IN THE HOST.SPV SCRIPT
			for ($OccId=$VT_DEP_CONF[$count][4];
				$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
				$OccId++)
			{
				my $rio_name=$VT_DEP_CONF[0][10];
				   $rio_name="$BINARY\_$count$OccId.rio" if($rio_name eq '');
				### SAVE THE FILENAMES TO GET DURING THE POSTPROCESSING
				print SPV "ERROR OFF\n" if($ERROR);
				print SPV "DELETE .$PATH_SEPARATOR$INTDIR$PATH_SEPARATOR$rio_name\n";
				print SPV "ERROR ON\n";
				$ERROR=1;
				print SPV "COPY $VTHOST : $rio_name .$PATH_SEPARATOR$INTDIR$PATH_SEPARATOR$rio_name\n";
			}
		}
		$count++;
	}

   print SPV "\n--------------------------- Inform TestRT of the rio produced ---------------------------\n";
   my $LogFile=$LOGFILENAME;
      $LogFile=TestRTUtil::LocaleStringReplace($LogFile, "\\\\", "\\\\");
   my $GeneratedFile=$spvName;
      $GeneratedFile=~ s/spv$/results/i;
   print SPV "COPY \"$GeneratedFile\" StudioHost:\"$LogFile\"\n";
   print SPV "-- If you change any rio file name, don't forget to change the name in the \"$GeneratedFile\" as well";
   close SPV;
   open RES, ">$GeneratedFile" or die &PerlTranslate("SystemTest.pl", "Cannot create file %1\n", "%1 is the name of the file", $GeneratedFile);
	$count=0;
	foreach (@VT_DEP_CONF)
	{
		my $BINARY=$VT_DEP_CONF[$count][0];
#		my $VT_WORKING_DIR="$VT_DEP_CONF[$count][7]";
#		my $LOCAL_DIR=$VT_DEP_CONF[$count][8];
		for ($OccId=$VT_DEP_CONF[$count][4];
			$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
			$OccId++)
		{
			my $rio_name=$VT_DEP_CONF[0][10];
			   $rio_name="$INTDIR/$BINARY\_$count$OccId.rio" if($rio_name eq '');
			print RES "RIO $rio_name\n";
		}
		$count++;
	}
   if(!$ATS_GENERATE_DEP)
   {
	print RES "DUM \"$GeneratedFile\"\n";
	print RES "DUM \"$spvName\"\n";
   }
   close RES;
   if($count ==0) {
	print &PerlTranslate("SystemTest.pl", "\nTestRT-E-DPLOYEMTY, The Deployment configuration is not valid.\n","");
	exit(1);
   }
}
################################################################################
#	Basically we produce and execute one or more spv files which are:    ###
#	-$ENV_SPV	: set some enironment variables on the remote host   ###
#	-$COPY_FILE_SPC : send virtual tester to a remote host		     ###
#	-$EXEC_SPV 	: execute the defined deployment		     ###
#	-$IMPORT_SPV	: get the execution result from remote host	     ###
################################################################################	

CheckIntDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
$ENV{'ATS_DIR'}=$BINDIR;

############ RF
if ($ATS_GENERATE_DEP) {
	generateTheSpvFile($ATS_GENERATED_SPV);
}
elsif($STAGE =~ /e/)
{
    
    if($ATS_NO_SPV)
    {
	if($Ini{'ATSEXEC'}){
		print &PerlTranslate("SystemTest.pl","ATSEXEC specified as:%1\n","%1 is the file","$TARGETDIR/cmd/$Ini{'ATSEXEC'}");
		do "$TARGETDIR/cmd/$Ini{'ATSEXEC'}";
	}else{
	    	my $count=0;my $OccId=0;my $binary;my $instance;
		my $command;
		foreach (@VT_DEP_CONF)
		{
		    $binary=$VT_DEP_CONF[$count][0];
		    $instance=$VT_DEP_CONF[$count][1];
		    $instance="" if ($instance eq "<default>");
		    my $riofile =$VT_DEP_CONF[0][10];
		    if($ATS_THREAD){
			### BUILD AND DUMP THE EXECUTION COMMAND IN THE HOST.SPV SCRIPT
			for ($OccId=$VT_DEP_CONF[$count][4];
				$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
				$OccId++)
			{
		    	   $riofile ="$INTDIR/$binary\_$count$OccId.rio" if($VT_DEP_CONF[0][10] eq '');
			   $command.=" -THREAD=$riofile,$instance,$OccId";
			}
		     } else {
		        if($count) { 
		        	print &PerlTranslate("SystemTest.pl", "WARNING: only one process can be started. So INSTANCE  $instance is not executed.\n","$instance is the instance name"); 
		        } else {
		    		$riofile ="$INTDIR/$binary\_$count$OccId.rio" if($VT_DEP_CONF[0][10] eq '');
				$command.=" -INSTANCE=".$instance if ($instance ne "");
				$command.=" -RIO=$riofile";
			}
		     }
		$count++;
		}
		print &PerlTranslate("SystemTest.pl", "WARNING: dynamic traces are not compatible with this execution mode. setting not taken into account!\n", "") if($ATS_DYN_FOLLOW);

		print &PerlTranslate("SystemTest.pl", "Executing %1...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $binary);
		$status = AtlExec ("$INTDIR$PATH_SEPARATOR$binary.$EXEEXT", $riofile, $ARGUMENTS.$command);
		if ($status != 0 )
		{
		    print &PerlTranslate("SystemTest.pl", "Executable terminated with error code %1.\n", "%1 is the error code number", $status);
		}
		open RES, ">$LOGFILENAME" or die &PerlTranslate("SystemTest.pl", "Cannot create file %1\n", "%1 is the name of the file", $LOGFILENAME);
		$count=0;
		foreach (@VT_DEP_CONF)
		{
			my $binary=$VT_DEP_CONF[$count][0];
			for ($OccId=$VT_DEP_CONF[$count][4];
				$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
				$OccId++)
			{
				$riofile=$VT_DEP_CONF[0][10];
				$riofile="$INTDIR/$binary\_$count$OccId.rio" if($VT_DEP_CONF[0][10] eq '');
				print RES "RIO $riofile\n";
				print     "RIO $riofile\n";
			}
			$count++;
		}
		close RES;
	}
    }
    else
    {
    	my $EXEC_SPV="exec.spv";
    	my $status;
	generateTheSpvFile($EXEC_SPV);
	launchLocalAgent();
	print &PerlTranslate("SystemTest.pl", "\nStarting execution using %1 script...\n","%1 is the spv script file",$EXEC_SPV);
	if (($ATS_DYN_FOLLOW)&&("x$TDF_CHANNEL" ne "x"))
	{
	   $status=SystemP("$BINDIR/atsspv $EXEC_SPV -STUDIO_MACH=$ADDRESS -STUDIO_PORT=$STUDIO_PORT -STUDIO_CHNUM=$TDF_CHANNEL");
	}
	else
	{
	   $status=SystemP("$BINDIR/atsspv $EXEC_SPV");
	}
        if ($status) {&PerlTranslate("SystemTest.pl", "Supervisor execution status $status\n",""); }
	unlink($EXEC_SPV);
	$EXEC_SPV=~s/\.spv$//;
	unlink($EXEC_SPV.".lis");
	unlink($EXEC_SPV.".results");
    }
}
#################################################################################
########       POSTPROCESSING OF AN AUTOMATICALLY GENERATED DEPLOYMENT   ########
#################################################################################

if ( $STAGE =~ /r/ )
{
    if(!$ATS_POSTPRO_ONLY)
    {
   	if(!$ATS_GENERATE_DEP)
   	{
		my %BinList={''};
		my @tdcList=();
		my @rioList=();

		$count=0;
		foreach (@VT_DEP_CONF)
		{
			my $BINARY=$VT_DEP_CONF[$count][0];
			my $VTADDR=$VT_DEP_CONF[$count][5];
			my $VT_PORT=$VT_DEP_CONF[$count][6];
			my $VT_WORKING_DIR="$VT_DEP_CONF[$count][7]";
			my $LOCAL_DIR=$VT_DEP_CONF[$count][8];
			for ($OccId=$VT_DEP_CONF[$count][4];
				$OccId<$VT_DEP_CONF[$count][4]+$VT_DEP_CONF[$count][3];
				$OccId++)
			{
				my $rio_name=$VT_DEP_CONF[0][10];
				   $rio_name="$INTDIR/$BINARY\_$count$OccId.rio" if($rio_name eq '');
				   $rio_name="$WORKING_DIR/".$rio_name;
				push(@rioList,$rio_name);
			}
			if(!$BinList{$BINARY}) 
			{
				$BinList{$BINARY}=1;
				push(@tdcList,"$WORKING_DIR/$INTDIR/$BINARY.tdc");
			}
			$count++
		}
		atsmerge(\@tdcList,\@rioList);
	}
    }
    else
    {
	### POSTPROCESSING OF A USER-EDITED DEPLOYMENT	#########
	atsmerge(\@TDC_LIST,\@RIO_LIST);
    }
}

sub atsmerge #(\@\@)
{
my ($tdcs,$rios)=@_;

	my $FileList;
	my $res;
	$ATPMODE=0;
	$ATQMODE=0;
	$ATCMODE=0;
	$ATTMODE=0;
	foreach $elem (@$tdcs)
	{
	    if($elem ne '')
	    {
	    	if(-r $elem)
	    	{
		    	$res|=1;
			$FileList.="\"$elem\" ";
		}
	    }
	}
	foreach $elem (@$rios)
	{
	    if($elem ne '')
	    {
	    	if(-r $elem)
	    	{
		    	$res|=2;
			$FileList.="\"$elem\" ";
			ParseAtsRioFile($elem);
		}
	    }
	}
	if($res == 3)
	{
		my $Options="";
		$Options.="-RA=$ATS_RA" if ($ATS_RA && ($ATS_RA ne "NONE"));
		$Options.=" -VA=$ATS_VAL_DISP" if ($ATS_VAL_DISP && ($ATS_VAL_DISP ne "NONE"));

		print &PerlTranslate("SystemTest.pl", "\nPostprocessing results...\n");
		print "*****************************************************\n"; # NO TRANSLATION
		my $status=SystemP("$BINDIR/atsmerge $FileList $Options -RDD=\"$NAME.rdd\"");
		if ($status>3)
		{
			die &PerlTranslate("SystemTest.pl", "Failed.\n");
		}
		open ( LOG, ">$LOGFILENAME" ) or die &PerlTranslate("SystemTest.pl", "Cannot open log file.\n");
		print  LOG "XRD $NAME.xrd\n";
		print  LOG "TSF $NAME.tsf\n";
		print  LOG "TDF $NAME.tdf\n";
		foreach $elem (@$rios)
		{
		    if($elem ne '')
		    {
			print  LOG "DUM $elem\n";
			print  LOG "DUM $elem.bak\n";
			$elem=~s/\.rio$/.rod/;
			print  LOG "DUM $elem\n";
		    }
		}
		close( LOG );
	}
	else
	{
		print &PerlTranslate("SystemTest.pl", "rio or tdc files does not exist\n");
	}
}
