#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                  -*-  Batch TDP Library compilation -*-                      #
#                                                                              #
#                                                                              #
#  Script used by attolcc (batch compilation launcher)                         #
#                                                                              #
#                                                                              #
# Required environment variables:                                              #
#   TESTRTDIR, ATLTGT                                                          #
#                                                                              #
################################################################################
#$Id: TDPBatchCC.pl,v 1.30 2007/07/23 18:41:59 rfrance Exp $
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

$SCRIPTDIR = $ENV{'TESTRTDIR'} . "/lib/scripts/";
$TARGETDIR = $ENV{'ATLTGT'};
$INTDIR = $ENV{'ATTOLOBJ'};
if ( $INTDIR eq "" ) {
  $INTDIR = ".";
}

# This script depends on the followings
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Products.pl";

#----------------------------------------------------------------------------

# Default settings
require $SCRIPTDIR . "BatchCCDefaults.pl";

#----------------------------------------------------------------------------

# Caller-known settings
require $ARGV[0];

#----------------------------------------------------------------------------

if ( $SETTINGS_OVERLOAD ne "" ) {
  if ( -f "$SETTINGS_OVERLOAD" ) {
    # The way to overload the above defaults
    require "$SETTINGS_OVERLOAD";
  } else {
    print &PerlTranslate("TDPBatchCC.pl", "Target Deployment Port Compilation - Warning: Perl file %1 to overload the default settings of Target Deployment Port library compilation does not exist\n", "%1 is the name of the file", $SETTINGS_OVERLOAD);
  }
}

#----------------------------------------------------------------------------

# Let's start (inifile read, etc.)
EnterInScript();

if ( $OBJEXT eq "" ) {
  $OBJEXT=$Ini{'OBJEXT'};
}

#----------------------------------------------------------------------------

# If no overload for them, $CPPFLAGS, $CFLAGS, @COMPLETE_INCLUDE_DIR_LIST and
# @GENERATION_CONDITION come from ini file:
if ($CPPFLAGS eq "") {
  $CPPFLAGS = $Ini{'DEFAULT_CPPFLAGS'};
}
if ($CFLAGS eq "") {
  $CFLAGS = $Ini{'DEFAULT_CFLAGS'};
}
if ($INCLUDE_PATHS ne "") {
  @COMPLETE_INCLUDE_DIR_LIST = split( /,/, $INCLUDE_PATHS);
}
if (scalar(@COMPLETE_INCLUDE_DIR_LIST) <= 0) {
  if ($Ini{'DEFAULT_INCLUDE_PATHS'} ne "") {
    @COMPLETE_INCLUDE_DIR_LIST = split( /,/, $Ini{'DEFAULT_INCLUDE_PATHS'} );
  }
}
if ($DEFINES ne "") {
  @GENERATION_CONDITION = split( /,/, $DEFINES);
}
if (scalar(@GENERATION_CONDITION) <= 0) {
  if ($Ini{'DEFAULT_DEFINES'} ne "") {
    @GENERATION_CONDITION = split( /,/, $Ini{'DEFAULT_DEFINES'} );
  }
}

#----------------------------------------------------------------------------

# Some checks
$TDP_LANGUAGE = uc($Ini{'LANGUAGE'});

if ( $ENV{'ATL_TMP_DIR'} ) {
  $LOGFILENAME = $ENV{'ATL_TMP_DIR'} . "/unused.log";
} else {
  $LOGFILENAME = "unused.log";
}
ClearLogFile($LOGFILENAME);

if ( $TDP_LANGUAGE=~/^ADA/ )
{
  print &PerlTranslate("TDPBatchCC.pl", "Error: the environment variable ATLTGT (%1) is currently set to an Ada Target Deployment Port\n", "%1 is the current value of ATLTGT environment variable", $ENV{'ATLTGT'});
  exit 1;
}

if ( ($TDP_LANGUAGE ne "C") && ($TDP_LANGUAGE ne "CPP") )
{
  print &PerlTranslate("TDPBatchCC.pl", "Error: ATLTGT (%1) is currently set to an unknown language (%2)\n", "%1 is the current value of ATLTGT environment variable and %2 is the language value", $ENV{'ATLTGT'}, $TDP_LANGUAGE);
  exit 1;
}

# Something to get the well known and unique out file
#$ATL_TRACES_FILE = $ATL_OUT_FILE if ($ATL_TRACES_FILE eq "");

#----------------------------------------------------------------------------

# Things in order to debug
if ( $ARGV[1] eq "--force" )
{
  # Make sure to recompile TDP library
  unlink ( GetTargetPackageLib($OBJEXT) );
}

#----------------------------------------------------------------------------

# Let's do it
if (MakeTargetPackageObject()) {
  # Some errors while compiling TDP library
  exit 1;
}

if ( -f $LOGFILENAME ) {
  unlink ($LOGFILENAME);
}

0;
# That's all Folks.
################################################################################
