#! /usr/bin/perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003 - 2004 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################



use strict;
use IO::File;
use File::stat;
use File::Basename;
use File::Copy;

# usage
# TestRT_Log2TestManager <project_path> <workspace_name> <command_file> <log_path>
# or
# TestRT_Log2TestManager <project_path> <command_file> <log_path>

my $count = scalar(@ARGV);

my $project_path;
my $workspace_name;
my $command_file;
my $log_path;

if ( scalar(@ARGV) eq 4 )
{
  ($project_path, $workspace_name, $command_file, $log_path) = @ARGV;
}
else
{
  ($project_path, $command_file, $log_path) = @ARGV;
}

# force the log to testmanager.log to avoid clearcase issue
my ($tmpPath,$tmpFileName) = $command_file =~ m/^(.*)\\([^\\]+)$/;
$command_file =  $tmpPath . "\\" . "testmanager.log";

my $project_name;
if ( $project_path =~ m/^.*\\([^\\]+)$/ ) {  $project_name = $1; }
else { die "Couldn't parse project_path: $project_path\n"; }

my $log_project = $log_path . "\\TestRT";
my $log_src_path = $log_project . "\\" . "src";
my $log_tests_path = $log_project . "\\" .  "tests";

my $log_workspace_path;
if ( $workspace_name ) { $log_workspace_path = $log_project . "\\" . $workspace_name; }
else { $log_workspace_path = $log_project . "\\" . $project_name; }

my $COPY_ACTION = "copy";
my $COPY_LINKS_ACTION = "copy, copy links";
my $COPY_LINKS_FIX_ACTION = "copy, copy links, fix linked paths";
my %file_actions = (
  "tdf" => $COPY_ACTION,
  "tio" => $COPY_ACTION,
  "tpf" => $COPY_ACTION,
  "tqf" => $COPY_ACTION,
  "xrd" => $COPY_LINKS_FIX_ACTION,
  "fdc" => $COPY_ACTION,
  "tsf" => $COPY_LINKS_FIX_ACTION
  );

&SetupLogDir();

&ProcessCommandFile();

sub ProcessCommandFile 
{
  my ($command, $filename, $extension, $dir, $file, $action, $project_name);
  my @command;
  
  # Open the file for read.
  # Read the file
  # Close the file
  open( COMMAND_FILE, "< $command_file" )
    or die "Couldn't open $command_file for read!";
  @command = <COMMAND_FILE>;
  close( COMMAND_FILE );
  
  foreach $filename (@command)
  {
    # Process based on file type.
    chomp($filename);
#    if ($filename =~ m/.*\\([^\"]*)[\"?]*$/ )
#    {
#      if ( $1 =~ m/(.*)\.(.*)$/ )
#      {
#        $file = $1;
#        $extension = $2;
#      }
#    }
#    else
#    {
#      die "Couldn't parse entry from commandfile: $command_file\n$filename";
#    }
    $filename =~ s/^\"?(.*)[\"?]$/$1/; # strip quotes, if present
    ($file, $dir, $extension) = fileparse($filename, qr/\.[^.]*/);
    $extension =~ s/^\.([^\"]*)$/$1/; #strip . and " from extension

    print "Extension = $extension\n";
    
    # Now process this file.
    $action = $file_actions{$extension};
    print "File $file, action = $action\n";
    if ( $action eq $COPY_ACTION )
    {
      &CopyFile($filename, $log_workspace_path);
    }
    elsif ( $action eq $COPY_LINKS_FIX_ACTION )
    {
      &CopyFileAndLinkedFilesAndFixPaths($filename, $log_workspace_path);
    }
    else
    {
      die "Unknown action($action) specified for file $filename\n";
    }
  }
}

sub SetupLogDir
{
  mkdir $log_path, 0777;
  # Create project dir in log dir
  mkdir $log_project, 0777;
  # Create workspace dir in project dir
  mkdir $log_workspace_path, 0777;
  # Create src and test dirs in project dir
  mkdir $log_src_path, 0777;
  mkdir $log_tests_path, 0777;
}

sub CopyFile #($;$)
{
  my $filename = shift;
  my $target_dir = shift;
  my $file = basename($filename);
  my $out_filename = $target_dir . "\\" . $file;
  
  open(IN_FILE, "< $filename") or return; # die "Couldn't open log file $filename\n";
  open(OUT_FILE, "> $out_filename")or return; # or die "Couldn't open log file $out_filename\n";
  select(OUT_FILE);

  while( <IN_FILE> )
  {
    print OUT_FILE $_;
  }
  select(STDOUT);
  
  close(IN_FILE);
  close(OUT_FILE);
}

sub CopyFileAndLinkedFilesAndFixPaths
{
  my $filename = shift;
  my $target_dir = shift;
  my $file = basename($filename);
  my $out_filename = $target_dir . "\\" . $file;
  my ($in_path, $src_file, $end_string, $new_string, $line_num, $out_path);
  my ($before_string, $after_string);

  my $src_dir = $log_src_path;
  # Replace all of the single \'s in the source dir with \\'s to match the
  # file format of the .TSF file.
  while ( $src_dir =~ s/([^\\])\\([^\\])/$1\\\\$2/ ) {}
  $src_dir .= "\\\\";

  open(IN_FILE1, "< $filename") or die "Couldn't open log file $filename\n";
  open(OUT_FILE1, "> $out_filename") or die "Couldn't open log file $out_filename\n";
  select(OUT_FILE1);

  while( <IN_FILE1> )
  {
    # i.e  source("C:\\PROGRAM FILES\\RATIONAL\\TESTREALTIME\\EXAMPLES\\BASESTATION\\SRC\\","UMTSCODE.C",1006015088);
    if ( $_ =~ m/source\(\"(.*)\"\,\"(.*)\"(.*)/ )
    {
      # This is source line from a tsf file -- we need to copy the referenced
      # file, and change the path in the copied TSF file to new copy of the
      # referenced file.
      $in_path = $1;
      $src_file = $2;
      $end_string = $3;
      
      #Replace the double \\'s with \s.
      while ( $in_path =~ s/\\\\/\\/ ) {}
      
      select(STDOUT);
      #Copy the file to the log src dir.
      CopyFile($in_path . "\\" . $src_file, $log_src_path);
      select(OUT_FILE1);
      
      $new_string = " source(\"$src_dir\",\"$src_file\"$end_string\n";
      print OUT_FILE1 $new_string;
    }
    # i.e. <LINK TARGET="C:\PROGRA~1\Rational\TESTRE~1\examples\BASEST~1\tests\UmtsCode2.ptu line 25">
    elsif ( $_ =~ m/(.*)<LINK TARGET=\"(.*) line (\d+)\">(.*)/ )
    {
      $before_string = $1;
      $in_path = $2;
      $line_num = $3;
      $after_string = $4;

      select(STDOUT);
      #Copy the file to the log tests dir.
      CopyFile($in_path, $log_tests_path);
      select(OUT_FILE1);
      
      $src_file = basename($in_path);
      
      $out_path = $log_tests_path . "\\" . $src_file;
      $new_string = "$before_string<LINK TARGET=\"$out_path line $line_num\">$after_string\n";
      print OUT_FILE1 $new_string;
    }
    # i.e. SO "C:\\PROGRAM FILES\\RATIONAL\\TESTREALTIME\\EXAMPLES\\BASESTATION_C\\SRC\\UMTSCODE.C" 1033134702 "txt"
    elsif ( $_ =~ m/SO \"(.*)\" (.*)/ )
    {
      # This is source line from a new tsf file -- we need to copy the referenced
      # file, and change the path in the copied TSF file to new copy of the
      # referenced file.
      $in_path = $1;
      $end_string = $2;
      $src_file = basename($in_path);

      #Replace the double \\'s with \s.
      while ( $in_path =~ s/\\\\/\\/ ) {}

      select(STDOUT);
      #Copy the file to the log src dir.
      CopyFile($in_path, $log_src_path);
      select(OUT_FILE1);

      $new_string = "SO \"$src_dir$src_file\" $end_string\n";
      print OUT_FILE1 $new_string;
    }
    else
    {
      print OUT_FILE1 $_;
    }
  }
  select(STDOUT);

  close(IN_FILE);
  close(OUT_FILE);
}
