#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                          -*-  Test Script node -*-                           #
#                                                                              #
#                                                                              #
################################################################################
#$Id: TestScript.pl,v 1.239 2007/10/23 11:40:08 rfrance Exp $
################################################################################


use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0] unless($main::TestRTCC);

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";

# RTRT tools functions
require $SCRIPTDIR . "Tools.pl";

# Target-dependant functions
die &PerlTranslate("TestScript.pl", "Unknown TDP... check configuration\n") unless (-d $TARGETDIR);
require $TARGETDIR . "/cmd/atl_cc.pl";
require $TARGETDIR . "/cmd/atl_link.pl";

require $SCRIPTDIR . "Products.pl";

my $atu_rename_file = "stubs_rename.opt";

################################################################################
# RTRT Unit Testing preprocessor execution
# Arguments:
#    $script : PTU test script file
#    $source : source file to be generated
# Return value: attolpreproc return value
#    $BINDIR
#    $LOGFILENAME
################################################################################
sub atu_prepro #($$)
{
  my ($script,$atu_out) = @_;

  # Suppress old features
  delete $ENV{'ATTOLLAN'} ;
  delete $ENV{'ATTOLCINC'} ;
  delete $ENV{'ATTOLTMPK'} ;
  delete $ENV{'ATTOLCPPINC'} ;
  delete $ENV{'ATW_NO_USR_INCLUDE'} ;

  # Let's environment
  $ENV{'ATTOLHOME'} = $ATTOLHOME;
  $ENV{'ATUDIR'} = $ATTOLHOME . $PATH_SEPARATOR."lib";
  $ENV{'ATUTGT'} = "$TARGETDIR";

  # Defaults options
  my $atu_prepro = "attolpreproC";
  my $atu_options = " ";
  my $ada_mode = 0;
  
  if ($TDP_LANGUAGE=~/^ADA/)
  {
    $ada_mode = 1;
     $atu_prepro = "attolpreproADA";
  }

  if ($LOGFILENAME)
  {
        $atu_options .= " -studio_log=\"$LOGFILENAME\"";
  }

  # Update options (same as attolstart)

  if ($Ini{'RESTR'})
  {
        $atu_options .= " -RESTRICTION=".$Ini{'RESTR'};
  }

  if ($TDP_LANGUAGE eq "C")
  {
     $atu_prepro = "attolpreproC";
     $atu_options .= " -STD_DEFINE=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."atus_c.def\"";
     unlink ($atu_rename_file);
     $atu_options .= " -RENAME=$atu_rename_file" if($ATU_RENAME_FUNC);
  }

  if ($TDP_LANGUAGE eq "CPP")
  {
     $atu_prepro = "attolpreproCPP";
     $atu_options .= " -STD_DEFINE=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."atus_cpp.def\"";
  }

  if ($ada_mode == 1)
    {
      if (! $PACKAGES_TESTED eq "") { $atu_options .= " -WITHSEPARATE";
      }
      my $status = BuildAdaLinkFile();
      if ( $status != 0 ) { exit(1); }     
    }

  if ($TDP_LANGUAGE eq "ADA83_V34")
  {
     $atu_prepro = "attolpreproADA83";
     $atu_options .= " -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."ada83.alk\"";
  }

  if ($TDP_LANGUAGE eq "ADA83")
  {
     $atu_options .= " -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."ada83.alk\" -ADALIEN=";
     if (($USER_ALK_FILE) && (-f $USER_ALK_FILE))
      {   
          $atu_options .= "\"" . $USER_ALK_FILE . "\"";
      }
      else
      {   
          $atu_options .= "adalink.alk";
      }
  }

  if (($TDP_LANGUAGE eq "ADA") || ($TDP_LANGUAGE eq "ADA95"))
  {
     $atu_options .= " -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."ada95.alk\" -ADALIEN=";
     if (($USER_ALK_FILE) && (-f $USER_ALK_FILE))
      {   
          $atu_options .= "\"" . $USER_ALK_FILE . "\"";
      }
      else
      {   
          $atu_options .= "adalink.alk";
      }
  }

  if ($ada_mode == 0)
    {
        # Add $INTDIR to include dir path, it may be necessary if
        # the user has included a C header file generated by csv2header.pl
    	push(@COMPLETE_INCLUDE_DIR_LIST,$INTDIR);
    }
  
  if(@COMPLETE_INCLUDE_DIR_LIST) {
	my $opt = " -incl=\"".join("\",\"",@COMPLETE_INCLUDE_DIR_LIST )."\"";
 	$opt=~ s,([\\\/])",$1.",g;
 	$atu_options .=$opt;
     }

  if (@VT_FAM_SELECTION) {
    $atu_options .=" -family=".join(",",@VT_FAM_SELECTION);
  }

  my $aux_srv = 0;
  my $aux_tst = 0;
  my $tst_option = "";
  my $srv_option = "";
  foreach $arg (@VT_ITEM_SELECTION) {
    my ($srv, $tst) =  split /\./, $arg, 2;
    if ($srv) {
      $aux_srv = $aux_srv +1;
      if ($aux_srv==1) { $srv_option = " -service=$srv"; }
      elsif($aux_tst==0) { $srv_option .= "\,$srv"; }
    }
    if ($tst) {
      $aux_tst = $aux_tst +1;
      if ($aux_tst==1) { $tst_option = " -test=$tst"; }
      else { $tst_option .= "\,$tst"; }
    }
  }
  $atu_options .= $srv_option;
  $atu_options .= $tst_option;

  if (@GENERATION_CONDITION) {
    $atu_options .=" -define=\"".join("\",\"",@GENERATION_CONDITION )."\"";
  }

  # Update options (specific attolprepro)

  # Settings

  if (!$ATU_SIMULATION)
  {
        $atu_options .= " -NOSIMULATION";
  }

  if ($ATU_IS_OPTION)
  {
        $atu_options .= " " . $ATU_OPTIONS;
  }

  # Init file

  if (($TDP_LANGUAGE eq "CPP") && ($Ini{'CLASS'}))
  {
        $atu_options .= " -CLASS=".$Ini{'CLASS'};
  }

  if ($Ini{'NOPATH'})
  {
        $atu_options .= " -NOPATH";
  }

  if ($ATU_REPORT_OUTLAY == 1)
  {
          $atu_options .= " -EXPAND";
  }
  elsif ($ATU_REPORT_OUTLAY == 2)
  {
          $atu_options .= " -COMPRESS";
  }

  # By default

  $atu_options .= " -FAST $Ini{'PREPRO_OPT'}";

  # Execution
  my $atu_command = $BINDIR . $PATH_SEPARATOR . $atu_prepro . " \"$script\"      \"$atu_out\"    $atu_options";
  my $status;
  if($Ini{'ATTOLPREPRO'}) {
     print "ATTOLPREPRO specified as:$TARGETDIR/cmd/$Ini{'ATTOLPREPRO'}\n";
     print "sub attolPrepro() executed with $atu_command\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLPREPRO'}";
     $status = attolPrepro($atu_command,'C',$script,$atu_out);
  }
  else
  {
     $status = SystemP($atu_command);
  }

  return $status;
}

################################################################################
# Making of a "PTU" test script file
# Arguments:
#   $ptu     : PTU test script file
# Return value: 0 if OK
# Used subroutines:
#   atu_prepro
################################################################################
sub ptu_make #($$)
{
  my ($ptu,$src) = @_; #$src unused

  #
  # Obsolete
  #
  #my ($ptu_dir, $ptu_name, $ptu_ext) = SplitFileName ($ptu);
  #my ($src_dir, $src_name, $src_ext) = SplitFileName( $src );
  my $atu_has_rename = 0;
  my $atu_out = "" ;
  my $status;

  # Out Files
  #$atu_out = "" ;
  if ($INTDIR)
  {
      $atu_out .= $INTDIR.$PATH_SEPARATOR;
  }
  $atu_out .= $ATU_INTERMEDIATE_FILE;
  $obj = $atu_out.".".$OBJEXT;
  $src = $atu_out.".".$SRCEXT;

  if ($STAGE =~ /p/ )
  {
    print &PerlTranslate("TestScript.pl", "Preprocessing PTU test script file...\n");
    
    $status = atu_prepro ($ptu,$src);
    if (($status!=0) && (($status!=3)||(!$CONTONWARN)))
    {
      die &PerlTranslate("TestScript.pl", "Generation failed.\n");
    }

    if ("$atu_out.cc" ne $src)
    {
      rename ("$atu_out.cc", $src);
    }

    ## Has something to rename ?
    if ($TDP_LANGUAGE eq "C")
    {
      if ( -f $atu_rename_file )
      {
        $atu_has_rename = 1;
      }
      autostub($ptu,$src) if($AUTOSTUB);
    }
  }
  
  ## Tester compilation
  if ($STAGE =~ /t/)
  {


   if (($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP"))
   {
   ## Do not forget the OBJ file in output log
   open ( LOG4, ">>$LOGFILENAME" ) or die &PerlTranslate("TestScript.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);

     push(@COMPLETE_INCLUDE_DIR_LIST,$TARGETDIR.$PATH_SEPARATOR."lib") if ($Ini{NOPATH});
     if ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || $atu_has_rename)
     {
      ## Tester instrumentation
      ## Intermediate files
      my $prepro_file = "$atu_out"."_aug.i";
      my $instr_file = "$atu_out"."_aug.$SRCEXT";

      ## C preprocessing
      print &PerlTranslate("TestScript.pl", "%1: Preprocessing...\n", "%1 is the file to be preprocessed", $src);

      $status = atl_cpp ($TDP_LANGUAGE, $src, $prepro_file, $CPPFLAGS.' '.$PPFLAGS,
                         \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST);
      EndStep($status);

      if ($status) { close( LOG4 ); return $status; }

      # Class ATTOL and $src are not instrumented
      my $instr_opts = BuildCCppInstrOptions($TDP_LANGUAGE,$src,
                                            ($TDP_LANGUAGE eq "CPP") ? "ATTOL" : "",
                                             $atu_has_rename ?
                                               $atu_rename_file :
                                               "");

      if ( $is_java ) 
      {
          ## Do not forget the JIR file in output log
          print  LOG4 "DUM $INTDIR/javi.jir\n";
      }
      else
      {
          print  LOG4 "DUM $instr_file\n";
          print  LOG4 "DUM $prepro_file\n";
      }

      $status = atl_instr ($TDP_LANGUAGE, $prepro_file, $instr_file, $instr_opts);
      if ($status) { close( LOG4 ); return $status; }

      unlink ($atu_rename_file);

      ## Compilation
      $status = Launch_cc_script($instr_file,$obj);

      ## Intermediate files remove
      unlink ($prepro_file, $instr_file) unless($status || $ENV{'ATTOLSTUDIO_VERBOSE'});
     }
     else
     {
      ## Compilation
      $status = Launch_cc_script($src,$obj);
     }

     print LOG4 "OBJ $obj\n" unless($status);
   }
   else
   {
    if ($TDP_LANGUAGE=~/^ADA/)
    {
      ## Compilation

      my $baseunitname = "";
      if ($INTDIR)
        {
          $baseunitname = $INTDIR.$PATH_SEPARATOR
        }
      $baseunitname .= $ATU_INTERMEDIATE_FILE;

      my $unitname = $baseunitname."_var_simule.ada";
      $status = Launch_cc_script($unitname,"");
      if ($status != 0) { return $status; }
      
      print LOG4 "OBJ ".lc($baseunitname."_var_simule.$OBJEXT\n");
      print LOG4 "OBJ ".lc($baseunitname."_var_simule.ali\n");

      $status = Launch_cc_script($src,"");
      if ($status != 0) { return $status; }
      
      print LOG4 "OBJ ".lc("$src.$OBJEXT\n");
      print LOG4 "OBJ ".lc("$src.ali\n");

      my $unitname = $baseunitname."_var_simule_B.ada";
      $status = Launch_cc_script($unitname,"");
      if ($status != 0) { return $status; }
      
      print LOG4 "OBJ ".lc($baseunitname."_var_simule_B.$OBJEXT\n");
      print LOG4 "OBJ ".lc($baseunitname."_var_simule_B.ali\n");

      my $unitname = $baseunitname."_fct_simule.ada";
      $status = Launch_cc_script($unitname,"");
      if ($status != 0) { return $status; }

      print LOG4 "OBJ ".lc($baseunitname."_fct_simule.$OBJEXT\n");
      print LOG4 "OBJ ".lc($baseunitname."_fct_simule.ali\n");

    }
    
    print LOG4 "OBJ ".lc($obj)."\n";
    print LOG4 "DUM ".lc($atu_out.".ali")."\n";
   }
  
   close( LOG4 );
  }

  return $status;
}

###########################################################################

sub getStubsFromPtu #($\@)
{
my $ptu = shift;
my $stub_lines = shift;

  # Extract the stubs from generated PTU
  open (PTU, $ptu) || die "Can not open the file \"$ptu\" to extract the list of STUBs\n";
  my $start;
  my $push = 0;
  my $lineNo = 0;
  foreach (<PTU>) {
    $lineNo ++;
    if (/^\s*DEFINE\s+STUB/) {
      $push = 1;
    } elsif (/^\s*END\s+DEFINE/ && $push) {
      $push = 0;
    } elsif ($push) {
      my $fct=extractFunctionName($_);
      push (@$stub_lines, $fct) if($fct);
    } elsif (/^--%c\s*(.+)$/) {
      # Is it the attolstartC command line
      $start.=$1." ";
    }
  }
  close (PTU);
  return ($start);
}

###########################################################################

sub extractFunctionName {
  my $line = shift;
  chomp $line;
  return("") unless($line =~ m/^\s*\#(.*)$/);
  $line = $1;
  #Does it look like a function, if so extract it
  # function prototype is one of
  # <type> <*> <function name> (
  my $functionName;
  if ($line =~ m/\W(\w+)\s*\(/) {
    $functionName = $1;
  }elsif ($line =~ m/\W(\w+)\s*;/) {# could be a var
    $functionName = $1;
  }
  #print "Extracted \"$functionName\" from \"$line\"\n"; 
  return ($functionName);
}


###########################################################################

sub getMissingStubs {
  my $exist = shift;
  my $required = shift;

  # Build of a list of just the name of the function being stubbed
  my %stubs;
  foreach (@$exist) {
    my $functionName = $_;
    $stubs{$functionName} = 1 if ($functionName);
  }

  # Now go over the list of required stubs eliminating those that are there
  # already
  my @missingStubs;
  foreach (@$required) {
      my $functionName = $_;
      push (@missingStubs, $_) unless (exists $stubs{$functionName});
  }
  return (@missingStubs);
}

###########################################################################

sub autostub {

  my ($ptu,$src) = @_;

  my $tmpFile="$INTDIR/atl_stubs.ptu";
  my @stubsInPtu;
  my $attolstartbin="$BINDIR$PATH_SEPARATOR" . "attolstart".$TDP_LANGUAGE;
  # Get the existing stubs from the PTU 
  my $start = getStubsFromPtu($ptu,@stubsInPtu);
  my ($d,$f,$e) = SplitFileName($ptu);
  if($start=~ /$f\.$e/){
	$start=~ s/\s*attolstart /$attolstartbin /i;
	$start=~ s/\"[^\"]+\.ptu\"/$tmpFile/i;
	$start=~ s/-studio_log=\"[^\"]+\"//;
	my $status = SystemP($start);
  	unless($status) {
  		
		# Extract the stubs from generated PTU
		my @required_stubs;
		getStubsFromPtu($tmpFile,@required_stubs);
		#print "required:",scalar(@required_stubs);
		# Get the missing stubs from the test script
		my @missing_stubs = getMissingStubs(\@stubsInPtu, \@required_stubs);
		if(scalar(@missing_stubs)){
			# Generate Stub in script c file
			open (STUB, ">>$src") || die "Can not open the file \"$src\" for writing\n";
			print STUB "\n/* Auto Stub Generation */\n";
			print "Generating stubs:";
			foreach (@missing_stubs) { 
				print STUB "int $_ (void) {_atl_add_error(\"W2 \",\"call of undeclared stub $_\");return (0);}\n";
				print $_;
			}
			close STUB;
			print "\n";
		} else {
			print "no missing stub found\n";
		}
	}
	unlink($tmpFile);
  }else{
  	print "TestRT-W-MISCOLIGE missing attolstart command line in PTU file $ptu: no generation possible\n";
  }
}

################################################################################
# RTRT System Testing preprocessor execution
# Arguments:
#    $script : PTS test script file
#    $source : source file to be generated
# Return value: atspreproc return value
#    $BINDIR
#    $LOGFILENAME
################################################################################
sub ats_prepro #($$$)
{
  my ($script,$interfaces,$source) = @_;

  # Let's associate right binary to any supported language
  # TBD !!!
  $AtsPrepro{'C'} = "atspreproC";
  $AtsPrepro{'CPP'} = "atspreproCPP";

  # Let's go for the environment
  # TBD !!!
  # temporary: -> to be replaced by ATLDIR
  $ENV{'ATLDIR'} = "$BINDIR";
  $ENV{'ATS_DIR'} = "$BINDIR";

  $ATS_ALLOCATION="" if($ATS_ALLOCATION eq "NONE");
  $ATS_TRACE ="" if($ATS_TRACE eq "NONE");

  my $ats_options = "";
  if ($ATS_ADD_OPTIONS ne "")
  {
        $ats_options .=" $ATS_ADD_OPTIONS";
  }
  if ($VT_INSTANCE)
  {
	$ats_options .=" -INSTANCE=$VT_INSTANCE";
  }
  if ($ATS_TRACE ne "")
  {
        $ats_options .=" -TRACE=".$ATS_TRACE;
  }
  elsif ($ATS_TRACE_CIRCULAR)
  {
        $ats_options .=" -TRACE=CIRCULAR -BUFSIZE=".$ATS_BUFSIZE;
  }
  if ($ATS_THREAD)
  {
	if ($ATS_THREAD_FUNCTION ne "") {
		$ats_options .= " -THREAD=$ATS_THREAD_FUNCTION";
	} else {
		$ats_options .= " -THREAD=ATL_Start";
	}
	if ($ATS_ALLOCATION eq ""){
		print &PerlTranslate("TestScript.pl", "WARNNING: Virtual Tester Memory allocation should not be Static when multiple instances are running\n", "");
	}	
	if($ATS_NOTSHARED)
	{
		$ats_options .= " -NOTSHARED";
	}
  }
  if ($ATS_ALLOCATION ne "")
  {
        $ats_options .=" -ALLOCATION=$ATS_ALLOCATION";
  }
  if (@VT_ITEM_SELECTION >0)
  {
	my $scn=join(",",@VT_ITEM_SELECTION);
	$ats_options.=" -SCN=$scn";
  }

  if (@VT_FAM_SELECTION >0)
  {
  	my $fam=join(",",@VT_FAM_SELECTION);
	$ats_options.=" -FAMILY=$fam";
  }
  if ($TDP_LANGUAGE eq "C")
  {
  	$ats_options .=" -STD_DEFINE=\"$TARGETDIR/ana/atus_c.def\" ";
  }
  if ($TDP_LANGUAGE eq "CPP")
  {
  	$ats_options .=" -STD_DEFINE=\"$TARGETDIR/ana/atus_cpp.def\" ";
  }

#print "TestScript: COMPLETE_INCLUDE_DIR_LIST=".join(",",@COMPLETE_INCLUDE_DIR_LIST)."\n";

  @Interfaces_List=split(",",$interfaces);
  my @interface_dir_list=();
  my @interface_name_list=();
  foreach $Interfaces_Elem (@Interfaces_List)
  {
	($interface_dir, $interface_name, $interface_ext) = SplitFileName( $Interfaces_Elem );
#	$interface_dir=~s/\\/\//g;
	$interface_dir=~s/\/\s*$//g;
	$interface_dir=~s/\\\s*$//g;
	push @interface_name_list,"$interface_name.$interface_ext";
	if(($interface_dir ne "") && (!grep(/^\Q$interface_dir\E$/,@interface_dir_list)))
	{
		push @interface_dir_list,$interface_dir;	
	}
  }
  foreach $Elem (@COMPLETE_INCLUDE_DIR_LIST)
  {
	if(!grep(/^\Q$Elem\E$/,@interface_dir_list))
	{
		push @interface_dir_list,$Elem;	
	}
  }
  my $interfaces=join(",",@interface_name_list);

#print GetCurrentDir(). " is the current dir\n";
  my $PreproInclude;
  if (@interface_dir_list)
    {
       $PreproInclude="-INCL=\"".join(",",@interface_dir_list)."\"";
    }

  # TDB: a remplacer par la vraie syntaxe !!!
  my $Command="";
  my ($src_dir, $src_name, $src_ext) = SplitFileName( $source );
  {
  $Command = "$BINDIR" . "$PATH_SEPARATOR" . $AtsPrepro{$TDP_LANGUAGE} . " \"$script\" \"$interfaces\"  \"$INTDIR$PATH_SEPARATOR$src_name.$src_ext\" $ats_options $PreproInclude";
  }

  # execution
  my $status = SystemP($Command);
  return $status;
}

################################################################################
# Making of a "PTS" test script file
# Arguments:
#   $pts     : PTS test script file
# Return value: 0 if OK
# Required global variables:
#   $INTDIR
# Used subroutines:
#   ats_prepro
################################################################################
sub pts_make #($$$)
{
  $ATOMODE=0;
  $ATUMODE=0;
  $ATSMODE=1;
  $ATPMODE=0;
  $ATQMODE=0;
  $ATCMODE=0;
  $ATTMODE=0;
  $ATKMODE=0;

  my ($pts,$interfaces,$src) = @_;

  my ($pts_dir, $pts_name, $pts_ext) = SplitFileName ($pts);
  my ($src_dir, $src_name, $src_ext) = SplitFileName( $src );
  my $obj = "$INTDIR/$pts_name.$pts_ext.$OBJEXT";
  my $status ;
  ## Do not forget the OBJ file in output log
  open ( LOG5, ">>$LOGFILENAME" ) or die &PerlTranslate("TestScript.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);

  ## PTS preprocessing
  if ( $STAGE =~ /p/ )
  {
	print &PerlTranslate("TestScript.pl", "\nMaking PTS source file...\n");
	print "*****************************************************\n"; # NO TRANSLATION
  	$status = ats_prepro ($pts,$interfaces,$src);
  	if ($status > 3)
  	{
   		 return $status;
  	}
  }
  ## System Testing compilation file
  $status = MakeCompilationOptionsFile ();
  if ($status != 0) { return $status; }
  

  ## Compilation
  push @COMPLETE_INCLUDE_DIR_LIST, $TARGETDIR.$PATH_SEPARATOR."lib";
  push @COMPLETE_INCLUDE_DIR_LIST, $INTDIR;
  @COMPLETE_INCLUDE_DIR_LIST=reverse (@COMPLETE_INCLUDE_DIR_LIST);
  push(@GENERATION_CONDITION,"ATL_SYSTEMTEST_DEBUG") if($ATL_SYSTEMTEST_DEBUG);

  my ($srcdir, $srcname, $srcext) = SplitFileName( $SRCFILE );

	if ( $STAGE =~ /c/ )
        {
        	# TBD: SV virer ce -DUSE_ATS !
        	print &PerlTranslate("TestScript.pl", "%1: Compiling...\n\n", "%1 is the name of the file to be compiled", $src);
        	$status = atl_cc ($TDP_LANGUAGE, "$INTDIR/$src",
                          "$INTDIR/$srcname.$OBJEXT",
                          $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS ,
                          \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST);
        	if ($status != 0) { return $status; }
	        if(($ATS_THREAD)&&(@VT_RDV_CONF)){
        	
			open RDV, ">$INTDIR/rdvtable.h" or die &PerlTranslate("SystemTest.pl", "Cannot create file %1\n", "%1 is the name of the file", "$INTDIR/rdvtable.h");
			print RDV "\n/*--- SET the RENDEZVOUS MEMBERS ---*/\n";
			print RDV "\n";
			print RDV "ATL_T_RDV rdvList[]={\n";
			my $count=0;
			my $sep=' ';
			foreach (@VT_RDV_CONF){
				print RDV "$sep {\"".$VT_RDV_CONF[$count][0]."\",".$VT_RDV_CONF[$count][1].",0,NULL}\n";
				$sep=',';
				$count++;
			}
			print RDV "};\n";
			print RDV "#define ATL_RDVTIMEOUT ".$ATS_RDV_TIMEOUT."000\n";
			close RDV;
                	print LOG5 "DUM $INTDIR/rdvtable.h\n";
		}

	
                print LOG5 "OBJ $INTDIR/$srcname.$OBJEXT\n";
                if (($ATS_THREAD)&&($ATS_THREAD_LAUNCHER)){
                	if(-r "$TARGETDIR/lib/AtsMain.c") {
                		push(@GENERATION_CONDITION,"ATL_Start=$ATS_THREAD_FUNCTION") if($ATS_THREAD_FUNCTION ne "");
                		push(@GENERATION_CONDITION,"ATTOLSTUDIO_VERBOSE") if($ENV{'ATTOLSTUDIO_VERBOSE'});
				$status = atl_cc ($TDP_LANGUAGE, "$TARGETDIR/lib/AtsMain.c",
					  "$INTDIR/AtsMain.$OBJEXT",
					  $CPPFLAGS.' '.$CFLAGS.' '.$PPFLAGS ,
					  \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST);
  				EndStep($status);
				if ($status != 0) { return $status; }
				push @OBJECTS, "$INTDIR/AtsMain.$OBJEXT";
                	} else {
                		print &PerlTranslate("TestScript.pl", "The Thread launcher AtsMain.c does not exist in this TDP.\n", "");
                	}
                }
	}

	if ( $STAGE =~ /l/ )
        {
		print &PerlTranslate("TestScript.pl", "%1: Linking...\n\n", "%1 is the name of the file to be linked", $src);
		my $pts_obj = "$PROJECT_PATH" . "$PATH_SEPARATOR" . "$INTDIR" . "$PATH_SEPARATOR" . "$srcname.$OBJEXT";
		my $found = 0;
		foreach $local_obj (@OBJECTS){
		  if ($local_obj eq $pts_obj){ $found = 1; }
		}
		if($found eq 0){
		  push @OBJECTS, "$INTDIR/$srcname.$OBJEXT";
		}
		MakeTargetPackageObject();
  		push( @OBJECTS, GetTargetPackageLib($OBJEXT) );
		$status = atl_link ( "$INTDIR/$srcname.$EXEEXT",
                             \@OBJECTS, $LDFLAGS,
                             \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
  		EndStep($status);
		if ($status != 0) { return $status; }
                
                print LOG5 "EXE $INTDIR/$srcname.$EXEEXT\n";
	}

  print  LOG5 "DUM $INTDIR/$srcname.$srcext\n";
  print  LOG5 "TDC $INTDIR/$srcname.tdc\n";
  print LOG5 "TSD $INTDIR/$srcname"."_dyn.tsf\n";
  close( LOG5 );
  return 0;
}

################################################################################
# RTRT Object Testing preprocessor execution
# Arguments:
#    $source : OTD/OTC file
#    $cc     : generated C++ file
#    $oti    : generated OTI file
#    $tdf    : TDF file
# Return value: atoprepro return value
# Required global variables:
#    $BINDIR
#    $LOGFILENAME
# Used global variables:
#    $ATO_PREPRO_MAX_ERRORS
#    $ATO_PREPRO_NODLINE
################################################################################
sub ato_prepro #($$$$)
{
   my ($source,$cc,$oti,$tdf) = @_;

   # command
   my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "atoprepro \"$source\" -G C++ -studio_log \"$LOGFILENAME\"";

   # Preserve the previous (if any) file in order to compare the new OTI
   # file against the previous one
   my $previous_oti_file = "";
   if ( -f $oti )
   {
      $previous_oti_file = "$oti.prev";
      rename($oti, $previous_oti_file);
   }

   # OTI generated file
   $Command .= " -OTI \"$oti\"";

   # C++ generated file
   $Command .= " -O \"$cc\"";

   # TDF file name
   $Command .= " -TDF \"$tdf\"";

   # options
   if ($ATO_PREPRO_MAX_ERRORS)
   {
      $Command .= " -E $ATO_PREPRO_MAX_ERRORS";
   }

   if ($ATO_PREPRO_NODLINE || $Ini{'NO_INSTR_GENERATED_LINE_INFO'})
   {
      $Command .= " -NODLINE";
   }

   if ($Ini{'NOPATH'})
   {
      $Command .= " -NOPATH";
   }
   $Command .= " ".$Ini{'PREPRO_OPT'} if($Ini{'PREPRO_OPT'});

   # execution
   my $status;
   if($Ini{'ATTOLPREPRO'}) {
      print "ATTOLPREPRO specified as:$TARGETDIR/cmd/$Ini{'ATTOLPREPRO'}\n";
      print "sub attolPrepro() executed with $Command\n";
      require "$TARGETDIR/cmd/$Ini{'ATTOLPREPRO'}";
      $status = attolPrepro($Command,'CPP',$source,$cc);
   }
   else
   {
      $status = SystemP($Command);
   }
 
   # Some late checks
   if ($OSNAME eq "win32" &&
       !$ATO_PREPRO_NODLINE &&
       !$Ini{'NO_INSTR_GENERATED_LINE_INFO'})
   {
      if (FindOutOptionInFile ( "--old_style_preprocessing",
                                 "$TARGETDIR/ana/atl.opp" ))
      {
        # Let's replace #line nnn "\\1\\2\\3" by #line nnn "\1\2\3" in $cc
        open(GENERATED_ATO_SRC, "<$cc") || die &PerlTranslate("TestScript.pl", "cannot open %1: %2\n", "%1 is the name of the file that cannot be opened and %2 is the reason given by the Operating System", $cc, $!);
        open(GENERATED_ATO_SRC_TMP, ">$cc.tmp") || die &PerlTranslate("TestScript.pl", "cannot create %1: %2\n", "%1 is the name of the file that cannot be created and %2 is the reason given by the Operating System", $cc.tmp, $!);

        while (<GENERATED_ATO_SRC>) {
          if (/^#line\s\s*([1-9][0-9]*)\s\s*"(.*)"$/) {
            my $number = $1;
            my $file = $2;
            #print "\$number=$number, \$file=$file\n";
            $file =~ s/\\\\/\\/g;
            print (GENERATED_ATO_SRC_TMP "#line $number \"$file\"\n");
          } else {
            print GENERATED_ATO_SRC_TMP;
          }
        }

        close(GENERATED_ATO_SRC);
        close(GENERATED_ATO_SRC_TMP);

        rename ("$cc.tmp", "$cc") || die &PerlTranslate("TestScript.pl", "rename %1 to %2 failed\n", "%1 is the original name and %2 is the new name", $cc.tmp, $cc);
        #copy("$cc.tmp", "$cc") || die &PerlTranslate("TestScript.pl", "copy %1 to %2 failed\n", "%1 is the original name and %2 is the new name", $cc.tmp, $cc);
        #unlink ("$cc.tmp");

        print &PerlTranslate("TestScript.pl", "Contraction of double backslashed #line to simple in the Object Testing generated file succeed\n") if ($ENV{'ATTOLSTUDIO_VERBOSE'});
      }
   }

   # Let's check if the generated OTI file is the same the previous one.
   # If they are identical, we need to tell Studio that it doesn't need
   # to recompile the source files.
   my $sources_recompilation_needed = 1;
   if (-f $oti && $previous_oti_file ne "" && -f $previous_oti_file)
   {
      if (FileDiff($oti, $previous_oti_file) == 0)
      {
        $sources_recompilation_needed = 0;
      }
   }
   if ($sources_recompilation_needed) {
      print "Recompiling the C++ source files is necessary.\n";
   } else {
       print "No need to recompile C++ source files.\n";
       rename($previous_oti_file, $oti);
       # This file must be set to exactly the same date as the otd
       $otd_date = (stat($source))[9];
       utime($otd_date, $otd_date, $oti);

   }
   if (-f $previous_oti_file) {
       unlink ($previous_oti_file);
    }

   return $status;
}

################################################################################
# Making of an "OTC" source file
# Arguments:
#   $otc     : OTC source file
#   $oti     : OTI generated file
# Return value: 0 if OK
# Required global variables:
#   $INTDIR
# Used subroutines:
#   ato_prepro
################################################################################
sub otc_make #($$)
{
   my ($otc,$oti) = @_;

   ## Unused files
   my $cc = "temp_cc";

   print &PerlTranslate("TestScript.pl", "OTC preprocessing...\n");

   if ($STAGE =~ /p/)
   {
     my $status = ato_prepro ($otc, $cc, $oti, "unused_tdf");

     if ($status != 0)
     {
        return $status;
     }
   }
   push(@OTIFILES ,$oti); #update the list for eclipse

   ## Intermediate files remove
   #Do not remove this temp file so it is not marked as "to be build" each
   #time an .otc node is made
   #unlink ($cc);

   return 0;
}

################################################################################
# Making of an "OTD" source file
# Arguments:
#   $otl     : OTD source file
#   $oti     : OTI generated file
#   $tdf     : TDF file
#   $src     : generated C++ file
# Return value: 0 if OK
# Required global variables:
#   $INTDIR
#   $RM
#   $CPPFLAGS
#   @COMPLETE_INCLUDE_DIR_LIST (merge between @INCLUDE_DIR_LIST and @INCLUDES)
# Used subroutines:
#   ato_prepro
#   atl_cpp
#   atl_cc
#   atl_instr
################################################################################
sub otl_make #($$$$)
{
   my ($otl,$oti,$tdf, $src) = @_;

   my ($otl_dir, $otl_name, $otl_ext) = SplitFileName ($otl);
   my ($src_dir, $src_name, $src_ext) = SplitFileName( $src );

   ## Intermediate files
   my $prepro_file = "$INTDIR/$otl_name.$otl_ext.i";
   my $instr_file = "$INTDIR/$otl_name.$otl_ext"."_aug.$src_ext";

   my $language = "CPP";
   my $status = 0;
   my $obj = "$INTDIR/$otl_name.$otl_ext.$OBJEXT";

   if ($STAGE =~ /p/)
   {

     ## OTD Preprocessing
     print &PerlTranslate("TestScript.pl", "%1: Object Testing preprocessing...\n", "%1 is the name of the file to be preprocessed by Object Testing", $otl_name.'.'.$otl_ext);
     my $status = ato_prepro ($otl, $src, $oti, $tdf);
     if($status!=0) { return 1; }

     ## Assume that _ATO_TestClass should not be instrumented
     my $exunit = "_ATO_TestClass";
     ## Assume that code from generated source should not be instrumented
     my $exfile = "$otl,$otl_dir$otl_name.dcl,$otl_dir$otl_name.stb,$src";

     ## C preprocessing
     print &PerlTranslate("TestScript.pl", "%1: Preprocessing...\n", "%1 is the name of the file to be preprocessed", $src);
     my @conditions=@GENERATION_CONDITION;
     push(@conditions,"RTRT_USE_TPDLL") if($ATL_RUNTIME_FORMAT eq "DLL");

     $status = atl_cpp ($language, $src, $prepro_file, $CPPFLAGS.' '.$PPFLAGS,
                      \@conditions, \@COMPLETE_INCLUDE_DIR_LIST);
     EndStep($status);
     return $status if ($status);

     open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("TestScript.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
     print  LOG "DUM $prepro_file\n";
     close( LOG );

     ## Coverage, Tracer and Object Testing Instrumentation
     print &PerlTranslate("TestScript.pl", "%1: Instrumenting...\n", "%1 is the name of the file to be instrumented", $src);
     @OTIFILES = ($oti, @OTIFILES);
     my $instr_opts = BuildCCppInstrOptions($language,$exfile,$exunit,"");
     $status = atl_instr ("CPP", $prepro_file, $instr_file, $instr_opts);
     if ($status != 0) { return $status; }
   }

   if ($STAGE =~ /t/)
   {
     ## ATK compilation file
     $status = MakeCompilationOptionsFile ();
     if ($status != 0) { return $status; }

     ## Compilation
     print &PerlTranslate("TestScript.pl", "%1: Compiling...\n", "%1 is the name of the file to be compiled", $src);
     my @includes_with_intdir = ($INTDIR, @COMPLETE_INCLUDE_DIR_LIST);
     push(@includes_with_intdir,$TARGETDIR.$PATH_SEPARATOR."lib") if ($Ini{NOPATH});

     $status = atl_cc ($language, $instr_file, $obj, $CPPFLAGS.' '.$CFLAGS, 
		     \@GENERATION_CONDITION, \@includes_with_intdir);
     EndStep($status);
     return $status if ($status);

     ## Do not forget the OBJ file in output log
     open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("TestScript.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
     print  LOG "OBJ $obj\n";
     close( LOG );
   }

   ## Intermediate files remove
   unlink ($prepro_file, $instr_file) if (!$ENV{'ATTOLSTUDIO_VERBOSE'});

   return 0;
}

################################################################################

sub TestScript
{
 my ($srcdir, $srcname, $srcext) = SplitFileName( $FILENAME );

 if (! $srcext)
 {
   die &PerlTranslate("TestScript.pl", "Cannot retrieve file name extension from %1", "%1 is the name of the file", $FILENAME);
 }
 elsif ($srcext eq "ptu")
 {

## Required information:
##   $FILENAME       : PTU source file
## Optional information:
##   <none> ...

   my $status = ptu_make ($FILENAME,$SRCFILE);

   if ($status != 0)
   {
       die &PerlTranslate("TestScript.pl", "Failed.\n");
   }

 }
 elsif ($srcext eq "otc")
 {

## Required information:
##   $FILENAME       : OTC source file
##   $OTIFILE        : OTI generated file
## Optional information:
##   $ATO_PREPRO_MAX_ERRORS : maximum number of errors
##   $ATO_PREPRO_NODLINE    : no #line generation

   print &PerlTranslate("TestScript.pl", "Making OTC source file...\n");
   my $status = otc_make ($FILENAME, $OTIFILE);

   if ($status != 0)
   {
       die &PerlTranslate("TestScript.pl", "Failed.\n");
   }

 }
 elsif ($srcext eq "otl" || $srcext eq "otd")
 {

## Required information:
##   $FILENAME     : OTD source file
##   @OTIFILES       : OTI source files
##   $OBJFILE        : object generated file
##   $TDFFILE        : TDF file
##   $OTIFILE        : OTI generated file
## Optional information:
##   $ATO_PREPRO_MAX_ERRORS : maximum number of errors
##   $ATO_PREPRO_NODLINE    : no #line generation
##   $ATO_INSTR_BODY        : methods generation mode (must be 'INLINE',
##                            'NAME_CONV' or 'MAP_FILE')
##   $ATO_INSTR_MAP_FILE    : map file name (required when 
##                            $ATO_INSTR_BODY eq 'MAP_FILE')
##   $ATO_INSTR_AUTO_OTI    : automatic retrieving of oti files according to
##                            name conventions
##   $ATO_INSTR_OTI_DIR     : OTI directory (required when $ATO_INSTR_AUTO_OTI)
##   $ATO_NO_AC             : AssertCheck features disabled
##   $ATO_NO_CAST           : CAST features disabled

   print &PerlTranslate("TestScript.pl", "Making OTD source file...\n");
   my $status = otl_make ($FILENAME, $OTIFILE, $TDFFILE, $SRCFILE);
   if ($status != 0)
   {
      die &PerlTranslate("TestScript.pl", "Failed.\n");
   }
 }
 elsif ($srcext eq "pts")
 {
   my $interfaces=join(",",@ATS_INTERFACE_FILES);
   unless(@ATS_INTERFACE_FILES){
   	print &PerlTranslate("TestScript.pl", "ERROR: no interface file selected\n");
   	$interfaces="X";
   }
   my $status = pts_make ($FILENAME, $interfaces, $SRCFILE);
   if ($status != 0)
   {
      die &PerlTranslate("TestScript.pl", "Failed.\n");
   }
 }
 elsif (($srcext eq "spv")&&(!$ATS_POSTPRO_ONLY)&&( $STAGE =~ /e/ ))
 {
   $ENV{'ATS_DIR'} =$BINDIR;
   $ATS_DIR=$BINDIR;
   launchLocalAgent();
   print &PerlTranslate("TestScript.pl", "\nStarting execution using %1 script...\n","%1 is the spv script file",$FILENAME);
   print "*****************************************************\n"; # NO TRANSLATION
   my $status;
   if (($ATS_DYN_FOLLOW)&&("x$TDF_CHANNEL" ne "x"))
   {
   	$status=SystemP("$ATS_DIR" . "$PATH_SEPARATOR" . "atsspv \"$FILENAME\" -STUDIO_MACH=$ADDRESS -STUDIO_PORT=$STUDIO_PORT -STUDIO_CHNUM=$TDF_CHANNEL");
   }
   else
   {
	$status=SystemP("$ATS_DIR" . "$PATH_SEPARATOR" . "atsspv \"$FILENAME\"");
   }
   if ($status) {&PerlTranslate("SystemTest.pl", "Supervisor execution status $status for %1\n","%1 is the spv script file",$FILENAME); }
 }
 return 0;
}

#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

CheckIntDir();
EnterInScript();

ClearLogFile($LOGFILENAME);
exit (TestScript());
}

1;
#
################################################################################
