#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                            -*-  Some tools -*-                               #
                                                                               #                                                                                                      #
################################################################################
#$Id: Tools.pl,v 1.232.2.1 2008/01/28 13:35:33 rfrance Exp $
################################################################################


use File::Copy;

require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "instrum.pl";

################################################################################
# the 'Touch' command for a file: change access and modification time.
# Arguments:
#    @file_name: list of file name to touch
# Return :
#    $number of file touched
# Required global variables:
#    none
################################################################################
sub atl_touch #($)
{ 
  my @file_names = @_;

  my $now = time;
  my $nb_file_touched = utime $now, $now, @file_names;
  return $nb_file_touched ;
}

################################################################################
# Copy for ada file (source and dest must be files)
# Arguments:
#    $source: source file to copy
#    $dest  : file to generate
#    $force_copy: 1: copy is always done,
#                 0: copy is donne if $dest doesn't exist.
# Required global variables:
#    none
################################################################################
sub atl_copy #($$$)
{ 
  my ($source,$dest,$force) = @_;

  #optimisation: do no copy file if exist 
  # ... but if force mode is on (same name but contains different things).
  if ( ! $force && -f $dest )
  {
    return ;
  }

  copy($source,$dest) or die &PerlTranslate("Tools.pl", "Unable to copy \"%1\": %2", "%1 is the name of the file and %2 is the reason given by the Operating System", $source, $!);
  #force the update of the date file to force compilation for 'make' compiler
  #nb: very useful for windows platform.
  atl_touch( $dest );
#touch-mano:  open (dest_file, ">>$dest" ) or die ("Cannot open $dest file.\n");
#touch-mano:  print dest_file "\n\--Just copyed\n";
#touch-mano:  close(dest_file);
}

################################################################################
# launch cc script and save result in log
# Arguments:
#   src (string)
# Return value:
#   status of atl_cc 
################################################################################
sub Launch_cc_script #($$)
{ 
  my ( $src, $obj ) = @_;
  print &PerlTranslate("Tools.pl", "%1: Compiling...\n", "%1 is file to be compiled", $src);
  my $status = atl_cc ($TDP_LANGUAGE, $src, $obj, $CPPFLAGS . " " . $CFLAGS,
                        \@GENERATION_CONDITION, \@COMPLETE_INCLUDE_DIR_LIST);
  EndStep($status);
  return $status if ($status);
  
  ##OBJ file in output log is written at above level
  return 0;
}

################################################################################
# RTRT Sockect Uploader execution
################################################################################
sub atl_socket_receiver #($$)
{
   my ($delimiter,$port) = @_;

   my $binopts      = "";
   my $binary       = "";

   $binary="$BINDIR" . "$PATH_SEPARATOR" . "trtpd";

   $binopts .= " --oneshot --status"; # Force close when disconnect with status file generation
   unlink("trtpd.log");

   if($delimiter) {
      $binopts .= " --delimiter " . ord($delimiter); # Force the delimiter
   }
   if($port) {
      $binopts .= " --port " . $port;  # Force the port number
   } else {
	$binopts .= " --port " . $UPLOADER_PORT; #use the default port number from host configuration
   }

   if ($ATUMODE || $ATSMODE)
   {
     $binopts .= " \"$RIOFILE\"";
   }
   if ($ATPMODE)
   {
     $binopts .= " \"$ATP_TPF_NAME\"";
   }
   if ($ATQMODE)
   {
     $binopts .= " \"$ATQ_TQF_NAME\"";
   }
   if ($ATCMODE)
   {
     $binopts .= " \"$ATC_TIO_NAME\"";
   }
   if ($ATOMODE || $ATKMODE || $ATJMODE || $ATTMODE)
   {
     $binopts .= " \"$TDFFILE\"";
   }

   my $command = "$binary $binopts";

   if ( ! -e $binary && ! -e "$binary.exe" )
   {
     print &PerlTranslate("Tools.pl", "Error: cannot find socket receiver binary: \"%1\"\n", "%1 is the name of the socket receiver binary", $binary);
     return 1;
   }

   if ( -e $binary ) {
     # Unix
     $command .= " &";
   } else {
     $command = "start " . $command;
   }

   #execution
   my $status = SystemP( $command );
   return $status;

}

################################################################################
# CheckBootClassPath
# Arguments:
#   none
# Return value: 
#   none
################################################################################
sub CheckBootClassPath
{
  my $nb_ok = 0;
  my $nb_ko = 0;
  my $item;
  my @project_bootclasspath = ();

  foreach $item (@JAVA_BOOTCLASSPATH) {
    if (! -d $item && ! -f $item) {
      $nb_ko++;
      print &PerlTranslate("Tools.pl", "BootClassPath Setting element %1 does not exist\n", "%1 is the element in the BootClassPath that does not exist", $item) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
    } else {
      $nb_ok++;
      push @project_bootclasspath, $item;
    }
  }
  if (!scalar(@JAVA_BOOTCLASSPATH)) {
    @JAVA_BOOTCLASSPATH = split (/,/, $Ini{'DEFAULT_BOOTCLASSPATH'});
    print &PerlTranslate("Tools.pl", "Info: BootClassPath Setting is empty\nAutomatically using Target Deployment Port DEFAULT_BOOTCLASSPATH\n") if ($ENV{'ATTOLSTUDIO_VERBOSE'});
  } elsif (!$nb_ok) {
    @JAVA_BOOTCLASSPATH = split (/,/, $Ini{'DEFAULT_BOOTCLASSPATH'});
    print &PerlTranslate("Tools.pl", "Info: BootClassPath Setting contains files or directories that do not exist\nAutomatically using Target Deployment Port DEFAULT_BOOTCLASSPATH\n") if ($ENV{'ATTOLSTUDIO_VERBOSE'});
  } elsif ($nb_ko) {
    @JAVA_BOOTCLASSPATH = split (/,/, $Ini{'DEFAULT_BOOTCLASSPATH'});
    print &PerlTranslate("Tools.pl", "Info: Some BootClassPath Settings do not exist\nAutomatically prepending with Target Deployment Port DEFAULT_BOOTCLASSPATH\n") if ($ENV{'ATTOLSTUDIO_VERBOSE'});
  } else {
    return;
  }

  $nb_ok = 0;
  $nb_ko = 0;
  foreach $item (@JAVA_BOOTCLASSPATH) {
    if (! -d $item && ! -f $item) {
      $nb_ko++;
      print &PerlTranslate("Tools.pl", "Target Deployment Port DEFAULT_BOOTCLASSPATH element %1 does not exist\n", "%1 is the element in the DEFAULT_BOOTCLASSPATH that does not exist", $item);
    } else {
      $nb_ok++;
    }
  }
  if (scalar(@project_bootclasspath)) {
    push @JAVA_BOOTCLASSPATH, @project_bootclasspath;
  }
  if (!$nb_ok) {
    print &PerlTranslate("Tools.pl", "Error: None of the Target Deployment Port DEFAULT_BOOTCLASSPATH items exist\nPlease Start Target Deployment Port Editor and Fix DEFAULT_BOOTCLASSPATH Basic Setting For Java\n");
    exit 1;
  } elsif ($nb_ko) {
    print &PerlTranslate("Tools.pl", "Warning: Some Target Deployment Port DEFAULT_BOOTCLASSPATH do not exist\nYou can fix that starting the Target Deployment Port Editor and modifying the DEFAULT_BOOTCLASSPATH Basic Setting For Java\n");
  }
}

################################################################################
# BuildJavaBootClassPath
# Arguments:
#   $separator
# Return value: 
#   classpath
################################################################################
sub BuildJavaBootClassPath #($$$)
{
  my ($option,$separator,$other) = @_;
  my $classpath = "$option";

  # push the bootclasspath and $other
  CheckBootClassPath();
  my @list = (@JAVA_BOOTCLASSPATH);

  if ($other) {
    push @list, $other;
  }

  if ( scalar(@list) > 0 ) {
    $classpath .= "\"" . join ("\"$separator\"", @list) . "\"";
  }

  return $classpath;
}

################################################################################
# BuildJavaClassPath
# Arguments:
#   $separator
# Return value: 
#   classpath
################################################################################
sub BuildJavaClassPath #($$$)
{
  my ($option,$separator,$other) = @_;
  my $classpath = "$option";

  # push $other and the classpath
  my @list = ();

  if ($other) {
    push @list, $other;
  }
  push @list, @JAVA_CLASSPATH;

  if ( scalar(@list) > 0 ) {
    $classpath .= "\"" . join ("\"$separator\"", @list) . "\"";
  }

  return $classpath;
}

################################################################################
# BuildCompleteJavaClassPath
# Arguments:
#   $separator
# Return value: 
#   classpath
################################################################################
sub BuildCompleteJavaClassPath #($$$)
{
  my ($option,$separator,$other) = @_;
  my $classpath = "$option";
  my $control = "$option";

  # push the bootclasspath, $other and the classpath
  CheckBootClassPath();
  my @list = (@JAVA_BOOTCLASSPATH);

  if ($other) {
    push @list, $other;
  }
  push @list, @JAVA_CLASSPATH;

  if ( scalar(@list) > 0 ) {
    $classpath .= "\"" . join ("\"$separator\"", @list) . "\"";
  }

  if ($classpath eq $control)
  {
    return "";
  }
  else
  {
    return $classpath;
  }  
}

################################################################################
# GetJavaClassName
# Arguments:
#   $executable
# Return value: 
#   classname
################################################################################
sub GetJavaClassName #($)
{
  my $executable = shift;

  my ($dir,$classname) =  ( $executable =~ /^((?:.*[:\\\/])?)(.*)/ );

  return $classname;
}

################################################################################
# CheckOrCreateManifestFile
# Arguments:
#   none
# Return value:
#   none
################################################################################
sub CheckOrCreateManifestFile
{
  if (!$JAVA_MANIFEST) {
    $JAVA_MANIFEST = "MANIFEST.MF";
    if (! -f $JAVA_MANIFEST) {
      print &PerlTranslate("Tools.pl", "Creating %1, a default Manifest File to be included in the jar file\n", "%1 is name of this default Manifest File", $JAVA_MANIFEST);
      open ( MF, ">$JAVA_MANIFEST" ) or die &PerlTranslate("Tools.pl", "Cannot create file %1.\n", "%1 is name of the file", $JAVA_MANIFEST);
      print  MF "Manifest-Version: 1.0\nCreated-By: Rational Software Corp.\n";
      close( MF );
    } else {
      print &PerlTranslate("Tools.pl", "Using %1 as Manifest File to be included in the jar file\n", "%1 is name of this default Manifest File", $JAVA_MANIFEST);
    }
    print &PerlTranslate("Tools.pl", "To avoid this automatic behavior, set a Manifest File in the Build|Linker settings\n");
  } else {
    if (! -f $JAVA_MANIFEST) {
      die &PerlTranslate("Tools.pl", "Error while creating Jar File: the file %1 does not exist\nCheck the Manifest File you entered in the Build|Linker settings or create this file\nClearing the name you provided in this setting, a default Manifest File will be generated\n", "%1 is name of the Manifest File", $JAVA_MANIFEST);
    }
  }
}
################################################################################
# BuildJvmpiOutFiles
# Arguments:
#   none
# Return value: 
#   None.
################################################################################
sub BuildJvmpiOutFiles
{
    $JPTNAME = "$INTDIR/$NAME.jpt";
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);

    if ( open(INPUTFL,$JPTNAME))
    {
	print  LOG "JPT $JPTNAME\n";
	# Extended part.
	if ($JAVA_TPF_FILENAME eq "<default>")
	{
	    $TPFNAME = "$INTDIR/$NAME.tpf";
	    print  LOG "TPF $TPFNAME\n";
	}
	else
	{
	    $TPFNAME = "$INTDIR/$JAVA_TPF_FILENAME.tpf";
	    print  LOG "TPF $TPFNAME\n";
	}
	
	if ($JAVA_TSF_FILENAME eq "<default>")
	{
	    $TSFNAME = "$INTDIR/$NAME.txf";#DR 1552 to avoid mix between trace and purify.
	    print  LOG "TXF $TSFNAME\n";
	}
	else
	{
	    $TSFNAME = "$INTDIR/$JAVA_TSF_FILENAME.txf";#DR 1552 to avoid mix between trace and purify.
	    print  LOG "TXF $TSFNAME\n";
	}
    }
    close( LOG );
}
################################################################################
# BuildJvmpiArgs
# Arguments:
#   none
# Return value: 
#   args
################################################################################
sub BuildJvmpiArgs #($$)
{

  my ($prefix,$suffix) = @_;

  if ( ! $ATPMODE ) {
    # No JVMPI
    return "";
  }

  # Prefix
  my $jvmpi_args = "$prefix";
  # JVMPI Agent name
  $jvmpi_args .= "pagent:";

  # Here is the trace file
  $JPTNAME = "$INTDIR/$NAME.jpt";
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "JPT $JPTNAME\n";
  close( LOG );
  $jvmpi_args .= "-OUT=[\"$JPTNAME\"]";

  # Hashtable sizes
  $jvmpi_args .= "-H_C$JAVA_CLASS_HASH_SIZE-H_O$JAVA_OBJECT_HASH_SIZE";

  # Trigger
  my $needs_methods = 0;
  if ( $JAVA_TRIGGER eq "ALL_GC" ) {
    $jvmpi_args .= "-D_GC";
  } elsif ( $JAVA_TRIGGER eq "METHOD" ) {
    $needs_methods = 1;
  }

  if ( $needs_methods ) {

      my $dump_ = "";
      my $mlist = "";
      my $some = 0;
      $mlist .= "-D_M[";

      foreach $dump_ (@ATG_DUMP_INCOMING_LIST) {

#Each method will be:
#package .... name method.
	  my $Classe = "";
	  my $Method = "";
	  my $index= 0;
	  my $length = 0;
	  $Classe = $dump_;
	  #Then cut at the last dot.

	  $index = rindex($dump_,".");
	  $length = length($dump_);

	  if ($index != -1)
	  {
	      $Method=substr($dump_,($index+1));
	      $Classe=substr($dump_,0,($index));
	  }
	  else
	  {
	      die &PerlTranslate("Tools.pl", "Jvmpi args, Cannot find class and method for export.\n");	      
	  }

	  $index =  index($Method,"*");
	  if ( $index != -1 )
	  {
	      $Method = "";
	  }

	  $mlist .= "[$Method,$Classe,1]"; # Each method should be <method>,<class>
	  $some = 1;
      }

      foreach $dump_ (@ATG_DUMP_RETURNING_LIST) {

	  my $Classe = "";
	  my $Method = "";
	  my $index= 0;
	  my $length = 0;
	  $Classe = $dump_;
	  #Then cut at the last dot.

	  $index = rindex($dump_,".");
	  $length = length($dump_);

	  if ($index != -1)
	  {
	      $Method=substr($dump_,($index+1));
	      $Classe=substr($dump_,0,($index));
	  }
	  else
	  {
	      die &PerlTranslate("Tools.pl", "Jvmpi args, Cannot find class and method for export.\n");	      
	  }

	  $index =  index($Method,"*");
	  if ( $index != -1 )
	  {
	      $Method = "";
	  }


	  $mlist .= "[$Method,$Classe,0]"; # Each method should be <method>,<class>
	  $some = 1;
      }
    
      #And dumping methods in case of U_T mode.
      if ( $ATJMODE ) {
	  $mlist .= "[runBare,j2meunit.framework.TestCase,0]"; # Each method should be <method>,<class>
	  $mlist .= "[runBare,junit.framework.TestCase,0]"; # Each method should be <method>,<class>
	  $some = 1;
      }
    
      $mlist .= "]";

      if ($some) {
	  $jvmpi_args .= $mlist;
      } else {
	  #No dump methods, but test the U_T case.
	  if ( $ATJMODE )	{
	      $mlist .= "-D_M[";
	      $mlist .= "[runBare,j2meunit.framework.TestCase,0]"; # Each method should be <method>,<class>
	      $mlist .= "[runBare,junit.framework.TestCase,0]"; # Each method should be <method>,<class>
	      $mlist .= "]";
	      $jvmpi_args .= $mlist;
	  }
      }
  }
  if ( $JAVA_HIT_KEY ) {
    $jvmpi_args .= "-P_T[$JAVA_STUDIO_PORT_NB]-H_N[localhost]";
  }
  if ( $JAVA_NEXT_GC ) {
    $jvmpi_args .= "-D_PGC";
  }

  # Filters
  if ( scalar(@JAVA_PACKAGE_ANALYZE) > 0 ) 
  {
      $jvmpi_args .= "-P_M[";
      foreach $spy_ (@JAVA_PACKAGE_ANALYZE) 
      {
	  $jvmpi_args .= "[,$spy_]";
      }
      $jvmpi_args .= "]";
  }


  if ( scalar(@JAVA_ANALYZE) > 0 ) 
  {
      $jvmpi_args .= "-F_M[";
      foreach $spy_ (@JAVA_ANALYZE) 
      {
	  
 	  my $Classe = "";
	  my $Method = "";
	  my $index= 0;
	  my $length = 0;
	  $Classe = $spy_;
	  #Then cut at the last dot.

	  $index = rindex($spy_,".");
	  $length = length($spy_);
	  
	  if ($index != -1)
	  {
	      $Method=substr($spy_,($index+1));
	      $Classe=substr($spy_,0,($index));
	  }
	  else
	  {
	      die &PerlTranslate("Tools.pl", "Jvmpi args, Cannot find class and method.\n");	      
	  }

	  $index = index($Method,"*");
	  if ( $index != -1)
	  {
	      $Method = "";
	  }
	  
	  $jvmpi_args .= "[$Method,$Classe]"; # Each method should be <method>,<class>,
	  # or ,<class> for all methods in this class

	  
	  
      }
      $jvmpi_args .= "]";
      # @JAVA_ANALYZE_ALLOC should be a sublist of @JAVA_ANALYZE
      if ( scalar(@JAVA_ANALYZE_ALLOC) > 0 ) {
	  $jvmpi_args .= "-O_M[";
	  foreach $spy_ (@JAVA_ANALYZE_ALLOC) {
	      
	      my $Classe = "";
	      my $Method = "";
	      my $index= 0;
	      my $length = 0;
	      $Classe = $spy_;
	      #Then cut at the last dot.
	      
	      $index = rindex($spy_,".");
	      $length = length($spy_);
	      
	      if ($index != -1)
	      {
		  $Method=substr($spy_,($index+1));
		  $Classe=substr($spy_,0,($index));
	      }
	      else
	      {
		  die &PerlTranslate("Tools.pl", "Jvmpi args, Cannot find class and method.\n");	      
	      }
	      
	      $jvmpi_args .= "[$Method,$Classe]"; # Each method should be <method>,<class>,
	      # or ,<class> for all methods in this class
	      
	  }
	  $jvmpi_args .= "]";
      }
  }

  if ( $JAVA_FILTER_OPTION eq "WITH" ) {
    $jvmpi_args .= "-D_O_N"; # With Objects
  } else {
    $jvmpi_args .= "-N_O"; # Without Objects
  }

  # Mode Component Testing for Java
  if ( $ATJMODE ) {
    $jvmpi_args .= "-U_T";
  }

  # User Name
  if ( $USERNAME ) {
    $jvmpi_args .= "-U_S[\"$USERNAME\"]";
  }

  # Date
  my $date = GetDate();
  $jvmpi_args .= "-D_U[\"$date\"]";

  # Node Name
  $jvmpi_args .= "-N_U[\"$NAME\"]"; #TBD

  # Suffix
  $jvmpi_args .= $suffix;

  return $jvmpi_args;
}

################################################################################
# GetJvmpiPath
# Arguments:
#   none
# Return value: 
#   path
################################################################################
sub GetJvmpiPath
{
  my $jvmpi_lib = ($OSNAME eq "win32") ? "pagent.dll" : "libpagent.so.1";
  my $jvmpi_path = "$TARGETDIR" . $PATH_SEPARATOR . "lib" . $PATH_SEPARATOR . "jvmpi";
  my $jvmpi_complete = $jvmpi_path . $PATH_SEPARATOR . $jvmpi_lib;
  if ( -f $jvmpi_complete ) {
    # It has been recompiled by the user
    return $jvmpi_path;
  }
  # It has not been recompiled by the user: use the one that is delivered
  return $BINDIR;
}

################################################################################
# atc_postpro
# Arguments:
#   $tiofile and $ciofile
# Return value:
#   result of command
################################################################################
sub atc_postpro #($$)
{
  my ($tiofile,$ciofile) = @_;

  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  # Unit Testing needs this file
  my $cio_option = "";
  if ($ciofile ne "")
  {
    $cio_option = " -cio=\"$ciofile\"";
  
    ## Do not forget the CIO file in output log
    print  LOG "DUM $ciofile\n";
  }
  print  LOG "DUM $NAME.atc.txt\n";
  close( LOG );

  my @fdc = map { '"' . $_ . '"' } @FDCFILES;

  if($TPM_GEN){
    $ENV{'ATTOL_TPM_USER'}=$USERNAME;
    $ENV{'ATTOL_TPM_ROOT'}=$ATTOL_TPM_ROOT; 
  }
  # command
  my $Command = "$BINDIR" . "$PATH_SEPARATOR" . "attolcov" .
                " @fdc" . ((-f $tiofile) ? " \"$tiofile\"" : "") .
                " -su=c" . " -output=\"$NAME.atc.txt\"" .
                $cio_option;

  # execution
  $Status = SystemP ($Command);
  #$Status && die &PerlTranslate("Tools.pl", "Failed.\n");
  if ($Status) {
	$ATC_REPORT_FAILED=1;
  }

  #print "Click here to get the simple Coverage Report $NAME.atc.txt\n";
  #WARN: this $NAME.atc.txt is not translated into local language
  return 0;
}

################################################################################
# TDPAdaNeedAttolCfg
# Arguments:
#   none
# Return value: 
#   1 if attol_cfg.ads was generated, 0 in other case.
################################################################################
sub TDPAdaNeedAttolCfg
{
  #fixed here, but changed as ask by settings by SplitTraces function
  my $tio_file_name = $ATL_OUT_FILE;
  my $nb_bit_branch = 31;

  if ( $ATC_INFORMATION == 1 )
  {
    $nb_bit_branch=1;
  }

  my $need_attol_cfg = 0;

  my $attol_cfg = "$INTDIR/attol_cfg.ads" ;

  if ( -f $attol_cfg )
  {
    my $atc_nb_bit_branch = 0;
    my $atc_TIO_file_name = "";

    open( RCFG, "<$attol_cfg" );
    while(<RCFG>)
    {
      chomp;
      if ( /^\s*atc_nb_bit_branch/ )
      {
        my $tmp = $_;
        $tmp =~ s/^\s*atc_nb_bit_branch.*:=//;
        $tmp =~ s/\s*;\s*$// ;
        $atc_nb_bit_branch = $tmp;
        $need_attol_cfg = ( $atc_nb_bit_branch != $nb_bit_branch );
      }
      if ( /^\s*atc_TIO_file_name/ )
      {
        $_ =~ s/^\s*atc_TIO_file_name[^\"]*\"// ;
        $_ =~ s/\".*$// ;
        $atc_TIO_file_name = $_;
        $need_attol_cfg = ( $atc_TIO_file_name ne $tio_file_name );
      }

      last if ( $need_attol_cfg );
    }
    close(RCFG);
    
  }
  else
  {
    $need_attol_cfg = 1;
  }

  return $need_attol_cfg;
}

################################################################################
# TDPAdaGenerateAttolCfg
#
# Arguments:
#   $language: ADA for ada95 TDP, ADA83 for ada83 TDP
# 
# Return value: 
#   none
################################################################################
sub TDPAdaGenerateAttolCfg
{
  my ($language)    = @_ ;
  my $nb_bit_branch = 31;

  if ( $ATC_INFORMATION == 1 )
  {
    $nb_bit_branch=1;
  }

  my $attol_cfg = "$INTDIR/attol_cfg.ads" ;

  my $len = length($TIO_FILE_NAME);
  open  ( CFG, ">$attol_cfg") || die &PerlTranslate("Tools.pl", "cannot open %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $attol_cfg, $!);

  print ( CFG "-- This is a generated package, do not edit\n");
  print ( CFG "package attol_cfg is\n" );
  print ( CFG " pragma preelaborate;\n" ) if($Ini{INSTRUM_OPT} =~ /INIT_UNSUPPORTED/);
  print ( CFG "  atc_nb_bit_branch : constant integer := $nb_bit_branch;\n");
  my $atl_out_file = "$ATL_OUT_FILE";
  if ( $Ini{'NOPATH'} || $Ini{'NOPATHOUT'} ) {
  	my ($dir, $base, $ext) = SplitFileName ($atl_out_file);
  	$atl_out_file = "$base.$ext";
  } elsif ($Ini{'ABSPATHOUT'} ) {
  	my $pwd=GetCurrentDir();
  	$atl_out_file=$pwd.$PATH_SEPARATOR.$atl_out_file;
  }
  $atl_out_length=length($atl_out_file);
  print ( CFG "  atc_TIO_file_name : constant string(1.." . $atl_out_length . ") := \"$atl_out_file\";\n");  print ( CFG "end attol_cfg;\n");

  close ( CFG );
}

################################################################################
# TDPAdaGenerate
# Arguments:
#   $language: ADA for ada95 TDP, ADA83 for ada83 TDP
# Return value: 
#   none
################################################################################
sub TDPAdaGenerate
{
  my ($language)    = @_ ;
  my $tdpada_inf  = "$INTDIR/tdpada.inf" ;
  my $need_update = 0;
  my $intext="ali";
     $intext=$Ini{'INTEXT'} if($Ini{'INTEXT'});

  #log opied files
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);

  #'attol_cfg.ads' depends on studio settings only
  print LOG "ADA $INTDIR/attol_cfg.ads\n";
  print LOG "DUM $INTDIR/attol_cfg.$intext\n";
  print LOG "OBJ $INTDIR/attol_cfg.$OBJEXT\n";
  if ( TDPAdaNeedAttolCfg() )
  {
    TDPAdaGenerateAttolCfg( $language );
  }

  #do I generate again TDP ?
  if ( ! -f $tdpada_inf ) 
  {
    $need_update = 1;
  }
  else
  {
    # this file contains Generated TDP Ada configuration
    open( INF, "<$tdpada_inf" )
        or die &PerlTranslate("Tools.pl", "Error: cannot open file '%1': %2", "%1 is the name of the file and %2 is the reason given by the Operating System", $tdpada_inf, $!);
    while(<INF>)
    {
      chomp;
      if ( /^ADA=([A-Z0-9]+)/ &&  $language ne $1 ) { $need_update=2; last; }
      if ( /^ATC=(\d+)/       && ! $ATCMODE == $1 ) { $need_update=3; last; }
      if ( /^ATU=(\d+)/       && ! $ATUMODE == $1 ) { $need_update=4; last; }
    }
    close(INF)
  }

  print LOG "DUM $tdpada_inf\n";

  #verbose/debug mode

  unless ( $need_update )
  {
    print &PerlTranslate("Tools.pl", "TDP Ada files up to date\n") if ( $ENV{ATTOLSTUDIO_VERBOSE} );
    return ;
  }
  print &PerlTranslate("Tools.pl", "Updating TDP Ada files (%1)\n", "%1 is the value that needs to be written... sorry!", $need_update) if ( $ENV{ATTOLSTUDIO_VERBOSE} );

  #start TDP generation by the information file:
  open(  INF, ">$tdpada_inf" )
                or die &PerlTranslate("Tools.pl", "Error: cannot open file '%1': %2", "%1 is the name of the file and %2 is the reason given by the Operating System", $tdpada_inf, $!);
  print  INF "ADA=$language\n" ;
  print  INF "ATC=$ATCMODE\n" ;
  print  INF "ATU=$ATUMODE\n" ;
  print  INF "ADA $INTDIR/attol_cfg.ads\n";

  #generate TDP files:
  my $attol_link="$TARGETDIR/lib/attol_link.adb";
  my $otherfiles;
  if ( $language eq "ADA83" )
  {
    if ( $ATUMODE )
    {
      atl_copy( "$TARGETDIR/lib/attol.ads", "$INTDIR/attol.ads", 0 );
      atl_copy( "$TARGETDIR/lib/attol.adb", "$INTDIR/attol.adb", 0 );
      atl_copy( "$TARGETDIR/lib/ada83/attol_tag.ads", "$INTDIR/attol_tag.ads" , 0);
      atl_copy( "$TARGETDIR/lib/ada83/attol_tag.adb", "$INTDIR/attol_tag.adb", 0 );
      
      print INF "ADA $INTDIR/attol.ads\n";
      print INF "ADA $INTDIR/attol.adb\n";
      print INF "ADA $INTDIR/attol_tag.ads\n";
      print INF "ADA $INTDIR/attol_tag.adb\n";

      print LOG "ADA $INTDIR/attol.ads\n";
      print LOG "ADA $INTDIR/attol.adb\n";
      print LOG "DUM $INTDIR/attol.$intext\n";
      print LOG "OBJ $INTDIR/attol.$OBJEXT\n";
      print LOG "ADA $INTDIR/attol_tag.ads\n";
      print LOG "ADA $INTDIR/attol_tag.adb\n";
      print LOG "DUM $INTDIR/attol_tag.$intext\n";
      print LOG "OBJ $INTDIR/attol_tag.$OBJEXT\n";

      if ( $ATCMODE ) { $attol_link="$TARGETDIR/lib/ada83/attol_link.adb"; }
    }
    if ( $ATCMODE )
    {
      atl_copy( "$TARGETDIR/lib/ada83/attolcov.ads", "$INTDIR/attolcov.ads", 0 );
      atl_copy( "$TARGETDIR/lib/ada83/attolcov.adb", "$INTDIR/attolcov.adb", 0 );
      print INF "ADA $INTDIR/attolcov.ads\n";
      print INF "ADA $INTDIR/attolcov.adb\n";
      
      print LOG "ADA $INTDIR/attolcov.ads\n";
      print LOG "ADA $INTDIR/attolcov.adb\n";
      print LOG "DUM $INTDIR/attolcov.$intext\n";
      print LOG "OBJ $INTDIR/attolcov.$OBJEXT\n";
    }
    $otherfiles="$TARGETDIR/lib/ada83/Files.lst" if(-f "$TARGETDIR/lib/ada83/Files.lst");
  }
  elsif ( $language=~/ADA/ )
  {
    if ( $ATUMODE )
    {
      atl_copy( "$TARGETDIR/lib/attol.ads", "$INTDIR/attol.ads", 0 );
      atl_copy( "$TARGETDIR/lib/attol.adb", "$INTDIR/attol.adb", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/attol_tag.ads", "$INTDIR/attol_tag.ads", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/attol_tag.adb", "$INTDIR/attol_tag.adb", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/attol_serv_95.ads", "$INTDIR/attol_serv_95.ads", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/attol_serv_95.adb", "$INTDIR/attol_serv_95.adb", 0 );
      print INF "ADA $INTDIR/attol.ads\n";
      print INF "ADA $INTDIR/attol.adb\n";
      print INF "ADA $INTDIR/attol_tag.ads\n";
      print INF "ADA $INTDIR/attol_tag.adb\n";
      print INF "ADA $INTDIR/attol_serv_95.ads\n";
      print INF "ADA $INTDIR/attol_serv_95.adb\n";

      print LOG "ADA $INTDIR/attol.ads\n";
      print LOG "ADA $INTDIR/attol.adb\n";
      print LOG "DUM $INTDIR/attol.$intext\n";
      print LOG "OBJ $INTDIR/attol.$OBJEXT\n";
      print LOG "ADA $INTDIR/attol_tag.ads\n";
      print LOG "ADA $INTDIR/attol_tag.adb\n";
      print LOG "DUM $INTDIR/attol_tag.$intext\n";
      print LOG "OBJ $INTDIR/attol_tag.$OBJEXT\n";
      print LOG "ADA $INTDIR/attol_serv_95.ads\n";
      print LOG "ADA $INTDIR/attol_serv_95.adb\n";
      print LOG "DUM $INTDIR/attol_serv_95.$intext\n";
      print LOG "OBJ $INTDIR/attol_serv_95.$OBJEXT\n";

      if ( $ATCMODE ) { $attol_link="$TARGETDIR/lib/ada95/attol_link.adb"; }
    }
    if ( $ATCMODE )
    {
      atl_copy( "$TARGETDIR/lib/ada95/atw.ads",          "$INTDIR/atw.ads", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/atw-coverage.ads", "$INTDIR/atw-coverage.ads", 0 );
      atl_copy( "$TARGETDIR/lib/ada95/atw-coverage.adb", "$INTDIR/atw-coverage.adb", 0 );
      print INF "ADA $INTDIR/atw.ads\n";
      print INF "ADA $INTDIR/atw-coverage.ads\n";
      print INF "ADA $INTDIR/atw-coverage.adb\n";
      
      print LOG "ADA $INTDIR/atw.ads\n";
      print LOG "DUM $INTDIR/atw.$intext\n";
      print LOG "OBJ $INTDIR/atw.o\n";
      print LOG "ADA $INTDIR/atw-coverage.ads\n";
      print LOG "ADA $INTDIR/atw-coverage.adb\n";
      print LOG "DUM $INTDIR/atw-coverage.$intext\n";
      print LOG "OBJ $INTDIR/atw-coverage.$OBJEXT\n";
    }
    $otherfiles="$TARGETDIR/lib/ada95/Files.lst" if(-f "$TARGETDIR/lib/ada95/Files.lst") ;
  }
  
  # common files
  atl_copy( "$TARGETDIR/lib/attol_link.ads", "$INTDIR/attol_link.ads", 0 );
  atl_copy( "$attol_link", "$INTDIR/attol_link.adb", 1 );
  atl_copy( "$TARGETDIR/lib/attol_serv.ads", "$INTDIR/attol_serv.ads", 0 );
  atl_copy( "$TARGETDIR/lib/attol_serv.adb", "$INTDIR/attol_serv.adb", 0 );
  atl_copy( "$TARGETDIR/lib/attol_comm.ads", "$INTDIR/attol_comm.ads", 0 );
  atl_copy( "$TARGETDIR/lib/attol_comm.adb", "$INTDIR/attol_comm.adb", 0 );
  atl_copy( "$TARGETDIR/lib/private_io.ads", "$INTDIR/private_io.ads", 0 );
  atl_copy( "$TARGETDIR/lib/private_io.adb", "$INTDIR/private_io.adb", 0 );
  print INF "ADA $INTDIR/attol_link.ads\n";
  print INF "ADA $INTDIR/attol_link.adb\n";
  print INF "ADA $INTDIR/attol_serv.ads\n";
  print INF "ADA $INTDIR/attol_serv.adb\n";
  print INF "ADA $INTDIR/attol_comm.ads\n";
  print INF "ADA $INTDIR/attol_comm.adb\n";
  print INF "ADA $INTDIR/private_io.ads\n";
  print INF "ADA $INTDIR/private_io.adb\n";

  print LOG "ADA $INTDIR/attol_link.ads\n";
  print LOG "ADA $INTDIR/attol_link.adb\n";
  print LOG "DUM $INTDIR/attol_link.$intext\n";
  print LOG "OBJ $INTDIR/attol_link.$OBJEXT\n";
  print LOG "ADA $INTDIR/attol_serv.ads\n";
  print LOG "ADA $INTDIR/attol_serv.adb\n";
  print LOG "DUM $INTDIR/attol_serv.$intext\n";
  print LOG "OBJ $INTDIR/attol_serv.$OBJEXT\n";
  print LOG "ADA $INTDIR/attol_comm.ads\n";
  print LOG "ADA $INTDIR/attol_comm.adb\n";
  print LOG "DUM $INTDIR/attol_comm.$intext\n";
  print LOG "OBJ $INTDIR/attol_comm.$OBJEXT\n";
  print LOG "ADA $INTDIR/private_io.ads\n";
  print LOG "ADA $INTDIR/private_io.adb\n";
  print LOG "DUM $INTDIR/private_io.$intext\n";
  print LOG "OBJ $INTDIR/private_io.$OBJEXT\n";

  if($otherfiles) {
  	my $d="ada95";
  	$d="ada83" if ( $language eq "ADA83" );

	open LST, "$otherfiles" || die &PerlTranslate("Tools.pl", "cannot open file $otherfiles", "");
	while(<LST>) {
	     chomp;
#	     print( "copy $TARGETDIR/lib/$d/$_.ad?","$INTDIR/$_.ad?\n");
	     atl_copy( "$TARGETDIR/lib/$d/$_.ads","$INTDIR/$_.ads",0);
	     atl_copy( "$TARGETDIR/lib/$d/$_.adb","$INTDIR/$_.adb",0);
	     print INF "ADA $INTDIR/$_.ads\n";
	     print INF "ADA $INTDIR/$_.adb\n";

	     print LOG "ADA $INTDIR/$_.ads\n";
	     print LOG "ADA $INTDIR/$_.adb\n";
	     print LOG "DUM $INTDIR/$_.$intext\n";
	     print LOG "OBJ $INTDIR/$_.$OBJEXT\n";
	}
	close LST;
  }
  close( LOG );
  close( INF );

}

################################################################################
# MakeCompilationOptionsFile
# Arguments:
#   none
# Return value: 
#   none
################################################################################
sub MakeCompilationOptionsFile
{
   $model = "$TARGETDIR/lib/options_model.h";
   $options = "$INTDIR/options.h";

   MakeCProductsHeaderFile ($model, $options);
   return 0;
}

################################################################################
# PostAttManage
# Arguments:
#   $tdffile
# Return value: 
#   none
################################################################################
sub PostAttManage #($)
{
  my $tdffile = shift;

  # $ATTMODE has yet been checked to be 1 by the caller

  my $attsplit = $BINDIR . $PATH_SEPARATOR . "attsplit";
  my $attloop  = $BINDIR . $PATH_SEPARATOR . "attloop";

  if ( $ATT_SPLIT == 1 )
  {
    if ( ! -e $attsplit && ! -e "$attsplit.exe" )
    {
      die &PerlTranslate("Tools.pl", "Error: attsplit binary not found.\n");
    }
    if ( $ATT_LOOP && !-e $attloop && ! -e "$attloop.exe" )
    {
      die &PerlTranslate("Tools.pl", "Error: attloop binary not found.\n");
    }

    my $tcf = $ATTOLHOME . "/lib/tracer.tcf";
    my @tsf = map { '"' . $_ . '"' } @TSFFILES;

    my $splitlog = $INTDIR . "/attsplit.log" ;
    unlink( $splitlog );

    $status = SystemP( "$attsplit \"$tcf\" @tsf \"$tdffile\" -p $ATT_SPLIT_PREFIX -s $ATT_SPLIT_SIZE -vw -studio_log $splitlog ");
    if ( $status != 0 ) { exit (1); }

    open(  LOG, ">>$LOGFILENAME") or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
    open( SLOG, "$splitlog"     ) or die &PerlTranslate("Tools.pl", "Cannot open file %1: %2\n", "%1 is the name of the file and %2 is the reason given by the Operating System", $splitlog, $!);

    #loop detection ?
    while ( <SLOG> )
    {
      if ( /^TDF / )
      {
        my $_tdf ="";

        ($_tdf) = ($_ =~ /^TDF (.*)$/);
        if ( $ATT_LOOP == 1 )
        {
          print &PerlTranslate("Tools.pl", "%1: searching for loops...\n", "%1 is the name of the file in which loops are searched", $_tdf);
          $status = SystemP( "$attloop -o \"$_tdf\" \"$tcf\" \"$_tdf\" @tsf $ATT_LOOP_FLAGS" );
          if ( $status != 0 ) { exit (1); }
        }
        print LOG "TDF $_tdf\n";
      }
    }

    close( LOG );
    close( SLOG );

  }
  else
  {
    #-------------------------------------------------------------------------
    ## Loop detection inside generated TDF file ...
    if ( $ATT_LOOP == 1 )
    {
      if ( ! -e $attloop && ! -e "$attloop.exe" )
      {
        die &PerlTranslate("Tools.pl", "Error: attloop binary not found.\n");
      }

      #TBD: reste a savoir
      my $tcf = $ATTOLHOME . "/lib/tracer.tcf";
      my @tsf = map { '"' . $_ . '"' } @TSFFILES;

      print &PerlTranslate("Tools.pl", "%1: searching for loops...\n", "%1 is the name of the file in which loops are searched", $tdffile);
      $status = SystemP( "$attloop -o \"$tdffile\" \"$tcf\" \"$tdffile\" @tsf $ATT_LOOP_FLAGS" );
      if ( $status != 0 ) { exit (1); }
    }

    #TBD: ??? nom fic. TDF si utilisation de ATTOL_TRACES
    open(  LOG, ">>$LOGFILENAME") or die &PerlTranslate("Tools.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
    print  LOG "TDF $tdffile\n";
    close( LOG );
  }
}

###########################################################################
# PutMemoryProfilingStatus
# This routine will print the eventual message corresponding to signal
# received if any
###########################################################################
sub PutMemoryProfilingStatus #($$)
{
        my ($ret,$pout) = @_ ;

        if ( $pout ) {
          $pout = open POUT,">>$pout";
          if ( $OSNAME ne "win32" && -f "core" ) {
            print POUT "PE COR\n";
          }
          print POUT "PX $ret\n";
          close POUT;
        }
}

################################################################################
# Run Tester or Application
################################################################################
sub AtlExec #($$$)
{
  my ($prg,$out,$arg) = @_;
  my $status;
  my $useExitCode=0;

  unlink($INTDIR."/atlendtest.txt"); #file generated by the UI split on "tdp end" token found in the .spt 
  $SOCKET_UPLOAD = 0;
  if ( lc($Ini{'SOCKET_UPLOAD'}) eq "true" ) {
    atl_socket_receiver($Ini{'SOCKET_UPLOAD_DELIMITER'},$UPLOADER_PORT);
    if($Ini{'SOCKET_UPLOAD_START_TIME'}) {sleep($Ini{'SOCKET_UPLOAD_START_TIME'});}
    $SOCKET_UPLOAD = 1;
  }

  #---------------------------------------------------------------------------
  ## Memory Profiling preparation...
  if ( lc($Ini{'EXIT_CODE_IN_MEMORY_PROFILE'}) eq "on" &&
       ! $SOCKET_UPLOAD &&
       $ATPMODE &&
       $TDP_LANGUAGE ne "JAVA" ) {
    $useExitCode = 1;
    if ( $OSNAME ne "win32" ) {
      unlink("core");
    }
  }

  if (!$EXECMODE || $EXECMODE eq "NORMAL") {
    $status = atl_exec ($prg,$out,$arg);
  } else {
    $useExitCode = 0;
    $status = atl_execdbg ($prg,$out,$arg);
  }

  #---------------------------------------------------------------------------
  ## Memory Profiling record exit code...
  if ( $useExitCode) {
    PutMemoryProfilingStatus($status,$out) unless ($ATL_MULTI_PROCESS);
  }
  
  #---------------------------------------------------------------------------
  ## Memory Profiling java part for the create files.
  if ( $ATPMODE && $TDP_LANGUAGE eq "JAVA" ) 
  {
    #Log is made according to files already created,
    #not according to the result of the execution.
    BuildJvmpiOutFiles();
  }

  return $status;
}

1;
################################################################################

