#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                        -*-  Unit Testing node -*-                            #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   @OBJECTS     : object files                                                #
#   $LDFLAGS     : link flags                                                  #
#   $LIBRARIES   : libraries list for linking                                  #
#   $TESTPROGRAM : executable name                                             #
#   $ARGUMENTS   : executable arguments                                        #
#   $EXECMODE    : execution mode ("NORMAL" or "DEBUG")                        #
#   $AMLFILE     : generated AML report                                        #
#   @OTSFILES    : list of OTS files                                           #
#   $BINDIR      : ATTOL binaries directory                                    #
#   $SCRIPTDIR   : Studio scripts directory                                    #
#   $TARGETDIR   : target package directory                                    #
#   $OUTDIR      : output directory                                            #
#   $LOGFILENAME : generated files log                                         #
################################################################################




# To do output in chronological order when stderr and stdout are both used
use FileHandle;
use File::Copy ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0] unless($main::TestRTCC);

# Miscellaneous functions 
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";

# Target package related functions
require $SCRIPTDIR . "Products.pl";
require $SCRIPTDIR . "codeChecker.pl" if($MISRA_MODE);

# Target-dependant functions
require $TARGETDIR ."/cmd/atl_link.pl";
require $TARGETDIR ."/cmd/atl_exec.pl";


################################################################################
# Attol Unit Testing postprocessor execution
# Arguments:
#    $atu_out    : traces file
#    $atu_xrd    : generated AML report
# Return value: attolpostpro return value
# Required global variables:
#    $BINDIR
# Used global variables:
################################################################################
sub atu_postpro #($$$)
{
  my ($atu_out,$atu_xrd,$atu_cio) = @_;

  # Suppress old features
  delete $ENV{'ATTOLLAN'} ;
  delete $ENV{'ATTOLCINC'} ;
  delete $ENV{'ATTOLTMPK'} ;
  delete $ENV{'ATTOLCPPINC'} ;
  delete $ENV{'ATW_NO_USR_INCLUDE'} ;

  # Let's environment
  $ENV{'ATTOLHOME'} = $ATTOLHOME;
  $ENV{'ATUDIR'} = $ATTOLHOME . $PATH_SEPARATOR."lib";
  $ENV{'ATUTGT'} = "$TARGETDIR";

  # Defaults options
  my $atu_postpro_name = "attolpostpro";
  my $atu_options = " ";
  my $ada_mode = 0;

  if ($TDP_LANGUAGE=~/ADA/)
  {
    $ada_mode = 1;
  }
  if ($LOGFILENAME)
  {
    $atu_options .= " -studio_log=\"$LOGFILENAME\"";
  }

  # Update options

  if (($TDP_LANGUAGE eq "C") || ($TDP_LANGUAGE eq "CPP") || ($TDP_LANGUAGE eq "ADA83_V34"))
  {
     $atu_postpro_name = "attolpostpro";
  }
  if ($TDP_LANGUAGE=~/ADA/)
  {
     $atu_postpro_name = "attolpostproada";
  }

  if ($ATCMODE && !$ATC_REPORT_FAILED && !$NO_COV_REPORT)
  {
    $atu_options .= " -cio=\"$atu_cio\"";
  }

  my $ATU_RI2 = "$PROJECT_PATH"."$PATH_SEPARATOR"."$NAME.ri2";
  if (($ATU_COMPARE) || ($ATU_REPORT_OPTIONS=~ /-compare/))
  {
    if (-f "$ATU_RI2")
    {
       ($dev1,$ino1,$mode1,$nlink1,$uid1,$gid1,$rdev1,$size1, $atime1,$mtime1,$ctime1,$blksize1,$blocks1) = stat($ATU_RI2);
       ($dev2,$ino2,$mode2,$nlink2,$uid2,$gid2,$rdev2,$size2, $atime2,$mtime2,$ctime2,$blksize2,$blocks2) = stat($UNITTESTING_TESTSCRIPTNAME);
       if ($mtime1 lt $mtime2)
       {
	       die &PerlTranslate("UnitTesting.pl", "Error: Unable to diff with file %1 (test script has been modified)\n       Please re-build without having the 'Diff of last two test runs' option turned on.\n", "%1 is the name of the file", "$ATU_RI2");
       }
       atl_copy("$ATU_RI2", "$atu_out.ri2", 1);
       $atu_options .= " -compare=strict" if ($ATU_COMPARE) ;
    }
    else
    {
       die &PerlTranslate("UnitTesting.pl", "Error: Unable to diff with file %1 (not found)\n       Please re-build without having the 'Diff of last two test runs' option turned on.\n", "%1 is the name of the file", "$ATU_RI2");
    }
  }
  else
  {
    atl_copy("$atu_out.rio", $ATU_RI2, 1);
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("UnitTesting.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
    print  LOG "RI2 $ATU_RI2\n";
    close( LOG );
  }

  if ($ATU_FILTER == 1)
  {
      $atu_options .= " -RA=ERR";
  }
  elsif ($ATU_FILTER == 2)
  {
      $atu_options .= " -RA=TEST";
  }
  elsif ($ATU_FILTER == 3)
  {
      $atu_options .= " -RA=COLLAPSE";
  }

  if ($ATU_STRATEGY == 0)
  {
      $atu_options .= " -VA=EVAL";
  }
  elsif ($ATU_STRATEGY == 1)
  {
      $atu_options .= " -VA=NOEVAL";
  }
  elsif ($ATU_STRATEGY == 2)
  {
      $atu_options .= " -VA=COMBINED";
  }

  # Postpro execution
  unlink ( "$atu_out.rod" );

  my $atu_command = $BINDIR . $PATH_SEPARATOR . $atu_postpro_name . " \"$atu_out\" \"$atu_out.rod\" $atu_options" ." ".$ATU_REPORT_OPTIONS;
  my $postpro_status = SystemP($atu_command);

  if ( ! -f "$atu_out.rod" )
  {
      die &PerlTranslate("UnitTesting.pl", "No result generated.\n");
  }

  if($TPM_GEN){
     # TPM initialization
     $ENV{'ATTOL_TPM_USER'}=$USERNAME;
     $ENV{'ATTOL_TPM_ROOT'}=$ATTOL_TPM_ROOT;
  }
  # Graphic Generation
  my $rtx_opt = "";  
  if ($MATLAB || $ATU_GRAPHICS || $ENV{'ATURTX'})
  {
     $rtx_opt = " -g ";  
  } 

  if ($ATU_SPLIT_REPORT)
  {
    $rtx_opt = $rtx_opt . " -s" . $ATU_SPLIT_REPORT;
  } 

  # Translator execution
  unlink ( "$atu_xrd" );

  my $headOption;
  if(open (HDR,">$atu_out.hdr")){
     if($RTRT_ACTIVATED){ #=1 on eclipse
	print HDR "Project=$PROJECT_NAME\n";
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_PATH\n";
	print HDR "Test name=$NAME\n";
     }else{
	print HDR "Project=$PROJECT_NAME\n";
	print HDR "Project file=$PROJECT_PATH.rtp\n";
	print HDR "Project configuration=$CONFIGURATION\n";
	print HDR "Workspace=$WORKSPACE_NAME\n";
	print HDR "Test node=$NAME\n";
     }
#	print HDR "=$\n";
     close HDR;
     $headOption=" \"-h$atu_out.hdr\"";
  }

  if($Ini{'REPORTHEADER'}) {
     print "REPORTHEADER specified as:$TARGETDIR/cmd/$Ini{'REPORTHEADER'}\n";
     print "sub reportHeader($atu_out.hdr,$RTRT_ACTIVATED)\n";
     require "$TARGETDIR/cmd/$Ini{'REPORTHEADER'}";
     reportHeader("$atu_out.hdr",$RTRT_ACTIVATED);
  }

  my $atu_command = $BINDIR . $PATH_SEPARATOR .  "rod2xrd" . $rtx_opt . $headOption. " \"-l$LOGFILENAME\"" . " \"-o$atu_xrd\" \"$atu_out.rod\"" ;
  my $trans_status = SystemP ($atu_command);
  $trans_status && die &PerlTranslate("UnitTesting.pl", "Xrd Translation Failed.\n");

  # Matlab Translation
  if ($MATLAB)
  {
     # MAT Generation
     my $atu_command = $BINDIR . $PATH_SEPARATOR .  "rod2mat " . " \"$atu_out.rod\" \"$MATLAB_GENERATEDDATA\" \"$MATLAB_INFORMATIONFILE\"" ;
     my $trans_status = SystemP ($atu_command);
  }

  # add temporary files into log 
  unlink("$atu_out.hdr");
  return 0;
}

################################################################################
# Link, execute and postprocess for ATU node
sub UnitTesting
{
my $status = 0;
my $ada_mode = 0;
my $OUTFILE = $ATL_OUT_FILE;

  # Out Files $RIOFILE is the var used by the Splitter
  if ($INTDIR)
  {
     $RIOFILE .= $INTDIR.$PATH_SEPARATOR;
  }
  $RIOFILE .= $ATU_INTERMEDIATE_FILE.".rio";

  if ($TDP_LANGUAGE=~/ADA/)
  {
     $ada_mode = 1;
  }  
 
  if($MISRA_MODE && @XOB) {
   my $res;
   if ($STAGE =~ /e/ ) {
      $res=codeChecker_link() if($STAGE =~ /l/ );
   }else{
      $res=codeChecker_link();
   }
  }

#----------------------------------------------------------------------------
## Linkage
#----------------------------------------------------------------------------
 if ($STAGE =~ /l/)
 {
  if ($ada_mode)
  { 
      my $cwdir = $WORKING_DIR;
      $cwdir=~s/\//$PATH_SEPARATOR/g;
      my $save_ctx = $ATL_MAIN_PROCEDURE;


      $ATL_MAIN_PROCEDURE = lc($ATU_INTERMEDIATE_FILE);
      $ATL_MAIN_PROCEDURE = clean_name_from_exotic_char($ATL_MAIN_PROCEDURE);
      $status = atl_link ($TESTPROGRAM_ABS, \@OBJECTS, $LDFLAGS,\@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);

      $ATL_MAIN_PROCEDURE = $save_ctx;
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
  }
  else
  {
      #----------------------------------------------------------------------------
      if ( $ATL_RUNTIME_FORMAT ne "NONE") {
         ## do not forget object file of target package.
         if (MakeTargetPackageObject()) {
  	   return 1;
         }
         push( @OBJECTS, GetTargetPackageLib($OBJEXT));
      }

      print &PerlTranslate("UnitTesting.pl", "Linking %1...\n", "%1 is the name of the program to be linked", $TESTPROGRAM);

      $status = atl_link ($TESTPROGRAM, \@OBJECTS, $LDFLAGS,
                          \@LIBRARY_DIR_LIST, $LIBRARIES, \@ADDITIONAL_LIBRARIES);
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
  }

  ## Do not forget the OBJ file in output log
  open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("UnitTesting.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  print  LOG "EXE $TESTPROGRAM\n";
  close( LOG );
 }

#----------------------------------------------------------------------------
## Execution
#----------------------------------------------------------------------------
 if ($STAGE =~ /e/ )
 {

  if ($ada_mode)
  {
    print &PerlTranslate("UnitTesting.pl", "Executing %1...\n", "%1 is the program being executed", $TESTPROGRAM_ABS . $ARGUMENTS);
    $status = AtlExec ($TESTPROGRAM_ABS, $RIOFILE, $ARGUMENTS);
    RenameFile($RIOFILE,$OUTFILE);
  }
  else
  {
    $OUTFILE =~ s/\\/\\\\/;
    print &PerlTranslate("UnitTesting.pl", "Executing %1...\n", "%1 is the program being executed", $TESTPROGRAM . $ARGUMENTS);
    $status = AtlExec ($TESTPROGRAM, $OUTFILE, $ARGUMENTS);
  }

  EndStep($status);
  if ($status != 0 )
  {
    print &PerlTranslate("UnitTesting.pl", "Executable terminated with error code %1.\n", "%1 is the value of the error code", $status);
  }

  SplitTraces ($OUTFILE);

  unless (-f $RIOFILE )
  {
    print &PerlTranslate("UnitTesting.pl", "No trace %1 generated.\n", "%1 is the name of the trace file", $OUTFILE);
    open RIO,">".$RIOFILE; # create an empty file anyway
    close RIO;    
  }
  if ( ! $used[$rio] )
  {
    print &PerlTranslate("UnitTesting.pl", "No RIO instruction found.\n");
#    return(1);
  }
 }

#----------------------------------------------------------------------------
## Postprocessing
#----------------------------------------------------------------------------
 if ($STAGE =~ /r/ )
 {
  my $atu_cio = "$INTDIR".$PATH_SEPARATOR."atu.cio";
  print &PerlTranslate("UnitTesting.pl", "Postprocessing...\n");

  if (($ATCMODE) && (!$NO_COV_REPORT)) {
    if ( ! $used[$tio] )
    {
      print &PerlTranslate("UnitTesting.pl", "No TIO instruction found.\n");
      $ATC_REPORT_FAILED=1;
    }
    else
    {
      $status = atc_postpro ($ATC_TIO_NAME, $atu_cio);
      $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
    }
  }
  my $rio = $RIOFILE;
  $rio=~s/\.rio$//;
  $status = atu_postpro ($rio, $XRDFILE, $atu_cio);
  $status && die &PerlTranslate("UnitTesting.pl", "Failed.\n");
 }
 return 0;
}

################################################################################
# Here is the beginning
################################################################################

unless($main::TestRTCC){

CheckIntDir();
CheckOutDir();
EnterInScript();
ClearLogFile($LOGFILENAME);
exit (UnitTesting());
}

1;
#
################################################################################
