#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                -*- Update Unit Testing Script -*-                            #
#                                                                              #
#                                                                              #
#  Script update for Unit Testing                                              #
#                                                                              #
#                                                                              #
################################################################################
#$Id: UpdateUnitTestingScript.pl,v 1.20 2007/10/05 08:46:54 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# This script depends on the followings (both studio'n Target Package)
require $ARGV[0]  unless($main::TestRTCC); #environment and global variables.
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";

my $lastStubBlockEnds;
my $lastInclude;
my $stubName;

sub UpdateUnitTestingScript 
{
print PerlTranslate("UpdateUnitTesting.pl", "Test Script update of %1...\n", "%1 is the name of the Test Script", $FILENAME);

  my ($tested_dir, $tested_name, $tested_ext) = SplitFileName($FILENAME);
  my $tested_lang = LanguageOf( $tested_ext,$TDP_LANGUAGE );
  my $date=localtime();

  # Create build temporary directory if it does not exist
  if (! -d $INTDIR) { mkdir ($INTDIR,0777); }

  # Defaults options
  my $tmpFile="$INTDIR/atl_stubs.ptu";
  $stubName=$tested_name."_new";
  my @incPtu;
  my @stubsInPtu;
  my @srvisInPtu;
  # Get the existing stubs from the PTU 
  my $start = readPtu($FILENAME,\@incPtu,\@stubsInPtu,\@srvisInPtu);
  #print "existing stubs:",join(",",@stubsInPtu),"\n\n\n";
  #print "existing services:",join(",",@srvisInPtu),"\n\n\n";
  #print "existing decarations:",join(",",@incPtu),"\n\n\n";
  my $lineNextStub=$lastStubBlockEnds;
  my $lineNextIncl=$lastInclude;
  my $lan=0;#0=Ada, 1=C
  if($start=~ /$tested_name\.$tested_ext/){
	$start=~ s/\"[^\"]+\.ptu\"/$tmpFile/i;
	$start=~ s/[-\/]studio_log=\"[^\"]+\"//;
	$lan=1 if($start=~ /\.c\"/);
	$lan=1 if($start=~ s/[-\/]STD_DEFINE=\"[^\"]+\"//);
	$lan=1 if($start=~ s/^attolstart /attolstartC /);
	if($lan){
		$start.=" -STD_DEFINE=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."atus_c.def\"" ;
#	} elsif ($TDP_LANGUAGE=~ /^ADA83/) {
#		$start.=" -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."ada83.alk\"" ;
	} else {
#		$start.=" -STANDARD=\"$TARGETDIR".$PATH_SEPARATOR."ana".$PATH_SEPARATOR."ada95.alk\"" ;
	  print PerlTranslate("UpdateUnitTesting.pl", "No update possible\n");
	  exit(0);
	}
	my $status;
	  if($Ini{'ATTOLSTART'}) {
	     print "ATTOLSTART specified as:$TARGETDIR/cmd/$Ini{'ATTOLSTART'}\n";
	     print "sub attolStart() executed with $atu_command\n";
	     require "$TARGETDIR/cmd/$Ini{'ATTOLSTART'}";
		my $lang='C';
		$lang='ADA' unless($lan);
	     $status = attolStart($start,$lang,'*',$FILENAME);
	  }
	  else
	  {
	     $status = SystemP($start);
	  }

	  $status=0 if($status==3 && $CONTONWARN);
	  if ($status)
	  {
	     print PerlTranslate("UpdateUnitTesting.pl", "Generation failed.\n");
	     return ($status);
	  }
  		
	# Extract the stubs & services from generated PTU
	my @required_inc;
	my @required_stubs;
	my @required_srvis;
	readPtu($tmpFile,\@required_inc,\@required_stubs,\@required_srvis);
	#print "required stubs:",join(",",@required_stubs),"\n\n\n";
	#print "required service:",join(",",@required_srvis),"\n\n\n";
	#print "required decarations:",join(",",@required_inc),"\n\n\n";
	# Get the missing stuffs from the test script
	my @missing_stubs = getMissing(0,\@stubsInPtu, \@required_stubs);
	my @missing_srvis = getMissing(1,\@srvisInPtu, \@required_srvis);
	#missing include file, var name and function name are all listed in the uniq array
	#there is no confusion between these 3 types
	my @missing_inc   = getMissing(0,\@incPtu, \@required_inc);
	#print "miss ",join(",",@missing_inc),"\n\n\n";
	$lineNextStub=0 unless(scalar(@missing_stubs));
	$lineNextIncl=0 unless(scalar(@missing_inc));
	if($lineNextStub || $lineNextIncl){
		rename($FILENAME,$FILENAME.".bak");
		my $update=1;
		# Generate includes,declaration and Stub in script 
		open (IN, "<$FILENAME.bak") || die "Can not open the file \"$FILENAME.bak\" for read\n";
		open (OU, ">$FILENAME") || die "Can not open the file \"$FILENAME\" for writing\n";
		my $lineNo;
		while(<IN>) {
		    $lineNo ++;
		    if((/^(\s*--%d)/) && ($update)) {
		    	print OU "$1 Updated on ",GetDate(),"\n";
		    	$update=0;
		    }
		    print OU ;
		    if($lineNextIncl == $lineNo){
			    print OU "\n--";
			    print OU "\n-- Updated on ",GetDate(),": New statement found";
			    print PerlTranslate("UpdateUnitTesting.pl", "Generating statement:\n");
			    my $last,$curr;
			    foreach (@missing_inc) { 
			    	$curr="var";
			    	$curr="includes" if(/\#include\s+/);
			    	$curr="extern" if(/^extern\s+/);
			    	print OU "\n--\n" if($last ne $curr);
				print OU $_,"\n";
				print $_,"\n";
				$last=$curr;
				}
			    print OU "-- End of new statements\n";
		    
		    }elsif($lineNextStub == $lineNo){
			    print OU "\n-- Updated on ",GetDate(),": New stubs found\n";
			    print OU "DEFINE STUB $stubName\n";
			    print "Generating stubs:\n";
			    foreach (@missing_stubs) { 
				print OU $_,"\n";
				print $_,"\n";
				}
			    print OU "END DEFINE\n";
		    }
		}
		close OU;close IN;
	} else {
		print PerlTranslate("UpdateUnitTesting.pl", "No missing stubs found\n");
	}
	if(scalar(@missing_srvis)){
		open (IN, "<$tmpFile") || die "Can not open the file \"$tmpFile\" for read\n";
		open (OU, ">>$FILENAME") || die "Can not open the file \"$FILENAME\" for writing\n";
		print OU "\n-- new service found on $date\n";
		print PerlTranslate("UpdateUnitTesting.pl", "Generating new services: ");
		my $push;
		while(<IN>){
		    if(/^\s*(SERVICE\s*\w+)/) {
		      if(grep(/^$1$/,@missing_srvis)){
		    	$push=1; 
		    	print $1,",";
			print OU "--------------------------------------------------------------------------------\n";
			print OU "-- Declaration of the service $1\n";
			print OU "--------------------------------------------------------------------------------\n";
		      }
		    }
		    if($push) {
		    	chomp;
			print OU $_,"\n";
		    }
		    if(/^\s*END\s+SERVICE/) {
		    	$push=0;
		    }
		}
		print "\n";
	} else {
		print PerlTranslate("UpdateUnitTesting.pl", "No new services\n");
	}
	#unlink($tmpFile);
  }else{
  	print PerlTranslate("UpdateUnitTesting.pl", "TestRT-W-MISCOLIGE missing attolstart command line in PTU file $FILENAME: no update possible\n");
  }

  return 0;
}

###########################################################################

sub readPtu  #($\@\@\@)
{
my $ptu = shift;
my $inc_lines = shift;
my $stub_lines = shift;
my $sercices = shift;

  # Extract the stubs from generated PTU
  open (PTU, $ptu) || die PerlTranslate("UpdateUnitTesting.pl", "Can not open the file \"%1\" to extract the list of STUBs\n",$ptu);
  my $start;
  my $push = 0;
  my $lineNo = 0;
  my $HeadLine = 0;
  my $comment = 0;
  foreach (<PTU>) {
    chomp;
    $lineNo ++;
    $comment=1 if(/^\#\/\*/);
    if((/^\#/) && (/\*\/$/)) {
    	$comment=0;
    	next;
    }
    $comment=0 if(/^----/);
    next if($comment);
    if (/^\s*DEFINE\s+STUB/) {
      if(/^\s*DEFINE\s+STUB\s*(\w+)/){
      	if($1 eq $stubName){
      	   $stubName.="2";
      	}
      }
      $push = 1;
      $HeadLine=1;
    } elsif (/^\s*DEFINE\s+MUET/) {
      if(/^\s*DEFINE\s+MUET\s*(\w+)/){
      	if($1 eq $stubName){
      	   $stubName.="2";
      	}
      }
      $push = 1;
      $HeadLine=1;
    } elsif (/^\s*BEGIN/) {
      $lastStubBlockEnds=$lineNo; # new stub after BEGIN if not DEFINE STUB
    } elsif (/^\s*HEADER/) {
      $lastInclude=$lineNo; # include should come after HEADER line
    } elsif (/^\s*END\s+DEFINE/ && $push) {
      $lastStubBlockEnds=$lineNo;
      $push = 0;
      $HeadLine=0;
    } elsif (/^\s*FIN\s+DEFINE/ && $push) {
      $lastStubBlockEnds=$lineNo;
      $push = 0;
    } elsif (/^\s*SERVICE\s+(\w+)/) {
      $HeadLine=1;
      push (@$sercices, $_);
    } elsif ($push) {
      push (@$stub_lines, $_) if(/^\s*\#/);
    } elsif ($HeadLine ==0) {
      push (@$inc_lines, $_) if(/^\s*\#/);
      $lastInclude=$lineNo if (/^\s*\#\#include\s+/)
    } 
    if (/^--%c\s*(.+)$/) {
      # Is it the attolstartC command line
      $start.=$1." ";
    }
  }
  close (PTU);
#print "stubs=",join(",",@$stub_lines),"\n";
  return ($start);
}

###########################################################################

sub extractFunctionName {
  my $line = shift;
  chomp $line;
  return("") unless($line =~ m/^\s*\#(.*)$/);
  $line = $1;
  #Does it look like a function, if so extract it
  # function prototype is one of
  # <type> <*> <function name> (
  my $functionName;
  if ($line =~ m/\#include\s*[\"\<]([\w\.]+)[\"\>]/) {
    $functionName = $1;  
  }elsif ($line =~ m/\W(\w+)\s*\(/) {
    $functionName = $1;
  }elsif ($line =~ m/\W(\w+)\s*;/) {# could be a var
    $functionName = $1;
  }elsif ($line =~ m/\W(\w+)\s*\[.*\];/) {# could be a var
    $functionName = $1;
  }
  #print "Extracted \"$functionName\" from \"$line\"\n"; 
  return ($functionName);
}


###########################################################################

sub getMissing {
  my $type = shift; #0=>stub 1=>service
  my $exist = shift;
  my $required = shift;

  # Build of a list of just the name of the function being stubbed
  my %stubs;
  foreach (@$exist) {
    my $functionName;
    $functionName= extractFunctionName($_) unless($type);
    $functionName= $_ if($type);
    $stubs{$functionName} = 1 if ($functionName);
  }
  #print "\n";
  # Now go over the list of required stubs eliminating those that are there
  # already
  my @missing;
  foreach (@$required) {
      my $functionName;
      $functionName= extractFunctionName($_) unless($type);
      $functionName= $_ if($type);
      push (@missing, $_) unless (exists $stubs{$functionName});
  }
  return (@missing);
}


#----------------------------------------------------------------------------

################################################################################
# Here is the beginning
################################################################################
unless($main::TestRTCC){

	EnterInScript();

	ClearLogFile( $LOGFILENAME );
	exit (UpdateUnitTestingScript());
	print PerlTranslate("UpdateUnitTesting.pl", "Done.\n");
}

1;
# That's all Folks.
################################################################################
