#!/usr/bin/perl


#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################


################################################################################
#                                                                              #
# Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  #
#                                                                              #
#                Product: Rational(R) Test RealTime                            #
#      Product Component: System Testing                                       #
#               Language: PERL                                                 #
#                                                                              #
#        Product version: 2003.06.00                                           #
#                                                                              #
#                   File: ats2pts.pl                                           #
#                                                                              #
# Rational, the Rational logo are registered trademarks of                     #
# Rational Software Corporation.                                               #
#                                                                              #
# The placing of the copyright notices on Rational source code is not          #
# intended to mean that they are published works, but are included in the      #
# event of inadvertent publication. The source code of Rational contains       #
# trade secrets and other valuable confidential information of Rational        #
# Software Corporation. Rational Software Corporation and its licensors        #
# retain ownership to Rational Products in both source and object code         #
# format, and related documentation and all rights therein including           #
# copyrights, patents and trade secrets.                                       #
#                                                                              #
#                                                                              #
# U.S. Government Rights                                                       #
#                                                                              #
# Use, duplication, or disclosure by the U.S. Government is subject to         #
# restrictions set forth in the applicable Rational License Agreement          #
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   #
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         #
# or FAR 52.227-14, as applicable.                                             #
#                                                                              #
# Unless otherwise agreed to in writing, use of Rational products is           #
# governed by the Rational Product License Agreement provided with             #
# the product.                                                                 #
#                                                                              #
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

sub SplitFileName #($)
{
  my $filename = shift;

  if (!$OSNAME) { $OSNAME = ( $^O=~/win32/i) ? "win32" : "unix"; }
  if ( $OSNAME eq "win32" ) { $PATH_SEPARATOR = '\\'; }
  else { $PATH_SEPARATOR = '/'; }
    
  my @file;
  if ((($dotPos = rindex $filename,".") < 0) || ((($slashPos = rindex $filename,"$PATH_SEPARATOR") >= $dotPos)))
  {
    @file = ($filename =~ /^((?:.*[:\\\/])?)(.*)/);
  } 
  else
  {  
    @file = ($filename =~ /^((?:.*[:\\\/])?)(.*)\.(.*)/);
  } 

  return @file;
}

my ($d,$n,$e) = SplitFileName($ARGV[0]);
if ($e eq "pl")
{
  require $ARGV[0] ; #environment and global variables.
  die "Environment variable BINDIR not found. Exiting.\n" if(!defined("$BINDIR")); # NO TRANSLATION
  require "$BINDIR" . "/../../../lib/scripts/Translate.pl";
} else {
  require $ENV{'TESTRTDIR'}."/lib/scripts/Translate.pl";
}

# test parameters 
if ( ! @ARGV[0] )
  {
    usage();
    exit(1);
  }

# global declarations
my $cot = "\"";
#min elapsed time taken into account to compute PAUSE and timeout default is 10*pollcycle
my $accuracytime=10;
my $pollcycle=10;
my @insts;
my $instlist;
my %messages;
my %addid;
my $v;
my $analyse=0;
my $outName="";
my $cptarg = 0;
foreach $currentarg ( @ARGV )
{
   if($currentarg =~/^-inst.*=/)
     {
	my($tmp,$list)=split(/=/,$currentarg);
        # list of instances to take into account
	@insts=split(/,/,$list);
     }
   elsif ($currentarg =~/^-accur.*=/)
     {
	my($tmp,$accur)=split(/=/,$currentarg);
	$accuracytime=$accur;
     }
   elsif ($currentarg =~/^-poll.*=/)
     {
	my($tmp,$poll)=split(/=/,$currentarg);
	$pollcycle=$poll;
     }
   elsif ($currentarg =~/\.xdf$/)
     {
	$inName=$currentarg;
     }
   elsif ($currentarg =~/\.pts$/)
     {
	$outName=$currentarg;
     }
   elsif ($cptarg != 0)
     {
	usage();
	exit(1);    
     }
   $cptarg = $cptarg + 1;
}

if($inName eq "")
{
	usage();
	exit(1);    
}
if(@insts[0] eq "")
{
    # get the list of instances to take into account
    @insts = getInstList($inName);
}
$instlist = join ', ', @insts;

my ($dirpath, $basename, $extension) = ("$inName" =~ /^((?:.*[:\\\/])?)(.*)\.(.*)/);

$Bname=$dirpath.$basename;
if($outName eq "") {$outName="$Bname.pts";}

open( SRC, "<$inName" ) or die &PerlTranslate("ats2pts.pl", "Cannot open file %1\n", "%1 is the name of the file", $inName);

# PTS generation
open( PTS, ">$outName" ) or die &PerlTranslate("ats2pts.pl", "Cannot create file %1\n", "%1 is the name of the file", "$outName");

print &PerlTranslate("ats2pts.pl", "TestRT-%1, Rational(R) Test RealTime Probe Test Script Generator\n", "%1 is the code of the message", "I-STARTEXEC");
print &PerlTranslate("ats2pts.pl", "TestRT-%1, Copyright(C) 2001-2004 Rational Software Corporation.\n", "%1 is the code of the message", "I-COPYRIGHT");

print PTS "--\n";
print PTS "-- Rational(R) Test RealTime Probe Test Script Generator\n";
print PTS "-- Copyright(C) 2001-2004 Rational Software Corporation.\n";
print PTS "--\n\n";

print PTS "HEADER ".$cot."$Bname".$cot.",".$cot."1.0".$cot.",".$cot."1.0".$cot."\n\n";
print PTS "DECLARE_INSTANCE $instlist \n\n";
print PTS "\n-- Include Section\n";
print PTS "INCLUDE \"",$basename,"_msg.hts\"\n";
print PTS "INCLUDE \"",$basename,"_communication.hts\"\n";
print PTS "INCLUDE \"",$basename,"_init.hts\"\n";
print PTS "\n-- Scenario Section\n";
print PTS "SCENARIO main\n";
print PTS "\n-- Please, add global initializations here\n";
print PTS "\n COMMENT Beginning Of The Scenario\n";
print PTS " RENDEZVOUS START_SCN\n\n";

#
# for each instance of the input list
#
foreach $simu (@insts)
{
  print PTS " INSTANCE $simu:\n";
  $lineno=0;
  # open and analyze the xdf file
  open( SRC, "<$inName" ) or die &PerlTranslate("ats2pts.pl", "Cannot open file %1\n", "%1 is the name of the file", $inName);
  while (<SRC>)
    {
      $lineno++;
      
      if ($lineno > 0)
	{
	  my ($time, $sndr, $mode, $rcvr, $message, $data ) =
	    ($_ =~ /([0-9]+)\s*ms\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*{\s*(.*)\s*}/ );
	  if ( $mode =~ /Out/ )
	    {
	      $analyse=0;
	      $sender=$sndr;
	      $receiver=$rcvr;
	    }
	  elsif ( $mode =~ /In/ )
	    {
	      $analyse=0;
	      $receiver=$sndr;
	      $sender=$rcvr;
	    }
	  else
	    {
	      $analyse++;
	      print &PerlTranslate("ats2pts.pl", "TestRT-%1, Unknown Message Mode (%2) at line: %3\n", "%1 is the code of the message, %2 is the unknown mode and %3 is the line number", "E-PROBEGENE", $mode, $lineno);
	      print "       $_"; # NO TRANSLATION
	    }
	  if(!$analyse)
	    {
	      if( $sender eq $simu) 
		{
		# don't take into account message between virtual testers
		  if(! grep(/^$receiver$/,@insts))
		    {
		      $v=$sender."_".$receiver;
		      if(! $addid{$v} )
			{
			  $addid{$v}=1;
			  print PTS "     ADD_ID($v, ID$v )\n";
			}
		    }
		}
	      elsif ( $receiver eq $simu)
		{
		# don't take into account message between virtual testers
		  if (! grep(/^$sender$/,@insts))
		    {
		      $v=$receiver."_".$sender;
		      if(! $addid{$v} )
			{
			  $addid{$v}=1;
			  print PTS "     ADD_ID($v, ID$v )\n";
			}
		    }
		}
	    }
	}
    }
  print PTS "\n";
  close (SRC);
  $lineno=0;
  $t=0;
  open( SRC, "<$inName" ) or die &PerlTranslate("ats2pts.pl", "Cannot open file %1\n", "%1 is the name of the file", $inName);
  while (<SRC>)
    {
      $lineno++;
      
      if ( $lineno>0 )
	{
	  my ($time, $sndr, $mode, $rcvr, $message, $data ) =
	    ($_ =~ /([0-9]+)\s*ms\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*,\s*(\w+)\s*{\s*(.*)\s*}/ );
	  
	  $dtime=$time-$t;
	  $dtime10=int($dtime/$pollcycle);
	  if ( $mode =~ /Out/ )
	    {
	      $analyse=0;
	      $sender=$sndr;
	      $receiver=$rcvr;
	    }
	  elsif ( $mode =~ /In/ )
	    {
	      $analyse=0;
	      $receiver=$sndr;
	      $sender=$rcvr;
	    }
	  else
	    {
	      $analyse++;
	      print &PerlTranslate("ats2pts.pl", "TestRT-%1, Unknown Message Mode (%2) at line: %3\n", "%1 is the code of the message, %2 is the unknown mode and %3 is the line number", "E-PROBEGENE", $mode, $lineno);
	      print "       $_"; # NO TRANSLATION
	    }
	  if(!$analyse)
	    {
	      if ( $sender eq $simu)
		{
		# don't take into account message between virtual testers
		  if (! grep(/^$receiver$/,@insts))
		    {
		      $v=$sender."_".$receiver;
		      if($dtime10>$accuracytime)
			{
			  print PTS "     PAUSE($dtime10)\n";
			}
		      print PTS "     VAR $message ,INIT={";
		      strucAnalyse($data);
		      print PTS "}\n     SEND($message , $v )\n\n";
		      $t=$time;
		    }
		  else
		    {
		      print PTS "     -- SEND to $receiver, $message, $data \n";
		    }
		}
	      elsif( $receiver eq $simu) 
		{
		# don't take into account message between virtual testers
		  if(! grep(/^$sender$/,@insts))
		    {
		      $v=$receiver."_".$sender;
		      print PTS "     DEF_MESSAGE $message ,EV={";
		      strucAnalyse($data);
		      print PTS "}\n     WAITTIL(MATCHED($message , $v),WTIME>";
		      if ($dtime10<$accuracytime) {
			print PTS "shortTimeout )\n\n";
		      } else {
			print PTS "$dtime10 )\n\n";
		      }
		      $t=$time;
		    }
		  else
		    {
		      print PTS "     -- RECV from $sender $message, $data \n";
		    }
		}
	    }
	}
    }
  print PTS " END INSTANCE\n";
  print PTS "\n";
}

print PTS " COMMENT End Of The Scenario\n";
print PTS " RENDEZVOUS END_SCN\n\n";
print PTS "END SCENARIO\n";
close (PTS);

print &PerlTranslate("ats2pts.pl", "TestRT-%1, The following System Testing script file has been generated: [%2]\n", "%1 is the code of the message and %2 is the name of the System Testing script file", "I-GENESCRIP", "$outName");
print &PerlTranslate("ats2pts.pl", "TestRT-%1, End of execution\n", "%1 is the code of the message", "I-ENDOFEXEC");

#
# Get INSTANCE list from .xdf file 
#
sub getInstList #(@)
{
    my $file_in = shift;
    my $p1, $p2, $line;
    my %ret;

    open  (FD_xdf, "<".$file_in) or &PerlTranslate("ats2pts.pl", "Cannot open file %1\n", "%1 is the name of the file", $file_in);

    while ($line = <FD_xdf>)
      {
	($p1, $p2) = split(/\s*,\s*/, $line); 
	$ret{$p2}++;
      }

    close (FD_xdf);

    return keys %ret;
}

#
# strucAnalyse($data)
#
sub strucAnalyse #()
{
my $data=shift;
my %HoH;
my $flagh=0;
foreach (split(/,/,$data))
	{
	my ($struc,$val)=split(/=\>/,$_);
	$val =~ s/^0$/0 /g;
	$val =~ s/\'/\\\'/g;
	$struc =~ s/ //g;
	$struc =~ s/\.\./\!\!/g;
	if($struc =~ s/\./\'\}\{\'/g){$flagh=1;}
	# when the msg is an array yy[7] -> {'7'}, yy disapear
	else {	if($struc =~ s/\w*\[([0-9!]*)\]/$1/) {$flagh=1;}}
	# when a sub field is an array xx.yy[7].zz -> xx'}{'yy'}{'7'}{'zz'}
	$struc =~ s/(\w*)\[([0-9!]*)\]/$1\'\}\{\'$2/g;
	$struc =~ s/\!\!/\.\./g;
	
	my $x='$HoH{\''.$struc."\'}=\'".$val."\';";
#print "eval=".$x."\n";
	eval $x || die &PerlTranslate("ats2pts.pl", "evaluation error:%1 for expression %2 \n","%1 is the perl error %2 is the perl expression",$^E,$x);
	}
if($flagh) {print_hash(%HoH);}
else	   {print PTS "$data"}
}

#
# print_hash()
#
sub print_hash #()
{
my %H=shift;
my $sp;
foreach $key (keys %H)
   {
#   print "key = $key\n";
#   print "value = ",$H{$key},"\n";
#   print "ref = ",ref($H{$key})."\n";
   if(ref($H{$key}) eq "HASH")
   	{
   	print PTS "$sp$key =>{";
   	print_hash(%{$H{$key}});
	print PTS "}";
   	}
   else
   	{
   	print PTS "$sp$key =>$H{$key}";
   	}
   $sp=",";
   }
}

#
# print usage
#
sub usage #()
{
    print &PerlTranslate("ats2pts.pl", "TestRT-%1, Usage: ats2pts.pl [-inst=instance1[,...]] [-accuracyTime=xx] [-pollcycle=xx] <file.xdf> \n", "%1 is the code of the message", "E-PROBEGENE");
}
