##!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Source Instrumentation launcher -*-                #
#                                                                              #
#                                                                              #
################################################################################
#$Id: attolcc_cl.pl,v 1.32.2.1 2008/01/30 23:04:35 rfrance Exp $
################################################################################


# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$TARGETDIR=$ENV{'ATLTGT'};
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";

require $SCRIPTDIR . "attolcclib.pl";
my $flagMarks="[-/]"; #possible option marks for the compiler command
my $flagMark="-"; #flag mark used by the script in generated cmd
my $forceOutMark="Fo"; #end by: " " (white space if the file name is separated, or nothing if the file is joined to the option)
$DefineFlag=$flagMark."D"; # can be used by additional treatment.
my $noLinkFlag="c";#this var may be "" if it does not exist. with -nolink option the link phase is done when object or library files are meet on cmd line

#list of known extensions
@asmExtList=qw(S s);
@cExtList=qw(c);
@cppExtList=qw(C CC cc CPP cpp c++ CXX cxx);
@objExtList=qw(o O obj OBJ);
my @libExtList=qw(a lib LIB);

 @objFiles=();
 $PreproFlags="";
 $CompilFlags="";
 $LinkFlags="";
 $TpCflags='';
@compOriginal=();
my $compiler;
# specific variables
my $sbr;
################################################################################
sub ParseCompilerArgs #(@)
{
  my @argv=@_;
  my $i=0;
  my $ret=0;
  $compiler=shift(@argv);
  while(@argv)
  {
  my $arg=shift(@argv);
     $i++;
     #debug ("\$arg[$i] = [$arg]\n");
     if($arg=~ /^\"?\@(.+)$/) { 
	        my $fileOpt=$1;
	        my $header;
		$fileOpt=~ s/\"//g; #suppress final cot
		# read the options in the file and push into the existing @argv to be analysed as other
		if($Ini{OPTCODING}){
			open OPTIN,"<$Ini{OPTCODING}",$fileOpt || die &PerlTranslate("attolcc.pl", "Cannot open the file %1\n", "%1 is the file name", $fileOpt);
		}else{
			open OPTIN,"<$fileOpt" || die &PerlTranslate("attolcc.pl", "Cannot open the file %1\n", "%1 is the file name", $fileOpt);
			read (OPTIN, $header, 2);
			my $nums = join(',',map ord, split(//,$header));
			if ($nums eq "255,254") {
			  my $encoding = ":encoding(UTF-16LE)";
			  print "Using encoding $encoding\n";
			  binmode(OPTIN, $encoding);
			} elsif ($nums eq "254,255") {
			  my $encoding = ":encoding(UTF-16BE)";
			  print "Using encoding $encoding\n";
			  binmode(OPTIN, $encoding);
			} else {
			  seek (OPTIN, 0, SEEK_SET);
			}
		}
		trace("#read option File $fileOpt");
		while (<OPTIN>){
			chomp($_); #remove return char if any. Do not use chop
			trace("#> $_");
			my @optInFile=splitArgs($_);
			push(@argv,@optInFile);
		}
		close OPTIN;
        next;# remove this option
     }
     #change \" by \\\"
#     $_=~ s/([^\\])\\"/$1\\\"/g; #modify "string" by \"string\"
     push(@compOriginal,$arg);
    
  #it is a flag
  if($arg=~ /^(\")?$flagMarks(.+)$/) {
     my $preflag=$1;
     my $postflag;
     my $flag=$2;
     if($preflag) {
     	$flag=~ s/(\")$// ;
     	$postflag=$1;
     }
     my $trtmt;
     #debug("flag:$flag");
     if($flag eq $noLinkFlag) {NoLink();next;} #register that no link are required
     if($flag =~/^Fo/) {
        my $OFile;
        if($flag =~/^Fo$/) {#the filename is on the next arg
           $OFile=shift(@argv);
           }
        elsif($flag =~/^Fo\"?(.*)$/) {
           $OFile=$1;
           $OFile=~ s/\"$//
          }
        
	#anyway forceOut file may be appended to the link command
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
        
        #register the requested name of the object file to generate and/or its directory
        #to register a directory the path must ends by a path separator
	my ($d,$n,$e) = SplitFileName($OFile);
	OutFileFound($OFile) if(grep(/^$e$/,@objExtList) || ($OFile =~ /[\\\/]$/));
	$NAME=$n if ($NAME eq "");
	}
     elsif($flag =~/^out:$/) {
        my $exe=shift(@argv);
	$arg .= takeNextArg($exe);
	my ($dir,$name,$ext) = SplitFileName($exe);
	$NAME=$name if ($NAME eq "");
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
	}
     elsif($flag =~/^(out:)\"?(.+)$/) {
     	my $exe=$2;
	my ($dir,$name,$ext) = SplitFileName($exe);
	$NAME=$name if ($NAME eq "");
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
#	$flag = $f.$cot.$lag.$cot if(($lag =~ / /) && !($lag =~ /\"$/));
	}
     elsif($flag =~/^Fe$/) {
	$arg .= takeNextArg(shift(@argv));
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
	}
     elsif($flag =~/^Fe/) {
#	$flag = $f.$cot.$lag.$cot if(($lag =~ / /) && !($lag =~ /\"$/));
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
	}
     elsif($flag =~/^C$/) {
        next;# remove this option
	}
     elsif($flag =~/^TC$/) {
        force_language(1);# force instrumentation as C
	}
     elsif($flag =~/^TP$/) {
        force_language(2);# force instrumentation as C++
	}
     elsif($flag =~/^[EP]$/) {
        $ret=1;# no instrumentation at all
	}
     elsif($flag =~/^EP$/) {
        $ret=1;# no instrumentation at all
	}
     elsif($flag =~/^help$/i) {
        $ret=1;# no instrumentation at all
	}
     elsif($flag =~/^Zs$/) {
        $ret=1;# no instrumentation at all
	}
     elsif(uc($flag) =~/^LIB$/) {
        $ret=1;# no instrumentation at all
	}
     elsif($flag=~ /^FI$/) {
	$arg .= takeNextArg(shift(@argv));
        $trtmt="cl";# suppress from compil and link
        }
     elsif($flag=~ /^FI/) {
        $trtmt="cl";# suppress from compil and link
        }
     elsif($flag=~ /^Y[cuX].*$/) { #the file is optional and cannot be seen as a src or obj or lib file
        $trtmt="c";# suppress from compil 
        }
     elsif($flag=~ /^M([LTD]d?)$/) {
     	my $opa="-M$1";
        $ADDFLAGS.=' '.$opa if (!($ADDFLAGS =~ /$opa/));
        Setting('ADDFLAGS',$ADDFLAGS);
        }
     elsif($flag=~ /^F[rR](\")?(.+)\1$/) {
	if($2=~/[\\\/]$/) {#if the option specify only a path, special treatment should be done at compilation time for each source file
		$sbr=$arg;
		next;# then remove this option for now
	}
	# if the file is specified, no treatment, let option as is
     }
     elsif($flag=~ /^Fp\"?(.+)$/) { 
	open FI,">".$1;
	close FI;
        next;# remove this option
     }
     elsif($flag=~ /^Fd$/) {
	$arg .= takeNextArg(shift(@argv));
        $trtmt="p";# suppress from preprocessing
        }
     elsif($flag=~ /^Fd/) { 
        $trtmt="p";# suppress from preprocessing 
     }
     elsif($flag=~ /^(F[adm])(.+)$/) {
        $trtmt="p";# suppress from prepro
     }
     elsif($flag =~/^D(.+)$/) { #for these flags the macro name is used by conformence checker
	push(@GENERATION_CONDITION,$1);
     }
     elsif($flag =~/^I(.+)$/) { #for these flags the path is used by conformence checker
	push(@COMPLETE_INCLUDE_DIR_LIST,$1);
     }
     elsif($flag =~/^D$/) { #for these flags the next arg must not be interpreted
	my $a=shift(@argv);
	$arg .= takeNextArg($a);
	push(@GENERATION_CONDITION,$a);
     }
     elsif($flag =~/^I$/) { #for these flags the next arg must not be interpreted
	my $a=shift(@argv);
	$arg .= takeNextArg($a);
	push(@COMPLETE_INCLUDE_DIR_LIST,$a);
     }
     elsif($flag =~/^U$/) { #for these flags the next arg must not be interpreted
	$arg .= takeNextArg(shift(@argv));
     }

    ################################################################################     
        #$trtmt="pcl";#set $trtmt to exclude this flag from commands line
	#	p means: this option must not be part of the preprocessing command
	#	c means: this option must not be part of the compilation command
	#	l means: this option must not be part of the link command
	#	r means: this option must not be part of the runtime compilation command
     #debug("$flag trtmt:$trtmt");# if($trtmt);
     if(!($trtmt =~/p/)){
	$PreproFlags .=$arg.$sp;
	diffBetweenPreproAndCompil() if($trtmt =~/c/); #registering that preprocessing and compil have not the same options
        }
     if(!($trtmt =~/c/)){
	$CompilFlags .=$arg.$sp;
	diffBetweenPreproAndCompil() if($trtmt =~/p/); #registering that preprocessing and compil have not the same options
        }
     if(!($trtmt =~/l/)){
	$LinkFlags   .=$arg.$sp;
        }
     if(!($trtmt =~/[cr]/)) #exclude if specified and if excluded from compiler command
	{
	$TpCflags   .=$arg.$sp;
	}
     } 
  else { #it is not a flag
     #is it file
     my $f=$arg;
     $f=~s/^\"//; #suppress begin cot if any
     $f=~s/\"$//; #suppress end cot if any
     my ($dir,$name,$ext) = SplitFileName($f);
     #debug("$f=>$dir,$name,$ext");
     if((grep(/^$ext$/,@asmExtList)) || (grep(/^$ext$/,@cExtList)) || (grep(/^$ext$/,@cppExtList))) { 
	#debug("src:$f");
	SourceFileFound($f);
	next;
	}
     elsif(grep(/^$ext$/,@objExtList)) {
	#debug("obj:$f");
	push(@objFiles,$f);
	$Link=1;
	next;
	}
     elsif(grep(/^$ext$/,@libExtList)) {
	#debug("lib:$f");
	$LinkFlags .=$arg.$sp;
	$Link=1;
	next;
	}
     else { #it is not a file
	#debug("cmd:$arg");
	$PreproFlags .=$arg.$sp;
	$CompilFlags .=$arg.$sp;
	$LinkFlags   .=$arg.$sp;	   	
	$TpCflags  .=$arg.$sp;	   	
	}#grep files
     }#is it a flag
  }
  return $ret;#return 0=OK, 1=start the stdCompil command then exit
}

###########################################################################
# takeNextArg
sub takeNextArg #($)
{
my $fn=shift;
push(@compOriginal,$fn);
return $sp.$fn;
}
###########################################################################
# preprocessing
sub preprocessing #($$$)
{
   my ($lang,$src,$out)=@_;
   print ">preprocessing of $src to $out\n";
   unlink($out);
   return atl_cpp($lang,$src,$out,$PreproFlags,"","");
#   return SystemL("cl.exe",$src,split(/ /,$PreproFlags),"-P");
}

###########################################################################
# compilation
sub compilation #($$$)
{
   my ($ext,$src,$out)=@_;
   my $cf=$CompilFlags;
   print ">compilation of $src to $out\n";
   	if($sbr){
   		my ($dir,$name,$ext) = SplitFileName($out);
   		$name.=".sbr";
		my $sbr2=$sbr;
		$sbr2=~s/(\")?$/$name$1/;
   		$cf.=$sp.$sbr2;
   	}
   return SystemL("$compiler $flagMark$noLinkFlag $cot$src$cot $cf $Ini{'ATTOLCC_CCFLAGS'} $flagMark$forceOutMark$cot$out$cot");
}

###########################################################################
# Link
sub Link #($)
{
   my $tpo=shift;
print ">Link with $tpo\n";
   $tpo="\"$tpo\"" if($tpo=~ /\s/);
return SystemL("$compiler \"".join("\" \"",@objFiles)."\" $tpo $LinkFlags $Ini{'ATTOLCC_LDFLAGS'}");
}

###########################################################################
# standard compilation using Original cmd line
sub stdCompil #()
{
return system($compiler,@compOriginal);
}
################################################################################

# First parse command-line
# Then instrument sources
Attolcc();

1;
#
################################################################################

