#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Source Instrumentation launcher -*-                #
#                                                                              #
#                                                                              #
################################################################################
#$Id: attolcc_gcc.pl,v 1.41.2.2 2008/01/30 23:04:35 rfrance Exp $
################################################################################
# useful for stdout/stderr direct viewing inside 'console' of studio window
# better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$TARGETDIR=$ENV{'ATLTGT'};
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";

require $SCRIPTDIR . "attolcclib.pl";
my $flagMarks="[-]"; #possible option marks for the compiler command
my $flagMark="-"; #flag mark used by the script in generated cmd
my $forceOutMark="o "; #end by: " " (white space if the file name is separated, or nothing if the file is joined)
$DefineFlag=$flagMark."D"; # can be used by additional treatment.
my $noLinkFlag="c";#this var may be "" if it does not exist. with -nolink option the link phase is done when object or library files are met on cmd line

#list of known extensions
@asmExtList=qw(S s);
@cExtList=qw(c i);
@cppExtList=qw(C CC cc CPP cpp c++ CXX cxx);
@objExtList=qw(o O obj OBJ so);
my @libExtList=qw(a lib LIB);
@objFiles=();

$PreproFlags="";
$CompilFlags="";
$LinkFlags="";
$TpCflags='';
@compOriginal=();
my $compiler;

################################################################################
sub ParseCompilerArgs #(@)
{
  my @argv=@_;
  my $i=0;
  my $ret=0;
  $compiler=shift(@argv);

  while(@argv)
  {
  my $arg=shift(@argv);
     $i++;
     #debug ("\$arg[$i] = [$arg]\n");
     push(@compOriginal,$arg);
    
     #it is a flag
  if($arg=~ /^(\")?$flagMarks(.+)$/) {
     my $preflag=$1;
     my $postflag;
     my $flag=$2;
     if($preflag) {
     	$flag=~ s/(\")$// ;
     	$postflag=$1;
     }
     my $trtmt;
     	#debug("flag:$flag");
     	if($flag eq $noLinkFlag) {NoLink();next;}
     	if($flag =~/^o/)
	{
		my $OFile;
		if($flag =~/^o$/)
		{#the filename is on the next arg
			$OFile=shift(@argv);
			$arg .= takeNextArg($OFile);		
		}
		elsif($flag =~/^o\"?(.+)$/)
		{
			$OFile=$1;
			$OFile=~ s/\"$//
		}
	#anyway forceOut file may be appended to the link command
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
        
        #register the requested name of the object file to generate and/or its directory
        #to register a directory the path must ends by a path separator
	my ($d,$n,$e) = SplitFileName($OFile);
	OutFileFound($OFile) if(grep(/^$e$/,@objExtList) || ($OFile =~ /[\\\/]$/));
	$NAME=$n if ($NAME eq "");
	}
	elsif($flag =~/^Xlinker$/) {
		$arg .= takeNextArg(shift(@argv));	    
		$trtmt="cp";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^[EPM]$/) {
		$ret=1;# no instrumentation at all
	}
	elsif($flag =~/^MM$/) {
		$ret=1;# no instrumentation at all
	}
	elsif($flag =~/^M[TF]$/) {
		$arg .= takeNextArg(shift(@argv));
		$trtmt="cl";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^M/) {
		#other Mx options are given to the prepro only
		$trtmt="clr";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^g3$/) {
		$PreproFlags .=$flagMark."g".$sp;
		diffBetweenPreproAndCompil();	
		$trtmt="p";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^dirafter$/ || $flag =~/^prefix$/ || $flag =~/^withprefix$/) {
		$arg .= takeNextArg(shift(@argv));
		$trtmt="cl";#set $trtmt to exclude this flag from commands line
	}
        # If -I- option, add -I$ATLTGT/lib to be able to compile the instrumented file
        elsif($flag =~/I-$/) {
                $arg .= " -I$TARGETDIR/lib";
        }
	elsif($flag =~/^frepo$/) {
		$trtmt="pc";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^S$/) {
		$trtmt="p";
	}
	elsif($flag =~/^xc$/) {
		force_language(1);# force instrumentation as C
	}
	elsif($flag =~/^xc\+\+$/) {
		force_language(2);# force instrumentation as C++
	}
	elsif($flag =~/^x$/) {
		$arg .= takeNextArg(shift(@argv));
		if ($flag=~/c\+\+/) {
			force_language(2);# force instrumentation as C++
		}
		else {
			force_language(1);# force instrumentation as C
		}
	}	
	elsif($flag =~/^include$/ || $flag =~/^macros$/) {
		$arg .= takeNextArg(shift(@argv));
		$trtmt="cl";#set $trtmt to exclude this flag from commands line
	}
        elsif($flag =~/^Werror$/) {
                $trtmt="cl";#set $trtmt to exclude this flag from commands line
        }
	elsif($flag =~/^Wp,/) { #for these flags the macro name is used by conformence checker
		$trtmt="cl";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^D(.+)$/) { #for these flags the macro name is used by conformence checker
		my $def=$1;
		if($def =~ /\(|\)/){$arg=$cot.$arg.$cot; $def=$cot.$def.$cot;}
		push(@GENERATION_CONDITION,$def);
	}
	elsif($flag =~/^I(.+)$/) { #for these flags the path is used by conformence checker
		push(@COMPLETE_INCLUDE_DIR_LIST,$1);
	}
	elsif($flag =~/^D$/) { #for these flags the next arg must not be interpreted
		my $a=shift(@argv);
		$arg .= takeNextArg($a);
		push(@GENERATION_CONDITION,$a);
	}
	elsif($flag =~/^I$/) { #for these flags the next arg must not be interpreted
		my $a=shift(@argv);
		$arg .= takeNextArg($a);
		push(@COMPLETE_INCLUDE_DIR_LIST,$a);
	}
	elsif($flag =~/^U$/) { #for these flags the next arg must not be interpreted
		$arg .= takeNextArg(shift(@argv));
	}
	# should make a treatment and/or remove this flag from one of command lines ?
	#	$trtmt="pcl";#set $trtmt to excluide this flag from commands line
		#	p means: this option must not be part of the preprocessing command
		#	c means: this option must not be part of the compilation command
		#	l means: this option must not be part of the link command
		#	r means: this option must not be part of the runtime compilation command

	#debug("trtmt:$trtmt") if($trtmt);
	if(!($trtmt =~/p/))
	{
		$PreproFlags .=$arg.$sp;
		diffBetweenPreproAndCompil() if($trtmt =~/c/);	
	}
	if(!($trtmt =~/c/))
	{
		$CompilFlags .=$arg.$sp;
		diffBetweenPreproAndCompil() if($trtmt =~/p/);	
	}
	if(!($trtmt =~/l/))
	{
		$LinkFlags   .=$arg.$sp;
	}
	if(!($trtmt =~/[cr]/)) #exclude if specified and if excluded from compiler command
	{
		$TpCflags   .=$arg.$sp;
	}
     } 
     else
     { #it is not a flag, is it a file?
        my $f=$arg;
        $f=~s/^\"//; #suppress begin cot if any
        $f=~s/\"$//; #suppress end cot if any
        my ($dir,$name,$ext) = SplitFileName($f);
	if((grep(/^$ext$/,@asmExtList)) || (grep(/^$ext$/,@cExtList)) || (grep(/^$ext$/,@cppExtList)))
	{ 
		#debug("src:$f");
		SourceFileFound($f);
		next;
	}
	elsif(grep(/^$ext$/,@objExtList))
	{
		#debug("obj:$f");
		push(@objFiles,$f);
		$Link=1;
		next;
	}
	elsif(grep(/^$ext$/,@libExtList))
	{
		#debug("lib:$arg");
		$LinkFlags .=$arg.$sp;
		$Link=1;
		next;
	}
	else
	{ #it is not a file
		#debug("cmd:$arg");
		$PreproFlags .=$arg.$sp;
		$CompilFlags .=$arg.$sp;
		$LinkFlags .=$arg.$sp;	   	
		$TpCflags  .=$arg.$sp;	   	
	}
     }
  } # End while
  return $ret;#return 0=OK, 1=start the stdCompil command then exit
}
###########################################################################
# takeNextArg
sub takeNextArg #($)
{
my $fn=shift;
if($fn=~ / /) {$fn = $cot.$fn.$cot;}
push(@compOriginal,$fn);
return $sp.$fn;
}

###########################################################################
# preprocessing
sub preprocessing #($$$)
{
   my ($lang,$src,$out)=@_;
   print ">preprocessing of $src to $out\n";
   unlink($out);
   $status=SystemL("$compiler \"$src\" $PreproFlags -E -o \"$out\"" );
#  fixlines not required since GCC 3.2
   if($ENV{'FIXLINES'} && $status==0){
	rename($out,"$out.fix");
	$status=SystemL( "$BINDIR/fixlines \"$out.fix\" \"$out\"" );
	unlink ("$out.fix" );
   }
   return $status;
}

###########################################################################
# compilation
sub compilation #($$$)
{
   my ($ext,$src,$out)=@_;
   print ">compilation of $src to $out\n";
   #my $ret=SystemL($compiler,$flagMark.$noLinkFlag,$src,split(/ /,$CompilFlags),split(/ /,$flagMark.$forceOutMark.$out));
   my $ret=SystemL("$compiler $flagMark$noLinkFlag $CompilFlags $Ini{'ATTOLCC_CCFLAGS'} $flagMark$forceOutMark \"$out\" \"$src\"");
   return $ret;
}

###########################################################################
# Link
sub Link #($)
{
   my $tpo=shift;
   print ">Link with $tpo\n";
   $tpo="\"$tpo\"" if($tpo=~ /\s/);
   #return SystemL($compiler, @objFiles,$tpo,split(/ /,$LinkFlags));
   return SystemL("$compiler \"".join("\" \"",@objFiles)."\" $tpo $LinkFlags $Ini{'ATTOLCC_LDFLAGS'}");
}

###########################################################################
# standard compilation using Original cmd line
sub stdCompil #()
{
   return system($compiler, @compOriginal);
}
################################################################################

# First parse command-line
# Then instrument sources
Attolcc();

1;
#
################################################################################
