#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Source Instrumentation launcher -*-                #
#                                                                              #
#                                                                              #
################################################################################
#$Id: attolcc_template.pl,v 1.5.2.1 2008/01/30 23:04:35 rfrance Exp $
################################################################################
# this script involve the customizable function of the attolcc instrumentation launcher
# some parts should be customized to be compliant with the build tool chain, and
# other parts can be customized very carefully.
# 2 other examples are provided attolcc_gcc.pl(gnu compiler) and attolcc_cl.pl(visual)
#
#
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
$TARGETDIR=$ENV{'ATLTGT'};
$SCRIPTDIR="$TESTRTDIR/lib/scripts/";

require $SCRIPTDIR . "attolcclib.pl";
@objFiles=();

$PreproFlags="";
$CompilFlags="";
$LinkFlags="";
$TpCflags='';
@compOriginal=();
my $compiler;

######################################### customizable part ############################################
my $flagMarks="[-/]"; #possible option marks available for this compiler/linker
my $flagMark="-"; #flag mark used by the intruemtation script in the generated cmd
my $forceOutMark="o "; #end by: " " (white space if the file name is separated, or nothing if the file is joined)
$DefineFlag=$flagMark."D"; # can be used by additional treatment.
my $noLinkFlag="c";#this var may be "" if it does not exist. with -nolink option the link phase is done when object or library files are met on cmd line

#list of known extensions
@asmExtList=qw(S s);
@cExtList=qw(c i);
@cppExtList=qw(C CC cc CPP cpp c++ CXX cxx);
@objExtList=qw(o O obj OBJ so);
my @libExtList=qw(a lib LIB);

###########################################################################
# the following function are similar to the TDP build function except they assume the options are properly
# generated because they come from the command line.
# nevertheless these function may need to be custumized to modify the option order or any other needs
#

###########################################################################
# preprocessing
sub preprocessing #($$$)
{
   my ($lang,$src,$out)=@_;
   print ">preprocessing of $src to $out\n";
   unlink($out);
   return atl_cpp($lang,$src,$out,$PreproFlags,"",""); #use TDP build function preprocessing what ever the $compiler value.
#   return SystemL("$compiler \"$src\" $PreproFlags -E -o \"$out\"" ); #use the $compiler value to make the preprocessing
}

###########################################################################
# compilation of each source making the application (TP.c[pp] is still compiled using the TDP build setting compilation function)
sub compilation #($$$)
{
   my ($ext,$src,$out)=@_;
   print ">compilation of $src to $out\n";
   return SystemL("$compiler $flagMark$noLinkFlag $CompilFlags $Ini{'ATTOLCC_CCFLAGS'} $flagMark$forceOutMark \"$out\" \"$src\"");
}

###########################################################################
# Link with the TP object file
sub Link #($)
{
   my $tpo=shift;
   print ">Link with $tpo\n";
   $tpo="\"$tpo\"" if($tpo=~ /\s/);
   return SystemL("$compiler \"".join("\" \"",@objFiles)."\" $tpo $LinkFlags $Ini{'ATTOLCC_LDFLAGS'}");
}

###########################################################################
# standard compilation using Original cmd line
sub stdCompil #()
{
   return system($compiler, @compOriginal);
}
######################## end of customizable part ############################ 


################################################################################
# this function is called by the attolcc() once the -- mark preceding the compiler command is found,
# So the first arg of the list @argv should be the compiler. 
sub ParseCompilerArgs #(@)
{
  my @argv=@_;
  my $i=0;
  my $ret=0;
  $compiler=shift(@argv); #compiler name

  while(@argv)
  {
  my $arg=shift(@argv);
     $i++;
     #debug ("\$arg[$i] = [$arg]\n");
     push(@compOriginal,$arg);
    
     #it is a flag
  if($arg=~ /^(\")?$flagMarks(.+)$/) {
     my $preflag=$1;
     my $postflag;
     my $flag=$2;
     if($preflag) {
     	$flag=~ s/(\")$// ;
     	$postflag=$1;
     }
     my $trtmt;
	######################################### customizable part ############################################
	# The goal of this part is to analyze each compiler/linker options which must be taken into account
	# because its meaning has a special meaning for TestRT and so instrumentation steps have to be modified
	#
	# multiple actions may be done for each option:
	# the curent option may be removed for one of the steps. $trtmt variable is used as:
	#	$trtmt="pcl";#set $trtmt to excluide this flag from commands line
		#	p means: this option must not be part of the preprocessing command
		#	c means: this option must not be part of the compilation command
		#	l means: this option must not be part of the link command
		#	r means: this option must not be part of the runtime compilation command
	# the option found means that instrumentation should not be done. In this case set $ret=1
	# the option means the source code is C++ even if the ext is .c and vice versa. In that case call
		#	force_language(1);# force instrumentation as C
		#	force_language(2);# force instrumentation as C++
	# the option found means that the Runtime (TP) must be compiled using special settings.	
	# in that case call 	(e.g. option /MT for visual or pthread meaning multi-threaded app =>Setting(atl_multi_threads, 1))
		# Setting(settingName, Value)
	# NoLink() have to be called when the no_link flag is found in the command line or if the compilerName means this is not a link cmd
	#
	# some options uses the next arg as parameter. in this case the read of this option should take the next arg as well and do not let the 
	# while loop read the next arg alone and so make a bad interpretation of it.
	# $arg.=takeNextArg(shift(@argv)); allows to remove the next arg from the arg list and format it to be appended to the current arg.
	# 
	#debug("text"); allows to print out debug data using a special print format
	#
	
     	#debug("flag:$flag");
     	if($flag eq $noLinkFlag) {NoLink();next;}# flag comilation only
     	if($flag =~/^o/) #o is the flag for output file
	{
		my $OFile;
		if($flag =~/^o$/)#o is the flag for output file
		{#the filename is on the next arg
			$OFile=shift(@argv);
			$arg .= takeNextArg($OFile);		
		}
		elsif($flag =~/^o\"?(.+)$/)#o is the flag for output file
		{
			$OFile=$1;
			$OFile=~ s/\"$//
		}
	#anyway forceOut file may be appended to the link command
        $trtmt="pc";#set $trtmt to exclude this flag from commands line
        
        #register the requested name of the object file to generate and/or its directory
        #to register a directory the path must ends by a path separator
	my ($d,$n,$e) = SplitFileName($OFile);
	OutFileFound($OFile) if(grep(/^$e$/,@objExtList) || ($OFile =~ /[\\\/]$/));
	$NAME=$n if ($NAME eq "");#$NAME is a setting allowing the provide the Top name in the report. this setting is used at link time
	}
	elsif($flag =~/^[EPM]$/) {#E, P or M means the command is not used to compile. so do not instrument in this case
		$ret=1;# no instrumentation at all
	}
	elsif($flag =~/^include$/) {#include means include the file (next arg) for preprocessing. So don t use it at prepro and compile
		$arg .= takeNextArg(shift(@argv));
		$trtmt="cl";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^Xlinker$/) { #Xlinker flag means the next arg is for the linker command only
		$arg .= takeNextArg(shift(@argv));	    
		$trtmt="cp";#set $trtmt to exclude this flag from commands line
	}
	elsif($flag =~/^D(.+)$/) { #for these flags the macro name is used by codereview
		push(@GENERATION_CONDITION,$1);
	}
	elsif($flag =~/^D$/) { #for these flags the next arg must not be interpreted
		my $a=shift(@argv);
		$arg .= takeNextArg($a);
		push(@GENERATION_CONDITION,$a);
	}
	elsif($flag =~/^I(.+)$/) { #for these flags the path is used by codereview
		push(@COMPLETE_INCLUDE_DIR_LIST,$1);
	}
	elsif($flag =~/^I$/) { #for these flags the next arg must not be interpreted
		my $a=shift(@argv);
		$arg .= takeNextArg($a);
		push(@COMPLETE_INCLUDE_DIR_LIST,$a);
	}
	######################## end of customizable part ###########################
	
	
	
	#debug("trtmt:$trtmt") if($trtmt);
	if(!($trtmt =~/p/))
	{
		$PreproFlags .=$arg.$sp;
		diffBetweenPreproAndCompil() if($trtmt =~/c/);	
	}
	if(!($trtmt =~/c/))
	{
		$CompilFlags .=$arg.$sp;
		diffBetweenPreproAndCompil() if($trtmt =~/p/);	
	}
	if(!($trtmt =~/l/))
	{
		$LinkFlags   .=$arg.$sp;
	}
	if(!($trtmt =~/[cr]/)) #exclude if specified and if excluded from compiler command
	{
		$TpCflags   .=$arg.$sp;
	}
     } 
     else
     { #it is not a flag, is it a file?
        my $f=$arg;
        $f=~s/^\"//; #suppress begin cot if any
        $f=~s/\"$//; #suppress end cot if any
        my ($dir,$name,$ext) = SplitFileName($f);
	if((grep(/^$ext$/,@asmExtList)) || (grep(/^$ext$/,@cExtList)) || (grep(/^$ext$/,@cppExtList)))
	{ 
		#debug("src:$f");
		SourceFileFound($f);
		next;
	}
	elsif(grep(/^$ext$/,@objExtList))
	{
		#debug("obj:$f");
		push(@objFiles,$f);
		$Link=1;
		next;
	}
	elsif(grep(/^$ext$/,@libExtList))
	{
		#debug("lib:$arg");
		$LinkFlags .=$arg.$sp;
		$Link=1;
		next;
	}
	else
	{ #it is not a file
		#debug("cmd:$arg");
		$PreproFlags .=$arg.$sp;
		$CompilFlags .=$arg.$sp;
		$LinkFlags .=$arg.$sp;	   	
		$TpCflags  .=$arg.$sp;	   	
	}
     }
  } # End while
  return $ret;#return 0=OK, 1=start the stdCompil command then exit
}
###########################################################################
# takeNextArg
sub takeNextArg #($)
{
my $fn=shift;
if($fn=~ / /) {$fn = $cot.$fn.$cot;}
push(@compOriginal,$fn);
return $sp.$fn;
}

################################################################################
# First parse command-line
# Then instrument sources
Attolcc();
################################################################################
1;
