#!/usr/bin/perl
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

################################################################################
#                                                                              #
# Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  #
#                                                                              #
#                Product: Rational(R) Test RealTime                            #
#      Product Component: System Testing                                       #
#               Language: PERL                                                 #
#                                                                              #
#        Product version: 2003.06.00                                           #
#                                                                              #
#                   File: csv2header.pl                                        #
#                                                                              #
# Rational, the Rational logo are registered trademarks of                     #
# Rational Software Corporation.                                               #
#                                                                              #
# The placing of the copyright notices on Rational source code is not          #
# intended to mean that they are published works, but are included in the      #
# event of inadvertent publication. The source code of Rational contains       #
# trade secrets and other valuable confidential information of Rational        #
# Software Corporation. Rational Software Corporation and its licensors        #
# retain ownership to Rational Products in both source and object code         #
# format, and related documentation and all rights therein including           #
# copyrights, patents and trade secrets.                                       #
#                                                                              #
#                                                                              #
# U.S. Government Rights                                                       #
#                                                                              #
# Use, duplication, or disclosure by the U.S. Government is subject to         #
# restrictions set forth in the applicable Rational License Agreement          #
# and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   #
# 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         #
# or FAR 52.227-14, as applicable.                                             #
#                                                                              #
# Unless otherwise agreed to in writing, use of Rational products is           #
# governed by the Rational Product License Agreement provided with             #
# the product.                                                                 #
#                                                                              #
################################################################################

# Add tools functions
require $SCRIPTDIR . "Fcts.pl";

# Export...

# Setting the default separator
$separator = ";";
$deciseparator = ".";
$thouseparator = " ";

sub genHeaderFileFromCSV  {
# Arg#1 : the .csv file
# Arg#2 : output dir
# Arg#3 : separator char
# Arg#4 : decimal separator
# Arg#5 : thousand separator

	my @arguments = @_;
# Parsing command line

 	$inputfile = $arguments[0];
 	$output = getOutputName($arguments[1], $inputfile);
	$separator = $arguments[2];
	$deciseparator = $arguments[3];
	$thouseparator = $arguments[4];

# Open input file
	open(FIN, "<".$inputfile) or die &PerlTranslate("csv2header.pl", "ERROR: Cannot open the input file '%1'\n", "%1 is the name of the file", $inputfile);

# First line contains the names of the arrays
	$line = <FIN>;
	$line = my_chomp($line);
	@noms = my_split($line);

# Second line contains the types of arrays
	$line = <FIN>;
	$line = my_chomp($line);
	@types = my_split($line);

# The rest contains data
	my @elems; # This is an array of arrays
	while ($line = <FIN>) {
	  $line = my_chomp($line);
	  $index = 0;
	  @tab = my_split($line);
	  foreach $elem(@tab) {
	    $elem =~ s,^\"+,,;
	    $elem =~ s,\"+$,,g;
	    if ($types[$index] =~ /^(string|char\s*\*)$/i) {
	      $elem = "\"" . $elem . "\"" ;
	    }
	    else
	    {
	      if ($elem ne '' and $types[$index] =~ /^char$/i)
	      {
		$elem = "'" . $elem . "'";
	      } else {
		$elem =~ s/\Q$thouseparator\E//g if ($thouseparator ne '');
		if($types[$index] =~ /(float|double)$/i){
			$elem =~ s/\Q$deciseparator\E/\./g;
			$elem .='.0' unless($elem=~ /\./);
		}
	      }
	    }
	    # To avoid to maintain a special index for each array, we have just
	    # to push the new element into the array
	    if ($elem ne '') {
	      push @{$elems[$index]}, $elem; # Adding an element to the subarray
	    }
	    $index++;
	  }
	}
	# Closing ressources
	close(FIN);

# Generate output file
  open(FOUT, ">".$output) or die &PerlTranslate("csv2header.pl", "ERROR: Cannot create the output file %1\n", "%1 is the name of the file", $output);
  # Some header informations...
  printf FOUT "/***********************************************\n";
  printf FOUT " * csv2header version 1.1 \n";
  @time = localtime();
  $date = sprintf("%d-%02d-%02d %02d:%02d:%02d", $time[5]+1900, $time[4]+1, $time[3], $time[2], $time[1], $time[0]);
  printf FOUT " * Generated:  $date\n";
  printf FOUT " * Input:      $inputfile\n";
  printf FOUT " ***********************************************/\n\n";
  my $protect = "_" . $output . "_";
  $protect =~ s/[\\\/\.\:\~]/_/g;
  printf FOUT "#ifndef $protect \n";
  printf FOUT "#define $protect \n";
  # Prints data
  my $thisSize = 0;
  for $i (0 .. $#elems) {
    $thisSize = $#{$elems[$i]} + 1;
    printf FOUT "\nint $noms[$i]_size = $thisSize;\n";
    $type_of_this = getType($types[$i]);
    printf FOUT "$type_of_this $noms[$i]" . "[] = {";
    for $j ( 0 .. $#{$elems[$i]} ) {
      if ($j > 0) {
        printf FOUT ", $elems[$i][$j]";
      } else {
        printf FOUT "$elems[$i][$j]";
      }
    }
    printf FOUT "};\n";
  }
  printf FOUT "\n#endif /* $protect */\n";
  close(FOUT);
} # gen_file

# Splits a line with the given character separator
sub my_split {
  my @ret = split(/\Q$separator\E/, shift(@_));
  return @ret;
}

# Creates the output filename
sub getOutputName {
  my $output = shift(@_);
  my $infile = shift(@_);
  my $os_type = $OS;
  my ($srcdir, $srcname, $srcext) = SplitFileName($infile);
  $output = $output . "/" . $srcname . ".h";
  return $output;
}

# Finds the C type of this variable
sub getType {
  my $str_type = shift(@_);
  if ($str_type eq '') {
    $str_type = "int"; # The default type
  }
  elsif($str_type =~ /^(string|char\s*\*)$/i) {
    $str_type = "char *";
  }
  else {
    if ($str_type =~ /^(signed |unsigned ){0,1}(short|int|long|float|double|char)$/i) {
      $str_type =~ tr/A-Z/a-z/;
    }
    else {
      die &PerlTranslate("csv2header.pl", "ERROR: '%1' is not a C predefined type!\n", "%1 is the name of the unknown type", $str_type);
    }
  }
  return $str_type;
}

# When a line was read, we must suppress the newline character at the end, 
# and eventually the extra-character used by Windows programs : the line feed...
sub my_chomp {
  my $the_line = shift(@_);
  chomp($the_line);
  $the_line =~ s/\015$//; # Discard ^M character
  return $the_line;
}

