#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2005-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                            -*-  Some instrum tools -*-                       #
                                                                               #                                                                                                      #
################################################################################
#$Id: instrum.pl,v 1.16 2007/06/14 17:15:24 rfrance Exp $
################################################################################


use File::Copy;

################################################################################
# RTRT Instrumentor execution
#
# Required global variables:
#    $BINDIR
#    $TARGETDIR
#    $LOGFILENAME
################################################################################
sub atl_instr #($$$$)
{
   my ($language,$input,$output,$options) = @_;

   my $binpath      = "";
   my $binopts      = "";
   my $binary       = "";
   my $post_binopts = "";
   my $pre_output   = "";

   $binpath  = $BINDIR;

   $_=$language;

   SWITCH: {
     /CPP/  && do {
         $binary="$binpath" . "$PATH_SEPARATOR" . "attolccp";
         $binopts  = "atl.hpp atl.opp" ;
         last SWITCH; 
       };
     /C/ && do {
         $binary="$binpath" . "$PATH_SEPARATOR" . "attolcc1";
         $binopts  = "atct.def" ;
         last SWITCH; 
     };
     /ADA/ && do {
         $binary="$binpath" . "$PATH_SEPARATOR" . "attolada";
         last SWITCH;
     };
     /JAVA/ && do {
         $binary="$binpath" . "$PATH_SEPARATOR" . "javi";
         my $opp = "$TARGETDIR/ana/atl.opp";
         #get -source from CFLAGS and pass it in .opp file if any
         my ($source_version) = ($CFLAGS =~ /-source[ =\t]*([0-9.]*)/);
         if( $source_version )
         {
           my $wopp = "$WORKING_DIR" . "$PATH_SEPARATOR" . "atljava.opp" ;
           #copy opp in working directory  
           atl_copy( $opp, $wopp, 1 );
           #add --source option
           open ( OPP, ">>$wopp" ) or die &PerlTranslate("instrum.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
           print  OPP "--source $source_version\n";
           close( OPP );
           $binopts = "\"-opp=$wopp\"";
         }
         else
         {
           if ( -f $opp ) {
             $binopts = "\"-opp=$opp\"";
           }
         }
         $pre_output = "-destdir=";
         last SWITCH;
     };
     /ASM/ && do {
         $binary=$TARGETDIR.$PATH_SEPARATOR."cmd".$PATH_SEPARATOR."$Ini{'ASMINSTR'}";
         return (0) if(!($Ini{'ASMINSTR'}));
         $binopts  = "" ;
         last SWITCH; 
     };

   }
    
   $post_binopts .= " -studio_log=\"$LOGFILENAME\"" ;
   my $command = "$binary \"$input\" \"$pre_output$output\" $binopts $options $Ini{'INSTRUM_OPT'} $post_binopts";

   #execution
   my $status;
   if($Ini{'ATTOLINSTR'}) {
     print "ATTOLINSTR specified as:$TARGETDIR/cmd/$Ini{'ATTOLINSTR'}\n";
     print "sub attolInstr() executed with $command\n";
     require "$TARGETDIR/cmd/$Ini{'ATTOLINSTR'}";
     $status = attolInstr($command,$language,$input,$output);
   }
   else
   {
     $status = SystemP( $command );
   }
 
   return $status;

}

################################################################################
# BuildGeneralCommonOptions
# Return value: 
#   the options string
################################################################################
sub BuildGeneralCommonOptions
{
  my ($lan,$exfile, $exunit) = @_;
  my $result = "";
  my $ada_mode = 0;

  if ($TDP_LANGUAGE=~/^ADA/){
    $ada_mode = 1;
  }

  # FDC Dir/File
  if ($ATCMODE)
  {
    if ($ATG_FDC_NAME_OPT == 1) # static files same as source directory
    {
      $result .= " -fdcdir";
    }
    elsif ($ATG_FDC_NAME_OPT == 2) # static files in given directory
    {
      $result .= " -fdcdir=\"$ATG_FDC_VALUE\"";
    }
    elsif ($ATG_FDC_NAME_OPT == 3) # static files in the given unique file
    {
      $result .= " -fdcname=\"$ATG_FDC_VALUE\"";
    }
  }

  # TSF Dir/File
  if (($TDP_LANGUAGE eq "C" || $TDP_LANGUAGE eq "CPP") &&
      ($ATTMODE || $ATPMODE || $ATQMODE))
  {
    if ($ATG_TSF_NAME_OPT == 1) # static files same as source directory
    {
      $result .= " -tsfdir";
    }
    elsif ($ATG_TSF_NAME_OPT == 2) # static files in given directory
    {
      $result .= " -tsfdir=\"$ATG_TSF_VALUE\"";
    }
    elsif ($ATG_TSF_NAME_OPT == 3) # static files in the given unique file
    {
      $result .= " -tsfname=\"$ATG_TSF_VALUE\"";
    }
  }

  $result .= " -nowarning" if($ATL_NOWARNING);

  # -exunit
  my $some = 0;
  my $components_mode = 0;
  foreach $unit_ (@ATG_SELECTIVE_CLASS)
  {
    if ($some)
    {
      $result .= ",\"$unit_\"";
    }
    else
    {
      $result .= " -exunit=\"$unit_\"";
      $some = 1;
      $components_mode = 1;
    }
  }
  if (!$ada_mode)
  {
    if ($exunit ne "")
    {
      if ($some)
      {
        $result .= ",\"$exunit\"";
      }
      else
      {
        $result .= " -exunit=\"$exunit\"";
      }
    }
  }

  # -exfile
  my $some_file = 0;
  if (!$ada_mode && !$components_mode)
  {
    $some_file = 0;
    foreach $exfile_ (@ATG_SELECTIVE_FILES)
    {
      if ($some_file)
      {
	$result .= ",\"$exfile_\"";
      }
      else
      {
	$result .= " -exfile=\"$exfile_\"";
	$some_file = 1;
      }
    }
  }
  if (!$ada_mode)
  {
    if ($exfile ne "")
    {
      if ($some_file)
      {
        $result .= ",\"$exfile\"";
      }
      else
      {
        $result .= " -exfile=\"$exfile\"";
      }
    }
  }
  
  # -noinstrdir
  if ($TDP_LANGUAGE eq "C" || $TDP_LANGUAGE eq "CPP")
  {
    $some = 0;
    foreach $exdirs_ (@ATG_EXCLUDED_DIRS)
    {
      if ($some)
      {
        $result .= ",\"$exdirs_\"";
      }
      else
      {
        $result .= " -noinstrdir=\"$exdirs_\"";
        $some = 1;
      }
    }
  }

  if ($ATTMODE)
  {
     # -no_unnamed_trace
     if (($TDP_LANGUAGE eq "CPP" || $TDP_LANGUAGE eq "JAVA" ) &&
         !$ATT_AGGREGATE_TRACES)
     {
       $result .= " -no_unnamed_trace";
     }
     # -no_data_trace
     if (!$DATA_TRACE_ON && ($lan eq "CPP"))
     {
       $result .= " -no_data_trace";
     }
  }

  # -dumpincoming
  $some = 0;
  foreach $dump_ (@ATG_DUMP_INCOMING_LIST)
  {
    if ($some)
    {
      $result .= ",\"$dump_\"";
    }
    else
    {
      $result .= " -dumpincoming=\"$dump_\"";
      $some = 1;
    }
  }

  # -dumpreturning
  $some = 0;
  foreach $dump_ (@ATG_DUMP_RETURNING_LIST)
  {
    if ($some)
    {
      $result .= ",\"$dump_\"";
    }
    else
    {
      $result .= " -dumpreturning=\"$dump_\"";
      $some = 1;
    }
  }

  # -dumpcalling
  if ($lan eq "C")
  {
    $some = 0;
    foreach $dump_ (@ATG_DUMP_CALLING_LIST)
    {
      if ($some)
      {
        $result .= ",\"$dump_\"";
      }
      else
      {
        $result .= " -dumpcalling=\"$dump_\"";
        $some = 1;
      }
    }
  }
  elsif (@ATG_DUMP_CALLING_LIST)
  {
    print &PerlTranslate("instrum.pl", "Warning: Dump Calling option is available for C language only.\n");
  }

  # -noinfo
  if ($ATG_HEADER_INSTR == 0)
  {
    $result .= " -noinfo";
  }

  # -instantiationmode=all
  if ($lan eq "CPP" && $ATG_INSTANTIATE_TEMPLATE)
  {
    $result .= " -instantiationmode=all";
  }

  # other options
  if ($ATG_MISC_OPTIONS)
  {
    $result .= " $ATG_MISC_OPTIONS";
  }

  if ($ATL_MAIN_PROCEDURE)
  {
    $result .= " -main=$ATL_MAIN_PROCEDURE" ;
  }

  return $result;
}

################################################################################
# BuildCoverageCommonOptions
# Arguments:
#   Allows block=atc option or not
# Return value: 
#   the options string
################################################################################
sub BuildCoverageCommonOptions
{
  my ($allow_atc_blocks, $allow_calls, $allow_conds) = @_;
  my $result = "";

  # Only for Coverage and for common language options.
  if ($ATCMODE)
  {
     if ( $ATC_INSTR_FUNCTIONS == 0 ) { $result .= " -proc"; }
     if ( $ATC_INSTR_FUNCTIONS == 1 ) { $result .= " -proc=ret"; }
     if ( $ATC_INSTR_FUNCTIONS == 2 ) { $result .= " -noproc"; }

     if ( $allow_calls ) {
       if ( $ATC_INSTR_CALLS == 0 ) { $result .= " -call"; }
       # if ( $ATC_INSTR_CALLS == 1 ) { $result .= " -nocall"; }
     }

     if ( $ATC_INSTR_BLOCKS == 0 ) { $result .= " -block"; }
     if ( $ATC_INSTR_BLOCKS == 1 ) { $result .= " -block=decision"; }
     if ( $ATC_INSTR_BLOCKS == 2 ) { $result .= " -block=logical"; }
     if ( $ATC_INSTR_BLOCKS == 3 )
     {
       if ( $allow_atc_blocks )
       {
         $result .= " -block=atc"; 
       } else {
         print &PerlTranslate("instrum.pl", "Warning: Block Coverage Type 'Asynchronous Transfert Control' not allowed, using 'Loops' instead.\n");
         $result .= " -block=logical";
       }
     }
     if ( $ATC_INSTR_BLOCKS == 4 ) { $result .= " -noblock"; }

     if ( $allow_conds ) {
       if ( $ATC_INSTR_CONDITIONS == 0 ) { $result .= " -cond"; }
       if ( $ATC_INSTR_CONDITIONS == 1 ) { $result .= " -cond=modified"; }
       if ( $ATC_INSTR_CONDITIONS == 2 ) { $result .= " -cond=force"; }
       # if ( $ATC_INSTR_CONDITIONS == 3 ) { $result .= " -nocond"; }
     }
  }

  # -excall=<file>
# if ($TDP_LANGUAGE eq "C")
  if ( $allow_calls )
  {
    my $some = 0;
    foreach $excall_ (@ATC_SELECTIVE_FUNCTIONS_LIST)
    {
      if (!$some)
      {
        open(EXCALL, ">$INTDIR/excall.opt");
        print(EXCALL "$excall_\n");
        $some = 1;
      }
      else
      {
        print(EXCALL "$excall_\n");
      }
    }
    if ($some)
    {
      close(EXCALL);
      $result .= " -excall=\"$INTDIR/excall.opt\"";
      #LOG
      open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("instrum.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
      print  LOG "OPT $INTDIR/excall.opt\n";
      close( LOG );
    }
  }
  elsif ( $TDP_LANGUAGE=~/ADA/)
  {
  # -excall=<unit>[,<unit>]
    my $any=0;
    my $opt;
    foreach $excall_ (@ATC_SELECTIVE_FUNCTIONS_LIST)
    {
      if ($any)
      {
        $opt="$opt,$excall_";
      } else {
        $any=1;
        $opt="$excall_";
      }
    }
    if ( $any )
    {
      $result .= " -excall=$opt";
    }
  }

  # -metrics
  if ( ! $TDP_LANGUAGE eq "JAVA" && $ATC_METRICS )
  {
    $result .= " -metrics";
  }

  # -nofullpathkey
  if ( $ATC_NOFULLPATHKEY )
  {
    $result .= " -nofullpathkey";
  }

  return $result;
}

################################################################################
# BuildAdaInstrOptions
# Arguments:
#   units to exclude
# Return value: 
#   the options string
################################################################################
sub BuildAdaInstrOptions #()
{
  my $result = "";

  $result = BuildGeneralCommonOptions($TDP_LANGUAGE,"","");

  if ($ATCMODE)
  {
     $result .= BuildCoverageCommonOptions( 1, 1, 1 );

     if ( $ATC_SELECTIVE_SPEC )
     {
       $result .= " -spec" 
     }
     if ( ! $ATC_INSTR_CTRL_PREFIX eq "" ) 
     {
       $result .= " -prefix=$ATC_INSTR_CTRL_PREFIX";
     }
     if ( $ATC_INSTR_CTRL_SUFFIX == 1 ) 
     {
       $result .= " -generatedname=checksum";
     }

     if ( $ATC_INFORMATION == 1 ) 
     {
       $result .= " -instr=inline";
     }

     if ( $TDP_LANGUAGE eq "ADA83" )
     {
       $result .= " -ada83";
     }

     if ( ! $PACKAGES_TESTED eq "" )
     {
       $result .= " -INJECT=$PACKAGES_TESTED";
     }

     if ( ! $TEST_ENTRY_POINT_NAME eq "" )
     {
       $result .= " -NAMEINJECT=$TEST_ENTRY_POINT_NAME";
     }
  }

  if ($ATTMODE)
  {
     print &PerlTranslate("instrum.pl", "Warning: Trace feature is not available for Ada Language.\n");
     $ATTMODE = 0;
  }

  if ($ATPMODE)
  {
     print &PerlTranslate("instrum.pl", "Warning: Memory Profiling feature is not available for Ada Language.\n");
     $ATPMODE = 0;
  }

  if ($ATQMODE)
  {
     print &PerlTranslate("instrum.pl", "Warning: Performance Profiling feature is not available for Ada Language.\n");
     $ATQMODE = 0;
  }

  if ($ATOMODE)
  {
     print &PerlTranslate("instrum.pl", "Warning: Object Testing feature is not available for Ada Language.\n");
     $ATOMODE = 0;
  }

  return $result;
}


################################################################################
# BuildJavaInstrOptions
# Arguments:
#   main source directory
# Return value: 
#   the options string
################################################################################
sub BuildJavaInstrOptions #()
{
  my $result = "";
  my $classpath;

  $result .= BuildGeneralCommonOptions($TDP_LANGUAGE,"","");

  if ($ATCMODE)
  {
     $result .= BuildCoverageCommonOptions( 0, 0, 0 );

     # -noternary
     if ( ! $ATC_SELECTIVE_TERNARY )
     {
       $result .= " -noternary";
     }

     # -compact|-count
     if ( $ATC_INFORMATION == 1 ) 
     {
       $result .= " -compact";
     }
     elsif ( $ATC_INFORMATION == 2 ) 
     {
       $result .= " -count";
     }

     # -comment
     if ( $ATC_USER_COMMENT ne "" )
     {
       $result .= " -comment=\"$ATC_USER_COMMENT\"";
     }
  }

  if ($ATTMODE)
  {
     $result .= " -trace";
  }

  if ($ATQMODE)
  {
     $result .= " -perfpro";
  }

  my $separator = ($OSNAME eq "win32") ? ";" : ":";

  if ($ATJMODE)
  {
     #a2m: $PROJECT_PATH: directory where startjava generates files to be inst.
     $classpath = BuildCompleteJavaClassPath("-classpath=",$separator,"$TARGETDIR/test$separator"."$PROJECT_PATH");
     $result .= " -jtest";
  }
  else
  {
     $classpath = BuildCompleteJavaClassPath("-classpath=",$separator,"");
  }

  $result .= " " . $classpath if ($classpath);

  return $result;
}


################################################################################
# BuildCCppInstrOptions
# Arguments:
#   files not to instrument
# Return value: 
#   the options string
################################################################################
sub BuildCCppInstrOptions #($$$$)
{
  my ($lan,$exfile, $exunit, $atu_rename_file) = @_;
  my $result = "";
  my $is_c = ($lan eq "CPP") ? 0 : 1;
  my @SETS=();

  if ($ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || $atu_rename_file ne "")
  {
    $result .= BuildGeneralCommonOptions($lan,$exfile, $exunit);
    $result .= " -noinstrdir=\"$TARGETDIR\"";
    if ($Ini{'NOPATH'})
    {
      $result .= " -nopath";
    }
    if ($Ini{'NO_INSTR_GENERATED_LINE_INFO'})
    {
      $result .= " -nodline";
    }
  }

  if ($atu_rename_file ne "")
  {
    $result .= " \@$atu_rename_file";
  }

  if ($ATCMODE)
  {
     $result .= BuildCoverageCommonOptions( 0, $is_c, 1 );

     # -noternary
     if (!$ATC_SELECTIVE_TERNARY)
     {
       $result .= " -noternary";
     }

     # -noinline
     if ($TDP_LANGUAGE eq "CPP" && !$ATC_SELECTIVE_INLINE)
     {
       $result .= " -noinline";
     }

     # -noinline
     if (!$ATC_SELECTIVE_INCLUDED)
     {
       $result .= " -noinclude";
     }

     # -compact|-count
     if ( $ATC_INFORMATION == 1 ) 
     {
       $result .= " -compact";
     }
     elsif ( $ATC_INFORMATION == 2 ) 
     {
       $result .= " -count";
     }

     # -comment
     if ($ATC_USER_COMMENT ne "")
     {
       $result .= " -comment=\"$ATC_USER_COMMENT\"";
     }
  }

  if ($ATTMODE)
  {
     $result .= " -trace";

     # -before_return_expr
     if ($lan eq "C" &&
         $ATT_CALLING_SEQUENCED)
     {
       $result .= " -before_return_expr";
     }

     # -no_template_note
     if ($lan eq "CPP" &&
         !$ATT_TEMPLATE_NOTES)
     {
       $result .= " -no_template_note";
     }
  }

  if ($ATPMODE)
  {
     $result .= " -mempro";

     # -noinspect
     if ($Ini{'ATP_ALTERNATE_GLOBALDATA_INSPECTION'})
     {
       $result .= " -noinspect";
     }
     else
     {
       $result .= " -noinspect=\"".join("\",\"",@ATP_INSTR_CTRL_LIST)."\"" if (@ATP_INSTR_CTRL_LIST);
       $result .= " -noinspectdir=\"".join("\",\"",@ATP_NOINSPECTDIR)."\"" if (@ATP_NOINSPECTDIR);
     }
  }

  if ($ATQMODE)
  {
     # -perfpro[=<os|process>]]
     $result .= " -perfpro";
     if ($ATL_CLOCK_KIND)
     {
       $result .= "=process";
     }
     if ($ATQ_COMPUTE_MINMAX)
     {
       push(@SETS,"MINMAX");
     }
  }
  # check language because a C file may be in the file list
  if (($ATKMODE || $ATOMODE) && (!$is_c))
  {
    my $comma = 0;
    foreach $otifile (@OTIFILES)
    {
      if ($comma)
      {
	$result .= ",\"$otifile\"";
      }
      else
      {
	$result .= " -otifile=\"$otifile\"";
        $comma = 1;
      }
#      push(@SETS,"ATKONLY") if(!$ATOMODE);
#     push(@SETS,"ATOMODE") if($ATOMODE);
    }
    if ($ATO_FRIEND)
    {
      $result .= " -friend_test_class";
    }
    if (defined ($ATO_INSTR_BODY))
    {
      $result .= " -body=$ATO_INSTR_BODY";
      if ($ATO_INSTR_BODY eq "MAP_FILE" && defined ($ATO_INSTR_MAP_FILE))
      {
	 $result .= " -mapfile=\"$ATO_INSTR_MAP_FILE\"";
      }
    }
    if ($ATK_INSTR_CONST)
    {
      $result .= " -instr_const";
    }
    if ($ATK_MT_SUPPORT)
    {
      $result .= " -mtsupport";
    }
  }

  push(@SETS,"CHECK_STACK") if($ATL_CHECK_STACK);
  push(@SETS,$USE_UNLOAD_DLL) if($USE_UNLOAD_DLL ne "" && $USE_UNLOAD_DLL ne "NONE");
  push(@SETS,"TPDLLIMPORT") if($ATL_RUNTIME_FORMAT eq "DLL");
  push(@SETS,"NO_SYS_INCLUDE") if($NO_SYS_INCLUDE);
  push(@SETS,"DYNTABLES") if($DYNTABLES);
  push(@SETS,"MULTITHREAD") if($ATL_MULTI_THREADS);
  push(@SETS,"LINENUMOPTIM") if($LINENUMOPTIM);
  $result .= " -SET=".join(",",@SETS) if(@SETS);
  return $result;
}

1;
################################################################################

