#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                          -*- javic command -*-                               #
#                                                                              #
#                                                                              #
################################################################################

use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

################################################################################
# Where is the TDP
$TARGETDIR = $ENV{'ATLTGT'};
# Where the Product is installed
$SCRIPTDIR = $ENV{'TESTRTDIR'} . "/lib/scripts/";
# The name of the log file
$LOGFILENAME = "javic.log";
# Which system ?
$OSNAME = ($^O=~/win32/i) ? "win32" : "unix";
# Default value for intermediate directory : can be overriden by $UserEnv
$INTDIR = "javi.jir";
# Ini File to be used
$INIFILE = "tp.ini";
# BootClassPath
my $BootClassPath = "";
# ClassPath
my $ClassPath = "";
my $ShowOnly;
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Products.pl";

# Where is the instrumentor binary
$BINDIR = GetBinDir() if ($BINDIR eq "");

################################################################################
# List of instrumentation options (the one before --)
my @InstrumentorOptions = ();
# Maybe -studio_log=<log-file> is found on command-line
my $UserLogFileName = "";
# Source Files List
my @SourceFiles = ();
# Instrumented Source Files List
my @InstrSourceFiles = ();
# Compiler command (first arg after --)
my $Compiler = "";
# ... and its position in the list
my $CompilerIndex = -1;
# List of instrumentation options (the one after --)
my @CompilerOptions = ();
# Destination Directory
my $DestDir = ".";
# Intermediate Directory
my $IntDirIndex = -1;
# ClassPath Argument position
my $ClassPathIndex = -1;
# argument of -source 
my $SourceVersionIndex = -1;
# maybe -opp=<opp-file> is found on instr. commannd line
my $UserOpp = "";

################################################################################
sub JavicFatalError
{
  my $msg = shift;

  print ("$SMALL_PRODUCT_NAME javic Error: " . $msg);
  exit (1);
}

################################################################################
sub JavicVerbose
{
  my $msg = shift;

  print ($msg) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
}

################################################################################
sub JavicDebug
{
  my $msg = shift;

  print ("$SMALL_PRODUCT_NAME javic Debug: " . $msg) if ($ENV{'ATTOLSTUDIO_DEBUG'});
}

################################################################################
sub JavicSystemPError
{
  my $cmd = shift;

  if ($ENV{'ATTOLSTUDIO_VERBOSE'})
  {
    exit (1);
  }
  else
  {
    JavicFatalError ("The following command-line caused an error:\n" .
                      $cmd . "\n");
  }
}

################################################################################
sub JavicEnvironment
{
  JavicVerbose ("Perl Script Command Line (" . scalar(@ARGV) . " args):\n" .
    "  javic.pl [" . join ("] [", @ARGV) . "]\n");
  JavicVerbose ("     ATLTGT     = " . $ENV{'ATLTGT'} . "\n");
  JavicVerbose ("     ATTOLBIN   = " . $ENV{'ATTOLBIN'} . "\n");
  JavicVerbose ("     TESTRTDIR  = " . $ENV{'TESTRTDIR'} . "\n");
}

################################################################################
sub ParseArgs
{
  my $i = 0;
  my $ignore_arg = 0;
parse_loop:
  foreach $arg (@ARGV) {
    JavicDebug ("\$arg[$i] = [$arg]\n");
    if ($ignore_arg)
    {
      $ignore_arg =0;
      next parse_loop;
    }
    if ($arg eq "--")
    {
      if ($CompilerIndex >= 0)
      {
        JavicFatalError ("Command string separator (--) found more than once\n");
      }
      $CompilerIndex = $i + 1;
      JavicDebug ("Found command string separator (--) at index $i\n");
    }
    else
    {
      if ($CompilerIndex >= 0)
      {
        my @file_list = glob($arg);
        if ($i == $CompilerIndex)
        {
          $Compiler = @ARGV[$CompilerIndex];
          JavicDebug ("\$CompilerIndex = $CompilerIndex\n");
        }
        elsif (-f $arg)
        {
          my $ext = GetExtension ($arg);
          JavicDebug ("Is $arg [$ext] a Java File?\n");
          my $language = LanguageOf ($ext, "Unknown");
          JavicDebug ("\$language = $language\n");
          if ($language eq "JAVA")
          {
            @SourceFiles = (@SourceFiles, $arg);
          }
          else
          {
            @CompilerOptions = (@CompilerOptions, $arg);
          }
        }
        elsif (scalar(@file_list)>0 && -f @file_list[0])
        {
          foreach $f (@file_list)
          {
            my $ext = GetExtension ($f);
            JavicDebug ("Is $f [$ext] a Java File?\n");
            my $language = LanguageOf ($ext, "Unknown");
            JavicDebug ("\$language = $language\n");
            if ($language eq "JAVA")
            {
              @SourceFiles = (@SourceFiles, $f);
            }
            else
            {
              @CompilerOptions = (@CompilerOptions, $f);
            }
          }
        }
        else
        {
          if ($arg eq "-d") {
            $IntDirIndex = $i + 1;
            @CompilerOptions = (@CompilerOptions, $arg);
          } elsif ($arg eq "-bootclasspath") {
            $BootClassPath .= ( $BootClassPath ? $CLASSPATH_SEPARATOR : "" ) . @ARGV[$i+1];
            @CompilerOptions = (@CompilerOptions, $arg);
          } elsif ($arg eq "-classpath") {
            $ClassPath .= ( $ClassPath ? $CLASSPATH_SEPARATOR : "" ) . @ARGV[$i+1];
            $ClassPathIndex = $i + 1;
            #skip classpath at next loop, so if the whole classpath exists on disk
            #don't get it as a java source file to compile.
            $ignore_arg=1;
          } elsif ($arg eq "-source") {
            $SourceVersionIndex = $i + 1;
            @CompilerOptions = (@CompilerOptions, $arg);
          } elsif ($ClassPathIndex != $i) {
            @CompilerOptions = (@CompilerOptions, $arg);
          }
        }
      }
      else
      {
        if (lc($arg) =~ /\-studio_log\=.*/)
        {
          ($UserLogFileName) = ($arg =~ /\-.*\=(.*)/);
          JavicDebug ("\$UserLogFileName=$UserLogFileName\n");
        }
        elsif (lc($arg) =~ /\-opp\=.*/)
        {
          ($UserOpp) = ($arg =~ /\-opp\=(.*)/);
          JavicDebug ("\$UserOpp=$UserOpp\n");
        }
        elsif (lc($arg) =~ /\-ve?/i)
        {
          #done by the launcher
        }
        elsif (lc($arg) =~ /\-\#\#?/) #show only
        {
          $showCmd=1;
        }
        elsif ( ! IsASetting($arg) )
        {
          @InstrumentorOptions = (@InstrumentorOptions, $arg);
        }
      }
    }
    $i += 1;
  }

  if ($CompilerIndex == -1)
  {
    JavicFatalError ("Command string separator (--) not found\n");
  }

  if (-z $Compiler)
  {
    JavicFatalError ("Compiler name not found\n");
  }

  if (scalar(@SourceFiles) == 0)
  {
    JavicFatalError ("No existing source files to instrument\n");
  }

  if ($BootClassPath eq "")
  {
    $BootClassPath = $Ini{'DEFAULT_BOOTCLASSPATH'};
  }

  if ($ClassPath eq "")
  {
    $ClassPath = $ENV{'CLASSPATH'};
  }

  if ($IntDirIndex != -1)
  {
    $DestDir = @ARGV[$IntDirIndex];
    $INTDIR = "$DestDir/javi.jir";
  }

  JavicDebug ("BootClassPath=$BootClassPath\n");
  JavicDebug ("ClassPath=$ClassPath\n");
  JavicDebug ("InstrumentorOptions (" . scalar(@InstrumentorOptions) . "):\n" .
    "  [" . join ("] [", @InstrumentorOptions) . "]\n");
  JavicDebug ("SourceFiles (" . scalar(@SourceFiles) . "):\n" .
    "  [" . join ("] [", @SourceFiles) . "]\n");
  JavicDebug ("Compiler:\n  [" . $Compiler . "]\n");
  JavicDebug ("CompilerOptions (" . scalar(@CompilerOptions) . "):\n" .
    "  [" . join ("] [", @CompilerOptions) . "]\n");

}

################################################################################
sub Instrument
{
  if (-f $LOGFILENAME)
  {
    ClearLogFile ($LOGFILENAME);
  }

  my $command  = $BINDIR . $PATH_SEPARATOR . "javi";

  if ($BootClassPath || $ClassPath) {
    my $classpath = "$BootClassPath" . "$CLASSPATH_SEPARATOR" . "$ClassPath";
    $command .= " -classpath=\"$classpath\"" . "$CLASSPATH_SEPARATOR" . ".";
    if ($ClassPath) {
      $ClassPath = "$TARGETDIR" . $PATH_SEPARATOR . "lib" . "$CLASSPATH_SEPARATOR" . $ClassPath;
    } else {
      $ClassPath = "$TARGETDIR" . $PATH_SEPARATOR . "lib";
    }
  }
  $command .= " \"-studio_log=$LOGFILENAME\"";
  $command .= " \"" . join ("\" \"", @InstrumentorOptions) . "\""
    if (scalar(@InstrumentorOptions) > 0);
  $command .= " \"" . join ("\" \"", @SourceFiles) . "\"";

  $command .= " \"-destdir=$DestDir\"";

  my $wopp;
  if( $SourceVersionIndex > 0 )
  {
    my $source_version = @ARGV[$SourceVersionIndex];
    my $opp =  $UserOpp;
    $wopp = "atl4java.opp";
    if( ! -f $opp )
    {
      $opp = "$TARGETDIR"."$PATH_SEPARATOR"."ana"."$PATH_SEPARATOR"."atl.opp";
      if( -f $opp )
      {
        atl_copy( $opp, $wopp, true );
      }
    }     
    open (OPP, ">>$wopp" ) or JavicFatalError("Cannot open $wopp ($!)");
    print OPP "--source $source_version\n";
    close (OPP);
    $command .= " \"-opp=$wopp\"";
  }
  elsif ( $UserOpp )
  {
    $command .= " \"-opp=$UserOpp\"";
  }

  my $status = SystemP ($command);
  JavicDebug ("\$command=($command) returned $status\n");
  if ($status)
  {
    JavicSystemPError ($command . "\n");
  }

  if (! open (LOG, "<$LOGFILENAME" ))
  {
    JavicFatalError ("Cannot open $LOGFILENAME");
  }
  if ($UserLogFileName &&
      ! open (USERLOG, ">>$UserLogFileName" ))
  {
    JavicFatalError ("Cannot open $UserLogFileName");
  }

  while ($line = <LOG>)
  {
    JavicDebug ($line);
    if ( $line =~ /^SRC /)
    {
      my ($src) = ($line =~ /^SRC (.*)/);
      JavicDebug ("\$src=$src\n");
      @InstrSourceFiles = (@InstrSourceFiles, $src);
    }
    if ($UserLogFileName)
    {
      print USERLOG $line;
    }
  }

  close (USERLOG) if ($UserLogFileName);
  close (LOG);

  ClearLogFile ($LOGFILENAME);

  # Some late checks
  if (scalar(@InstrSourceFiles) == 0)
  {
    JavicFatalError ("No instrumented source files to compile\n");
  }

  unlink ("$wopp") if ($wopp);
}

################################################################################
sub Old_Compile
{
  $command  = $Compiler;

  system('echo "==========================="');
  system('echo "Current directory is "; pwd');
  system('echo "==========================="');

### Modification 10/07/2003 to use sourcepath option
    ### this line has been modified by the line below => $command .= " \"" . join ("\" \"", @InstrSourceFiles) . "\"";
  my $IntDir = $INTDIR;
  $IntDir =~ s,/javi.jir,/,;
    $command .= " -sourcepath  \"" . join ("\" \"", $IntDir ) . "\"";
### End Modification

  open (JAVA_SRC, ">java_src.txt") || die "Can not open temporary file java_src.txt\n";

  foreach (@InstrSourceFiles)
    {
      my $rel_path = $_;
      $rel_path =~ s,.*/javi.jir/,javi.jir/,;
      print JAVA_SRC $rel_path, "\n";
    }
  close (JAVA_SRC);

#  $command .= " \"" . join ("\" \"", @InstrSourceFiles) . "\"";
  $command .= " \@java_src.txt";

  $command .= " \"" . GetJavaProductFile() . "\"";
  $command .= " \"" . join ("\" \"", @CompilerOptions) . "\""
    if (scalar(@CompilerOptions) > 0);
  my $OldClasspath = $ENV{CLASSPATH};
  $ENV{CLASSPATH} = $ClassPath . $CLASSPATH_SEPARATOR . "$TARGETDIR" . $PATH_SEPARATOR . "lib";
  $ENV{CLASSPATH} =~ s/:\.://g;

#  $command .= " -classpath \"" . "$ClassPath" . $CLASSPATH_SEPARATOR . "$TARGETDIR" . $PATH_SEPARATOR . "lib" . "\"";
  JavicDebug ("CLASSPATH = (".$ENV{CLASSPATH}.")\n");
  if ($IntDirIndex == -1)
  {
    $command .= " -d .";
  }
  $status = SystemP ($command);
  unlink ("java_src.txt");
  JavicDebug ("\$command=($command) returned $status\n");
  if ($status)
  {
    JavicSystemPError ($command . "\n");
  }
  $ENV{CLASSPATH} = $OldClasspath;
}

################################################################################
sub Compile
{
  print("===========================\n");
  print("Current directory is '" . GetCurrentDir() . "'\n");
  print("=========================== \n");

  # Put the classpath in the environment variable
  my $OldClasspath = $ENV{CLASSPATH};
  $ENV{CLASSPATH} = $ClassPath . $CLASSPATH_SEPARATOR . "$TARGETDIR" . $PATH_SEPARATOR . "lib";
  JavicVerbose ("CLASSPATH = (".$ENV{CLASSPATH}.")\n");

  open (JAVA_SRC, ">java_src.txt") || die "Can not open temporary file java_src.txt\n";
  my $SourceLineLength = 0;
  foreach (@InstrSourceFiles)
    {
	print JAVA_SRC $_, "\n";
	$SourceLineLength += length $_;
	if ($SourceLineLength > 30000)
	{
	    # Line will be come to long so compile what we have
	    close (JAVA_SRC);
	    my $command  = $Compiler;
	    $command .= " \@java_src.txt";
	    $command .= " \"" . GetJavaProductFile() . "\"";
	    $command .= " \"" . join ("\" \"", @CompilerOptions) . "\"" if (scalar(@CompilerOptions) > 0);
	    $command .= " -d ." if ($IntDirIndex == -1);
	    JavicDebug ("command=($command)\n");
	    $status = SystemP ($command);
	    JavicDebug ("returned $status\n");
	    if ($status)
	    {
		JavicSystemPError ($command . "\n");
	    }
	    # Reset for the next compile
	    $SourceLineLength = 0;
	    open (JAVA_SRC, ">java_src.txt") || die "Can not open temporary file java_src.txt\n";
	}

    }
  # Run the compiler
  close (JAVA_SRC);
  my $command  = $Compiler;
  $command .= " \@java_src.txt";
  $command .= " \"" . GetJavaProductFile() . "\"";
  $command .= " \"" . join ("\" \"", @CompilerOptions) . "\"" if (scalar(@CompilerOptions) > 0);
  $command .= " -d ." if ($IntDirIndex == -1);
  JavicDebug ("command=($command)\n");
  $status = SystemP ($command);
  unlink ("java_src.txt");
  JavicDebug ("returned $status\n");
  if ($status)
  {
      JavicSystemPError ($command . "\n");
  }

  $ENV{CLASSPATH} = $OldClasspath;
}

################################################################################

# Maybe something to print first
JavicEnvironment();

# Let's start here
EnterInScript();

# Default settings
require $SCRIPTDIR . "BatchJavaDefaults.pl";

# First parse command-line
ParseArgs();

# Then Instrument what can be instrumented
Instrument();
# Let's prepare TDP
PrepareJavaTargetPackage();
# Then Compile the Instrumented Files
Compile();

1;
#
################################################################################
