#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                          -*- metcc command -*-                               #
#                                                                              #
#                                                                              #
################################################################################

require $SCRIPTDIR . "Fcts.pl";

################################################################################
# Where is the TDP
$TARGETDIR = $ENV{'ATLTGT'};

$BINDIR = GetBinDir() if ($BINDIR eq "");

# The name of the log file
$LOGFILENAME = "metcc.log";
# Which system ?
$OSNAME = ($^O=~/win32/i) ? "win32" : "unix";

#Default Ini File
$INIFILE="tp.ini";
require $SCRIPTDIR . "Fcts.pl";

################################################################################
# List of instrumentation options (the one before --)
my @MetricsOptions = ();
# Maybe -studio_log=<log-file> is found on command-line
my $UserLogFileName = "";
# Compiler command (first arg after --)
my $Compiler = "";
# ... and its position in the list
my $CompilerIndex = -1;
# Source Files List
my @CSourceFiles = ();
my @CPPSourceFiles = ();
# List of defines (after --)
my @DefinesList = ();
# List of include-dirs (after --)
my @IncludeDirs = ();
# List of compiler options (the one after --)
my @StandardDefine = ();
# Standard definition file for attolstartC
my @CompilerOptions = ();
# Destination Directory
my $MetricsDir = ".";
# Destination Directory Index
my $MetricsDirIndex = -1;
# Directory Restriction
my @MetricsDirRestrict = ();
my $MetricsDirRestrictIndex = -1;

################################################################################
sub MetccFatalError #($)
{
  my $msg = shift;

  print ("$SMALL_PRODUCT_NAME metcc Error: " . $msg);
  exit (1);
}

################################################################################
sub MetccVerbose #($) 
{
  my $msg = shift;

  print ($msg) if ($ENV{'ATTOLSTUDIO_VERBOSE'});
}

################################################################################
sub MetccDebug #($)
{
  my $msg = shift;

  print ("$SMALL_PRODUCT_NAME metcc Debug: " . $msg) if ($ENV{'ATTOLSTUDIO_DEBUG'});
}

################################################################################
sub MetccSystemPError #($)
{
  my $cmd = shift;

  if ($ENV{'ATTOLSTUDIO_VERBOSE'})
  {
    exit (1);
  }
  else
  {
    &MetccFatalError ("The following command-line caused an error:\n" .
                      $cmd . "\n");
  }
}

################################################################################
sub FileArg #($)
{
  my $f = shift;
  
    my $ext = &GetExtension ($f);
    &MetccDebug ("Is $f [$ext] a C/C++ File?\n");
    my $language = &LanguageOf ($ext, "Unknown");
    &MetccDebug ("\$language = $language\n");
    if ($language eq "C")
    {
      push(@CSourceFiles, $f);
    }
    elsif ($language eq "CPP")
    {
      push(@CPPSourceFiles, $f);
    }
    else
    {
      push(@CompilerOptions, $f);
    }
}
################################################################################
sub getMetccOptions #($$)
{
  my ($i,$arg) = @_;
  &MetccDebug ("Metcc option[$i]=$arg ");
        if ($i == $CompilerIndex)
        {
          $Compiler=$arg;
	  &MetccDebug ("compiler name \$CompilerIndex = $CompilerIndex\n");
          return(1); # getMetccOptions job ends
        }
	elsif($i == $MetricsDirIndex)
	{
	   $MetricsDir = $arg;
	   &MetccDebug ("MetricsDir\n");
	}
	elsif($i == $MetricsDirRestrictIndex)
	{
	   push(@MetricsDirRestrict,split(/,/,$arg));
	   &MetccDebug ("MetricsDirRestrict $arg\n");
	}
	elsif ($arg eq "--")
	{
	   $CompilerIndex = $i + 1;
	   &MetccDebug ("command string separator\n");
	}
	elsif (lc($arg) =~ /\-studio_log\=(.*)$/)
	{
	  $UserLogFileName = $1;
	  &MetccDebug ("\$UserLogFileName=$UserLogFileName\n");
	}
	elsif (lc($arg) =~ /^\-metrics\=(.*)$/)
	{
	  $MetricsDir = $1;
	  &MetccDebug ("MetricsDir $MetricsDir\n");	  
	}
	elsif (lc($arg) =~ /^\-metrics$/)
	{
	  $MetricsDirIndex = $i + 1;
	  &MetccDebug ("MetricsDir option next\n");	  
	}
	elsif (lc($arg) =~ /^\-metrics_in_source$/)
	{
	  $MetricsInSource = 1;
	  &MetccDebug ("MetricsDir in source dir\n");	  
	}
	elsif (lc($arg) =~ /^\-restrict_dir_metrics=(.*)$/)
	{
#	  my ($dirs) = ($arg =~ /^\-restrict_dir_metrics=(.*)$/);
#	  my @dirlist = split(/,/,$dirs);
#	  @MetricsDirRestrict = (@MetricsDirRestrict, @dirlist);
	  push(@MetricsDirRestrict,split(/,/,$1));
	  &MetccDebug ("MetricsDirRestrict....\n");	  
	}
	elsif (lc($arg) =~ /^\-restrict_dir_metrics$/)
	{
	  $MetricsDirRestrictIndex = $i +1;
	  &MetccDebug ("MetricsDirRestrict option next\n");
	}
	else
	{
	  push(@MetricsOptions, $arg);
	  &MetccDebug ("MetricsOptions $arg\n");
	}
     return (0); # not finish
}
################################################################################
sub pushCompilerOptions #($)
{
   push(@CompilerOptions,@_);
}
################################################################################
sub pushStandardDefine #($)
{
   push(@StandardDefine,@_);
}
################################################################################
sub pushIncludeDirs #($)
{
   push(@IncludeDirs,@_);
}
################################################################################
sub pushDefinesList #($)
{
   push(@DefinesList,@_);
}
################################################################################
sub MetccCheckArgs
{
  if ($CompilerIndex == -1)
  {
    &MetccFatalError ("Command string separator (--) not found\n");
  }

  if (-z $Compiler)
  {
    &MetccFatalError ("Compiler name not found\n");
  }

  &MetccDebug ("MetricsDir:\n  [" . $MetricsDir . "]\n");
  &MetccDebug ("MetricsDirRestrict (" . scalar(@MetricsDirRestrict) . "):\n" .
    "  [" . join ("] [", @MetricsDirRestrict) . "]\n");
  &MetccDebug ("MetricsOptions (" . scalar(@MetricsOptions) . "):\n" .
    "  [" . join ("] [", @MetricsOptions) . "]\n");
  &MetccDebug ("CSourceFiles (" . scalar(@CSourceFiles) . "):\n" .
    "  [" . join ("] [", @CSourceFiles) . "]\n");
  &MetccDebug ("CPPSourceFiles (" . scalar(@CPPSourceFiles) . "):\n" .
    "  [" . join ("] [", @CPPSourceFiles) . "]\n");
  &MetccDebug ("Compiler:\n  [" . $Compiler . "]\n");
  &MetccDebug ("DefinesList (" . scalar(@DefinesList) . "):\n" .
    "  [" . join ("] [", @DefinesList) . "]\n");
  &MetccDebug ("IncludeDirs (" . scalar(@IncludeDirs) . "):\n" .
    "  [" . join ("] [", @IncludeDirs) . "]\n");
  &MetccDebug ("StandardDefine (" . scalar(@StandardDefine) . "):\n" .
    "  [" . join ("] [", @StandardDefine) . "]\n");
  &MetccDebug ("CompilerOptions (" . scalar(@CompilerOptions) . "):\n" .
    "  [" . join ("] [", @CompilerOptions) . "]\n");

}

################################################################################
sub MetccEnvironment
{
  &MetccVerbose ("Perl Script Command Line (" . scalar(@ARGV) . " args):\n" .
    "  metcc.pl [" . join ("] [", @ARGV) . "]\n");
  &MetccVerbose ("     ATLTGT     = " . $ENV{'ATLTGT'} . "\n");
  &MetccVerbose ("     ATTOLBIN   = " . $ENV{'ATTOLBIN'} . "\n");
  &MetccVerbose ("     TESTRTDIR  = " . $ENV{'TESTRTDIR'} . "\n");
}

################################################################################
sub ComputeCMetrics
{
  if (-f $LOGFILENAME)
  {
    &ClearLogFile ($LOGFILENAME);
  }

  if (scalar(@CSourceFiles) == 0)
  {
    &MetccDebug("No C source file to compute\n");
    return;
  }

  my
  #$command  = "$BINDIR" . "$PATH_SEPARATOR" . "attolstartC";
  $command  = "attolstartC"; # bindir is in the PATH and make problem with "Program File"
  $command .= ".exe" if ($OSNAME eq "win32");

  $command .= " \"" . join ("\" \"", @CSourceFiles) . "\"";

  $command .= " -studio_log=\"$LOGFILENAME\"";

  $command .= " \"" . join ("\" \"", @MetricsOptions) . "\""
    if (scalar(@MetricsOptions) > 0);
  
  $command .= " -metrics";
  $command .= "=\"$MetricsDir\"" unless($MetricsInSource);

  if (scalar(@MetricsDirRestrict))
  {
    $command .= " -restrict_dir_metrics=\"" . join ("\",\"", @MetricsDirRestrict) . "\"";
  }
  
  if (scalar(@DefinesList))
  {
    $command .= " -define=\"" . join ("\",\"", @DefinesList) . "\"";
  }
  
  if (scalar(@IncludeDirs))
  {
    $command .= " -incl=\"" . join ("\",\"", @IncludeDirs) . "\"";
  }

  if (scalar(@StandardDefine))
  {
    $command .= " -std_define=\"" . join ("\",\"", @StandardDefine) . "\"";
  }
  else
  {
    $command .= " -std_define=\"" . $TARGETDIR . $PATH_SEPARATOR . "ana" . $PATH_SEPARATOR . "atus_c.def\"";
  }
  
  $command .= " ".$Ini{'METRIC_OPT'} if($Ini{'METRIC_OPT'});
  
  my $status = SystemP ($command);
  &MetccDebug ("\$command=($command) returned $status\n");
  if ($status)
  {
    &MetccSystemPError ($command . "\n");
  } 
  
  if (! open (LOG, "<$LOGFILENAME" ))
  {
    &MetccFatalError ("Cannot open $LOGFILENAME\n");
  } 
  if ($UserLogFileName &&
      ! open (USERLOG, ">>$UserLogFileName" ))
  {
    &MetccFatalError ("Cannot open $UserLogFileName\n");
  } 
  
  while ($line = <LOG>)
  {
    &MetccDebug ($line);
    if ( $line =~ /^MET /)
    {
      my ($src) = ($line =~ /^MET (.*)/);
      &MetccDebug ("\$src=$src\n");
    }
    if ($UserLogFileName)
    {
      print USERLOG $line;
    }
  }

  close (USERLOG) if ($UserLogFileName);
  close (LOG);

  &ClearLogFile ($LOGFILENAME);
}

################################################################################
sub ComputeCPPMetrics
{
  if (-f $LOGFILENAME)
  {
    &ClearLogFile ($LOGFILENAME);
  }

  if (scalar(@CPPSourceFiles) == 0)
  {
    &MetccDebug("No CPP source file to compute\n");
    return;
  }

  my
  #$command  = "$BINDIR" . "$PATH_SEPARATOR" . "atostart";
  $command  = "atostart";  # bindir is in the PATH and make problem with "Program File"

  $command .= " \"" . join ("\" \"", @CPPSourceFiles) . "\"";

  $command .= " -studio_log \"$LOGFILENAME\"";

  $command .= " \"" . join ("\" \"", @MetricsOptions) . "\""
    if (scalar(@MetricsOptions) > 0);
  if($MetricsInSource){
  	$command .= " -metrics_in_source";
  } else {
  	$command .= " -metrics \"$MetricsDir\"";
  }
  if (scalar(@MetricsDirRestrict))
  {
    $command .= " -restrict_dir_metrics \"" . join ("\" -restrict_dir_metrics \"", @MetricsDirRestrict) . "\"";
  }
  
  if (scalar(@DefinesList))
  {
    $command .= " -D\"" . join ("\" -D\"", @DefinesList) . "\"";
  }
  
  if (scalar(@IncludeDirs))
  {
    $command .= " -I\"" . join ("\" -I\"", @IncludeDirs) . "\"";
  }
  $command .= " ".$Ini{'METRIC_OPT'} if($Ini{'METRIC_OPT'});

  my $status = SystemP ($command);
  &MetccDebug ("\$command=($command) returned $status\n");
  if ($status)
  {
    &MetccSystemPError ($command . "\n");
  } 
  
  if (! open (LOG, "<$LOGFILENAME" ))
  {
    &MetccFatalError ("Cannot open $LOGFILENAME\n");
  } 
  if ($UserLogFileName &&
      ! open (USERLOG, ">>$UserLogFileName" ))
  {
    &MetccFatalError ("Cannot open $UserLogFileName\n");
  } 
  
  while ($line = <LOG>)
  {
    &MetccDebug ($line);
    if ( $line =~ /^MET /)
    {
      my ($src) = ($line =~ /^MET (.*)/);
      &MetccDebug ("\$src=$src\n");
    }
    if ($UserLogFileName)
    {
      print USERLOG $line;
    }
  }

  close (USERLOG) if ($UserLogFileName);
  close (LOG);

  &ClearLogFile ($LOGFILENAME);
}

################################################################################

1;
