#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2005-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

# These remote function help to execute the application on a remote machine
#This file allows the execution on remote machine when the remote machine is a unix/linux like, OS.
#This file does not include functions allowing to compile on remote machine.
#This file include 3 different way of disk sharing with the remote machine.

sub Remote_Host 		# returns remote host name
{
	if (defined $ENV{'REMOTE_HOST'}) {
		return $ENV{'REMOTE_HOST'};	# use env. var if defined
	} else {
		return $Ini{'REMOTE_HOST'};	# else fallback to basic setting
	}		
}

sub Remote_User 		# returns remote user name
{
	if (defined $ENV{'REMOTE_USER'}) {
		return $ENV{'REMOTE_USER'};	# use env. var if defined
	} elsif ( $Ini{'REMOTE_USER'} ne "") {
		return $Ini{'REMOTE_USER'};	# else fallback to basic setting if set
	} else {
		return ( ($OSNAME eq "win32") ? $ENV{'USERNAME'} : $ENV{'USER'});	# else use local user name
	}
}

sub Remote_Shell 		# returns remote shell command
{
	my $method = $Ini{'REMOTE_CONNECTION'};
	my $remote_host = Remote_Host();
	my $remote_user = Remote_User();	
	my $remote_shell =  "";

	if ($method=~ m/^ssh/) {
		if ($OSNAME eq "win32") {
			$method=~ s,^ssh$,ssh2,;
			$remote_shell =  "$method $remote_user\@$remote_host";	# Windows: assume SSH Communications' SSH client
		} else {
			$remote_shell =  "$method -n $remote_user\@$remote_host";	# UNIX/Linux: assume OpenSSH SSH client
		}
	} else {
		$method="rsh" unless($method);
		if ($OSNAME eq "win32") {
			$remote_shell =  "$method $remote_host -l $remote_user ";
		} else {
			$remote_shell =  "$method -n -l $remote_user $remote_host";
		}
	}
	return ($remote_shell);
}

sub Remote_Path #($)		# returns remote paths name as used on the remote
{
	my $path = shift;
	my $remote_path = $path;
	$remote_path=~s/\\+/\//g;						# map remaining '\'s to '/'s
	
	my $local_dir=$Ini{'SHARED_LOCAL_DIR'}; $local_dir=~s/\\+/\//g;

	$remote_path=~s/^$local_dir/$Ini{'SHARED_REMOTE_DIR'}/;	# map local share to remote 'user' share
	return ($remote_path);
}

sub atl_exec1 #($$$)	# host and remote share the project directory as disk space
{					# SHARED_REMOTE_DIR specify the remote path of the shared directory. SHARED_LOCAL_DIR specify the local path of the shared directory
my ( $exe,$out,$parameters )= @_;
	my $cwd = cwd(); 
	my $remote_dir = Remote_Path ($cwd);
	my $log="remote.$$.log";
	my $remote_exe = $exe;

	my $remote_command = 	Remote_Shell();
	$remote_command .= " cd $remote_dir;";
	$remote_command .= " chmod a+x $remote_exe;";
	$remote_command .= " export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$remote_dir;";
	$remote_command .= " $remote_exe $parameters;";
	$remote_command .= " echo \"status=\$?\" > $log"; # 2>&1

	$status=SystemP($remote_command);

	if(open(IN,"<$log"))
	{
		my $remote_host = Remote_Host();
		while(<IN>)
		{
			chomp;
			if (/^status=\s*(\d*)\s*$/) {
				$status=$1;
				LAST;
			}
			print "[$remote_host] status $status\n";
		}
		close(IN);
		unlink ( $log );
	}
	return ($status);
}

sub atl_exec2 #($$$)	# host and remote share a fixed work directory as disk space (basic setting NOPATHOUT is required)
{					# SHARED_REMOTE_DIR specify the remote tmp work directory. SHARED_LOCAL_DIR specify the tmp local work directory
my ( $exe,$out,$parameters )= @_;
	my $cwd = cwd(); 
	my $log="remote.$$.log";
    my ($exedir,$exebase,$exeext)= SplitFileName ($exe);
	my $remote_exe = $exebase.'.'.$exeext;
	my ($outdir,$outbase,$outext)= SplitFileName ($out);
	my $remote_out = $outbase.'.'.$outext;
	my $remote_dir = $Ini{SHARED_REMOTE_DIR};
	my $local_dir  = $Ini{SHARED_LOCAL_DIR};
	
	copy($exe,$local_dir.'/'.$remote_exe);
	
	my $remote_command = 	Remote_Shell();
	$remote_command .= " cd $remote_dir;";
	$remote_command .= " chmod a+x $remote_exe;";
	$remote_command .= " export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$remote_dir;";
	$remote_command .= " $remote_exe $parameters;";
	$remote_command .= " echo \"status=\$?\" > $log"; # 2>&1

	$status=SystemP($remote_command);

	rename($local_dir.'/'.$log,$log);
	rename($local_dir.'/'.$remote_out,$out);
	
	if(open(IN,"<$log"))
	{
		my $remote_host = Remote_Host();
		while(<IN>)
		{
			chomp;
			if (/^status=\s*(\d*)\s*$/) {
				$status=$1;
				LAST;
			}
			print "[$remote_host] status $status\n";
		}
		close(IN);
		unlink ( $log );
	}
	return ($status);
}


sub atl_exec3 #($$$)	# host and remote does not share any disk space (basic setting NOPATHOUT/NOPATH is required)
{					# SHARED_REMOTE_DIR specify the remote working directory
					# REMOTE_FTP_USER specify the userName used at ftp connection
					# REMOTE_FTP_PASWD specify the password used at ftp connection
					# REMOTE_FTP_DIR specify the destination directory for all copied file
					# once copied the rsh/ssh command copy the file into its own directory (working directory)
					# then execute it
					# the FTP account must have read access on the exec/rsh account and vice-versa
#FTP account and USER(exec) account on the remote machine may be the same if you modify the .rhosts file as:
#add the the host machine name or the host machine IP followed by a blanc space and '+' to authorize the host machine to connect to the 
#remote machine with and without passwd. So the host machine could connect to the remote via ftp with passwd and rsh without passwd.
#Both using the same account and so the same directory.
#Note that /etc/hosts.equiv should have the same content as well.
my ( $exe,$out,$parameters )= @_;
	my $remote_host = Remote_Host();
	my $remote_user = Remote_User ();
	my $remote_ftp_user = $Ini{'REMOTE_FTP_USER'};
	unless($remote_ftp_user ) {$remote_ftp_user=$remote_user;}
	my $cwd = cwd(); 
	my $log="remote.$$.log";
             my ($exedir,$exebase,$exeext)= SplitFileName ($exe);
	my $remote_exe = $exebase.'.'.$exeext;
	my ($outdir,$outbase,$outext)= SplitFileName ($out);
	$outdir =~ s/[\\\/]+$//;
	my $remote_out = $outbase.'.'.$outext;
	my $remote_dir = $Ini{SHARED_REMOTE_DIR};
	my $remote_ftp_dir=$Ini{'REMOTE_FTP_DIR'};
	my $remote_dir_is_ftp=1 if($remote_dir eq $remote_ftp_dir);
	#send the executable on the remote machine
	open SCR,">send.txt";
       if($OSNAME eq "win32") {
	print SCR "open $remote_host\n";
       } else {
	print SCR "ftp -v -n $remote_host<<EOF\n";
	print SCR "user ";
       }
	print SCR $remote_ftp_user ,"\n";
	print SCR $Ini{'REMOTE_FTP_PASSWD'} ,"\n";
	print SCR "cd $remote_ftp_dir\n";
	print SCR "binary\n";
	print SCR "put $exe\n";
	print SCR "bye\n";
	close SCR;
       if($OSNAME eq "win32") {
	SystemP("ftp -s:send.txt");
       } else {
	chmod(0755,"send.txt");
	SystemP( "./send.txt");
       }
	unlink("send.txt");
	
	#execute the test
	my $remote_command = 	Remote_Shell();
	$remote_command .= " cd $remote_dir ;";
	$remote_command .= " rm $remote_out ;"; #remove the existing result file if any
	my $remote_rm=$remote_command;
	$remote_command .= " cp $remote_ftp_dir/$remote_exe . ;" unless($remote_dir_is_ftp); #copy exe from the FTP directory
	$remote_command .= " chmod a+rx $remote_exe ;"; #set the exec flags
	$remote_command .= " export LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$remote_dir ;";
	$remote_command .= " ./$remote_exe $parameters ;"; #execute it
	$remote_command .= " \"echo status=\$? > $log\";"; #get the exec status
	$remote_command .= " rm $remote_exe"; # 2>&1 #then remove the exe file

	$status=SystemP($remote_command);
	sleep(2);
	
	#get the result files from the remote machine 
	open SCR,">get.txt";
       if($OSNAME eq "win32") {
	print SCR "open $remote_host\n";
       } else {
	print SCR "ftp -v -n $remote_host<<EOF\n";
	print SCR "user ";
       }
	print SCR $remote_ftp_user ,"\n";
	print SCR $Ini{'REMOTE_FTP_PASSWD'} ,"\n";
  unless($remote_dir_is_ftp) {
	print SCR "cd ",$remote_ftp_dir ,"\n";
	print SCR "del $remote_exe\n";
  }
	print SCR "cd $remote_dir\n";
	print SCR "get $log\n";
	print SCR "lcd $outdir \n";
	print SCR "get $remote_out\n";
  if($remote_dir_is_ftp) {
	print SCR "del $log\n";
	print SCR "del $remote_out\n";
  }
	print SCR "bye\n";
	close SCR;

       if($OSNAME eq "win32") {
	SystemP("ftp -s:get.txt");
       } else {
	chmod(0755,"get.txt");
	SystemP( "./get.txt");
       }
	unlink("get.txt");
	
  unless($remote_dir_is_ftp) {
	$remote_command = $remote_rm; #remove the result file
	$remote_command .= " rm $log "; #remove the log file

	SystemP($remote_command); 
  }  	
	if(open(IN,"<$log"))
	{
		while(<IN>)
		{
			chomp;
			if (/^\s*status=\s*(\d*)\s*$/) {
				$status=$1;
				LAST;
			}
			print "[$remote_host] execution status $status\n";
		}
		close(IN);
		unlink ( $log );
	}
	return ($status);
}
1;

