##############################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
##############################################################################
#
# Author: Simon Washbrook
#
# Development notes:
# Option -html not implemented as it does not work
#
##############################################################################


#use strict;
#use TestRTUtil;
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
# This script depends on the followings (both studio'n Target Package)
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Split.pl";

BEGIN {
  if ($^O eq "MSWin32") {
   use File::DosGlob 'glob';
 }
}

$ATKMODE=1;
$ATCMODE=1;
$ATTMODE=1;
$ATPMODE=1;
$ATQMODE=1;

my $run;
my $dirtp=".";
my $projectName="temp";
my $filter;
my (@xrd,@rtx,@crc,@met,@fdc,@tio,@tsf,@tpf,@tqf,@tdf,@spt, @studio_args);
$LOGFILENAME="studioreport.log";
@existed = (0, 0, 0, 0, 0, 0);

##############################################################################
# Help

sub help {
  my ($d,$n,$e) = TestRTUtil::SplitFileName($0);
  print "$n : Creates a temporary project with the test result files passed asparameter.\nThese results can then be exported to HTML\n";
  print "Command line:\n";
  print "\t$n [-help] [-html <dir>] [-keep] [<report files>]\n";
  print "Options:\n";
  print "\t-help Displays this help message\n";
  print "\t-html <dir> create the directory for HTML exports\n";
  print "\t-keep Keeps the temporary project that was generated\n";
  print "\t+filter when the spt file data require filtering using the TDP decode.pl\n";
  print "\tlist of space seperated file names. Only test result files are accepted.\n";
  print "Test result files must have the following extensions\n";
  print "\t.spt        global result file generated by the execution\n";
  print "\t.xrd        Component Testing report file\n";
  print "\t.rtx        Graphical report file\n";
  print "\t.crc        Rule Checker report file\n";
  print "\t.met        Static Metric report file\n";
  print "\t.fdc, .tio  Code Coverage report files\n";
  print "\t.tsf, .tpf  Memory Profiling report files\n";
  print "\t.tsf, .tqf  Performance profiling report files\n";
  print "\t.tsf, .tdf  Runtime Tracing report file\n";
  print "\t.log, .xtp  file providing the listing of files like attolccfiles.log\n";
  print "Files must have absolute paths or be relative to the current directory.\n";
  print "\n\n(c) Copyright 2005-2007 IBM Corp. All right reserved\n";
}
###########################################################################
# end called at the end of the instrumentation (link time)
sub reports
{
  my $log=shift;
  
	my $otsfiles;
	my $staticfiles;
	open LOG, "<$log";
	while(<LOG>)
	{
		chomp;
		if(/^OTS (.+)$/){
		  $otsfiles.=$1.' ';
		}elsif (/^#/){
			#line ignored
		}elsif (/^DUM (.+)$/){
		  my $f=$1;
	  	  unlink($f);
		}elsif (/^(XOB|OTI|CPP|OPT|RI2)/){
			#file is ignored
		}elsif (/^SPT (.+)$/){
		  my $f=$1;
		  my @l=grep(/^\Q$f\E$/, @spt);
		  unless(scalar(@l)) {push(@ARGV,$f);}
		}elsif (/^[A-Z]{3} (.+)$/){
		  my $f=$1;
		  my @l=grep(/^\Q$f\E$/, @ARGV);
		  push(@ARGV,$f) unless(scalar(@l));
		}else{
		  push(@ARGV,$_) unless(/^[#;]/); # this is not a log generated by us
		}
	}
	close(LOG);
	if($otsfiles){
		my $command="atopospro -ots $otsfiles -tdf atltrace.tdf -xrd contract.xrd";
		print "Running the command \n$command\n" ;
		system ($command);
		push(@ARGV,"contract.xrd");
	}
}

##############################################################################
# Parse parameters to extract report file types

sub parseArgs {

  unless (@ARGV) {
	if(-f "attolccfiles.log")
	{
		reports("attolccfiles.log");
	} elsif(-f "TestRTccfiles.log") {
		reports("TestRTccfiles.log");
	} elsif(-f "TestRTccfiles.xtp") {
		push(@ARGV,"TestRTccfiles.xtp");
	} else {
    		print "\nERROR: No arguments\n";
    		help();
    		exit 1;
	}
  }

  my %args;
  my $lmth;
  while (@ARGV) {
    $_ = shift @ARGV;
    s/\"//g;
    print "arg: $_\n" if ($args{verbose});
    if (/\*/) {
      print "\nExpanding argument:\n$_\n" if ($args{verbose});
      #unshift(@studio_args, "\"".$_."\"");
      $_ =~ s,\\,/,g;
      $_ =~ s/(\s)/\\$1/g;
      my @files = glob $_;
#     
      unshift(@ARGV,@files);

    } elsif (/^-help$/) {
      help();
      exit 0;
     } elsif (/^-html$/) {
       my $lmth=1; #my
       $args{html} = shift @ARGV;
       push(@studio_args, "-html $args{html}"); 
       unless (-d $args{html}) {
	mkdir ($args{html},0777) || die "Can not create the html output directory \"".$args{html}."\"\n";
       }
    } elsif (/^-keep$/) {
      $args{keep} = 1;
    } elsif (/^-debug$/) {
      $args{verbose} = 1;
    } elsif (/\+filter$/){
       $filter=1;
    } elsif (/^-/) {
      print "\nERROR: unknown option \"$_\"\n";
      help();
      exit 1;
    } elsif (/\.xrd$/i) {
      push (@xrd, $_);
    } elsif (/\.rtx$/i) {
      push (@rtx, $_);
    } elsif (/\.crc$/i) {
      push (@crc, $_);
    } elsif (/\.met$/i) {
      push (@met, $_);
    } elsif (/\.fdc$/i) {
      push (@fdc, $_);
    } elsif (/\.tio$/i) {
      push (@tio, $_);
    } elsif (/\.tsf$/i) {
      push (@tsf, $_);
    } elsif (/\.tpf$/i) {
      push (@tpf, $_);
    } elsif (/\.tqf$/i) {
      push (@tqf, $_);
    } elsif (/\.tdf$/i) {
      push (@tdf, $_);
    } elsif (/\.spt$/i) {
      my $f=$_;
      my @l=grep(/^\Q$f\E$/, @spt);
      push (@spt, $f) unless(scalar(@l));
    } elsif ((/\.log$/i) || (/\.xtp$/i)) {
      my $f=$_;
      my ($xdir,$xn,$xe)=SplitFileName($f);
      $dirtp=$xdir if($xdir ne "");
      $dirtp=~ s,[\\\/]$,,;
      $projectName = $xn;
      reports ($f);
    } else {
      print "\nERROR: unknown file \"$_\"\n";
      help();
      exit (1);
    }
  }
  @xrd=sort @xrd;
  @crc=sort @crc;
  @met=sort @met;
  @fdc=sort @fdc;
  @tsf=sort @tsf;
  
  if($filter){
    if (($ENV{'ATLTGT'}) &&  (-r "$ENV{'ATLTGT'}/cmd/decode.pl")){
    	require "$ENV{'ATLTGT'}/cmd/decode.pl";
    }else{
          die &PerlTranslate("studioreport.pl", "decode option require decode.pl in the TDP. Not found\n","");
    }
  }

$ATC_TIO_NAME="attolcov.tio";	$ATC_TIO_NAME=$tio[0] if(@tio);
$ATP_TPF_NAME="purifylt.tpf";	$ATP_TPF_NAME=$tpf[0] if(@tpf);
$ATQ_TQF_NAME="quantifylt.tqf";	$ATQ_TQF_NAME=$tqf[0] if(@tqf);
$TDFFILE="atltrace.tdf";	$TDFFILE=     $tdf[0] if(@tdf);

if ( -f $ATC_TIO_NAME ) { $existed[$tio] = 1; }
if ( -f $ATP_TPF_NAME ) { $existed[$tpf] = 1; }
if ( -f $ATQ_TQF_NAME ) { $existed[$tqf] = 1; }
if ( -f $TDFFILE      ) { $existed[$tdf] = 1; }
  foreach my $spt (@spt) {
  	if(-f $spt) {
	  # Let's do it
	  decode($spt) if($filter);
	  print "splitting $spt to $ATC_TIO_NAME $ATP_TPF_NAME $ATQ_TQF_NAME $TDFFILE \n" if ($args{verbose});
	  if (SplitTraces($spt) != 0) {
	    exit(1);
	  }
	 $tdf_file_mode = "a" ; # for the next spt
	} else {print &PerlTranslate("studioreport.pl", "%1 Not found. May be already splitted!\n","%1 is the file to split",$spt);}
  }
  if ( ! $used[$tio] && ! $existed[$tio] ) { unlink ($output_files[$tio]); } else {push(@tio,$ATC_TIO_NAME) if((@fdc) && !(@tio)); }
  if ( ! $used[$tpf] && ! $existed[$tpf] ) { unlink ($output_files[$tpf]); } else {push(@tpf,$ATP_TPF_NAME) if((@tsf) && !(@tpf)); }
  if ( ! $used[$tqf] && ! $existed[$tqf] ) { unlink ($output_files[$tqf]); } else {push(@tqf,$ATQ_TQF_NAME) if((@tsf) && !(@tqf)); }
  if ( ! $used[$tdf] && ! $existed[$tdf] ) { unlink ($output_files[$tdf]); } else {push(@tdf,$TDFFILE)      if((@tsf) && !(@tdf)); }
  unlink ("$LOGFILENAME");


  $args{RTX} = join(',', @rtx);
  $args{XRD} = join(',', @xrd);
  $args{CRC} = join(',', @crc);
  $args{MET} = join(',', @met);
  $args{FDC} = join(',', @fdc);
  $args{TIO} = join(',', @tio);
  $args{TSF} = join(',', @tsf);
  $args{TPF} = join(',', @tpf);
  $args{TQF} = join(',', @tqf);
  $args{TDF} = join(',', @tdf);
  $args{STUDIO} = join(' ',@studio_args).' "'.join('" "', @xrd, @crc, @met, @fdc, @tio, @tsf, @tpf, @tqf, @tdf, @rtx ).'"';
  $run="-execute" if($lmth);
  return %args;
}


##############################################################################
# Generate project

sub generateProject {
  my %args = @_;
  my $typeCode="487543129";
#  my %typeCode=('CRC','2138245498_CRC','XRD','2138245498_XRD','FDC','2138245498_FDC','TIO','2138245498_TIO','TSF','2138245498_TSF' ,'TPF','2138245498_TPF' ,'TQF','2138245498_TQF' ,'TDF','2138245498_TDF','MET','2138245498_MET');
#  my %typeCode=('CRC','2138245498_1242109961','XRD','2138245498_1242437650','FDC','2138245498_1242107404','TIO','2138245498_1241842462','TSF','2138245498_1242306846' ,'TPF','2138245498_1242307102' ,'TQF','2138245498_1242307358' ,'TDF','2138245498_1242304030','MET','2138245498_1243483911');
#  my %typeCode=('CRC', ,'XRD','150676_384','FDC','150676_467','TIO','150676_397','TSF','150676_432' ,'TPF','150676_438' ,'TQF','150676_436' ,'TDF','150676_414');

  open (PRJ, ">$dirtp/${projectName}.rtp") || die "Cannot open the project \"$dirtp/${projectName}.rtp\"\n";
  print PRJ <<"END";
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE project SYSTEM "TestRealTime/project.dtd" >
<project>
  <group>
    <node>
      <data>
        <name>${projectName}</name>
        <quid>239471478</quid>
        <renameable>false</renameable>
        <deletable>false</deletable>
        <open>true</open>
        <sorted>false</sorted>
      </data>
      <current_configuration>viewer</current_configuration>
      <state>0</state>
      <background>false</background>
      <configuration_list>
        <configuration>
          <name>viewer</name>
          <executionFlag>24127</executionFlag>
        </configuration>
      </configuration_list>
    </node>
    <group_child>
      <group>
        <node>
          <data>
            <name>viewer</name>
            <quid>239471645</quid>
            <renameable>true</renameable>
            <deletable>true</deletable>
            <open>true</open>
            <sorted>false</sorted>
          </data>
          <current_configuration>viewer</current_configuration>
          <state>0</state>
          <background>false</background>
          <outfilename_list>
END
  foreach my $type (qw(XRD CRC FDC TIO TSF TPF TQF TDF MET RTX)) {
    print PRJ <<"END";
              <outfilename>
                <filetype>${typeCode}_${type}</filetype>
                <filepath>$args{$type}</filepath>
              </outfilename>
END
}
  print PRJ <<"END";
          </outfilename_list>
          <configuration_list>
            <configuration>
              <name>viewer</name>
            </configuration>
          </configuration_list>
        </node>
        <group_child>
        </group_child>
      </group>
    </group_child>
  </group>
  <object_file>${projectName}.inf</object_file>
  <sort_method>0</sort_method>
  <root_configuration_list>
    <configuration>
      <name>viewer</name>
      <targetName>viewer</targetName>
      <executionFlag>24127</executionFlag>
    </configuration>
  </root_configuration_list>
  <version_file>1</version_file>
</project>
END

  close (PRJ);

  # Generate .inf file
  open (INF, ">$dirtp/${projectName}.inf") || die "Cannot open the project \"$dirtp/${projectName}.inf\"\n";
  print INF <<"END";
<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE objects SYSTEM "C:\Program files\Rational\TestRealTime\lib\project.dtd" >
<objects>
  <version_file>1</version_file>
</objects>

END
  close (INF);

}

##############################################################################
# Run studio with the project.

sub runStudio {
  my %args = @_;

  my $command = "studio ". $args{STUDIO};
    $command .= " $run \"$dirtp/${projectName}.rtp\" ";
  print "Running the command \n$command\n" if ($args{verbose});
  system ($command);
}

##############################################################################
# remove temporary project
sub removeProject {
  my %args = @_;

  unless ($args{keep}) {
    unlink ("$dirtp/${projectName}.rtp");
    unlink ("$dirtp/${projectName}.inf");
  }
}

##############################################################################
# Main program
sub main {
  my %args = parseArgs();
  generateProject(%args);
  runStudio(%args);
  removeProject(%args);
}

###########################################################################

main();
1;

