#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                          -*-  Source Browsing -*-                            #
#                                                                              #
#                                                                              #
#                                                                              #
#  Perl-Script         tags.pl                                                 #
#  Version             v4.0                                                    #
#                                                                              #
#  $Revision: 1.55 $                                                           #
#  $Date: 2007/07/25 12:26:55 $                                                #
#                                                                              #
#  Launch tags tool to determinate control structures into program files       #
#                                                                              #
#  Usage:                                                                      #
#       TestRTperl tags.pl <file list>                                         #
#                                                                              #
#  Problems:                                                                   #
#	Source Compilation: Spec=ads; Bdy=adb                                  #
#	RecompileAdaRuntimeLibraries: Verification NAME/LNAME only             #
#                                                                              #
################################################################################


use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

# Configuration file
require $ARGV[0];

# Miscellaneous functions
require $SCRIPTDIR . "Fcts.pl";


################################################################################
# Here is the beginning
################################################################################
EnterInScript();


###########################################################################
# This routine analyze the command line of script
sub CommandLineAnalyze
{
  return (1);
}

###########################################################################
# Print the USAGE
sub AttolCommandUsage
{
   die &PerlTranslate("tags.pl", "Use command like that : TestRTperl tags.pl <environment file> <file>\nExiting\n");
}

###########################################################################
# return the name of tag script
sub tagscriptOf #($)
{
  local ($ext) = shift ;
  my $FILE = "";

#  if ( $ext[0] ne '.' ) 
#  {
#    $ext = "." . $ext ;
#  }

#print "ext = $ext\n";

  if ( $OSNAME eq "win32" ) {
    $_ = uc $ext;
    SWITCH: {
      (/^H$/ || /^HH$/ || /^HPP$/ || /^C$/ || /^I$/ || /^CPP$/ || /^CXX$/ || /^CC$/ || /^RC\d?$/)
        && do { $FILE="tag_c.pl"; last SWITCH; };

      (/^ADA$/ || /^ADS$/ || /^ADB$/)
        && do { $FILE="tag_ada.pl"; last SWITCH; };

      (/^JAVA$/ || /^JAV$/)
        && do { $FILE="tag_java.pl"; last SWITCH; };

      /^PTU$/
        && do { $FILE="tag_ptu.pl"; last SWITCH; };

      (/^OTD$/ || /^DCL$/ || /^STB$/ || /^OTC$/)
        && do { $FILE="tag_otd.pl"; last SWITCH; };

      do {
        $FILE = "tag_" . lc($ext) . ".pl";
      }
    }
  } else {
    $_ = $ext;
    SWITCH: {
      (/^c$/ || /^i$/ || /^C$/ || /^cpp$/ || /^cxx$/ || /^CXX$/ || /^CC$/ || /^cc$/ || /^H$/ || /^h$/ || /^hh$/ || /^hpp$/)
        && do { $FILE="tag_c.pl"; last SWITCH; };

      (/^ada$/ || /^ads$/ || /^adb$/)
        && do { $FILE="tag_ada.pl"; last SWITCH; };

      (/^java$/ || /^jav$/)
        && do { $FILE="tag_java.pl"; last SWITCH; };

      /^ptu$/
        && do { $FILE="tag_ptu.pl"; last SWITCH; };

      (/^otd$/ || /^dcl$/ || /^stb$/ || /^otc$/)
        && do { $FILE="tag_otd.pl"; last SWITCH; };

      do {
        $FILE = "tag_" . "$ext" . ".pl";
      }
    }
  }

  return $FILE;
}

###########################################################################
# Decide what to do

# analyze command line to get phases
CommandLineAnalyze() || AttolCommandUsage();

my ($dirpath,$basename,$extension) = SplitFileName($FILENAME);

# here is the place to get the corresponding header file, TBD with dependencies
my $args     = $FILENAME;

my $command = "";
my $remove_prepro_file=0;

my $header_mode;
if (uc($extension) eq "H")
{
   $header_mode = 1;
}

my $scriptt = tagscriptOf($extension);

$TagScript="$ATTOLHOME" . "$PATH_SEPARATOR" . "lib" . "$PATH_SEPARATOR"
           . "scripts" . "$PATH_SEPARATOR" . "$scriptt";

if(!(-f $TagScript))
{
  print &PerlTranslate("tags.pl", "No analysis script for file %1\n", "%1 is the name of the file to be tagged", $FILENAME);
  open ( LOG, ">tags" ) or die &PerlTranslate("Source.pl", "Cannot open log file: %1\n", "%1 is the reason given by the Operating System", $!);
  close LOG;
  exit(0);
}

$command = "$BINDIR" . "$PATH_SEPARATOR" . "TestRTperl -I$ATTOLHOME"
         . "$PATH_SEPARATOR" . "lib" . "$PATH_SEPARATOR" . "perl "
         . "$TagScript " . "\"$ARGV[0]\"";

my $status = SystemP($command);

if ($status != 0)
{
    print &PerlTranslate("tags.pl", "Unable to complete command for %1\n", "%1 is the name of the file to be tagged", $FILENAME);
}

exit($status);
#
###############################################################################
