#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################



#use XML::Parser;
use strict;
use File::Basename;
use File::Copy;
use FileHandle ;

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$main::SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
require $main::SCRIPTDIR . "Translate.pl";

my @VersionList=qw(OM SR3 McKinley McKinleyRSR3 Baltic);
#-update:
my $source_file;
my $source_version ;
my $target_version=$VersionList[$#VersionList];


############################################################################
##
#
sub printVersionList
{
  foreach(@VersionList) {
  	print $_."\n"; # NO TRANSLATION
  }
}

############################################################################
##
#
sub showUsage
{
  print STDERR &PerlTranslate("tdpver.pl", "Usage:\n  TestRTperl tdpver.pl [options]\n\nOptions:\n -list : list of all versions.\n -update <file> : TDP file to update\n -fromver <ver> : TDP file version\n -tover <ver>   : TDP version of generated file\n");
  exit 1;
}

sub updateLabels
{
	my $line=shift;

}
############################################################################
##
#
sub parseOptions
{
  my $option_is=0;
  my $opt;

  for ( @ARGV )
  {
    $opt=$_;

    if ($option_is ==0 )
    {
      SWITCH: {
       /^-list$/ && do { printVersionList(); exit 0; } ;
       /^-update$/ && do { $option_is=1; last SWITCH; };
       /^-fromver$/ && do { $option_is=2; last SWITCH; };
       /^-tover$/ && do { $option_is=4; last SWITCH; };
       ( /^-h$/ || /^-help$/ || /^--help$/ ) && do { showUsage(); };
       /^-.*/ && do { print STDERR &PerlTranslate("tdpver.pl", "Error: unknown option %1\n", "%1 is the unknown option", $_); exit 1; };
      } #SWITCH
    }
    elsif ( $option_is == 1 )
    {
      $source_file = $opt;
      $option_is=0;
    }
    elsif ( $option_is == 2 )
    {
      $source_version = $opt;
      $option_is=0;
    }
    elsif ( $option_is == 4 )
    {
      $target_version = $opt;
      $option_is=0;
    }
    else
    {
      print STDERR &PerlTranslate("tdpver.pl", "Error: unhandled option %1 in script\n", "%1 is the unhandled option", $option_is);
      exit 1;
    }
  } #for
  if ($source_file eq "")
  {
    print &PerlTranslate("tdpver.pl", "Give a parameter to the %1 option\n", "%1 is the option", "-update");
    showUsage();
  }
  if ($target_version eq "")
  {
    print &PerlTranslate("tdpver.pl", "Give a parameter to the %1 option\n", "%1 is the option", "-tover");
    showUsage();
  }
} #parseOptions

sub convertFromMcKinleyRSR3ToBaltic #($$)
{
	my $inFile=shift;
	my $ouFile=shift;

	$inFile=~s/\\/\\\\/;
	$ouFile=~s/\\/\\\\/;
	my $source_type="";
	my $buffer;
	my $writeThisLine = 1;
	my $asmextFound=0;
		  
	print &PerlTranslate("tdpver.pl", "Converting from McKinleyRSR3 release to Baltic\n");
	open(IN,"<$inFile") || die &PerlTranslate("tdpver.pl", "Cannot open input file %1\n", "%1 is the input file name", $inFile);
	open(OU,">$ouFile") || die &PerlTranslate("tdpver.pl", "Cannot open output file %1\n", "%1 is the output file name", $ouFile);
	while(<IN>)
	{
		#
		# Change release number Version="Baltic"and get the language
		#
		if (/^<TDP.* Version=\"McKinleyRSR3\" .*$/)
		{
		  if(/type=\"([A-Z0-9]+)\"/) {$source_type=$1}
		  else { die &PerlTranslate("tdpver.pl", "Cannot find the TDP type\n") };
		  $_ =~ s/^<TDP(.*) Version=\"McKinleyRSR3\" (.*)$/<TDP$1 Version=\"Baltic\" $2/;
		}
		elsif(/^\s*<IS_[AC] /) {
		print OU <<"END";
<IS_ALL Doc="IS" Label="For All" >
<SETTINGS>
<COMMENT/>
<ASMEXT Doc="ISASMEXT" Label="asm File Extension" >
<COMMENT/>
<VALUE/>
</ASMEXT>
</SETTINGS>
</IS_ALL>
END
		}
		elsif(/^\s*<ASMEXT\s/) {
			$buffer="";
			$asmextFound=1;
			$writeThisLine=0;
		}
		elsif(/^\s*<\/SPRINTF>/) {
		print OU <<"END";
<ITEM>
<OPTIONAL_MACRO Defined="0" Doc="SPRINTFINTEGERFORMAT" Name="RTRT_SPRINTF_INTEGER_FORMAT" Value="%ld" Label="sprintf integer format" >
<COMMENT/>
</OPTIONAL_MACRO>
</ITEM>
<ITEM>
<OPTIONAL_MACRO Defined="0" Doc="SPRINTFUNSIGNEDFORMAT" Name="RTRT_SPRINTF_UNSIGNED_FORMAT" Value="%ud" Label="sprintf unsigned format" >
<COMMENT/>
</OPTIONAL_MACRO>
</ITEM>
END
		}
		elsif(/^\s*<\/DATA_TYPES>/) {
		print OU <<"END";
<ITEM>
<BINARY_MACRO Doc="PTR64BITS" Name="RTRT_64_BITS_PTR" Value="0" Label="Pointers uses 64 bits" >
<COMMENT/>
</BINARY_MACRO>
</ITEM>
END
		}
		elsif(/^\s*<\/STRING_SUPPORT>/) {
		print OU <<"END";
<ITEM>
<OPTIONAL_MACRO Defined="0" Doc="ATUSTRINGSIZE" Name="ATU_STRING_SIZE" Value="1024" Label="string size" >
<COMMENT/>
</OPTIONAL_MACRO>
</ITEM>
<ITEM>
<BINARY_MACRO Doc="STRINGCONCATSUPPORTED" Name="RTRT_STRING_CONCAT_SUPPORTED" Value="1" Label="string concat supported" >
<COMMENT/>
</BINARY_MACRO>
</ITEM>
END
		}
		elsif(/^\s*<\/INITSUPPORTED>/) {
		print OU <<"END";
<ITEM>
<BINARY_MACRO Doc="RAMCLEAN" Name="RTRT_RAM_CLEAN" Value="0" Label="ram clean" >
<COMMENT/>
</BINARY_MACRO>
</ITEM>
END
		}
		elsif(/^\s*<\/SETTINGS_IO>/) {
		print OU <<"END";
<ITEM>
<OPTIONAL_MACRO Defined="0" Doc="INITOPENMODE" Name="RTRT_INIT_MODE" Value="&quot;w&quot;" Label="init open mode" >
<COMMENT/>
</OPTIONAL_MACRO>
</ITEM>
<ITEM>
<OPTIONAL_MACRO Defined="0" Doc="OPENOPENMODE" Name="RTRT_OPEN_MODE" Value="&quot;a&quot;" Label="open open mode" >
<COMMENT/>
</OPTIONAL_MACRO>
</ITEM>
END
		}
		
		if($writeThisLine) {print OU ;}
		else {$buffer.=$_;}
		
		if($asmextFound) {
		   if(/^\s*<VALUE\/>/) {
		   	$asmextFound=2; # no value found
		   }
		   if(/^\s*<VALUE><\/VALUE>/) {
		   	$asmextFound=2; # no value found
		   }
		   if(/^\s*<\/ASMEXT>/) {
		   	if($asmextFound==1) {print OU $buffer ;} #print the buffer if it contains a value
		   	$asmextFound=0;
		   	$writeThisLine=1;
		   }
		
		}
	}
	close(IN);
	close(OU);
}

sub convertFromMcKinleyToMcKinleyRSR3 #($$)
{
	my $inFile=shift;
	my $ouFile=shift;

	$inFile=~s/\\/\\\\/;
	$ouFile=~s/\\/\\\\/;
	my $source_type="";
	my $writeThisLine = 1;
	my $atlLinkFound = 0;
	my $macroRedefFound = 0;
		  
	print &PerlTranslate("tdpver.pl", "Converting from McKinley release to McKinleyRSR3\n");
	open(IN,"<$inFile") || die &PerlTranslate("tdpver.pl", "Cannot open input file %1\n", "%1 is the input file name", $inFile);
	open(OU,">$ouFile") || die &PerlTranslate("tdpver.pl", "Cannot open output file %1\n", "%1 is the output file name", $ouFile);
	while(<IN>)
	{
		#
		# Change release number Version="McKinleyRSR3"and get the language
		#
		if (/^<TDP.* Version=\"McKinley\" .*$/)
		{
		  if(/type=\"([A-Z0-9]+)\"/) {$source_type=$1}
		  else { die &PerlTranslate("tdpver.pl", "Cannot find the TDP type\n") };
		  $_ =~ s/^<TDP(.*) Version=\"McKinley\" (.*)$/<TDP$1 Version=\"McKinleyRSR3\" $2/;
		}
	   	   
		if($source_type eq "ADA") {
		  #
		  # Change label for FREE_TEXT_ADD_PRIV_IO_ADB
		  #
		  if(/FREETEXTADDPRIVIOADB/) {
			s^User-defined function bodies^User-defined code inserted at the bottom of the body^;
		  }
		  #
                  # add FREE_TEXT_TOP_ADD_PRIV_IO_ADB section in Ada xdp files
	          #
                  s^</FREE_TEXT_WITH_PRIV_IO_ADB>^</FREE_TEXT_WITH_PRIV_IO_ADB>
<FREE_TEXT_TOP_ADD_PRIV_IO_ADB Doc="FREETEXTTOPADDPRIVIOADB" Label="User-defined code inserted at the top of the body" >
<COMMENT/>
<VALUE>-- Insert additional code here
</VALUE>
</FREE_TEXT_TOP_ADD_PRIV_IO_ADB>^;
			
		  print OU if ($writeThisLine);
		  $writeThisLine = 1;
	   	}	
	   	elsif($source_type eq "CCPP") {
		  #
		  # Basic Settings (ASM, DLL and LIB extension)
		  #
		  if (/<\/EXEEXT>/) {
		  	print OU "</EXEEXT>\n";
			print OU "<ASMEXT Doc=\"ISASMEXT\" Label=\"asm File Extension\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</ASMEXT>\n";
			print OU "<DLLEXT Doc=\"ISDLLEXT\" Label=\"dll File Extension\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>dll</VALUE>\n";
			print OU "</DLLEXT>\n";
			print OU "<LIBEXT Doc=\"ISLIBEXT\" Label=\"Library File Extension\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>lib</VALUE>\n";
			print OU "</LIBEXT>\n";
			$writeThisLine = 0;
		  }

		  #
		  # Basic Settings (Default TP DLL/LIB CC/LD Flags and ASM Flags)
		  #
		  if (/<\/DEFAULT_LIBS>/) {
			print OU "</DEFAULT_LIBS>\n";
			print OU "<DEFAULT_ASMFLAGS Doc=\"ISDEFAULT_ASMFLAGS\" Label=\"Default Assembly Flags\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</DEFAULT_ASMFLAGS>\n";
			print OU "<DEFAULT_TPLIB_CCFLAGS Doc=\"ISDEFAULT_TPLIB_CCFLAGS\" Label=\"Default Compilation Flags for the TP library\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</DEFAULT_TPLIB_CCFLAGS>\n";
			print OU "<DEFAULT_TPDLL_CCFLAGS Doc=\"ISDEFAULT_TPDLL_CCFLAGS\" Label=\"Default Compilation Flags for the TP dll\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</DEFAULT_TPDLL_CCFLAGS>\n";
			print OU "<DEFAULT_TPLIB_LDFLAGS Doc=\"ISDEFAULT_TPLIB_LDFLAGS\" Label=\"Default Link Flags for the TP library\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</DEFAULT_TPLIB_LDFLAGS>\n";
			print OU "<DEFAULT_TPDLL_LDFLAGS Doc=\"ISDEFAULT_TPDLL_LDFLAGS\" Label=\"Default Link Flags for the TP dll\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</DEFAULT_TPDLL_LDFLAGS>\n";
			$writeThisLine = 0;
		  }

		  #
		  # Assembler function
		  #
		  if (/<COMPILATION Doc=\"COMPILATION\" Label=\"Compilation function\" >/) {
			print OU "<ASSEMBLER Doc=\"ASSEMBLER\" Label=\"Assembler function\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUEFCTPERL>\n";
			print OU "<COMMENT/>\n";
			print OU "<PROTO>sub atl_asm(\$\$\$\$\\@\\@)\n";
			print OU "{\n";
			print OU "\tmy ( \$lang,\$src,\$out,\$cflags,\$Defines,\$Includes) = \@_;</PROTO>\n";
			print OU "<VALUE>\n";
			print OU "\tdie \"This TDP does not support Assembler\\n\";\n";
			print OU "</VALUE>\n";
			print OU "<TAIL>}</TAIL>\n";
			print OU "</VALUEFCTPERL>\n";
			print OU "</ASSEMBLER>\n";
		  }


		  #
		  # Library function
		  #
		  if (/<LINK Doc=\"LINK\" Label=\"Link function\" >/) {
			print OU "<LIBRARY Doc=\"LIBRARY\" Label=\"Library function\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUEFCTPERL>\n";
			print OU "<COMMENT/>\n";
			print OU "<PROTO>sub atl_lib(\$\\@\$\\@\$\\@\$)\n";
			print OU "{\n";
			print OU "\tmy (\$exe,\$Objects,\$ldflags,\$LibPath,\$libs,\$OtherLibs,\$Format) = \@_;</PROTO>\n";
			print OU "<VALUE>\n";
			print OU "\tprint \"WARNING: This TDP is not able to build library\\n\";\n";
			print OU "</VALUE>\n";
			print OU "<TAIL>}</TAIL>\n";
			print OU "</VALUEFCTPERL>\n";
			print OU "</LIBRARY>\n";
		  }

		  #
		  # Add $OtherLibs parameters to the Link function
		  #
		  if (/<PROTO>sub atl_link/) {
		  	print OU "<PROTO>sub atl_link(\$\\@\$\\@\$\\@)\n";
		  	$writeThisLine = 0;
		  	$atlLinkFound = 1;
		  }
		  if ((/<\/PROTO>/) && ($atlLinkFound)) {
		  	print OU "\tmy (\$exe,\$Objects,\$ldflags,\$LibPath,\$libs,\$OtherLibs) = \@_;</PROTO>\n";
		  	$writeThisLine = 0;
		  	$atlLinkFound = 0;
		  }
		  
		  #
		  # Linkage Directives
		  #
		  if (/<\/LINKAGE>/) {
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"DLLIMPORT\" Name=\"RTRT_DLL_IMPORT\" Label=\"dll Import Directive\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"DLLEXPORT\" Name=\"RTRT_DLL_EXPORT\" Label=\"dll Export Directive\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"FAMILYINITVALUE\" Name=\"RTRT_FAMILY_INIT_VALUE\" Label=\"Family Initialization Value Directive\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>&amp;_fini</VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"FAMILYTYPE\" Name=\"RTRT_FAMILY_TYPE\" Label=\"Family Type Directive\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>void *</VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
		  }

		  #
		  # Initialized variable support and checks
		  #
		  if (/<\/INITSUPPORTED>/) {
			print OU "<ITEM>\n";
			print OU "<BINARY_MACRO Doc=\"RAMSETRANDOMLY\" Name=\"RTRT_RAM_SET_RANDOMLY\" Value=\"0\" Label=\"Ram Set Randomly\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</BINARY_MACRO>\n";
			print OU "</ITEM>\n";
		  }

		  #
		  # signal function availability
		  #
		  if (/<ONGLET Doc=\"SIGNAL\" Label=\"usr_signal\" >/) {
		  	print OU "<ONGLET Doc=\"SIGNAL\" Label=\"Settings\" >\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"POSTSIG\" Name=\"RTRT_POST_SIG\" Label=\"Post Sig\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "</ONGLET>\n";
		  }

		  #
		  # new section IO Settings
		  #
		  if(/<GENERAL_IO Doc=\"IOTITLE\" Label=\"Data retrieval\" >/) {
			print OU "<SETTINGS_IO Doc=\"SETTINGSIOTITLE\" Label=\"IO Settings\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"OBSTOOLSFINISHED\" Name=\"ATL_OBSTOOLS_FINISHED\" Label=\"Obstools Finished\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE></VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<OPTIONAL_MACRO Defined=\"0\" Doc=\"BUFFERSIZE\" Name=\"RTRT_BUFFER_SIZE\" Value=\"1024\" Label=\"Buffer Size\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</OPTIONAL_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "</SETTINGS_IO>\n";
		  }

		  #
		  # malloc/free functions availability
		  #
		  if (/<ONGLET Doc=\"HEAP\" Label=\"usr_malloc\" >/) {
			print OU "<ONGLET Doc=\"HEAP\" Label=\"Settings\" >\n";
			print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"ATPHASHSIZE\" Name=\"ATP_HASH_SIZE\" Label=\"Hash Size\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>6</VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<OPTIONAL_MACRO Defined=\"0\" Doc=\"REDZONEHEADSIZE\" Name=\"ATP_RED_ZONE_HEAD_SIZE\" Value=\"0\" Label=\"Red Zone Head Size\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</OPTIONAL_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<OPTIONAL_MACRO Defined=\"0\" Doc=\"REDZONETAILSIZE\" Name=\"ATP_RED_ZONE_TAIL_SIZE\" Value=\"0\" Label=\"Red Zone Tail Size\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</OPTIONAL_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<BINARY_MACRO Doc=\"ATPCLEAN\" Name=\"ATP_CLEAN\" Value=\"0\" Label=\"ATP Clean\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</BINARY_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<BINARY_MACRO Doc=\"NOHOOKGUARD\" Name=\"RTRT_NO_HOOK_GUARD\" Value=\"0\" Label=\"No Hook Guard\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</BINARY_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "</ONGLET>\n";
		  }


		  #
		  # Clock Interface
		  #
		  if (/<ONGLET Doc=\"UNITTYPE\" Label=\"Unit Type\" >/) {
		  	print OU "<ONGLET Doc=\"UNITTYPE\" Label=\"Unit Type\" >\n";
		  	print OU "<ITEM>\n";
		  	print OU "<TEXT_MACRO Doc=\"ATUDELAY\" Name=\"RTRT_ATU_DELAY\" Label=\"ATU Delay\" >\n";
		  	print OU "<COMMENT/>\n";
		  	print OU "<VALUE>0</VALUE>\n";
		  	print OU "</TEXT_MACRO>\n";
		  	print OU "</ITEM>\n";
		  	$writeThisLine = 0;
		  }
		  
		  #
		  # Thread Interface
		  #
		  if (/<ONGLET Doc=\"THREADS\" Label=\"General\" >/) {
		  	print OU "<ONGLET Doc=\"THREADS\" Label=\"General\" >\n";
		  	print OU "<ITEM>\n";
			print OU "<TEXT_MACRO Doc=\"THREADISALIVE\" Name=\"RTRT_MT_THREAD_ISALIVE(id)\" Label=\"Thread Is Alive\" >\n";
			print OU "<COMMENT/>\n";
			print OU "<VALUE>(id)</VALUE>\n";
			print OU "</TEXT_MACRO>\n";
			print OU "</ITEM>\n";
			print OU "<ITEM>\n";
			print OU "<BINARY_MACRO Doc=\"THREADSINSPECTIONFORBIDDEN\" Name=\"RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN\" Value=\"0\" Label=\"Other Threads Inspection Forbidden\" >\n";
			print OU "<COMMENT/>\n";
			print OU "</BINARY_MACRO>\n";
			print OU "</ITEM>\n";
		  	$writeThisLine = 0;
		  }

		  #
		  # add FREE_TEXT_BEFORE_FOR_INSTR_AND_GEN section in C/C++ xdp files
		  #
		  if (/COMPILERSPECIFICS/) {
		  	print OU "<FREE_TEXT_BEFORE_FOR_INSTR_AND_GEN Doc=\"FREETEXTBEFOREINSTRANDGEN\" Label=\"Include files visible to all generated and instrumented files\" >\n";
		  	print OU "<COMMENT/>\n";
			print OU "<VALUE>/* Insert Here any includes free functions or macros */\n";
			print OU "</VALUE>\n";
			print OU "</FREE_TEXT_BEFORE_FOR_INSTR_AND_GEN>\n";
		  }

		  #
		  # Convert VALUEFCTPERL into VALUEFCTH for LONGJMP and MEMLIBEXTENSION
		  #
		  if (/\"Macro Redefinition\"/) { $macroRedefFound++; }
		  if (/VALUEFCTPERL/ && ($macroRedefFound>0)) {
			$_=~s/VALUEFCTPERL/VALUEFCTH/;
			print OU $_;
		  	$writeThisLine = 0;
			if ($macroRedefFound == 2) { $macroRedefFound = 0; }
			$macroRedefFound++;
		  }

		  print OU if ($writeThisLine);
		  $writeThisLine = 1;
	 	} #type = CCPP
		else {
		  print OU;
		}

	}
	close(IN);
	close(OU);
	
}

sub convertFromSR3ToMcKinley #($$)
{
	my $inFile=shift;
	my $ouFile=shift;

	$inFile=~s/\\/\\\\/;
	$ouFile=~s/\\/\\\\/;
	my $removeBlock=0;
	my $copyBlock=0;
	my $aBlock="";
	my $source_type="";

	print &PerlTranslate("tdpver.pl", "Converting from SR3 release to McKinley\n");
	open(IN,"<$inFile") || die &PerlTranslate("tdpver.pl", "Cannot open input file %1\n", "%1 is the input file name", $inFile);
	open(OU,">$ouFile") || die &PerlTranslate("tdpver.pl", "Cannot open output file %1\n", "%1 is the output file name", $ouFile);
	while(<IN>)
	{
		#
		# Change release number Version="McKinley"and get the language
		#
		if (/^<TDP.* Version=\"SR3\" .*$/)
		{
			if(/type=\"([A-Z0-9]+)\"/) {$source_type=$1}
			else			{die &PerlTranslate("tdpver.pl", "Cannot find the TDP type\n")};
			$_ =~ s/^<TDP(.*) Version=\"SR3\" (.*)$/<TDP$1 Version=\"McKinley\" $2/;
		}
	   if($source_type eq "ADA")
	   {
		#
                # add PRIV_INIT routine in Ada xdp files
                #
                s^</PRIV_CLOCK>^</PRIV_CLOCK>
                <PRIV_INIT Doc="ADAPRIVINIT" Label="Initialization function" >
                <COMMENT/>
                <VALUEFCTADA>
                <COMMENT/>
                <PROTO>procedure priv_init(I : in integer) is</PROTO>
                <VALUE>begin
                null;</VALUE>
                <TAIL>end priv_init;</TAIL>
                </VALUEFCTADA>
                </PRIV_INIT>^;
		print OU;
	   }
	   elsif($source_type eq "CCPP")
	   {
		#
		# STD_INCLUDE Var
		#
		if(/<\/SETTINGS>/)
		{
		   print OU "    <INIFIELD Doc=\"STDINCLUDE\" >\n";
                   print OU "     <COMMENT></COMMENT>\n";
                   print OU "     <KEY>STD_INCLUDE</KEY>\n";
                   print OU "     <VALUE></VALUE>\n";
                   print OU "    </INIFIELD>\n";
		}
		#
		#
		#
		if(/VARIABLEINITSUPPORTEDTITLE/)
		{
		s^Initialized global variable support^Initialized variable support and checks^;
		}
		#
		#
		#
		if(/<\/INITSUPPORTED>/)
		{
		   print OU "      <ITEM>\n";
		   print OU "       <BINARY_MACRO Doc=\"CHECKCALLSTACK\" Name=\"RTRT_CHECK_STACK\" Value=\"0\" Label=\"check call stack variables\" >\n";
		   print OU "        <COMMENT/>\n";
		   print OU "       </BINARY_MACRO>\n";
		   print OU "      </ITEM>\n";
		}
		#
		# Add ptr alignment
		#
		if(/<\/DATA_TYPES>/)
		{
		   print OU "     <ITEM>\n";
		   print OU "      <TEXT_MACRO Doc=\"PTR2INT\" Name=\"RTRT_PTR_TO_INT(p)\" Label=\"pointer to integer conversion\" >\n";
		   print OU "       <COMMENT/>\n";
		   print OU "       <VALUE>((RTRT_U_INT32)p)</VALUE>\n";
		   print OU "       </TEXT_MACRO>\n";
		   print OU "     </ITEM>\n";
		   print OU "     <ITEM>\n";
		   print OU "      <BINARY_MACRO Doc=\"PTRALIGNMENT\" Name=\"RTRT_CARE_ABOUT_PTR_ALIGNMENT\" Value=\"0\" Label=\"take care about alignment when memory checking\" >\n";
		   print OU "       <COMMENT/>\n";
		   print OU "      </BINARY_MACRO>\n";
		   print OU "     </ITEM>\n";    
		}
		#
		# Add Line Marker
		#
		if(/LINESPLITMARKER/ && /<OPTIONAL_MACRO\s+(\w+)="(.*)"\s+(\w+)="(.*)"\s+(\w+)="(.*)"\s+(\w+)="(.*)"\s+(\w+)="(.*)"\s*>/)
		{
  	          my $c="^";
		  my %A;
		  $A{$1}=$2;
		  $A{$3}=$4;
		  $A{$5}=$6;
		  $A{$7}=$8;
#		  $A{$9}=$10;
		  if($A{'Defined'} eq "1")
		  {
		      $c="\\n".substr($A{'Value'},1,1);
		  }
		print OU "        <OPTIONAL_MACRO Defined=\"$A{'Defined'}\" Doc=\"LINESPLITMARKER\" Name=\"RTRT_LINE_SPLIT_MARKER_BEGIN\" ";
		print OU "Value=\"&quot;$c&quot;\" Label=\"Dump Separator begin\" >\n";
		print OU "         <COMMENT\/>\n";
		print OU "        <\/OPTIONAL_MACRO>\n";
		print OU "       <\/ITEM>\n";
		print OU "       <ITEM>\n";
		print OU "        <OPTIONAL_MACRO Defined=\"$A{'Defined'}\" Doc=\"LINESPLITMARKER\" Name=\"RTRT_LINE_SPLIT_MARKER_END\" ";
		print OU "Value=\"&quot;$c&quot;\" Label=\"Dump Separator end\" >\n";
		next;
		  
		}
       
		#
		# add get date
		#
		s^</EXIT>^</EXIT>
    <DATE Doc="DATETITLE" Label="date function availability" >
     <COMMENT/>
     <CHOICE Name="RTRT_GET_DATE" Value="RTRT_NONE" >
      <COMMENT/>
      <CHOICEDATA Sensitive="0" >RTRT_NONE</CHOICEDATA>
      <CHOICEDATA Sensitive="1" >RTRT_USR</CHOICEDATA>
      <ONGLET Doc="EXIT" Label="usr_date" >
       <VALUEFCTC>
        <COMMENT/>
        <PROTO>char *usr_date()
{</PROTO>
        <VALUE>	/* Code here your own get date function */
	return ((char *)0);</VALUE>
        <TAIL>}</TAIL>
       </VALUEFCTC>
      </ONGLET>
     </CHOICE>
    </DATE>^;
		
		#
		# add get_pid
		#
		s^</GETENV>^</GETENV>
     <GETPID Doc="GETPIDTITLE" Label="getpid function availability" >
      <COMMENT/>
      <CHOICE Name="RTRT_GETPID" Value="RTRT_NONE" >
       <COMMENT/>
       <CHOICEDATA Sensitive="0" >RTRT_NONE</CHOICEDATA>
       <CHOICEDATA Sensitive="0" >RTRT_STD</CHOICEDATA>
       <CHOICEDATA Sensitive="1" >RTRT_USR</CHOICEDATA>
       <ONGLET Doc="GETPID" Label="usr_getpid" >
        <VALUEFCTC>
         <COMMENT/>
         <PROTO>int usr_getpid()
{</PROTO>
         <VALUE>	/* Code here your getpid function */
	 return(0);</VALUE>
         <TAIL>}</TAIL>
        </VALUEFCTC>
       </ONGLET>
      </CHOICE>
     </GETPID>^;		
		#
		# remove usr_append and add usr_init. init and open must do what old open did
		#
		s/usr_open/usr_init/;
		if($copyBlock)
		{
			if($aBlock eq "") {
				$aBlock=$_;
			}else{
				$aBlock=$aBlock."\n".$_;
			}
			if(/<\/VALUE>/)
			{
				$copyBlock=0;
			}
		}
		if(/RTRT_FILE_TYPE usr_init/) {$copyBlock=1;}
		s/usr_append/usr_open/;
		if($removeBlock)
		{
			if(/<\/VALUE>/)
			{
				$removeBlock=0;
			}
			next;
		}
		print OU;
		if(/RTRT_FILE_TYPE usr_open/) {$removeBlock=1;print OU $aBlock;}
	   } #type = CCPP
	   else {
		print OU;
	   }
	}
	close(IN);
	close(OU);
}

sub convertFromOMToSR3 #($$)
{
	my $inFile=shift;
	my $ouFile=shift;

	$inFile=~s/\\/\\\\/;
	$ouFile=~s/\\/\\\\/;

	print &PerlTranslate("tdpver.pl", "Converting from OM release to SR3\n");
	open(IN,"<$inFile") || die &PerlTranslate("tdpver.pl", "Cannot open input file %1\n", "%1 is the input file name", $inFile);
	open(OU,">$ouFile") || die &PerlTranslate("tdpver.pl", "Cannot open output file %1\n", "%1 is the output file name", $ouFile);
	while(<IN>)
	{
		#
		# Change Labels (NO TRANSLATION)
		#
		s/Unit Testing/Component Testing/g;
		s/Object Testing/Component Testing for C++/g;
		s/Test RealTime System Testing/System Testing/g;
		s/Test RealTime runtime observation features/runtime analysis features/g;
		s/Test RealTime testing features/testing features/g;
		s/Test RealTime generated files/generated files/g;
		s/Test and observation results output/Data retrieval/g;
		s/Data retrieval and error output/Data retrieval and Error message output/g;
		s/Runtime observation features for C/Runtime analysis features for C/g;
		s/runtime observation features for C\+\+/Runtime analysis features for C\+\+/g;
		#
		# Add release number: There was not initially
		#
		s/^(.*)<TDP (.*)$/$1<TDP Version=\"SR3\" $2/;
		#
		# Add LongJmp Functions
		#
		s^</SIGNAL>^</SIGNAL>
     <LONGJMP Label=\"longjmp availability and tracking\" Doc=\"LONGJMP\">
      <COMMENT/>
      <CHOICE Name=\"RTRT_LONGJMP\" Value=\"RTRT_NONE\">
       <COMMENT/>
       <CHOICEDATA Sensitive=\"0\">RTRT_NONE</CHOICEDATA>
       <CHOICEDATA Sensitive=\"1\">RTRT_USR</CHOICEDATA>
       <ONGLET Label=\"longjmp functions\" Doc=\"LONGJMP\">
        <VALUEFCTC>
         <COMMENT/>
         <PROTO>void _atl_stack_jump(void *,short);
#define STACK_GOING_UP 0
#define STACK_GOING_DOWN 1</PROTO>
         <VALUE>#include &lt;setjmp.h>
void RTRT_longjmp(jmp_buf env,int val)
{
	/* Call the actual longjmp function */
	/* after having called _atl_stack_jump ( [Stack Pointer Register Address], [STACK_GOING_UP|STACK_GOING_DOWN] );  */
	_atl_stack_jump(env[0],STACK_GOING_UP);
	longjmp(env,val);
}
void RTRT__longjmp(jmp_buf env,int val)
{
	/* Call the actual longjmp function */
	/* after having called _atl_stack_jump ( [Stack Pointer Register Address], [STACK_GOING_UP|STACK_GOING_DOWN] );  */
	_atl_stack_jump(env[0],STACK_GOING_UP);
	_longjmp(env,val);
}</VALUE>
         <TAIL/>
        </VALUEFCTC>
       </ONGLET>
       <ONGLET Label=\"Macro Redefinition\" Doc=\"LONGJMP\">
        <VALUEFCTPERL>
         <COMMENT/>
         <PROTO/>
         <VALUE>#define longjmp RTRT_longjmp
#define _longjmp RTRT__longjmp</VALUE>
         <TAIL/>
        </VALUEFCTPERL>
       </ONGLET>
      </CHOICE>
     </LONGJMP>^;
		#
		# usr_clock_init
		#
		s^(.*) Label=\"Unit Type\"(.*)$^     <ONGLET Label=\"usr_clock_init\" Doc=\"CLOCK\">
      <VALUEFCTC>
       <COMMENT/>
       <PROTO>void usr_clock_init(void)
{</PROTO>
       <VALUE>	/* This function will be called only once when starting the program */
	/* You may code here clock resolution computation */</VALUE>
       <TAIL>}</TAIL>
      </VALUEFCTC>
     </ONGLET>
$1 Label=\"Unit Type\"$2^;
		#
		# STATIC_IN_STATIC... And EXTERN_C_FORBIDDEN
		#
		s^</CPP_SPECIFICS>^     <ITEM>
      <BINARY_MACRO Name=\"RTRT_STATIC_AND_EXTERN_C_FORBIDDEN\" Value=\"1\" Label=\"NotUsed\" Doc=\"STATICANDEXTERNCFORBIDDEN\">
       <COMMENT/>
      </BINARY_MACRO>
     </ITEM>
     <ITEM>
      <BINARY_MACRO Name=\"RTRT_STATIC_IN_STATIC_INLINE_ALLOWED\" Value=\"1\" Label=\"NotUsed\" Doc=\"STATICINSTATICINLINE\">
       <COMMENT/>
      </BINARY_MACRO>
     </ITEM>
    </CPP_SPECIFICS>^;

		print OU;
	}
	close(IN);
	close(OU);
}

############################################################################
sub getVersionFromFile #($)
{
	my $inFile=shift;
	my $line=0;
	my $ver=$VersionList[0];
	open(INF,"<$inFile") || die &PerlTranslate("tdpver.pl", "Cannot open input file %1\n", "%1 is the input file name", $inFile);
	while(<INF>){
	last if ($line > 3);
	$line++;
	#print $line.$_."\n";
	   if(/Version="(.*?)"/){
		$ver=$1;
		print "TDP $ver\n";
		last;
	   }

	}
	close INF;
	return $ver;
}

############################################################################
##
#
##
############################################################################
$|=1;
STDOUT->autoflush(1);
STDERR->autoflush(1);
parseOptions();
$source_version="";

$source_version=getVersionFromFile($source_file) if($source_version eq "");

my @tempFiles;
my $i;

my $indexFromVersion=-1;
$i=0;
foreach(@VersionList)
{
	$indexFromVersion=$i if($_ eq $source_version);
	$i++;
}
die &PerlTranslate("tdpver.pl", "Cannot identify from version parameter in the known list of initial version\n") if($indexFromVersion==-1);

my $indexToVersion=-1;
$i=0;
foreach(@VersionList)
{
	$indexToVersion=$i if($_ eq $target_version);
	$i++;
}
die &PerlTranslate("tdpver.pl", "Cannot identify to version parameter in the known list of destination version\n") if($indexToVersion==-1);
exit(0) if($indexToVersion == $indexFromVersion);
die &PerlTranslate("tdpver.pl", "Detected incoherent fromVersion toVersion parameters\n") if($indexToVersion<=$indexFromVersion);

my $j=$indexFromVersion;
my $inFile=$source_file;
my $saveFile=$inFile."_".$VersionList[$j];
$saveFile=~s/\\/\\\\/g;
copy($inFile,$saveFile) || die &PerlTranslate("tdpver.pl", "Cannot read %1 or write %2\n","%1 is the source file %2 is the safe copy",$inFile,$saveFile);
while($j<$indexToVersion)
{
	if ($ENV{'TDP_VERSIONS'}) {
		my $saveFile=$inFile."_".$VersionList[$j];
		$saveFile=~s/\\/\\\\/g;
		copy($inFile,$saveFile) || die &PerlTranslate("tdpver.pl", "Cannot read %1 or write %2\n","%1 is the source file %2 is the safe copy",$inFile,$saveFile);
	}

	my $ouFile=$inFile."_".($j+1);
	$inFile=~s/\\/\\\\/g;
	$ouFile=~s/\\/\\\\/g;
	my $funcToCall="convertFrom$VersionList[$j]To$VersionList[$j+1](\"$inFile\",\"$ouFile\")";
	my $Status=eval $funcToCall;
        if(!defined($Status))
        {
          my $msg=&PerlTranslate("tdpver.pl", "The update function %1 does not exists\n", "%1 is the function name", $funcToCall);
	  die "$msg\n$@"; # NO TRANSLATION (see above)
        }
	$inFile=$ouFile;
	push(@tempFiles,$ouFile);
	$j++;
}

if($inFile ne $source_file)
{
	copy($inFile,$source_file) || die &PerlTranslate("tdpver.pl", "Cannot replace %1. Check the permissions!\n", "%1 is source file name", $source_file);
}
# 
# CleanUp
#
foreach(@tempFiles) { unlink($_); }


exit(0);
