

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
*       (C) 2003 Rational Software Corporation. All Rights Reserved.           *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2002.05.20                                           *
*                                                                              *
*                   File: agtconf.c                                            *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#include "agtconf.h"
#ifdef AGT_PROC_TABLE
#include "glob.h"
#endif

/* -------------------------------------------------------------- */
/* ----------------- BEGIN OF CUSTOMIZABLE PART ----------------- */
/* -------------------------------------------------------------- */

/******************************************************************
 *
 *  Process handling functions 
 *
 ******************************************************************/
#ifdef AGT_CUSTOM
/* -------------------------------------------------------------- */
/* NAME         : AGT_processStatus                               */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to check if a specified   */
/*                process is over or still alive.                 */
/* RETURN VALUE : TRUE if the process is over. FALSE otherwise.   */
/* INPUT        : process ID                                      */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */
T_BOOLEAN AGT_processStatus DP ((process_id)) BP
	AGT_PID_T process_id EP
{
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_processStop                                 */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to kill the process       */
/*                specified by the 'process_id' parameter.        */
/* RETURN VALUE : TRUE if the process has been killed or FALSE    */
/* INPUT        : process ID                                      */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */

T_BOOLEAN AGT_processStop DP ((process_id)) BP
	AGT_PID_T process_id EP
{
}


/* -------------------------------------------------------------- */
/* NAME         : AGT_processLaunch                               */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to launch a process.      */
/*                When writing this line in a supervisor script:  */
/*                  DO machine:myexec param1 param2               */
/*                the 'command' parameter of the function is a    */
/*                string array which contains "myexec", "param1"  */
/*                and "param2" as elements.                       */
/*                As 'myexec' is an executable (or entry point)   */
/*                you may need to execute it in a forked task.    */
/* RETURN VALUE : process ID of the forked task.                  */
/* INPUT        : entry point for the process to be launched.     */
/*                port number for the rendezvous socket.          */
/*                command to execute in the process to launch.    */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */
AGT_PID_T AGT_processLaunch DP ((entrypoint, command)) BP
	AGT_ENTRY_POINT entrypoint SP
	char *command[MAX_ARGUMENT] EP
{
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_openNullSocket                              */
/* MODULE(S)    : traagt.c                                        */
/* DESCRIPTION  : opens a NULL socket                             */
/* RETURN VALUE : a null-socket identifier                        */
/* INPUT        :                                                 */
/* OUTPUT       :                                                 */
/* -------------------------------------------------------------- */
T_INT AGT_openNullSocket ()
{
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_dupStdToSocket                              */
/* MODULE(S)    : traagt.c                                        */
/* DESCRIPTION  : duplicates standard input, output and error to  */
/*                the specified socket                            */
/* RETURN VALUE :                                                 */
/* INPUT        : indentifier for the specified socket            */
/*                                                                */
/* -------------------------------------------------------------- */
void AGT_dupStdToSocket DP ((socket_id)) BP
		T_INT socket_id EP
{
}
#endif /* AGT_CUSTOM */


/******************************************************************
 *
 *  Memory handling functions 
 *
 ******************************************************************/

PT_VOID CSTmalloc DP ((size)) BP
          CONST T_ENTIER size EP
{
  register PT_VOID zone;

  zone = malloc (size);
  if (zone == NULL)
    {
      CSTprintf("FATAL ERROR on malloc,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
  return (zone);
}

PT_VOID CSTrealloc DP ((oldptr, size)) BP
          PT_VOID oldptr SP
          CONST T_ENTIER size EP
{
  PT_CAR    zone;

  if (oldptr)
    zone = realloc (oldptr, size);
  else
    zone = malloc (size);

  if (zone == NULL)
    {
      CSTprintf("FATAL ERROR on realloc,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
  return ((PT_VOID) zone);
}

void CSTmemset DP ((str, carpad, taille)) BP
          PT_CAR str SP
          T_CAR carpad SP
          T_ENTIER taille EP
{
  void *ret;
  ret = memset ((void *) str, (T_ENTIER) carpad, (size_t) taille);
}

void CSTfree DP ((block)) BP
          PT_VOID block EP
{
  free (block);
}

void CSTexit DP ((num)) BP
          CONST T_ENTIER num EP
{
    exit(num);
}

/******************************************************************
 *
 *  File handling functions 
 *
 ******************************************************************/
#ifndef NO_FILE_SYSTEM
T_ENTIER CSTunlink DP ((path)) BP
          CONST PT_CAR path EP
{
  return (unlink (path));
}

AGT_FILE_DES CSTfopen DP ((filename, mode)) BP
          CONST PT_CAR filename SP
          CONST PT_CAR mode EP
{

  return(fopen (filename, mode));
}

void CSTfclose DP ((fd)) BP
          AGT_FILE_DES fd EP
{
  T_ENTIER  result;

  result = fclose (fd);
  if (result == EOF)
    {
      CSTprintf( "FATAL ERROR on fclose,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
}

T_BOOLEAN CSTfeof DP ((fd)) BP
          const AGT_FILE_DES fd EP
{
  T_BOOLEAN  result;
  result = FALSE;

  if (feof (fd))
    {
      result = TRUE;
    }
  return (result);
}

T_BOOLEAN CSTchdir DP ((dirname)) BP
	PT_CAR dirname EP
{
  return((chdir(dirname) == 0));
}

PT_CAR CSTgetcwd DP (()) BP
	void EP
{
  return(getcwd(NULL, 1000));
}

T_BOOLEAN CSTmkdir DP ((dirname)) BP
	PT_CAR dirname EP
{
  return((mkdir((char *) dirname, S_IRUSR | S_IWUSR | S_IXUSR) == 0));
}

T_BOOLEAN CSTrmdir DP ((dirname)) BP
	PT_CAR dirname EP
{
  return((rmdir(dirname) == 0));
}

#ifdef ANSI_C
void      CSTfprintf (AGT_FILE_DES fd, const PT_CAR format,...)
#else
void      CSTfprintf (fd, format, va_alist)
     AGT_FILE_DES     fd;
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif
  result = vfprintf (fd, format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTprintf("FATAL ERROR on fprintf,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
}

#endif /* NO_FILE_SYSTEM */

/******************************************************************
 *
 *  String handling functions 
 *
 ******************************************************************/

#ifdef ANSI_C
void      CSTsprintf (const PT_CAR str, const PT_CAR format,...)
#else
void      CSTsprintf (str, format, va_alist)
     PT_CAR    str;
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif

  result = vsprintf (str, format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTprintf("FATAL ERROR on sprintf (%s),\n Please contact your administrative.\n",
               format);
      CSTexit (K_CST_EXIT_FAT);
    }
}

#ifdef ANSI_C
void      CSTprintf (const PT_CAR format,...)
#else
void      CSTprintf (format, va_alist)
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif
  result = vprintf (format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTexit (K_CST_EXIT_FAT);
    }
}

T_ENTIER CSTstrcmp DP ((str1, str2)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  T_ENTIER  result;

  result = strcmp (str1, str2);
  return (result);
}

T_ENTIER CSTstrncmp DP ((str1, str2, lng)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 SP
          CONST T_ENTIER lng EP
{
  T_ENTIER  result;

  result = strncmp (str1, str2, lng);
  return (result);
}

void CSTstrcpy DP ((str1, str2)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  strcpy (str1, str2);
}

T_ENTIER CSTstrlen DP ((str)) BP
          CONST PT_CAR str EP
{
  size_t    result;

  result = strlen (str);
  return ((T_ENTIER) result);
}

T_ENTIER CSTatoi DP ((str)) BP
          CONST PT_CAR str EP
{
  return (atoi (str));
}

PT_CAR CSTstrchr DP ((str, car)) BP
          CONST PT_CAR str SP
          CONST T_CAR car EP
{
  return (strchr (str, (T_ENTIER) car));
}

PT_CAR CSTstrstr DP ((str1, str2)) BP
          PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  return (strstr (str1, str2));
}

/* -------------------------------------------------------------- */
/* -----------------  END OF CUSTOMIZABLE PART  ----------------- */
/* -------------------------------------------------------------- */


/* -------------------------------------------------------------- */
/* NAME         :                                                 */
/* MODULE(S)    :                                                 */
/* DESCRIPTION  :                                                 */
/* RETURN VALUE :                                                 */
/* INPUT        :                                                 */
/* OUTPUT       :                                                 */
/* EXAMPLE      :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */



