

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2004 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
*          (C) 2003 Rational Software Corporation. All Rights Reserved.        *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2002.05.20                                           *
*                                                                              *
*                   File: agtconf.c                                            *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#include "agtconf.h"
#ifdef AGT_PROC_TABLE
#include "glob.h"
#endif
#include "atwstr.h"

/* -------------------------------------------------------------- */
/* ----------------- BEGIN OF CUSTOMIZABLE PART ----------------- */
/* -------------------------------------------------------------- */

/******************************************************************
 *
 *  Process handling functions
 *
 ******************************************************************/
#ifdef AGT_CUSTOM

#ifdef AGT_PROC_TABLE
extern AGT_ENTRY_POINT Tmain;
void AGT_InitProcTable  ()
{
        strcpy(AgtTasks[0].f_name,"Tmain");
        AgtTasks[0].f_pointer=Tmain;
}

#endif /* AGT_PROC_TABLE */

/* -------------------------------------------------------------- */
/* NAME         : AGT_processStatus                               */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to check if a specified   */
/*                process is over or still alive.                 */
/* RETURN VALUE : TRUE if the process is over. FALSE otherwise.   */
/* INPUT        : process ID                                      */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */
T_BOOLEAN AGT_processStatus DP ((process_id)) BP
	AGT_PID_T process_id EP
{
    /* VxWorks Example */
	if (taskIdVerify (process_id) == ERROR)
	{
		return TRUE;
	}
	return FALSE;
    /*
    return TRUE;
    */
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_processStop                                 */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to kill the process       */
/*                specified by the 'process_id' parameter.        */
/* RETURN VALUE : TRUE if the process has been killed or FALSE    */
/* INPUT        : process ID                                      */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */

T_BOOLEAN AGT_processStop DP ((process_id)) BP
	AGT_PID_T process_id EP
{
    /* VxWorks Example */
        if (taskDeleteForce(process_id) == ERROR)
	{
		return FALSE;
        }
	return TRUE;
    /*
    return FALSE;
    */
}


/* -------------------------------------------------------------- */
/* NAME         : AGT_processLaunch                               */
/* MODULE(S)    : proagt.c                                        */
/* DESCRIPTION  : This function is used to launch a process.      */
/*                When writing this line in a supervisor script:  */
/*                  DO machine:myexec param1 param2               */
/*                the 'command' parameter of the function is a    */
/*                string array which contains "myexec", "param1"  */
/*                and "param2" as elements.                       */
/*                As 'myexec' is an executable (or entry point)   */
/*                you may need to execute it in a forked task.    */
/* RETURN VALUE : process ID of the forked task.                  */
/* INPUT        : entry point for the process to be launched.     */
/*                command to execute in the process to launch.    */
/* OUTPUT       :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */
AGT_PID_T AGT_processLaunch DP ((entrypoint, command)) BP
	AGT_ENTRY_POINT entrypoint SP
	char *command[MAX_ARGUMENT] EP
{
  ATL_PT_ARG vt_arg;
  T_BOOLEAN tester = FALSE;
  int i, index, proc;
  int priority=100, options=0, stackSize=16000;
  int args[10]={0,0,0,0,0,0,0,0,0,0};

  vt_arg = (ATL_PT_ARG) CSTmalloc (sizeof (ATL_T_ARG));
  vt_arg->atl_riofilename = NULL;
  vt_arg->atl_filters = "";
  vt_arg->atl_instance = NULL;
  vt_arg->atl_occid = 0;

  index=1;
  i=0;
  while (command[index])
  {
  	if (CSTstrstr(command[index], "-INSTANCE="))
  	{
	    vt_arg->atl_instance=CSTstrstr(command[index], "-INSTANCE=")+10;
	    tester=TRUE;
  	}
	else if (CSTstrstr(command[index], "-RIO="))
  	{
  	    vt_arg->atl_riofilename=CSTstrstr(command[index], "-RIO=")+5;
	    tester=TRUE;
  	}
	else if (CSTstrstr(command[index], "-OCCID="))
  	{
  	    vt_arg->atl_occid=CSTatoi(CSTstrstr(command[index], "-OCCID=")+7);
	    tester=TRUE;
  	}
	else if (CSTstrstr(command[index], "-PRIORITY="))
	    priority=CSTatoi((command[index])+10);
	else if (CSTstrstr(command[index], "-OPTIONS="))
	    options=CSTatoi((command[index])+9);
	else if (CSTstrstr(command[index], "-STACKSIZE="))
	    stackSize=CSTatoi((command[index])+11);
	else
	{
	    args[i]=CSTatoi((command[index]));
	    i++;
	}
  	index++;
  }

  if (tester)
  {
        proc = taskSpawn (command[0], priority, options, stackSize,
                          entrypoint, (int) vt_arg,
                          0, 0, 0, 0, 0, 0, 0, 0, 0);
  }
  else
  {
        proc = taskSpawn (command[0], priority, options, stackSize,
        		  entrypoint, args[0], args[1], args[2],
			  args[3], args[4], args[5], args[6],
			  args[7], args[8], args[9]);
  }
  if (proc == ERROR)
  {
        printf("Error while trying to spawn the task %s\n", command[0]);
  	return(-1);
  }
  else
      	return(proc);
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_openNullSocket                              */
/* MODULE(S)    : traagt.c                                        */
/* DESCRIPTION  : opens a NULL socket                             */
/* RETURN VALUE : a null-socket identifier                        */
/* INPUT        :                                                 */
/* OUTPUT       :                                                 */
/* -------------------------------------------------------------- */
T_INT AGT_openNullSocket ()
{
    return(open("/dev/null", O_RDWR, 0644));
}

/* -------------------------------------------------------------- */
/* NAME         : AGT_dupStdToSocket                              */
/* MODULE(S)    : traagt.c                                        */
/* DESCRIPTION  : duplicates standard input, output and error to  */
/*                the specified socket                            */
/* RETURN VALUE :                                                 */
/* INPUT        : indentifier for the specified socket            */
/*                                                                */
/* -------------------------------------------------------------- */
void AGT_dupStdToSocket DP ((socket_id)) BP
		T_INT socket_id EP
{
    /* VxWorks Example */
    setvbuf (stdin, NULL, _IONBF, 0);
    setvbuf (stdout, NULL, _IONBF, 0);
    setvbuf (stderr, NULL, _IONBF, 0);
    ioGlobalStdSet (fileno (stdin), socket_id);
    ioGlobalStdSet (fileno (stdout), socket_id);
    ioGlobalStdSet (fileno (stderr), socket_id);
}
#endif /* AGT_CUSTOM */


/******************************************************************
 *
 *  Memory handling functions
 *
 ******************************************************************/

PT_VOID CSTmalloc DP ((size)) BP
          CONST T_ENTIER size EP
{
  register PT_VOID zone;

  zone = malloc (size);
  if (zone == NULL)
    {
      CSTprintf("FATAL ERROR on malloc,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
  return (zone);
}

PT_VOID CSTrealloc DP ((oldptr, size)) BP
          PT_VOID oldptr SP
          CONST T_ENTIER size EP
{
  PT_CAR    zone;

  if (oldptr)
    zone = realloc (oldptr, size);
  else
    zone = malloc (size);

  if (zone == NULL)
    {
      CSTprintf("FATAL ERROR on realloc,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
  return ((PT_VOID) zone);
}

void CSTmemset DP ((str, carpad, taille)) BP
          PT_CAR str SP
          T_CAR carpad SP
          T_ENTIER taille EP
{
  void *ret;
  ret = memset ((void *) str, (T_ENTIER) carpad, (size_t) taille);
}

void CSTfree DP ((block)) BP
          PT_VOID block EP
{
  free (block);
}

void CSTexit DP ((num)) BP
          CONST T_ENTIER num EP
{
}

/******************************************************************
 *
 *  File handling functions
 *
 ******************************************************************/
#ifndef NO_FILE_SYSTEM
T_ENTIER CSTunlink DP ((path)) BP
          CONST PT_CAR path EP
{
  return (unlink (path));
}

AGT_FILE_DES CSTfopen DP ((filename, mode)) BP
          CONST PT_CAR filename SP
          CONST PT_CAR mode EP
{

  return(fopen (filename, mode));
}

void CSTfclose DP ((fd)) BP
          AGT_FILE_DES fd EP
{
  T_ENTIER  result;

  result = fclose (fd);
  if (result == EOF)
    {
      CSTprintf( "FATAL ERROR on fclose,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
}

T_BOOLEAN CSTfeof DP ((fd)) BP
          const AGT_FILE_DES fd EP
{
  T_BOOLEAN  result;
  result = FALSE;

  if (feof (fd))
    {
      result = TRUE;
    }
  return (result);
}

T_BOOLEAN CSTchdir DP ((dirname)) BP
	PT_CAR dirname EP
{
    char *newdirname, *p, *q;
    char dirsep[2];
    unsigned int i,j,k;

    dirsep[0] = K_CST_PATH_CUT;
    dirsep[1] = '\0';
    if (CSTstrstr(dirname,":"))
    {
	/* Absolute path */
	newdirname = dirname;
    }
    else
    {
	/* Relative path */
	newdirname = CSTgetcwd();
	newdirname = strcat(newdirname, dirsep);
	newdirname = strcat(newdirname, dirname);
    }
    while(CSTstrstr(newdirname,".."))
    {
	p=CSTstrstr(newdirname, "..");
	q=p-2;
	p=p+2;
	while(((*q)!='/') && ((*q)!='\\')) q--;
	j=(unsigned int)(q-dirname);
	k=(unsigned int)(CSTstrlen(newdirname)+newdirname-p+1);
	for(i=0; i<k; i++)
		newdirname[j+i]=*(p+i);
    }
    for(i=0; i<CSTstrlen(newdirname); i++)
	if ((newdirname[i]=='/') || (newdirname[i]=='\\'))
	    newdirname[i] = K_CST_PATH_CUT;
    return((chdir(newdirname)==OK));
}

PT_CAR CSTgetcwd DP (()) BP EP
{
  T_CAR dirname[MAX_FILENAME_LENGTH];

  ioDefPathGet(dirname);
  return(atwstrClone(dirname));
}

T_BOOLEAN CSTmkdir DP ((dirname)) BP
	PT_CAR dirname EP
{
  return((mkdir(dirname) == OK));
}

T_BOOLEAN CSTrmdir DP ((dirname)) BP
	PT_CAR dirname EP
{
  return((rmdir(dirname) == OK));
}

#ifdef ANSI_C
void      CSTfprintf (AGT_FILE_DES fd, const PT_CAR format,...)
#else
void      CSTfprintf (fd, format, va_alist)
     AGT_FILE_DES     fd;
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif
  result = vfprintf (fd, format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTprintf("FATAL ERROR on fprintf,\n Please contact your administrative.\n");
      CSTexit (K_CST_EXIT_FAT);
    }
}

#endif /* NO_FILE_SYSTEM */

/******************************************************************
 *
 *  String handling functions
 *
 ******************************************************************/

#ifdef ANSI_C
void      CSTsprintf (const PT_CAR str, const PT_CAR format,...)
#else
void      CSTsprintf (str, format, va_alist)
     PT_CAR    str;
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif

  result = vsprintf (str, format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTprintf("FATAL ERROR on sprintf (%s),\n Please contact your administrative.\n",
               format);
      CSTexit (K_CST_EXIT_FAT);
    }
}

#ifdef ANSI_C
void      CSTprintf (const PT_CAR format,...)
#else
void      CSTprintf (format, va_alist)
     PT_CAR    format;
     va_dcl

#endif
{
  T_ENTIER  result;
  va_list   va;

#ifdef ANSI_C
  va_start (va, format);
#else
  va_start (va);
#endif
  result = vprintf (format, va);
  va_end (va);
  if (result == EOF)
    {
      CSTexit (K_CST_EXIT_FAT);
    }
}

T_ENTIER CSTstrcmp DP ((str1, str2)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  T_ENTIER  result;

  result = strcmp (str1, str2);
  return (result);
}

T_ENTIER CSTstrncmp DP ((str1, str2, lng)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 SP
          CONST T_ENTIER lng EP
{
  T_ENTIER  result;

  result = strncmp (str1, str2, lng);
  return (result);
}

void CSTstrcpy DP ((str1, str2)) BP
          CONST PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  strcpy (str1, str2);
}

T_ENTIER CSTstrlen DP ((str)) BP
          CONST PT_CAR str EP
{
  size_t    result;

  result = strlen (str);
  return ((T_ENTIER) result);
}

T_ENTIER CSTatoi DP ((str)) BP
          CONST PT_CAR str EP
{
  return (atoi (str));
}

PT_CAR CSTstrchr DP ((str, car)) BP
          CONST PT_CAR str SP
          CONST T_CAR car EP
{
  return (strchr (str, (T_ENTIER) car));
}

PT_CAR CSTstrstr DP ((str1, str2)) BP
          PT_CAR str1 SP
          CONST PT_CAR str2 EP
{
  return (strstr (str1, str2));
}

/* -------------------------------------------------------------- */
/* -----------------  END OF CUSTOMIZABLE PART  ----------------- */
/* -------------------------------------------------------------- */


/* -------------------------------------------------------------- */
/* NAME         :                                                 */
/* MODULE(S)    :                                                 */
/* DESCRIPTION  :                                                 */
/* RETURN VALUE :                                                 */
/* INPUT        :                                                 */
/* OUTPUT       :                                                 */
/* EXAMPLE      :                                                 */
/*                                                                */
/* -------------------------------------------------------------- */



