

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2004 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
*         (C) 2003 Rational Software Corporation. All Rights Reserved.         *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2002.05.20                                           *
*                                                                              *
*                   File: agtconf.h                                            *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef _AGTCONF_H
#define _AGTCONF_H

/**************************************************************
 *
 * Generic defines
 *
 *************************************************************/

#define USER_MAIN       /* 'main' is not allowed as an entry point      */
#define NO_FTP          /* file transfer protocol not available         */
#define AGT_PROC_TABLE  /* a process table containing all entry-points  */
#define NO_SECURITY     /* permissions and acces control not available  */

#if 0
#define NO_FILE_SYSTEM  /* file system not available on the target      */
#define NO_ENV_VAR      /* environment variables not available          */
#define NO_FTP          /* file transfer protocol not available         */
#define NO_SECURITY     /* permissions and acces control not available  */
#define USER_MAIN       /* 'main' is not allowed as an entry point      */
#define POSIX_SIGNAL    /* POSIX signal handling available on target    */
#define AGT_PROC_TABLE  /* a process table containing all entry-points  */
			/* is used 					*/
#endif

/**************************************************************
 *
 * Standard includes
 *
 *************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <hostLib.h>
#include <symLib.h>
#include <sockLib.h>
#include <taskLib.h>
#include <ioLib.h>
#include <envLib.h>
#include <sysSymTbl.h>
#include <sys/times.h>

/**************************************************************
 *
 * Type definitions
 *
 *************************************************************/
/* type definition for a process ID */
typedef int AGT_PID_T;

/* type definition for a file descriptor */
typedef FILE * AGT_FILE_DES;

/* type definition for an entry point */
#ifndef AGT_PROC_TABLE
typedef char *AGT_ENTRY_POINT;
#else
typedef int (*AGT_ENTRY_POINT)();
#endif

/* Standard types */
typedef int T_INT;
typedef int *PT_INT;
typedef void T_VOID;
typedef void *PT_VOID;
typedef long T_LONG;
typedef unsigned long T_ULONG;
typedef double T_DOUBLE;
typedef unsigned int T_UINT;
typedef char T_CHAR;
typedef unsigned char T_UCHAR;
typedef char *PT_CHAR;
typedef float T_FLOAT;
#undef TRUE
#undef FALSE
typedef enum { FALSE=0, TRUE=1 } T_BOOLEAN;
typedef enum { SYSTEM_OK=0, SYSTEM_ERROR=-1 } T_RETURN;

/* no debug statement available */
#define atsdebug(x)

/* Specify the path separator */
/* #define K_CST_PATH_CUT '\\' */ 	/* Windows-like */
#define K_CST_PATH_CUT '/'		/* Unix-like */

/* maximum number of arguments authorized when launching a process */
#define MAX_ARGUMENT 50

/* VxWorks uses ANSI C */
#define ANSI_C

#ifdef ANSI_C		/* Definitions for ANSI C prototypes */

#define CONST const
#define PP(ARGS) ARGS
#define DP(ARGS)
#define BP (
#define SP ,
#define EP )
#define VOID (void)

#else 			/* Definition for Kernighan & Ritchie C prototypes */

#define CONST
#define PP(ARGS) ()
#define DP(ARGS) ARGS
#define BP
#define SP ;
#define EP ;
#define VOID

#endif



/**************************************************************
 *
 * Customized function prototypes
 *
 *************************************************************/
#ifdef USER_MAIN
#define AGT_MAIN		\
	int atsagtd (int port_number) {
#endif

#define AGT_INIT AGT_InitProcTable();

#ifdef AGT_CUSTOM
extern AGT_PID_T AGT_processLaunch (
			AGT_ENTRY_POINT entrypoint,
			char *arguments[MAX_ARGUMENT]);
extern T_BOOLEAN AGT_processStop (AGT_PID_T process_id);
extern T_BOOLEAN AGT_processStatus (AGT_PID_T process_id);
extern T_INT AGT_openNullSocket();
extern T_VOID AGT_dupStdToSocket(T_INT socket_id);
#endif


/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/
/*--------------- END OF CUSTOMIZABLE PARTS -----------------*/
/*-----------------------------------------------------------*/
/*-----------------------------------------------------------*/

/* NO_SECURITY and NO_FTP must be defined if no file    */
/* system is available                                  */
#ifdef NO_FILE_SYSTEM
#ifndef NO_SECURITY
#define NO_SECURITY
#endif
#ifndef NO_FTP
#define NO_FTP
#endif
#endif

/* Longeur de la chaine de type T_CHAINE */
#define K_CST_LG_CHAINE 2000

#ifdef DEBUG
#define debug(a) a
#ifndef assert
#define assert(c) if (!(c)) printf("Assert failed %s/%d\n",__FILE__,__LINE__)
#endif
#else
#define debug(a)
#endif

/* Valeur de sortie du preprocesseur, et du postprocesseur    */
#define K_CST_EXIT_OK 0  /* pas d'avertissements ni d'erreurs */
#define K_CST_EXIT_WAR 3 /* des avertissements pas d'erreurs  */
#define K_CST_EXIT_ERR 5 /* des erreurs                       */
#define K_CST_EXIT_FAT 7 /* erreur fatale                     */
#define K_CST_EXIT_INT 9 /* erreur interne                    */

/* Valeur retourne par CSTrecentfile */
#define K_CST_NOFILE 0
#define K_CST_NEWER  1
#define K_CST_OLDER  2

#define SNULL (PT_CAR)0

/* declarations des types elementaires */
typedef T_INT T_ENTIER;
typedef PT_INT PT_ENTIER;
typedef T_UINT T_UENTIER;
typedef T_CHAR T_CAR;
typedef T_UCHAR T_UCAR;
typedef PT_CHAR PT_CAR;
typedef T_FLOAT T_REEL;
typedef T_CHAR T_TABCAR[K_CST_LG_CHAINE];


#define M_CSTmin(a,b) ((a<b)?a:b)
#define M_CSTmax(a,b) ((a>b)?a:b)

#define M_CSTfree(ARGS) CSTfree((PT_VOID)ARGS)
#define CSTsscanf sscanf

/* prototypage des services externes du module cstsyst.c */
extern PT_VOID   CSTmalloc     PP(( const T_ENTIER size ));
extern PT_VOID   CSTrealloc    PP(( PT_VOID oldptr, const T_ENTIER size ));
extern void      CSTexit       PP(( const T_ENTIER num ));
extern void      CSTfree       PP(( PT_VOID block ));
extern void      CSTsprintf    PP(( const PT_CAR str, const PT_CAR format, ... ));
extern void      CSTprintf     PP(( const PT_CAR format, ... ));


extern T_ENTIER  CSTstrcmp     PP(( const PT_CAR str1, const PT_CAR str2 ));
extern T_ENTIER  CSTstrncmp    PP(( const PT_CAR str1, const PT_CAR str2,
                                    const T_ENTIER lng ));
extern void      CSTstrcpy     PP(( const PT_CAR str1, const PT_CAR str2 ));
extern T_ENTIER  CSTstrlen     PP(( const PT_CAR str ));
extern T_ENTIER  CSTatoi       PP(( const PT_CAR str ));
extern PT_CAR    CSTstrchr     PP(( const PT_CAR str , const T_CAR car ));
extern PT_CAR    CSTstrstr     PP((       PT_CAR str1 , const PT_CAR str2 ));
extern void      CSTmemset     PP(( PT_CAR str, T_CAR carpad, T_ENTIER taille));

#ifndef NO_FILE_SYSTEM
extern void      CSTfprintf    PP(( AGT_FILE_DES fd, const PT_CAR format, ... ));
extern T_ENTIER  CSTunlink     PP (( const PT_CAR path ));
extern AGT_FILE_DES  CSTfopen   PP(( const PT_CAR filename,
                                   const PT_CAR mode));
extern void      CSTfclose  PP(( AGT_FILE_DES fd ));
extern T_BOOLEAN CSTfeof PP(( const AGT_FILE_DES fd ));
extern T_BOOLEAN CSTchdir 	PP(( PT_CAR dirname));
extern T_BOOLEAN CSTmkdir 	PP(( PT_CAR dirname));
extern T_BOOLEAN CSTrmdir 	PP(( PT_CAR dirname));
extern PT_CAR CSTgetcwd 	PP(( ));
#endif /* NO_FILE_SYSTEM */

void AGT_InitProcTable  ();

#endif /* _AGTCONF_H */
