

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2004 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: atsagtd.c                                            *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)atsagtd $Revision: 1.25 $ $Date: 2005/11/09 21:29:05 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  ATSAGTD                                                  *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.25 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2005/11/09 21:29:05 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Programme principal de l'agent de supervision                  *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* Inclusion des entetes standards SystemTest */
#include "atwstr.h"

#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

/* Inclusion des entetes des modules utilises */
#include "ctxagt.h"
#include "cmdagt.h"
#include "tcpip.h"
#include "rdvagt.h"
#include "dynfagt.h"

#include "cmd_msg.h"

/* Duree en miliseconde d'attente de reception de message (1000ms = 1s) */
#define K_AGT_MS_ATTENTE_MSG      1000

#ifdef WIN32
typedef struct
  {
    PT_REF_CONTEXT refContext;
    PT_CAR    machine;
    T_SOCKET_STREAM port_connexion;
  }
T_THREAD_PARAM, *PT_THREAD_PARAM;

static DWORD WINAPI ThreadFunc (LPVOID lpvThreadParm);
#endif                          /* WIN32 */

/* Declaration des fonctions interne au module */
static void p_AGTtraitementCommandes
#ifdef WIN32
   PP ((PT_REF_CONTEXT refContext,PT_CAR commande,T_ENTIER canal_principal));
#else
   PP ((PT_REF_CONTEXT refContext,T_ENTIER canal_principal));
#endif

static void p_AGTtraitementConnexion PP ((PT_REF_CONTEXT refContext,
                                          PT_CAR machine,
                                          T_SOCKET_STREAM port_connexion));

static void p_AGTtraitementConnexTcpip PP ((PT_REF_CONTEXT refContext,
                                            T_ENTIER alire,
                                            PT_SOCKET socket_a_lire,
                                            T_SOCKET_STREAM port_connexion,
                                            atwstr ** buffer,
                                            T_BOOLEAN * fin));

static void p_AGTtraitementConnexRdv PP ((PT_REF_CONTEXT refContext,
                                          T_ENTIER alire,
                                          PT_SOCKET socket_a_lire,
                                          T_SOCKET_STREAM port_connexion));

static void p_AGTtraitementDynflw PP ((PT_REF_CONTEXT refContext,
				       T_ENTIER alire,
				       PT_SOCKET socket_a_lire,
				       T_SOCKET_STREAM port_connexion,
				       atwstr ** buffer,
				       T_BOOLEAN * fin));

static atwstr *p_AGTtraiteDonneesSuper PP ((PT_REF_CONTEXT refContext,
                                            atwstr * buffer,
                                            T_BOOLEAN * fin,
                                            T_SOCKET_STREAM port_connexion));

#ifdef UNIX
#define POSIX_SIGNAL /* on UNIX, POSIX signals are available */
#endif

#ifdef POSIX_SIGNAL
/******************************************************************************
*
*  NAME
*       fonction d'interception du signal emis lors de la fin d'un fils
*
*  ALGORITHME
*
******************************************************************************/
void handler_sig DP ((sig)) BP
          T_ENTIER sig EP
{
  int       pid;
  int       etat;

  do
    {
      errno = 0;
      pid = waitpid (-1, &etat, WNOHANG);
    }
  while ((pid == -1) && (errno == EINTR));
  signal (sig, handler_sig);
}
#endif  /* UNIX */

/****************************************************************************
 * Programme de gestion des communictations avec le superviseur
 *************************************************************************** */
#ifdef USER_MAIN
AGT_MAIN
#else           /* USER_MAIN */
int main DP ((argc, argv)) BP
     int argc  SP
     char    **argv EP
{
#endif          /* USER_MAIN */
  PT_REF_CONTEXT refContext;
  T_ENTIER  etat_sortie = 3;

#ifdef  AGT_INIT
  AGT_INIT
#endif

#ifdef UNIX
  atwstr   *machine = NULL;
  T_SOCKET_STREAM socket_inetd;
  PT_CAR    usage = "TestRT-I-USAGE: atsagtd.bin <TCP port number> \n\tor launch via inetd\n";
#else
#ifndef USER_MAIN
  PT_CAR    usage = "TestRT-I-USAGE: atsagtd.exe <TCP port number> \n";
#endif
#endif

#ifdef POSIX_SIGNAL
  signal (SIGCHLD, handler_sig); /* if child status changed, execute handler_sig */
#endif /* POSIX_SIGNAL */

  /* Initialisation des sous modules */
  atwstrStartup ();
  refContext = CTXinit (LAST_CTX);
  if        ((refContext != NULL) && (TCPIPinit (refContext) == SYSTEM_OK))
    {
#ifndef AGT_CUSTOM
      /* Traite la ligne de commande (seulement pour UNIX-like et WIN32) */
      switch (argc)
	{
#ifdef WIN32
	case 4:
	  {
	    /* close port_principal */
	    TCPIPclose_stream (refContext, atoi (argv[3]));
	    p_AGTtraitementConnexion (refContext, argv[1], atoi (argv[2]));
	    CSTexit (0);
	    break;
	  }
	case 2:
	  p_AGTtraitementCommandes (refContext, argv[0], CSTatoi (argv[1]));

	  break;

#endif  /* WIN32 */
#ifdef UNIX
	case 2:
	  p_AGTtraitementCommandes (refContext, CSTatoi (argv[1]));
	  break;
	case 1:           /* Lancement via inetd -> stdin et stdout port de connexion */

	  socket_inetd = (T_SOCKET_STREAM) STDIN_FILENO;
	  machine = TCPIPnomMachineCliente_stream (refContext, socket_inetd);

	  if (machine)
	    {
	      p_AGTtraitementConnexion (refContext, machine, socket_inetd);
	      machine = atwstrDestroy (machine);
	    }
	  else
	    {
	      CSTfprintf (stderr, "%s", usage);
	      etat_sortie = 1;
	    }

	  break;
#endif /* UNIX */

	default:
	  CSTfprintf (stderr, "%s", usage);
	  etat_sortie = 1;
	  break;
	}
#else /* AGT_CUSTOM */
#ifdef USER_MAIN
      p_AGTtraitementCommandes (refContext, port_number);
#else
      p_AGTtraitementCommandes (refContext, argv[1]);
#endif
#endif /* AGT_CUSTOM */
      (void) TCPIPfin (refContext);
    }
  else
    {
      CSTprintf ("Internal error with TCPIP...\n");
    }

  /* Liberation du module de gestion des chaines */
  atwstrShutdown ();
  if (CTXfin (refContext) != SYSTEM_OK)
    {
      etat_sortie = 3;
    }

  CSTexit (etat_sortie);
  return(0);
}

/******************************************************************************
*
*  NAME
*       Boucle d'acceptation des connexions et des commandes emanant des
*       superviseurs
*
*  ALGORITHME
*
******************************************************************************/
#ifdef WIN32
static void p_AGTtraitementCommandes DP ((refContext,
                                          commandName,
                                          canal_principal)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR commandName SP
          T_ENTIER canal_principal EP
#else
static void p_AGTtraitementCommandes DP ((refContext,
                                          canal_principal)) BP
          PT_REF_CONTEXT refContext SP
          T_ENTIER canal_principal EP
#endif                          /* WIN32 */

{
  T_SOCKET_STREAM port_principal;
  T_SOCKET_STREAM port_connexion;
  T_TABCAR  machine;

  T_BOOLEAN LoopActiv = TRUE;
#ifdef AGT_CUSTOM
  PT_CAR args[MAX_ARGUMENT];
#endif

#ifdef WIN32
  BOOL      ret;
  T_TABCAR  commande;
  atwstr * commandeName2; /* espace dans les pathname */

#endif                          /* WIN32 */

  port_principal = TCPIPsocket_named_stream (refContext,
                                             NULL,
                                             NULL,
                                             &canal_principal);

  if (port_principal == SYSTEM_ERROR)
    {
      CSTprintf ("atsagtd.bin : Fatal error, unable to bind to the port %d\n", canal_principal);
      CSTexit (1);
    }

  /* Lancement de la boucle infinie de traitement */
  while(LoopActiv)
    {
      /* Accepte une connexion et traite les commnade en provenant */
      if (TCPIPaccept_stream (refContext, port_principal, &port_connexion,
                              machine, NULL)
          == SYSTEM_OK)
        {
#ifndef AGT_CUSTOM
#ifdef UNIX
          switch (fork ())
            {
            case -1:           /* Erreur */
              perror ("fork");
              CSTexit (-1);
              break;

            case 0:
              /* Processus fils */
              TCPIPclose_stream (refContext, port_principal);
              p_AGTtraitementConnexion (refContext, machine, port_connexion);
              CSTexit (0);
              break;

            default:
              /* Pere */
              TCPIPclose_stream (refContext, port_connexion);
              break;
            }
#endif /* UNIX */
#ifdef WIN32
          /* Parametres CreateProcess() WIN32 */
          STARTUPINFO siStartInfo;
          PROCESS_INFORMATION piProcInfo;

          memset (&siStartInfo, 0, sizeof (STARTUPINFO));
          memset (&piProcInfo, 0, sizeof (PROCESS_INFORMATION));
          siStartInfo.cb = sizeof (STARTUPINFO);
          siStartInfo.dwFlags = 0;
          siStartInfo.lpReserved = NULL;
          siStartInfo.lpReserved2 = NULL;
          siStartInfo.cbReserved2 = 0;
          siStartInfo.lpDesktop = NULL;
          siStartInfo.lpTitle = (LPTSTR) NULL;

	  /* espace dans les pathname (WIN32) */
          commandeName2 = atwstrCreate();
          commandeName2 = atwstrAppend(commandeName2, "\"");
          commandeName2 = atwstrAppend(commandeName2, commandName);
          commandeName2 = atwstrAppend(commandeName2, "\"");
          //printf("nom de la commande : %s\n", commandeName2);
	  /* fin de modification : espace dans les pathname */

          sprintf (commande, "%s %s %d %d", commandeName2, machine, port_connexion, port_principal);

	  /* espace dans les pathname (liberation de l'espace alloue) */
          commandeName2 = atwstrDestroy(commandeName2);
	  /* fin de modification : espace dans les pathname */

          debug (CSTprintf ("Creation du processus fils %s \n", commande));
          ret = CreateProcess (NULL, commande, NULL, NULL,
                            TRUE, 0, NULL, NULL, &siStartInfo, &piProcInfo);
          debug (CSTprintf ("Creation du processus fils ret= %d GetLastError = %d\n", ret, GetLastError ()));
          CloseHandle (piProcInfo.hThread);
          (void) TCPIPclose_stream (refContext, port_connexion);
#endif /* WIN32 */
#else /* AGT_CUSTOM */
	  args[0]=(char *)malloc(5);
	  args[1]=(char *)malloc(10);
	  args[2]=(char *)malloc(10);
	  args[3]=(char *)malloc(10);
	  CSTstrcpy(args[0], "agtd");
	  CSTsprintf(args[1], "%d", refContext);
	  CSTsprintf(args[2], "%d", machine);
	  CSTsprintf(args[3], "%d", port_connexion);
	  args[4]=NULL;
          if (AGT_processLaunch((AGT_ENTRY_POINT)p_AGTtraitementConnexion, args)==-1)
            {
              TCPIPclose_stream (refContext, port_connexion);
              CSTexit (-1);
            }
#endif /* AGT_CUSTOM */
        }
      else
        {
          perror ("TCPIPaccept_stream");
        }
#ifdef WIN32
      Sleep (1000);
#endif
    }
}

#ifdef WIN32
/******************************************************************************
*
*  NAME
*        Thread WIN 32 : Execution p_AGTtraitementConnexion
*
*  ALGORITHME
*
******************************************************************************/
DWORD WINAPI ThreadFunc DP ((lpvThreadParm)) BP
          LPVOID lpvThreadParm EP
{
  PT_THREAD_PARAM lpvParametre = (PT_THREAD_PARAM) lpvThreadParm;
  DWORD     dwResult = 0;

  if (lpvParametre != NULL)
    {
      p_AGTtraitementConnexion (lpvParametre->refContext,
                                lpvParametre->machine,
                                lpvParametre->port_connexion);
    }
  ExitThread (dwResult);
  return (dwResult);
}
#endif                          /* WIN32 */

/******************************************************************************
*
*  NAME
*       Recoit les messages des testeurs et les envoie au superviseur
*
*  ALGORITHME
*
******************************************************************************/
static void p_AGTtraitementConnexion DP ((refContext, machine, port_connexion)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR machine SP
          T_SOCKET_STREAM port_connexion EP
{
  T_ENTIER  alire = 0;
  T_ENTIER  longueur = 0;
  atwstr   *result = NULL;
  PT_SOCKET sockets_a_lires = NULL;
  atwstr   *buffer_tcpip = NULL;
  atwstr   *buffer_dynflw = NULL;
  T_BOOLEAN fin, fin_dynflw;
#ifdef UNIX                                   /* reprise patch H323 */
  T_ENTIER surveille_process;
#endif

  debug (CSTprintf ("p_AGTtraitementConnexion\n"));

#if defined(AGT_CUSTOM) || defined (WIN32)
  /*
   * Pour VXWorks et WIN32, cette fonction est execute par un thread,
   * ou le contexte de reference a ete duplique.
   */
  refContext = CTXinit (LAST_CTX);
  if (refContext == NULL)
    {
      return;
    }
  TCPIPinit (refContext);
#endif

  /* Envoie le message d'identification de l'agent */
  result = atwstrClone (g_chaine_ret[K_CMD_MSG_HELO]);
  result = atwstrAppend (result, "\r\n");
  longueur = atwstrLength (result);

  /* Envoi du message de connexion */
  TCPIPsend_stream (refContext,
                    port_connexion,
                    result,
                    &longueur);
  result = atwstrDestroy (result);

  /* Initialise le module de commande */
  if ((CMDinit (refContext, machine) == SYSTEM_OK)
      && (RDVAinit (refContext) == SYSTEM_OK)
      && (DYNFAinit (refContext) == SYSTEM_OK))
    {
      TCPIPabonne_stream (refContext, port_connexion);
      fin = FALSE;
      fin_dynflw = FALSE;
#ifdef UNIX                                   /* reprise patch H323 */
      surveille_process=0;
#endif
      while (!fin)
        { /* boucle */
	  /* Lance la surveillance du module de commande */

#ifdef UNIX                                   /* reprise patch H323 */
           if (surveille_process > 1000)
           {
             surveille_process=0;
#endif

             /* Lance la surveillance du module de commande */
             if ((result = CMDsurveille (refContext)))
               {
                 /* Envoie la reponse au superviseur */
                 result = atwstrAppend (result, "\r\n");
                 longueur = atwstrLength (result);
		 TCPIPsend_stream (refContext,
                                   port_connexion,
                                   result,
                                   &longueur);
                 result = atwstrDestroy (result);
               }
#ifdef UNIX                                   /* reprise patch H323 */
           }
#endif

          alire = TCPIPsurveille_sockets (refContext,
                                          &sockets_a_lires,
                                          K_AGT_MS_ATTENTE_MSG);
          if (sockets_a_lires)
            {
              p_AGTtraitementConnexTcpip (refContext,
                                          alire,
                                          sockets_a_lires,
                                          port_connexion,
                                          &buffer_tcpip,
                                          &fin);

              p_AGTtraitementConnexRdv (refContext,
                                        alire,
                                        sockets_a_lires,
                                        port_connexion);

              p_AGTtraitementDynflw (refContext,
				     alire,
				     sockets_a_lires,
				     port_connexion,
				     &buffer_dynflw,
				     &fin_dynflw);

              /* Liberation de la memoire allouee */
              M_CSTfree (sockets_a_lires);
              sockets_a_lires = NULL;
#ifdef UNIX                                   /* reprise patch H323 */
	      surveille_process+=1;
#endif
            }
#ifdef UNIX                                   /* reprise patch H323 */
	  else
	    {
	      surveille_process+=100;
	    }
#endif
        }

      /* Nous n'ecoutons plus la socket de commande */
      TCPIPdesabonne_stream (refContext,
                             port_connexion);

      /* Liberation du module de rendez-vous */
      (void) RDVAfin (refContext);

      /* Liberation du module de suivi */
      (void) DYNFAfin (refContext);

      /* Liberation du module de commande */
      (void) CMDfin (refContext);
    }
  (void) TCPIPclose_stream (refContext, port_connexion);

#if defined (AGT_CUSTOM) || defined(WIN32)
  /*
   * Pour VXWorks cette fonction est execute par un thread, ou le contexte
   * de reference a ete duplique.
   */
  (void) TCPIPfin (refContext);
  (void) CTXfin (refContext);
#endif

}

/******************************************************************************
*
*  NAME
*       Recoit les messages du superviseur et les execute
*
*  ALGORITHME
*
******************************************************************************/
static void p_AGTtraitementConnexTcpip DP ((refContext,
                                            alire,
                                            socket_a_lire,
                                            port_connexion,
                                            pbuffer,
                                            fin))     BP
          PT_REF_CONTEXT refContext SP
          T_ENTIER alire SP
          PT_SOCKET socket_a_lire SP
          T_SOCKET_STREAM port_connexion SP
          atwstr ** pbuffer SP
          T_BOOLEAN * fin EP
{
  T_ENTIER  index = 0;
  T_ENTIER  longueur = 0;
  T_BOOLEAN estdeconnecte;

  /* Recherche la socket de commande dans le tableau des sockets a lire */
  while ((index < alire) && (socket_a_lire[index].stream != port_connexion))
    {
      index++;
    }

  /* Lire les donnees de la connexion si celle-ci a ete trouvee */
  if (index < alire)
    {
      /* Recoit des donnees du reseau */
      if (*pbuffer)
        {
          /* recupere la taille du buffer avant la lecture */
          longueur = atwstrLength (*pbuffer);
        }
      (void) TCPIPrecv_texte_stream (refContext,
                                     port_connexion,
                                     pbuffer,
                                     &estdeconnecte);

      if ((!estdeconnecte) && (longueur != atwstrLength (*pbuffer)))
        {
          /* On decode les donnees recues */
          *pbuffer = p_AGTtraiteDonneesSuper (refContext,
                                              *pbuffer,
                                              fin,
                                              port_connexion);
        }
      else
        {
          /* La connexion est perdue --> fin du traitement */
          *fin = TRUE;
        }
    }
}

/******************************************************************************
*
*  NAME
*       Traite les RDV/cxions des clients et envoie la synthese au superviseur
*
*  ALGORITHME
*
******************************************************************************/
static void p_AGTtraitementConnexRdv DP ((refContext,
                                          alire,
                                          socket_a_lire,
                                          port_connexion)) BP
          PT_REF_CONTEXT refContext SP
          T_ENTIER alire SP
          PT_SOCKET socket_a_lire SP
          T_SOCKET_STREAM port_connexion EP
{
  atwstr   *result;

  result = RDVAtraite_connexions (refContext,
                                  alire,
                                  socket_a_lire);
  if (result)
    {
      /* Envoi des connexions traitees au superviseur */
      T_ENTIER  longueur;

      result = atwstrAppend (result, "\r\n");
      longueur = atwstrLength (result);
      TCPIPsend_stream (refContext, port_connexion, result, &longueur);
      result = atwstrDestroy (result);
    }
}

/******************************************************************************
*
*  NAME
*       Traite les connexions et remontees de traces/c.i.t. des clients
*       et envoie la synthese au superviseur
*
*  ALGORITHME
*
******************************************************************************/
static void p_AGTtraitementDynflw DP ((refContext,
				       alire,
				       socket_a_lire,
				       port_connexion,
				       pbuffer,
				       fin))     BP
          PT_REF_CONTEXT   refContext SP
          T_ENTIER         alire SP
          PT_SOCKET        socket_a_lire SP
          T_SOCKET_STREAM  port_connexion SP
          atwstr           ** pbuffer SP
          T_BOOLEAN        * fin EP
{
  atwstr   *result;

  result = DYNFAtraite_suividyn (refContext, alire, socket_a_lire,
				 pbuffer, fin);

  if (result)
    {
      T_ENTIER  longueur;

      /* AFFICHAGE PRINCIPAL DE DEBUG DE L'AGENT */
      /* fprintf (stderr, "%s\n", result); */

      result = atwstrAppend (result, "\r\n");
      longueur = atwstrLength (result);

      /* transmission au superviseur des remontees traitees */
      TCPIPsend_stream (refContext, port_connexion, result, &longueur);
      result = atwstrDestroy (result);
    }
}


/******************************************************************************
*
*  NAME
*       Recoit les messages du superviseur et les execute
*
*  ALGORITHME
*
******************************************************************************/
static atwstr *p_AGTtraiteDonneesSuper DP ((refContext,
                                            buffer,
                                            fin,
                                            port_connexion)) BP
          PT_REF_CONTEXT refContext SP
          atwstr * buffer SP
          T_BOOLEAN * fin SP
          T_SOCKET_STREAM port_connexion EP
{
  PT_CAR    buffer_fin = buffer + atwstrLength (buffer);
  PT_CAR    debut_cmd = buffer;
  PT_CAR    fin_cmd = NULL;
  atwstr   *result = NULL;

  while ((debut_cmd != buffer_fin) &&
         (!*fin) &&
         (fin_cmd = CSTstrstr (debut_cmd, "\r\n")))
    {
      /* Les separateurs ont ete trouves */
      /* On delimite la commande */
      *fin_cmd = '\0';
      result = CMDexecute (refContext, debut_cmd, fin);
      if (result)
        {
          /* envoi de la reponse au superviseur */
          T_ENTIER  longueur;
          result = atwstrAppend (result, "\r\n");
          longueur = atwstrLength (result);
          TCPIPsend_stream (refContext, port_connexion, result, &longueur);
          result = atwstrDestroy (result);
        }
      /* Repositionne le caractere efface ci-dessus */
      *fin_cmd = '\r';
      /* Le debut de la prochaine commande se situe apres les separateurs */
      debut_cmd = fin_cmd + 2;
    }

  buffer = atwstrTrunc (buffer, 0, debut_cmd - buffer);

  return (buffer);
}
