

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
*      (C) 2003 Rational Software Corporation. All Rights Reserved.            #
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2002.05.20                                           *
*                                                                              *
*                   File: atwstr.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
/*=====================================================
 *  MODULE :  atwstr
 *
 *  FILE :  atwstr.c
 *
 *  AUTHOR : E.M
 *
 *  CREATION DATE   : 10/07/1997
 *
 *  PUBLIC FUNCTIONS :
 *         void    atwstrStartup()
 *         void    atwstrShutdown()
 *         atwstr *atwstrCreate()
 *         atwstr *atwstrDestroy(atwstr *str)
 *         atwstr *atwstrClone(const char *s)
 *         atwstr *atwstrConcat(const char *str1, const char *str2, ...)
 *         atwstr *atwstrExtract(atwstr *str,  unsigned long b, unsigned long n)
 *         atwstr *atwstrAppend(atwstr *str, const char* s)
 *         atwstr *atwstrInsert(atwstr *str, const char *s, unsigned long b, unsigned long n)
 *         atwstr *atwstrTrunc(atwstr *str, unsigned long b, unsigned long n)
 *         unsigned long atwstrLength(const atwstr *str)
 *         int           atwstrCompare(const atwstr *str1, const atwstr *str2)
 *
 *
 *  MODIFICATION DATE :
 *
 *  SINOPSYS : Module de gestion de chaine de
 *              caracteres dynamique
 *=====================================================*/
/* BC remove warnings */
#ifndef AGT_CUSTOM
static char *cvs_id = "@(#) $Id: atwstr.c,v 1.12 2007/01/16 14:12:13 build Exp $";
#endif

#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#include "atwstr.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* liste d'argument (atwstrConcat) */
#ifdef ANSI_C
#include <stdarg.h>
#else
#include <varargs.h>
#endif

/*===============================================
        DEFINITION STRUCTURES DE DONNEES INTERNES
        AU MODULE
 ================================================ */

#define ATWSTR_MIN_ALLOC_SIZE      40
#define ATWSTR_UNKNOWN_SIZE        0
#define ATWSTR_MAGIC_NUMBER        (char)0xaa
/*
 * entete precedent le contenu d'une chaine
 */
struct atwstrHeader
  {
    char      magicNumber;      /* allouee/desallouee */
    unsigned  size;             /* taille allouee */
    unsigned  length;           /* taille effective */
  };

/*
 * Structure info debug
 */
#ifdef ATWSTR_DEBUG
typedef struct atwstrCellMemory atwstrCellMemory;
struct atwstrCellMemory
  {
    atwstr   *memory;           /* chaine allouee */
    char     *fileName;         /* nom du fichier */
    long      nuLine;           /* numero de ligne */
    atwstrCellMemory *next;     /* cellule suivante */
  };

struct atwstrDebugInfo
  {
    atwstrCellMemory *memoryList;       /* liste des allocations memoires */
    long      allocStr;         /* total chaines allouees */
    long      reallocStr;       /* total chaines reallouees */
    long      freeStr;          /* total chaines desallouees */
    long      allocBytes;       /* cumul octets alloues */
    long      reallocBytes;     /* cumul octets realloues */
    long      freeBytes;        /* cumul octets desalloues */
  };

static struct atwstrDebugInfo debugInfo =
{(atwstrCellMemory *) 0, 0, 0, 0, 0, 0, 0};

#endif                          /* ATWSTR_DEBUG */

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrAssertFail(fo, p, f, l, m)
 *
 * PARAMETERS :        const char *fo, const char *p, const char *f, const int l, const char *m
 *
 * RETURN VALUE :      void
 *
 * DEFINITION :        Appelee si une assertion n'est pas verifiee
 *                     affiche un message formate et met fin au programme
 *
 *==================================================================*/
static void _atwstrAssertFail
#ifdef ANSI_C                 /* ANSI */
          (const char *fo, const char *p, const char *f, long l, const char *m)
#else                           /* NON_ANSI */
          (fo, p, f, l, m)
     char     *fo;
     char     *p;
     char     *f;
     long      l;
     char     *m;

#endif
{
  fprintf (stderr, fo, p, f, l, m);
  exit (1);
}

/*
 * definition des macros pour les precondition et assertions utilisees
 */
#ifdef ANSI_C
#define ATW_ASSERT(p, m)      ((p)?(void)0:(void)_atwstrAssertFail( \
                  "Assertion failed : %s, file %s, line %d, %s \n", #p, __FILE__, __LINE__, m))
#define PRECONDITION(p)   ((p)?(void)0:(void)_atwstrAssertFail( \
                  "Precondition failed : %s, file %s, line %d\n", #p, __FILE__, __LINE__, ""))
#else
#define ATW_ASSERT(p, m)      ((p)?(void)0:(void)_atwstrAssertFail( \
                  "Assertion failed : %s, file %s, line %d, %s \n", "", __FILE__, __LINE__, m))
#define PRECONDITION(p)   ((p)?(void)0:(void)_atwstrAssertFail( \
                  "Precondition failed : %s, file %s, line %d\n", "", __FILE__, __LINE__, ""))
#endif

#define _atwstrGetHeader(str)  ((struct atwstrHeader*)((char*)str - sizeof (struct atwstrHeader)))

#ifdef ATWSTR_DEBUG
/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrAddCellMemory(str, f, l)
 *
 * PARAMETERS :        atwstr *str, char *f, long l
 *
 * RETURN VALUE :      void
 *
 * DEFINITION :        Ajoute dans la liste des chaines allouees
 *
 *
 *==================================================================*/
static void _atwstrAddCellMemory
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, char *f, long nu)
#else                           /* NON_ANSI */
          (str, f, l)
     atwstr   *str;
     char     *f;
     long      nu;

#endif
{
  atwstrCellMemory *cell = (atwstrCellMemory *) malloc (sizeof (atwstrCellMemory));

  ATW_ASSERT (cell != (atwstrCellMemory *) 0, "Internal error memory full");

  cell->next = debugInfo.memoryList;
  cell->memory = str;
  cell->fileName = f;
  cell->nuLine = nu;
  debugInfo.memoryList = cell;
}

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrDelCellMemory(str)
 *
 * PARAMETERS :        atwstr *str
 *
 * RETURN VALUE :      void
 *
 * DEFINITION :        Enleve de la liste des chaines allouees str
 *
 *==================================================================*/
static void _atwstrDelCellMemory
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str)
#else                           /* NON_ANSI */
          (str)
     atwstr   *str;

#endif
{
  atwstrCellMemory *cell,
           *previous;

  previous = (atwstrCellMemory *) 0;
  for (cell = debugInfo.memoryList; cell; cell = cell->next)
    {
      if (cell->memory == str)
        {
          if (previous)
            previous->next = cell->next;
          else
            debugInfo.memoryList = cell->next;
          free (cell);
          return;
        }
      previous = cell;
    }
  /*
   * la chaine liberee doit se trouver dans la liste
   */
  ATW_ASSERT (cell != (atwstrCellMemory *) 0, "Internal error in free cell debug");
}

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrWriteReport()
 *
 * PARAMETERS :        void
 *
 * RETURN VALUE :      void
 *
 * DEFINITION :        Imprime le rapport des memory leak
 *
 *==================================================================*/
static void _atwstrWriteReport
#ifdef ANSI_C                 /* ANSI */
          (void)
#else                           /* NON_ANSI */
          ()
#endif
{
  atwstrCellMemory *cell;
  struct atwstrHeader *head;

  fprintf (stderr, "==========================================================\n");
  fprintf (stderr, " ATTOL TESTWARE:      ATWSTR Usage Report   (DEBUG MODE)  \n");
  fprintf (stderr, "==========================================================\n");
  fprintf (stderr, "----------------------------------------------------------\n");
  fprintf (stderr, "  Leak memory \n");
  fprintf (stderr, "----------------------------------------------------------\n");
  fprintf (stderr, "\n");

  for (cell = debugInfo.memoryList; cell; cell = cell->next)
    {
      head = _atwstrGetHeader (cell->memory);
      fprintf (stderr, "|--module: %s   leak at line: %ld (%u bytes lost)\n",
               cell->fileName, cell->nuLine, head->size);
    }

  fprintf (stderr, "----------------------------------------------------------\n");
  fprintf (stderr, "  General \n");
  fprintf (stderr, "----------------------------------------------------------\n\n");
  fprintf (stderr, "Number of allocated string    %ld\n", debugInfo.allocStr);
  fprintf (stderr, "Number of reallocated string  %ld\n", debugInfo.reallocStr);
  fprintf (stderr, "Number of freed string        %ld\n", debugInfo.freeStr);

  fprintf (stderr, "==========================================================\n");
  fprintf (stderr, "Allocated memory size         %ld\n", debugInfo.allocBytes);
  fprintf (stderr, "Reallocated memory size       %ld\n", debugInfo.reallocBytes);
  fprintf (stderr, "Freed memory size             %ld\n", debugInfo.freeBytes);
  fprintf (stderr, "==========================================================\n");
  fprintf (stderr, "Number of bytes lost          %ld\n",
           debugInfo.allocBytes - debugInfo.freeBytes);
  fprintf (stderr, "==========================================================\n");
}

#endif                          /* ATWSTR_DEBUG */

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrAllocStr(size, file, line)
 *
 * PARAMETERS :        unsigned long size
 *
 * RETURN VALUE :      atwstr *str
 *
 * DEFINITION :        Alloue une nouvelle chaine
 *
 *==================================================================*/
static atwstr *_atwstrAllocStr
#ifdef ANSI_C                 /* ANSI */
#ifdef ATWSTR_DEBUG
          (unsigned size, char *file, long line)
#else
          (unsigned size)
#endif
#else                           /* NON_ANSI */
#ifdef ATWSTR_DEBUG
          (size, file, line)
     unsigned  size;
     char     *file;
     long      line;

#else
          (size)
     unsigned  size;

#endif
#endif
{
  struct atwstrHeader *head;
  char     *str;

  if ((size != ATWSTR_UNKNOWN_SIZE) && (size > ATWSTR_MIN_ALLOC_SIZE))
    size += size >> 3;          /* ajout de 1/8 de la taille en plus */
  else
    size = ATWSTR_MIN_ALLOC_SIZE;

  /*
   * ajout de la taille de l'entete et du '\0' terminal
   */
  size += sizeof (struct atwstrHeader) + 1;

  str = (char *) malloc (size);

  ATW_ASSERT (str != (atwstr *) 0, "Allocation fail");

  /*
   * mise a jour entete
   */
  head = (struct atwstrHeader *) str;
  head->magicNumber = ATWSTR_MAGIC_NUMBER;
  head->size = (unsigned) size;
  head->length = (unsigned) 0;

  /*
   * premier caractere a '\O'
   */
  *(str += sizeof (struct atwstrHeader)) = '\0';

#ifdef ATWSTR_DEBUG
  _atwstrAddCellMemory ((atwstr *) str, file, line);
  debugInfo.allocStr += 1;
  debugInfo.allocBytes += size;
#endif

  return ((atwstr *) str);
}

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrReallocStr(str, size, file, line)
 *
 * PARAMETERS :        atwstr *str, unsigned long size, char *file, long line
 *
 * RETURN VALUE :      atwstr *str
 *
 * DEFINITION :        Realloue une nouvelle chaine
 *
 *==================================================================*/
static atwstr *_atwstrReallocStr
#ifdef ANSI_C                 /* ANSI */
#ifdef ATWSTR_DEBUG
          (atwstr * str, unsigned size, char *file, long line)
#else
          (atwstr * str, unsigned size)
#endif
#else                           /* NON_ANSI */
#ifdef ATWSTR_DEBUG
          (str, size, file, line)
     atwstr   *str;
     unsigned  size;
     char     *file;
     long      line;

#else
          (str, size)
     atwstr   *str;
     unsigned  size;

#endif
#endif
{
  char     *str2;
  struct atwstrHeader *head;

  if ((size > 0) && (size > ATWSTR_MIN_ALLOC_SIZE))
    size += size >> 3;
  else
    size = ATWSTR_MIN_ALLOC_SIZE;

  size += sizeof (struct atwstrHeader) + 1;

  str2 = (char *) realloc ((char *) _atwstrGetHeader (str), size);
  ATW_ASSERT (str2, "Reallocation fail");
  head = (struct atwstrHeader *) str2;
  str2 += sizeof (struct atwstrHeader);

#ifdef ATWSTR_DEBUG
  debugInfo.reallocStr += 1;
  debugInfo.reallocBytes += size - head->size;
  debugInfo.allocBytes += size - head->size;
  _atwstrDelCellMemory (str);
  _atwstrAddCellMemory ((atwstr *) str2, file, line);
#endif

  head->size = size;
  return ((atwstr *) str2);
}

/*===================================================================
 *
 * PRIVATE FUNCTION :  _atwstrFreeStr(str)
 *
 * PARAMETERS :        atwstr *str
 *
 * RETURN VALUE :      atwstr *
 *
 * DEFINITION :        Desalloue un chaine dynamique
 *
 *==================================================================*/
static atwstr *_atwstrFreeStr
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str)
#else                           /* NON_ANSI */
          (str)
     atwstr   *str;

#endif
{
  char     *str2;
  struct atwstrHeader *head = _atwstrGetHeader (str);

#ifdef ATWSTR_DEBUG
  _atwstrDelCellMemory (str);
  debugInfo.freeStr += 1;
  debugInfo.freeBytes += head->size;
#endif

  head->magicNumber = (char) 0xff;
  str2 = (char *) head;

  /*
   * on libere la zone occupee par la chaine
   */
  free (str2);

  return ((atwstr *) 0);
}

/*===================================================================
 *
 * PRIVATE MACRO :     _atwstrIsValidStr(str)
 *
 * PARAMETERS :        atwstr *str
 *
 * RETURN VALUE :      int
 *
 * DEFINITION :        Verifie si la chaine str est bien une chaine
 *                     dynamique
 *
 *==================================================================*/
#define _atwstrIsValidStr(str)   ((int)((_atwstrGetHeader(str))->magicNumber == ATWSTR_MAGIC_NUMBER))

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrStartup()
 *
 * PARAMETERS :       void
 *
 * RETURN VALUE :     void
 *
 * DEFINITION :       Debute l'utilisation du module
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
void      atwstrStartupDebug
#ifdef ANSI_C                 /* ANSI */
          (void)
#else                           /* NON_ANSI */
          ()
#endif
{
}
#endif

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrShutdown()
 *
 * PARAMETERS :       void
 *
 * RETURN VALUE :     void
 *
 * DEFINITION :       Met fin a l'utilisation du module
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
void      atwstrShutdownDebug
#ifdef ANSI_C                 /* ANSI */
          (void)
#else                           /* NON_ANSI */
          ()
#endif
{
  _atwstrWriteReport ();
}
#endif

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrCreate()
 *
 * PARAMETERS :       void
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Cree une nouvelle chaine dynamique de contenu nul
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrCreateDebug
#ifdef ANSI_C                 /* ANSI */
          (char *file, long line)
#else                           /* NON_ANSI */
          (file, line)
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrCreate
#ifdef ANSI_C                 /* ANSI */
          (void)
#else                           /* NON_ANSI */
          ()
#endif
#endif
{
#ifdef ATWSTR_DEBUG
  return (_atwstrAllocStr (ATWSTR_UNKNOWN_SIZE, file, line));
#else
  return (_atwstrAllocStr (ATWSTR_UNKNOWN_SIZE));
#endif
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrDestroy(str)
 *
 * PARAMETERS :       void
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Desalloue une chaine dynamique
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrDestroyDebug
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, char *file, long line)
#else                           /* NON_ANSI */
          (str, file, line)
     atwstr   *str;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrDestroy
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str)
#else                           /* NON_ANSI */
          (str)
     atwstr   *str;

#endif
#endif
{
#ifdef ATWSTR_DEBUG
  PRECONDITION (str);
  ATW_ASSERT (_atwstrIsValidStr (str) != 0, "Bad atwstr string");
#endif

  return (_atwstrFreeStr (str));
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrClone(c)
 *
 * PARAMETERS :       const char *c
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Retourne une chaine dynamique de contenu identique
 *                    a 'c'
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrCloneDebug
#ifdef ANSI_C                 /* ANSI */
          (const char *c, char *file, long line)
#else                           /* NON_ANSI */
          (c, file, line)
     char     *c;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrClone
#ifdef ANSI_C                 /* ANSI */
          (const char *c)
#else                           /* NON_ANSI */
          (c)
     char     *c;

#endif
#endif
{
  atwstr   *str;
  unsigned  lc;
  struct atwstrHeader *head;

#ifdef ATWSTR_DEBUG
  PRECONDITION (c);
#endif

  if (c == (char *) 0)
    {
#ifdef ATWSTR_DEBUG
      str = atwstrCreate (file, line);
#else
      str = atwstrCreate ();
#endif

    }
  else
    {
      lc = strlen (c);

#ifdef ATWSTR_DEBUG
      str = _atwstrAllocStr (lc, file, line);
#else
      str = _atwstrAllocStr (lc);
#endif

      ATW_ASSERT (memcpy (str, c, lc + 1) == str, "String copy failed");
      head = _atwstrGetHeader (str);
      head->length = lc;
    }

  return (str);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrExtract(c, b, n)
 *
 * PARAMETERS :       const char *c, unsigned b, unsigned n
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Retourne une chaine dynamique par extraction
 *                    de la chaine c a partir du caractere en position
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrExtractDebug
#ifdef ANSI_C                 /* ANSI */
          (const char *c, unsigned b, unsigned n, char *file, long line)
#else                           /* NON_ANSI */
          (c, b, n, file, line)
     char     *c;
     unsigned  b;
     unsigned  n;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrExtract
#ifdef ANSI_C                 /* ANSI */
          (const char *c, unsigned b, unsigned n)
#else                           /* NON_ANSI */
          (c, b, n)
     char     *c;
     unsigned  b;
     unsigned  n;

#endif
#endif
{
  atwstr   *str;
  struct atwstrHeader *head;
  int       len;

  len = _atwstrIsValidStr (c) ? _atwstrGetHeader (c)->length : strlen (c);
  if (b >= len)
    return atwstrCreate ();     /* rien a extraire */

  /* ajuster le nombre de caracteres a extraire */
  if (b + n > len)
    n = len - b;

#ifdef ATWSTR_DEBUG
  PRECONDITION (c != (char *) 0);
  PRECONDITION (b + n <= strlen (c));
  str = _atwstrAllocStr (n, file, line);
#else
  str = _atwstrAllocStr (n);
#endif

  ATW_ASSERT (memcpy (str, (c + b), n) == str, "String copy failed");
  head = _atwstrGetHeader (str);
  head->length = n;
  str[n] = '\0';

  return (str);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrAppend(str, c)
 *
 * PARAMETERS :       atwstr *str, const char *c
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Concatene la chaine c a la chaine str, si la chaine
 *                    str est trop petite la zone est automatique agrandie
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrAppendDebug
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, const char *c, char *file, long line)
#else                           /* NON_ANSI */
          (str, c, file, line)
     atwstr   *str;
     char     *c;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrAppend
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, const char *c)
#else                           /* NON_ANSI */
          (str, c)
     atwstr   *str;
     char     *c;

#endif
#endif
{
  struct atwstrHeader *head;
/* BC remove warnings */
/*  struct atwstrHeader *c_head; */
  unsigned  lc;

#ifdef ATWSTR_DEBUG
  PRECONDITION (c != (char *) 0);
  PRECONDITION (str != (atwstr *) 0);
  ATW_ASSERT (_atwstrIsValidStr (str) != 0, "Bad atwstr string");
#endif

  if (c == (char *) 0)
    return (str);

  head = _atwstrGetHeader (str);

  lc = strlen (c);
  if ((head->length + lc) > (head->size - sizeof (struct atwstrHeader) - 1))
    {
      /*
       * la taille de la chaine est trop petite il faut la reallouer
       */
#ifdef ATWSTR_DEBUG
      str = _atwstrReallocStr (str, head->length + lc, file, line);
#else
      str = _atwstrReallocStr (str, head->length + lc);
#endif
    }

  ATW_ASSERT (strcat (str, c) == str, "String concat failed");
  head = _atwstrGetHeader (str);
  head->length += lc;

  return (str);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrInsert(str, c, b, n)
 *
 * PARAMETERS :       atwstr *str, const char *c, unsigned b, unsigned n
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Insere les n caractere de la chaine c dans la chaine str
 *                    a la position b
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrInsertDebug
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, const char *c, unsigned b, unsigned n, char *file, long line)
#else                           /* NON_ANSI */
          (str, c, b, n, file, line)
     atwstr   *str;
     char     *c;
     unsigned  b;
     unsigned  n;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrInsert
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, const char *c, unsigned b, unsigned n)
#else                           /* NON_ANSI */
          (str, c, b, n)
     atwstr   *str;
     char     *c;
     unsigned  b;
     unsigned  n;

#endif
#endif
{
  struct atwstrHeader *head = _atwstrGetHeader (str);
  int       i;
  int       len;
/* BC remove warnings */
  struct atwstrHeader *headInsStr;
  headInsStr = _atwstrGetHeader ((atwstr *) c);

#ifdef ATWSTR_DEBUG
  PRECONDITION (c != (char *) 0);
  PRECONDITION (str != (atwstr *) 0);
  ATW_ASSERT (_atwstrIsValidStr (str) != 0, "Bad atwstr string");
  PRECONDITION (n <= strlen (c));
  PRECONDITION (b <= head->length);
#endif

  /* check sur le nombre demande en fonction de la longueur de la chaine */
  len = strlen (c);
  if (n > len)
    n = len;                    /* limiter les car. inseres */

/*ADM : deja fait !=>   head = _atwstrGetHeader(str); */
  if ((head->length + n) > (head->size - sizeof (struct atwstrHeader) - 1))
    {
      /*
       * la taille de la chaine est trop petite il faut la reallouer
       */
#ifdef ATWSTR_DEBUG
      str = _atwstrReallocStr (str, head->length + n, file, line);
#else
      str = _atwstrReallocStr (str, head->length + n);
#endif
      head = _atwstrGetHeader (str);    /* en cas de reallocation */
    }

  /* decaler partie apres zone inseree */
  for (i = head->length; i >= (int) b; i--)
    *(str + n + i) = *(str + i);

  /* inserer la chaine c */
  for (i = 0; i < n; i++)
    *(str + b + i) = *(c + i);

/*         for (i=0; i<n; i++)
 *        {
 *                *(str+b+n+i) = *(str+b+i);
 *                *(str+b+i)   = *(c+i);
 *        }
 */
  head = _atwstrGetHeader (str);
/* pas besoin \0 gere comme un autre caractere
 *          *(str+n+head->length) = '\0';
 */
  head->length += n;

  return (str);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrTrunc(str, b, n)
 *
 * PARAMETERS :       atwstr *str, unsigned b, unsigned n
 *
 * RETURN VALUE :     atwstr *
 *
 * DEFINITION :       Supprime les n caractere de la chaine str a partir
 *                    de la position b
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
atwstr   *atwstrTruncDebug
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, unsigned b, unsigned n, char *file, long line)
#else                           /* NON_ANSI */
          (str, b, n, file, line)
     atwstr   *str;
     unsigned  b;
     unsigned  n;
     char     *file;
     long      line;

#endif
#else
atwstr   *atwstrTrunc
#ifdef ANSI_C                 /* ANSI */
          (atwstr * str, unsigned b, unsigned n)
#else                           /* NON_ANSI */
          (str, b, n)
     atwstr   *str;
     unsigned  b;
     unsigned  n;

#endif
#endif
{
  struct atwstrHeader *head = _atwstrGetHeader (str);
  int       i;

#ifdef ATWSTR_DEBUG
  PRECONDITION (str != (atwstr *) 0);
  ATW_ASSERT (_atwstrIsValidStr (str) != 0, "Bad atwstr string");
  PRECONDITION (b + n <= head->length);
#endif

  /* test des bornes de tronquature suivant type str */
  if (b >= head->length)
    return str;                 /* rien a tronquer */

  /* nombre de caracteres a tronquer plus grand que la chaine */
  if ((int) b + (int) n - 1 >= (int) head->length)
    n = head->length - b;

  for (i = 0; i < (head->length - b - n + 1); i++)
    *(str + b + i) = *(str + b + n + i);
  head->length -= n;

  /*
   * La taille de la chaine a ete modifie, on va reallouer la chaine
   * afin de recuperer de la place memoire si cela est possible
   */
#ifdef ATWSTR_DEBUG
  str = _atwstrReallocStr (str, head->length, file, line);
#else
  str = _atwstrReallocStr (str, head->length);
#endif

  return (str);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrLength(str)
 *
 * PARAMETERS :       const atwstr *str
 *
 * RETURN VALUE :     unsigned
 *
 * DEFINITION :       Retourne la longueur de la chaine dynamique
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
unsigned  atwstrLengthDebug
#ifdef ANSI_C                 /* ANSI */
          (const atwstr * str, char *file, long line)
#else                           /* NON_ANSI */
          (str, file, line)
     atwstr   *str;
     char     *file;
     long      line;

#endif
#else
unsigned  atwstrLength
#ifdef ANSI_C                 /* ANSI */
          (const atwstr * str)
#else                           /* NON_ANSI */
          (str, b, n)
     atwstr   *str;

#endif
#endif
{
  struct atwstrHeader *head = _atwstrGetHeader (str);

#ifdef ATWSTR_DEBUG
  PRECONDITION (str != (atwstr *) 0);
  ATW_ASSERT (_atwstrIsValidStr (str) != 0, "Bad atwstr string");
#endif

  return (head->length);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrCompare(str1, str2)
 *
 * PARAMETERS :       const atwstr *str1, const atwstr *str2
 *
 * RETURN VALUE :     int
 *
 * DEFINITION :       Retourne le resultat de la comparaison entre
 *                    la chaine str1 et la chaine str2
 *                    0 (str1 == str2)
 *                    <0 (str1 < str2)
 *                    >0 (str1 > str2)
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
int       atwstrCompareDebug
#ifdef ANSI_C                 /* ANSI */
          (const atwstr * str1, const atwstr * str2, char *file, long line)
#else                           /* NON_ANSI */
          (str1, str2, file, line)
     atwstr   *str1;
     atwstr   *str2;
     char     *file;
     long      line;

#endif
#else
int       atwstrCompare
#ifdef ANSI_C                 /* ANSI */
          (const atwstr * str1, const atwstr * str2)
#else                           /* NON_ANSI */
          (str1, str2)
     atwstr   *str1;
     atwstr   *str2;

#endif
#endif
{
  struct atwstrHeader *head1 = _atwstrGetHeader (str1);
  struct atwstrHeader *head2 = _atwstrGetHeader (str2);
  int       res;

#ifdef ATWSTR_DEBUG
  PRECONDITION (str1 != (atwstr *) 0);
  PRECONDITION (str2 != (atwstr *) 0);
  ATW_ASSERT (_atwstrIsValidStr (str1) != 0, "Bad atwstr string");
  ATW_ASSERT (_atwstrIsValidStr (str2) != 0, "Bad atwstr string");
#endif

  res = head1->length - head2->length;
  if (res == 0)
    return (strcmp (str1, str2));
  else
    return (res);
}

/*===================================================================
 *
 * PUBLIC FUNCTION :  atwstrConcat( str1, str2, va_alist )
 *
 * PARAMETERS :       const atwstr *str1, const atwstr *str2, va_alist
 *
 * RETURN VALUE :     int
 *
 * DEFINITION :       Retourne le resultat de la concatenation des chaines
 *                    donnees en parametres sous forme atwstr.
 *
 *==================================================================*/
#ifdef ATWSTR_DEBUG
        /* En mode debug les deux derniers argument file* et line sont
           compris dans la liste des arguments */
atwstr   *atwstrConcat
#ifdef ANSI_C                 /* ANSI */
          (const char *str1, const char *str2,...)
#else                           /* NON_ANSI */
          (va_alist)
     va_dcl

#endif
#else
atwstr   *atwstrConcat
#ifdef ANSI_C                 /* ANSI */
          (const char *str1, const char *str2,...)
#else                           /* NON_ANSI */
          (va_alist)
     va_dcl

#endif
#endif
{
  va_list   ap;
  char     *str;
/* BC remove warnings */
/*  struct atwstrHeader *head; */
  atwstr   *res;

#ifndef ANSI_C
  char     *str1,
           *str2;

#endif
#ifdef ATWSTR_DEBUG
  char     *file;
  long      line;

#endif

#ifdef ANSI_C
  va_start (ap, str2);
#else
  va_start (ap);
  str1 = va_arg (ap, char *);
  str2 = va_arg (ap, char *);

#ifdef ATWSTR_DEBUG
  ATW_ASSERT (str1 != NULL, "Missing argument");
  ATW_ASSERT (str2 != NULL, "Missing argument");
#endif
#endif

  /* creation avec premiere chaine, puis ajout avec les autres */
  res = atwstrClone (str1);
  res = atwstrAppend (res, str2);

  /* liste des parametres : fin par NULL */
  str = va_arg (ap, char *);

  while (str != NULL)
    {
      res = atwstrAppend (res, str);
      str = va_arg (ap, char *);
    }

#ifdef ATWSTR_DEBUG
  /* parametres de debug */
  file = va_arg (ap, char *);
  line = va_arg (ap, long);

  /* ajouter ici ATW_ASSERT et PRECONDITION ?? */
#endif

  return res;
}
