

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: dynfagt.c                                            *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)dynfagt $Revision: 1.18 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module DYNFAGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.18 $                                         *
 *% Auteur  Stephane Leroy                                           *
 *% Date    $Date: 2007/01/16 14:12:51 $                             *
 *                                                                   *
 *% Description                                                      *
 *    Dynamic follow-up management + inter-tester communication      *
 *    (*agent* part)                                                 *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* submodules header inclusion */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#ifdef NO_ENV_VAR
#include "glob.h"
#endif

#include "atwstr.h"
#include "ctxagt.h"
#include "tcpip.h"
#include "envagt.h"

#include "dynfagt.h"

#include "cmd_msg.h"

/* Module constants definition */
#define K_ATS_DYNF_PORT       "ATS_PORT_TSTAGT"
#define K_ATS_DYNF_FIN_OK                  "Ok"
#define K_ATS_DYNF_FIN_KO                  "Ko"

typedef struct S_PRIVATE_LISTE_TESTEURS
  {
    T_SOCKET_STREAM socket;
    struct S_PRIVATE_LISTE_TESTEURS *suivant;
  }
T_PRIVATE_LISTE_TESTEURS, *PT_PRIVATE_LISTE_TESTEURS;

/* Module local context */
typedef struct
  {
    T_BOOLEAN                   not_init;
    PT_PRIVATE_LISTE_TESTEURS   debut_liste_testeurs;
    T_SOCKET_STREAM             socket_dynf;
    T_ENTIER                    port_dynf;
  }
T_PRIVATE_CONTEXT_DYNF, *PT_PRIVATE_CONTEXT_DYNF;

/* #define ATS_DYNAGT_DEBUG */
#ifdef ATS_DYNAGT_DEBUG
void p_debug_affiche_liste_testeurs (PT_REF_CONTEXT refContext);
#endif 


/******************************************************************************
*
*  NAME
*   Free memory associated to a list of testers 
*
******************************************************************************/
static void p_libere_liste_testeurs DP ((liste_testeurs)) BP
          PT_PRIVATE_LISTE_TESTEURS liste_testeurs EP
{
  PT_PRIVATE_LISTE_TESTEURS p_testeur_suivant;

  debug (CSTprintf ("~~p_libere_liste_testeurs\n"));

  while (liste_testeurs)
    {
      /* cancel current tester in the list and skip to the next one */
      p_testeur_suivant = liste_testeurs->suivant;
      M_CSTfree (liste_testeurs);
      liste_testeurs = p_testeur_suivant;
    }
}

/******************************************************************************
*
*  NAME
*   Cancel a tester - on the basis of its socket number -
*   in the list of the testers who did subscribe to the dynamic following
*
******************************************************************************/
static int p_supprime_testeur DP ((refContext, 
				   tester_socket)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM tester_socket EP
{
  PT_PRIVATE_CONTEXT_DYNF   context;
  T_PRIVATE_LISTE_TESTEURS  *p_deb, *p_prec, *p;

  context = (PT_PRIVATE_CONTEXT_DYNF) CTXget (refContext, DYNFAGT_CTX);
  
  p_prec = NULL;
  p_deb  = context->debut_liste_testeurs;
  p      = p_deb;

  while (p!=NULL)
    {
      if (p->socket==tester_socket)
	{
	  if (p!=p_deb)
	    {
	      p_prec->suivant = p->suivant;
	    }
	  else
	    {
	      context->debut_liste_testeurs = p->suivant;
	    }
	  M_CSTfree (p);
	  return (0);
	}
      else
	{
	  p_prec = p;
	  p = p->suivant;
	}
    }
  return (1);
}

/******************************************************************************
*
*  NAME
*   Initialization of the module.
*
******************************************************************************/
T_RETURN DYNFAinit DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_DYNF context;
#ifndef NO_ENV_VAR
  T_CAR     numero_port[20];
#endif 

  if (Context == NULL)
    {
      return (SYSTEM_ERROR);
    }
  else
    {
      context = (PT_PRIVATE_CONTEXT_DYNF)CSTmalloc (sizeof (T_PRIVATE_CONTEXT_DYNF));
    }

  CTXset (Context, (void *) context, DYNFAGT_CTX);

  context->port_dynf = 0;
  context->socket_dynf = TCPIPsocket_named_stream (Context, NULL, NULL,
						   &(context->port_dynf));
  if (context->socket_dynf == SYSTEM_ERROR)
    {
      /* Error in socket creation */
      return (SYSTEM_ERROR);
    }

  debug ( CSTprintf ("~~ port dynf : %d\n", context->port_dynf));

/* MODIF BCORMIER : environt variables are not available */
#ifndef NO_ENV_VAR
/* #ifndef VXWORKS */
  /* 
   * Sets the env. variable reflecting port number to use for trace upraising.
   */
  CSTsprintf (numero_port, "%d", context->port_dynf);
  ENVputenv (Context, K_ATS_DYNF_PORT, numero_port);
#else
  AgtGlobals.DYNFport = context->port_dynf;
#endif 

  TCPIPabonne_stream (Context, context->socket_dynf);

  context->not_init = FALSE;
  context->debut_liste_testeurs = NULL;

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*   Termination of the module.
*
******************************************************************************/
T_RETURN DYNFAfin DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_DYNF context;

  context = (PT_PRIVATE_CONTEXT_DYNF) CTXget (Context, DYNFAGT_CTX);

  if ((context == NULL) && context->not_init)
    {
      return (SYSTEM_ERROR);
    }

  /* freeing tester list */
  p_libere_liste_testeurs (context->debut_liste_testeurs);

  (void) TCPIPclose_stream (Context, context->socket_dynf);

  M_CSTfree (context);

  CTXset (Context, (void *) NULL, DYNFAGT_CTX);

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*   Returns number of the port to use for the dyn. follow-up
*
******************************************************************************/
T_ENTIER DYNFAport_dynf DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_DYNF context;

  context = (PT_PRIVATE_CONTEXT_DYNF) CTXget (Context, DYNFAGT_CTX);

  if ((context == NULL) || context->not_init)
    {
      return (SYSTEM_ERROR);
    }
  else
    {
      return (context->port_dynf);
    }
}

/******************************************************************************
*
*  NAME
*   Welcome procedure for a tester in its "boot" phasis
*
******************************************************************************/
static atwstr   *p_DYNFAtraite_boot_testeur DP ((refContext, context)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_CONTEXT_DYNF context EP
{
  /* Connection of a tester for dynamic follow-up */
  atwstr   *result = NULL;
  T_PRIVATE_LISTE_TESTEURS *p_testeur = NULL;
  T_SOCKET_STREAM nouvelle_socket;
  T_ENTIER longueur;

  debug (CSTprintf ("~~>p_DYNFA_boot_testeur\n") );

  if (TCPIPaccept_stream (refContext, context->socket_dynf, &nouvelle_socket,
                          NULL, NULL)
      == SYSTEM_ERROR)
    {
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_ACCEPTFAILD]);
      /* Error message returned to supervisor */
      return (result);
    }

  /* 
   * Connection is established
   */

  p_testeur = (PT_PRIVATE_LISTE_TESTEURS)CSTmalloc (sizeof (T_PRIVATE_LISTE_TESTEURS));
  /* new connection subscription */
  p_testeur->socket = nouvelle_socket;
  TCPIPabonne_stream (refContext, nouvelle_socket);

  { /* get the namming "identifiant/occurrence"     */
    /* and send an ACK/NACK to the connected tester */

    T_ENTIER   recv_status;
    T_BOOLEAN  estdeconnecte;
    atwstr *   boot_raised_msg = NULL;

    /* get the "boot" msg (denom. identifiant/occurrence) */
    recv_status = TCPIPrecv_texte_stream (refContext, nouvelle_socket, 
					  &boot_raised_msg,
					  &estdeconnecte);

    if ( (recv_status == SYSTEM_ERROR) || estdeconnecte ) 
      { 
	/* send a NACK to emitting tester */
	longueur = CSTstrlen (K_ATS_DYNF_FIN_KO);
	TCPIPsend_stream (refContext, nouvelle_socket, K_ATS_DYNF_FIN_KO,
			  &longueur);

	/* Error message returned to supervisor */
	result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_BOOTRECVFAILD]);
	return (result);
      }
    else
      {
	T_TABCAR msg_spv;

	/* Transmission of the message by return to supervisor */
	/* (ack/nack will be done by supervisor later (after checking) */
	CSTsprintf(msg_spv,"%d %s", nouvelle_socket, boot_raised_msg);
	result = atwstrClone (g_chaine_ret[K_CMD_MSG_RAISEDBOOTINFODYNF]);
	result = atwstrAppend (result, msg_spv);
      }
  }

  /* Add tester at the head of the list */
  p_testeur->suivant = context->debut_liste_testeurs;
  context->debut_liste_testeurs = p_testeur;

  /* send the to-be-returned message to supervisor */
  return (result);
}


/******************************************************************************
*
*  NAME
*   Treat ONE line upcoming from a tester
*    (rio line)(tester end)(intersend)(interrecv)
*
******************************************************************************/
static atwstr *p_DYNFAtraite_ligne DP ((refContext,
					tester_socket,
					rioline)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM tester_socket SP
          atwstr * rioline EP
{
  atwstr   *result = NULL;
  T_BOOLEAN sortie_block = FALSE;
  T_TABCAR  str_socket;

  CSTsprintf (str_socket,"%d ", tester_socket);

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"End_of",CSTstrlen("End_of"))==0))
    {
      /* CASE: reception of a normal end of a tester */

      /* stop listening to this socket */
      TCPIPdesabonne_stream (refContext, tester_socket);

      /* get this tester out of the testers list */
      p_supprime_testeur(refContext, tester_socket);

      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RAISEDENDINFODYNF]);

      sortie_block = TRUE;
    }      

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"TSF_",CSTstrlen("TSF_"))==0))
    { /* CASE: reception of a line with 'TSF' header */

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_notify",CSTstrlen("TSF_notify"))==0))
	{
	  /* CASE: reception of a TSF block supposed to be used later */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFNOTIFY]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_declare_ni",CSTstrlen("TSF_declare_ni"))==0))
	{
	  /* CASE: reception of a declaration of new TSF instance /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFDCLNI]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_synchro",CSTstrlen("TSF_synchro"))==0))
	{
	  /* CASE: reception of an action to be traced /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFSYNCHRO]);
	  sortie_block = TRUE;
	}

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_send",CSTstrlen("TSF_send"))==0))
	{
	  /* CASE: reception of a message to be traced /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFSENDMSG]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_waittil",CSTstrlen("TSF_waittil"))==0))
	{
	  /* CASE: reception of a message to be traced /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFWAITTILMSG]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_begin_waittil",CSTstrlen("TSF_begin_waittil"))==0))
	{
	  /* CASE: reception of a notification for waittil begin /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFBEGINWAITTIL]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_end_waittil",CSTstrlen("TSF_end_waittil"))==0))
	{
	  /* CASE: reception of a notification for a waittil end /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFENDWAITTIL]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_inittermexcept",CSTstrlen("TSF_inittermexcept"))==0))
	{
	  /* CASE: reception of a notification for a init/term/exception block /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFINITTERMEXCEPTBLCK]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_intersend",CSTstrlen("TSF_intersend"))==0))
	{
	  /* CASE: reception of a message to be traced /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFINTERSENDMSG]);
	  sortie_block = TRUE;
	}      

      if ((! sortie_block) && 
	  (CSTstrncmp(rioline,"TSF_interrecv",CSTstrlen("TSF_interrecv"))==0))
	{
	  /* CASE: reception of a message to be traced /t TSF */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_TSFINTERRECVMSG]);
	  sortie_block = TRUE;
	}      
    }

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"INTERSEND",CSTstrlen("INTERSEND"))==0))
    {
      /* CASE: reception of an INTERSEND */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_CIT_INTERSEND]);
      sortie_block = TRUE;
    }      

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"INTERRECV",CSTstrlen("INTERRECV"))==0))
    {
      /* CASE: reception of an INTERRECV */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_CIT_INTERRECV]);
      sortie_block = TRUE;
    }      

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"TIMEOUTRQST",CSTstrlen("TIMEOUTRQST"))==0))
    {
      /* CASE: reception of timeout information request */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_CIT_TIMEOUTRQST]);
      sortie_block = TRUE;
    }

  if ((! sortie_block) && 
      (CSTstrncmp(rioline,"STDIOLKAVAIL",CSTstrlen("STDIOLKAVAIL"))==0))
    {
      /* CASE: reception of studio connection information request */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_STDIOLKAVAIL]);
      sortie_block = TRUE;
    }

  if (! sortie_block)
    {
      /* CASE: reception of a rio line */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RAISEDINFODYNF]);
    }

  /* finishing the assembling of the msg to be sent to supervisor: */
  /* format: <TOKEN> : <NUM.SOCKET> <RIO LINE>   */
  result = atwstrAppend (result, str_socket);
  result = atwstrAppend (result, rioline);

  /* debug Principal */ /* fprintf(stderr, "[%s]\n", rioline); */

  return (result);
}


/******************************************************************************
*
*  NAME
*   Treatment of upraising data coming from ONE SINGLE tester
*     1) Atomization (in simple lines)
*     2) Loop over calls to the function that treats a single line of 1 tester
*     3) result fusionning (trame separators insertion)
*
******************************************************************************/
static atwstr *p_DYNFAtraite_plusieurs_lignes DP ((refContext,
						   tester_socket,
						   buffer,
						   new_buffer)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM tester_socket SP
          atwstr * buffer SP
          atwstr ** new_buffer EP
{
  PT_CAR    buffer_fin = buffer + atwstrLength (buffer);
  PT_CAR    debut_rioline = buffer;
  PT_CAR    fin_cmd = NULL;
  atwstr   *result = NULL;
  atwstr   *result_temporaire = NULL;

  fin_cmd = CSTstrstr (debut_rioline, "\r\n");

  while ((debut_rioline != buffer_fin) &&
/*       (!*fin) && */
         (fin_cmd = CSTstrstr (debut_rioline, "\r\n")))
    {
      /* separator found : let's mark the end of line */
      *fin_cmd = '\0';

      result_temporaire = p_DYNFAtraite_ligne(refContext, tester_socket,
					      debut_rioline);
      if (result_temporaire)
        {
          if (result)
            {
              /* concat the resulting lines (trame separators insertion) */
              result = atwstrAppend (result, "\r\n");
              result = atwstrAppend (result, result_temporaire);
            }
          else
            {
              result = atwstrClone (result_temporaire);
            }
          result_temporaire = atwstrDestroy (result_temporaire);
        }

      /* make the previously changed character come back at the end of line */
      *fin_cmd = '\r';

      /* start of next line = after separators */
      debut_rioline = fin_cmd + 2;
    }

  buffer = atwstrTrunc (buffer, 0, debut_rioline - buffer);

  /* go ahead into buffer : inform the calling procedure via return param */
  *new_buffer = buffer;

  return (result);
}


/******************************************************************************
*
*  NAME
*   Treatment of *SEVERAL* upraising lines of a *SINGLE* tester
*   + transmission to the supervisor
*
******************************************************************************/
static atwstr   *p_DYNFAtraite_remontees_monotesteur DP ((refContext, context, pptesteur, pbuffer, fin_dynf)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_CONTEXT_DYNF context SP
          T_PRIVATE_LISTE_TESTEURS ** pptesteur SP
          atwstr ** pbuffer SP
          T_BOOLEAN * fin_dynf EP
{
  /* reception of ONE or SEVERAL messages coming from a SINGLE tester */
  T_ENTIER longueur = 0;
  atwstr   *result = NULL;
  T_PRIVATE_LISTE_TESTEURS *ptesteur = *pptesteur;
  T_BOOLEAN estdeconnecte = FALSE;

  if (*pbuffer)
    {
      /* get length of buffer before reading the socket */
      longueur = atwstrLength (*pbuffer);
    }

  if (TCPIPrecv_texte_stream (refContext, ptesteur->socket, pbuffer,
                              &estdeconnecte)
      == SYSTEM_ERROR)
    {
      /* error message returned to supervisor */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_LINERECVFAILD]);
      return (result);
    }

  if ((!estdeconnecte) && (longueur != atwstrLength (*pbuffer)))
    {
      atwstr * new_buffer = NULL;

      /* decode received data */
      result = p_DYNFAtraite_plusieurs_lignes (refContext,ptesteur->socket,
					       *pbuffer,&new_buffer);
      *pbuffer  = new_buffer;
      *fin_dynf = FALSE;
    }
  else
    {
      if (estdeconnecte)
	{ 
	  T_TABCAR  str_socket;

	  /* CASE : disconnected tester */
	  CSTsprintf (str_socket,"%d ", ptesteur->socket);
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DECONNECTEDTESTERDYNF]); 
	  result = atwstrAppend (result, str_socket);
	}
      else
	{
	  /* CASE : received line abnormaly empty */
	  result = atwstrClone (g_chaine_ret[K_CMD_MSG_DYNF_LINERECVFAILD]);
	}

      /* let's unsubscribe "faulty" socket */
      TCPIPdesabonne_stream (refContext, ptesteur->socket);

      /* connection is lost --> end of treatment */
      *fin_dynf = TRUE;
    }

  return (result);
}

/******************************************************************************
*
*  NAME
*   Management of the connections used for the dynamic follow-up
*    - get the testers boots, info lines, i.t.c.
*    - concat of the whole and return of the result via function return
*
******************************************************************************/
atwstr   *DYNFAtraite_suividyn DP ((Context, nb_connexions, connexions, pbuffer, fin_dynf)) BP
          PT_REF_CONTEXT Context SP
          T_ENTIER nb_connexions SP
          PT_SOCKET connexions SP
          atwstr ** pbuffer SP
          T_BOOLEAN * fin_dynf EP
{
  atwstr   *result = NULL;
  atwstr   *result_temporaire = NULL;
  T_ENTIER  index;
  PT_PRIVATE_LISTE_TESTEURS *pptesteurs;
  PT_PRIVATE_CONTEXT_DYNF context;

  context = (PT_PRIVATE_CONTEXT_DYNF) CTXget (Context, DYNFAGT_CTX);

  if ((context == NULL) && context->not_init)
    {
      return NULL;
    }

  /* Parse array of connections opened for reading */
  for (index = 0; index < nb_connexions; index++)
    {
      /* check if socket to be treated belongs to this module.
       * If so, launch associated treatment
       */
      if (connexions[index].stream == context->socket_dynf)
        {
          /* new connection of a tester is detected */
	  /* => boot phasis of a tester             */
          result_temporaire = p_DYNFAtraite_boot_testeur (Context, context);
        }
      else
        {
	  /* connexion is already known :                          */
	  /* upraising rio line OR upraising end_of_tester message */

	  /* search connection in the tester list */
          pptesteurs = &(context->debut_liste_testeurs);

          while ((*pptesteurs) &&
                 ((*pptesteurs)->socket != connexions[index].stream))
            {
              pptesteurs = &((*pptesteurs)->suivant);
            }

          if (*pptesteurs)
            {
              /* known tester: treat upcoming data */
	      result_temporaire = p_DYNFAtraite_remontees_monotesteur (Context,
								       context,
								       pptesteurs,
								       pbuffer,
								       fin_dynf);
            }
          else
            {
	      /* UNKNOWN TESTER: we ignore it !!! */
	    }
        }
      if (result_temporaire)
        {
          if (result)
            {
              /* Here, we queue line packets associated to differents */
	      /* testers  (insertion of typical trame separator) */
              result = atwstrAppend (result, "\r\n");
              result = atwstrAppend (result, result_temporaire);
            }
          else
            {
              result = atwstrClone (result_temporaire);
            }
          result_temporaire = atwstrDestroy (result_temporaire);
        }
    }
  return (result);
}



/******************************************************************************
*
*  NAME
*   Treatment of **descending** data (lines) from the supervisor to the testers.
*
******************************************************************************/
atwstr   *DYNFAdescente_ligne DP ((refContext, tester_socket, msg)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM tester_socket SP
          PT_CAR msg EP
{
  T_ENTIER longueur;
  longueur = CSTstrlen (msg);
  /* debug PRINCIPAL */ /* fprintf (stderr, "DESCENTE de <%s> vers %d\n", msg, tester_socket); */
  TCPIPsend_stream (refContext, tester_socket, msg, &longueur);
  return (NULL);
}



/* ----------------- FONCTIONS DE DEBUG ------------------ */

#ifdef ATS_DYNAGT_DEBUG
static void p_debug_affiche_liste_testeurs DP ((refContext)) BP
     PT_REF_CONTEXT refContext EP
{
  T_PRIVATE_LISTE_TESTEURS *p;
  PT_PRIVATE_CONTEXT_DYNF context;

  printf("\n\nCurrent tester list: ");

  context = (PT_PRIVATE_CONTEXT_DYNF) CTXget (refContext, DYNFAGT_CTX);
  if ((context == NULL) && context->not_init)
    {
      printf("NULL context !\n");
      return;
    }

  p = context->debut_liste_testeurs;
  while (p!=NULL)
    {
      printf("%d ",p->socket);
      p = p->suivant;
    }
  printf("\n");
}
#endif /* ATS_DYNAGT_DEBUG */
