

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: envagt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)envagt $Revision: 1.14 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  ENVAGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.14 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Gestion des variables d'environnement                          *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* Inclusion des fichiers d'entete du modules */
#include "atwstr.h"
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif


/* Inclusion de l'entete du modules */
#include "ctxagt.h"
#include "envagt.h"

#ifdef UNIX
/* Lien avec le tableau des variables d'environnement */
extern char **environ;

#endif                          /* UNIX */

/******************************************************************************
*  NAME
*       Initialise le module de gestion des variables d'environement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN ENVinit DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  return (SYSTEM_OK);
}

/******************************************************************************
*  NAME
*       Desinitialise le module de gestion des variables d'environnement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN ENVfin DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  return (SYSTEM_ERROR);
}

#ifndef NO_ENV_VAR
/******************************************************************************
*  NAME
*       Affecte une variable d'environnement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN ENVputenv DP ((Contexte, env, val)) BP
          PT_REF_CONTEXT Contexte SP
          PT_CAR env SP
          PT_CAR val EP
{
  atwstr   *environement;
  T_RETURN  t_return = SYSTEM_ERROR;

  if (env && val)
    {
#ifdef WIN32
      return (((SetEnvironmentVariable ((LPCTSTR) env, (LPCTSTR) val)) != 0)
              ? SYSTEM_OK : SYSTEM_ERROR);

      t_return = (T_RETURN) _putenv (atwstrClone ((char *) environement));
#else
      /*
       * Alloue la memoire pour la chaine de caracteres en 
       * l'initialisant avec la valeur de la chaine env
       */
      environement = atwstrClone (env);

      /* Copie dans la chaine : env=val */
      environement = atwstrAppend (atwstrAppend (environement, "="),
                                   (char *) val);

      /* 
       * Creation de la variable d'environnement ...
       * Attention la chaine passer a putenv est celle definissant la variable.
       * C'est pour cela que celle ci est dupliquee
       */
      t_return = (T_RETURN) putenv (atwstrClone ((char *) environement));

      /* Libere la memoire de la chaine de caractere */
      environement = atwstrDestroy (environement);
#endif
    }
  return (t_return);
}

/******************************************************************************
*  NAME
*       Desafecte une variable d'environnement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN ENVunsetenv DP ((Contexte, env)) BP
          PT_REF_CONTEXT Contexte SP
          PT_CAR env EP
{
  T_RETURN  t_return = SYSTEM_ERROR;

#ifdef UNIX
  char    **pvariable;

  if (env)
    {
      /* Recherche la variable dans la liste */
      for (pvariable = environ; *pvariable; pvariable++)
        {
          if ((CSTstrncmp (*pvariable, env, CSTstrlen (env)) == 0)
              &&
              ((*pvariable)[CSTstrlen (env)] == '='))
            {
              /* On a trouve la variable */
              /* On supprime celle-ci en placant le caractere \0 en debut */
              **pvariable = '\0';
              t_return = SYSTEM_OK;
              break;
            }
        }
    }
#endif                          /* UNIX */

  return (t_return);
}

/*******************************************************************************
*
*  NAME
*       Recupere une variable d'environnement
*
*  ALGORITHME
*
*******************************************************************************/
atwstr   *ENVgetenv DP ((Contexte, env)) BP
          PT_REF_CONTEXT Contexte SP
          PT_CAR env EP
{
  atwstr   *valeur;

#ifdef WIN32
  int       ret;
  char      pvaleur[MAX_PATH + 1];

#else
  PT_CAR    pvaleur;

#endif

#ifdef WIN32
  /*
   * Recupere la valeur de la variable d'environnement, sauf si la chaine
   * est NULL.
   */
  ret = GetEnvironmentVariable (env, pvaleur, (MAX_PATH + 1));
  if (ret && env && pvaleur)
/* BCORMIER #endif */

/* BCORMIER #ifdef UNIX */
#else
    /* 
     * Recupere la valeur de la variable d'environnement, sauf si la chaine 
     * est NULL.
     */
    if (env && (pvaleur = getenv (env)))
#endif

/* BCORMIER #if defined(UNIX) || defined(WIN32) */
      {
        valeur = atwstrClone (pvaleur);
        return (valeur);
      }
    else
      {
        return (NULL);
      }
/* BCORMIER #endif */
}

/*********************************************************************
 *                                                                   *
 * NAME
 *   Evalue la chaine en remplacant les variables d'environnement    *
 *   chaine debutant par le marqueur par leur valeur
 *                                                                   *
 *********************************************************************/
atwstr   *ENVevalue_env DP ((Contexte, ligne, marqueur)) BP
          PT_REF_CONTEXT Contexte SP
          PT_CAR ligne SP
          T_CAR marqueur EP

{
  /* Suite de la chaine de caracteres non encore traitee */
  atwstr   *result;
  PT_CAR    pligne;
  T_CAR     caractere_a_inserer[2];
  T_BOOLEAN estEntreAccolade = FALSE;

  result = atwstrCreate ();
  pligne = (char *) ligne;
  caractere_a_inserer[1] = '\0';

  if (!pligne)
    {
      return (result);
    }

  /* 
   * La ligne est bien definie
   */

  /* Parcours de la ligne caractere par caractere */
  while (*pligne)
    {
      /* Recherche du prochain marqueur */
      while ((*pligne) && (*pligne != marqueur))
        {
          /* le caractere '\' annule le marqueur 
           *    -> on saute le '\' et le caractere suivant en le copiant
           */
          if ((*pligne == '\\') && (*(pligne + 1) == marqueur))
            {
              /* On saute le caractere '\' */
              (*pligne)++;
              /* On sauvegarde le marqueur et on le saute */
              caractere_a_inserer[0] = *pligne++;
              result = atwstrAppend (result, caractere_a_inserer);
            }
          else
            {
              caractere_a_inserer[0] = *pligne++;
              result = atwstrAppend (result, caractere_a_inserer);
            }
        }

      /* Recuperation de la variable d'environnement */
      if (*pligne == marqueur)
        {
          atwstr   *variable;
          PT_CAR    val;

          variable = atwstrCreate ();

          /* Saute le marqueur */
          pligne++;

          if (*(pligne) == '{')
            {
              estEntreAccolade = TRUE;

              /* La variable est delimitee par des accolades */
              pligne++;

              /* On recupere le nom de la variable */
              while (*pligne && (*pligne != '}'))
                {
                  caractere_a_inserer[0] = *pligne++;
                  variable = atwstrAppend (variable, caractere_a_inserer);
                }
              /* Saut du delimiteur '}' */
              pligne++;
            }
          else
            {
              estEntreAccolade = FALSE;

              /* La variable n'est pas delimitee par des accolades */

              /* On recupere le nom de la variable */
              while (*pligne &&
                     (
                       ((*pligne >= 'A') && (*pligne <= 'Z')) ||
                       ((*pligne >= 'a') && (*pligne <= 'z')) ||
                       ((*pligne >= '0') && (*pligne <= '9')) ||
                       (*pligne == '_')
                     )
                )
                {
                  caractere_a_inserer[0] = *pligne++;
                  variable = atwstrAppend (variable, caractere_a_inserer);
                }
            }

          /* On copie la valeur */
          val = ENVgetenv (Contexte, variable);
          if (val)
            {
              result = atwstrAppend (result, val);
              val = atwstrDestroy (val);
            }
          else
            {
              /* Copie la variable tel que */
              caractere_a_inserer[0] = marqueur;
              result = atwstrAppend (result, caractere_a_inserer);
              if (estEntreAccolade)
                {
                  caractere_a_inserer[0] = '{';
                  result = atwstrAppend (result, caractere_a_inserer);
                }
              result = atwstrAppend (result, variable);
              if (estEntreAccolade)
                {
                  caractere_a_inserer[0] = '}';
                  result = atwstrAppend (result, caractere_a_inserer);
                }
            }
          variable = atwstrDestroy (variable);
        }
    }

/*  debug (CSTprintf ("ENVevalue = %s\n", result);)*/

  /* Retourne la nouvelle chaine */
    return (result);
}
#endif /* NO_ENV_VAR */
