

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2004 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: fs_agt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)fs_agt $Revision: 1.17 $ $Date: 2005/11/09 21:29:06 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  FS_AGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.17 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2005/11/09 21:29:06 $                             *
 *                                                                   *
 *% Description                                                      *
 *    Gestion du file system, et transfert de fichier                *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/


#ifdef WIN32
#include "io.h"
#endif


/* Inclusion des entetes standards SystemTest */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#include "atwstr.h"

#include <assert.h>


/* Inclusion des entetes des sous-modules */
#include "ctxagt.h"
#include "tcpip.h"

/* Inclusion de l'entete du module */
#include "fs_agt.h"
#include "fs_agtp.h"

/* Constantes internes du module */
#define FS_TAILLE_MAX_NOM_REPERTOIRE    2044
#define K_CHEMIN_DEFAUT                 "/bin:/usr/bin:";

/******************************************************************************
*  NAME
*       Initialise le module de gestion des variables d'environnement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN FSinit DP ((refContext, en_serveur)) BP
          PT_REF_CONTEXT refContext SP
          T_BOOLEAN en_serveur EP
{

  PT_PRIVATE_CONTEXTE_FS contexte;

  if (refContext == NULL)
    {
      return (SYSTEM_ERROR);
    }
  contexte = (PT_PRIVATE_CONTEXTE_FS) CSTmalloc (sizeof (T_PRIVATE_CONTEXTE_FS));
#ifndef NO_FTP
  contexte->mode_binaire = TRUE;

  /* Creation de la socket utilisee pour le FTP serveur */
  if (en_serveur)
    {
      contexte->port_ftp = 0;
      contexte->socket_ftp = TCPIPsocket_named_stream (refContext,
                                                       NULL,
                                                       NULL,
                                                       &contexte->port_ftp);
      if (contexte->socket_ftp == SYSTEM_ERROR)
        {
          /* Erreur de creation de la socket FTP */
          return (SYSTEM_ERROR);
        }
    }
  else
    {
      contexte->port_ftp = -1;
      contexte->socket_ftp = SYSTEM_ERROR;
    }
#endif                          /* NO_FTP */
  contexte->fs_notinit = FALSE;

  if (CTXset (refContext, (void *) contexte, FSAGT_CTX) != SYSTEM_OK)
    {
      return SYSTEM_ERROR;
    }
  return (SYSTEM_OK);
}

/******************************************************************************
*  NAME
*       Desinitialise le module de gestion des variables d'environnement
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN FSfin DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || (contexte->fs_notinit))
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }
#ifndef NO_FTP
  contexte->port_ftp = -1;
  if (contexte->socket_ftp != SYSTEM_ERROR)
    {
      TCPIPclose_stream (refContext, contexte->socket_ftp);
      contexte->socket_ftp = SYSTEM_ERROR;
    }
  contexte->fs_notinit = TRUE;
#endif                          /* NO_FTP */
  M_CSTfree (contexte);

  CTXset (refContext, (void *) NULL, FSAGT_CTX);

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*       Changement de repertoire courant
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN FSchdir DP ((refContext, nom_repertoire)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_repertoire EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

#ifdef UNIX
  return ((chdir (nom_repertoire) == 0) ? SYSTEM_OK : SYSTEM_ERROR);
#endif                          /* UNIX */
#ifdef WIN32
  return (((SetCurrentDirectory (nom_repertoire)) != 0) ? SYSTEM_OK : SYSTEM_ERROR);
#endif                          /* WIN32 */
#ifdef AGT_CUSTOM
#ifndef NO_FILE_SYSTEM
  if (CSTchdir(nom_repertoire))
	  return (SYSTEM_OK);
  else
#endif /* NO_FILE_SYSTEM */
	  return (SYSTEM_ERROR);
#endif
}

/******************************************************************************
*
*  NAME
*       Recuperation du repertoire courant
*
*  ALGORITHME
*
******************************************************************************/
atwstr   *FSgetcwd DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

#ifdef WIN32
  char      pbuf[MAX_PATH + 1];
#else
  PT_CAR    pbuf;
#endif

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      return (NULL);
    }

#ifdef UNIX
  pbuf = getcwd (NULL, FS_TAILLE_MAX_NOM_REPERTOIRE + 1);
#endif                          /* UNIX */
#ifdef WIN32
  GetCurrentDirectory ((MAX_PATH + 1), (LPTSTR) & pbuf);
#endif                          /* WIN32 */
#ifdef AGT_CUSTOM
  pbuf=
#ifndef NO_FILE_SYSTEM
	CSTgetcwd();
#else
	NULL;
#endif /* NO_FILE_SYSTEM */
#endif

  if (pbuf)
    {
      return (atwstrClone (pbuf));
    }
  else
    {
      return (NULL);
    }
}

/******************************************************************************
*
*  NAME
*       Creation d'un repertoire
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN FSmkdir DP ((refContext, nom_repertoire)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_repertoire EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

#ifdef UNIX
  /*
   * Cree le repertoire avec les droits en  Lecture, Ecriture et Execution pour
   * l'utilisateur de l'agent
   */
  return ((mkdir ((char *) nom_repertoire, S_IRUSR | S_IWUSR | S_IXUSR) == 0)
          ? SYSTEM_OK
          : SYSTEM_ERROR);
#endif                          /* UNIX */
#ifdef WIN32
  /*
   * Cree le repertoire sous WIN32
   */
  return (((CreateDirectory (nom_repertoire, NULL)) != 0) ? SYSTEM_OK : SYSTEM_ERROR);
#endif                          /* WIN32 */
#ifdef AGT_CUSTOM
#ifndef NO_FILE_SYSTEM
  if (CSTmkdir(nom_repertoire))
      return(SYSTEM_OK);
  else
#endif /* NO_FILE_SYSTEM */
      return(SYSTEM_ERROR);
#endif
}

/******************************************************************************
*
*  NAME
*       Suppression d'un repertoire
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN FSrmdir DP ((refContext, nom_repertoire)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_repertoire EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

#ifdef UNIX
  return (((rmdir (nom_repertoire)) == 0) ? SYSTEM_OK : SYSTEM_ERROR);
#endif                          /* UNIX */
#ifdef WIN32
  /*
   * Supprime le repertoire sous WIN32
   */
  return (((RemoveDirectory (nom_repertoire)) != 0) ? SYSTEM_OK : SYSTEM_ERROR);
#endif                          /* WIN32 */
#ifdef AGT_CUSTOM
#ifndef NO_FILE_SYSTEM
  if (CSTrmdir(nom_repertoire))
      return(SYSTEM_OK);
  else
#endif /* NO_FILE_SYSTEM */
      return(SYSTEM_ERROR);
#endif
}

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Positionne le transfert en mode ascii
*
*  ALGORITHME
*
******************************************************************************/
void FSsetmodascii DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return;
    }

  contexte->mode_binaire = FALSE;
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Positionne le transfert en mode binaire
*
*  ALGORITHME
*
******************************************************************************/
void FSsetmodbin DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return;
    }

  contexte->mode_binaire = TRUE;
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       fournit le numero de port a utiliser pour le transfert de fichier
*
*  ALGORITHME
*
******************************************************************************/
T_ENTIER FSgetportftp DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

  return (TCPIPport_stream (refContext, contexte->socket_ftp));
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Initialise la connexion ftp
*
*  ALGORITHME
*
******************************************************************************/
static T_RETURN p_FSinit_ftp DP ((contexte,
                                  refContext,
                                  nom_fichier,
                                  socket_ftp,
                                  en_serveur,
                                  machine,
                                  service,
                                  port))    BP
          PT_PRIVATE_CONTEXTE_FS contexte SP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier SP
          T_SOCKET_STREAM * socket_ftp SP
          T_BOOLEAN en_serveur SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  atsdebug (assert (socket_ftp != NULL));
  atsdebug (assert (contexte != NULL));

  /* Etablie connexion sur le port de transfert */
  if (en_serveur)
    {
      if (TCPIPaccept_stream (refContext, contexte->socket_ftp, socket_ftp, NULL, NULL)
          == SYSTEM_ERROR)
        {
          atsdebug (perror ("socket_ftp = TCPIPaccept_stream"));
          return (SYSTEM_ERROR);
        }
    }
  else
    {
      *socket_ftp = TCPIPsocket_stream (refContext);
      if ((*socket_ftp == SYSTEM_ERROR)
          ||
      (TCPIPconnect_stream (refContext, *socket_ftp, machine, service, port)
       ==
       SYSTEM_ERROR))
        {
          atsdebug (perror ("TCPIPconnect_stream"));
          return (SYSTEM_ERROR);
        }
    }
  return (SYSTEM_OK);
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur
*
*  ALGORITHME
*       Binaire:
*               Copie du la memoire dans le fichier sans intervention.
*
*       ASCII:
*               Parcourir la chaine caractere par caractere
*               Si le caractere est \r et le suivant \n
*                       copier \n dans le fichier
*                       sauter les deux caracteres
*               Sinon
*                       copie le caractere dans le fichier
*                       sauter le caractere
*
*
******************************************************************************/
static T_RETURN p_FSwrite DP ((contexte, refContext, descripteur, memoire, nb_octets)) BP
          PT_PRIVATE_CONTEXTE_FS contexte SP
          PT_REF_CONTEXT refContext SP
          FILE * descripteur SP
     void     *memoire SP
               T_ENTIER nb_octets EP
     {
       T_BOOLEAN erreur_copie = FALSE;
       T_ENTIER  nb_octets_ecris;

                 atsdebug (assert (contexte != NULL));

       if        (contexte->mode_binaire)
         {
           /* Copie binaire bloc/bloc */
           do
             /* Copie le fichier en le protegeant des interruptions */
             {
               errno = 0;
               nb_octets_ecris = fwrite (memoire,
                                         sizeof (T_CAR),
                                         nb_octets,
                                         descripteur);
             }
           while     ((nb_octets_ecris == -1) && (errno == EINTR));

           erreur_copie = nb_octets_ecris == -1;
         }
       else
         {
           PT_CAR    memoire_texte = (PT_CAR) memoire;
           PT_CAR    fin_memoire = memoire_texte + nb_octets;

           /* Copie ascii caractere par caractere, en traduisant \r\n -> \n
            * Attention, ceci depend de la machine et du systeme cible
            */
           while ((memoire_texte < fin_memoire) && (!erreur_copie))
             {
               if ((*memoire_texte == '\r') && (*(memoire_texte + 1) == '\n'))
                 {
                   T_CAR     lettre_ecrite;

                   lettre_ecrite = putc ('\n', descripteur);
                   erreur_copie = lettre_ecrite != '\n';
                   atsdebug (if (erreur_copie) CSTprintf ("ERREUR putc(\\n)\n"));
                   memoire_texte++;
                 }
               else
                 {
                   T_CAR     lettre_ecrite;

                   lettre_ecrite = putc (*memoire_texte, descripteur);
                   erreur_copie = lettre_ecrite != *memoire_texte;
                   atsdebug (if (erreur_copie) CSTprintf ("ERREUR putc(%c)\n",
                                                       *memoire_texte));
                 }
               memoire_texte++;
             }
         }
       return (erreur_copie ? SYSTEM_ERROR : SYSTEM_OK);
     }
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur
*
*  ALGORITHME
*
******************************************************************************/
static T_RETURN p_FSputf DP ((contexte,
                              refContext,
                              nom_fichier,
                              en_serveur,
                              machine,
                              service,
                              port))    BP
          PT_PRIVATE_CONTEXTE_FS contexte SP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier SP
          T_BOOLEAN en_serveur SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  T_BOOLEAN erreur_copie = FALSE;
  T_BOOLEAN deconnexion = FALSE;
  T_SOCKET_STREAM socket_ftp = SYSTEM_ERROR;
  FILE     *descripteur_fichier = NULL;
  atwstr   *memoire_tampon = NULL;
  T_CAR     buffer[1024];
  T_ENTIER  nb_octets_lus = 0;
  void     *p_buffer = NULL;

  atsdebug (assert (contexte != NULL));

  if (contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

  /* Initialisation de la connexion de transfert */
  if (p_FSinit_ftp (contexte,
                    refContext,
                    nom_fichier,
                    &socket_ftp,
                    en_serveur,
                    machine,
                    service,
                    port)
      ==
      SYSTEM_ERROR)
    {
      /* Erreur lors de la creation de la connexion de ftp */
      return (SYSTEM_ERROR);
    }

  /* Creation du fichier cible */

  descripteur_fichier = CSTfopen (nom_fichier, contexte->mode_binaire ? "wb" : "w");

  if (descripteur_fichier == NULL)
    {
      /* Erreur lors de la creation du fichier */
      TCPIPclose_stream (refContext, socket_ftp);
      return (SYSTEM_ERROR);
    }

  /* Copie des donnees jusqu'a la fin de la connexion */
  do
    {
      /* Lit les donnees provenant du superviseur */
      if (contexte->mode_binaire)
        {
          nb_octets_lus = TCPIPrecv_buffer_stream (refContext,
                                                   socket_ftp,
                                                   buffer,
                                                   sizeof (buffer),
                                                   &deconnexion);
          p_buffer = buffer;
        }
      else
        {
          nb_octets_lus = TCPIPrecv_texte_stream (refContext,
                                                  socket_ftp,
                                                  &memoire_tampon,
                                                  &deconnexion);
          p_buffer = memoire_tampon;
        }

      /* Les copies dans le fichier */
      if (nb_octets_lus > 0)
        {
          /* On a recue des donnes, et la connexion n'est pas perdue */
          erreur_copie =
            p_FSwrite (contexte,
                       refContext,
                       descripteur_fichier,
                       p_buffer,
                       nb_octets_lus)
            ==
            SYSTEM_ERROR;
          atsdebug (if (erreur_copie) CSTprintf ("~~~putf ERREUR\n"));
        }
      if (memoire_tampon)
        {
          memoire_tampon = atwstrTrunc (memoire_tampon,
                                        0,
                                        atwstrLength (memoire_tampon));
        }
    }
  while ((!deconnexion) && (!erreur_copie));

  /* Ferme le ficher cree */
  CSTfclose (descripteur_fichier);

  /* on ajoute le droit en execute si mode binaire */
  if (contexte->mode_binaire)
    {
#ifdef WIN32
      typedef int mode_t;
      mode_t    mask;

      _umask (mask = _umask (0));
      erreur_copie = (_chmod (nom_fichier, (~mask) & 0777) == -1);
#else
      mode_t    mask;

      umask (mask = umask (0));
      erreur_copie = (chmod (nom_fichier, (~mask) & 0777) == -1);
#endif
    }

  /* Ferme le canal de communication de transfert */
  TCPIPclose_stream (refContext,
                     socket_ftp);

  /* Liberation de la memoire tampon */
  if (memoire_tampon)
    {
      atwstrDestroy (memoire_tampon);
    }

  return (erreur_copie ? SYSTEM_ERROR : SYSTEM_OK);
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Copie un fichier vers le superviseur
*
*  ALGORITHME
*       Binaire:
*               Faire une operation de lecture dans le fichier
*               Copie des donnes lues dans la memoire sans intervention.
*
*       ASCII:
*               Lire le fichier premier dans car_cour
*               si un caracter \n est rencontre
*                       le remplacer par les caracteres \r\n
*               Sinon
*                       copier car_cour
*               jusqu'a la fin du buffer ou du tampon
*
******************************************************************************/
static T_RETURN p_FSread DP ((contexte, refContext, descripteur, memoire, taille_memoire)) BP
          PT_PRIVATE_CONTEXTE_FS contexte SP
          PT_REF_CONTEXT refContext SP
          FILE * descripteur SP
          PT_CAR memoire SP
          PT_ENTIER taille_memoire EP
{
  T_BOOLEAN erreur_copie = FALSE;
  T_ENTIER  nb_octets_memorises = 0;
  T_CAR     caractere;

  atsdebug (assert (contexte != NULL));

  if (contexte->mode_binaire)
    {
      /* Copie binaire bloc/bloc */
      do
        /* Copie le fichier en le protegeant des interruptions */
        {
          errno = 0;
          nb_octets_memorises = fread (memoire,
                                       sizeof (T_CAR),
                                       *taille_memoire,
                                       descripteur);
        }
      while ((nb_octets_memorises == -1) && (errno == EINTR));

      erreur_copie = nb_octets_memorises == -1;
    }
  else
    {
      /* lecture du fichier caractere par caractere et remplissage de
       * la memoire avec ces caracteres en ajoutant le caractere \r
       * devant le caractere \n.
       * On arrete le remplissage du buffer a l'avant derniere cellule
       * a cas ou le dernier est un \n (qui deviendrait un \r\n  ...
       */

      while ((nb_octets_memorises < *taille_memoire - 1)
             &&
             ((caractere = fgetc (descripteur)) != EOF)
             &&
             !CSTfeof (descripteur))
        {
          if (!ferror (descripteur))
            {
              /* Copie ascii caractere par caractere, en traduisant
               *        \n -> \r\n
               */
              if (caractere == '\n')
                {
                  *memoire++ = '\r';
                  nb_octets_memorises++;
                }
              *memoire++ = caractere;
              nb_octets_memorises++;
            }
          else
            {
              erreur_copie = TRUE;
            }
        }
    }
  *taille_memoire = nb_octets_memorises;
  return (erreur_copie ? SYSTEM_ERROR : SYSTEM_OK);
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur
*
*  ALGORITHME
*
******************************************************************************/
static T_RETURN p_FSgetf DP ((contexte,
                              refContext,
                              nom_fichier,
                              en_serveur,
                              machine,
                              service,
                              port))    BP
          PT_PRIVATE_CONTEXTE_FS contexte SP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier SP
          T_BOOLEAN en_serveur SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  T_BOOLEAN erreur_copie = FALSE;
  T_BOOLEAN deconnexion = FALSE;
  T_SOCKET_STREAM socket_ftp = SYSTEM_ERROR;
  FILE     *descripteur_fichier = NULL;
  T_TABCAR  memoire_tampon;
  PT_CAR    pmemoire_tampon;
  T_ENTIER  longueur;

  atsdebug (assert (contexte != NULL));

  /* Initialisation de la connexion de transfert */
  if (p_FSinit_ftp (contexte,
                    refContext,
                    nom_fichier,
                    &socket_ftp,
                    en_serveur,
                    machine,
                    service,
                    port)
      ==
      SYSTEM_ERROR)
    {
      /* Erreur lors de la creation de la connexion de ftp */
      return (SYSTEM_ERROR);
    }

  /* Ouverture du fichier destination */

  descripteur_fichier = CSTfopen (nom_fichier, contexte->mode_binaire ? "rb" : "r");

  if (descripteur_fichier == NULL)
    {
      TCPIPclose_stream (refContext, socket_ftp);
      return (SYSTEM_ERROR);
    }

  /* Copie des donnees jusqu'a la fin de la connexion */
  do
    {
      longueur = sizeof (memoire_tampon);

      /* Lit les donnees du fichier */
      erreur_copie =
        p_FSread (contexte, refContext, descripteur_fichier,
                  memoire_tampon,
                  &longueur)
        ==
        SYSTEM_ERROR;

      if ((longueur > 0) && !erreur_copie)
        {
          /* On envoie la ligne au superviseur */
          erreur_copie =
            TCPIPsend_stream (refContext, socket_ftp, memoire_tampon, &longueur)
            ==
            SYSTEM_ERROR;
          atsdebug (if (erreur_copie) CSTprintf ("~~~getf ERREUR\n"));
        }
    }
  while (!CSTfeof (descripteur_fichier)
         &&
         !ferror (descripteur_fichier)
         &&
         !erreur_copie);

  /* Ferme le ficher cree */
  CSTfclose (descripteur_fichier);

  /* Ferme le canal de communication de transfert */
  TCPIPclose_stream (refContext, socket_ftp);

  return (erreur_copie ? SYSTEM_ERROR : SYSTEM_OK);
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Envoi un fichier du superviseur en mode serveur
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSputf DP ((refContext, nom_fichier)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }
  return (p_FSputf (contexte, refContext, nom_fichier, TRUE, NULL, NULL, -1));
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur en mode serveur
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSgetf DP ((refContext, nom_fichier)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

  return (p_FSgetf (contexte, refContext, nom_fichier, TRUE, NULL, NULL, -1));
}
#endif                          /* NO_FTP */

#ifndef NO_FILE_SYSTEM
/******************************************************************************
*
*  NAME
*       Suppression d'un fichier
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSunlink DP ((refContext, nom_fichier)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }
/*
#ifdef WIN32
  return (((DeleteFile (nom_fichier)) != 0) ? SYSTEM_OK : SYSTEM_ERROR);
#else
*/
  return (CSTunlink (nom_fichier) ? SYSTEM_ERROR : SYSTEM_OK);
/*
#endif
*/
}
#endif                          /* NO_FILE_SYSTEM */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Envoie un fichier du superviseur en mode client
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSputf_to DP ((refContext, nom_fichier, machine, service, port)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }
  return (p_FSputf (contexte, refContext, nom_fichier, FALSE, machine, service, port));
}
#endif                          /* NO_FTP */

#ifndef NO_FTP
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur en mode client
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSgetf_from DP ((refContext, nom_fichier, machine, service, port)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR nom_fichier SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }

  return (p_FSgetf (contexte, refContext, nom_fichier, FALSE, machine, service, port));
}
#endif                          /* NO_FTP */

#ifdef UNIX
/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur en mode client
*
*  ALGORITHME
*
*******************************************************************************/
static T_RETURN p_FSestExecutable_unix DP ((commande,
                                            chemins,
                                            separateur_repertoire,
                                            separateur_chemin,
                                            est_executable,
                                            est_dans_lun_des_chemins)) BP
          PT_CAR commande SP
          PT_CAR chemins SP
          T_CAR separateur_repertoire SP
          T_CAR separateur_chemin SP
          T_BOOLEAN * est_executable SP
          T_BOOLEAN * est_dans_lun_des_chemins EP
{
  /* Hypothese : la cmd est de taille < a 80 caracteres */
  T_CAR     lettre[2];
  PT_CAR    pchemins;
  atwstr   *instruction;
  atwstr   *nom_complet_instruction;
  T_BOOLEAN nest_pas_executable = TRUE;
  T_BOOLEAN nest_pas_trouve = TRUE;
  T_BOOLEAN recherche_continue = TRUE;
  struct stat etat_fichier;
  uid_t     mon_id = getuid ();
  gid_t     mon_id_de_groupe = getgid ();

  lettre[1] = '\0';
  instruction = atwstrCreate ();
  nom_complet_instruction = atwstrCreate ();
  if (!instruction || !nom_complet_instruction)
    {
      return (SYSTEM_ERROR);
    }

  if (chemins)
    {
      pchemins = chemins;
    }
  else
    {
      pchemins = K_CHEMIN_DEFAUT;
    }

  /* Determine le nom de l'instruction */
  while ((*commande != '\0') && (*commande != ' ') && (*commande != '\t'))
    {
      lettre[0] = *commande++;
      instruction = atwstrAppend (instruction, lettre);
    }

  /* Recherche de l'executable dans le PATH... */
  nom_complet_instruction = atwstrClone (instruction);
  do
    /* Recherche le fichier dans le PATH (chemins) */
    {
      /* Verifie le ficher courant */
      if (stat (nom_complet_instruction, &etat_fichier) == 0)
        {
          nest_pas_trouve = FALSE;
          if (mon_id == etat_fichier.st_uid)
            {
              /* l'instruction appartient au lanceur
               * -> verifie les droits d'execution de l'utilisateur
               */
              nest_pas_executable = !(etat_fichier.st_mode & S_IXUSR);
            }
          else if (mon_id_de_groupe == etat_fichier.st_gid)
            {
              /* L'instruction appartient au groupe du lanceur
               * -> verifie les droits d'execution pour le groupe
               */
              nest_pas_executable = !(etat_fichier.st_mode & S_IXGRP);
            }
          else
            {
              /* Verifie les droits d'execution des autres utilisateurs */
              nest_pas_executable = !(etat_fichier.st_mode & S_IXOTH);
            }
        }
      /* Passe au repertoire suivant si l'executable n'est pas trouve */
      if (nest_pas_executable)
        {
          /* Copie du nouveau repertoire */
          nom_complet_instruction = atwstrTrunc (nom_complet_instruction,
                                                 0,
                                    atwstrLength (nom_complet_instruction));

          while ((*pchemins != '\0') && (*pchemins != ':'))
            {
              lettre[0] = *pchemins;
              pchemins++;
              nom_complet_instruction = atwstrAppend (nom_complet_instruction,
                                                      lettre);
            }
          if (atwstrLength (nom_complet_instruction) > 0)
            {
              lettre[0] = separateur_repertoire;
              nom_complet_instruction = atwstrAppend (nom_complet_instruction,
                                                      lettre);
              nom_complet_instruction = atwstrAppend (nom_complet_instruction,
                                                      instruction);
              if (*pchemins == separateur_chemin)
                {
                  pchemins++;
                }
            }
        }
    }
  while (nest_pas_executable
         &&
         (atwstrLength (nom_complet_instruction) > 0));
  *est_dans_lun_des_chemins = !nest_pas_trouve;
  *est_executable = !nest_pas_executable;
  instruction = atwstrDestroy (instruction);
  nom_complet_instruction = atwstrDestroy (nom_complet_instruction);
  return (SYSTEM_OK);

}
#endif

/******************************************************************************
*
*  NAME
*       Recoit un fichier du superviseur en mode client
*
*  ALGORITHME
*
*******************************************************************************/
T_RETURN FSestExecutable DP ((refContext,
                              commande,
                              chemins,
                              separateur_repertoire,
                              separateur_chemin,
                              est_executable,
                              est_dans_lun_des_chemins)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR commande SP
          PT_CAR chemins SP
          T_CAR separateur_repertoire SP
          T_CAR separateur_chemin SP
          T_BOOLEAN * est_executable SP
          T_BOOLEAN * est_dans_lun_des_chemins EP
{
  PT_PRIVATE_CONTEXTE_FS contexte;

  contexte = (PT_PRIVATE_CONTEXTE_FS) CTXget (refContext, FSAGT_CTX);

  if ((contexte == NULL) || contexte->fs_notinit)
    {
      /* Module non initialise */
      return (SYSTEM_ERROR);
    }
#ifdef UNIX
  return (p_FSestExecutable_unix (commande,
                                  chemins,
                                  separateur_repertoire,
                                  separateur_chemin,
                                  est_executable,
                                  est_dans_lun_des_chemins));
#else
  return (SYSTEM_OK);
#endif
}
