

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: proagt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)proagt $Revision: 1.24 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  PROAGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.24 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Gestion de processus pour l'agent de supervision               *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* inclusion des entetes standart SystemTest */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#include "atwstr.h"
#include "ctxagt.h"
#include "tcpip.h"

#include "traagt.h"

/* Inclusion de l'entete du module */
#include "proagt.h"
#include "proagtp.h"


#ifdef AGT_PROC_TABLE
#include "glob.h"
#endif /* AGT_PROC_TABLE */
#include "rdvagt.h"

#include "fs_agt.h"

/* Nombre maximun de processus gere par le module */
#define PRO_TAILLE_TABLEAU_PROCESSUS    1024
#define K_SEPARATEUR_REPERTOIRE         '/'
#define K_SEPARATEUR_CHEMINS            ':'

/******************************************************************************
*
*  NAME
*       Initialisation de la librairie
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN PROinit DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  if ((contexte == NULL) && (TRAAGTinit (refContext) == SYSTEM_OK))
    {
      contexte = (PT_PRIVATE_CONTEXTE_PRO) CSTmalloc (sizeof (T_PRIVATE_CONTEXTE_PRO));

      contexte->tab_dernier = 0;
      contexte->proagt_not_init = FALSE;
      contexte->taille_tab = PRO_TAILLE_TABLEAU_PROCESSUS;
      contexte->tab_processus = (PT_TAB_PROCESSUS)CSTmalloc (contexte->taille_tab * sizeof (T_TAB_PROCESSUS));
      contexte->active_trace = TRUE;
#ifdef UNIX
      signal (SIGCHLD, SIG_DFL);
      /*
       * ATTENTION le SIG_IGN est different de SIG_DFL. Si le SIG_IGN est 
       * positionne, alors la fonction waitpid ne fonctionne plus !!!
       */
#endif

      CTXset (refContext, contexte, PROAGTD_CTX);

      return (SYSTEM_OK);
    }
  else
    {
      /* La librairie est deja initialisee */
      return (SYSTEM_ERROR);
    }
}

/******************************************************************************
*
*  NAME
*       Initialisation de la librairie
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN PROfin DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  T_ENTIER  index;
  T_ENTIER  etat;
  T_PROCESSUS identifiant;
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  if ((contexte == NULL) || (contexte->proagt_not_init))
    {
      /* La librairie n'est pas initialisee !! */
      return (SYSTEM_ERROR);
    }
  else
    {
      /* Arret de toutes les commandes lancer lors de la session */
      for (index = 0; index < contexte->tab_dernier; index++)
        {
          /* Recupere l'etat des processus en cours d'execution */
          if (contexte->tab_processus[index].etat == PRO_ENCOURS)
            {
              identifiant = index + 1;
              PROstop (refContext, identifiant, &etat);
            }
        }
      CSTfree (contexte->tab_processus);

      contexte->proagt_not_init = TRUE;
      TRAAGTfin (refContext);

      M_CSTfree (contexte);
      CTXset (refContext, NULL, PROAGTD_CTX);
      return (SYSTEM_OK);
    }
}

/******************************************************************************
*
*  NAME
*       recupere l'etat des processus terminee
*
*  ALGORITHME
*
******************************************************************************/
static T_BOOLEAN p_PROwaitpid DP ((contexte, refContext, index_proc)) BP
          PT_PRIVATE_CONTEXTE_PRO contexte SP
          PT_REF_CONTEXT refContext SP
          T_ENTIER index_proc EP
{
  T_BOOLEAN esttermine = FALSE;

#ifndef AGT_CUSTOM
#ifdef UNIX
  T_ENTIER  etat=0;
  pid_t     pid = 0;

  /* Recupere l'etat du processus... */
  do
    {
      errno = 0;
      pid = waitpid (contexte->tab_processus[index_proc].identifiant,
                     &etat, WNOHANG);
    }
  while ((pid == -1) && (errno == EINTR));

  /* Verifie l'etat du processus */
  if ((pid == -1)
      ||
      ((pid > 0) && (WIFEXITED (etat))))
    {
      /* Le processus est MORT */
      contexte->tab_processus[index_proc].etat_sortie = WEXITSTATUS (etat);
      contexte->tab_processus[index_proc].etat = PRO_TERMINE;
      esttermine = TRUE;
    }
#endif                          /* UNIX */
#ifdef WIN32
  DWORD     dwExitCode;
  HANDLE    pid = contexte->tab_processus[index_proc].identifiant;

  /* Verifie si le process est actif */
  if (GetExitCodeProcess (pid, &dwExitCode) == FALSE)
    {
      debug (CSTprintf ("GetExitCode Error = %d, dwExitCode = %d",
                        GetLastError (), dwExitCode));
      return (esttermine);
    }
  if (dwExitCode != STILL_ACTIVE)
    {
      int       ret;

      contexte->tab_processus[index_proc].etat = PRO_TERMINE;
      contexte->tab_processus[index_proc].etat_sortie = dwExitCode;
      esttermine = TRUE;
      ret = CloseHandle (pid);
      debug (CSTprintf ("CloseHandle = %d, LastError = %d",
                        ret, GetLastError ()));

    }
#endif		/* WIN32 */
#else		/* AGT_CUSTOM */
  AGT_PID_T  pid;
  pid = contexte->tab_processus[index_proc].identifiant;

  /* Verifie si le thread est actif */
  if (AGT_processStatus(pid))
    {
      contexte->tab_processus[index_proc].etat = PRO_TERMINE;
      esttermine = TRUE;
    }
#endif 		/* AGT_CUSTOM */


  return (esttermine);
}

/******************************************************************************
*
*  NAME
*       Split the command line into an array
*
*  ALGORITHME
*  	split the string 'pcommand' in an array of strings
*  	each element of the array is an argument
*
******************************************************************************/

T_ENTIER splitCommand DP ((pcommand, arguments)) BP
      PT_CAR    pcommand SP
      PT_CAR    arguments[MAX_ARGUMENT] EP
{
  T_ENTIER  index;
  T_BOOLEAN inside_arg;
  T_BOOLEAN between_double_quotes;
  T_ENTIER  shifting;


  /* local variables initialisation */
  index = 0;
  inside_arg = FALSE;
  between_double_quotes = FALSE;
  shifting = 0;
  arguments[0] = NULL;

  /* the command should not be null */
  if (pcommand == NULL)
      return(-1);

  while (*pcommand && (index < MAX_ARGUMENT))
    {
      switch (*pcommand)
        {
        case ' ':
        case '\t':
          {
            if (!between_double_quotes)
              {
                /* separator, delete it, replace it by \0 and go further */
                *pcommand = '\0';
                inside_arg = FALSE;
              }
            else if (!inside_arg)
              {
                arguments[index++] = pcommand;
                inside_arg = TRUE;
              }
            pcommand++;
            break;
          }
        case '\"':
          {
            /* \t and spaces are no longer separator between double quotes so switch mode */
            between_double_quotes = !between_double_quotes;
            /* shift */
            shifting++;
            break;
          }
        case '\\':
          {
            /* shift */
            shifting++;
            *pcommand = pcommand[shifting];
            if (*pcommand == '\"')
              {
                if (!inside_arg)
                  {
                    /* debut d'un argument avec guillemet compris */
                    arguments[index++] = pcommand;
                    inside_arg = TRUE;
                  }
                pcommand++;
              }
            break;
          }
        default:
          {
            if (!inside_arg)
              {
                /* new argument */
                arguments[index++] = pcommand;
                inside_arg = TRUE;
              }
            pcommand++;
            break;
          }
        }
      if (shifting > 0)
        *pcommand = pcommand[shifting];
    }
  /* end of arguments array */
  arguments[index] = NULL;
  return(index);
}	/* end of splitCommand */


#ifdef UNIX
/******************************************************************************
*
*  NAME
*       Commute le processus sur l'execution de la commande passe en parametre
*
*  ALGORITHME
*
******************************************************************************/
static pid_t pPROClanceProcessus_unix DP ((contexte, refContext, commande)) BP
          PT_PRIVATE_CONTEXTE_PRO contexte SP
          PT_REF_CONTEXT refContext SP
          PT_CAR commande EP
{
  /* Compte le nombre d'argument */
  pid_t     identifiant;
  PT_CAR    pcommande;
  T_ENTIER  index = 0;
  PT_CAR    arguments[MAX_ARGUMENT + 3];
  T_BOOLEAN en_cours_arg;
  T_BOOLEAN en_cours_guil;
  T_ENTIER  decalage;
  atwstr   *clone_cmd;

  identifiant = fork ();

  if (identifiant != 0)
    {
      /* Retour pour le processus Pere */
      return (identifiant);
    }

  /* 
   * Code du processus FILS !!! 
   */

  /*
   * (suite patch H323): fermeture des connexions RDV en cours
   */
#ifdef UNIX
  RDVAclose (refContext);
#endif

  /* Activation des traces si necessaire... */
  (void) TRAAGTactive (refContext, contexte->active_trace);

  /* Recopie car modification de la chaine */
  clone_cmd = atwstrClone (commande);
  pcommande = clone_cmd;

  splitCommand(pcommande, arguments);

  /* Section de debbugage */
  debug ( 
          index = 0;
          while (arguments[index])
          {
          CSTfprintf (stderr, "~~ exec %d: %s\n", index, arguments[index]);
          index++;
          }
  )

    /* Lance l'execution de la commande */
    execvp (arguments[0], (char **) arguments);

  clone_cmd = atwstrDestroy (clone_cmd);

  /* L'execution de la commande na pas abouti -> arret du fils 
   * et envoi d'un code d'erreur au module 
   */
  CSTexit (K_CODE_EXEC_FAILD);
  return (identifiant);
}
#endif                          /* UNIX */

#ifdef WIN32
/******************************************************************************
*
*  NAME
*       Demarre le thread sur l'execution de la commande passe en parametre
*
*  ALGORITHME
*
******************************************************************************/
HANDLE pPROClanceProcessus_win32 DP ((contexte, refContext, commande)) BP
          PT_PRIVATE_CONTEXTE_PRO contexte SP
          PT_REF_CONTEXT refContext SP
          PT_CAR commande EP

{

  BOOL      ret;
  HANDLE    identifiant;

  /* Parametres CreateProcess() WIN32 */
  STARTUPINFO siStartInfo;
  PROCESS_INFORMATION piProcInfo;

  memset (&siStartInfo, 0, sizeof (STARTUPINFO));
  memset (&piProcInfo, 0, sizeof (PROCESS_INFORMATION));
  siStartInfo.cb = sizeof (STARTUPINFO);
  siStartInfo.dwFlags = 0;
  siStartInfo.lpReserved = NULL;
  siStartInfo.lpReserved2 = NULL;
  siStartInfo.cbReserved2 = 0;
  siStartInfo.lpDesktop = NULL;
  siStartInfo.lpTitle = (LPTSTR) NULL;

  /* Activation des traces si necessaire... */
  if (TRAAGTactive (refContext, contexte->active_trace, &siStartInfo) == SYSTEM_ERROR)
    return (-1);

  /* Creation du process WIN32 */

  ret = CreateProcess (NULL, commande, NULL, NULL,
                       TRUE, 0, NULL, NULL, &siStartInfo, &piProcInfo);
  CloseHandle (piProcInfo.hThread);
  identifiant = piProcInfo.hProcess;

  /* Creation de process sous WIN32 retourne 0 si echec */
  if (ret)
    {
      return (identifiant);
    }
  else
    {
      debug (CSTprintf ("Error CreateProcess = %d\n", GetLastError ()));
      return (-1);
    }
}
#endif                          /* WIN32 */


/******************************************************************************
*
*  NAME
*       Cree un nouveau processus executant la commande passe en parametre
*
*  ALGORITHME
*
******************************************************************************/
T_PROCESSUS PROcree_proc DP ((refContext, commande)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR commande EP
{
  T_PROCESSUS pid = -1;

  AGT_PID_T identifiant_processus = (AGT_PID_T)-1;
  PT_CAR    args[MAX_ARGUMENT];
  T_ENTIER nbArgs;

#ifdef AGT_CUSTOM
  AGT_ENTRY_POINT entrypoint;
#endif

#ifdef UNIX
  T_BOOLEAN est_executable;
  T_BOOLEAN est_trouve;
#endif

#ifdef AGT_PROC_TABLE
  T_ENTIER i;
  T_BOOLEAN nextElt = TRUE;
#endif

  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

#ifndef AGT_CUSTOM
#ifdef UNIX
  if ((FSestExecutable (refContext, commande, getenv ("PATH"),
                        K_SEPARATEUR_REPERTOIRE, K_SEPARATEUR_CHEMINS,
                        &est_executable, &est_trouve)
       == SYSTEM_OK)
      && (est_executable))
    {
      identifiant_processus = pPROClanceProcessus_unix (contexte, refContext,
                                                        commande);
    }
#endif                          /* UNIX */
#ifdef WIN32
  identifiant_processus = pPROClanceProcessus_win32 (contexte,
                                                     refContext,
                                                     commande);
#endif                          /* WIN32 */
#else				/* AGT_CUSTOM */

  /* Activate traces before launching a process */
  TRAAGTactive (refContext, contexte->active_trace);

  nbArgs = splitCommand(commande, args);
  if (args[0]==NULL)
  	return (-1);
  
#ifndef AGT_PROC_TABLE
  CSTstrcpy(entrypoint, args[0]);
#else   /* AGT_PROC_TABLE */
  for (i=0; i<MAX_TASKS && nextElt; i++)
  {
      if ((CSTstrcmp(AgtTasks[i].f_name, args[0])==0)
	&& (AgtTasks[i].f_pointer!=NULL))
      {
  	entrypoint = AgtTasks[i].f_pointer;
	nextElt = FALSE;
      }
  }
  if (nextElt)
      return(-1);
#endif  /* AGT_PROC_TABLE */ 
  identifiant_processus = AGT_processLaunch(entrypoint, args);
#endif                          /* AGT_CUSTOM */

  if (identifiant_processus != -1)
    {
      /* Ajoute le nouveau processus dans la table des processus */
      contexte->tab_processus[contexte->tab_dernier].identifiant = identifiant_processus;
      contexte->tab_processus[contexte->tab_dernier].etat = PRO_ENCOURS;
      contexte->tab_processus[contexte->tab_dernier].etat_sortie = -1;

      /* 
       * Attention: l'identifiant du processus est sont numero et non sa
       * position dans le tableau.
       */

      contexte->tab_dernier++;
      pid = contexte->tab_dernier;
      if (contexte->tab_dernier == contexte->taille_tab)
        {
          contexte->taille_tab += PRO_TAILLE_TABLEAU_PROCESSUS;
          contexte->tab_processus = (PT_TAB_PROCESSUS)CSTrealloc (contexte->tab_processus,
                           contexte->taille_tab * sizeof (T_TAB_PROCESSUS));
        }
    }
  return ((T_PROCESSUS) pid);
}

/******************************************************************************
*
*  NAME
*       Retourne l'etat d'un processus
*
*  ALGORITHME
*
******************************************************************************/
void p_PROetat DP ((contexte, refContext, index_proc)) BP
          PT_PRIVATE_CONTEXTE_PRO contexte SP
          PT_REF_CONTEXT refContext SP
          T_PROCESSUS index_proc EP
{
  if (contexte->tab_processus[index_proc].etat != PRO_TERMINE)
    {
      (void) p_PROwaitpid (contexte, refContext, index_proc);
    }
}

/******************************************************************************
*
*  NAME
*       Retourne l'etat d'un processus
*
*  ALGORITHME
*
******************************************************************************/
T_ETATPROC PROetat DP ((refContext, identifiant, etat_proc)) BP
          PT_REF_CONTEXT refContext SP
          T_PROCESSUS identifiant SP
          PT_ENTIER etat_proc EP
{
  T_ENTIER  index_proc = identifiant - 1;
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  if ((0 <= index_proc) && (index_proc < contexte->tab_dernier))
    {
      /* Verifie que le processus n'est pas termine */
      if (contexte->tab_processus[index_proc].etat != PRO_TERMINE)
        {
          p_PROetat (contexte, refContext, index_proc);
        }

      *etat_proc = contexte->tab_processus[index_proc].etat_sortie;
      return (contexte->tab_processus[index_proc].etat);
    }
  else
    {
      return (PRO_INCONNU);
    }
}

/******************************************************************************
*
*  NAME
*       Stop l'execution d'un processus
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN PROstop DP ((refContext, identifiant, etat_sortie)) BP
          PT_REF_CONTEXT refContext SP
          T_PROCESSUS identifiant SP
          PT_ENTIER etat_sortie EP
{
  T_ENTIER  index_proc = identifiant - 1;
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  /* Tue le processus si celui-ci fait partie de la base */
  if ((0 <= index_proc) && (index_proc < contexte->tab_dernier))
    {
      /* Verifie que le processus n'est pas deja arrete */
      *etat_sortie = contexte->tab_processus[index_proc].etat_sortie;

      /* Envoie un signal si le processus est en cour d'execution */
      if (contexte->tab_processus[index_proc].etat == PRO_ENCOURS)
        {
#ifndef AGT_CUSTOM
#ifdef UNIX

          /* Envoi du signal d'arret au processus */
          if (kill (contexte->tab_processus[index_proc].identifiant, SIGINT)
              != 0)
            {
              return (SYSTEM_ERROR);
            }
          /* Attente pour etre certain que le processus a recu et traite le QUIT */
          sleep (1);
          if (kill (contexte->tab_processus[index_proc].identifiant, SIGKILL) != 0)
            {
	      CSTfprintf(stderr, "Unable to kill process %d\n", 
		      contexte->tab_processus[index_proc].identifiant);
              return (SYSTEM_ERROR);
            }
          /* Attente pour etre certain que le processus a recu et traite le KILL */
          sleep (1);
#endif                          /* UNIX */
#ifdef WIN32
          if (!TerminateProcess (contexte->tab_processus[index_proc].identifiant, 0))
            {
              return (SYSTEM_ERROR);
            }
#endif                          /* WIN32 */
#else 				/* AGT_CUSTOM */

          /* Tue le thread designe */
          if (!AGT_processStop(contexte->tab_processus[index_proc].identifiant))
            {
              return (SYSTEM_ERROR);
            }
#endif                          /* AGT_CUSTOM */
        }
      return (SYSTEM_OK);
    }
  else
    /* index_proc >= contexte->tab_dernier => UTILISATION INCORRECTE */
    {
      return (SYSTEM_ERROR);
    }
}

/******************************************************************************
*
*  NAME
*       Surveille les processus "fils". Retourne le nombre de processus dont
*       la fin d'execution a ete detecte.
*
*  ALGORITHME
*
******************************************************************************/
T_ENTIER PROsurveille DP ((refContext, procfinis)) BP
          PT_REF_CONTEXT refContext SP
          PT_PROC_SURV procfinis EP
{
  /* Variable de boucle */
  int       index;

  /* Defini le nombre de processus dont la fin a ete detectee */
  int       nb_detectes = 0;

  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  if (!procfinis)
    {                           /* Le tableau fournis n'est pas cree !!! */
      return (-1);
    }

  debug (CSTprintf ("PROsurveille avec Pooling (%d/%d)....\n",
                    contexte->tab_dernier, contexte->taille_tab));
  
#ifdef UNIX
  { /* (suite patch H323) */

    pid_t     pid = 0;
    T_ENTIER  etat =0;

    /* Recupere l'etat des processus fils... */
    do
      {
        errno = 0; 
        pid = waitpid (-1,&etat,WNOHANG);

        /* Verifie l'etat du processus */
        if ((pid == -1) || ((pid > 0) && (WIFEXITED (etat))))
	  {
	    if (pid==-1) {
	      /* special treatment to capture all children's stops */ 
	      for (index = 0; index < contexte->tab_dernier; index++)
		{
		  /* Recupere l'etat des processus en cours d'execution */
		  if ((contexte->tab_processus[index].etat == PRO_ENCOURS)
		      && p_PROwaitpid (contexte, refContext, index))
		    {
		      procfinis->identifiant = index + 1;
		      procfinis->etat = contexte->tab_processus[index].etat_sortie;
		      procfinis++;
		      nb_detectes++;
		    }
		} 
	    } else { /* pid!=-1 */
	      /* Un processus fils est MORT */
	      for (index = 0; index < contexte->tab_dernier; index++)
		{
		  /* Trouver lequel dans la table des processus */
		  if (contexte->tab_processus[index].identifiant == pid)
		    {
		      procfinis->identifiant = index + 1;
		      procfinis->etat = WEXITSTATUS (etat);
		      procfinis++;
		      contexte->tab_processus[index].etat_sortie = WEXITSTATUS (etat);
		      contexte->tab_processus[index].etat = PRO_TERMINE;
		      nb_detectes++;
		    }
		}
	    }
	  }
      } while (pid>0); /* un fils a termin */
  }
  /* #ifdef WIN32 *//* version Windows ne prend pas l'evol H323 */
#else /* UNIX */

  /* Parcours de la table complete des processus */
  for (index = 0; index < contexte->tab_dernier; index++)
    {
      /* Recupere l'etat des processus en cours d'execution */
      if ((contexte->tab_processus[index].etat == PRO_ENCOURS)
	  && p_PROwaitpid (contexte, refContext, index))
	{
	  /* Enregistre dans le tableau l'identifiant du processus */
	  /* 
	   * !!! L'identifiant est decale de 1 avec l'index 
	   */
	  procfinis->identifiant = index + 1;
	  procfinis->etat = contexte->tab_processus[index].etat_sortie;
	  procfinis++;
	  nb_detectes++;
	}
    }
#endif /* UNIX */ 
  
  /* Retourne le nombre de processus don la terminaison a ete detectee */
  return (nb_detectes);
}

/******************************************************************************
*
*  NAME
*       Active les traces
*
*  ALGORITHME
*
******************************************************************************/
void PROactive_trace DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  contexte->active_trace = TRUE;
}

/******************************************************************************
*
*  NAME
*       Desactive les traces
*
*  ALGORITHME
*
******************************************************************************/
void PROdesactive_trace DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_PRO contexte;

  contexte = (PT_PRIVATE_CONTEXTE_PRO) CTXget (refContext, PROAGTD_CTX);

  contexte->active_trace = FALSE;
}

/******************************************************************************
*
*  NAME
*       Cree port de trace 
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN PROport_de_trace DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  return (T_RETURN)(TRAAGTport (refContext));
}

/******************************************************************************
*
*  NAME
*       Cree port de trace 
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN PROetablie_trace DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  return (TRAAGTetablie (refContext));
}
