

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: rdvagt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)rdvagt $Revision: 1.16 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module RDVAGT                                                    *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.16 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Gestion des rendez-vous de l'agent                             *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* Entete des sous modules utilise par RDVAGT */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#ifdef NO_ENV_VAR
#include "glob.h"
#endif

#include "atwstr.h"
#include "ctxagt.h"
#include "tcpip.h"
#include "envagt.h"
#include "rdvagt.h"
#include "cmd_msg.h"

/* Definition des constantes du modules */
#define K_ATS_RDV_PORT          "ATS_PORT_RDV"
#define K_ATS_RDV_FIN_OK        "Ok"
#define K_ATS_RDV_FIN_KO        "Ko"

/* Declaration des types internes */
typedef struct S_PRIVATE_LISTE_CLIENTS
  {
    T_SOCKET_STREAM client;
    struct S_PRIVATE_LISTE_CLIENTS *suivant;
  }
T_PRIVATE_LISTE_CLIENTS, *PT_PRIVATE_LISTE_CLIENTS;

typedef struct S_PRIVATE_LISTE_RDVA
  {
    atwstr   *nom;
    T_ENTIER  nb_clients;
    T_PRIVATE_LISTE_CLIENTS *clients;
    struct S_PRIVATE_LISTE_RDVA *suivant;
  }
T_PRIVATE_LISTE_RDVA, *PT_PRIVATE_LISTE_RDVA;

/* Contexte du module */
typedef struct
  {
    T_BOOLEAN not_init;
    PT_PRIVATE_LISTE_RDVA debut_liste_rdv;
    PT_PRIVATE_LISTE_CLIENTS debut_liste_nouveaux_clients;
    T_SOCKET_STREAM socket_rdv;
    T_ENTIER  port_rdv;
  }
T_PRIVATE_CONTEXT_RDV, *PT_PRIVATE_CONTEXT_RDV;

/******************************************************************************
*
*  NAME
*   Libere une liste de client
*
******************************************************************************/
static void p_libere_liste_clients DP ((refContext, ok, clients)) BP
          PT_REF_CONTEXT refContext SP
          T_BOOLEAN ok SP
          PT_PRIVATE_LISTE_CLIENTS clients EP
{
  PT_PRIVATE_LISTE_CLIENTS pclient_suivant;
  T_ENTIER  longueur;

  /* Le pointeur clients doit imperativement etre renseigne */
  debug (assert (clients != NULL));
  debug (CSTprintf ("~~p_libere_liste_clients %d %d\n", ok, clients->client));

  while (clients)
    {
      /* Ferme la connexion */
      if (clients->client > 0)
        {
          if (ok)
            {
              longueur = CSTstrlen (K_ATS_RDV_FIN_OK);
              TCPIPsend_stream (refContext, clients->client, K_ATS_RDV_FIN_OK,
                                &longueur);
            }
          else
            {
              longueur = CSTstrlen (K_ATS_RDV_FIN_KO);
              TCPIPsend_stream (refContext, clients->client, K_ATS_RDV_FIN_KO,
                                &longueur);
            }
          (void) TCPIPclose_stream (refContext, clients->client);
        }
      /* Supprime le client courant et passe au suivant */
      pclient_suivant = clients->suivant;
      M_CSTfree (clients);
      clients = pclient_suivant;
    }
}

/******************************************************************************
*
*  NAME
*   Libere un rdv
*
******************************************************************************/
static void p_libere_rdv DP ((refContext, ok, prdv)) BP
          PT_REF_CONTEXT refContext SP
          T_BOOLEAN ok SP
          PT_PRIVATE_LISTE_RDVA prdv EP
{
  /* Le pointeur prdv doit imperativement etre renseigne */
  debug (assert (prdv != NULL));
  debug (CSTprintf ("~~p_libere_rdv %d %x\n", ok, prdv));

  if (prdv->clients)
    {
      p_libere_liste_clients (refContext, ok, prdv->clients);
    }
  /* Libere la memoire alloue pour le nom */
  if (prdv->nom)
    {
      atwstrDestroy (prdv->nom);
    }
  /* Libere la memoire alloue au rdv */
  M_CSTfree (prdv);
}

/******************************************************************************
*
*  NAME
*   Libere la liste globale des rdv
*
******************************************************************************/
static void p_libere_liste_rdv DP ((refContext, context)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_CONTEXT_RDV context EP
{
  T_PRIVATE_LISTE_RDVA *rdv;
  T_PRIVATE_LISTE_RDVA *rdv_suivant;

  debug (CSTprintf ("~~p_libere_liste_rdv\n"));
  debug (assert (context != NULL));

  /* Parcours la liste des rdvs en les supprimantes */
  rdv = (T_PRIVATE_LISTE_RDVA *) context->debut_liste_rdv;
  while (rdv != NULL)
    {
      /* Libere le rdv en erreur */
      rdv_suivant = rdv->suivant;
      p_libere_rdv (refContext, FALSE, rdv);
      rdv = rdv_suivant;
    }

  context->debut_liste_rdv = NULL;

  /* Libere la liste des nouveaux clients */
  if (context->debut_liste_nouveaux_clients != NULL)
    {
      PT_PRIVATE_LISTE_CLIENTS pliste;

      pliste = (PT_PRIVATE_LISTE_CLIENTS) context->debut_liste_nouveaux_clients;
      p_libere_liste_clients (refContext, FALSE, pliste);
      context->debut_liste_nouveaux_clients = NULL;
    }
}

/******************************************************************************
*
*  NAME
*   Initialisation de la librairie.
*
******************************************************************************/
T_RETURN RDVAinit DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_RDV context;
#ifndef NO_ENV_VAR
  T_CAR     numero_port[20];
#endif

  if (Context == NULL)
    {
      return (SYSTEM_ERROR);
    }
  else
    {
      context = (PT_PRIVATE_CONTEXT_RDV)CSTmalloc (sizeof (T_PRIVATE_CONTEXT_RDV));
    }

  CTXset (Context, (void *) context, RDVAGT_CTX);

  context->port_rdv = 0;
  context->socket_rdv = TCPIPsocket_named_stream (Context, NULL, NULL,
                                                  &(context->port_rdv));
  if (context->socket_rdv == SYSTEM_ERROR)
    {
      /* Erreur de creation de la socket */
      return (SYSTEM_ERROR);
    }

  debug (CSTprintf ("~~ port rdv : %d\n", context->port_rdv));

#ifndef NO_ENV_VAR
  /* 
   * Positionne la variable d'environement contenant le numero de port
   * a utiliser pour les rdv.
   */
  CSTsprintf (numero_port, "%d", context->port_rdv);
  ENVputenv (Context, K_ATS_RDV_PORT, numero_port);
#else
  AgtGlobals.RDVport =  context->port_rdv;
#endif  

  TCPIPabonne_stream (Context, context->socket_rdv);

  context->not_init = FALSE;
  context->debut_liste_rdv = NULL;
  context->debut_liste_nouveaux_clients = NULL;

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*   Liberation de la librairie.
*
******************************************************************************/
T_RETURN RDVAfin DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_RDV context;

  context = (PT_PRIVATE_CONTEXT_RDV) CTXget (Context, RDVAGT_CTX);

  if ((context == NULL) && context->not_init)
    {
      return (SYSTEM_ERROR);
    }

  p_libere_liste_rdv (Context, context);
  (void) TCPIPclose_stream (Context, context->socket_rdv);

  M_CSTfree (context);

  CTXset (Context, (void *) NULL, RDVAGT_CTX);

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*   Retourne le numero de port a utiliser pour les rendez-vous
*
******************************************************************************/
T_ENTIER RDVAport_rdv DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_RDV context;

  context = (PT_PRIVATE_CONTEXT_RDV) CTXget (Context, RDVAGT_CTX);

  if ((context == NULL) || context->not_init)
    {
      return (SYSTEM_ERROR);
    }
  else
    {
      return (context->port_rdv);
    }
}

/******************************************************************************
*
*  NAME
*   Procedure d'accueil d'un nouveau client.
*
******************************************************************************/
atwstr   *p_RDVAtraite_nouveau_client DP ((refContext, context)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_CONTEXT_RDV context EP
{
  /* Connexion d'un nouveau client a un rendez-vous */
  atwstr   *result = NULL;
  T_PRIVATE_LISTE_CLIENTS *p_client = NULL;
  T_SOCKET_STREAM nouvelle_socket;

  debug (CSTprintf ("~~>p_RDVA_nouveau_client\n"));

  if (TCPIPaccept_stream (refContext, context->socket_rdv, &nouvelle_socket,
                          NULL, NULL)
      == SYSTEM_ERROR)
    {
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_ACCEPTFAILD]);

      /* Message d'erreur retourne au superviseur */
      return (result);
    }

  /* 
   * La connexion est etablie 
   */

  p_client = (T_PRIVATE_LISTE_CLIENTS*)CSTmalloc (sizeof (T_PRIVATE_LISTE_CLIENTS));
  /* Abonne la nouvelle connexion */
  p_client->client = nouvelle_socket;
  TCPIPabonne_stream (refContext, nouvelle_socket);

  /* Ajout du nouveau client au debut de la liste */
  p_client->suivant = context->debut_liste_nouveaux_clients;
  context->debut_liste_nouveaux_clients = p_client;

  debug (CSTprintf ("~~ client { %d, %x}\n",
                    p_client->client,
                    p_client->suivant));

  /* Pas de message a envoyer au superviseur */
  return (NULL);
}

/******************************************************************************
*
*  NAME
*   Creation d'un nouveau rendez-vous
*
******************************************************************************/
T_PRIVATE_LISTE_RDVA *p_RDVAcree_rdv DP ((context, nom_rdv)) BP
          PT_PRIVATE_CONTEXT_RDV context SP
          atwstr * nom_rdv EP
{
  T_PRIVATE_LISTE_RDVA *pliste_rdv = NULL;

  debug (CSTprintf ("~~>p_RDVAcree_rdv %s\n", nom_rdv));
  debug (assert (nom_rdv != NULL));

  /* Creation du nouveau rendez-vous et insertion en debut de la 
   * liste des rendez-vous.
   */
  pliste_rdv = (T_PRIVATE_LISTE_RDVA *) CSTmalloc (sizeof (T_PRIVATE_LISTE_RDVA));
  pliste_rdv->nom = nom_rdv;
  pliste_rdv->nb_clients = 0;

  /* Insertion en debut de la liste */
  pliste_rdv->suivant = context->debut_liste_rdv;
  pliste_rdv->clients = NULL;
  context->debut_liste_rdv = pliste_rdv;

  return (pliste_rdv);
}

/******************************************************************************
*
*  NAME
*   Prend en compte la nouvelle demande de rendez-vous
*
******************************************************************************/
atwstr   *p_RDVAnouveau_rdv DP ((context, nom_rdv, ppclient)) BP
          PT_PRIVATE_CONTEXT_RDV context SP
          atwstr * nom_rdv SP
          T_PRIVATE_LISTE_CLIENTS ** ppclient EP
{
  PT_CAR    pnom_rdv;
  T_PRIVATE_LISTE_CLIENTS *pclient = *ppclient;
  T_PRIVATE_LISTE_RDVA *pliste_rdv;
  atwstr   *result;

  debug (assert (ppclient != NULL));
  debug (CSTprintf ("~~>nouveau_rdv %s\n", nom_rdv));

  /* Traite le cas de debugage via telnet et supprime les caracteres
   *  "\r\n" de la fin de chaine
   */
  pnom_rdv = nom_rdv + atwstrLength (nom_rdv);
  if ((*(pnom_rdv - 1) == '\n') && (*(pnom_rdv - 2) == '\r'))
    {
      /* Supprime les 2 derniers caracteres */
      nom_rdv = atwstrTrunc (nom_rdv,
                             atwstrLength (nom_rdv) - 2,
                             2);
    }

  /* Recherche du rendez-vous dans la liste deja declare */
  pliste_rdv = context->debut_liste_rdv;
  while ((pliste_rdv) && (CSTstrcmp (nom_rdv, pliste_rdv->nom)))
    {
      pliste_rdv = pliste_rdv->suivant;
    }

  if (!pliste_rdv)
    {
      pliste_rdv = p_RDVAcree_rdv (context, nom_rdv);
      if (!pliste_rdv)
        {
          result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_MEMORYFAILD]);
          return (result);
        }
    }

  /*
   * pliste_rdv pointe sur le nouveau rendez-vous qui a ete
   * insere dans la liste.
   */

  /* Supprime le client de la liste des nouveaux abonnes */
  *ppclient = pclient->suivant;

  /* Insere le client en tete de la liste des clients du 
   * rendez-vous */
  pclient->suivant = pliste_rdv->clients;
  pliste_rdv->clients = pclient;
  pliste_rdv->nb_clients++;

  /* Defini le message a envoyer au superviseur */
  result = atwstrClone (g_chaine_ret[K_CMD_MSG_WAITRDV]);
  result = atwstrAppend (result, nom_rdv);
  return (result);
}

/******************************************************************************
*
*  NAME
*   Procedure d'abonnement d'un client
*   ATTENTION: la validite du parametre n'est pas verifiee !!!!
*
******************************************************************************/
atwstr   *p_RDVAtraite_abonement_client DP ((refContext, context, ppclient)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_CONTEXT_RDV context SP
          T_PRIVATE_LISTE_CLIENTS ** ppclient EP
{
  /* Reception d'UN message en provenance d'un client */
  atwstr   *result = NULL;
  atwstr   *rendezvous = NULL;
  T_PRIVATE_LISTE_CLIENTS *pclient = *ppclient;
  T_BOOLEAN estdeconnecte;

  if (TCPIPrecv_texte_stream (refContext, pclient->client, &rendezvous,
                              &estdeconnecte)
      == SYSTEM_ERROR)
    {
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_RECVFAILD]);

      /* Message d'erreur retourne au superviseur */
      return (result);
    }

  if ((!estdeconnecte) && rendezvous)
    {
      /* Prendre en compte la nouvelle demande de rendez-vous */
      result = p_RDVAnouveau_rdv (context, rendezvous, ppclient);
    }
  else
    {
      TCPIPdesabonne_stream (refContext, pclient->client);
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_RECVFAILD]);
    }
  return (result);
}

/******************************************************************************
*
*  NAME
*   Reception d'un message inattendu d'un client !!!
*
******************************************************************************/
atwstr   *p_RDVclient_erreur DP ((refContext, prdv, pclient)) BP
          PT_REF_CONTEXT refContext SP
          PT_PRIVATE_LISTE_RDVA prdv SP
          PT_PRIVATE_LISTE_CLIENTS pclient EP
{
  atwstr   *result;
  atwstr   *texte;
  T_BOOLEAN estdeconnecte;

  /* Les pointeurs ne doivent pas etre NULL !!! */
  debug (assert ((pclient != NULL) && (prdv != NULL)));

  (void) TCPIPrecv_texte_stream (refContext, pclient->client, &texte,
                                 &estdeconnecte);

  /* Genere le message d'erreur */
  result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_CLIENTERR]);
  result = atwstrAppend (result, prdv->nom);

  if (estdeconnecte)
    {
      /* Genere le message d'erreur */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_QUITRDV]);
      result = atwstrAppend (result, prdv->nom);
      TCPIPdesabonne_stream (refContext, pclient->client);
      TCPIPclose_stream (refContext, pclient->client);
      pclient->client = -pclient->client;
    }
  else
    {
      /* Genere le message d'erreur */
      result = atwstrClone (g_chaine_ret[K_CMD_MSG_RDV_CLIENTERR]);
      result = atwstrAppend (result, prdv->nom);
    }

  /* Libere la memoire du texte... */
  if (texte)
    {
      (void) atwstrDestroy (texte);
    }
  return (result);
}

/******************************************************************************
*
*  NAME
*   Effectue la gestion des connexions utilisees pour les RDV
*
******************************************************************************/
atwstr   *RDVAtraite_connexions DP ((Context, nb_connexions, connexions)) BP
          PT_REF_CONTEXT Context SP
          T_ENTIER nb_connexions SP
          PT_SOCKET connexions EP
{
  atwstr   *result = NULL;
  atwstr   *result_temporaire = NULL;
  T_ENTIER  index;
  PT_PRIVATE_LISTE_CLIENTS *ppclients;
  PT_PRIVATE_LISTE_CLIENTS clients;
  PT_PRIVATE_LISTE_RDVA *pprdv;
  T_BOOLEAN enRecherche;
  PT_PRIVATE_CONTEXT_RDV context;

  context = (PT_PRIVATE_CONTEXT_RDV) CTXget (Context, RDVAGT_CTX);

  if ((context == NULL) && context->not_init)
    {
      return NULL;
    }

  /* Parcours de tableau des connexions a lire */
  for (index = 0; index < nb_connexions; index++)
    {
      /* Recherche si la socket a traiter fait partie des sockets du module.
       * Si c'est le cas, lance le traitement correspondant
       */
      if (connexions[index].stream == context->socket_rdv)
        {
          /* Un nouvelle connexion est detectee */
          result_temporaire = p_RDVAtraite_nouveau_client (Context, context);
        }
      else
        {
          /* Recherche la connexion dans la liste des nouveaux clients */
          ppclients = &(context->debut_liste_nouveaux_clients);
          while ((*ppclients)
                 &&
                 ((*ppclients)->client != connexions[index].stream))
            {
              ppclients = &((*ppclients)->suivant);
            }
          if (*ppclients)
            {
              /* Traitement de la reception du traitement du message
               * d'abonement du rendez-vous (normalement)
               */
              result_temporaire = p_RDVAtraite_abonement_client (Context,
                                                                 context,
                                                                 ppclients);
            }
          else
            {
              /* Rechercher dans la liste des abonnes en rendez-vous */
              /* Ceci en parcourant la liste des clients des rdvs */
              enRecherche = TRUE;

              /* Se positionne sur le 1er client (cad 1er client du 1er rdv) */
              pprdv = &(context->debut_liste_rdv);
              clients = *pprdv ? (*pprdv)->clients : NULL;
              while (
                      clients
                      &&
                      *pprdv
                      &&
                      (
                        enRecherche = (
                                        clients->client
                                        !=
                                        connexions[index].stream
                        )
                      )
                )

                {
                  /* Passe au client suivant ... */
                  if (clients->suivant)
                    {
                      /* du meme rendez-vous */
                      clients = clients->suivant;
                    }
                  else
                    {
                      /* le premier client du prochain rendez-vous */

                      pprdv = &(*pprdv)->suivant;
                      /* Positoinement sur le premier client s'il y en a ... */
                      clients = *pprdv ? (*pprdv)->clients : NULL;

                    }
                }
              if (!enRecherche)
                {
                  /* L'evenement provient d'un rendez-vous. */
                  result_temporaire = p_RDVclient_erreur (Context, *pprdv,
                                                          clients);
                }
            }
        }
      if (result_temporaire)
        {
          if (result)
            {
              /* Le message resultat contient deja des message -> on insere
               * le separateur de trame.
               */
              result = atwstrAppend (result, "\r\n");
              result = atwstrAppend (result, result_temporaire);
            }
          else
            {
              result = atwstrClone (result_temporaire);
            }
          result_temporaire = atwstrDestroy (result_temporaire);
        }
    }
  return (result);
}

/******************************************************************************
*
*  NAME
*   Termine un rendez-vous
*
******************************************************************************/
void RDVAtermine DP ((Context, nom, ok)) BP
          PT_REF_CONTEXT Context SP
          PT_CAR nom SP
          T_BOOLEAN ok EP
{
  PT_PRIVATE_LISTE_RDVA *ppliste_rdv;
  PT_PRIVATE_LISTE_RDVA prdv_suivant;
  PT_PRIVATE_CONTEXT_RDV context;

  context = (PT_PRIVATE_CONTEXT_RDV) CTXget (Context, RDVAGT_CTX);

  if ((context == NULL) && context->not_init)
    {
      return;
    }

  /* Recherche du rendez-vous concerne... */
  ppliste_rdv = &(context->debut_liste_rdv);

  while ((*ppliste_rdv) && (CSTstrcmp (nom, (*ppliste_rdv)->nom)))
    {
      ppliste_rdv = &((*ppliste_rdv)->suivant);
    }
  if (*ppliste_rdv)
    {
      /* Suppresion du rendez-vous */
      prdv_suivant = (*ppliste_rdv)->suivant;
      p_libere_rdv (Context, ok, *ppliste_rdv);
      *ppliste_rdv = prdv_suivant;
    }
}

/******************************************************************************
*
*  NAME
*   Ferme les connexions des rdv en cours
*
******************************************************************************/
void RDVAclose DP ((Context)) BP
          PT_REF_CONTEXT Context EP
{
  PT_PRIVATE_CONTEXT_RDV context;
  PT_PRIVATE_LISTE_CLIENTS p_client;
  T_PRIVATE_LISTE_RDVA *rdv;

  context = (PT_PRIVATE_CONTEXT_RDV) CTXget (Context, RDVAGT_CTX);


  p_client = context->debut_liste_nouveaux_clients;

  rdv = (T_PRIVATE_LISTE_RDVA *) context->debut_liste_rdv;
  while (rdv != NULL)
    {
      p_client = rdv->clients;
      while (p_client)
      {
       if (p_client->client>0)
        {
          (void) TCPIPclose_stream (Context, p_client->client);
        }
       p_client = p_client->suivant;
      }
      rdv = rdv->suivant;
    }
}
