

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: scuagt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)scuagt $Revision: 1.18 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  SCUAGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.18 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Controle de l'acces a l'agent                                  *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* Inclusion des entetes necessaires a la partie publique du module */
#include "atwstr.h"

#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#ifndef display
#define display(a)
#endif

#include "ctxagt.h"
#include "envagt.h"

/* Inclusion de l'entete publique du module */
#include "scuagt.h"

/* Inclusion de l'entete privee du module */
#include "scuagtp.h"

#ifndef NO_SECURITY
/*******************************************************************************
 * NAME
 *      Creation d'un utilisateur a partir d'une ligne du fichier
 *      d'autorisation
 *
 ******************************************************************************/
static PT_LISTE_UTILISATEURS p_SCUcree_utilisateur DP ((ligne,
                                                        machine)) BP
          PT_CAR ligne SP
          PT_CAR machine EP
{
  T_BOOLEAN a_inserer = FALSE;
  T_TABCAR  auto_machine;
  T_TABCAR  auto_utilisateur;
  PT_CAR    pnom;
  PT_LISTE_UTILISATEURS putilisateurs = NULL;
  struct hostent *info_reseau_machine;

  /* Recuperation des champs de la ligne */
  if (CSTsscanf (ligne, "%s %s", auto_machine, auto_utilisateur) != 2)
    {
      /* Le fichier est mal formate -> fin */
      display(printf("atsagtd: authorization file format error\n");)
      return (NULL);
    }

  /*
   * Le fichier est bien formate...
   */
  if (CSTstrcmp (auto_machine, "+") == 0)
    {
      a_inserer = TRUE;
    }
  else
    {
      info_reseau_machine = gethostbyname (auto_machine);
      if (info_reseau_machine)
        {
          if ((CSTstrcmp (info_reseau_machine->h_name, machine) == 0) ||
              (CSTstrcmp (machine, "localhost") == 0) ||
              (CSTstrcmp (machine, "loopback") == 0) ||
              (CSTstrcmp (machine, "127.0.0.1") == 0))
            {
              a_inserer = TRUE;
            }
        }
    }

  if (a_inserer)
    {
      /* L'autorisation est a prendre en compte
       * Creation d'utilisateur pointe par le pointeur pointe par
       * pointeur_putilisateur
       */
      putilisateurs = (PT_LISTE_UTILISATEURS)CSTmalloc (sizeof (T_LISTE_UTILISATEURS));

      /* Definit si l'utilisateur est autorise ou non */
      if (auto_utilisateur[0] == '-')
        {
          putilisateurs->utilisateur.autorise = FALSE;
        }
      else
        {
          putilisateurs->utilisateur.autorise = TRUE;
        }

      /* Affecte le nom de l'utilisateur (s'il est defini)
       * Attention on fait l'hypothese qu'un login contient
       * plus de 2 lettres...
       */
      if ((auto_utilisateur[0] == '+')
          ||
          (auto_utilisateur[0] == '-')
        )
        {
          /* On saute l'indicateur */
          pnom = auto_utilisateur + 1;
        }
      else
        {
          /* Le nom d'utilisateur ne contient pas d'indicateur */
          pnom = auto_utilisateur;
        }
      if (*pnom != '\0')
        {
          putilisateurs->utilisateur.nom = atwstrClone (pnom);
        }
      else
        {
          putilisateurs->utilisateur.nom = NULL;
        }
      putilisateurs->suivant = NULL;
    }

  return (putilisateurs);
}

/*******************************************************************************
 * NAME
 *      Creation du module d'autorisation d'une machine
 *
 ******************************************************************************/
T_SECURITE SCUcree DP ((ContexteRef, machine, nom_fichier)) BP
          PT_REF_CONTEXT ContexteRef SP
          PT_CAR machine SP
          PT_CAR nom_fichier EP
{
  PT_LISTE_UTILISATEURS *pointeur_putilisateurs = NULL;
  PT_LISTE_UTILISATEURS premier_utilisateur = NULL;
  FILE     *fichier_autorisation;
  atwstr   *nom_fichier_droit = NULL;
  atwstr   *utilisateur_local = NULL;
  T_TABCAR  memoire_tampon;
  PT_CAR    p_memoire_tampon = NULL;
  struct passwd *definition_acces_courant;
  struct hostent *info_reseau_machine;
  char hostname[100]; /* a la hache */
static char errorDisplayed=0;
#ifdef UNIX
#define slash_string "/"
#define slash_char   '/'

  /*
   * Definit le nom du fichier d'autorisation ($HOME/.atsagtd)
   */

  definition_acces_courant = getpwuid (getuid ());
  if (definition_acces_courant)
    {
      nom_fichier_droit = atwstrClone (definition_acces_courant->pw_dir);
    }
  else
    {
      /* Echec de recuperation de la definition de l'utilisateur courant */
      return (NULL);
    }

#endif                          /* UNIX */

#ifdef WIN32
#define slash_string "\\"
#define slash_char   '\\'

  nom_fichier_droit = ENVgetenv (ContexteRef, "ATS_DIR"); /* use it if defined */
  if (nom_fichier_droit == NULL)
  {
	  nom_fichier_droit = ENVgetenv (ContexteRef, "TESTRTDIR");
      if (nom_fichier_droit == NULL)
    	return (NULL);
      nom_fichier_droit = atwstrAppend (nom_fichier_droit, "\\bin\\intel\\win32\\");
  }
  debug (if (!nom_fichier_droit) nom_fichier_droit = atwstrClone ("C:\\TEMP\\"));

#endif                          /* WIN32 */

  if (nom_fichier_droit[atwstrLength (nom_fichier_droit) - 1] != slash_char)
    {
      /* Ajoute le separateur dans nom de repertoire */
      nom_fichier_droit = atwstrAppend (nom_fichier_droit, slash_string);
    }
  nom_fichier_droit = atwstrAppend (nom_fichier_droit, nom_fichier);

  /*
   * Ouverture du fichier d'autorisation (en lecture)
   */

  fichier_autorisation = CSTfopen (nom_fichier_droit, "r");
  debug (CSTprintf ("ouverture %s %s\n", nom_fichier_droit,
                    fichier_autorisation ? "OK" : "KO"));

  /* Fait pointer pointeur_putilisateur sur premier_utilisateur qui lui
   * va pointer sur le premier utilisateur de la liste
   */
  pointeur_putilisateurs = &premier_utilisateur;

  /* Le pointeur est affecte NULL pour effectuer la lecture de la 1ere ligne */
  p_memoire_tampon = NULL;

  /* Parcours du fichier et construction  de la liste des autorisations */
  while (fichier_autorisation
         &&
         (!CSTfeof (fichier_autorisation)))
    {
      /* lecture d'une ligne dans le fichier des autorisations */
      do
        /*
         * L'operation est repetee si elle a ete interrompue par un signal
         */
        {
          errno = 0;
          p_memoire_tampon = fgets (memoire_tampon,
                                    sizeof (memoire_tampon),
                                    fichier_autorisation);
          debug (perror ("fgets"));
          debug (CSTprintf ("ligne %s\n", memoire_tampon));
        }
      while (!p_memoire_tampon && (errno == EINTR) &&
             !CSTfeof (fichier_autorisation));
      if (p_memoire_tampon)
        {
          /* Suppression des commentaires */
          p_memoire_tampon = CSTstrchr (memoire_tampon, '#');
          if (p_memoire_tampon)
            {
              *p_memoire_tampon = '\0';
            }
		  if((memoire_tampon[0] != '\0') && (memoire_tampon[0] != '\n'))
			{
			  /* Affecte le pointeur d'utilisateur pointe par pointeur_putilisateur */
			  debug (CSTprintf ("p_SCUcree_utilisateur %s %s\n",
								memoire_tampon, machine));
			  *pointeur_putilisateurs = p_SCUcree_utilisateur (memoire_tampon,
															   machine);

			  if (*pointeur_putilisateurs)
				{
				  /* Un nouvel utilisateur a ete cree...
				   * -> Positionne le pointeur de pointeur sur le pointeur du
				   *    prochain utilisateur.
				   */
				  pointeur_putilisateurs = &((*pointeur_putilisateurs)->suivant);
				}
			}
        }
    }
  if (fichier_autorisation)
    CSTfclose (fichier_autorisation);
    display(  else if(!errorDisplayed){ errorDisplayed=1; printf("atsagtd: authorization file %s not found. Use current machine:user\n",nom_fichier_droit);})

#if defined(UNIX) || defined(WIN32)
  /* Autorise l'utilisateur qui a lance l'agent sur la machine courante... */
  if (gethostname (hostname,100) == 0)
    {
      /* Recupere le nom complet de la machine */
      info_reseau_machine = gethostbyname (hostname);
      if (info_reseau_machine)
        {
          /* Construit la ligne :
           * -> "machine_local login_courant"
           */
          utilisateur_local = atwstrClone (info_reseau_machine->h_name);
          utilisateur_local = atwstrAppend (utilisateur_local, " ");
#ifdef UNIX
          utilisateur_local = atwstrAppend (utilisateur_local,
                                         definition_acces_courant->pw_name);
#else
        {
          char username[100];
          int size=99;
          GetUserName(username,&size);
          utilisateur_local = atwstrAppend (utilisateur_local,username);
        }
#endif

          /* Insere la nouvelle autorisation */
          *pointeur_putilisateurs = p_SCUcree_utilisateur (utilisateur_local,
                                                           machine);
          pointeur_putilisateurs = &((*pointeur_putilisateurs)->suivant);
        }
    }
#endif
  return ((T_SECURITE) premier_utilisateur);
}

/*******************************************************************************
 * NAME
 *      Creation du module d'autorisation d'une machine
 *
 ******************************************************************************/
void SCUdetruit DP ((ContexteRef, psecurite)) BP
          PT_REF_CONTEXT ContexteRef SP
          PT_SECURITE psecurite EP
{
  PT_LISTE_UTILISATEURS putilisateurs = NULL;
  PT_LISTE_UTILISATEURS putilisateurs_suivant = NULL;

  if (*psecurite)
    {
      putilisateurs = (PT_LISTE_UTILISATEURS) * psecurite;
      /* Liberation de le memoire utilisee par le liste des utilisateurs */
      while (putilisateurs)
        {
          putilisateurs_suivant = putilisateurs->suivant;
          M_CSTfree (putilisateurs);
          putilisateurs = putilisateurs_suivant;
        }
      *psecurite = NULL;
    }
}

/*******************************************************************************
 * NAME
 *      Determine l'autorisation d'un utilisateur
 *
 ******************************************************************************/
T_BOOLEAN SCUestAutorise DP ((ContexteRef, psecurite, nom)) BP
          PT_REF_CONTEXT ContexteRef SP
          PT_SECURITE psecurite SP
          PT_CAR nom EP
{
  PT_LISTE_UTILISATEURS putilisateurs = (PT_LISTE_UTILISATEURS)0;
  T_BOOLEAN autorisation = FALSE;
  debug (CSTprintf ("~~ %s estAutorise ?\n", nom));

  if (*psecurite)
    {
      putilisateurs = (PT_LISTE_UTILISATEURS)(*psecurite);
      /* Parcours de la liste des utilisateurs et recherche du nom...
         Si le nom est trouve on retourne l'autorisation correspondante
         Sinon si une indication generale est trouvee on la retourne
         Sinon on retourne un refus d'autorisation...
       */
      while (putilisateurs)
        {
          if (putilisateurs->utilisateur.nom)
            {
              /* Le nom de l'utilisateur est defini... */
              if (CSTstrcmp (nom, putilisateurs->utilisateur.nom) == 0)
                {
                  /* l'utilisateur recherche est trouve... */
                  autorisation = putilisateurs->utilisateur.autorise;

                  /*
                   * Arret de la boucle !!!
                   */

                  putilisateurs = NULL;
                }
              else
                {
                  /* Suite du parcours de la liste */
                  putilisateurs = putilisateurs->suivant;
                }
            }
          else
            {
              /* Le nom n'est pas indique -> l'autorisation concerne tout le
               * monde.
               */
              autorisation = putilisateurs->utilisateur.autorise;
              /* Suite du parcours de la liste */
              putilisateurs = putilisateurs->suivant;
            }
        }

      debug (CSTprintf ("~~ %s %s autorise !\n",
                        nom,
                        autorisation ? "est" : "n'est pas");
        )

        return (autorisation);
    }
  else
    {
      return (FALSE);
    }
}
#endif    /* ifndef NO_SECURITY (ex- UNIX  ou WIN32) */
