

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: tcpip.c                                              *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)tcpip $Revision: 1.26 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  TCPIP                                                    *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.26 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Gestion de la communication avec TCPIP                         *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/


#if defined (RS6000) || defined (LINUX)
#define SOCKLEN_T socklen_t
#else
#define SOCKLEN_T int
#endif

#ifdef LINUX
#include <unistd.h>
#include <sys/time.h>
#endif


#ifdef RS6000
#include <strings.h>
#endif


#ifdef WIN32
#include <winsock.h>
#endif

#include <assert.h>

/* Inclusion des entetes standarts SystemTest */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#include "atwstr.h"

/* Inclusion de l'entete du module */
#include "ctxagt.h"
#include "tcpip.h"


/* Definit les types internes definissant les adresses des prises (socket) */
typedef struct sockaddr_in T_SOCKADDR_IN,
         *PT_SOCKADDR_IN;
typedef struct sockaddr T_SOCKADDR,
         *PT_SOCKADDR;

#ifndef AGT_CUSTOM
/*
#if defined(UNIX) || defined(WIN32)
*/
typedef struct hostent T_HOSTENT,
         *PT_HOSTENT;
typedef struct servent T_SERVENT,
         *PT_SERVENT;
#endif   /* AGT_CUSTOM */

/* Determination du type de communication a utiliser */
typedef enum
  {
    TCPIP_STREAM = 0, TCPIP_DGRAM = 1
  }
T_TYPE_SOCKET;

/* Definition des types utiliser pour le temps */

typedef struct timeval S_TIMEVAL;
typedef struct timeval *SP_TIMEVAL;


typedef struct
  {
    /* Constante et variable indiquant si la librairie a ete initialisee */
    T_BOOLEAN notinit;
    /* Masque utilise pour designer les sockets a surveiller avec select */
#ifdef LINUX
    fd_set    masque_lecture;
#else
    struct fd_set masque_lecture;
#endif
    long max_fd;
  }
T_PRIVATE_CONTEXTE_TCPIP, *PT_PRIVATE_CONTEXTE_TCPIP;

/* #ifndef AGT_CUSTOM */
#ifdef UNIX
/* Variable utilisee par la fonction gethostbyname pour les erreurs */
#ifndef AIX43
extern int h_errno;
#endif
#endif

/* Constanest specifiant les prototypes de communication */
#define TCPIP_PROTO_SERVICE_STREAM      "tcp"
#define TCPIP_PROTO_SERVICE_DGRAM       "udp"

/* Taille de la liste d'attente des connexions d'une socket */
#define TCPIP_LISTENSIZE        SOMAXCONN

#ifdef UNIX
#define POSIX_SIGNAL /* on UNIX, POSIX signals are available */
#endif

/* 
 * ------ Fonctions/procedures communes a tous les modes de communication -----
 */

/******************************************************************************
*
*  NAME
*   Initialisation de la librairie.
*
*  ALGORITHME
*       Initialise la librairie de communication pour Windows
*       Si l'initialisation s'est bien effectuee alors g_tcpip_int = 1

******************************************************************************/
T_RETURN TCPIPinit DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_TCPIP pcontexte;

#ifdef WIN32
  WSADATA   WSAData;
  int       err;
  int       optionValue = SO_SYNCHRONOUS_NONALERT;

#endif                          /* WIN32 */

  pcontexte = (PT_PRIVATE_CONTEXTE_TCPIP)CSTmalloc (sizeof (T_PRIVATE_CONTEXTE_TCPIP));

  CTXset (refContext, pcontexte, TCPIP_CTX);

  /* Initialisation des masques des abonnes a la surveillance */
  pcontexte->notinit = FALSE;
  FD_ZERO (&(pcontexte->masque_lecture));
  pcontexte->max_fd=0;

#ifdef WIN32
  /* 
   * Pour Windows NT/95 il faut initialiser les couches reseau
   */

     if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
       {
	 pcontexte->notinit = TRUE;
	 return (SYSTEM_ERROR);
       }

     /* SLY: Correction DR. 811 */ 

#ifndef SPV_NT4	
     err = setsockopt (INVALID_SOCKET,
     SOL_SOCKET,
     SO_OPENTYPE,
     (char *) &optionValue,
     sizeof (optionValue));

     if (err != NO_ERROR)
     {
     pcontexte->notinit = TRUE;
     return (SYSTEM_ERROR);
     }
#endif


#endif                           /* WIN32 */
#ifdef UNIX 
  signal (SIGPIPE, SIG_IGN);  /* ignore signal */
#endif  /* UNIX */
  return (SYSTEM_OK);
}

/* ne semble pas utilise */
/*
#ifdef VXWORKS
T_RETURN TCPIPclone DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_TCPIP pcontexte;

  pcontexte = CSTmalloc (sizeof (T_PRIVATE_CONTEXTE_TCPIP));

  pcontexte->notinit = TRUE;
  FD_ZERO (&(pcontexte->masque_lecture));
  pcontexte->max_fd=0;

  CTXset (refContext, pcontexte, TCPIP_CTX);
}
#endif  
*/
/******************************************************************************
*
*  NAME
*       Fermeture de la librairie
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN TCPIPfin DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      /* La librairie n'est pas initialisee !! */
      return (SYSTEM_ERROR);
    }
  else
    {
#ifdef WIN32
      if (WSACleanup () == 0)
        return (SYSTEM_OK);
      else
        return (SYSTEM_ERROR);
#endif                          /* WIN32 */
      contexte->notinit = TRUE;
      return (SYSTEM_OK);
    }
}

/*******************************************************************************
*
*  NAME
*    Definit une adresse d'une prise de communication
*
*  ALGORITHM
*
*****************************************************************************/
static T_RETURN TCPIPset_addr_in DP ((psin,
                                      hostname,
                                      service,
                                      portnum,
                                      type))    BP
          PT_SOCKADDR_IN psin SP
          PT_CAR hostname SP
          PT_CAR service SP
     	  int portnum SP
          T_TYPE_SOCKET type EP
     {
#ifndef AGT_CUSTOM
/*
#if defined (UNIX) || defined(WIN32)
*/
       /* Structure utilisee pour decrire une machine */
       PT_HOSTENT phe = NULL;

       /* Structure utilisee pour decrire du service */
       PT_SERVENT sserv = NULL;
#endif     /* AGT_CUSTOM */
#ifdef WIN32
#define EHOSTUNREACH WSAEHOSTUNREACH
#define EAGAIN WSAEADDRNOTAVAIL
#define ENXIO WSAEADDRNOTAVAIL
#endif

       /* Initialise a 0 tous les champs de la structure psin */
       if (psin == NULL)
         {
           return (SYSTEM_ERROR);
         }
       CSTmemset ((char *)psin, 0, sizeof (T_SOCKADDR_IN));

       /* La librairie ne traite que les sockets inet */
       psin->sin_family = AF_INET;

       /* Definit l'adresse de la machine */
#ifndef AGT_CUSTOM
       if (hostname == NULL)
         {
#endif                          /* UNIX ou WIN32 */
           /* L'adresse doit etre definei sur la machine locale */
           psin->sin_addr.s_addr = htonl (INADDR_ANY);
#ifndef AGT_CUSTOM
         }
       else
         {
           /* Recherche l'adresse de la machine */
           phe = gethostbyname (hostname);
           if (phe == NULL)
             {
               /* 
                * Convertit la variable h_ernno non-standard en errno 
                */
               switch (h_errno)
                 {
                 case HOST_NOT_FOUND:
                   errno = EHOSTUNREACH;
                   break;
                 case TRY_AGAIN:
                   errno = EAGAIN;
                   break;
                 case NO_RECOVERY:
                   errno = EAGAIN;
                   break;
                 case NO_ADDRESS:
                   errno = ENXIO;
                   break;
                 default:
                   errno = 0;
                   break;
                 }
               return (SYSTEM_ERROR);
             }
           /* Copie l'adresse trouvee de la machine */
           memcpy ((void *) &(psin->sin_addr),
                   (void *) phe->h_addr,
                   phe->h_length);
         }

       /* Affecte le numero de port de la communication */
       if (service)
         {
           /* defnition du prototype de la communication */
           PT_CAR    proto = NULL;

           /* Recherche le numero de port correspondant au service */
           if (type == TCPIP_STREAM)
             {
               proto = TCPIP_PROTO_SERVICE_STREAM;
             }
           else
             {
               proto = TCPIP_PROTO_SERVICE_DGRAM;
             }
           if ((sserv = getservbyname (service, proto)) == NULL)
             {
               return (SYSTEM_ERROR);
             }
           portnum = sserv->s_port;

         }

#endif                          /* UNIX ou WIN32 */
       /* Affecte le numero de port */
       psin->sin_port = htons (portnum);

       /* L'adresse du port de communication est determinee */
       return (SYSTEM_OK);
     }

/*******************************************************************************
*
*  NAME
*    Creation d'une prise de communication
*
*  ALGORITHM
*    Cree une socket de la famille AF_INET et de type "type"
*    Definit les options de la socket
*    Retourne la descripteur de fichier correspondant
*
*****************************************************************************/
static T_SOCKET TCPIPcreate_socket DP ((type)) BP
          T_TYPE_SOCKET type EP
{

  /* Protocole de la communication */
  T_ENTIER  typ_proto;

  /* Descripteur de fichier de la prise de communication */
  T_SOCKET  loc_sck;

  /* Variable utilisee pour definir les options de la socket */
  T_ENTIER  on = 1;

  loc_sck.masque = -1;

  /* Determination du protocole de communication */
  if (type == TCPIP_STREAM)
    {
      typ_proto = SOCK_STREAM;
    }
  else
    {
      typ_proto = SOCK_DGRAM;
    }
  /* Creation de la prise de communication */
  if ((loc_sck.masque = socket (AF_INET, typ_proto, 0)) == -1)
    {
      return (loc_sck);
    }

  /*
   * Specifie que l'eventuelle adresse de la socket peut etre definie parmis
   * les adresses deja utilisees et qui ont ete liberees.
   * Ceci est indispensable sous Windows                                        
   */
  if (setsockopt (loc_sck.masque,
                  SOL_SOCKET,
                  SO_REUSEADDR,
                  (char *) &on,
                  sizeof (T_ENTIER))
      == -1)
    {
      loc_sck.masque = -1;
      return (loc_sck);
    }

  /* retourne le descripteur de fichier correspondant a la socket creee */
  return (loc_sck);
}

/*******************************************************************************
*
*  NAME
*    Nommage d'une socket 
*
*  ALGORITHM
*    Definit l'adresse de communication avec :
*               + le Nom de la machine ;
*               + le numero de port.
*    Lie la socket a cette adresse.
*    Dimensionne le buffer de reception des demandes de connexions
*
*****************************************************************************/
static T_RETURN TCPIPbind DP ((sock, psin)) BP
          T_SOCKET sock SP
          PT_SOCKADDR_IN psin EP
{
  SOCKLEN_T longueur = sizeof (T_SOCKADDR_IN);

  /* Lie la socket avec l'adresse */
  if (bind (sock.masque, (PT_SOCKADDR) psin, longueur) == -1)
    {
      return (SYSTEM_ERROR);
    }
  else if (psin)
    {
      /* Recuperation de l'adresse effective du port de communication */
      getsockname (sock.masque, (PT_SOCKADDR) psin, (SOCKLEN_T *)&longueur);
    }

  /* Definit la taille de la liste d'attente */
  if (listen (sock.masque, TCPIP_LISTENSIZE) == -1)
    {
      return (SYSTEM_ERROR);
    }
  return (SYSTEM_OK);
}

/*******************************************************************************
*
*  NAME
*    Creation d'une socket nommee
*
*  ALGORITHM
*    Cree une socket
*    Definit l'adresse a lier a la socket
*    Lie l'adresse avec la socket
*    Retourne le descripteur de fichier de la socket
*
*****************************************************************************/
static T_SOCKET TCPIPcreate_named_socket DP ((hostname,
                                              service,
                                              port,
                                              type))    BP
          PT_CAR hostname SP
          PT_CAR service SP
          PT_ENTIER port SP
          T_TYPE_SOCKET type EP
{
  T_SOCKET  prise;              /* Descripteur de fichier de la socket */
  T_SOCKADDR_IN sin;            /* Adresse du port de communication */

  /* Cree la prise de communication */
  if ((prise.masque = TCPIPcreate_socket (type).masque) != SYSTEM_ERROR)
    {
      /* Definit l'adresse du port de communication puis lie celle-ci avec
       * la socket creee
       */
      if (TCPIPset_addr_in (&sin, hostname, service, *port, type)
          ==
          SYSTEM_ERROR)
        {
          /* Echec de definition de l'adresse */
          prise.masque = SYSTEM_ERROR;
        }
      else if (TCPIPbind (prise, &sin) == SYSTEM_ERROR)
        {                       /* Nommage de la prise a echoue */
          prise.masque = SYSTEM_ERROR;
        }
      else
        {
          *port = ntohs (sin.sin_port);
        }
    }
  return (prise);
}

/*******************************************************************************
*
*  NAME
*    Recupere le port de la socket
*
*  ALGORITHM
*      N/A
*
*****************************************************************************/
static T_ENTIER TCPIPport DP ((sock)) BP
          T_SOCKET sock EP
{
  T_SOCKADDR_IN psin;
  SOCKLEN_T l_psin = sizeof (psin);

  /* Recupere l'adresse complete de la socket */
  if (getsockname (sock.masque, (PT_SOCKADDR) & psin, (SOCKLEN_T *)&l_psin) == 0)
    {
      /* retourne le numero de port */
      return (ntohs (psin.sin_port));
    }
  else
    {
      /* Retourne -1 pour marquer l'erreur */
      return (-1);
    }
}

/*******************************************************************************
*
*  NAME
*    Abonnement d'une socket a la surveillance
*
*  ALGORITHM
*      N/A
*
*****************************************************************************/
static T_RETURN TCPIPabonne_socket DP ((contexte,
                                        sock))    BP
          PT_PRIVATE_CONTEXTE_TCPIP contexte SP
          T_SOCKET sock EP
{
  long tmp = (long)sock.masque + 1L;
  atsdebug (assert (contexte != NULL));

  /* Marque le descripteur de la socket pour la commande select */
  FD_SET (sock.masque, &(contexte->masque_lecture));
  if (tmp > (contexte->max_fd))
  {
   contexte->max_fd=tmp;
  }

  return (SYSTEM_OK);
}

/*******************************************************************************
*
*  NAME
*    Desabonnement d'une socket a la surveillance
*
*  ALGORITHM
*      N/A
*
*****************************************************************************/
static T_RETURN TCPIPdesabonne_socket DP ((contexte,
                                           sock))    BP
          PT_PRIVATE_CONTEXTE_TCPIP contexte SP
          T_SOCKET sock EP
{
  atsdebug (assert (contexte != NULL));

  /* Marque le descripteur de la socket pour la commande select */
  FD_CLR (sock.masque, &(contexte->masque_lecture));

  return (SYSTEM_OK);
}

/*******************************************************************************
*
*  NAME
*    Attend la reception d'un message (de connexion, deconnexion ou de donnees)
*    sur les sockets etant abonnees.
*
*  ALGORITHM
*       Initialise la duree d'attente
*       Initialise le masque des fd a surveiller
*       appel la fonction systeme select (en repetant si un signal est recu
*       construit le tableau des sockets a traiter...   
*
*****************************************************************************/
T_ENTIER TCPIPsurveille_sockets DP ((refContext,
                                     sockets_a_lires, duree_ms)) BP
          PT_REF_CONTEXT refContext SP
          PT_SOCKET * sockets_a_lires SP
          T_ENTIER duree_ms EP
{
  /* Masque utilise par select pour definir les socket a surveiller */
#ifdef LINUX
  fd_set    masque_lecture;
#else
  struct fd_set masque_lecture;
#endif

  /* Variables utilisees pour la boucle de recuperation des sockets */
  T_ENTIER  index;
  T_ENTIER  indexL = 0;

  /* Variable utilisee pour optimiser la rechercher des sockets */
  T_ENTIER  nb_enregistrees = 0;

  /* Variable contenant le nombre de sockets devant etre traitees */
  T_ENTIER  nb_sockets_receptrices;

  /* Liste des sockets a lires */
  PT_SOCKET socketsALires;

  /* Gestion de la duree d'attente */
  S_TIMEVAL duree;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  /* Verifie que la librairie est bien initilisee */
  if ((contexte == NULL) || contexte->notinit || !sockets_a_lires
      || *sockets_a_lires)
    {
      /* Le module n'est pas initialise OU
       * le pointeur de parametre n'est pas defini
       * Le table de pointeur n'est pas vide !!!
       */
      return (SYSTEM_ERROR);
    }

  /* Definition du temps d'attente :
   *    duree(en mili seconde) = duee.tv_sec * 1000 + duree.tc_usec*10
   */
  duree.tv_sec = duree_ms / 1000;	/* Nb secondes */
  duree.tv_usec = (duree_ms % 1000) * 10;	/* Nb de miliseconde restant * 10 */

  /* 
   * Met a jour les masques de surveillance avec les masques de 
   * recherche
   */
#ifdef LINUX
  memcpy ((void *) &masque_lecture,
          (void *) &(contexte->masque_lecture),
          (size_t) sizeof (fd_set));
#else
  memcpy ((void *) &masque_lecture,
          (void *) &(contexte->masque_lecture),
          (size_t) sizeof (struct fd_set));

#endif

#ifdef POSIX_SIGNAL /* || defined(VXWORKS) */
  do
    /* Attend un evenement sur les sockets surveillees
     * en repetant l'operation si on est interrompu par un signal
     */
    {
      errno = 0;
#endif      /* POSIX_SIGNAL */
      nb_sockets_receptrices = select (contexte->max_fd,
#ifdef HP_PAS_HP
                                       (int *)
#endif
                                       &masque_lecture,
                                       0,
                                       0,
                                       &duree);
#ifdef POSIX_SIGNAL /* || defined(VXWORKS) */
    }
  while ((nb_sockets_receptrices == -1) && (errno == EINTR));
#endif      /* POSIX_SIGNAL */

  /* Verifie si la fonction s'est correctement terminee */
  switch (nb_sockets_receptrices)
    {
    case -1:
/* ChT DEBUG _USE_BIG_FDS 
      printf("FATAL ERROR while calling select function\n");
      printf("Using _USE_BIG_FDS flag, and FD_SETSIZE=%d\n",FD_SETSIZE);
      printf("And max fd=%d\n",contexte->max_fd);
      printf("errno=%d\n",errno);
      exit(-1);
 ChT DEBUG _USE_BIG_FDS */
      return (SYSTEM_ERROR);
      break;
    case 0:
      socketsALires = NULL;
      break;
    default:
      socketsALires = (PT_SOCKET)CSTmalloc (nb_sockets_receptrices * sizeof (T_SOCKET));
      break;
    }

  /* 
   * Recuperation des numeros de socket a traiter et construction des
   * tableaux resultats socket_a_lire et socket_pour_ecrire 
   */
  for (index = 0; nb_enregistrees < nb_sockets_receptrices; index++)
    {
      /* 
       * Ajoute la valeur index dans le tableau socket_a_lire si le 
       * descripteur correspondant a ete detecte en lecture par select 
       */
      if (FD_ISSET (index, &masque_lecture))
        {
          socketsALires[indexL++].masque = index;
          nb_enregistrees++;
        }
    }

  /* Retourne le tableau des sockets... */
  *sockets_a_lires = socketsALires;
  return (nb_sockets_receptrices);
}

/*******************************************************************************
*
*  NAME
*    Fermeture d'une socket 
*
*  ALGORITHM
*    Pour windows appel de la fonction closesocket
*    Pour UNIX appel de la fonction close
*
*****************************************************************************/
static T_RETURN TCPIPclose_socket DP ((contexte,
                                       sock))    BP
          PT_PRIVATE_CONTEXTE_TCPIP contexte SP
          T_SOCKET sock EP
{
  atsdebug (assert (contexte != NULL));

  /* Desabonne la socket de la surveillance */
  (void) TCPIPdesabonne_socket (contexte, sock);

#ifdef WIN32
  return ((T_RETURN)closesocket (sock.masque));
#else                           /* WIN32 */
  return ((T_RETURN)close (sock.masque));
#endif                          /* WIN32 */
}

/* 
 * Fonctions / procedures pour la communication en mode connecte
 */

/*******************************************************************************
*
*  NAME
*    Creation d'une socket TCP
*
*
*  ALGORITHM
*    TCPIPcreate_socket(TCPIP_STREAM)
*
*****************************************************************************/
T_SOCKET_STREAM TCPIPsocket_stream DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  return (TCPIPcreate_socket (TCPIP_STREAM).stream);
}

/*******************************************************************************
*
*  NAME
*    Creation d'une socket TCP permettant d'attendre des connexions.
*       -> Cette socket est nommee
*
*
*  ALGORITHM
*    TCPIPcreate_socket_named(TCPIP_STREAM)
*
*****************************************************************************/
T_SOCKET_STREAM TCPIPsocket_named_stream DP ((refContext,
                                              machine,
                                              service,
                                              port))    BP
          PT_REF_CONTEXT refContext SP
          PT_CAR machine SP
          PT_CAR service SP
          PT_ENTIER port EP
{
  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  return (TCPIPcreate_named_socket (machine,
                                    service,
                                    port,
                                    TCPIP_STREAM)
          .stream);
}

/*******************************************************************************
*
*  NAME
*    Recupere le port de la socket
*
*  ALGORITHM
*      N/A
*
*****************************************************************************/
T_ENTIER TCPIPport_stream DP ((refContext,
                               sock))    BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }
  prise.stream = sock;
  return (TCPIPport (prise));
}

/******************************************************************************
*
*  NAME
*    AccepteAbonne une socket TCP a la surveillance
*
*  ALGORITHM
*
*****************************************************************************/
T_RETURN TCPIPabonne_stream DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.stream = sock;
  return (TCPIPabonne_socket (contexte, prise));
}

/*******************************************************************************
*
*  NAME
*    Desabonne une socket TCP a la surveillance
*
*  ALGORITHM
*
*****************************************************************************/
T_RETURN TCPIPdesabonne_stream DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.stream = sock;
  return (TCPIPdesabonne_socket (contexte, prise));
}

/*******************************************************************************
*
*  NAME
*    Accepte une connexion
*
*  ALGORITHM
*    *sock_client = accept(fd, adr, adr_lg)
*    *port_client = htons(adr->sin_port);
*    
*
*****************************************************************************/
T_RETURN TCPIPaccept_stream DP ((refContext,
                                 sock_serveur,
                                 sock_client,
                                 machine,
                                 port_client)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock_serveur SP
          PT_SOCKET_STREAM sock_client SP
          PT_CAR machine SP
          PT_ENTIER port_client EP
{
  /* Adresse de la nouvelle prise de communication */
  T_SOCKADDR_IN adr;
  SOCKLEN_T adr_lg = sizeof (adr);
  PT_PRIVATE_CONTEXTE_TCPIP contexte;

#ifndef AGT_CUSTOM
  PT_HOSTENT phost;
#endif  /* AGT_CUSTOM */

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  do
    /* Attend et accepte l'arrivee d'une demande de connexion 
     * en repetant l'operation si celle-ci a ete interrompue 
     * par un signal.
     */
    {
      errno = 0;
      *sock_client = accept (sock_serveur, (PT_SOCKADDR) & adr, (SOCKLEN_T *)&adr_lg);
    }
  while ((*sock_client == -1) && (errno == EINTR));

  /* Verifie que l'operation s'est bien realisee */
  if (*sock_client == -1)
    {
      return (SYSTEM_ERROR);
    }
  else
    {
      if (machine)
        {
          /* Le client souhaite avoir le nom de la machine */

#ifndef AGT_CUSTOM
          phost = gethostbyaddr ((char *) &adr.sin_addr,
                                 sizeof (adr.sin_addr),
                                 AF_INET);
          if (phost)
            {
              CSTstrcpy (machine, phost->h_name);
            }
#endif   /* AGT_CUSTOM */

        }
      if (port_client)
        {
          *port_client = htons (adr.sin_port);
        }

      return (SYSTEM_OK);
    }
}

/*******************************************************************************
*
*  NAME
*    Essai de se connecter sur un hote
*
*  ALGORITHM
*    definit l' adresse de l'hote hote
*    utilise l'instruction systeme connect
*
*****************************************************************************/
T_RETURN TCPIPconnect_stream DP ((refContext, sock, machine, service, port)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock SP
          PT_CAR machine SP
          PT_CAR service SP
          T_ENTIER port EP
{
  /* Adresse de la machine hote */
  T_SOCKADDR_IN adr;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  /* Definit l'adresse du port de communication */
  if (TCPIPset_addr_in (&adr, machine, service, port, TCPIP_STREAM)
      == SYSTEM_ERROR)
    {
      return (SYSTEM_ERROR);
    }

  /* Etablit une connexion */
  return ((T_RETURN)connect (sock, (PT_SOCKADDR) & adr, sizeof (adr)));
}

/******************************************************************************
*
*  NAME
*    Recupere les informations d'une socket
*
*  ALGORITHM
*
*****************************************************************************/
atwstr   *TCPIPnomMachineCliente_stream DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock EP

{
  T_SOCKADDR_IN addr;
  SOCKLEN_T  addrlen = sizeof (T_SOCKADDR_IN);

#ifndef AGT_CUSTOM
  PT_HOSTENT phost = NULL;
#endif      /* AGT_CUSTOM */

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (NULL);
    }

  if (getpeername (sock, (PT_SOCKADDR) & addr, (SOCKLEN_T *)&addrlen) == 0)
    {
      /* Le client souhaite avoir le nom de la machine */

#ifndef AGT_CUSTOM
      phost = gethostbyaddr ((char *) &(addr.sin_addr),
                             sizeof (addr.sin_addr),
                             AF_INET);
      if (phost)
        {
          return (atwstrClone (phost->h_name));
        }
      else
        {
          return (NULL);
        }
#endif    /* AGT_CUSTOM */

    }

  return (NULL);
}

/*******************************************************************************
*
*  NAME
*    Envoi de donnees sur une socket en mode connecte
*
*  ALGORITHM
*    TCPIPcreate_socket(TCPIP_DGRAM)
*
*****************************************************************************/
T_RETURN TCPIPsend_stream DP ((refContext, sock, buf, buffer_lg)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock SP
     void *buf SP
               PT_ENTIER buffer_lg EP
     {
char* buffer = (char*)buf;
       PT_PRIVATE_CONTEXTE_TCPIP contexte;

                 contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

       if        ((contexte == NULL) || contexte->notinit)
         {
           return (SYSTEM_ERROR);
         }

       do
         /* Envoie le message et repete l'operation si une interruption est 
          * recue 
          */
         {
           errno = 0;
           *buffer_lg = send (sock, buffer, *buffer_lg, 0);
         }
       while ((*buffer_lg == -1) && (errno == EINTR));

       atsdebug (CSTprintf ("--> Envoie de %d octets\n", *buffer_lg);)

         if (*buffer_lg == -1)
         {
           return (SYSTEM_ERROR);
         }
       else
         {
           return (SYSTEM_OK);
         }
     }

/*******************************************************************************
*
*  NAME
*    Reception d'un message binaire sur une socket TCP
*
*  ALGORITHM
*
*****************************************************************************/
T_ENTIER TCPIPrecv_buffer_stream DP ((refContext,
                                      sock,
                                      buf,
                                      taille_buffer,
                                      est_deconnecte)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock SP
     void *buf SP
               T_ENTIER taille_buffer SP
               T_BOOLEAN * est_deconnecte EP
     {
char* buffer = (char*)buf;
       T_ENTIER  nb_octets_recus;

       PT_PRIVATE_CONTEXTE_TCPIP contexte;

                 contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

       if        ((contexte == NULL) || contexte->notinit)
         {
           return (SYSTEM_ERROR);
         }

       do
         /* Lit le message et repete l'operation si une interruption est 
          * recue 
          */
         {
           errno = 0;
           nb_octets_recus = recv (sock,
                                   (char *) buffer,
                                   taille_buffer,
                                   0);
           atsdebug (CSTprintf ("--< Reception de %d octets\n", nb_octets_recus);)
         }
       while ((nb_octets_recus == -1) && (errno == EINTR));

       switch (nb_octets_recus)
         {
         case -1:
           /* Erreur lors de l'appel systeme recv */
         case 0:
           /* Perte de connexion */
           *est_deconnecte = TRUE;
           break;
         default:
           break;
         }
       return (nb_octets_recus);
     }

/*******************************************************************************
*
*  NAME
*    Reception d'un message sur une socket TCP
*
*  ALGORITHM
*
*****************************************************************************/
T_ENTIER TCPIPrecv_texte_stream DP ((refContext,
                                     sock,
                                     buffer,
                                     est_deconnecte)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock SP
          atwstr ** buffer SP
          T_BOOLEAN * est_deconnecte EP
{
  T_TABCAR  memoire_tampon;
  T_ENTIER  longueur_tampon = sizeof (memoire_tampon) - 1;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  do
    /* Lit le message et repete l'operation si une interruption est 
     * recue 
     */
    {
      errno = 0;
      longueur_tampon = recv (sock,
                              (char *) memoire_tampon,
                              longueur_tampon,
                              0);
      atsdebug (CSTprintf ("--< Reception (TEXTE) de %d octets\n>>\t%s\n",
                        longueur_tampon,
                        memoire_tampon));
    }
  while ((longueur_tampon == -1) && (errno == EINTR));

  if (longueur_tampon == -1)
    {
      /* Erreur lors de l'appel systeme recv */
      atsdebug (perror ("recv");)

        * est_deconnecte = TRUE;
    }
  else
    {
      /* Reception de donnees */
      if (longueur_tampon)
        {
          /* On marque la fin du tampon */
          memoire_tampon[longueur_tampon] = '\0';

          /* Insertion du message dans le buffer du client */
          if (*buffer)
            {
              /* Celui-ci existe deja --> On insere a la suite */
              *buffer = atwstrAppend (*buffer, memoire_tampon);
            }
          else
            {
              /* Il faut creer le buffer */
              *buffer = atwstrClone (memoire_tampon);
            }
          *est_deconnecte = FALSE;
        }
      else
        {
          /* Lorsque recv()==0 alors la connexion est perdue */
          *est_deconnecte = TRUE;
        }
    }
  return (longueur_tampon);
}

/*****************************************************************************
*
*  NAME
*    Fermeture d'une socket TCP
*
*  ALGORITHM
*    TCPIPclose_socket(sock)
*
******************************************************************************/
T_RETURN TCPIPclose_stream DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_STREAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.stream = sock;
  return (TCPIPclose_socket (contexte, prise));
}

/* 
 * ------ Fonctions/procedures pour la communication en mode non connecte -----
 */

/******************************************************************************
*
*  NAME
*    Creation d'une socket UDP
*
*
*  ALGORITHM
*    TCPIPcreate_socket(TCPIP_DGRAM)
*
*****************************************************************************/
T_SOCKET_DGRAM TCPIPsocket_dgram DP ((refContext)) BP
          PT_REF_CONTEXT refContext EP
{
  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  return (TCPIPcreate_socket (TCPIP_DGRAM).dgram);
}

/******************************************************************************
*
*  NAME
*    Creation d'une socket UDP permettant de recevoir des donnees
*       -> Cette socket est nommee
*
*
*  ALGORITHM
*    TCPIPcreate_socket_named(TCPIP_STREAM)
*
*****************************************************************************/
T_SOCKET_DGRAM TCPIPsocket_named_dgram DP ((refContext, machine, service, port)) BP
          PT_REF_CONTEXT refContext SP
          PT_CAR machine SP
          PT_CAR service SP
          PT_ENTIER port EP
{

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  return (TCPIPcreate_named_socket (machine,
                                    service,
                                    port,
                                    TCPIP_DGRAM)
          .dgram);
}

/*******************************************************************************
*
*  NAME
*    Recupere le port de la socket
*
*  ALGORITHM
*      N/A
*
*****************************************************************************/
T_ENTIER TCPIPport_dgram DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.dgram = sock;
  return (TCPIPport (prise));
}

/*******************************************************************************
*
*  NAME
*    AccepteAbonne une socket UDP a la surveillance
*
*  ALGORITHM
*
*****************************************************************************/
T_RETURN TCPIPabonne_dgram DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.dgram = sock;
  return (TCPIPabonne_socket (contexte, prise));
}

/*******************************************************************************
*
*  NAME
*    Desabonne une socket UDP a la surveillance
*
*  ALGORITHM
*
*****************************************************************************/
T_RETURN TCPIPdesabonne_dgram DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.dgram = sock;
  return (TCPIPdesabonne_socket (contexte, prise));
}

/*******************************************************************************
*
*  NAME
*    Envoi d'un message sur une socket UDP 
*    en specifiant la prise de communication de 
*    l'emetteur
*
*  ALGORITHM
*    TCPIPsend_socket(sock)
*
*****************************************************************************/

T_RETURN TCPIPsendto_dgram DP ((refContext,
                                sock,
                                buffer,
                                buffer_lg,
                                machine_dest,
                                port_dest)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock SP
     void     *buffer SP
               PT_ENTIER buffer_lg SP
               PT_CAR machine_dest SP
               T_ENTIER port_dest EP
     {
       /* Adresse de la machine hote */
       T_SOCKADDR_IN adr;

       PT_PRIVATE_CONTEXTE_TCPIP contexte;

                 contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

       if        ((contexte == NULL) || contexte->notinit)
         {
           return (SYSTEM_ERROR);
         }

       /* Defini l'adresse du port de communication */
       if        (TCPIPset_addr_in (&adr,
                                    machine_dest,
                                              (PT_CAR) NULL,
                                    port_dest,
                                    TCPIP_STREAM)
                  == SYSTEM_ERROR)
         {
           return (SYSTEM_ERROR);
         }

       do
         /* Envoie le message et repete l'operation si une interruption est 
          * recue 
          */
         {
           errno = 0;
           *buffer_lg = sendto (sock,
                                (char *) buffer,
                                *buffer_lg,
                                0,
                                (PT_SOCKADDR) & adr,
                                sizeof (adr));
         }
       while ((*buffer_lg == -1) && (errno == EINTR));
       if (*buffer_lg == -1)
         {
           return (SYSTEM_ERROR);
         }
       else
         {
           return (SYSTEM_OK);
         }
     }

/*******************************************************************************
*
*  NAME
*    Reception d'un message sur une socket UDP en recuperant l'adresse de 
*    la prise de communication de l'emetteur
*
*  ALGORITHM
*    TCPIPrecv_socket(sock)
*
*****************************************************************************/
T_ENTIER TCPIPrecvfrom_texte_dgram DP ((refContext,
                                        sock,
                                        buffer,
                                        machine_orig,
                                        port_orig)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock SP
          atwstr ** buffer SP
          PT_CAR machine_orig SP
          PT_ENTIER port_orig EP
{
  /* Adresse de la machine emettrice */
  T_SOCKADDR_IN adr;
  SOCKLEN_T  adr_len;

#ifndef AGT_CUSTOM
  PT_HOSTENT machine;
#endif     /* AGT_CUSTOM */

  T_TABCAR  memoire_tampon;
  T_ENTIER  longueur_tampon = sizeof (memoire_tampon) - 1;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  /* Initialisation de la structure adresse */
  CSTmemset ((char *) &adr, 0, sizeof (T_SOCKADDR));

  do
    /* 
     * Envoie le message et repete l'operation si une interruption est 
     * recue 
     */
    {
      errno = 0;

      longueur_tampon = recvfrom (sock,
                                  (char *) memoire_tampon,
                                  longueur_tampon,
                                  0,
                                  (PT_SOCKADDR) & adr,
                                  (SOCKLEN_T *)&adr_len);

    }
  while ((longueur_tampon == -1) && (errno == EINTR));

  /* Une erreur est detectee */
  if (longueur_tampon == -1)
    {
      return (longueur_tampon);
    }

  /* Determine le nom de la machine destination */

#ifndef AGT_CUSTOM
  if (machine = gethostbyaddr ((char *) &adr.sin_addr, adr_len, AF_INET))
    {
      CSTstrcpy (machine_orig, machine->h_name);
    }
#endif  /* AGT_CUSTOM */

  /* Affecte le port de la prise de communication de l'emetteur */
  *port_orig = htons (adr.sin_port);

  /* Insertion du message dans le buffer du client */
  if (*buffer)
    {
      /* Celui-ci existe deja --> On insere a la suite */
      *buffer = atwstrAppend (*buffer, memoire_tampon);
    }
  else
    {
      /* Il faut creer le buffer */
      *buffer = atwstrClone (memoire_tampon);
    }
  return (longueur_tampon);
}

/*******************************************************************************
*
*  NAME

*    Fermeture d'une socket UDP
*
*  ALGORITHM
*    TCPIPclose_socket(sock)
*
*****************************************************************************/
T_RETURN TCPIPclose_dgram DP ((refContext, sock)) BP
          PT_REF_CONTEXT refContext SP
          T_SOCKET_DGRAM sock EP
{
  T_SOCKET  prise;

  PT_PRIVATE_CONTEXTE_TCPIP contexte;

  contexte = (PT_PRIVATE_CONTEXTE_TCPIP) CTXget (refContext, TCPIP_CTX);

  if ((contexte == NULL) || contexte->notinit)
    {
      return (SYSTEM_ERROR);
    }

  prise.dgram = sock;
  return (TCPIPclose_socket (contexte, prise));
}
