

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.  *
*                                                                              *
*                Product: Rational(R) Test RealTime                            *
*                                                                              *
*      Product Component: System Testing Agent                                 *
*                                                                              *
*               Language: C                                                    *
*                                                                              *
*        Product version: 2003.06.00                                           *
*                                                                              *
*                   File: traagt.c                                             *
*                                                                              *
* Rational, the Rational logo are registered trademarks of                     *
* Rational Software Corporation.                                               *
*                                                                              *
* The placing of the copyright notices on Rational source code is not          *
* intended to mean that they are published works, but are included in the      *
* event of inadvertent publication. The source code of Rational contains       *
* trade secrets and other valuable confidential information of Rational        *
* Software Corporation. Rational Software Corporation and its licensors        *
* retain ownership to Rational Products in both source and object code         *
* format, and related documentation and all rights therein including           *
* copyrights, patents and trade secrets.                                       *
*                                                                              *
*                                                                              *
* U.S. Government Rights                                                       *
*                                                                              *
* Use, duplication, or disclosure by the U.S. Government is subject to         *
* restrictions set forth in the applicable Rational License Agreement          *
* and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                   *
* 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,         *
* or FAR 52.227-14, as applicable.                                             *
*                                                                              *
* Unless otherwise agreed to in writing, use of Rational products is           *
* governed by the Rational Product License Agreement provided with             *
* the product.                                                                 *
*******************************************************************************/
#ifndef AGT_CUSTOM
static char WHAT_LINE[] = "@(#)traagt $Revision: 1.13 $ $Date: 2007/01/16 14:12:51 $";
#endif

/*********************************************************************
 *                                                                   *
 *% Module  TRAAGT                                                   *
 *                                                                   *
 *                         Identification                            *
 *                         --------------                            *
 *% Version $Revision: 1.13 $                                         *
 *% Auteur  Jean-Francois Marronnier                                 *
 *% Date    $Date: 2007/01/16 14:12:51 $                               *
 *                                                                   *
 *% Description                                                      *
 *    Gestion des traces retransmises par l'agent                    *
 *% Fdescription                                                     *
 *                                                                   *
 *********************************************************************/

/* Definition des modules utilises */
#ifndef AGT_CUSTOM
#include "cstsyst.h"
#else
#include "agtconf.h"
#endif

#include "atwstr.h"

#include "ctxagt.h"
#include "tcpip.h"

/* Definition du module */
#include "traagt.h"

typedef struct
  {
    T_BOOLEAN traagt_not_init;
    T_SOCKET_STREAM socket_trace;
    T_ENTIER  port_trace;
    T_ENTIER  port_devnull;
    T_SOCKET_STREAM socket_de_trace;
  }
T_PRIVATE_CONTEXT_TRA, *PT_PRIVATE_CONTEXT_TRA;

/******************************************************************************
*
*  NAME
*       Initialisation de la librairie
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN TRAAGTinit DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  PT_PRIVATE_CONTEXT_TRA contexte;

  if (Contexte != NULL)
    {
      contexte = (PT_PRIVATE_CONTEXT_TRA)CSTmalloc (sizeof (T_PRIVATE_CONTEXT_TRA));
    }
  else
    {
      return (SYSTEM_ERROR);
    }

  /*creation de la socket de trace sur :
   *    -> la machine locale (""),
   *    -> sans specifier le service (le port est defini pas l'OS
   * Et retourne le numero de port
   */
  contexte->port_trace = 0;
  contexte->socket_de_trace = -1;

#ifdef AGT_CUSTOM
  contexte->port_devnull = AGT_openNullSocket();
#else 
#ifdef UNIX
  contexte->port_devnull = open ("/dev/null", O_RDWR);
#endif                          /* UNIX */
#ifdef WIN32
  contexte->port_devnull = 0;
#endif                          /* WIN32 */
#endif /* AGT_CUSTOM */

  if ((contexte->socket_trace = TCPIPsocket_named_stream (Contexte, NULL, NULL,
                                                   &(contexte->port_trace)))
      == SYSTEM_ERROR)
    {
      return (SYSTEM_ERROR);
    }

  /* la "socket" est cree */

  contexte->socket_de_trace = contexte->port_devnull;

  contexte->traagt_not_init = FALSE;

  CTXset (Contexte, (void *) contexte, TRAAGT_CTX);

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*       Libere la librairie
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN TRAAGTfin DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  PT_PRIVATE_CONTEXT_TRA contexte;

  contexte = (PT_PRIVATE_CONTEXT_TRA) CTXget (Contexte, TRAAGT_CTX);

  if ((contexte == NULL) || contexte->traagt_not_init)
    {
      return (SYSTEM_ERROR);
    }

  /* ferme la socket de trace */
  if (contexte->socket_trace != -1)
    {
      (void) TCPIPclose_stream (Contexte, contexte->socket_trace);
    }
  if (contexte->socket_de_trace != -1)
    {
      (void) TCPIPclose_stream (Contexte, contexte->socket_de_trace);
    }

  contexte->port_trace = -1;

  contexte->traagt_not_init = TRUE;

  M_CSTfree (contexte);

  CTXset (Contexte, (void *) NULL, TRAAGT_CTX);

  return (SYSTEM_OK);
}

/******************************************************************************
*
*  NAME
*       Retourne le port de gestion  trace
*
*  ALGORITHME
*
******************************************************************************/
T_ENTIER TRAAGTport DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  PT_PRIVATE_CONTEXT_TRA contexte;

  contexte = (PT_PRIVATE_CONTEXT_TRA) CTXget (Contexte, TRAAGT_CTX);

  if ((contexte == NULL) || contexte->traagt_not_init)
    {
      return (SYSTEM_ERROR);
    }

  return (contexte->port_trace);
}

/******************************************************************************
*
*  NAME
*       Etablit une connexion sur le port de trace
*
*  ALGORITHME
*
******************************************************************************/
T_RETURN TRAAGTetablie DP ((Contexte)) BP
          PT_REF_CONTEXT Contexte EP
{
  PT_PRIVATE_CONTEXT_TRA contexte;

  contexte = (PT_PRIVATE_CONTEXT_TRA) CTXget (Contexte, TRAAGT_CTX);

  if ((contexte == NULL) || contexte->traagt_not_init)
    {
      return (SYSTEM_ERROR);
    }
  return (TCPIPaccept_stream (Contexte, contexte->socket_trace,
                              &contexte->socket_de_trace, NULL, NULL));
}

/******************************************************************************
*
*  NAME
*       Positionne les entrees standards sur le port de trace
*
*  ALGORITHME
*
******************************************************************************/
#ifdef WIN32
T_RETURN TRAAGTactive DP ((Contexte, oui, siStartInfo)) BP
          PT_REF_CONTEXT Contexte SP
          T_BOOLEAN oui SP
          STARTUPINFO * siStartInfo EP
#else
T_RETURN TRAAGTactive DP ((Contexte, oui)) BP
          PT_REF_CONTEXT Contexte SP
          T_BOOLEAN oui EP
#endif
{
  PT_PRIVATE_CONTEXT_TRA contexte;
  T_ENTIER  port_a_utiliser;

  contexte = (PT_PRIVATE_CONTEXT_TRA) CTXget (Contexte, TRAAGT_CTX);

#ifdef WIN32
  if ((contexte == NULL) || contexte->traagt_not_init || (siStartInfo == NULL))
#else
  if ((contexte == NULL) || contexte->traagt_not_init)
#endif
    {
      return (SYSTEM_ERROR);
    }

  if (oui)
    {
      port_a_utiliser = contexte->socket_de_trace;
    }
  else
    {
      port_a_utiliser = contexte->port_devnull;
    }

  if (port_a_utiliser != -1)
    {
      /* On duplique le canal de trace sur l'entree, la sortie nominale et
       * sur la sortie d'erreur standard
       */

#ifdef AGT_CUSTOM
      AGT_dupStdToSocket(port_a_utiliser);
#else
#ifdef UNIX
      dup2 (port_a_utiliser, STDIN_FILENO);
      dup2 (port_a_utiliser, STDOUT_FILENO);
      dup2 (port_a_utiliser, STDERR_FILENO);

      /* Pour le procesus FILS on ferme les connexions du pere.
       * Ceci permet de ne pas bloquer le port de trace, et donc
       * de deconnecter ... */
      close (contexte->port_devnull);
      contexte->port_devnull = -1;
      TCPIPclose_stream (Contexte, contexte->socket_de_trace);
      contexte->socket_de_trace = -1;
#endif                          /* UNIX */
#ifdef WIN32
      debug (printf ("TRACE ON port=%d\n", port_a_utiliser));
/*
 * Ne fonctionne pas avec les binaires 32 bits  
 *
 siStartInfo->hStdInput = port_a_utiliser; 
 siStartInfo->hStdOutput = port_a_utiliser;
 siStartInfo->hStdError = port_a_utiliser;
 *
 */
      SetStdHandle (STD_INPUT_HANDLE, (HANDLE) port_a_utiliser);
      SetStdHandle (STD_OUTPUT_HANDLE, (HANDLE) port_a_utiliser);
      SetStdHandle (STD_ERROR_HANDLE, (HANDLE) port_a_utiliser);
#endif                          /* WIN32 */
#endif /* AGT_CUSTOM */

      return (SYSTEM_OK);
    }
  else
    {
      return (SYSTEM_ERROR);
    }
}
