
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

-------------------------------------------------------------------------------
-- $Revision: 1.7.4.1 $
-- $Descr: Ada83 package DIRECT_IO$
-------------------------------------------------------------------------------
--
-- Ada83 Source Definition,
-- From LRM Ada83 Chapter 14.2.5
-- Modified to handle Implementation Dependent Definitions
--
-------------------------------------------------------------------------------
with IO_EXCEPTIONS;
with RTRT_Imp83 ;
generic
       type ELEMENT_TYPE is private;
package DIRECT_IO is 

       type FILE_TYPE is limited private; 

       type    FILE_MODE is (IN_FILE, INOUT_FILE, OUT_FILE);
       type    COUNT     is range 0 .. RTRT_Imp83.Direct_IO_Count_Last; 
       subtype POSITIVE_COUNT is COUNT range 1 .. COUNT'LAST;  

       -- File management 

       procedure CREATE(FILE : in out FILE_TYPE;
                        MODE : in FILE_MODE := INOUT_FILE;
                        NAME : in STRING := "";
                        FORM : in STRING := ""); 

       procedure OPEN  (FILE : in out FILE_TYPE;
                        MODE : in FILE_MODE;
                        NAME : in STRING;
                        FORM : in STRING := "");    

       procedure CLOSE (FILE : in out FILE_TYPE);
       procedure DELETE(FILE : in out FILE_TYPE);  
       procedure RESET (FILE : in out FILE_TYPE; MODE : in FILE_MODE);
       procedure RESET (FILE : in out FILE_TYPE); 

       function MODE   (FILE : in FILE_TYPE) return FILE_MODE;
       function NAME   (FILE : in FILE_TYPE) return STRING;
       function FORM   (FILE : in FILE_TYPE) return STRING; 

       function IS_OPEN(FILE : in FILE_TYPE) return BOOLEAN;                                                 

       -- Input and output operations 

       procedure READ (FILE : in FILE_TYPE; ITEM : out ELEMENT_TYPE; FROM :
        POSITIVE_COUNT);
       procedure READ (FILE : in FILE_TYPE; ITEM : out ELEMENT_TYPE);

       procedure WRITE(FILE : in FILE_TYPE; ITEM : in  ELEMENT_TYPE; TO :
        POSITIVE_COUNT);
       procedure WRITE(FILE : in FILE_TYPE; ITEM : in ELEMENT_TYPE);

       procedure SET_INDEX(FILE : in FILE_TYPE; TO : in POSITIVE_COUNT); 

       function INDEX(FILE : in FILE_TYPE) return POSITIVE_COUNT;
       function SIZE (FILE : in FILE_TYPE) return COUNT; 

       function END_OF_FILE(FILE : in FILE_TYPE) return BOOLEAN; 

       -- Exceptions 

       STATUS_ERROR : exception renames IO_EXCEPTIONS.STATUS_ERROR;
       MODE_ERROR   : exception renames IO_EXCEPTIONS.MODE_ERROR;
       NAME_ERROR   : exception renames IO_EXCEPTIONS.NAME_ERROR;
       USE_ERROR    : exception renames IO_EXCEPTIONS.USE_ERROR;
       DEVICE_ERROR : exception renames IO_EXCEPTIONS.DEVICE_ERROR;
       END_ERROR    : exception renames IO_EXCEPTIONS.END_ERROR;
       DATA_ERROR   : exception renames IO_EXCEPTIONS.DATA_ERROR;    

private

     -- implementation-dependent
     for FILE_MODE use ( IN_FILE=>0, INOUT_FILE=>1, OUT_FILE=>2 );
     -- implementation-dependent
     Type File_Type is new RTRT_Imp83.Direct_IO_File_Type ;

end DIRECT_IO;  
--
-------------------------------------------------------------------------------
