
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.6.4.1 $
-- $Descr: Ada95 package: Ada.Numerics.Complex_Types$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 G.1.1
--
--------------------------------------------------------------------------
package Ada.Numerics.Complex_Types is

    pragma Pure(Complex_Types);

    type Complex is
             record
                Re, Im : Float;
             end record;

    type Imaginary is private;

    i : constant Imaginary;
    j : constant Imaginary;

    function Re (X : Complex)   return Float;
    function Im (X : Complex)   return Float;
    function Im (X : Imaginary) return Float;

    procedure Set_Re (X  : in out Complex;
                      Re : in     Float);
    procedure Set_Im (X  : in out Complex;
                      Im : in     Float);
    procedure Set_Im (X  :    out Imaginary;
                      Im : in     Float);

    function Compose_From_Cartesian (Re, Im : Float) return Complex;
    function Compose_From_Cartesian (Re     : Float) return Complex;
    function Compose_From_Cartesian (Im     : Imaginary) return Complex;

    function Modulus (X     : Complex) return Float;
    function "abs"   (Right : Complex) return Float renames Modulus;

    function Argument (X     : Complex)   return Float;
    function Argument (X     : Complex;
                       Cycle : Float) return Float;

    function Compose_From_Polar (Modulus, Argument        : Float)
             return Complex;
    function Compose_From_Polar (Modulus, Argument, Cycle : Float)
             return Complex;

    function "+"       (Right : Complex) return Complex;
    function "-"       (Right : Complex) return Complex;
    function Conjugate (X     : Complex) return Complex;

    function "+" (Left, Right : Complex) return Complex;
    function "-" (Left, Right : Complex) return Complex;
    function "*" (Left, Right : Complex) return Complex;
    function "/" (Left, Right : Complex) return Complex;

    function "**" (Left : Complex; Right : Integer) return Complex;

    function "+"       (Right : Imaginary) return Imaginary;
    function "-"       (Right : Imaginary) return Imaginary;
    function Conjugate (X     : Imaginary) return Imaginary renames "-";
    function "abs"     (Right : Imaginary) return Float;

    function "+" (Left, Right : Imaginary) return Imaginary;
    function "-" (Left, Right : Imaginary) return Imaginary;
    function "*" (Left, Right : Imaginary) return Float;
    function "/" (Left, Right : Imaginary) return Float;

    function "**" (Left : Imaginary; Right : Integer) return Complex;

    function "<"  (Left, Right : Imaginary) return Boolean;
    function "<=" (Left, Right : Imaginary) return Boolean;
    function ">"  (Left, Right : Imaginary) return Boolean;
    function ">=" (Left, Right : Imaginary) return Boolean;

    function "+" (Left : Complex;   Right : Float) return Complex;
    function "+" (Left : Float; Right : Complex)   return Complex;
    function "-" (Left : Complex;   Right : Float) return Complex;
    function "-" (Left : Float; Right : Complex)   return Complex;
    function "*" (Left : Complex;   Right : Float) return Complex;
    function "*" (Left : Float; Right : Complex)   return Complex;
    function "/" (Left : Complex;   Right : Float) return Complex;
    function "/" (Left : Float; Right : Complex)   return Complex;

    function "+" (Left : Complex;   Right : Imaginary) return Complex;
    function "+" (Left : Imaginary; Right : Complex)   return Complex;
    function "-" (Left : Complex;   Right : Imaginary) return Complex;
    function "-" (Left : Imaginary; Right : Complex)   return Complex;
    function "*" (Left : Complex;   Right : Imaginary) return Complex;
    function "*" (Left : Imaginary; Right : Complex)   return Complex;
    function "/" (Left : Complex;   Right : Imaginary) return Complex;
    function "/" (Left : Imaginary; Right : Complex)   return Complex;

    function "+" (Left : Imaginary; Right : Float) return Complex;
    function "+" (Left : Float; Right : Imaginary) return Complex;
    function "-" (Left : Imaginary; Right : Float) return Complex;
    function "-" (Left : Float; Right : Imaginary) return Complex;
    function "*" (Left : Imaginary; Right : Float) return Imaginary;
    function "*" (Left : Float; Right : Imaginary) return Imaginary;
    function "/" (Left : Imaginary; Right : Float) return Imaginary;
    function "/" (Left : Float; Right : Imaginary) return Imaginary;

private

    type Imaginary is new Float;

    i : constant Imaginary := 1.0;
    j : constant Imaginary := 1.0;

end Ada.Numerics.Complex_Types;
--
--------------------------------------------------------------------------
