
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.7.4.1 $
-- $Descr: Ada95 package: Ada.Real_Time$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 D.8 - Monotonic Time
-- Modified to handle Implementation Dependent Definitions
--
--------------------------------------------------------------------------
with RTRT_Imp95 ;

package Ada.Real_Time is

    type Time is private;

    Time_First : constant Time;
    Time_Last : constant Time;
    Time_Unit : constant := RTRT_Imp95.c_Ada_Real_Time_Time_Unit ;

    -- (blank)


    type Time_Span is private;
    Time_Span_First : constant Time_Span;
    Time_Span_Last : constant Time_Span;
    Time_Span_Zero : constant Time_Span;
    Time_Span_Unit : constant Time_Span;


    Tick : constant Time_Span;
    function Clock return Time;


    function "+" (Left : Time; Right : Time_Span) return Time;
    function "+" (Left : Time_Span; Right : Time) return Time;
    function "-" (Left : Time; Right : Time_Span) return Time;
    function "-" (Left : Time; Right : Time) return Time_Span;


    function "<" (Left, Right : Time) return Boolean;
    function "<="(Left, Right : Time) return Boolean;
    function ">" (Left, Right : Time) return Boolean;
    function ">="(Left, Right : Time) return Boolean;


    function "+" (Left, Right : Time_Span) return Time_Span;
    function "-" (Left, Right : Time_Span) return Time_Span;
    function "-" (Right : Time_Span) return Time_Span;
    function "*" (Left : Time_Span; Right : Integer) return Time_Span;
    function "*" (Left : Integer; Right : Time_Span) return Time_Span;
    function "/" (Left, Right : Time_Span) return Integer;
    function "/" (Left : Time_Span; Right : Integer) return Time_Span;

    function "abs"(Right : Time_Span) return Time_Span;


    function "<" (Left, Right : Time_Span) return Boolean;
    function "<="(Left, Right : Time_Span) return Boolean;
    function ">" (Left, Right : Time_Span) return Boolean;
    function ">="(Left, Right : Time_Span) return Boolean;


    function To_Duration (TS : Time_Span) return Duration;
    function To_Time_Span (D : Duration) return Time_Span;


    function Nanoseconds  (NS : Integer) return Time_Span;
    function Microseconds (US : Integer) return Time_Span;
    function Milliseconds (MS : Integer) return Time_Span;

    type Seconds_Count is range 
                   RTRT_Imp95.t_Ada_Real_Time_Seconds_Count'first ..
                   RTRT_Imp95.t_Ada_Real_Time_Seconds_Count'last ;

    procedure Split(T : in Time; SC : out Seconds_Count; TS : out Time_Span);
    function Time_Of(SC : Seconds_Count; TS : Time_Span) return Time;

private
    -- not specified by the language

    Type Time is new RTRT_Imp95.t_Ada_Real_Time_Time ;

    Time_First : constant Time
     :=  Time(RTRT_Imp95.c_Ada_Real_Time_Time_First);
    Time_Last : constant Time
     :=  Time(RTRT_Imp95.c_Ada_Real_Time_Time_Last) ;

    type Time_Span is new RTRT_Imp95.t_Ada_Real_Time_Time_Span ;

    Time_Span_First : constant Time_Span
     := Time_Span(RTRT_Imp95.c_Ada_Real_Time_Time_Span_First);
    Time_Span_Last : constant Time_Span
     := Time_Span(RTRT_Imp95.c_Ada_Real_Time_Time_Span_Last );
    Time_Span_Zero : constant Time_Span
     := Time_Span(RTRT_Imp95.c_Ada_Real_Time_Time_Span_Zero );
    Time_Span_Unit : constant Time_Span
     := Time_Span(RTRT_Imp95.c_Ada_Real_Time_Time_Span_Unit );


    Tick : constant Time_Span;

end Ada.Real_Time;
--
--------------------------------------------------------------------------
