
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.6.4.1 $
-- $Descr: Ada95 package: Ada.Strings.Maps$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 A.4.2 - The Package Strings.Maps
-- Modified to handle Implementation Dependent Definitions
--
--------------------------------------------------------------------------
with RTRT_Imp95 ;

package Ada.Strings.Maps is

   pragma Preelaborate(Maps);

   -- Representation for a set of character values:
   type Character_Set is private;

   Null_Set : constant Character_Set;

   type Character_Range is
            record
               Low  : Character;
               High : Character;
            end record;
          -- Represents Character range Low..High

   type Character_Ranges is array (Positive range <>) of Character_Range;

   function To_Set    (Ranges : in Character_Ranges) return Character_Set;

   function To_Set    (Span   : in Character_Range)  return Character_Set;

   function To_Ranges (Set    : in Character_Set)    return Character_Ranges;

   function "="   (Left, Right : in Character_Set) return Boolean;

   function "not" (Right : in Character_Set)       return Character_Set;
   function "and" (Left, Right : in Character_Set) return Character_Set;
   function "or"  (Left, Right : in Character_Set) return Character_Set;
   function "xor" (Left, Right : in Character_Set) return Character_Set;
   function "-"   (Left, Right : in Character_Set) return Character_Set;

   function Is_In (Element : in Character;
                   Set     : in Character_Set)
             return Boolean;

   function Is_Subset (Elements : in Character_Set;
                       Set      : in Character_Set)
             return Boolean;

   function "<=" (Left  : in Character_Set;
                  Right : in Character_Set)
             return Boolean renames Is_Subset;

   -- Alternative representation for a set of character values:
   subtype Character_Sequence is String;

   function To_Set (Sequence  : in Character_Sequence) return Character_Set;

   function To_Set (Singleton : in Character)          return Character_Set;

   function To_Sequence (Set  : in Character_Set)      return Character_Sequence;

   -- Representation for a character to character mapping:
   type Character_Mapping is private;

   function Value (Map     : in Character_Mapping;
                   Element : in Character)
             return Character;

   Identity : constant Character_Mapping;

   function To_Mapping (From, To : in Character_Sequence) return Character_Mapping;

   function To_Domain (Map : in Character_Mapping) return Character_Sequence;
   function To_Range  (Map : in Character_Mapping) return Character_Sequence;

   type Character_Mapping_Function is
             access function (From : in Character) return Character;

private

     -- not specified by the language
     Type Character_Set is 
      new RTRT_Imp95.t_Ada_Strings_Maps_Character_Set ;

     Type Character_Mapping is 
      new RTRT_Imp95.t_Ada_Strings_Maps_Character_Mapping ;

     Null_Set : constant Character_Set 
                := Character_Set(RTRT_Imp95.c_Ada_Strings_Maps_Null_Set) ;

     Identity : constant Character_Mapping 
                := Character_Mapping(RTRT_Imp95.c_Ada_Strings_Maps_Identity) ;

end Ada.Strings.Maps;
--
--------------------------------------------------------------------------
