
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.6.4.1 $
-- $Descr: Ada95 package: Ada.Strings.Unbounded$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 A.4.5 - Unbounded-Length Strings Handling
-- Modified to handle Implementation Dependent Definitions
--
--------------------------------------------------------------------------
with RTRT_Imp95 ;

with Ada.Strings.Maps;
package Ada.Strings.Unbounded is

          pragma Preelaborate(Unbounded);

          type Unbounded_String is private;

          Null_Unbounded_String : constant Unbounded_String;

          function Length (Source : in Unbounded_String) return Natural;

          type String_Access is access all String;
          procedure Free (X : in out String_Access);

       -- Conversion, Concatenation, and Selection functions

          function To_Unbounded_String (Source : in String)
             return Unbounded_String;

          function To_Unbounded_String (Length : in Natural)
             return Unbounded_String;

          function To_String (Source : in Unbounded_String) return String;

          procedure Append (Source   : in out Unbounded_String;
                            New_Item : in Unbounded_String);

          procedure Append (Source   : in out Unbounded_String;
                            New_Item : in String);

          procedure Append (Source   : in out Unbounded_String;
                            New_Item : in Character);

          function "&" (Left, Right : in Unbounded_String)
             return Unbounded_String;

          function "&" (Left : in Unbounded_String; Right : in String)
             return Unbounded_String;

          function "&" (Left : in String; Right : in Unbounded_String)
             return Unbounded_String;

          function "&" (Left : in Unbounded_String; Right : in Character)
             return Unbounded_String;

          function "&" (Left : in Character; Right : in Unbounded_String)
             return Unbounded_String;

          function Element (Source : in Unbounded_String;
                            Index  : in Positive)
             return Character;

          procedure Replace_Element (Source : in out Unbounded_String;
                                     Index  : in Positive;
                                     By     : in Character);

          function Slice (Source : in Unbounded_String;
                          Low    : in Positive;
                          High   : in Natural)
             return String;

          function "="  (Left, Right : in Unbounded_String) return Boolean;

          function "="  (Left : in Unbounded_String; Right : in String)
            return Boolean;

          function "="  (Left : in String; Right : in Unbounded_String)
            return Boolean;

          function "<"  (Left, Right : in Unbounded_String) return Boolean;

          function "<"  (Left : in Unbounded_String; Right : in String)
            return Boolean;

          function "<"  (Left : in String; Right : in Unbounded_String)
            return Boolean;

          function "<=" (Left, Right : in Unbounded_String) return Boolean;

          function "<="  (Left : in Unbounded_String; Right : in String)
            return Boolean;

          function "<="  (Left : in String; Right : in Unbounded_String)
            return Boolean;

          function ">"  (Left, Right : in Unbounded_String) return Boolean;

          function ">"  (Left : in Unbounded_String; Right : in String)
            return Boolean;

          function ">"  (Left : in String; Right : in Unbounded_String)
            return Boolean;

          function ">=" (Left, Right : in Unbounded_String) return Boolean;

          function ">="  (Left : in Unbounded_String; Right : in String)
            return Boolean;

          function ">="  (Left : in String; Right : in Unbounded_String)
            return Boolean;

       -- Search subprograms

          function Index (Source   : in Unbounded_String;
                          Pattern  : in String;
                          Going    : in Direction := Forward;
                          Mapping  : in Maps.Character_Mapping
                                       := Maps.Identity)
             return Natural;

          function Index (Source   : in Unbounded_String;
                          Pattern  : in String;
                          Going    : in Direction := Forward;
                          Mapping  : in Maps.Character_Mapping_Function)
             return Natural;

          function Index (Source : in Unbounded_String;
                          Set    : in Maps.Character_Set;
                          Test   : in Membership := Inside;
                          Going  : in Direction  := Forward) return Natural;

          function Index_Non_Blank (Source : in Unbounded_String;
                                    Going  : in Direction := Forward)
             return Natural;

          function Count (Source   : in Unbounded_String;
                          Pattern  : in String;
                          Mapping  : in Maps.Character_Mapping
                                       := Maps.Identity)
             return Natural;

          function Count (Source   : in Unbounded_String;
                          Pattern  : in String;
                          Mapping  : in Maps.Character_Mapping_Function)
             return Natural;

          function Count (Source   : in Unbounded_String;
                          Set      : in Maps.Character_Set)
             return Natural;

          procedure Find_Token (Source : in Unbounded_String;
                                Set    : in Maps.Character_Set;
                                Test   : in Membership;
                                First  : out Positive;
                                Last   : out Natural);

       -- String translation subprograms

          function Translate (Source  : in Unbounded_String;
                              Mapping : in Maps.Character_Mapping)
             return Unbounded_String;

          procedure Translate (Source  : in out Unbounded_String;
                               Mapping : in Maps.Character_Mapping);

          function Translate (Source  : in Unbounded_String;
                              Mapping : in Maps.Character_Mapping_Function)
             return Unbounded_String;

          procedure Translate (Source  : in out Unbounded_String;
                               Mapping : in Maps.Character_Mapping_Function);

       -- String transformation subprograms

          function Replace_Slice (Source   : in Unbounded_String;
                                  Low      : in Positive;
                                  High     : in Natural;
                                  By       : in String)
             return Unbounded_String;

          procedure Replace_Slice (Source   : in out Unbounded_String;
                                   Low      : in Positive;
                                   High     : in Natural;
                                   By       : in String);

          function Insert (Source   : in Unbounded_String;
                           Before   : in Positive;
                           New_Item : in String)
             return Unbounded_String;

          procedure Insert (Source   : in out Unbounded_String;
                            Before   : in Positive;
                            New_Item : in String);

          function Overwrite (Source    : in Unbounded_String;
                              Position  : in Positive;
                              New_Item  : in String)
             return Unbounded_String;

          procedure Overwrite (Source    : in out Unbounded_String;
                               Position  : in Positive;
                               New_Item  : in String);

          function Delete (Source  : in Unbounded_String;
                           From    : in Positive;
                           Through : in Natural)
             return Unbounded_String;

          procedure Delete (Source  : in out Unbounded_String;
                            From    : in Positive;
                            Through : in Natural);

          function Trim (Source : in Unbounded_String;
                         Side   : in Trim_End)
             return Unbounded_String;

          procedure Trim (Source : in out Unbounded_String;
                          Side   : in Trim_End);

          function Trim (Source : in Unbounded_String;
                         Left   : in Maps.Character_Set;
                         Right  : in Maps.Character_Set)
             return Unbounded_String;

          procedure Trim (Source : in out Unbounded_String;
                          Left   : in Maps.Character_Set;
                          Right  : in Maps.Character_Set);

          function Head (Source : in Unbounded_String;
                         Count  : in Natural;
                         Pad    : in Character := Space)
             return Unbounded_String;

          procedure Head (Source : in out Unbounded_String;
                          Count  : in Natural;
                          Pad    : in Character := Space);

          function Tail (Source : in Unbounded_String;
                         Count  : in Natural;
                         Pad    : in Character := Space)
             return Unbounded_String;

          procedure Tail (Source : in out Unbounded_String;
                          Count  : in Natural;
                          Pad    : in Character := Space);

          function "*" (Left  : in Natural;
                        Right : in Character)
             return Unbounded_String;

          function "*" (Left  : in Natural;
                        Right : in String)
             return Unbounded_String;

          function "*" (Left  : in Natural;
                        Right : in Unbounded_String)
             return Unbounded_String;

private

     -- not specified by the language
     Type Unbounded_String is 
        new RTRT_Imp95.t_Ada_Strings_Unbounded_Unbounded_String ;

     Null_Unbounded_String : constant Unbounded_String
        := Unbounded_String(RTRT_Imp95.c_Ada_Strings_Unbounded_Null_Unbounded_String) ;

end Ada.Strings.Unbounded;
--
--------------------------------------------------------------------------
