
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.6.4.1 $
-- $Descr: Ada95 package: Ada.Text_IO.Editing$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 F.3.3 - The Package Text_IO.Editing
-- Modified to handle Implementation Dependent Definitions
--
--------------------------------------------------------------------------
with RTRT_Imp95 ;

Package Ada.Text_IO.Editing is

          type Picture is private;


          function Valid (Pic_String      : in String;
                          Blank_When_Zero : in Boolean := False) return Boolean;

          function To_Picture (Pic_String      : in String;
                               Blank_When_Zero : in Boolean := False)
             return Picture;

          function Pic_String      (Pic : in Picture) return String;
          function Blank_When_Zero (Pic : in Picture) return Boolean;

          Max_Picture_Length  : constant 
             := RTRT_Imp95.c_Ada_Text_IO_Editing_Max_Picture_Length ;

          Picture_Error       : exception;

          Default_Currency    : constant String    := "$";
          Default_Fill        : constant Character := '*';
          Default_Separator   : constant Character := ',';
          Default_Radix_Mark  : constant Character := '.';

          generic
             type Num is delta <> digits <>;
             Default_Currency   : in String    :=  Text_IO.Editing.Default_Currency;
             Default_Fill       : in Character :=  Text_IO.Editing.Default_Fill;
             Default_Separator  : in Character :=  Text_IO.Editing.Default_Separator;
             Default_Radix_Mark : in Character :=  Text_IO.Editing.Default_Radix_Mark;
          package Decimal_Output is
             function Length (Pic      : in Picture;
                              Currency : in String := Default_Currency)
                return Natural;

             function Valid (Item     : in Num;
                             Pic      : in Picture;
                             Currency : in String := Default_Currency)
                return Boolean;

             function Image (Item       : in Num;
                             Pic        : in Picture;
                             Currency   : in String    := Default_Currency;
                             Fill       : in Character := Default_Fill;
                             Separator  : in Character := Default_Separator;
                             Radix_Mark : in Character := Default_Radix_Mark)
                return String;

             procedure Put (File       : in File_Type;
                            Item       : in Num;
                            Pic        : in Picture;
                            Currency   : in String    := Default_Currency;
                            Fill       : in Character := Default_Fill;
                            Separator  : in Character := Default_Separator;
                            Radix_Mark : in Character := Default_Radix_Mark);

             procedure Put (Item       : in Num;
                            Pic        : in Picture;
                            Currency   : in String    := Default_Currency;
                            Fill       : in Character := Default_Fill;
                            Separator  : in Character := Default_Separator;
                            Radix_Mark : in Character := Default_Radix_Mark);

             procedure Put (To         : out String;
                            Item       : in Num;
                            Pic        : in Picture;
                            Currency   : in String    := Default_Currency;
                            Fill       : in Character := Default_Fill;
                            Separator  : in Character := Default_Separator;
                            Radix_Mark : in Character := Default_Radix_Mark);
          end Decimal_Output;

private

    -- not specified by the language
    Type Picture is new RTRT_Imp95.t_Ada_Text_IO_Editing_Picture ; 

end Ada.Text_IO.Editing;
--
--------------------------------------------------------------------------
