
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.6.4.1 $
-- $Descr: Ada95 package: Interfaces.C.Pointers$
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 B.3.2 - The Generic Package Interfces.C.Pointers
--
--------------------------------------------------------------------------
generic

   type Index is (<>);
   type Element is private;
   type Element_Array is array (Index range <>) of aliased Element;
   Default_Terminator : Element;

package Interfaces.C.Pointers is

   pragma Preelaborate(Pointers);

          type Pointer is access all Element;

          function Value(Ref        : in Pointer;
                         Terminator : in Element := Default_Terminator)
             return Element_Array;

          function Value(Ref    : in Pointer;
                         Length : in ptrdiff_t)
             return Element_Array;

          Pointer_Error : exception;

          -- C-style Pointer arithmetic

          function "+" (Left : in Pointer;   Right : in ptrdiff_t) return Pointer;
          function "+" (Left : in ptrdiff_t; Right : in Pointer)   return Pointer;
          function "-" (Left : in Pointer;   Right : in ptrdiff_t) return Pointer;
          function "-" (Left : in Pointer;   Right : in Pointer)   return ptrdiff_t;

          procedure Increment (Ref : in out Pointer);
          procedure Decrement (Ref : in out Pointer);

          pragma Convention (Intrinsic, "+");
          pragma Convention (Intrinsic, "-");
          pragma Convention (Intrinsic, Increment);
          pragma Convention (Intrinsic, Decrement);

          function Virtual_Length (Ref        : in Pointer;
                                   Terminator : in Element := Default_Terminator)
             return ptrdiff_t;

          procedure Copy_Terminated_Array (Source     : in Pointer;
                                           Target     : in Pointer;
                                           Limit      : in ptrdiff_t := ptrdiff_t'Last;
                                           Terminator : in Element :=  Default_Terminator);

          procedure Copy_Array (Source  : in Pointer;
                                Target  : in Pointer;
                                Length  : in ptrdiff_t);

end Interfaces.C.Pointers;
--
--------------------------------------------------------------------------
