
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

--------------------------------------------------------------------------
-- $Revision: 1.7.4.1 $
-- $Descr: Ada95 package: Interfaces.C $
--------------------------------------------------------------------------
--
-- Ada95 Source Definition,
-- From LRM Ada95 B.3 - Interfacing With C
-- Modified to handle Implementation Dependent Definitions
--
--------------------------------------------------------------------------
with RTRT_Imp95 ;

package Interfaces.C is

          pragma Pure(C);

          -- Declarations based on C's <limits.h>

          CHAR_BIT  : constant := RTRT_Imp95.c_Interfaces_C_CHAR_BIT;  -- typically 8
          SCHAR_MIN : constant := RTRT_Imp95.c_Interfaces_C_SCHAR_MIN;  -- typically -128
          SCHAR_MAX : constant := RTRT_Imp95.c_Interfaces_C_SCHAR_MAX;  -- typically 127
          UCHAR_MAX : constant := RTRT_Imp95.c_Interfaces_C_UCHAR_MAX;  -- typically 255

          -- Signed and Unsigned Integers
          type int   is range  RTRT_Imp95.t_Interfaces_C_int 'first.. 
	                     RTRT_Imp95.t_Interfaces_C_int 'last;
          type short is range  RTRT_Imp95.t_Interfaces_C_short 'first.. 
	                     RTRT_Imp95.t_Interfaces_C_short 'last;
          type long  is range  RTRT_Imp95.t_Interfaces_C_long 'first.. 
	                     RTRT_Imp95.t_Interfaces_C_long 'last;

          type signed_char is range SCHAR_MIN .. SCHAR_MAX;
          for signed_char'Size use CHAR_BIT;

          type unsigned       is mod  
	                     RTRT_Imp95.t_Interfaces_C_unsigned 'modulus;
          type unsigned_short is mod  
	                     RTRT_Imp95.t_Interfaces_C_unsigned_short'modulus;
          type unsigned_long  is mod  
	                     RTRT_Imp95.t_Interfaces_C_unsigned_long 'modulus;

          type unsigned_char is mod (UCHAR_MAX+1);
          for unsigned_char'Size use CHAR_BIT;

          subtype plain_char is character range  
	       RTRT_Imp95.t_Interfaces_C_plain_char 'first.. 
	       RTRT_Imp95.t_Interfaces_C_plain_char 'last;

          type ptrdiff_t is range  RTRT_Imp95.t_Interfaces_C_ptrdiff_t 'first.. RTRT_Imp95.t_Interfaces_C_ptrdiff_t 'last;

          type size_t is mod RTRT_Imp95.t_Interfaces_C_size_t'modulus ;

          -- Floating Point

          type C_float     is digits RTRT_Imp95.t_Interfaces_C_C_float'digits;

          type double      is digits RTRT_Imp95.t_Interfaces_C_double'digits;

          type long_double is digits 
	                   RTRT_Imp95.t_Interfaces_C_long_double'digits;

          -- Characters and Strings

          type char is new standard.character range  
	     RTRT_Imp95.t_Interfaces_C_char 'first.. 
	     RTRT_Imp95.t_Interfaces_C_char 'last;

          nul : constant char := char'First;

          function To_C   (Item : in Character) return char;

          function To_Ada (Item : in char) return Character;

          type char_array is array (size_t range <>) of aliased char;
          pragma Pack(char_array);
          for char_array'Component_Size use CHAR_BIT;

          function Is_Nul_Terminated (Item : in char_array) return Boolean;

          function To_C   (Item       : in String;
                           Append_Nul : in Boolean := True)
             return char_array;

          function To_Ada (Item     : in char_array;
                           Trim_Nul : in Boolean := True)
             return String;

          procedure To_C (Item       : in String;
                          Target     : out char_array;
                          Count      : out size_t;
                          Append_Nul : in Boolean := True);

          procedure To_Ada (Item     : in char_array;
                            Target   : out String;
                            Count    : out Natural;
                            Trim_Nul : in Boolean := True);

          -- Wide Character and Wide String

          type wchar_t is new wide_character range  
	      RTRT_Imp95.t_Interfaces_C_wchar_t 'first.. 
	      RTRT_Imp95.t_Interfaces_C_wchar_t 'last;

          wide_nul : constant wchar_t := wchar_t'First;

          function To_C   (Item : in Wide_Character) return wchar_t;
          function To_Ada (Item : in wchar_t       ) return Wide_Character;

          type wchar_array is array (size_t range <>) of aliased wchar_t;

          pragma Pack(wchar_array);

          function Is_Nul_Terminated (Item : in wchar_array) return Boolean;

          function To_C   (Item       : in Wide_String;
                           Append_Nul : in Boolean := True)
             return wchar_array;

          function To_Ada (Item     : in wchar_array;
                           Trim_Nul : in Boolean := True)
             return Wide_String;

          procedure To_C (Item       : in  Wide_String;
                          Target     : out wchar_array;
                          Count      : out size_t;
                          Append_Nul : in  Boolean := True);

          procedure To_Ada (Item     : in  wchar_array;
                            Target   : out Wide_String;
                            Count    : out Natural;
                            Trim_Nul : in  Boolean := True);

          Terminator_Error : exception;

end Interfaces.C;
--
--------------------------------------------------------------------------
