
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

-------------------------------------------------------------------------------
--  $Revision: 1.16.2.1 $ 
--  $Descr: Link between UniTesting and coverage for Ada83 TDP$
-------------------------------------------------------------------------------
-- NOTE :
--    --? comments may be used to optimize this runtime
-------------------------------------------------------------------------------



-------------------------------------------------------------------------------
--
--  ATTOLCOV Package body
--
-------------------------------------------------------------------------------

--DR:1381
with attol_serv ;
pragma elaborate( attol_serv );

package body attolcov is

  do_runtime_initialization : boolean := true;
  procedure runtime_initialization ;
--------------------------------------------------------------------------------
-- services/resources for Attol UniTest mode 
--                        and dump functionalities
-------------------------------------------------------------------------------

  -- buffers to store some interesting information
   -- string image size of integer and branches (enough for 32bits number)
  MAX_NUMBER_LENGTH : CONSTANT := 10; 
   -- length of xxx_name
  MAX_NAME_LENGTH : CONSTANT := 100 ; 
   -- length of output buffer
  MAX_BUF_LENGTH : CONSTANT := 128 ;
   -- must be enough to store 'NT "<NT_test_name>" <integer> <integer>'
   --   := MAX_NAME_LENGTH+2*MAX_NUMBER_LENGTH+6 ; 

  TYPE t_name   IS RECORD
		len : integer := 0;
		str : STRING( 1..MAX_NAME_LENGTH );
	END RECORD ;

  TYPE t_buffer IS RECORD
		len  : integer := 0 ;
		str  : STRING( 1..MAX_BUF_LENGTH ) ;
	END RECORD ;

  -- internal buffer for output.
  buffer : t_buffer ;

  -- This flag is set when a call of register_service has appear
  -- and until unregister call.
  ATU_header : boolean := FALSE ;

  -- This variable memorize localy the latest service number
  service_number : integer := 0 ;
  -- This variable memorize localy the latest service name
  service_name   : t_name ;

  -- This variable memorize localy the latest test
  test_number : integer := 0 ;
  -- This variable memorize localy the latest test name
  test_name   : t_name ;

  -- This variable is service_name " " test_name (may be truncated)
  NT_test_name : t_name ;
 
-------------------------------------------------------------------------------
-- end of Attol UniTest mode functionnalities
-------------------------------------------------------------------------------

  -- UNCHECKED CONVERSION 
  -----------------------------------------------------------------------------

  -- used to convert a t_link address into an access to a t_link
  function a2l is new unchecked_conversion(address,p_link);
  -- used to convert a t_branch address into an access to a t_branch
  function a2branch is new unchecked_conversion(address,p_branch);
  -- used to convert a t_mcVector address into an access to t_mcVector
  function a2mcVector is new unchecked_conversion(address,p_mcVector);
  -- used to convert a t_mcInfo address into an access to t_mcInfo
  function a2mcInfo is new unchecked_conversion(address,p_mcInfo);
    
  -- This internal errors can appear (but we hope not!).
  type t_InternalError is ( EI_MCV_NOT_FOUND,
			    EI_MCV_BAD_EVAL,
			    EI_MCV_BAD_BC_VECTOR  );

  -- These flags remember if an internal error occurs until dump is not done.
  internal_error : array(t_internalError) of boolean ;



  -- REPOSITORY LIST MANAGMENT
  -----------------------------------------------------------------------------

  header : p_link := NULL;  -- first element of list
  
  footer  : p_link := NULL; -- last element of list (append new element)

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : buf_flush()                                            --
--                                                                           --
--    Flush the buffer to the output.                                        --
--                                                                           --
-------------------------------------------------------------------------------
procedure buf_flush( file : in out private_io.priv_file ) is
begin
     private_io.priv_writeln( file, buffer.str(1..buffer.len) );
     buffer.len := 0;
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : buf_print()                                            --
--                                                                           --
--    Append the integer image to the buffer                                 --
--    using digit inverted hexadecimal format                                --
-------------------------------------------------------------------------------
procedure buf_print( i : in integer ) is
  tmp        : string(1..MAX_NUMBER_LENGTH);
  j          : integer := 1;
  len        : integer := 0;
  rest,digit : integer ;
begin
     rest := i ;
     loop
         digit := rest rem 16 ;
         rest  := rest  /  16 ;
         case digit is
           when  0.. 9 => tmp(j) := CHARACTER'VAL(digit+48);
           when 10..15 => tmp(j) := CHARACTER'VAL(digit+55);
	   when others => null; -- too bad
	 end case ;
	 len  := len+1; 
	 j    := j+1;
	 exit when rest = 0 ;
     end loop ;

     ---- copy number image into buffer
     buffer.str(buffer.len+1..buffer.len+len) := tmp(1..j-1);
     buffer.len := buffer.len + len ;
   
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : buf_print()                                            --
--                                                                           --
--    Append the branch image to the buffer                               --
--                                                                           --
-------------------------------------------------------------------------------
procedure buf_print( b : in branch ) is
begin
     buf_print(integer(b));
end ;


-------------------------------------------------------------------------------
-- INTERNAL SERVICE : buf_print()                                            --
--                                                                           --
--    Append the character to the buffer                                     --
--                                                                           --
-------------------------------------------------------------------------------
procedure buf_print( c : in character ) is
begin
     buffer.len := buffer.len +1 ;
     buffer.str( buffer.len ) := c ;
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : buf_print()                                            --
--                                                                           --
--    Append the string to the buffer                                        --
--                                                                           --
-------------------------------------------------------------------------------
procedure buf_print( s : in string ) is
  len : integer ;
begin
     len := s'last-s'first+1 ;
     buffer.str( buffer.len+1..len+buffer.len ) := s(s'first..s'last) ;
     buffer.len := buffer.len + len ;
end ;


-------------------------------------------------------------------------------
-- INTERNAL SERVICE : reset_traces()                                         --
--                                                                           --
--    Reset traces tables.                                                   --
--                                                                           --
-------------------------------------------------------------------------------
procedure reset_traces( link : in p_link ) is
begin
      -- TP traces
      for i in link.TP_first .. link.TP_last loop
          link.TP(i) := 0;
      end loop;
      -- TB traces
      for i in link.TB_first .. link.TB_last loop
          link.TB(i) := 0;
      end loop;
      -- TA traces
      for i in link.TA_first .. link.TA_last loop
          link.TA(i) := 0;
      end loop;
      -- TE traces
      for i in link.TE_first .. link.TE_last loop
          link.TE(i) := 0;
      end loop;
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : resetInternalError                                     --
--                                                                           --
--    Reset internal error flags.                                            --
--                                                                           --
-------------------------------------------------------------------------------
procedure resetInternalErrors is
begin
      for i in t_internalError loop
         internal_error( i ) := false ;
      end loop ;
end ;

-------------------------------------------------------------------------------
-- SERVICE : link_desc( address, string32b, string32b )                      --
--                                                                           --
--     Append a new link in repository list.                                 --
--                                                                           --
-------------------------------------------------------------------------------
procedure link_desc ( adr_link : in address; key,checksum: in string32b )is
     link : p_link ;
begin
--INIT_UNSUPPORTED: uncomment 3 lines below:
--       if ( do_runtime_initialization ) then
--         runtime_initialization;
--       end if;    

       link := a2l( adr_link );
       if ( header = NULL) then
	-- Initialization
        private_io.priv_init(0);
        -- empty list, update header.
       	header := link;
       	link.prev := NULL;
       	link.next := NULL;
       	footer := link;
       else
         -- insert at the end of the list : after footer
	footer.next := link;
	link.prev   := footer ;
	footer      := link ;
	link.next   := NULL;
       end if;
       reset_traces( link );
       link.key := key ;
       link.checksum := checksum ;

end link_desc ;


-------------------------------------------------------------------------------
-- SERVICE : flow_hit( t_link'address, t_coverage, branch )                  --
--                                                                           --
--    This service increment given branch counter.  Remember, this service   --
-- may run in count AND pass. mode.                                          --
--                                                                           --
-------------------------------------------------------------------------------
procedure flow_hit( l:in system.address; c:in t_coverage; b:in natural)
is
 plink : p_link ;
begin
     plink := a2l( l );

     case c is 
 
       when Proc
	  | Ret   => if ( plink.all.TP(b)  < branch'last ) then
			  plink.all.TP(b) := branch'succ( plink.all.TP(b) );
		     end if ;

       when Block
	  | Implicit
	  | Logical
	  | Atc   => if ( plink.all.TB(b)  < branch'last ) then
			  plink.all.TB(b) := branch'succ( plink.all.TB(b) );
		     end if ;

       when Call  => if ( plink.all.TA(b)  < branch'last ) then
			  plink.all.TA(b) := branch'succ( plink.all.TA(b) );
		     end if ;

       when Cond  => if ( plink.all.TE(b)  < branch'last ) then
			  plink.all.TE(b) := branch'succ( plink.all.TE(b) );
		     end if ;
     end case ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : count_hit( a_br )                                               --
--                                                                           --
--    This service increment given branch counter.  Remember, this service   --
-- may run in count mode.                                                    --
--                                                                           --
-------------------------------------------------------------------------------
procedure count_hit( a_br:in system.address )
is
 p_br : p_branch ;
begin
     p_br := a2branch( a_br );

     if ( p_br.all  < branch'last ) then
	  p_br.all := branch'succ( p_br.all );
     end if ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : inline_hit( a_br )                                              --
--                                                                           --
--    This service set given branch counter.  Remember, this service         --
-- run in pass mode.                                                         --
--                                                                           --
-------------------------------------------------------------------------------
procedure inline_hit( a_br:in system.address )
is
 p_br : p_branch ;
begin
     p_br := a2branch( a_br );
     p_br.all := 1 ;
end ;




-------------------------------------------------------------------------------
-- SERVICE : flow_hit( address ) return boolean                              --
--                                                                           --
--  This function must return TRUE.                                          --
-------------------------------------------------------------------------------
function flow_hit( l:in system.address; c: in t_coverage; b:in natural)
  return boolean
is
begin
     flow_hit( l, c, b );
     return true ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : count_hit( address ) return boolean                             --
--                                                                           --
--  This function must return TRUE.                                          --
-------------------------------------------------------------------------------
function count_hit( a_br:in system.address)
  return boolean
is
begin
     count_hit( a_br );
     return true ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : inline_hit( address ) return boolean                            --
--                                                                           --
--  This function must return TRUE.                                          --
-------------------------------------------------------------------------------
function inline_hit( a_br:in system.address)
  return boolean
is
begin
     inline_hit(a_br);
     return true ;
end ;



-------------------------------------------------------------------------------
-- SERVICE : flow_hit( address ) return boolean                              --
--                                                                           --
--  This function must return given value 'cond'.                            --
-------------------------------------------------------------------------------
function flow_hit( cond : in boolean ;
                   l:in system.address; c: in t_coverage; b:in natural)
  return boolean
is
begin
     flow_hit( l, c, b );
     return cond ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : count_hit( cond,address ) return boolean                        --
--                                                                           --
--  This function must return given value 'cond'.                            --
-------------------------------------------------------------------------------
function count_hit( cond : in boolean ; a_br:in system.address )
  return boolean
is
begin
     count_hit( a_br );
     return cond ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : inline_hit( cond, address ) return boolean                      --
--                                                                           --
--  This function must return TRUE.                                          --
-------------------------------------------------------------------------------
function inline_hit( cond : in boolean ; a_br:in system.address)
  return boolean
is
begin
     inline_hit(a_br);
     return cond ;
end ;



-------------------------------------------------------------------------------
-- SERVICE : flow_unhit( t_link'address, t_coverage, branch, old_value )     --
--                                                                           --
--    This service back to an old value of a branch                          --
--                                                                           --
-------------------------------------------------------------------------------
procedure flow_unhit( l:in system.address; c:in t_coverage; b:in natural; 
                 ov: in branch)
is
 plink : p_link ;
begin
     plink := a2l( l );

     case c is 
 
       when Proc
	  | Ret   => if plink.all.TP(b) > ov then plink.all.TP(b) := ov ; end if;

       when Block
	  | Implicit
	  | Logical
	  | Atc   => if plink.all.TB(b) > ov then plink.all.TB(b) := ov ; end if;

       when Call  => if plink.all.TA(b) > ov then plink.all.TA(b) := ov ; end if;

       when Cond  => if plink.all.TE(b) > ov then plink.all.TE(b) := ov ; end if;

     end case ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : count_unhit( t_link'address, t_coverage, branch, old_value )    --
--                                                                           --
--    This service back to an old value of a branch                          --
--                                                                           --
-------------------------------------------------------------------------------
procedure count_unhit( a_br:in system.address; ov: in branch )
is
 p_br : p_branch ;
begin
     p_br := a2branch( a_br );

     if p_br.all  > ov then
        p_br.all := ov ; 
     end if;
end ;


-------------------------------------------------------------------------------
-- SERVICE : flow_get_branch( t_link'address, t_coverage, branch )           --
--                                                                           --
--    This service back to an old value of a branch                          --
--                                                                           --
-------------------------------------------------------------------------------
function flow_get_branch( l:in system.address; c:in t_coverage; b:in natural )
         return branch
is
 plink : p_link ;
begin
     plink := a2l( l );

     case c is 
 
       when Proc
	  | Ret   => return plink.all.TP(b) ;

       when Block
	  | Implicit
	  | Logical
	  | Atc   => return plink.all.TB(b) ;

       when Call  => return plink.all.TA(b) ;

       when Cond  => return plink.all.TE(b) ;

     end case ;
end ;
-------------------------------------------------------------------------------
-- SERVICE : count_get_branch( t_link'address, t_coverage, branch )          --
--                                                                           --
--    This service back to an old value of a branch                          --
--                                                                           --
-------------------------------------------------------------------------------
function count_get_branch( a_br:in system.address )
         return branch
is
 p_br : p_branch ;
begin
     p_br := a2branch( a_br );
     return p_br.all ;
end ;


-------------------------------------------------------------------------------
-- SERVICE : flow_hitCALL( value,l,c,b) return private_type                  --
--                                                                           --
-- Used to set call branches in expressions                                  --
-------------------------------------------------------------------------------
function flow_hitCALL( value : in private_type   ;
                       adr_l : in system.address ;
                       c     : in t_coverage     ; 
                       b     : in natural        ) return private_type is
begin
     flow_hit( adr_l, c, b );
     return private_type'(value) ; --workaround for return-by-ref (LRM-6.5)
end ;           
-------------------------------------------------------------------------------
-- SERVICE : count_hitCALL( value, a_br ) return private_type                --
--                                                                           --
-- Used to set call branches in expressions                                  --
-------------------------------------------------------------------------------
function count_hitCALL( value : in private_type   ;
                        a_br  : in system.address ) return private_type is
begin
     count_hit( a_br );
     return private_type'(value) ; --workaround for return-by-ref (LRM-6.5)
end ;           
-------------------------------------------------------------------------------
-- SERVICE : inline_hitCALL( value, a_br ) return private_type               --
--                                                                           --
-- Used to set call branches in expressions.                                 --
-------------------------------------------------------------------------------
function inline_hitCALL( value : in private_type   ;
                         a_br  : in system.address ) return private_type is
 p_br : p_branch ;
begin
     p_br := a2branch( a_br );
     p_br.all := 1; -- inline is "pass" mode
     return private_type'(value) ; --workaround for return-by-ref (LRM-6.5)
end ;           
     

-------------------------------------------------------------------------------
-- SERVICE : hitCE( value,l,b ) return boolean                               --
--                                                                           --
-- Used to set elementary condition branches                                 --
-------------------------------------------------------------------------------
function hitCE( value : in boolean        ; 
		adr_l : in system.address ;
		b     : in natural        ) return boolean is

begin
     if value then
        flow_hit( adr_l, t_coverage'(Cond), b );
     else
        flow_hit( adr_l, t_coverage'(Cond), b+1 );
     end if ;
     return value ;
end ;

-------------------------------------------------------------------------------
-- SERVICE : compareElement( e1,e2 ) boolean                                 --
--                                                                           --
-- Comparison of to multiple condition vector element.                       --
-- Taking care of 'X' (2) value.                                             --
-------------------------------------------------------------------------------
function compareElement( e1, e2 : t_mcElement ) return boolean is
begin
     if ( ( e1 = 2 ) or else ( e2 = 2 ) ) then
        return true ;  -- always the same
     else
        return e1 = e2 ; -- must exactly the same
     end if ;
end ;

-------------------------------------------------------------------------------
-- SERVICE searchVector( n,m,v,index ) return boolean                        --
--                                                                           --
-- Search vector V in matrix M and return found state.                       --
-- If found, return the index of vector in matrix.                           -- 
-------------------------------------------------------------------------------
procedure searchVector( n_mvector : in  natural    ;
		        pm        : in  t_mcMatrix ;
		        pv        : in  p_mcVector ;
		        index     : out natural    ;
			found     : out boolean    ) is

  l_found : boolean ;

begin
     for i in 1..n_mvector
     loop
	 index := i;
         for j in 1..pv.all.n_element 
         loop
		-- try all vector element.
		l_found := compareElement( pm(i,j),
				         pv.all.vector(j)  ) ;
                -- exit then loop when we sure that's the good vector
                exit when not l_found ; 
         end loop ;

         -- stop search if vector is found.
         exit when l_found ;
     end loop ;

     found := l_found ;
end ;

-------------------------------------------------------------------------------
-- SERVICE : hitCC( l,pi,pv,b,value ) return boolean                         --
--                                                                           --
-- Used ti set multiple condition branch.                                    --
-------------------------------------------------------------------------------
function hitCC( adr_l : in system.address ;
		adr_i : in system.address ;
		adr_v : in system.address ;
		b     : in natural        ;
		value : in boolean        ) return boolean is

 found   : boolean ;
 v_index : integer := 0 ;

 pi : p_mcInfo   ;
 pv : p_mcVector ;

begin
     pv := a2mcVector( adr_v );
     pi := a2mcInfo( adr_i );

     -- search for vector V in pi and set corresponding branch.
     if ( value )
     then
        -- searching in true matrix
	searchVector( pi.all.n_ctrue, pi.all.ctrue, pv,
		      v_index, found );
     else
        -- searching in false vectors
        searchVector( pi.all.n_cfalse, pi.all.cfalse, pv,
		      v_index, found );
     end if ;

     -- now vector must be found, else it's an internal error.
     if ( found )
     then
        -- vector found : set corresponding branch
        if ( value ) then
           flow_hit( adr_l, t_coverage'(Cond), b+v_index-1 );
        else
           flow_hit( adr_l, t_coverage'(Cond), b+pi.all.n_ctrue+v_index-1 );
        end if ;

     else
        -- internal error detected.

        -- perhaps this vector stand in the other value matrix ?
        if ( value )
        then
           -- searching in true matrix
	   searchVector( pi.all.n_cfalse, pi.all.cfalse, pv,
			 v_index, found );
        else
           -- searching in false vectors
           searchVector( pi.all.n_ctrue, pi.all.ctrue, pv,
			 v_index, found );
        end if ;

        -- print different messages
        if ( found )
        then
            -- vector is bad evaluated : set flag
            internal_error( EI_MCV_BAD_EVAL ) := true ;
	else
            -- vector not found in both matrix : set flag
	    internal_error( EI_MCV_NOT_FOUND ):= true ;
	end if ;
    end if ;

    -- do not forget this :
    return value ;
end ;

-------------------------------------------------------------------------------
-- SERVICE : hitBC( l,pi,pv,b,value ) return boolean                         --
--                                                                           --
-- Used to set multiple condition branch for binary conditions.              --
-------------------------------------------------------------------------------
function hitBC( adr_l : in system.address ;
		adr_v : in system.address ;
		b     : in natural    ;
		value : in boolean ) return boolean is

 pv : p_mcVector ;
 br_vector : integer := 0;
 power2 : natural :=0;
begin
     pv := a2mcVector( adr_v );

     power2 :=1;
     -- compute vector branch ID
     for j in 1..pv.all.n_element 
     loop
        case pv.all.vector(j) is
          when 0 => null;
          when 1 => br_vector := br_vector + power2 ;
	  when 2 => -- this case must not appears
                    internal_error( EI_MCV_BAD_BC_VECTOR ) := true ;
        end case ;
      power2 := power2 + power2;
     end loop;

     flow_hit( adr_l, t_coverage'(Cond), b+br_vector );
     
     -- do not forget this :
     return value ;
end ;


-------------------------------------------------------------------------------
-- SERVICE mcDefault( v ) return boolean                                     --
--                                                                           --
-- Default initialisation of multiple condition matrix                       --
-- Always return true (instrumentation requirement)                          --
-------------------------------------------------------------------------------
function mcDefault( adr_v : in system.address ) return boolean is

  pv : p_mcVector ;
begin
     pv := a2mcVector( adr_v );

     for i in pv.all.vector'range
     loop
          pv.all.vector( i ) := 2 ; -- undetermined value.
     end loop ;

     return true ; -- always
end ;

-------------------------------------------------------------------------------
-- SERVICE mcVSet( v,index,value ) return boolean                            --
--                                                                           --
-- Set given element of multiple condition vector.                           --
-------------------------------------------------------------------------------
function mcVSet( adr_v : in system.address ;
		 index : in natural        ;
                 value : in boolean        ) return boolean is

  pv : p_mcVector ;

begin
     pv := a2mcVector( adr_v );

     if ( value ) 
     then
         pv.all.vector( index ) := 1 ;
     else
         pv.all.vector( index ) := 0 ;
     end if ;
    
     return value ;
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : dumpInternalError( file)                               --
--                                                                           --
-- Print given error messages                                                --
-------------------------------------------------------------------------------
procedure dumpInternalErrors( file : in out private_io.priv_file ) is
begin
     for i in t_InternalError 
     loop
         if internal_error( i ) 
         then
             -- print 
	     case i is
	       when EI_MCV_BAD_EVAL      => buf_print( "EI 49" );
	       when EI_MCV_NOT_FOUND     => buf_print( "EI 50" );
               when EI_MCV_BAD_BC_VECTOR => buf_print( "EI 51" );
	     end case ;
	     buf_flush( file );
             -- reset
             internal_error( i ) := false ;
         end if ;
     end loop ;
end ;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : dump_test_header( file )                               --
--                                                                           --
--   Dump test header .                                                      --
--                                                                           --
-------------------------------------------------------------------------------
procedure dump_test_header( file : in out private_io.priv_file ) is
begin
---------------------------------------------------------------
-- For Attol UniTest mode : dump the name of the service/test
---------------------------------------------------------------
	  if ATU_header 
          then -- Full NT header
              buf_print( "NT """ );
	      buf_print( NT_test_name.str(1..NT_test_name.len));
	      buf_print( """ " );
	      buf_print( test_number );
	      buf_print( ' ');
	      buf_print( service_number );
	      buf_flush( file );
          else
              -- get environment variable test name 
	      buf_print( "NT """"" );
	      buf_flush( file );
          end if ;
---------------------------------------------------------------
-- End od UniTest mode adaptation
---------------------------------------------------------------
      buf_print( "DT """"" );
      buf_flush( file ) ;
      if ( count_mode ) then
	 buf_print( "CO" );
	 buf_flush( file ) ;
      end if ;
end dump_test_header ;     
     

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : dump_link_header( file, link )                         --
--                                                                           --
--   Dump link header (FT / DC ).                                            --
--                                                                           --
-------------------------------------------------------------------------------
procedure dump_link_header( file : in out private_io.priv_file; link : in p_link ) is
begin
      buf_print( "FT " );
      buf_print( link.key );
      buf_flush( file ) ;
      buf_print( "DC " );
      buf_print( link.checksum );
      buf_flush( file ) ;
end dump_link_header ;


-------------------------------------------------------------------------------
-- INTERNAL SERVICE : dump( file, string, branches )                         --
--                                                                           --
--   Dump given branches, prepend to each line given prefix                  --
--                                                                           --
-------------------------------------------------------------------------------
procedure dump( file    : in out private_io.priv_file; 
                prefix  : in string; 
                b       : in branches      ) is
begin
     if ( count_mode ) then
        for i in b'range loop
            if ( b(i) /= 0 ) then -- dump branch line only if it was hit
                   buf_print( prefix );
		   buf_print( ' ' );
		   buf_print( i );
		   buf_print( ' ' );
		   buf_print( b(i) );
		   buf_flush( file ) ;
            end if ;
        end loop;                   
     else -- compact mode
        for i in b'range loop
            if ( b(i) /= 0 ) then -- dump only if branch was hit
		   buf_print( prefix );
		   buf_print( ' ' );
		   buf_print( i );
	   	   buf_print( ' ' );
		   buf_flush( file );
            end if ;
        end loop;                   
     end if ;
end;


-------------------------------------------------------------------------------
-- SERVICE : dump                                                            --
--                                                                           --
--   Dump all hitten branch listed in repository                             --
--                                                                           --
-------------------------------------------------------------------------------
procedure dump is
    iterator  : p_link := header ;
    file      : private_io.priv_file ;
    -- dump C traces
    ret : integer;
begin
      -- force to dump FT/DC informations
      private_io.priv_open ( file, attol_cfg.atc_TIO_file_name );
      dump( file );
      private_io.priv_close( file );

      -- dump C traces
      -- ret:=C_atc_Dump;
end dump;
  
--------------------------------------------------------------------
-- RUNTIME_INITIALIZATION()                                       --
--                                                                --
--   This procedure is used to initialize runtime in mode.        --
--------------------------------------------------------------------
procedure runtime_initialization is
begin
     resetInternalErrors ;
     do_runtime_initialization := false;
end;

-------------------------------------------------------------------------
-- Specific services for Attol UniTest mode functionnalities
-------------------------------------------------------------------------

  -----------------------------------------------------------------------
  -- ASSIGN_NAME   (Internal)
  -- Make an t_name with a STRING
  -----------------------------------------------------------------------
  PROCEDURE assign_name( name : in out t_name; str : in STRING ) IS
    str_l : integer ;
  BEGIN
       str_l := str'LAST-str'FIRST+1 ;
       IF ( str_l > MAX_NAME_LENGTH )
       THEN
           -- Sorry this string is too wide : truncate it!
	   name.str(1..MAX_NAME_LENGTH ) := 
                str( str'FIRST..str'FIRST+MAX_NAME_LENGTH+1 ) ;
	   name.len:= MAX_NAME_LENGTH ;
       ELSE
	   name.str(1..str_l) := str ;
	   name.len:= str_l ;
       END IF ;
  END assign_name ;


  -----------------------------------------------------------------------
  -- REGISTER_SERVICE
  -- This procedure Register a new service with it number and name
  --
  -----------------------------------------------------------------------
  PROCEDURE register_service(  number : IN integer ; name   : IN STRING ) IS
  BEGIN
	service_number := number ;
	assign_name( service_name, name );
	ATU_header := TRUE ;
  END register_service ;
 

  -----------------------------------------------------------------------
  -- REGISTER_TEST
  -- This procedure Register a new test with :
  --
  -----------------------------------------------------------------------
  PROCEDURE register_test( number : IN integer; name   : IN STRING ) IS
    free_l : integer ;
    cpy_l  : integer ;
  BEGIN
	test_number := number ;
	assign_name( test_name, name );
	-- concatenate service and test name
        NT_test_name := service_name ;
	if NT_test_name.len< MAX_NAME_LENGTH 
	then
	   NT_test_name.len:= NT_test_name.len+1 ;
	   NT_test_name.str( NT_test_name.len) := '/' ;
        end if ;
	if NT_test_name.len < MAX_NAME_LENGTH 
	then
            free_l := NT_test_name.str'LAST-NT_test_name.len+1 ;
	    if free_l > test_name.len
	    then
		cpy_l := test_name.len;
	    else
		cpy_l := free_l ;
	    end if ;
	    NT_test_name.str( NT_test_name.len+1.. NT_test_name.len+cpy_l )
		:= test_name.str( 1..cpy_l );
	    NT_test_name.len := NT_test_name.len + cpy_l ;
	end if ;
  END register_test ;


  -----------------------------------------------------------------------
  -- UNREGISTER
  -- This procedure remove all registered service/test
  --
  -----------------------------------------------------------------------
  PROCEDURE unregister IS
  BEGIN
	service_number  := 0;
	service_name.len:= 0;
	test_number     := 0;
	test_name.len   := 0;
	ATU_header := FALSE ;
  END unregister ;

  --------------------------------------------------------------------
  -- dump():                                                        --
  --                                                                --
  --   This procedure dump repository traces using given file desc. --
  --------------------------------------------------------------------
  procedure dump( file : in out private_io.priv_file ) is
      iterator : p_link := header ;
  begin
        -- force to dump FT/DC informations
        dumpInternalErrors( file );
        dump_test_header( file );
        while ( iterator /= NULL ) loop
             dump_link_header( file, iterator );
             dump( file, "TP", iterator.TP );
             dump( file, "TB", iterator.TB );
             dump( file, "TA", iterator.TA );
             dump( file, "TE", iterator.TE );
             -- reset traces
	     reset_traces( iterator );
             -- next link
             iterator := iterator.next;
        end loop;
  end dump;

-------------------------------------------------------------------------------
-- INTERNAL SERVICE : reset                                                  --
--                                                                           --
--    Reset all traces tables.                                               --
--                                                                           --
-------------------------------------------------------------------------------
  procedure reset is
      iterator : p_link := header ;
  begin
       while ( iterator /= NULL ) loop
           -- reset traces
	     reset_traces( iterator ) ;
	     -- next link
             iterator := iterator.next;
       end loop;
  end reset;

------------------------------------------------------------------------
-- end of UniTest mode adaptation
------------------------------------------------------------------------

--INIT_UNSUPPORTED: comment 2 lines below:
begin
    runtime_initialization ;
end attolcov ;
--                                                                    --
------------------------------------------------------------------------
