
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

-------------------------------------------------------------------------------
--  $Revision: 1.11.2.1 $ 
--  $Descr: Coverage for Ada83 TDP$
-------------------------------------------------------------------------------
-- NOTE :
--    --? comments may be used to optimize this runtime
--
--    If STORAGE_ERROR is raised at one of the 'Storage_Size use lines,
--    comment this lines.  This representation clause means that no 
--    allocation of this type will be made.  Perhaps it's not supported
--    by your compiler (see LRM Ada95 chapter 13.11)
-------------------------------------------------------------------------------



-------------------------------------------------------------------------------
-- ATTOLCOV Package Specification   
--
-------------------------------------------------------------------------------


with system; use system;
with unchecked_conversion;
with private_io ; 
with attol_cfg ;

package attolcov is

--INIT_UNSUPPORTED: uncomment 1 line below:
--pragma preelaborate;

  -- max count of a branch. Expr. is 2^(atc_nb_bit_branch)-1  
  size_branch   : constant integer 
                  :=     2**(attol_cfg.atc_nb_bit_branch-1)
                     + ( 2**(attol_cfg.atc_nb_bit_branch-1) - 1 );

  -- available range for a branch
  type branch is new integer range 0..size_branch;
  --?  for  branch'size use attol_cfg.atc_nb_bit_branch ;

  type p_branch is access branch;
   -- Mean that no allocation will be made for this type.
   for p_branch'Storage_Size use 0;

  -- This flag is set to TRUE for count mode.
  count_mode : constant boolean := (attol_cfg.atc_nb_bit_branch > 1) ;
  
  
  -- unconstraint array of branch :
  type branches is array( natural range <> ) of branch;
  type p_branches is access branches ;
  --? pragma pack( branches );
  -- Beware using pack and for .. use in same time (may produce an exception).

   -- Mean that no allocation will be made for this type.
   for p_branches'Storage_Size use 0;

  -- All available coverage type (last mode coverage is provides
  --  for compatibility only ).
  type t_coverage is ( Proc, Ret, Block, Implicit, Logical, Atc, Call, Cond );

  -- t_link is the branch repository type.
  -- It can be use in a double chained list.
  type t_link (
               TB_first, TB_last, 
               TP_first, TP_last,
               TA_first, TA_last,
               TE_first, TE_last : natural 
             ) ;
  type p_link is access t_link;
   -- Mean that no allocation will be made for this type.
   for p_link'Storage_Size use 0;
  
  -- NB : 32 bits integer generated by instrumentation, are manipulated
  --      in their string form ( " 42360456"), this string is 10 characteres
  -- width.
  subtype string32b is  string(1..10) ; -- "          ".."4294967295"
   
  -- branch repository now expanded :
  type t_link( 
               TB_first, TB_last, 
               TP_first, TP_last,
               TA_first, TA_last,
               TE_first, TE_last : natural 
             ) is record
      key,checksum : string32b; 
      TB        : branches( TB_first..TB_last );
      TP        : branches( TP_first..TP_last );
      TA        : branches( TA_first..TA_last );
      TE        : branches( TE_first..TE_last );
      next      : p_link                       ;
      prev      : p_link                       ;
    end record;

  -- Multiple condition Vector, Matrix and Information types.
  type t_mcElement is new integer range 0..2 ;
  for  t_mcElement'size use 3 ; -- pack this datas.

  type t_mcArray is array ( natural range <> ) of t_mcElement ;

  type t_mcVector( n_element : natural ) is
       record
         vector : t_mcArray( 1..n_element );
       end record ;

  type t_mcMatrix is array ( natural range <>, natural range <> )
                        of t_mcElement;

  type t_mcInfo( n_ctrue, n_cfalse, n_element : natural ) is
       record
          ctrue  : t_mcMatrix( 1..n_ctrue,  1..n_element );
          cfalse : t_mcMatrix( 1..n_cfalse, 1..n_element );
       end record ;

  type p_mcVector is access t_mcVector ;
   -- Mean that no allocation will be made for this type.
   for p_mcVector'Storage_Size use 0;

  type p_mcInfo   is access t_mcInfo   ;
   -- Mean that no allocation will be made for this type.
   for p_mcInfo'Storage_Size use 0;
  

  -----------------------------------
  -- mode dependent functionnalities

  -- mode FLOW
  procedure flow_hit( l:in system.address; c:in t_coverage; b:in natural );
  function  flow_hit( l:in system.address; c:in t_coverage; b:in natural )
            return boolean ;
  function  flow_hit( cond:in boolean;
                      l:in system.address; c:in t_coverage; b:in natural )
            return boolean ;
  function  flow_get_branch( l:in system.address; c:in t_coverage; b:in natural )
            return branch ;
  procedure flow_unhit( l:in system.address; c:in t_coverage; b:in natural;
                   ov :in branch ); 

  generic
    type private_type is limited private;
  function  flow_hitCALL( value :in private_type  ;
                          adr_l :in system.address;
                          c     :in t_coverage    ; 
                          b     :in natural       )
            return private_type;

  -- mode COUNT 
  procedure count_hit( a_br :in system.address );
  function  count_hit( a_br :in system.address )
            return boolean ;
  function  count_hit( cond :in boolean; a_br:in system.address )
            return boolean ;
  function  count_get_branch( a_br:in system.address )
            return branch ;
  procedure count_unhit( a_br:in system.address; ov :in branch ); 

  generic
    type private_type is limited private;
  function  count_hitCALL( value : in private_type  ;
                           a_br  : in system.address)
            return private_type;

  -- mode INLINE
  procedure inline_hit( a_br :in system.address );
  function  inline_hit( a_br :in system.address )
            return boolean ;
  function  inline_hit( cond :in boolean; a_br:in system.address )
            return boolean ;

  generic
    type private_type is limited private;
  function  inline_hitCALL( value : in private_type  ;
                            a_br  : in system.address)
            return private_type ;


  ---------------------------------------
  -- function without mode dependencies

  -- Add adr_link in package's list of branch repository.
  procedure link_desc ( adr_link : in address; key,checksum: in string32b );

  -- set condition branches
  function hitCE( value : in boolean; adr_l: in system.address; b : in natural )
    return boolean ;

  -- default initialisation of multiple condition Vector.
  function mcDefault( adr_v : in system.address ) return boolean ;

  -- set multiple condition branch
  function hitCC( adr_l : in system.address ;
		  adr_i : in system.address ;
		  adr_v : in system.address ;
		  b     : in natural    ;
		  value : in boolean ) return boolean ;

  -- set multiple condition branch for binary conditions
  function hitBC( adr_l : in system.address ;
		  adr_v : in system.address ;
		  b     : in natural    ;
		  value : in boolean ) return boolean ;

  -- set Element of given multiple condition vector
  function mcVSet( adr_v : in system.address ;
	           index : in natural    ;
		   value : in boolean    ) return boolean ;



  -- A call of dump procedure, dump all listed repositories branch in traces file,
  -- then theses branch are reset.
  procedure dump ;

  -- reset all momerized repositories branch
  procedure reset;

-----------------------------------------------------------------------------------
-- services for Attol UniTest mode functionnalities
-----------------------------------------------------------------------------------
 
  -- Declare a new service start with their number and name
  procedure register_service( number : in integer; name : in STRING );

  -- Declare a new test start with their number and name
  procedure register_test( number : in integer; name : in STRING );

  -- undeclare current service 
  procedure unregister ;

  -- dump traces in given file 
  procedure dump( file : in out private_io.priv_file );

  -- dump C traces
  -- function C_atc_Dump return integer;
  -- pragma Import ( C, C_atc_Dump, "atc_dump_driver" );

-----------------------------------------------------------------------------------
-- end of Attol UniTest mode functionnalities
-----------------------------------------------------------------------------------
  
end attolcov;

--
-----------------------------------------------------------------------------------

