
------------------------------------------------------------------------
--
--  Licensed Materials - Property of IBM
--  Restricted Material of IBM
--  IBM Rational Test RealTime
--  (c) Copyright IBM Corp. 2003-2005 All Rights Reserved.
--
--  US Governement Users Restricted Rights - Use, duplication or
--  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
------------------------------------------------------------------------

-- Product version: 7.0.5.1

-------------------------------------------------------------------------------
-- $Revision: 1.13.4.2 $
-- $Descr: UniTesting part of TDP for Ada$
-------------------------------------------------------------------------------


with attol_serv ;
with attol_comm ;
with attol_link ;

use attol_serv;
use attol_comm;

package body attol is

  Mode_nonil            : constant boolean      := false                ;
  Mode_Active_tab       : constant boolean      := true                ;

  subtype T_STRING    is STRING (1 .. 12);
  subtype intervale   is integer range 1 .. 3000;
  type    t_array_idx is array(attol_int range 1..60) of attol_int;

  array_idx             : t_array_idx ;
  curr_array_idx        : attol_int range 0..60 :=0;

  local_posit           : intervale             := intervale'first      ;
  num_var_loc           : priv_int              := 0                    ;
  level_courant         : priv_int              := 1024                 ;
  cpu_tim               : priv_int              := 0                    ;
  Prof_Str              : Priv_Int              := 0                    ;
  Prof_Array            : Priv_Int              := 0                    ;

  procedure opf_ada(Name: in String;Key: in string) is
    date_sys:string(1..50);
    lg_date:integer;
  begin
    mode_tab            := false        ;
    resultat_global     := true         ;
    num_var_loc         := 0            ;
    local_posit         := intervale'first      ;
    priv_open(rio,Name(Name'first.. Name'last-1));
    put("E97 ");
    Put(Key);
    put_line(rio);
    priv_date(date_sys,lg_date);
    if (lg_date/=0) then
      put("H0 ");
      put(date_sys(1..lg_date));
      put_line(rio);
    end if;
  end opf_ada;

  procedure dcl_op_ada(numero:in attol_int; Name:in string) is
  begin
    mode_tab            := false        ;
    mode_cst            := 0            ;
    resultat_global     := true         ;
    num_var_loc         := 0            ;
    put('O');
    put_dec_image(longest_integer(numero));
    put_line(rio);
    -- Mode Coverage :
    attol_link.register_service( numero, Name );
  end;

  procedure dcl_tst_ada(numero:in attol_int;Name: in string) is
  begin
    mode_tab            := false        ;
    mode_cst            := 0            ;
    resultat_global     := true         ;
    num_var_loc         := 0            ;
    put('T');
    put_dec_image((longest_integer(numero)));
    put_line(rio);
    -- Mode Coverage :
    attol_link.register_test( numero, Name );
  end;

  procedure clf_ada is
  begin
    Put("E00");
    put_line(rio);
    priv_close(rio);
    -- Mode coverage
    attol_link.unregister ;
  end clf_ada;

  procedure start_chrono_ada is
  begin
    cpu_tim:=priv_clock;
  end start_chrono_ada;

  procedure stop_chrono_ada  is
    local_tim:priv_int;
  begin
    if (num_muet/=0) then
      edv_chrono_muet_ada(0,0,0);
    end if;
    local_tim:=priv_clock;
    if (local_tim>=0) then
      put(clock_unit);
      put_dec_image(longest_integer((((local_tim-cpu_tim+(clock_divide/2))*clock_multiply)/clock_divide)-clock_offset));
      put_line(rio);
    end if;
    -- Mode Coverage
    attol_link.out_cov( rio ) ;
  end stop_chrono_ada ;

  procedure excep_ada(num: in attol_int;param: in string:= "") is
  begin
    put('W');
    put_dec_image((longest_integer(num)));
    put(' ');
    put(param);
    put_line(rio);
  end excep_ada ;

-- help writing in rio file

  procedure Put_Var_Line_Beginning(Number : in attol_int) is
  begin
    for i in attol_int range Array_Idx'First..Curr_Array_Idx loop
      put_dec_image(longest_integer(array_idx(i)));
      Put('|');
    end loop; -- for i
    put_dec_image(longest_integer(Number));
    put(' ');
  end; -- procedure Put_Var_Line_Beginning


  procedure Put_Integer_In_Rio(Valeur : in Attol_big_Int; base : in Attol_Int) is
  begin
    if (base = 16) then
      put_hex_image(valeur);
    else
      if (base = 2) then
        put_bin_image(valeur);
      else
        put_dec_image(valeur);
      end if;
    end if;
  End; -- procedure Put_Integer_In_Rio;

  procedure Put_Char_In_Rio(Valeur : in Character) is
  begin
    put_dec_image (character'pos(valeur));
  end; -- procedure Put_Char_In_Rio;

  procedure Put_Boolean_In_Rio(Valeur : in Boolean) is
  begin
    if Valeur then
      put("TRUE");
    else
      put("FALSE");
    end if;
  end; -- procedure Put_Boolean_In_Rio;

  procedure Put_String_In_Rio(Valeur : in String) is
  begin
    for i in Valeur'first .. Valeur'last loop
      put(Valeur(i));
      if valeur(i) ='"' then
        put('"');
      end if;
    end loop;
  end; -- procedure Put_String_In_Rio;

  procedure Put_Address_In_Rio(Valeur : in access_proautan) is
    function conv_int is new unchecked_conversion(access_proautan,int_size_access);
  begin
    if (valeur=null) then
      put("NULL");
    else
      if ((Mode_Nonil) or else (Valeur = Cst_Nonil)) then
        put("NONIL");
      else
        put_hex_image(longest_integer(conv_int(valeur)));
      end if;
    end if;
  end; -- procedure Put_Address_In_Rio;

-- beginning of level memorization

  procedure Set_Multi_Test(Num_Ins:in Attol_Int;Num_Loop:in Attol_Int;Idx_Loop:in Attol_Int) is
  begin
    put("R");
    put_dec_image(longest_integer(Num_Ins));
    put(" ");
    put_dec_image(longest_integer(Num_Loop));
    put(" ");
    put_dec_image(longest_integer(Idx_Loop));
    Put_Line(Rio);
  end;

  procedure attol_bcl(numero:in attol_int;indice:in attol_int) is
  begin
    array_idx(Curr_Array_Idx + 2 -Numero):=indice;
  end;

  procedure set_mode_tab_ada(numero:in Attol_Int;Nbdim:in Attol_int) is
  begin
    curr_array_idx:=Curr_Array_Idx+1;
    Array_Idx(Curr_Array_Idx):=Numero;
    curr_array_idx:=Curr_Array_Idx+Nbdim-1;
    mode_tab:=Mode_Active_tab;
    Prof_Array := Prof_Array +1;
    if (Prof_Array = 1)
      then
        resultat_global:=True;
    end if;
  end set_mode_tab_ada;

  procedure unset_mode_tab_ada(numero:in Attol_Int;Nbdim:in Attol_Int) is
  begin
    Prof_Array := Prof_Array -1;
    if (Prof_Array=0) then
      mode_tab:=False;
    end if;
    curr_array_idx:=(Curr_Array_Idx-Nbdim);
    if ((not Mode_Tab) and (Numero > 0)) then
      put('A');
      Put_Var_Line_Beginning(numero);
      if (resultat_global) then
        put(" OK RA=T");
      else -- not (resultat_global)
        put(" BAD RA=F");
      end if ; -- if (resultat_global)
      put_line(rio);
    end if;
  end unset_mode_tab_ada;

  procedure set_mode_str_ada(numero:in attol_int) is
  begin
    curr_array_idx := curr_array_idx +1;
    array_idx(Curr_Array_Idx):= Numero;
  end set_mode_str_ada;

  procedure unset_mode_str_ada(numero:in Attol_Int) is
  begin
    Curr_Array_Idx := Curr_Array_Idx -1;
  end unset_mode_str_ada;

-- end of level memorization

-- edvs

  -- integer
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      attol_big_int       ;
                    numero      :       in      attol_int       ;
                    min         :       in      attol_big_int       ;
                    max         :       in      attol_big_int       ;
                    base        :       in      attol_int:=10   ;
                          valid     :     in    ModeValid := StdValue) is
  begin
    if ((valid/=StdValue) or (not mode_tab) Or ((CheckMode = Check) and (Valeur not in Min .. max))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      Put_Integer_In_Rio(Valeur,base);
      if ((Checkmode = UnCheck) or ((Valeur in Min .. max)and then (valid=StdValue))) then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        Put_Integer_In_Rio(Min,base);
        put(" MA=");
        Put_Integer_In_Rio(Max,base);
      end if;
      put_line(rio);
    end if;
  end; -- edv_ada integer

  -- Compares three strings
  function comp3str(min:string; val:string; max:string) return boolean is
  -- return true if min<= val and val <= max
  type t_etat is (unknow, strictly_great_min , strictly_less_max ,out_range, in_range);
  etat : t_etat;
  i_val,i_min,i_max:integer;
  begin
    etat := unknow;
    i_val :=val'first;
    i_min :=min'first;
    i_max :=max'first;
    while (etat < out_range)
    loop
      if (i_val<=val'last)
        then
          if ((etat /= strictly_great_min) and then (i_min <= min'last))
            then
              if (min(i_min)<val(i_val))
                then
                  if (etat = unknow)
                    then
                      etat := strictly_great_min;
                    else
                      etat := in_range;
                    end if;
                else
                  if (min(i_min)>val(i_val))
                    then
                      etat := out_range;
                    end if;
                end if;
            else
              etat := strictly_great_min;
            end if;
          if (etat < strictly_less_max)
            then
              if (i_max <= max'last)
                then
                  if (max(i_max)>val(i_val))
                    then
                      if (etat = unknow)
                        then
                          etat := strictly_less_max;
                        else
                          etat := in_range;
                        end if;
                    else
                      if (max(i_max)<val(i_val))
                        then
                          etat := out_range;
                        end if;
                    end if;
                else
                  etat := out_range;
                end if;
            end if;
          i_val := i_val+1;
          i_min := i_min+1;
          i_max := i_max+1;
        else
          if ((etat = strictly_great_min) or else (i_min   > min'last))
            then
              etat := in_range;
            else
              etat := out_range;
            end if;
        end if;
    end loop;

    return (etat=in_range);
  end;

  -- string
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      string          ;
                    numero      :       in      attol_int       ;
                    min         :       in      string          ;
                    max         :       in      string          ;
                          valid     :     in    ModeValid := StdValue) is
   valOk:boolean;
 begin
    valOk:=Comp3str (min, Valeur, Max) ;
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (not valOk))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      put_string(valeur);
      if ((Checkmode = UnCheck) or (valOk and then (valid=StdValue))) then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        put_string(min);
        put(" MA=");
        put_string(max);
      end if;
      put_line(rio);
    end if; -- mode_tab
  end edv_ada; -- string

  -- character
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      character       ;
                    numero      :       in      attol_int       ;
                    min         :       in      character       ;
                    max         :       in      character       ;
                          valid     :     in    ModeValid := StdValue) is
  begin
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (Valeur not in Min .. max))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      Put_Char_In_Rio(Valeur);
      if ((CheckMode = UnCheck) or ((Valeur in Min .. max)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
     end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        Put_Char_In_Rio(Min);
        put(" MA=");
        Put_Char_In_Rio(Max);
      end if;
      put_line(rio);
    end if; -- mode_tab
  end edv_ada; -- character

  -- boolean
  procedure edv_ada(CheckMode   :       in      modecheck       ;
                    valeur      :       in      boolean         ;
                    numero      :       in      attol_int       ;
                    min         :       in      boolean         ;
                    max         :       in      boolean         ;
                          valid     :     in    ModeValid := StdValue) is
  begin
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (Valeur not in Min .. max))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      Put_Boolean_In_Rio(Valeur);
      if ((CheckMode = UnCheck) or ((Valeur in Min .. max)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        Put_Boolean_In_Rio(Min);
        put(" MA=");
        Put_Boolean_In_Rio(Max);
      end if;
      put_line(rio);
    end if; -- mode_tab
  end Edv_ada; -- Boolean

  -- enum 1
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      string          ;
                    numero      :       in      attol_int       ;
                    min         :       in      string          ;
                    max         :       in      string          ;
                    ok          :       in      boolean         ;
                          valid     :     in    ModeValid := StdValue) is

  begin
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (not ok))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      put(valeur);
      if ((CheckMode = UnCheck) or ((ok)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        Put_String_In_Rio(Min);
        put(" MA=");
        Put_String_In_Rio(Max);
      end if;
      put_line(rio);
    end if; -- mode_tab
  end edv_ada; -- Enum 1

  -- Enum 2
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      attol_big_int       ;
                    numero      :       in      attol_int       ;
                    min         :       in      attol_big_int       ;
                    max         :       in      attol_big_int       ;
                    ok          :       in      boolean         ;
                          valid     :     in    ModeValid := StdValue) is
  begin
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (not ok))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      put_dec_image(longest_integer(valeur));
      if ((CheckMode = UnCheck) or ((ok)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        put_dec_image(longest_integer(min));
        put(" MA=");
        put_dec_image(longest_integer(max));
      end if;
      put_line(rio);
    end if; -- mode_tab
  end edv_ada; -- Enum 2

  -- float
  procedure edv_ada(CheckMode   :       in      ModeCheck       ;
                    valeur      :       in      attol_float     ;
                    numero      :       in      attol_int       ;
                    min         :       in      attol_float     ;
                    max         :       in      attol_float     ;
                    ok          :       in      boolean         ;
                          valid     :     in    ModeValid := StdValue) is
  begin
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (not ok))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      put_float_image(priv_float(valeur));
      if ((CheckMode = UnCheck) or ((ok)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        put(" MI=");
        put_float_image(priv_float(min));
        put(" MA=");
        put_float_image(priv_float(max));
      end if;
      put_line(rio);
    end if; -- mode_tab
  end edv_ada; -- float

  -- access
  procedure edv_ada(CheckMode   :       in      ModeCheck         ;
                    valeur      :       in      access_proautan   ;
                    numero      :       in      attol_int         ;
                    va_val      :       in      access_proautan   ;
                          valid     :     in    ModeValid := StdValue)  is
    Ok : Boolean;
  begin
    if ((valeur=va_val) or ((Valeur /= null) and (va_val = Cst_Nonil))) then
      Ok := True;
    else
      Ok := False;
    end if;
    if ((valid/=StdValue) or (not mode_tab) or ((CheckMode = Check) and (not ok))) then
      put('V');
      Put_Var_Line_Beginning(numero);
      Put_Address_In_Rio(Valeur);
      if ((CheckMode = UnCheck) or ((ok)and then (valid=StdValue))) Then
        put(" RA=T");
      else
        put(" RA=F");
        resultat_global:=false;
      end if;
      if ((CheckMode = Check)and then (valid = StdValue)) then
        Put(" MI=");
        Put_Address_In_Rio(Va_Val);
        Put(" MA=");
        Put_Address_In_Rio(Va_Val);
      end if;
      put_line(rio);
    end if; -- mode_tab
  end; -- access

-- end edv function

-- except dealing
  procedure erv_ada(numero:in attol_int) is
  begin
    put("W4");
    put_line(rio);
    put('V');
    Put_Var_Line_Beginning(Numero);
    put(" ? RA=F");
    put_line(rio);
    resultat_global:=false;
  end erv_ada;

--
  procedure dcl_fac_rep_ada(numero:in attol_int;facteur:in attol_int) is
  begin
    put('R');
    put_dec_image(longest_integer(numero));
    put(' ');
    put_dec_image(longest_integer(facteur));
    put_line(rio);
  end dcl_fac_rep_ada;

-- ptu function

  -- Integer and enum 2
  procedure ptu_ada(valeur      :       in      attol_big_int       ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      Put_Integer_In_Rio(Valeur,base);
      put_line(rio);
    end if;
  end Ptu_Ada;

  -- string and enum 1
  procedure ptu_ada(valeur      :       in      string          ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      if base=0 then
        put_enum(valeur);
      else
        put_string(valeur);
      end if;
      put_line(rio);
    end if;
  end ptu_ada;


  -- character
  procedure ptu_ada(valeur      :       in      character       ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      Put_Char_In_Rio(Valeur);
      put_line(rio);
    end if;
 end ptu_ada;


  -- boolean
  procedure ptu_ada(valeur      :       in      boolean         ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      Put_Boolean_In_Rio(Valeur);
      put_line(rio);
    end if;
  end ptu_ada;

  -- real
  procedure ptu_ada(valeur      :       in      attol_float     ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      put_float_image(priv_float(valeur));
      put_line(rio);
    end if;
  end ptu_ada;

  -- access
  procedure ptu_ada(valeur      :       in      Access_Proautan ;
                    numero      :       in      attol_int       ;
                    base        :       in      attol_int:=10   ) is
  begin
    if (not Mode_Tab) then
      put('N');
      Put_Var_Line_Beginning(numero);
      Put_Address_In_Rio(Valeur);
      put_line(rio);
    end if;
  end ptu_ada;

-- end ptu function

  procedure edv_chrono_muet_ada(muet,status,num_app:attol_int) is
  begin
    if (num_muet/=0) then
      if ((muet/=num_muet)or(res_muet/=status)) then
        put('L');
        put_dec_image(longest_integer(num_muet));
        put(' ');
        if (res_muet/=0) then
          put('N');
        else
          put('O');
        end if;
        put(' ');
        put_dec_image(longest_integer(min_muet));
        put(' ');
        put_dec_image(longest_integer(max_muet));
        put_line(rio);
        num_muet := muet;
        res_muet := status;
        min_muet := num_app;
        max_muet := num_app;
      else
        max_muet := num_app;
      end if;
    else
       num_muet := muet;
       res_muet := status;
       min_muet := num_app;
       max_muet := num_app;
    end if;
  end edv_chrono_muet_ada;

  procedure EDV_MUET_CHECK(numero:in attol_int;indice:in attol_int) is
  begin
    null;
    put('M');
    put_dec_image(longest_integer(numero));
    put(' ');
    put_dec_image(longest_integer(indice));
    put_line(rio);
  end EDV_MUET_CHECK;

  procedure Searchpos(index    : in Attol.Attol_Int     ;
                      tabrange : in Attol.Stub_Range    ;
                      Error    : in out Attol.Attol_Int ;
                      pos      : in out Attol.Attol_Int ) is
    last : Attol.Attol_int := -1;
  begin
    pos := -1;
    error := 0;
    Search : for i in Attol.Attol_int range tabrange'first..tabrange'last loop
      if ((index in tabrange(i).min..tabrange(i).max) or else ((tabrange(i).min = 1) and then (tabrange(i).max = 0))) then
        pos := i;
        exit search;
      end if; -- i in min..max or 'others' config
      if ((tabrange(i).min /= 0)or else(tabrange(i).max /= 0)) then
        last := i;
      end if; -- (min /= 0) or else (max /= 0)
    end loop search; -- for
    if (pos = -1) then
      error := 1;
      if (last = -1) then
        pos := tabrange'first;
      else
        pos := last;
      end if; -- last = -1
    end if; -- pos = -1
  end; -- procedure searchpos()

  procedure Begin_Check_Stub(Num_Stub : in Attol_Int        ;
                             tabrange : in Attol.Stub_Range ;
                             Index    : in Attol_Int        ) is
     Maxpos     : Attol.Attol_Int := -1;
     Maxslice   : Attol.Attol_Int := Attol_Int'first;
     Othersmode : Attol.Attol_Int := 0;
  begin
     Attol.Edv_Muet_Check(Num_Stub,0);
     for I in Attol.Attol_int range tabrange'first..tabrange'last loop
        if ((Maxslice < tabrange(i).Max)  and then  (Othersmode = 0)) then
           Maxslice := tabrange(i).Max;
           Maxpos := I;
        end if; -- tabrange(i).max > maxslice
        if ((tabrange(i).Max = 0)  and then  (tabrange(i).Min = 1)) then -- 'others'
           Othersmode := 1;
           Maxpos := I;
        end if; -- 'others'
     end loop; -- for
     if (Othersmode = 1) then
        ATTOL.EDV_ADA(UnCheck,longest_integer(Index),0,0,Attol.Attol_big_int'LAST,10,StdValue); -- always true
     else
        ATTOL.EDV_ADA(Check,longest_integer(Index),0,longest_integer(Maxslice),longest_integer(Maxslice),10,StdValue);
     end if; -- mode 'others'
  end Begin_Check_Stub;


  procedure Extract_Val(Tab     : in String      ;
                        Cpt     : in out Natural ;
                        Val     : in out Integer    ;
                        TrueVal : in Attol_Int   ) is
     Signe : Integer := 1;
  begin
    if (Tab(Cpt) = '-') then
      Signe := -1;
      Cpt := Cpt + 1;
    end if;
    Val := 0;
    while Tab(Cpt) in '0'..'9' loop
      Val := 10*Val + (Character'pos(Tab(Cpt))-48);
      Cpt := Cpt + 1;
    end loop;
    if (Tab(Cpt) = 'o') then
      Val := TrueVal;
      Cpt := Cpt + 1;
    end if;
  end;

  Function Present(Tab      : String  ;
                   Maxlevel : Natural ) return Integer is
    Lvl       : Natural := Curr_Array_Idx-Maxlevel+2;
    Val1,Val2 : Integer := 0;
    Cpt       : Natural := 2; -- to avoid first parenthesis
    Lvlbegin  : Natural := lvl;
    LvlSave   : Natural;
  begin
    while (Lvl >= Lvlbegin) loop
      Extract_Val(Tab,Cpt,Val1,Array_Idx(lvl));
      if (Tab(Cpt) = '.') then
        Cpt := Cpt + 1;
        Extract_Val(Tab,Cpt,Val2,Array_Idx(lvl));
      else
        Val2 := Val1;
      end if;
      if (Array_Idx(lvl) in Val1..Val2) then
        while (Tab(Cpt) /= ':') and (Tab(Cpt) /= '(') loop
          Cpt := Cpt + 1; -- skip to avalaible value
        end loop;
        if (Tab(Cpt) = ':') then
          Cpt := Cpt + 1; -- skip to avalaible value
          Extract_Val(Tab,Cpt,Val1,0);
          return Val1;
        end if;
        Cpt := Cpt + 1;
        Lvl := Lvl + 1;
      else
        case Tab(Cpt) is
          when '|' =>
            Cpt := Cpt + 1;
          when '(' => -- skip to next same-level close parenthesis
            LvlSave := Lvl+1;
            Cpt := Cpt + 1;
            while (LvlSave /= Lvl) loop
              if (Tab(Cpt) = '(') then
                LvlSave := LvlSave + 1;
              elsif (Tab(Cpt) = ')') then
                LvlSave := LvlSave - 1;
              end if;
              Cpt := Cpt + 1;
            end loop;
            case Tab(Cpt) is
              when ')' =>
                return 0; -- no remaining avalaible choice
              when ',' =>
                Cpt := Cpt + 1;
              when others =>
                null; -- never append
            end case;
          when ':' =>
            Cpt := Cpt + 1;
            Extract_Val(Tab,Cpt,Val1,0); -- Val1 will never be used
            case Tab(Cpt) is
              when ')' =>
                return 0; -- no remaining avalaible choice
              when ',' =>
                Cpt := Cpt + 1;
              when others =>
                null; -- never append
            end case;
          when others =>
            null; -- never append
        end case;
      end if;
    end loop;
    return 0;
  end;

end attol;

--
-------------------------------------------------------------------------------


