

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atc.c,v 1.24.2.2 2008/03/18 10:55:49 build Exp $
* $Descr: Coverage Library Source (included by TP.c[pp]) $
*******************************************************************************/

#if ATL_MULTI_THREADS

RTRT_MT_CRITICAL_VAR_DECL(atc_critical)

#endif /* ATL_MULTI_THREADS */


/*******************************************************************************
* Global variables
*******************************************************************************/

RTRT_GLOBALVAR _atc_T_cov *_atc_cov_first;

#if defined(ATL_SIGNAL) && ATC_SIGNAL
int _atc_catch_reset;
/*
#if !RTRT_KR
_ATL_EXTERN_C void _atc_register_signal ( void );
#else
_ATL_EXTERN_C void _atc_register_signal ( );
#endif
*/
#endif /* ATL_SIGNAL */


/*******************************************************************************
*  Service: _atc_ierror
*******************************************************************************/
#if _ATL_ERROR_IMPLEMENTED
#define ATL_ERROR_COV(c,s) _atc_ierror(c);ATL_ERROR(c,s)
#else
#define ATL_ERROR_COV(c,s) _atc_ierror(c)
#endif
RTRT_STATICFCT void _atc_ierror
#if !RTRT_KR
( int icode )
#else
( icode )
  int icode;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TIO_EI );
  atl_line_inc ( unsigned_reverse_image_16 ( icode, atl_line_cur ) );
  atl_line_dump ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

/*******************************************************************************
*  Service: _atc_reset_table
*******************************************************************************/
RTRT_STATICFCT void _atc_reset_table
#if !RTRT_KR
( _atc_T_count *table,
  RTRT_U_INT32 nb_elt  )
#else
( table, nb_elt )
  _atc_T_count *table;
  RTRT_U_INT32 nb_elt;
#endif
{
  RTRT_U_INT32 i;

  for ( i = 0; i < _ATC_NB_ELT(nb_elt); i++ )
  {
    table[i] = (_atc_T_count)0;
  }
}

/*******************************************************************************
*  Service: _atc_reset
*******************************************************************************/
RTRT_STATICFCT void _atc_reset
#if !RTRT_KR
( void )
#else
( )
#endif
{
  _atc_T_cov *cov;

  for ( cov = _atc_cov_first; cov; cov = cov->next )
  {
    _atc_reset_table ( cov->proc, cov->nb_proc );
    _atc_reset_table ( cov->call, cov->nb_call );
    _atc_reset_table ( cov->bloc, cov->nb_bloc );
    _atc_reset_table ( cov->cond, cov->nb_cond );
  }
}

/*******************************************************************************
*  Service: _atc_link
*******************************************************************************/
_ATL_C_GLOBALFCT void _atc_link
#if !RTRT_KR
( _atc_T_cov *cov )
#else
( cov )
  _atc_T_cov *cov;
#endif
{
  if ( ! _ATL_INITIALIZED )
  {
    /* This branch is reached once in the process life */
    atl_early_init ( );
  }
  RTRT_MT_CRITICAL_VAR_LOCK(atc_critical)
  if ( ! ( cov->link & (unsigned char)0x01 ) )
  {
    if ( ( cov->link & (unsigned char)0x0E ) != _ATC_CONST )
    {
      ATL_ERROR_COV (0x20,(char*)"Coverage: Mismatch between Coverage Information selection for TDP compilation and for instrumentation; check your settings (Coverage Information: Default/Compact Hit Data/Count Information) or command-line options (Pass/Compact/Count)");
      cov->link |= (unsigned char)0x01;
    }
    else
    {
      _atc_reset_table ( cov->proc, cov->nb_proc );
      _atc_reset_table ( cov->call, cov->nb_call );
      _atc_reset_table ( cov->bloc, cov->nb_bloc );
      _atc_reset_table ( cov->cond, cov->nb_cond );
      cov->next = _atc_cov_first;
      _atc_cov_first = cov;
      cov->link |= (unsigned char)0x01;
    }
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atc_critical)
}

/*******************************************************************************
*  Service: _atc_set
*******************************************************************************/
#if ATC_INFORMATION == 1
_ATL_C_GLOBALFCT void _atc_set
#if !RTRT_KR
( _atc_T_count *table,
  RTRT_U_INT32 n )
#else
( table, n )
  _atc_T_count *table;
  RTRT_U_INT32 n;
#endif
{
  table[n>>3] |= (_atc_T_count) ( 0x01 << ( n & 0x07 ) );
}
#endif /* ATC_INFORMATION == 1 */
#if ATC_INFORMATION == 2
void _atc_set
#if !RTRT_KR
( _atc_T_count *count )
#else
( count )
  _atc_T_count *count;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atc_critical)
  if ( *count < _ATC_MAX_COUNT )
  {
    (*count)++;
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atc_critical)
}
#endif /* ATC_INFORMATION == 2 */

/*******************************************************************************
*  Service: _atc_get
*******************************************************************************/
#if ATC_INFORMATION == 1
_ATL_C_GLOBALFCT _atc_T_count _atc_get
#if !RTRT_KR
( _atc_T_count *table,
  RTRT_U_INT32 n )
#else
( table, n )
  _atc_T_count *table;
  RTRT_U_INT32 n;
#endif
{
  return table[n>>3] & (_atc_T_count) ( 0x01 << ( n & 0x07 ) );
}
#endif /* ATC_INFORMATION == 1 */

/*******************************************************************************
*  Service: _atc_unset
*******************************************************************************/
#if ATC_INFORMATION == 1
_ATL_C_GLOBALFCT void _atc_unset
#if !RTRT_KR
( _atc_T_count *table,
  RTRT_U_INT32 n,
  _atc_T_count back )
#else
( table, n, back )
  _atc_T_count *table;
  RTRT_U_INT32 n;
  _atc_T_count back;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atc_critical)
  if ( back == 0 )
  {
    table[n>>3] &= ~ ( (_atc_T_count) ( 0x01 << ( n & 0x07 ) ) );
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atc_critical)
}
#else /* ATC_INFORMATION == 1 */
void _atc_unset
#if !RTRT_KR
( _atc_T_count *elt,
  _atc_T_count back )
#else
( elt, back )
  _atc_T_count *elt;
  _atc_T_count back;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atc_critical)
#if ATC_INFORMATION == 2
  if ( *elt != 0 )
  {
    (*elt)--;
  }
#else /* ATC_INFORMATION == 2 */
  if ( back == 0 )
  {
    *elt = 0;
  }
#endif /* ATC_INFORMATION == 0 */
  RTRT_MT_CRITICAL_VAR_UNLOCK(atc_critical)
}
#endif /* ATC_INFORMATION != 1 */

/*******************************************************************************
*  Service: _atc_cond_case
*******************************************************************************/
_ATL_C_GLOBALFCT void _atc_cond_case
#if !RTRT_KR
( char *spy,
  _atc_T_count *count,
  RTRT_U_INT32 n )
#else
( spy, count, n )
  char *spy;
  _atc_T_count *count;
  RTRT_U_INT32 n;
#endif
{
  if ( *spy )
  {
    *spy = 0x00;
    _ATC_SET ( 0, count, n );
  }
}

/*******************************************************************************
*  Service: _atc_cond_sync_bit
*******************************************************************************/
_ATL_C_GLOBALFCT int _atc_cond_sync_bit
#if !RTRT_KR
( int expr,
  RTRT_U_INT32 *cond_elt,
  int shift )
#else
( expr, cond_elt, shift )
  int expr;
  RTRT_U_INT32 *cond_elt;
  int shift;
#endif
{
  if ( expr )
  {
    (*cond_elt) |= (RTRT_U_INT32)(((RTRT_U_INT32)0x00000001)<<shift);
  }
  return expr;
}

/*******************************************************************************
*  Service: _atc_cond_end_bit
*******************************************************************************/
_ATL_C_GLOBALFCT int _atc_cond_end_bit
#if !RTRT_KR
( int expr,
  _atc_T_count *count,
  RTRT_U_INT32 n,
  RTRT_U_INT32 *offset )
#else
( expr, count, n, offset )
  int expr;
  _atc_T_count *count;
  RTRT_U_INT32 n;
  RTRT_U_INT32 *offset;
#endif
{
  _ATC_SET ( 0, count, n + *offset );
  return expr;
}

/*******************************************************************************
*  Service: _atc_cond_init
*******************************************************************************/
_ATL_C_GLOBALFCT void _atc_cond_init
#if !RTRT_KR
( RTRT_U_INT16 *cond_elt,
  int len )
#else
( cond_elt, len )
  RTRT_U_INT16 *cond_elt;
  int len;
#endif
{
  int i;

  for ( i = ( len - 1 ) / 8; i >= 0; i-- )
  {
    cond_elt[i] = (RTRT_U_INT16)0;
  }
}

/*******************************************************************************
*  Service: _atc_cond_end
*******************************************************************************/
_ATL_C_GLOBALFCT int _atc_cond_end
#if !RTRT_KR
( int expr,
  _atc_T_count *count,
  RTRT_U_INT32 n,
  RTRT_U_INT16 *cond_elt,
  int len,
  RTRT_U_INT16 *cases,
  int frontiere )
#else
( expr, count, n, cond_elt, len, cases, frontiere )
  int expr;
  _atc_T_count *count;
  RTRT_U_INT32 n;
  char *cond_elt;
  int len;
  char **cases;
  int frontiere;
#endif
{
  int branch, i, first;

  for ( branch = 0;
        ( ( first = branch * ( ( len + 7 ) / 8 ) ),
          ( cases[first] != (RTRT_U_INT16)0xFFFF ) );
        branch++ )
  {
    for ( i = ( len - 1 ) / 8;
          i >= 0 && cond_elt[i] == cases[first+i];
          i-- );
    if ( i < 0 )
    {
      if ( expr ? ( branch >= frontiere ) : ( branch < frontiere ) )
      {
        ATL_ERROR_COV ( 0x31,(char*)"Coverage: A composition of conditions evaluates in an unexpected result, contact your vendor" );
      }
      else
      {
        _ATC_SET ( 0, count, n + branch );
      }
      return expr;
    }
  }
  ATL_ERROR_COV ( 0x32,(char*)"Coverage: A composition of conditions reached with an unexpected input vector, contact your vendor" );
  return expr;
}

/*******************************************************************************
*  Service: _atc_dump_table
*******************************************************************************/
RTRT_STATICFCT void _atc_dump_table
#if !RTRT_KR
( char *op,
  _atc_T_count *table,
  RTRT_U_INT32 len )
#else
( op, table, len )
  char *op;
  _atc_T_count *table;
  RTRT_U_INT32 len;
#endif
#if ATC_INFORMATION == 1
{
  RTRT_U_INT32 i;
  _atc_T_count mask;
#ifdef _NO_ZERO_EXPR
  char dummy;
#endif

  for ( i = 0, mask = 0x01;
        i < len;
        i++, mask <<= 1 )
  {
    if ( ! ( mask & 0xFF ) ) /* Sometimes, RTRT_U_INT8 can not be 8-bit width */
    {
      mask = 0x01; /* The looping bit completed a loop */
    }
    if ( table[i>>3] & mask )
    {
      atl_line_add ( op );
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( i, atl_line_cur ) );
      atl_line_dump ( );
    }
  }
}
#else /* ATC_INFORMATION == 1 */
{
  RTRT_U_INT32 i;

  for ( i = 0; i < len; i++ )
  {
    if ( _ATC_ELT_TAB ( table, i ) > 0 )
    {
      atl_line_add ( op );
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( i, atl_line_cur ) );
#if ATC_INFORMATION == 2
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( _ATC_ELT_TAB(table,i), atl_line_cur ) );
#endif /* ATC_INFORMATION == 2 */
      atl_line_dump ( );
    }
  }
}
#endif /* ATC_INFORMATION == 1 */

/*******************************************************************************
*  Service: _atc_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atc_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
  _atc_T_cov *cov;
 char *date;

  if ( reset & _ATCPQ_COV)
  {
	  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	  /* New Test */
	  ATL_LINE_ADD_OPCODE ( ATL_TIO_NT );
#if USE_ATU
          _atl_obstools_test_name ( 1 );
#else /* USE_ATU */
          _atl_obstools_test_name ( );
#endif /* USE_ATU */
	  atl_line_dump ( );

	  ATL_LINE_ADD_OPCODE ( ATL_TIO_DT );
	  date = priv_date();
	  if ( date != (char *)0 )
	  {
		atl_line_add_char ( ' ' );
		atl_line_add_char ( RTRT_QUOT );
		atl_line_add ( date );
		atl_line_add_char ( RTRT_QUOT );
	  }
	  else
	  {
		ATL_LINE_ADD_OPCODE ( " 0" );
	  }
	  atl_line_dump ( );

#if ATC_INFORMATION == 2
	  ATL_LINE_ADD_OPCODE ( ATL_TIO_CO );
	  atl_line_dump ( );
#endif /* ATC_INFORMATION == 2 */

	  for ( cov = _atc_cov_first; cov; cov = cov->next )
	  {
		ATL_LINE_ADD_OPCODE_SPACE ( ATL_TIO_FT );
		atl_line_inc ( unsigned_reverse_image_16 ( cov->key, atl_line_cur ) );
		atl_line_dump ( );
		ATL_LINE_ADD_OPCODE_SPACE ( ATL_TIO_DC );
		atl_line_inc ( unsigned_reverse_image_16 ( cov->checksum, atl_line_cur ) );
		atl_line_dump ( );

		_atc_dump_table ( (char*)RTRT_CONST_STRING(ATL_TIO_TP), cov->proc, cov->nb_proc );
		_atc_dump_table ( (char*)RTRT_CONST_STRING(ATL_TIO_TA), cov->call, cov->nb_call );
		_atc_dump_table ( (char*)RTRT_CONST_STRING(ATL_TIO_TB), cov->bloc, cov->nb_bloc );
		_atc_dump_table ( (char*)RTRT_CONST_STRING(ATL_TIO_TE), cov->cond, cov->nb_cond );
	  }
  }


  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
  if ( reset & _ATCPQ_RESET_COV)
  {
    _atc_reset ( );
  }
}

/*******************************************************************************
*  remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
void _atc_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
#if !ATL_DYN_TABLE

	_atc_T_cov *thedata, *dataPrev;
	RTRT_MT_CRITICAL_VAR_LOCK(atc_critical)

	/* remove coverage data for unloaded code */
	thedata = _atc_cov_first;
	while((thedata) && (thedata->Family == family))
	{
		thedata = thedata->next;
	}
	_atc_cov_first = thedata;
	dataPrev = thedata;
	while(thedata)
	{
	   if(thedata->Family == family)
	   {
		 dataPrev->next = thedata->next;
	   }
	   else
	   {
		   dataPrev = thedata;
	   }
	   thedata = thedata->next;
	}
	RTRT_MT_CRITICAL_VAR_UNLOCK(atc_critical)
#endif /* ATL_DYN_TABLE */
}
#endif
/*******************************************************************************
*  Dump Driver
*******************************************************************************/
#if ATC_DUMP_DRIVER

enum {
        _ATC_DD_HEADER_TEST,
        _ATC_DD_HEADER_DATE,
#if ATC_INFORMATION == 2
        _ATC_DD_HEADER_COUNT,
#endif /* ATC_INFORMATION == 2 */
        _ATC_DD_FILE_KEY,
        _ATC_DD_FILE_CHECKSUM,
        _ATC_DD_LOOP_PROC,
        _ATC_DD_LOOP_CALL,
        _ATC_DD_LOOP_BLOC,
        _ATC_DD_LOOP_COND
};

RTRT_GLOBALVAR int _atc_dd_step, _atc_dd_reset;
RTRT_GLOBALVAR RTRT_U_INT32 _atc_dd_index;
RTRT_GLOBALVAR _atc_T_cov *_atc_dd_current;

/*******************************************************************************
*  Service: _atc_dd_open
*******************************************************************************/
_ATL_C_GLOBALFCT void _atc_dd_open
#if !RTRT_KR
( int dump_header,
  int reset )
#else
( dump_header, reset )
  int dump_header;
  int reset;
#endif
{
  if ( dump_header )
  { /* Dump of NT (new test) and DT (date of test) lines */
    _atc_dd_step = _ATC_DD_HEADER_TEST;
  }
  else
  {
    _atc_dd_step = _ATC_DD_HEADER_DATE + 1;
  }
  /* reset value of 0 is deprecated: 1 is assumed */
  _atc_dd_reset = reset = 1;
  _atc_dd_current = _atc_cov_first;
}

/*******************************************************************************
*  Service: _atc_dd_branch
*******************************************************************************/
RTRT_STATICFCT void _atc_dd_branch
#if !RTRT_KR
( const char *op,
  _atc_T_count *table,
  int branch )
#else
( op, table, branch )
  char *op;
  _atc_T_count *table;
  int branch;
#endif
{
#if ATC_INFORMATION == 1

  if ( table[branch>>3] & ( (_atc_T_count) ( 0x01 << ( branch & 0x07 ) ) ) )
  {
    _atl_obstools_str_add ( op );
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             branch,
                             _atl_obstools_str_cur ) );
    if ( _atc_dd_reset )
    {
      table[branch>>3] &= ~ ( (_atc_T_count) ( 0x01 << ( branch & 0x07 ) ) );
    }
  }

#else /* ATC_INFORMATION == 1 */

  if ( _ATC_ELT_TAB ( table, branch ) )
  {
    _atl_obstools_str_add ( op );
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             branch,
                             _atl_obstools_str_cur ) );
#if ATC_INFORMATION == 2
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                             _ATC_ELT_TAB(table,branch),
                             _atl_obstools_str_cur ) );
#endif /* ATC_INFORMATION == 2 */
    if ( _atc_dd_reset )
    {
      table[branch] = (_atc_T_count)0;
    }
  }

#endif /* ATC_INFORMATION == 1 */
}

/*******************************************************************************
*  Service: _atc_dd_read_internal
*******************************************************************************/
RTRT_STATICFCT int _atc_dd_read_internal
#if !RTRT_KR
( char *dump_str )
#else
( dump_str )
  char *dump_str;
#endif
{
  /* Let's go for one trace line */
  _atl_obstools_str_init ( dump_str )

  switch ( _atc_dd_step )
  {
    case _ATC_DD_HEADER_TEST :
      _atc_dd_step++;
      _atl_obstools_str_add ( ATL_TIO_NT );
#if USE_ATU
      _atl_obstools_test_name ( 1 );
#else /* USE_ATU */
      _atl_obstools_test_name ( );
#endif /* USE_ATU */
      break;

    case _ATC_DD_HEADER_DATE :
      _atc_dd_step++;
      _atl_obstools_str_add ( ATL_TIO_DT );
      _atl_obstools_str_add ( " 0" );
      break;

#if ATC_INFORMATION == 2
    case _ATC_DD_HEADER_COUNT :
      _atc_dd_step++;
      _atl_obstools_str_add ( ATL_TIO_CO );
      break;
#endif /* ATC_INFORMATION == 2 */

    case _ATC_DD_FILE_KEY :
      if ( _atc_dd_current == (_atc_T_cov *)0 )
      {
        return 0;
      }
      _atc_dd_step++;
      _atl_obstools_str_add ( ATL_TIO_FT );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               _atc_dd_current->key,
                               _atl_obstools_str_cur ) );
      break;

    case _ATC_DD_FILE_CHECKSUM :
      _atc_dd_step++;
      _atc_dd_index = 0;
      _atl_obstools_str_add ( ATL_TIO_DC );
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_reverse_image_16 (
                               _atc_dd_current->checksum,
                               _atl_obstools_str_cur ) );
      break;

    case _ATC_DD_LOOP_PROC :
      if ( _atc_dd_index < _atc_dd_current->nb_proc )
      {
        _atc_dd_branch ( ATL_TIO_TP, _atc_dd_current->proc, _atc_dd_index );
        _atc_dd_index++;
        break;
      }
      _atc_dd_step++;
      _atc_dd_index = 0;

    case _ATC_DD_LOOP_CALL :
      if ( _atc_dd_index < _atc_dd_current->nb_call )
      {
        _atc_dd_branch ( ATL_TIO_TA, _atc_dd_current->call, _atc_dd_index );
        _atc_dd_index++;
        break;
      }
      _atc_dd_step++;
      _atc_dd_index = 0;

    case _ATC_DD_LOOP_BLOC :
      if ( _atc_dd_index < _atc_dd_current->nb_bloc )
      {
        _atc_dd_branch ( ATL_TIO_TB, _atc_dd_current->bloc, _atc_dd_index );
        _atc_dd_index++;
        break;
      }
      _atc_dd_step++;
      _atc_dd_index = 0;

    case _ATC_DD_LOOP_COND :
      if ( _atc_dd_index < _atc_dd_current->nb_cond )
      {
        _atc_dd_branch ( ATL_TIO_TE, _atc_dd_current->cond, _atc_dd_index );
        _atc_dd_index++;
        break;
      }
      _atc_dd_current = _atc_dd_current->next;
      if ( _atc_dd_current == (_atc_T_cov *)0 )
      {
        return 0;
      }
      _atc_dd_step = _ATC_DD_FILE_KEY;
      _atc_dd_index = 0;
      break;
  }

  return 1;
}

/*******************************************************************************
*  Service: _atc_dd_read
*******************************************************************************/
_ATL_C_GLOBALFCT int _atc_dd_read
#if !RTRT_KR
( char *dump_str )
#else
( dump_str )
  char *dump_str;
#endif
{
  int ret;

  while ( ( ret = _atc_dd_read_internal ( dump_str ) ) &&
          ( dump_str[0] == (char)0 ) );
  /* ret==0: finished,
   * ret!=0: needs more calls */
  return ret;
}

#endif /* ATC_DUMP_DRIVER */

#if defined(ATL_SIGNAL) && ATC_SIGNAL

/*******************************************************************************
*  Service: _atc_signal_receipt
*  Goal: called each time the process catch an environment defined signal
*******************************************************************************/
RTRT_STATICFCT void _atc_signal_receipt
#if !RTRT_KR
( int sig )
#else
( sig )
  int sig;
#endif
{
  _atc_dump ( _atc_catch_reset );

  /* Re-arm signal-catching function */
  ATL_SIGNAL ( sig, _atc_signal_receipt );
}

/*******************************************************************************
*  Service: _atc_register_signal
*  Goal: called once to register a signal catch handler to dump traces
*******************************************************************************/
RTRT_STATICFCT void _atc_register_signal
#if !RTRT_KR
( void )
#else
( )
#endif
{
  int signal_to_catch;

#ifndef ATL_GETENV
  signal_to_catch = SIGUSR1;
#else
  {
    char *value;

    signal_to_catch = 0;
    if ( ( value = ATL_GETENV ( "ATC_SIGNAL_DUMP" ) ) )
    {
      char *p;

      p = value;
      while ( *p >= '0' && *p <= '9' )
      {
        signal_to_catch *= 10;
        signal_to_catch += *p - '0';
        p++;
      }
      if ( *p == ':' )
      {
        p++;
        if ( ( *p == '2' ) &&
             ( *(p+1) == (char)0 ) )
        {
          _atc_catch_reset += _ATCPQ_CLOSE;
        }
      }
    }
    if ( signal_to_catch <= 0 )
    {
      signal_to_catch = SIGUSR1;
    }
  }
#endif /* ATL_GETENV */
  if ( ATL_SIGNAL ( signal_to_catch, _atc_signal_receipt ) == SIG_ERR )
  {
    ATL_ERROR ( 0x28,"Coverage: Can't catch signal" );
  }
}

#endif /* defined(ATL_SIGNAL) && ATC_SIGNAL */

/*******************************************************************************
*  Service: atc_early_init
*  Goal: initializes global variables
*******************************************************************************/
_ATL_C_GLOBALFCT void atc_early_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
  /* This function is called once in the process life */
  RTRT_MT_CRITICAL_VAR_INIT(atc_critical)
  _atc_cov_first = (_atc_T_cov *)0;
#if USE_ATU
  _atc_atu_num_last_op = -1;
#endif /* USE_ATU */
#if defined(ATL_SIGNAL) && ATC_SIGNAL
  _atc_catch_reset = _ATCPQ_ALL;
  _atc_register_signal ( );
#endif /* defined(ATL_SIGNAL) && ATC_SIGNAL */
}

#ifdef _ATL_EXIT_PENDING
/*******************************************************************************
*  Service: atc_exit_pending
*  Goal: called when it is about time to exit
*******************************************************************************/
_ATL_C_GLOBALFCT void atc_exit_pending
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! USE_ATU && ! USE_ATO
  _atc_dump ( _ATCPQ_COV );
#endif /* USE_ATU && USE_ATO */
}
#else
/*******************************************************************************
*  Service: atc_late_exit
*  Goal: called when it is time to quit
*******************************************************************************/
_ATL_C_GLOBALFCT void atc_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! USE_ATU && ! USE_ATO
  _atc_dump ( _ATCPQ_COV );
RTRT_MT_CRITICAL_VAR_DESTROY(atc_critical)
#endif /* USE_ATU && USE_ATO */
}
#endif

