

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atcapi.h,v 1.7.4.1 2008/03/18 10:55:49 build Exp $
* $Descr: Coverage API Header (to be included directly in user code) $
*******************************************************************************/


#ifndef _ATCAPI_INCLUDED
#define _ATCAPI_INCLUDED


/*******************************************************************************
* Coverage Dump Driver API
*
*   This API provides end-user a way to write the dynamic trace lines (in a
*   .tio file) his own way
*   Please read example and notes at the end of this file
*******************************************************************************/

/* C++ Users: if compiler *does not* define __cplusplus, please define it
 * just before inclusion as:

#define __cplusplus
#include "atcapi.h"

 * */

#if defined(__cplusplus)
extern "C" {
#endif

extern void _atc_dd_open ( int, int );
extern int _atc_dd_read ( char * );

#if defined(__cplusplus)
} // end of extern "C"
#endif


/* Here is an example of use (atcdriver.c):

#include "atcapi.h"

int atc_dump_driver ( void )
{
  char str[64];

  _atc_dd_open ( 1, 0 );
  while ( _atc_dd_read ( &str[0] ) )
  {
    printf ( "%s\n", str );
  }
  return 0;
}

 * Notes:
 *
 * - The TestRT Target Deployment Port sould be compiled with the
 *   ATC_DUMP_DRIVER flag set to 1 (see products_model.h and your copy of this
 *   file)
 *
 * - The lib sub-directory of the TestRT Target Deployment Port must be added
 *   to the compiler search list (using -I for example):
 *     cc -c atcdriver.c -I$ATLTGT/lib (Unix users)
 *     cl /c atcdriver.c /I%ATLTGT%\lib (Windows users)
 *
 * - Both atcdriver.o (or .obj) and TestRT Target Deployment Port object files
 *   must be linked with your instrumented application.
 *
 * - RTRT_IO TestRT Target Deployment Port adaptation point must be defined
 *   to RTRT_NONE
 *
 * - C++ Users should define __cplusplus before including atcapi.h if their
 *   compiler does not already do this.
 *
 * - size of buffer containing successively the TestRT Coverage lines should
 *   be decided regarding the maximum length of user test name + 5 (if used) and
 *   should not be less than 64
 *
 * - printf is provided only as an example of what to do with the built lines
 *
 * - the int arguments of _atc_dd_open() are:
 *     int dump_header: 1 implies the dump of test name; O does not
 *         -> use 0 if and only if this NT line is produced your own way
 *     int reset: 1 implies the reset to 0 of the internal Coverage tables at
 *                the end of the test; 0 does not
 *         -> use 1 if you plan to call the dump driver several times during
 *            the same execution of your software, in order to save dump time
 *
*/


#endif /* _ATCAPI_INCLUDED */
