

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atk.c,v 1.13.2.1 2008/03/18 10:55:49 build Exp $
* $Descr: Object Testing Library Source (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* TDF Contract Check codes
*******************************************************************************/

#define _ATK_STYLE_ENTER_METHOD        "ST 100"
#define _ATK_STYLE_EXIT_METHOD         "ST 101"
#define _ATK_STYLE_INVARIANT_OK        "ST 102"
#define _ATK_STYLE_INVARIANT_KO        "ST 103"
#define _ATK_STYLE_PRECOND_OK          "ST 104"
#define _ATK_STYLE_PRECOND_KO          "ST 105"
#define _ATK_STYLE_POSTCOND_OK         "ST 106"
#define _ATK_STYLE_POSTCOND_KO         "ST 107"
#define _ATK_STYLE_STATE               "ST 108"
#define _ATK_STYLE_ILLEGAL_TRANSITION  "ST 109"
#define _ATK_STYLE_MULTIPLE_TRANSITION "ST 110"

/*******************************************************************************
* TSF Contract Check special notes
*******************************************************************************/

#define _ATK_TSF_ILLEGAL_TRANSITION "fffffffch"
#define _ATK_TSF_MULTIPLE_TRANSITION "fffffffbh"

/*******************************************************************************
* Macros
*******************************************************************************/

#if ATK_BREAK_ON_ASSERT_FAILED
/*! Action to be done when an error is encountered */
#define _ATK_BREAK priv_check_failed ();
#else
#define _ATK_BREAK
#endif /* ATK_STOP_ON_ERROR */

#if ATL_MULTI_THREADS && USE_ATT
#define _ATK_CHECK_THREAD  int thread_index = _att_check_thread((_AtlStackLinkP)0);
#else
#define _ATK_CHECK_THREAD
#endif

/*! TSF ID of a Contract Check text */
#define _ATK_TEXT(i) i+_ATL_TDF_TEXT_OFFSET

/*******************************************************************************
* Variables
*******************************************************************************/

#if ATL_MULTI_THREADS == ATL_NO

/*! Lock variable to disable checkings while a check is being performed */
RTRT_STATICVAR _atk_Boolean _atk_checks_lock;

#else

RTRT_STATICVAR _atk_Boolean _atk_checks_lock_table[ATL_THREADS_MAX+1];
#define _atk_checks_lock _atk_checks_lock_table[thread_index]

#endif /* ATL_MULTI_THREADS */

/*******************************************************************************
* Runtime routines
*******************************************************************************/

_ATL_C_GLOBALFCT void _atk_enter_method
  (const char *name,
   _atk_DynamicInfo &di)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(di.f);
   _atl_tdf_line(_ATK_STYLE_ENTER_METHOD);
   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
   ATL_LINE_ADD_OPCODE ("AC \"");
   atl_buffer_string_image (name);
   ATL_LINE_ADD_OPCODE (" {\"");
   atl_line_inc (_atl_tdf_id_image (di.c, atl_line_cur));
   _atl_tdf_line_dump ();
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ATL_C_GLOBALFCT void _atk_exit_method
  (const char *name,
   _atk_DynamicInfo &di)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(di.f);
   _atl_tdf_line(_ATK_STYLE_EXIT_METHOD);
   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
   ATL_LINE_ADD_OPCODE ("AC \"} ");
   atl_buffer_string_image (name);
   atl_line_add_char (RTRT_QUOT);
   atl_line_inc (_atl_tdf_id_image (di.c, atl_line_cur));
   _atl_tdf_line_dump ();
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ATL_C_GLOBALFCT _atk_Boolean _atk_assert_invariant
  (_atk_Boolean eval,
   _atk_Id inv,
   _atk_DynamicInfo &di
#if USE_ATT
                       ,
   RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
  )
{
   if (!eval)
   {
      _ATK_BREAK
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,inv,eval ?  1 :  0);
#if !ATK_DUMP_SUCCESS
   if (!eval)
   {
      _atl_tdf_line(_ATK_STYLE_INVARIANT_KO);
#else
      _atl_tdf_line(eval ? _ATK_STYLE_INVARIANT_OK : _ATK_STYLE_INVARIANT_KO);
#endif /* ATK_DUMP_SUCCESS */
      _atl_tdf_line_1(ATL_TDF_NO,_ATK_TEXT(inv));
#if USE_ATT
      if (tracer_inst)
	 _atl_current_instance = tracer_inst;
      else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
	 _atl_current_instance = di.c;
      _atl_tdf_line_1("CI",_atl_current_instance);
#if !ATK_DUMP_SUCCESS
   }
#endif /* !ATK_DUMP_SUCCESS */
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_atk_Boolean _atk_assert_pre_cond (_atk_Boolean eval,
				   _atk_Id pre,
				   _atk_DynamicInfo &di
#if USE_ATT
				   , RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
				   )
{
   if (!eval)
   {
      _ATK_BREAK
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,pre,eval ?  1 :  0);
#if !ATK_DUMP_SUCCESS
   if (!eval)
   {
      _atl_tdf_line(_ATK_STYLE_PRECOND_KO);
#else
      _atl_tdf_line(eval ? _ATK_STYLE_PRECOND_OK : _ATK_STYLE_PRECOND_KO);
#endif /* ATK_DUMP_SUCCESS */
      _atl_tdf_line_1(ATL_TDF_NO,_ATK_TEXT(pre));
#if USE_ATT
      if (tracer_inst)
	 _atl_current_instance = tracer_inst;
      else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
	 _atl_current_instance = di.c;
      _atl_tdf_line_1("CI",_atl_current_instance);
#if !ATK_DUMP_SUCCESS
   }
#endif /* !ATK_DUMP_SUCCESS */
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_atk_Boolean _atk_assert_pre_cond (char* eval,
				   _atk_Id pre,
				   _atk_DynamicInfo &di
#if USE_ATT
				   , RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
				   )
{
   if (!eval)
   {
      _ATK_BREAK
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,pre,  1);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return (_atk_Boolean)1;
}


_atk_Boolean _atk_assert_post_cond (_atk_Boolean eval,
				    _atk_Id post,
				    _atk_DynamicInfo &di
#if USE_ATT
				    , RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
				    )
{
   if (!eval)
   {
      _ATK_BREAK
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f); /* modifi les trace ATT sans utiliser le mutex att_producer ??? RF*/
   _atl_tdf_line_2(ATL_TDF_OT,post,eval ?  1 :  0);
#if !ATK_DUMP_SUCCESS
   if (!eval)
   {
      _atl_tdf_line(_ATK_STYLE_POSTCOND_KO);
#else
      _atl_tdf_line(eval ? _ATK_STYLE_POSTCOND_OK : _ATK_STYLE_POSTCOND_KO);
#endif /* ATK_DUMP_SUCCESS */
_atl_tdf_line_1(ATL_TDF_NO,_ATK_TEXT(post));
#if USE_ATT
      if (tracer_inst)
	 _atl_current_instance = tracer_inst;
      else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
	 _atl_current_instance = di.c;
      _atl_tdf_line_1("CI",_atl_current_instance);
#if !ATK_DUMP_SUCCESS
   }
#endif /* !ATK_DUMP_SUCCESS */
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}
_atk_Boolean _atk_assert_post_cond (char* eval,
				    _atk_Id post,
				    _atk_DynamicInfo &di
#if USE_ATT
				    , RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
				    )
{
   if (!eval)
   {
      _ATK_BREAK
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,post, 1 );
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return (_atk_Boolean)1;
}

_ATL_C_GLOBALFCT void _atk_transition_i
  (_atk_Id trans,
   _atk_DynamicInfo &di)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_1(ATL_TDF_OT,trans);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ATL_C_GLOBALFCT void _atk_illegal_transition
  (_atk_Id state_src,
   _atk_DynamicInfo &di
#if USE_ATT
                       ,
   RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
  )
{
   _ATK_BREAK
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,state_src, 2);
   _atl_tdf_line(_ATK_STYLE_ILLEGAL_TRANSITION);
   _atl_tdf_line(ATL_TDF_NO " " _ATK_TSF_ILLEGAL_TRANSITION);
#if USE_ATT
   if (tracer_inst)
      _atl_current_instance = tracer_inst;
   else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
      _atl_current_instance = di.c;
   _atl_tdf_line_1("CI",_atl_current_instance);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ATL_C_GLOBALFCT void _atk_multiple_transitions
  (_atk_Id state_src,
   _atk_DynamicInfo &di
#if USE_ATT
                       ,
   RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
  )
{
   _ATK_BREAK
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_2(ATL_TDF_OT,state_src, 3);
   _atl_tdf_line(_ATK_STYLE_MULTIPLE_TRANSITION);
   _atl_tdf_line(ATL_TDF_NO " " _ATK_TSF_MULTIPLE_TRANSITION);
#if USE_ATT
   if (tracer_inst)
      _atl_current_instance = tracer_inst;
   else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
      _atl_current_instance = di.c;
   _atl_tdf_line_1("CI",_atl_current_instance);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ATL_C_GLOBALFCT void _atk_current_state
  (_atk_Id state,
   _atk_DynamicInfo &di
#if USE_ATT
                       ,
   RTRT_U_INT32 tracer_inst
#endif /* USE_ATT */
  )
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATK_CHECK_THREAD
   _atl_print_change_source(di.f);
   _atl_tdf_line_1(ATL_TDF_OT,state);
   _atl_tdf_line(_ATK_STYLE_STATE);
   _atl_tdf_line_1(ATL_TDF_NO,_ATK_TEXT(state));
#if USE_ATT
      if (tracer_inst)
	 _atl_current_instance = tracer_inst;
      else
#if USE_ATO
      if (!_ato_in_stub)
#endif
#endif
	 _atl_current_instance = di.c;
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

#if ATL_MULTI_THREADS && USE_ATT
_ATL_C_GLOBALFCT _atk_Boolean _atk_get_lock (int thread_index)
#else
_ATL_C_GLOBALFCT _atk_Boolean _atk_get_lock ()
#endif /* ATL_MULTI_THREADS && USE_ATT */
{
   return _atk_checks_lock;
}

#if ATL_MULTI_THREADS && USE_ATT
_ATL_C_GLOBALFCT void _atk_set_lock (_atk_Boolean lock,int thread_index)
#else
_ATL_C_GLOBALFCT void _atk_set_lock (_atk_Boolean lock)
#endif /* ATL_MULTI_THREADS && USE_ATT */
{
   _atk_checks_lock = lock;
}

RTRT_STATICFCT void atk_early_init ()
{
#if ! RTRT_RAM_CLEAN

#if ATL_MULTI_THREADS == ATL_NO
   _atk_checks_lock = _atl_false;
#else
   int i;

   for (i = 0; i < ATL_THREADS_MAX; i++)
   {
      _atk_checks_lock_table[i] = _atl_false;
   }
#endif
#endif
}

#ifdef _ATL_EXIT_PENDING
RTRT_STATICFCT void atk_exit_pending ()
{
}
#else
RTRT_STATICFCT void atk_late_exit ()
{
}
#endif

_ATL_C_GLOBALFCT _atk_Instance _atk_add_class
  (_atk_Id cid,
   _atk_File file)
{
   _atk_Instance ret;

   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(file);
   _atl_tdf_line_2(ATL_TDF_NI,cid,++_atl_highest_instance);
   ret = _atl_highest_instance;
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return ret;
}

_ATL_C_GLOBALFCT RTRT_U_INT32 _atk_add_source_file
  (RTRT_U_INT32 key,
   RTRT_U_INT32 checksum)
{
   RTRT_U_INT32 result;
   if (!_ATL_INITIALIZED)
   {
      atl_early_init ();
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   result = ++_atl_highest_source;
   _atl_print_source_file (result, key, checksum);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return result;
}

#if USE_ATT
_ATL_C_GLOBALFCT _atk_DynamicInfo &_atk_trace_info (_atk_Instance cls)
{
   static _atk_DynamicInfo dinfo;
   dinfo.f = _atl_current_source;
   dinfo.c = cls;
   return dinfo;
}
#endif /* USE_ATT */
