

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atk.h,v 1.40.4.1 2008/03/18 10:55:49 build Exp $
* $Descr: Contract Check Header (included across TP.h) $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef ATK_H_INCLUDED
#define ATK_H_INCLUDED



/*******************************************************************************
* Types
*******************************************************************************/

#define _atk_Boolean  _atl_bool
#define _atk_Id       RTRT_U_INT32
#define _atk_File     RTRT_U_INT32
#define _atk_Instance RTRT_U_INT32
#define _atk_Car   char
#define _atk_Int   int
#define _atk_Uint  unsigned int
#define _atk_Ulong unsigned long

typedef RTRT_INT32 _atk_StateType;

struct atk_TransDesc {
   _atk_Ulong Id;
   _atk_StateType DestStateNo;
};

struct atk_StateDesc {
   _atk_Car *Name;
   _atk_Ulong Id;
   _atk_StateType TransNb;
   const atk_TransDesc *Trans;
};

struct _atk_DynamicInfo
{
   _atk_File f;
   _atk_Instance c;
   _atk_DynamicInfo () : f(0), c(0) {}
};

/*******************************************************************************
* TestRT Contract Check Routines
*******************************************************************************/

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_enter_method        (const char*,
                                                     _atk_DynamicInfo &);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_exit_method         (const char*,
                                                     _atk_DynamicInfo &);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX _atk_Boolean  _atk_assert_invariant    (_atk_Boolean, _atk_Id,
						     _atk_DynamicInfo &
#if USE_ATT
						    ,RTRT_U_INT32
#endif
						    );

extern ATL_TP_PROTO_PREFIX _atk_Boolean  _atk_assert_pre_cond     (_atk_Boolean,
					       _atk_Id,
					       _atk_DynamicInfo &
#if USE_ATT
					       ,RTRT_U_INT32
#endif
					       );
extern ATL_TP_PROTO_PREFIX _atk_Boolean  _atk_assert_pre_cond     (char*,
					       _atk_Id,
					       _atk_DynamicInfo &
#if USE_ATT
					       ,RTRT_U_INT32
#endif
					       );

extern ATL_TP_PROTO_PREFIX _atk_Boolean  _atk_assert_post_cond    (_atk_Boolean,
					       _atk_Id,
					       _atk_DynamicInfo &
#if USE_ATT
					       ,RTRT_U_INT32
#endif
					       );
extern ATL_TP_PROTO_PREFIX _atk_Boolean  _atk_assert_post_cond    (char*,
					       _atk_Id,
					       _atk_DynamicInfo &
#if USE_ATT
					       ,RTRT_U_INT32
#endif
					       );

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_transition_i        (_atk_Id,
                                                     _atk_DynamicInfo &);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_illegal_transition  (_atk_Id,
						     _atk_DynamicInfo &
#if USE_ATT
						    ,RTRT_U_INT32
#endif
						    );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_multiple_transitions(_atk_Id,
						     _atk_DynamicInfo &
#if USE_ATT
						    ,RTRT_U_INT32
#endif
						    );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void          _atk_current_state       (_atk_Id,
						     _atk_DynamicInfo &
#if USE_ATT
						    ,RTRT_U_INT32
#endif
						    );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX _atk_Instance _atk_add_class           (_atk_Id, _atk_File);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX RTRT_U_INT32   _atk_add_source_file     (RTRT_U_INT32 key,
						     RTRT_U_INT32 checksum);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX _atk_DynamicInfo &
			   _atk_trace_info          (_atk_Instance cls);

#if ATL_MULTI_THREADS && USE_ATT
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX _atk_Boolean _atk_get_lock (int thread_index);
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atk_set_lock (_atk_Boolean lock,int thread_index);
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX _atk_Boolean _atk_get_lock ();
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atk_set_lock (_atk_Boolean lock);
#endif /* ATL_MULTI_THREADS && USE_ATT */

/*******************************************************************************
* Instrumentation macros
*******************************************************************************/

#if ATK_CONST_ASSERTIONS
/*! Suffix to add to checking routines */
#define _ATK_CONST_SUFFIX const
#else
#define _ATK_CONST_SUFFIX
#endif /* ATK_CONST_ASSERTIONS */

/*! Name for state chart checking routine */
#define _ATK_STATE_NAME _atk_check_state_chart

/*! Name for state computing routine */
#define _ATK_STATE_CHECK_NAME _atk_check_state

/*! Fields for state handling */
#define _ATK_STATE_FIELDS protected: _atk_StateType _atk_state;

#define _ATK_MUTEX_FIELDS protected: \
   RTRT_MT_REC_MUTEX_DECL(_atk_mutex) \
   void _atk_mutex_lock () { RTRT_MT_REC_MUTEX_LOCK(_atk_mutex) } \
   void _atk_mutex_unlock () { RTRT_MT_REC_MUTEX_UNLOCK(_atk_mutex) }

/*! Fields for lock handling, when a base class already has these fields (this
 * macro is split into 3 pieces because classes name cannot be passed as
 * macro arguments) */
#define _ATK_INH_MUTEX_FIELDS1 protected: \
   void _atk_mutex_lock () {
#define _ATK_INH_MUTEX_FIELDS2 ::_atk_mutex_lock (); } \
   void _atk_mutex_unlock () {
#define _ATK_INH_MUTEX_FIELDS3 ::_atk_mutex_unlock (); }

/*! State computing routine return type */
#define _ATK_STATE_RT void

#if USE_ATT
/*! State computing routine prototype */
#define _ATK_STATE_PROTO _ATK_STATE_NAME (const _att_StackLink &_att_object)
//#define _ATK_STATE_PROTO _ATK_STATE_NAME (RTRT_U_INT32 tracer_inst)
#else
#define _ATK_STATE_PROTO _ATK_STATE_NAME ()
#endif /* USE_ATT */

/*! State computing routine declaration */
#define _ATK_STATE_DECL protected: virtual _ATK_STATE_RT _ATK_STATE_PROTO

/*! State chart checking routine return type */
#define _ATK_STATE_CHECK_RT _atk_Boolean

/*! State chart checking routine prototype */
#define _ATK_STATE_CHECK_PROTO \
   _ATK_STATE_CHECK_NAME (_atk_StateType State) _ATK_CONST_SUFFIX

/*! State chart checking routine declaration */
#define _ATK_STATE_CHECK_DECL private: \
   _ATK_STATE_CHECK_RT _ATK_STATE_CHECK_PROTO

/*! State chart standard intialization */
#define _ATK_STATE_INIT _atk_state=-1;

/*! State chart initialization in a constructor */
#define _ATK_STATE_CTOR_INIT _atk_state(-1)

/*! Mutex fields initalization */
#define _ATK_MUTEX_INIT RTRT_MT_REC_MUTEX_INIT(_atk_mutex)

/*! Mutex lock */
#define _ATK_MUTEX_LOCK _atk_mutex_lock();

/*! Mutex release */
#define _ATK_MUTEX_UNLOCK _atk_mutex_unlock();

/*! Mutex destroy */
#define _ATK_MUTEX_DESTROY RTRT_MT_REC_MUTEX_DESTROY(_atk_mutex)

#if ATL_MULTI_THREADS && USE_ATT
#ifndef _ATO_INSTRUMENTED_GENERATED_FILE
#define _ATK_LOCK if (!_atk_get_lock(_att_object.StackLink->ThreadIndex)) { _atk_set_lock(_atl_true,_att_object.StackLink->ThreadIndex);
#define _ATK_UNLOCK _atk_set_lock(_atl_false,_att_object.StackLink->ThreadIndex); }
#else /* _ATO_INSTRUMENTED_GENERATED_FILE */
#define _ATK_LOCK if (!_atk_get_lock(1)) { _atk_set_lock(_atl_true,1);
#define _ATK_UNLOCK _atk_set_lock(_atl_false,1); }
#endif /* _ATO_INSTRUMENTED_GENERATED_FILE */
#else /* ATL_MULTI_THREADS && USE_ATT */
/*! Lock-protected code start */
#define _ATK_LOCK if (!_atk_get_lock()) { _atk_set_lock(_atl_true);
/*! Lock-protected code end */
#define _ATK_UNLOCK _atk_set_lock(_atl_false); }
#endif /* ATL_MULTI_THREADS && USE_ATT */

/*! Name for invariants checking routine */
#define _ATK_INV_NAME _ATK_check_invariants

/*! Invariants checking routine return type */
#define _ATK_INV_RT void

#if USE_ATT
/*! Invariants checking routine prototype */
#define _ATK_INV_PROTO \
   _ATK_INV_NAME (const _att_StackLink &_att_object) _ATK_CONST_SUFFIX
//   _ATK_INV_NAME (RTRT_U_INT32 tracer_inst) _ATK_CONST_SUFFIX
#else
#define _ATK_INV_PROTO _ATK_INV_NAME () _ATK_CONST_SUFFIX
#endif /* USE_ATT */

/*! Invariants checking routine declaration */
#define _ATK_INV_DECL protected: virtual _ATK_INV_RT _ATK_INV_PROTO

#if USE_ATT
/*! Retrieves current ATT instance by "standard" method */
#define _ATK_ATT_INSTANCE _att_object
//#define _ATK_ATT_INSTANCE _att_object.StackLink->InstanceId
/*! Retrieves current ATT instance by "field" method */
//#define _ATK_ATT_FIELD _att_field._att_this()
#define _ATK_ATT_FIELD _att_Field_StackLink(_att_field._att_this())
/*! No current ATT instance special value */
#define _ATK_ATT_NULL _att_Field_StackLink(0)
//#define _ATK_ATT_NULL 0
/*! Retrieves current ATT instance in a ATK-checking routine */
#define _ATK_ATT_COPY _att_object
//#define _ATK_ATT_COPY tracer_inst
#else
#define _ATK_ATT_NULL
#define _ATK_ATT_COPY
#endif /* USE_ATT */

#if ATL_MULTI_THREADS && USE_ATT

class _ATK_CB { int thread; public:_ATK_CB(int i) {thread=i; } ~_ATK_CB() { _atk_set_lock(_atl_false,thread); } };
class _ATK_CA { int thread; public:_ATK_CA(int i) {thread=i; } ~_ATK_CA() { _atk_set_lock(_atl_true,thread); } };
/*! _atk_old object creation macro in instrumented routines */
#define _ATK_OLD_DEF(t) \
   _ATK_CB _atk_cb(_att_object.StackLink->ThreadIndex); {_atk_set_lock(_atl_true,_att_object.StackLink->ThreadIndex); t _atk_old (*this); _atk_set_lock(_atl_false,_att_object.StackLink->ThreadIndex); {_ATK_CA _atk_ca(_att_object.StackLink->ThreadIndex);
#else /* ATL_MULTI_THREADS && USE_ATT */
class _ATK_CB { public: ~_ATK_CB() { _atk_set_lock(_atl_false); } };
class _ATK_CA { public: ~_ATK_CA() { _atk_set_lock(_atl_true); } };
/*! _atk_old object creation macro in instrumented routines */
#define _ATK_OLD_DEF(t) \
   _ATK_CB _atk_cb; {_atk_set_lock(_atl_true); t _atk_old (*this); _atk_set_lock(_atl_false); {_ATK_CA _atk_ca;
#endif /* ATL_MULTI_THREADS && USE_ATT */
/*! _atk_old destruction handling in instrumented routines */
#define _ATK_OLD_CLOSE }}

#if RTRT_STATIC_IN_STATIC_INLINE_ALLOWED
#define _ATK_DINFO_STATIC_VAR_DECL static _atk_DynamicInfo dinfo;
#define _ATK_DINFO_VAR_NAME(cid,cs1,cs2) dinfo
#define _ATK_DINFO_VAR_GLOBAL_DECL(cid,cs1,cs2)
#define _ATK_DINFO_VAR_GLOBAL_DEF(cid,cs1,cs2)
#else
#define _ATK_DINFO_STATIC_VAR_DECL
#define _ATK_DINFO_INT1(x,y) RTRT_CONCAT_MACRO(x,y)
#define _ATK_DINFO_INT2(x,y) RTRT_CONCAT_MACRO(x,y)
#define _ATK_DINFO_VAR_NAME(cid,cs1,cs2) _ATK_DINFO_INT1(_ATK_DINFO_INT2(_atk_dinfo_,cid),_ATK_DINFO_INT2(cs1,cs2))
#define _ATK_DINFO_VAR_GLOBAL_DECL(cid,cs1,cs2) \
   extern _atk_DynamicInfo _ATK_DINFO_VAR_NAME(cid,cs1,cs2);
#define _ATK_DINFO_VAR_GLOBAL_DEF(cid,cs1,cs2) \
   _atk_DynamicInfo _ATK_DINFO_VAR_NAME(cid,cs1,cs2);
#endif

#if USE_ATT

#define _ATK_DINFO_PARAM _atk_Boolean inst
#define _ATK_DINFO_PNAME inst

#if USE_ATO
#define _ATK_DINFO_STUB_CASE && !_ato_in_stub
#else
#define _ATK_DINFO_STUB_CASE
#endif /* USE_ATO */

/*! Dynamic-info static method body */
#define _ATK_DINFO_BODY(cid,cs1,cs2) \
   _ATK_DINFO_STATIC_VAR_DECL \
   if (!_ATK_DINFO_VAR_NAME(cid,cs1,cs2).c) { \
      if (!_ATK_DINFO_VAR_NAME(cid,cs1,cs2).f) \
	_ATK_DINFO_VAR_NAME(cid,cs1,cs2).f = _atk_add_source_file(cs1,cs2); \
      if (inst _ATK_DINFO_STUB_CASE) \
	_ATK_DINFO_VAR_NAME(cid,cs1,cs2).c = _atk_add_class(cid,_ATK_DINFO_VAR_NAME(cid,cs1,cs2).f); \
   } \
   return _ATK_DINFO_VAR_NAME(cid,cs1,cs2);

#else

#define _ATK_DINFO_PARAM
#define _ATK_DINFO_PNAME

#define _ATK_DINFO_BODY(cid,cs1,cs2) \
   _ATK_DINFO_STATIC_VAR_DECL \
   if (!_ATK_DINFO_VAR_NAME(cid,cs1,cs2).f) { \
      _ATK_DINFO_VAR_NAME(cid,cs1,cs2).f = _atk_add_source_file(cs1,cs2); \
      _ATK_DINFO_VAR_NAME(cid,cs1,cs2).c = \
	_atk_add_class(cid,_ATK_DINFO_VAR_NAME(cid,cs1,cs2).f); \
   } \
   return _ATK_DINFO_VAR_NAME(cid,cs1,cs2);

#endif /* USE_ATT */

/*! Dynamic-info static method declaration */
#define _ATK_STATIC_CLASS_INFO_DECL \
   protected : \
      static _atk_DynamicInfo &_atk_dinfo(_ATK_DINFO_PARAM);

/*! Dynamic-info static method definition */
#define _ATK_STATIC_CLASS_INFO(cid,cs1,cs2) \
   protected: \
      static _atk_DynamicInfo &_atk_dinfo(_ATK_DINFO_PARAM) \
         { _ATK_DINFO_BODY(cid,cs1,cs2) }

/*! Dynamic-info static method definition for derived classes of a ATK-
 *  instrumented class (this macro is split into 2 pieces because classes names
 *  cannot be passed as macros arguments) */
#define _ATK_STATIC_INH_CLASS_INFO1 \
   protected : static _atk_DynamicInfo &_atk_dinfo(_ATK_DINFO_PARAM) { return
#define _ATK_STATIC_INH_CLASS_INFO2 \
   ::_atk_dinfo(_ATK_DINFO_PNAME); }

#define _ATK_STATIC_SPEC_CLASS_INFO1 _atk_DynamicInfo &
#define _ATK_STATIC_SPEC_CLASS_INFO2(cid,cs1,cs2) \
   ::_atk_dinfo(_ATK_DINFO_PARAM) { _ATK_DINFO_BODY(cid,cs1,cs2) }

#if USE_ATT
#define _ATK_CLASS_FILE_CLS _atk_dinfo (_att_object.StackLink->InstanceId?_atl_false:_atl_true), _att_object.StackLink->InstanceId
//#define _ATK_CLASS_FILE_CLS _atk_dinfo (tracer_inst?_atl_false:_atl_true), tracer_inst
#define _ATK_CLASS_FILE_ACM _atk_dinfo (_atl_true), (RTRT_U_INT32)0
#define _ATK_CLASS_FILE_TRM _atk_dinfo (_atl_false), _att_object.StackLink->InstanceId
#else
#define _ATK_CLASS_FILE_CLS _atk_dinfo ()
#define _ATK_CLASS_FILE_ACM _atk_dinfo ()
#endif /* USE_ATT */

#if RTRT_STATIC_IN_INLINE_ALLOWED
#define _ATK_STATIC_CONST static const
#else
#define _ATK_STATIC_CONST
#endif /* RTRT_STATIC_IN_INLINE_ALLOWED */

#define _ATK_DEFINE_METHOD_NAME(n) _ATK_STATIC_CONST char *_atk_method_name=n;

#if USE_ATT
#define _DINFO_PARAM _atl_true
#else
#define _DINFO_PARAM
#endif /* USE_ATT */

#if ATL_MULTI_THREADS && USE_ATT
#define _ATK_THREAD_NO 1
#else
#define _ATK_THREAD_NO
#endif /* ATL_MULTI_THREADS && USE_ATT */

#define _ATK_ENTER_METHOD \
   if (!_atk_get_lock(_ATK_THREAD_NO)) \
      _atk_enter_method(_atk_method_name, _atk_dinfo (_DINFO_PARAM));

#define _ATK_EXIT_METHOD \
   if (!_atk_get_lock(_ATK_THREAD_NO)) \
      _atk_exit_method(_atk_method_name, _atk_dinfo (_DINFO_PARAM));

#if USE_ATT
#define _ATK_DINFO_CLS_PARAM _att_object.StackLink->InstanceId?_atl_false:_atl_true
//#define _ATK_DINFO_CLS_PARAM tracer_inst?_atl_false:_atl_true
#else
#define _ATK_DINFO_CLS_PARAM
#endif /* USE_ATT */

#define _atk_transition(i) _atk_transition_i(i,_atk_dinfo(_ATK_DINFO_CLS_PARAM))

#define _ATK_TEST_CLASS_FRIEND friend class ::_ATO_TestClass;
#define _ATK_TEST_CLASS_DECL class _ATO_TestClass;

/* Backward-compatibility macros */
#ifndef _ATO_Car
#define _ATO_Car _atk_Car
#define _ATO_Int _atk_Int
#define _ATO_Uint _atk_Uint
#define _ATO_Ulong _atk_Ulong
#endif

#define _ATO_check_state _atk_check_state
#define _ATO_State _atk_state
#define _ATO_StateType _atk_StateType
#define ATO_TransDesc atk_TransDesc
#define ATO_StateDesc atk_StateDesc
#define _ATO_old _atk_old

#define _ATO_CLASS_FILE_CLS _ATK_CLASS_FILE_CLS
#define _ATO_CLASS_FILE_ACM _ATK_CLASS_FILE_ACM
#define _ATO_STATIC_CONST _ATK_STATIC_CONST

#define _ATO_assert_invariant _atk_assert_invariant
#define _ATO_assert_pre_cond _atk_assert_pre_cond
#define _ATO_assert_post_cond _atk_assert_post_cond
#define _ATO_illegal_transition _atk_illegal_transition
#define _ATO_multiple_transitions _atk_multiple_transitions
#define _ATO_current_state _atk_current_state
#define _ATO_transition(i) _atk_transition(i)

#endif
