

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: ato.c,v 1.34.2.1 2008/03/18 10:55:49 build Exp $
* $Descr: Object Testing Library Source (included by TP.c[pp]) $
*******************************************************************************/

/*******************************************************************************
* Global variables
*******************************************************************************/

/*! ATO-instances stack */
static _ato_Id instances[ATO_MAX_INSTANCES];

/*! Run/call/check-stub stack */
static _ato_Id messages[ATO_MAX_INSTANCES];

/*! inst_files[n] is the source file of instance instances[n] */
static _ato_Id inst_files[ATO_MAX_INSTANCES];

/*! Current instances stack level (0 = test script) */
static int current_depth;

/*! Current run/call/check_stub stack level (0 = global RUN) */
static int current_message;

/*! Last ON ERROR message id encountered */
static unsigned int mc_id;

/*! Last ON ERROR id block encountered */
static _ato_Id last_error;

/*! Last ON ERROR's file dynamic number */
static _ato_Id last_error_file;

/*! Current PRINT id */
static _ato_Id current_print;

/*! Current PRINT file id */
static _ato_File current_print_file;

#if ATO_PRINT_BUFFER_SIZE
/*! PRINT buffer */
static char print_buffer[ATO_PRINT_BUFFER_SIZE];

/*! Pointer to the next character in print_buffer to be filled */
static char *print_buffer_next;

/*! Number of remaining characters in print_buffer */
static int print_buffer_remaining;

/*! A PRINT buffer is being filled */
static _ato_Boolean in_print;
#endif /* ATO_PRINT_BUFFER_SIZE */

#if USE_ATC || USE_ATP || USE_ATQ
/*! Current block name is updated here */
static char *_ato_obstools_block_name;
#endif /* USE_ATC || USE_ATP || USE_ATQ */

#if USE_ATK && USE_ATT
RTRT_GLOBALVAR ATL_TP_PROTO_PREFIX _ato_Boolean _ato_in_stub;
#endif

/*******************************************************************************
* TSF special IDs
*******************************************************************************/

#define _ATO_TSF_ON_ERROR_DEFAULT (RTRT_U_INT32)(-1)
#define _ATO_TSF_UNEXPECTED_EXCEPTION "fffffffeh"
#define _ATO_TSF_EXCEPTION_THROWN "fffffffdh"

/*******************************************************************************
* Macros
*******************************************************************************/

#if ATO_BREAK_ON_CHECK_FAILED
/*! Executed code on error */
#define _ATO_BREAK priv_check_failed ();
#else
#define _ATO_BREAK
#endif /* _ATO_BREAK_ON_ERROR */

/*! Current instance dynamic number in instances stack */
#define _ATO_CUR_INSTANCE instances[current_depth] ? \
   instances[current_depth] : \
   instances[current_depth]=++_atl_highest_instance

/*! Push an instance in instances stack */
#define _ATO_PUSH_INSTANCE instances[current_depth++]

/*! Pop the current instance from instances stack */
#define _ATO_POP_INSTANCE instances[--current_depth]

/*! Retrieves previous instance in instances stack */
#define _ATO_PREV_INSTANCE current_depth ? instances[current_depth-1] : 0

/*! File dynamic number of current instance */
#define _ATO_CUR_INST_FILE inst_files[current_depth]

/*! File dynamic number of previous instance (0 if current instance \
is the OTD script */
#define _ATO_PREV_INST_FILE current_depth ? inst_files[current_depth-1] : 0

/*! Current message in run/call/check-stub stack */
#define _ATO_CUR_MESSAGE messages[current_message-1]

/*! Pops a message in run/call/check-stub stack */
#define _ATO_POP_MESSAGE messages[--current_message]

/*! Push message m in run/call/check-stub stack */
#define _ATO_PUSH_MESSAGE(m) messages[current_message++]=_ATO_MSG(m)

/*! Generates a CI instruction if previous instance has changed to another
 *  because of another TDF user (atk,att) */
#if USE_ATT || USE_ATK
#define _ATO_CHANGE_INSTANCE \
   if (_ATO_PREV_INSTANCE != _atl_current_instance) _ato_back_to_instance ();
#else
#define _ATO_CHANGE_INSTANCE
#endif

#if ATL_MULTI_THREADS && USE_ATT
#define _ATO_CHECK_THREAD int thread_index = _att_check_thread((_AtlStackLinkP)0);
#else
#define _ATO_CHECK_THREAD
#endif

/*! TSF ID of an Object Testing instance */
#define _ATO_INST(i) i+_ATL_TDF_INSTANCE_OFFSET
/*! TSF ID of an Object Testing run/call/check-stub */
#define _ATO_MSG(i) i+_ATL_TDF_MESSAGE_OFFSET
/*! TSF ID of an ObjectTesting text */
#define _ATO_TEXT(i) i+_ATL_TDF_TEXT_OFFSET

#define ATL_LINE_MUTEX_BEGIN RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
#define ATL_LINE_MUTEX_END RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

/*******************************************************************************
* Runtime routines
*******************************************************************************/

static void _ato_back_to_instance ()
{
   if(_ATO_PREV_INST_FILE)
   {
      _atl_print_change_source(_ATO_PREV_INST_FILE);
   }
   RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
   _ATO_CHECK_THREAD
   ATL_LINE_ADD_OPCODE(ATL_TDF_CI);
   atl_line_inc (_atl_tdf_id_image (_atl_current_instance=_ATO_PREV_INSTANCE,
				    atl_line_cur));
   _atl_tdf_line_dump();
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

void _ato_enter_test_class (_ato_Id tclass,
			    _ato_File tcl_file,
                            _ato_Id run,
			    _ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _ATO_CHANGE_INSTANCE
   _atl_print_change_source(run_file);
   _ATO_CUR_INST_FILE=tcl_file;
   _atl_tdf_line_2(ATL_TDF_ME,_ATO_PUSH_MESSAGE(run),_ATO_CUR_INSTANCE);
   _atl_print_change_source(tcl_file);
   _atl_tdf_line_2(ATL_TDF_NI,_ATO_INST(tclass), _atl_current_instance = _ATO_PUSH_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,tclass);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_test_class (_ato_File tclass_file,_ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_tdf_line_1(ATL_TDF_CF,run_file);
   _atl_tdf_line_1(ATL_TDF_MR,_ATO_CUR_MESSAGE);
   _atl_print_change_source(tclass_file);
   _atl_tdf_line_2(ATL_TDF_DI,_ATO_POP_INSTANCE,_ATO_POP_MESSAGE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_enter_test_suite (_ato_Id suite,
			    _ato_File ts_file,
			    _ato_Id run,
                            _ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _ATO_CHANGE_INSTANCE
   _atl_print_change_source(run_file);
   _ATO_CUR_INST_FILE=ts_file;
   _atl_tdf_line_2(ATL_TDF_ME,_ATO_PUSH_MESSAGE(run),_ATO_CUR_INSTANCE);
   _atl_print_change_source(ts_file);
   _atl_tdf_line_2(ATL_TDF_NI,_ATO_INST(suite),_atl_current_instance = _ATO_PUSH_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,suite);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_test_suite (_ato_File suite_file,_ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_tdf_line_1(ATL_TDF_CF,run_file);
   _atl_tdf_line_1(ATL_TDF_MR,_ATO_CUR_MESSAGE);
   _atl_print_change_source(suite_file);
   _atl_tdf_line_2(ATL_TDF_DI,_ATO_POP_INSTANCE,_ATO_POP_MESSAGE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_enter_test_case (_ato_Id tcase,
			   _ato_File tc_file,
			   _ato_Id run,
			   _ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _ATO_CHANGE_INSTANCE
   _atl_print_change_source(run_file);
   _ATO_CUR_INST_FILE=tc_file;
   _atl_tdf_line_2(ATL_TDF_ME,_ATO_PUSH_MESSAGE(run),_ATO_CUR_INSTANCE);
   _atl_print_change_source(tc_file);
   _atl_tdf_line_2(ATL_TDF_NI,_ATO_INST(tcase),_atl_current_instance = _ATO_PUSH_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,tcase);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_test_case (_ato_File tcase_file,_ato_File run_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_tdf_line_1(ATL_TDF_CF,run_file);
   _atl_tdf_line_1(ATL_TDF_MR,_ATO_CUR_MESSAGE);
   _atl_print_change_source(tcase_file);
   _atl_tdf_line_2(ATL_TDF_DI,_ATO_POP_INSTANCE,_ATO_POP_MESSAGE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_enter_proc (_ato_Id proc,
		      _ato_File proc_file,
		      _ato_Id call,
		      _ato_Id call_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _ATO_CHANGE_INSTANCE
   _atl_print_change_source(call_file);
   _ATO_CUR_INST_FILE=proc_file;
   _atl_tdf_line_2(ATL_TDF_ME,_ATO_PUSH_MESSAGE(call),_ATO_CUR_INSTANCE);
   _atl_print_change_source(proc_file);
   _atl_tdf_line_2(ATL_TDF_NI,_ATO_INST(proc),_atl_current_instance = _ATO_PUSH_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,proc);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_proc (_ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(file);
   _atl_tdf_line_1(ATL_TDF_MR,_ATO_CUR_MESSAGE);
   _atl_tdf_line_2(ATL_TDF_DI,_ATO_POP_INSTANCE,_ATO_POP_MESSAGE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_enter_stub (_ato_Id stub,
		      _ato_File stub_file,
		      _ato_Id chk_stub,
		      _ato_File chk_stub_file)
{
   atl_early_init();
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
#if !USE_ATT
   _ATO_CHANGE_INSTANCE
#endif
   _atl_print_change_source(chk_stub_file);
   _ATO_CUR_INST_FILE=stub_file;
   _atl_tdf_line_2(ATL_TDF_ME,_ATO_PUSH_MESSAGE(chk_stub),_ATO_CUR_INSTANCE);
   _atl_print_change_source(stub_file);
   _atl_tdf_line_2(ATL_TDF_NI,_ATO_INST(stub),_atl_current_instance = _ATO_PUSH_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,stub);
#if USE_ATK && USE_ATT
   _ato_in_stub = _atl_true;
#endif
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_stub (_ato_File stub_file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
#if USE_ATK && USE_ATT
   _ato_in_stub = _atl_false;
#endif
   _atl_print_change_source(stub_file);
   _atl_tdf_line_1(ATL_TDF_MR,_ATO_CUR_MESSAGE);
   _atl_tdf_line_2(ATL_TDF_DI,_ATO_POP_INSTANCE,_ATO_POP_MESSAGE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_prologue (_ato_Id prologue, _ato_File pfile)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(pfile);
   _atl_tdf_line_1(ATL_TDF_OT,prologue);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_epilogue (_ato_Id epilogue, _ato_File efile)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(efile);
   _atl_tdf_line_1(ATL_TDF_OT,epilogue);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_enter_error (_ato_Id error,
		       _ato_Id file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   error = error ? _ATO_MSG(error) : _ATO_TSF_ON_ERROR_DEFAULT;
   last_error_file = file;
   mc_id = ++_atl_last_mc_id;
   _atl_tdf_line_3(ATL_TDF_MC,mc_id, last_error = error,_ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exit_error (_ato_Id to_instance)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(last_error_file);
   _atl_tdf_line_3(ATL_TDF_ME,last_error,_ATO_PREV_INSTANCE,mc_id);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

_ato_Boolean _ato_check (_ato_Boolean eval,
                         _ato_Id chk,
			 _ato_File file)
{
   RTRT_U_INT32  r;
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);

   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line_1(_ATO_STYLE_GLYPH_KO,_ATO_TEXT(chk));
      r= 0;
   }
   else
   {
   	  _atl_tdf_line_1(_ATO_STYLE_GLYPH_OK,_ATO_TEXT(chk));
      r= 1;
   }

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   _atl_tdf_line_2(ATL_TDF_OT,chk,r);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_ato_Boolean _ato_check_property (_ato_Boolean eval,
                                  _ato_Id chk_prop,
			          _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line(_ATO_STYLE_CHECK_PROPERTY_KO);
#if ATO_DUMP_SUCCESS
   }
   else
   {
      _atl_tdf_line(_ATO_STYLE_CHECK_PROPERTY_OK);
   }
#endif /* ATO_DUMP_SUCCESS */
      _atl_tdf_line_1(ATL_TDF_NO,_ATO_TEXT(chk_prop));
      _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
#if !ATO_DUMP_SUCCESS
   }
#endif /* !ATO_DUMP_SUCCESS */
   _atl_tdf_line_2(ATL_TDF_OT,chk_prop,eval ?  1 :  0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_ato_Boolean _ato_check_method (_ato_Boolean eval,
                                _ato_Id chk_mth,
				_ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line(_ATO_STYLE_CHECK_METHOD_KO);
#if ATO_DUMP_SUCCESS
   }
   else
   {
      _atl_tdf_line(_ATO_STYLE_CHECK_METHOD_OK);
   }
#endif /* ATO_DUMP_SUCCESS */
      _atl_tdf_line_1(ATL_TDF_NO,_ATO_TEXT(chk_mth));
      _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
#if !ATO_DUMP_SUCCESS
   }
#endif /* !ATO_DUMP_SUCCESS */
   _atl_tdf_line_2(ATL_TDF_OT,chk_mth,eval ?  1 :  0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_ato_Boolean _ato_check_exception (_ato_Boolean eval,
                                   _ato_Id chk_exc,
				   _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line(_ATO_STYLE_CHECK_EXCEPTION_KO);
#if ATO_DUMP_SUCCESS
   }
   else
   {
      _atl_tdf_line(_ATO_STYLE_CHECK_EXCEPTION_OK);
   }
#endif /* ATO_DUMP_SUCCESS */
      _atl_tdf_line_1(ATL_TDF_NO,_ATO_TEXT(chk_exc));
      _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
#if !ATO_DUMP_SUCCESS
   }
#endif /* !ATO_DUMP_SUCCESS */
   _atl_tdf_line_2(ATL_TDF_OT,chk_exc,eval ?  1 :  0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_ato_Boolean _ato_check_prepos (_ato_Boolean eval,
                                _ato_Id statmt,
			        _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _atl_print_change_source(file);
   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line(_ATO_STYLE_PREPOS_KO);
#if ATO_DUMP_SUCCESS
   }
   else
   {
      _atl_tdf_line(_ATO_STYLE_PREPOS_OK);
   }
#endif /* ATO_DUMP_SUCCESS */
      _atl_tdf_line_2("AC ",_ATO_TEXT(statmt),_ATO_PREV_INSTANCE);
#if !ATO_DUMP_SUCCESS
   }
#endif /* !ATO_DUMP_SUCCESS */
   _atl_tdf_line_2(ATL_TDF_OT,statmt,eval ?  1 :  0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

_ato_Boolean _ato_check_stub (_ato_Boolean eval,
		              _ato_Id chk_stb,
		              _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   if (!eval)
   {
      _ATO_BREAK
      _atl_tdf_line(_ATO_STYLE_CHECK_STUB_KO);
#if ATO_DUMP_SUCCESS
   }
   else
   {
      _atl_tdf_line(_ATO_STYLE_CHECK_STUB_OK);
#endif /* ATO_DUMP_SUCCESS */
   }
   _atl_tdf_line_1(ATL_TDF_NO,_ATO_TEXT(chk_stb));
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   _atl_tdf_line_2(ATL_TDF_OT,chk_stb,eval ?  1 :  0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
   return eval;
}

void _ato_unexpected_exception (_ato_Id where,
			        _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   _ATO_BREAK
   _atl_tdf_line(_ATO_STYLE_UNEXPECTED_EXCEPTION);
   _atl_tdf_line(ATL_TDF_NO" "_ATO_TSF_UNEXPECTED_EXCEPTION);
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   _atl_tdf_line_2(ATL_TDF_OT,where, 0);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_exception_in_stub ()
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(_ATO_PREV_INST_FILE);
   _atl_tdf_line(_ATO_STYLE_EXCEPTION_THROWN);
   _atl_tdf_line(ATL_TDF_NO" "_ATO_TSF_EXCEPTION_THROWN);
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   _atl_tdf_line_2(ATL_TDF_OT,_atl_current_instance, 2);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_comment (_ato_Id comment,
		   _ato_File file)
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(file);
   _atl_tdf_line(_ATO_STYLE_COMMENT);
   _atl_tdf_line_1(ATL_TDF_NO,_ATO_TEXT(comment));
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   _atl_tdf_line_1(ATL_TDF_OT,comment);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

#if ATO_PRINT_BUFFER_SIZE
static void _ato_print_buffer_add (const char * s)
{
  while ((*s != '\0') && (print_buffer_remaining > 3))
  {
    if (*s=='\n')
    {
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = 'n';
      s++;
    }
    else if (*s=='\r')
    {
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = 'r';
      s++;
    }
    else if (*s==RTRT_QUOT)
    {
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = RTRT_QUOT;
      s++;
    }
    else if (*s=='\\')
    {
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = '\\'; print_buffer_remaining--;
      *print_buffer_next++ = '\\';
      s++;
    }
    else
      *print_buffer_next++ = *s++;
    print_buffer_remaining--;
  }
  *print_buffer_next = '\0';
}

static void _ato_print_buffer_inc (int l)
{
   if (l > print_buffer_remaining)
   {
      return;
   }
   print_buffer_next += l;
   print_buffer_remaining -= l;
}
#endif /* ATO_PRINT_BUFFER_SIZE */

void _ato_print_start (_ato_Id print, _ato_File file)
{
   current_print = print;
   current_print_file = file;
#if ATO_PRINT_BUFFER_SIZE
   print_buffer_next = print_buffer;
   print_buffer_remaining = ATO_PRINT_BUFFER_SIZE;
   _ato_print_buffer_add (ATL_TDF_NO);
   /*
     _ato_print_buffer_add (" ");
     _ato_print_buffer_inc (_atl_tdf_id_image (current_print, print_buffer_next));
   */
   *print_buffer_next++ = ' ';
   *print_buffer_next++ = RTRT_QUOT;
   print_buffer_remaining-=6; /* 6 additional chars for internal purpose */
   _ato_print_buffer_add ( (char *)ATL_MIME_NAME );
    *print_buffer_next++ = RTRT_QUOT;
    *print_buffer_next++ = ' ';
    *print_buffer_next++ = RTRT_QUOT;

   in_print = _atl_false;
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print_end ()
{
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source (current_print_file);
#if ATO_PRINT_BUFFER_SIZE
   *print_buffer_next++ = RTRT_QUOT;
   *print_buffer_next = '\0';
   _atl_tdf_line(_ATO_STYLE_PRINT);
   _atl_tdf_line(print_buffer);
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
#endif /* ATO_PRINT_BUFFER_SIZE */
   _atl_tdf_line_1(ATL_TDF_OT,current_print);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
}

void _ato_print (char c, char *expr)
{
   char s[2] = { c, (char)0 };

#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   /*
   _ato_print_buffer_inc (char_image (c, print_buffer_next));
   */
   _ato_print_buffer_add (s);
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   _atl_tdf_note_string ((char *)s);

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}



#if RTRT_STRING
void _ato_print (const char *s, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   _ato_print_buffer_add ("\"\\\"");
   */
   /*
   _ato_print_buffer_inc (string_image (s, print_buffer_next,
				        print_buffer_remaining));
   */
   _ato_print_buffer_add (s);
   /*
   _ato_print_buffer_add ("\\\"\"");
   */
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   _atl_tdf_note_string ((char *)s);

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}
#endif /* RTRT_STRING */

void _ato_print (const void *p, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (_atl_tdf_id_image ((unsigned long int)p,
					     print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (unsigned_image_16 ((unsigned long int)p, atl_line_cur));
   ATL_LINE_ADD_OPCODE ("h\"");
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print (signed int n, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (integer_image_10 (n, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);

   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (integer_image_10 (n, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print (unsigned int p, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (unsigned_image_10 (p, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (unsigned_image_10 (p, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print (signed long int n, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (integer_image_10 (n, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (integer_image_10 (n, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print (unsigned long int p, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (unsigned_image_10 (p, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (unsigned_image_10 (p, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END

   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

#ifdef RTRT_INT64

void _ato_print (RTRT_INT64 n, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (integer_image_10 (n, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (integer_image_10 (n, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

#endif /* RTRT_INT64 */

#ifdef RTRT_U_INT64

void _ato_print (RTRT_U_INT64 p, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (unsigned_image_10 (p, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (unsigned_image_10 (p, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

#endif /* RTRT_U_INT64 */

#if RTRT_FLOAT
void _ato_print (float f, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (float_image (f, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (float_image (f, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_print (double d, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (double_image (d, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line(_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (double_image (d, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}

#if RTRT_LONG_DOUBLE
void _ato_print (long double d, char *expr)
{
#if ATO_PRINT_BUFFER_SIZE
  /*
   if (in_print)
      _ato_print_buffer_add (", ");
   _ato_print_buffer_inc (string_image (expr, print_buffer_next,
				        print_buffer_remaining));
   _ato_print_buffer_add (" = ");
   */
   _ato_print_buffer_inc (double_image (d, print_buffer_next));
   in_print = _atl_true;
#else
   RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
   _ATO_CHECK_THREAD
   _atl_print_change_source(current_print_file);
   _atl_tdf_line (_ATO_STYLE_PRINT);

   ATL_LINE_MUTEX_BEGIN
   ATL_LINE_ADD_OPCODE(ATL_TDF_NO " \"");
   /*
   atl_buffer_string_image (expr);
   ATL_LINE_ADD_OPCODE (" = ");
   */
   atl_line_inc (double_image (d, atl_line_cur));
   atl_line_add_char (RTRT_QUOT);
   _atl_tdf_line_dump ();
   ATL_LINE_MUTEX_END
   _atl_tdf_line_1(ATL_TDF_CI,_atl_current_instance = _ATO_PREV_INSTANCE);
   RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
#endif /* ATO_PRINT_BUFFER_SIZE */
}
#endif /* RTRT_LONG_DOUBLE */
#endif /* RTRT_FLOAT */

void _ato_init (char *tracefile,
	        _ato_Id instance_off,
		_ato_Id message_off,
		_ato_Id text_off)
{
  atl_early_init ();
  current_print = (_ato_Id)0;
  current_print_file = (_ato_File)0;
#if ATO_PRINT_BUFFER_SIZE
  in_print = _atl_false;
#endif /* ATO_PRINT_BUFFER_SIZE */
}

void _ato_term ()
{
   atl_exit ();
}

extern "C" {

static void ato_early_init ()
{
  current_depth = 0;
  current_message = 0;
  last_error = (_ato_Id)0;
#if USE_ATC || USE_ATP || USE_ATQ
  _ato_obstools_block_name = (char *)0;
#endif /* USE_ATC || USE_ATP || USE_ATQ */
#if USE_ATK && USE_ATT
  _ato_in_stub = _atl_false;
#endif
}

#ifdef _ATL_EXIT_PENDING
static void ato_exit_pending ()
#else
static void ato_late_exit ()
#endif
{
  _ato_obstools_set_block_name ("CleanUp");
  /* mark the end of the test */
  _ato_obstools_dump ();
}

} /* extern "C" */

RTRT_U_INT32 _ato_add_source_file (RTRT_U_INT32 key,
                                  RTRT_U_INT32 checksum)
{
  RTRT_U_INT32 result;
  RTRT_MT_CRITICAL_VAR_LOCK(atl_tdf_mutex)
  result = ++_atl_highest_source;
  _atl_print_source_file (result, key, checksum);
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_tdf_mutex)
  return result;
}

void _ato_obstools_set_block_name (char *name)
{
#if USE_ATC || USE_ATP || USE_ATQ
  _ato_obstools_block_name = name;
#endif /* USE_ATC || USE_ATP || USE_ATQ */
}

void _ato_obstools_dump ()
{
#if USE_ATC || USE_ATP || USE_ATQ
  _atl_obstools_dump (1);
#endif /* USE_ATC || USE_ATP || USE_ATQ */
}

