
/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atp.c,v 1.118.2.1 2008/03/18 10:55:49 build Exp $
* $Descr: PurifyLT Library Source (included by TP.c[pp]) $
*******************************************************************************/
#if ATP_BREAK_ON_ERROR
#define CALL_ON_ATP_ERROR priv_check_failed();
#else
#define CALL_ON_ATP_ERROR
#endif /* BREAK_ON_ATP_ERROR */
#ifndef ATP_HEAP_INIT
#define ATP_HEAP_INIT
#endif /* ATP_HEAP_INIT */

/* possible print or call to function when customer need it */
#ifndef CALL_ON_ATP_BLOCK_ERROR
#define CALL_ON_ATP_BLOCK_ERROR CALL_ON_ATP_ERROR
#endif /* CALL_ON_ATP_BLOCK_ERROR */
/*******************************************************************************
* Library Allocation Functions:
*******************************************************************************/

#if ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_USR )
#define _atp_malloc        malloc
#define _atp_calloc        calloc
#define _atp_realloc       realloc
#define _atp_free          free
#endif

#ifndef RTRT_DO_MALLOCH
#define RTRT_DO_MALLOCH RTRT_DO_MALLOC
#endif /* RTRT_DO_MALLOCH */

#ifndef RTRT_DO_MALLOCB
#define RTRT_DO_MALLOCB RTRT_DO_MALLOC
#endif /* RTRT_DO_MALLOCB */

#ifndef RTRT_DO_MALLOCF
#define RTRT_DO_MALLOCF RTRT_DO_MALLOC
#endif /* RTRT_DO_MALLOCF */

#ifndef RTRT_DO_FREEH
#define RTRT_DO_FREEH RTRT_DO_FREE
#endif /* RTRT_DO_FREEH */

#ifndef RTRT_DO_FREEB
#define RTRT_DO_FREEB RTRT_DO_FREE
#endif /* RTRT_DO_FREEB */

#ifndef RTRT_DO_FREEF
#define RTRT_DO_FREEF RTRT_DO_FREE
#endif /* RTRT_DO_FREEF */

#ifndef RTRT_DO_FREEM /* can be defined by the TDP to make any special action */
/* pt is the memory ptr to free, px is a ptr to the freeParam, bp is a ptr on AtpBlock structure */
#define RTRT_DO_FREEM(pt,px,bp) RTRT_DO_FREE(pt)
#endif /* RTRT_DO_FREEM */

#ifndef ATP_REPORTS_FUM /* custom may avoid FUM reports */
#define ATP_REPORTS_FUM 1
#endif /* ATP_REPORTS_FUM */

/*******************************************************************************
* Internal block data structure
*******************************************************************************/

typedef struct _AtpBlock_ {
        char                *Ptr;
        struct _AtpBlock_   *Next;
        struct _AtpBlock_   *Dump;
#if ATP_CHECK_SELECTED_BLOCKS
        struct _AtpBlock_   *ChkPrev;
        struct _AtpBlock_   *ChkNext;
#endif
        _AtlCallStackInfo    StackInfoTable[ATP_CALL_STACK_SIZE];
        RTRT_SIZE_T          Size;
        RTRT_U_INT16         Status;
#ifdef NEED_BLOCK_DATA
		_AtpBlockData		 AtpBlockData; /* could be used for special TDP */
#endif
} _AtpBlock, *_AtpBlockP;

#if ATP_REPORTS_FFM_FMWL

typedef struct _AtpFreeQ_ {
        _AtpBlockP           Block;
        _AtlCallStackInfo    StackInfoTable[ATP_CALL_STACK_SIZE];
        struct _AtpFreeQ_   *Next;
#if ATP_KEEP_CONTENT
        RTRT_U_INT16         BlockCks;
#endif
#if RTRT_FREE_PARAM
		_freeParam			 freeParam;
#endif
} _AtpFreeQ, *_AtpFreeQP;


#endif /* ATP_REPORTS_FFM_FMWL */

/* Size of hash key in bits; number of entries depends of that */
/* Should be modifiable from settings */
/* Let's start with 4096 entries on native targets and 64 on crossed ones */
#ifndef ATP_HASH_SIZE
#if RTRT_IO != RTRT_STD
#define ATP_HASH_SIZE 6
#else
#define ATP_HASH_SIZE 12
#endif
#endif

#define _ATP_ENTRIES_CARD (1<<ATP_HASH_SIZE)
#if RTRT_64_BITS_PTR == 0
#define _ATP_ENTRIES_MASK ((_ATP_ENTRIES_CARD-1)<<3)
#define _ATP_HASH(ptr) (((RTRT_U_INT32)ptr)&_ATP_ENTRIES_MASK)>>3
#else
#define _ATP_ENTRIES_MASK (_ATP_ENTRIES_CARD-1)
#define _ATP_HASH(ptr) ((RTRT_U_INT32)(((RTRT_U_INT64)ptr>>3)&0xFFFFFFFFUL)&_ATP_ENTRIES_MASK)
#endif /* RTRT_64_BITS_PTR */

/* _AtpBlock.Status is made of  0b 11 00  00 11  1 11 0 11 11 */
typedef enum {
        _ATP_BLOCK_STATUS_MLK,
        _ATP_BLOCK_STATUS_MPK,
        _ATP_BLOCK_STATUS_MIU
} _AtpBlockStatus;

/* MIU/MPK pointer origin */
#define _ATP_BLOCK_AREA_GLOBAL    0 /* 0<<2, generate 1 in the report */
#define _ATP_BLOCK_AREA_HEAP      4 /* 1<<2, report 2*/
#define _ATP_BLOCK_AREA_STACK     8 /* 2<<2, report 3*/
#define _ATP_BLOCK_AREA_HEAP_MPK 12 /* 3<<2, report 4*/

/* _GET_ align value to bit0, _IS_ does not align */
#define _ATP_SET_BLOCK_STATUS(s,v) ((s)&=0xFFFC,(s)|=((RTRT_U_INT16)(v)))
#define _ATP_GET_BLOCK_STATUS(s)   ((_AtpBlockStatus)((s)&0x03))

/* set the Area as well as Status(MIU/MPK) and clear the RR bit */
#define _ATP_SET_BLOCK_AREA_STATUS(s,v,a) ((s)&=0xFFF0,(s)|=((RTRT_U_INT16)(v)|(RTRT_U_INT16)(a)))

#define _ATP_SET_AREA_STATUS(s,v) ((s)&=0xFFF3,(s)|=((RTRT_U_INT16)(v)))
#define _ATP_GET_AREA_STATUS(s)   (((s)&0x0C)>>2)

#define _ATP_SET_BLOCK_REV(s) ((s)|=0x10)
#define _ATP_IS_BLOCK_REV(s)  ((s)&0x10)

#define _ATP_CLEAR_BLOCK_STATUS(s) ((s)&=0xFFE0) /* clear STATUS and AREA and RR */


#define _ATP_SET_ABWL_STATUS_UPSTREAM(s) ((s)|=0x20)
#define _ATP_IS_ABWL_STATUS_UPSTREAM(s)  ((s)&0x20)
#define _ATP_SET_ABWL_STATUS_DOWNSTREAM(s) ((s)|=0x40)
#define _ATP_IS_ABWL_STATUS_DOWNSTREAM(s)  ((s)&0x40)

#define _ATP_SET_FMWL_STATUS(s) ((s)|=0x80)
#define _ATP_IS_FMWL_STATUS(s)    ((s)&0x80)

#define _ATP_SET_BLOCK_STATUS_FREE(s) ((s)|=((RTRT_U_INT16)(0x100)))
#define _ATP_IS_BLOCK_STATUS_FREE(s) (((s)&0x100))
#define _ATP_SET_BLOCK_STATUS_CHK(s) ((s)|=((RTRT_U_INT16)(0x200)))
#define _ATP_IS_BLOCK_STATUS_CHK(s) (((s)&0x200))

#define _ATP_SET_ALLOC_STATUS(s,v) ((s)&=0x3FFF,(s)|=((RTRT_U_INT16)(v))<<14)
#define _ATP_GET_ALLOC_STATUS(s)   (s>>14)

#if ATP_REPORTS_ABWL

#define _ATP_USER_SIZE(s) ((s)-ATP_RED_ZONE_HEAD_SIZE-ATP_RED_ZONE_TAIL_SIZE)
#define _ATP_ACTUAL_SIZE(s) ((s)+ATP_RED_ZONE_HEAD_SIZE+ATP_RED_ZONE_TAIL_SIZE)
#define _ATP_ACTUAL_USER_DATA_CHAR(p) (((char *)(p))+ATP_RED_ZONE_HEAD_SIZE)
#define _ATP_ACTUAL_USER_DATA(p) (void *)_ATP_ACTUAL_USER_DATA_CHAR(p)
#define _ATP_ACTUAL_INTERNAL_DATA(p) (((char *)(p))-ATP_RED_ZONE_HEAD_SIZE)

#else /* ATP_REPORTS_ABWL */

#define _ATP_USER_SIZE(s) (s)
#define _ATP_ACTUAL_SIZE(s) (s)
#define _ATP_ACTUAL_USER_DATA_CHAR(p) (char *)(p)
#define _ATP_ACTUAL_USER_DATA(p) (p)
#define _ATP_ACTUAL_INTERNAL_DATA(p) (char *)(p)

#endif /* ATP_REPORTS_ABWL */

/*******************************************************************************
* Global variables
*******************************************************************************/

#ifndef _ATP_DEBUG_HASH
#define _ATP_DEBUG_HASH 0
#endif

/* Hash Table */
RTRT_GLOBALVAR _AtpBlockP *_atp_hash_table;
#if _ATP_DEBUG_HASH
RTRT_GLOBALVAR int _atp_hash_table_count[_ATP_ENTRIES_CARD];
RTRT_GLOBALVAR int _atp_hash_table_max[_ATP_ENTRIES_CARD];
#endif

#if ATP_REPORTS_FFM_FMWL
RTRT_GLOBALVAR _AtpFreeQP _atp_first_freeq, _atp_last_freeq;
RTRT_GLOBALVAR RTRT_U_INT32 _atp_freeq_length;
RTRT_GLOBALVAR RTRT_U_INT32 _atp_freeq_size;
#endif /* ATP_REPORTS_FFM_FMWL */

#if ATP_REPORTS_FFM_FMWL || ATP_REPORTS_ABWL
RTRT_GLOBALVAR unsigned char _atp_dead_beef[4];
#endif /* ATP_REPORTS_FFM_FMWL */

#if ATP_CHECK_SELECTED_BLOCKS
RTRT_GLOBALVAR _AtpBlockP _atp_chk_list;
#endif
RTRT_STATICVAR _AtpBlockP _atp_dump_list;

#ifndef RTRT_NO_HOOK_GUARD
#if _ATP_ALLOC_API_NEEDED
#if ATL_MULTI_THREADS == ATL_YES
RTRT_GLOBALVAR RTRT_U_INT8 _atp_hook_guard_table[ATL_THREADS_MAX+1];
#define _atp_hook_guard \
   _atp_hook_guard_table[thread_index]
#else /* ATL_MULTI_THREADS == ATL_YES */
RTRT_GLOBALVAR RTRT_U_INT8 _atp_hook_guard;
#endif /* ATL_MULTI_THREADS == ATL_YES */
#endif /* _ATP_ALLOC_API_NEEDED */
#endif /* RTRT_NO_HOOK_GUARD */

/* Some Statistic Variables */
RTRT_GLOBALVAR RTRT_U_INT32 _atp_total_blocks;
RTRT_GLOBALVAR RTRT_U_INT32 _atp_unfreed_blocks;
RTRT_GLOBALVAR RTRT_U_INT32 _atp_max_blocks;
RTRT_GLOBALVAR RTRT_SIZE_T _atp_total_bytes;
RTRT_GLOBALVAR RTRT_SIZE_T _atp_unfreed_bytes;
RTRT_GLOBALVAR RTRT_SIZE_T _atp_max_bytes;
#if USE_ATT && ATT_ON_THE_FLY
RTRT_STATICVAR RTRT_U_INT8  _atl_MPK_detected;
RTRT_STATICVAR RTRT_U_INT8  _atl_MLK_detected;
#endif
/* Min and Max Allocated Addresses */
RTRT_GLOBALVAR char *_atp_minimum_addr;
RTRT_GLOBALVAR char *_atp_maximum_addr;

#if USE_ATU || USE_ATO
/* Test driver top stack addr */
RTRT_GLOBALVAR char *_atp_test_driver_top_stack;
#endif

/* First of Global Data Checker Functions */
RTRT_GLOBALVAR _AtpGlobalsFuncTypeP _atp_first_globals_func_data;

/* Test started or not */
RTRT_GLOBALVAR short _atp_test_started;

#if ATL_MULTI_THREADS
RTRT_MT_CRITICAL_VAR_DECL(atp_critical)
#endif /* ATL_MULTI_THREADS */

#if ATP_REPORTS_FIU
#if RTRT_IO == RTRT_STD
RTRT_GLOBALVAR RTRT_U_INT8 _atp_previously_opened[ATL_STD_IO_MAX_FD-3];
#endif /* RTRT_IO == RTRT_STD */
#endif /* ATP_REPORTS_FIU */

RTRT_STATICFCT void _atp_dump_err
#if !RTRT_KR
( char * err,RTRT_INT_SIZE_PTR v);
#else
( err,v );
#endif


/*******************************************************************************
* Block data structure management
*******************************************************************************/

/*******************************************************************************
*  Service: _atp_insert_block
*******************************************************************************/
RTRT_STATICFCT void _atp_insert_block
#if !RTRT_KR
( _AtpBlockP block )
#else
( block )
  _AtpBlockP block;
#endif
{
  _AtpBlockP current, prev;
  int hash;

  if ( _atp_hash_table == (_AtpBlockP *)0 )
  {
    int i;

    RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
    _atp_hash_table = (_AtpBlockP *) RTRT_DO_MALLOCH ( sizeof ( _AtpBlockP ) *
                                                       _ATP_ENTRIES_CARD );
    if ( _atp_hash_table == (_AtpBlockP *)0 )
    {
      RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
      ATL_ERROR ('M', "PurifyLT: No memory left" );
      return;
    }
    for ( i = 0;
          i < _ATP_ENTRIES_CARD;
          i++ )
    {
      _atp_hash_table[i] = (_AtpBlockP)0;
#if _ATP_DEBUG_HASH
      _atp_hash_table_count[i] = 0;
      _atp_hash_table_max[i] = 0;
#endif
    }
    RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
  }

  block->Next = (_AtpBlockP)0;

  hash = _ATP_HASH(block->Ptr);
  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
#if _ATP_DEBUG_HASH
  _atp_hash_table_count[hash]++;
  if ( _atp_hash_table_count[hash] > _atp_hash_table_max[hash] )
  {
    _atp_hash_table_max[hash] = _atp_hash_table_count[hash];
  }
#endif

  for ( current = _atp_hash_table[hash], prev = (_AtpBlockP)0;
        current && ( current->Ptr < block->Ptr );
        prev = current, current = current->Next );
  if ( prev )
  {
    block->Next = prev->Next;
    prev->Next = block;
  }
  else
  {
    if ( _atp_hash_table[hash] )
    {
      block->Next = _atp_hash_table[hash];
    }
    _atp_hash_table[hash] = block;
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
}

/*******************************************************************************
*  Service: _atp_find_block
*******************************************************************************/
RTRT_STATICFCT _AtpBlockP _atp_find_block
#if !RTRT_KR
( char *ptr,
  _AtpBlockP hash_entry,
  _AtpBlockP *prev )
#else
( ptr, hash_entry, prev )
  char *ptr;
  _AtpBlockP hash_entry;
  _AtpBlockP *prev;
#endif
{
  _AtpBlockP current;

  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  for ( current = hash_entry, *prev = (_AtpBlockP)0;
        current && ( current->Ptr < ptr );
        *prev = current, current = current->Next );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)

  return ( current);
}

/*******************************************************************************
*  Service: _atp_find_link
*******************************************************************************/
RTRT_STATICFCT _AtpBlockP _atp_find_link
#if !RTRT_KR
( char *ptr,
  _AtpBlockP *prev )
#else
( ptr, prev )
  char *ptr;
  _AtpBlockP *prev;
#endif
{
  _AtpBlockP current;

  if ( _atp_hash_table == (_AtpBlockP *)0 )
  {
    return (_AtpBlockP)0;
  }
  current=_atp_find_block(ptr,_atp_hash_table[_ATP_HASH(ptr)],prev);

  return ( current && ( current->Ptr == ptr ) ) ?
           current : (_AtpBlockP)0;
}

#if ATP_REPORTS_ABWL
#if !RTRT_KR
RTRT_STATICFCT int _atp_check_for_abwl ( _AtpBlockP );
#else
RTRT_STATICFCT int _atp_check_for_abwl ( );
#endif
#endif /* ATP_REPORTS_ABWL */

#if ATP_CHECK_SELECTED_BLOCKS
#if !RTRT_KR
RTRT_STATICFCT void _atp_suppress_chk( _AtpBlockP block );
#else
RTRT_STATICFCT void _atp_suppress_chk( );
#endif
#endif

/*******************************************************************************
*  Service: _atp_suppress_block
*******************************************************************************/
RTRT_STATICFCT void _atp_suppress_block
#if !RTRT_KR
( _AtpBlockP link,
  _AtpBlockP prev,
  char delayed )
#else
( link, prev, delayed)
  _AtpBlockP link;
  _AtpBlockP prev;
   char delayed ;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  _atp_unfreed_blocks--;
  _atp_unfreed_bytes -= link->Size;

#if _ATP_DEBUG_HASH
  {
    int hash;

    hash = _ATP_HASH(link->Ptr);
    _atp_hash_table_count[hash]--;
  }
#endif
  if ( prev )
  {
    prev->Next = link->Next;
  }
  else
  {
    _atp_hash_table[_ATP_HASH(link->Ptr)] = link->Next;
  }

#if ATP_CHECK_SELECTED_BLOCKS
  if ((!delayed) && (_ATP_IS_BLOCK_STATUS_CHK(link->Status)))
  {
	  _atp_suppress_chk(link);
  }
#endif
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)

#if ATP_REPORTS_ABWL
  _atp_check_for_abwl ( link );
#endif /* ATP_REPORTS_ABWL */
  if (!delayed)
  {
      RTRT_DO_FREEB ( (void *)link );
  }
}


#if ATP_REPORTS_FFM_FMWL && ATP_KEEP_CONTENT

/*******************************************************************************
*  Service: _atp_block_cks
*******************************************************************************/
RTRT_STATICFCT RTRT_U_INT16 _atp_block_cks
#if !RTRT_KR
( char *addr,
  RTRT_U_INT32 size )
#else
( addr, size )
  char *addr;
  RTRT_U_INT32 size;
#endif
{
  char *ptr;
  char *ptrEnd=addr + size;
  RTRT_U_INT16 blockCks=0;
  for ( ptr = addr;
        ptr < ptrEnd;
        ptr++ )
  {
    blockCks += *ptr;
  }
return (blockCks);
}

#endif /* ATP_REPORTS_FFM_FMWL && ATP_KEEP_CONTENT */


/*******************************************************************************
*  Service: _atp_fill_dead_beef
*******************************************************************************/
#if ATP_REPORTS_FFM_FMWL || ATP_REPORTS_ABWL
RTRT_STATICFCT void _atp_fill_dead_beef
#if !RTRT_KR
( char *addr,
  RTRT_U_INT32 size )
#else
( addr, size )
  char *addr;
  RTRT_U_INT32 size;
#endif
{
  char *ptr;
  int i;

  for ( ptr = addr, i = 0;
        ptr < addr + size;
        ptr++, i = ( i + 1 ) & 0x03 )
  {
    *ptr = (char) _atp_dead_beef[i];
  }
}

#endif /* ATP_REPORTS_FFM_FMWL || ATP_REPORTS_ABWL */

#if ATP_REPORTS_FFM_FMWL

#if !RTRT_KR
RTRT_STATICFCT int _atp_check_for_fmwl ( _AtpFreeQP );
#else
RTRT_STATICFCT int _atp_check_for_fmwl ( );
#endif

/*******************************************************************************
*  Service: _atp_check_freeq
*******************************************************************************/
RTRT_STATICFCT int _atp_check_freeq
#if !RTRT_KR
( )
#else
( )
#endif
{
  _AtpFreeQP freeq;
  int err=0;
  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  for (freeq = _atp_first_freeq;
  		freeq;
  		freeq=freeq->Next)
  {
    err |=_atp_check_for_fmwl ( freeq );
#if ATP_REPORTS_ABWL
    err |=_atp_check_for_abwl ( freeq->Block );
#endif /* ATP_REPORTS_ABWL */
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
  return err;
}

/*******************************************************************************
*  Service: _atp_suppress_from_freeq
*******************************************************************************/
RTRT_STATICFCT void _atp_suppress_from_freeq
#if !RTRT_KR
( RTRT_U_INT32 requested_size )
#else
( requested_size )
  RTRT_U_INT32 requested_size;
#endif
{
  _AtpFreeQP freeq;
  int err=0;

  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  while ( ( _atp_first_freeq != (_AtpFreeQP)0 ) &&
          ( ( _atp_freeq_length + 1 > ATP_MAX_FREEQ_LENGTH ) ||
            ( _atp_freeq_size + requested_size > ATP_MAX_FREEQ_SIZE ) ) )
  {
    freeq = _atp_first_freeq;
    _atp_first_freeq = _atp_first_freeq->Next;
    err |=_atp_check_for_fmwl ( freeq );
    _atp_freeq_length--;
    _atp_freeq_size -= freeq->Block->Size;
#if ATP_REPORTS_ABWL
    err |=_atp_check_for_abwl ( freeq->Block );
#endif /* ATP_REPORTS_ABWL */
#if ATP_CHECK_SELECTED_BLOCKS

    if (_ATP_IS_BLOCK_STATUS_CHK(freeq->Block->Status))
    {
	  _atp_suppress_chk(freeq->Block);
    }
#endif
    RTRT_DO_FREEM ( (void *)_ATP_ACTUAL_INTERNAL_DATA(freeq->Block->Ptr),(&(freeq->freeParam)),(freeq->Block) );
    RTRT_DO_FREEB ( (void *)freeq->Block );
    RTRT_DO_FREEF ( (void *)freeq );
  }
  if ( _atp_first_freeq == (_AtpFreeQP)0 )
  {
    _atp_last_freeq = (_AtpFreeQP)0;
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
  if(err)
  {
	  _atp_dump_err ( "CHK FREEQ",0 );
	  ATL_BUFFER_DUMP ( );
  }
}

/*******************************************************************************
*  Service: _atp_insert_into_freeq
*******************************************************************************/
RTRT_STATICFCT _AtpFreeQP _atp_insert_into_freeq
#if !RTRT_KR
( _AtpBlockP block )
#else
( block )
  _AtpBlockP block;
#endif
{
  _AtpFreeQP freeq;
  _AtlStackLinkP stack_link;
  int i;

  if((block->Size > ATP_MAX_FREEQ_BLOCK_SIZE) || (block->Size ==0))
  {
	  return (_AtpFreeQP)0;
  }

  _atp_suppress_from_freeq ( block->Size );
#if ATP_CHECK_SELECTED_BLOCKS
  _ATP_SET_BLOCK_STATUS_FREE(block->Status);
#endif

  /* New FreeQ */
  freeq = (_AtpFreeQP) RTRT_DO_MALLOCF ( sizeof ( _AtpFreeQ ) );
  if ( ! freeq )
  {
    ATL_ERROR ('M', "PurifyLT: No memory left" );
    return (_AtpFreeQP)0;
  }

#if !ATP_KEEP_CONTENT
  _atp_fill_dead_beef ( block->Ptr, block->Size );
#else
  freeq->BlockCks = _atp_block_cks ( block->Ptr, block->Size );
#endif /* ATP_KEEP_CONTENT */

  stack_link = _atl_get_stack_current ( );

  /* Initializations */
  freeq->Block = block;
  for ( i = 0;
        i < ATP_CALL_STACK_SIZE &&
        stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        i++, stack_link = stack_link->Up )
  { /* Fill call stack copy */
    freeq->StackInfoTable[i].Info = stack_link->Info;
#ifdef ATL_T_FAMILY
	freeq->StackInfoTable[i].Key 	  = stack_link->Info.Source->Key;
	freeq->StackInfoTable[i].CheckSum = stack_link->Info.Source->CheckSum;
#endif
  }
  if ( i < ATP_CALL_STACK_SIZE )
  { /* Copy is not full */
    freeq->StackInfoTable[i].Info.Source = (_AtlStackSourceP)0;
  }
  freeq->Next = (_AtpFreeQP)0;

  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  /* Linking */
  if ( _atp_last_freeq )
  {
    _atp_last_freeq->Next = freeq;
  }
  else
  {
    _atp_first_freeq = freeq;
  }
  _atp_last_freeq = freeq;
  _atp_freeq_length++;
  _atp_freeq_size += block->Size;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
  return freeq;
}

#endif /* ATP_REPORTS_FFM_FMWL */

/*******************************************************************************
*  Service: _atp_create_block
*******************************************************************************/
RTRT_STATICFCT void _atp_create_block
#if !RTRT_KR
( char *ptr,
  RTRT_U_INT32 size ,
  RTRT_U_INT16 AllocType)
#else
( ptr, size, AllocType)
  char *ptr;
  RTRT_U_INT32 size;
  RTRT_U_INT16 AllocType;
#endif
{
  _AtpBlockP block;
  _AtlStackLinkP stack_link;
  int i;

  /* New block */
  block = (_AtpBlockP) RTRT_DO_MALLOCB ( sizeof ( _AtpBlock ) );
  if ( ! block )
  {
    ATL_ERROR ('M', "PurifyLT: No memory left" );
    return;
  }

  stack_link = _atl_get_stack_current ( );

  /* Initializations */
  block->Ptr = ptr;
  for ( i = 0;
        i < ATP_CALL_STACK_SIZE &&
        stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        i++, stack_link = stack_link->Up )
  { /* Fill call stack copy */
    block->StackInfoTable[i].Info = stack_link->Info;
#ifdef ATL_T_FAMILY
    block->StackInfoTable[i].Key = stack_link->Info.Source->Key;
    block->StackInfoTable[i].CheckSum = stack_link->Info.Source->CheckSum;
#endif
  }
  if ( i < ATP_CALL_STACK_SIZE )
  { /* Copy is not full */
    block->StackInfoTable[i].Info.Source = (_AtlStackSourceP)0;
  }
  block->Size = size;
  block->Status=0;
#if ATP_REPORTS_FIM
  _ATP_SET_ALLOC_STATUS(block->Status,AllocType);
#endif
#if ATP_CHECK_SELECTED_BLOCKS
  block->ChkPrev=(_AtpBlockP)0;
  block->ChkNext=(_AtpBlockP)0;
#endif
  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
  /* Min and Max update */
  if ( _atp_minimum_addr > ptr )
  {
    _atp_minimum_addr = ptr;
  }
  if ( _atp_maximum_addr < ptr + size )
  {
    _atp_maximum_addr = ptr + size;
  }

  /* Computing Statistics */
  _atp_total_blocks++;
  _atp_unfreed_blocks++;
  if ( _atp_unfreed_blocks > _atp_max_blocks )
  {
    _atp_max_blocks = _atp_unfreed_blocks;
  }
  _atp_total_bytes += block->Size;
  _atp_unfreed_bytes += block->Size;
  if ( _atp_unfreed_bytes > _atp_max_bytes )
  {
    _atp_max_bytes = _atp_unfreed_bytes;
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)

#if ATP_REPORTS_ABWL
  /* Filling red zones */
#if ATP_RED_ZONE_HEAD_SIZE != 0
_atp_fill_dead_beef ( ptr - ATP_RED_ZONE_HEAD_SIZE, ATP_RED_ZONE_HEAD_SIZE );
#endif
#if ATP_RED_ZONE_TAIL_SIZE != 0
_atp_fill_dead_beef ( ptr + size, ATP_RED_ZONE_TAIL_SIZE );
#endif

#endif /* ATP_REPORTS_ABWL */

  /* Block Insertion */
  _atp_insert_block ( block );
}


/*******************************************************************************
* Dump services
*******************************************************************************/

/*******************************************************************************
*  Service: _atp_dump_start_test
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_start_test
#if !RTRT_KR
( void )
#else
( )
#endif
{
  if ( ! _atp_test_started )
  {
    char *date;

    _atp_test_started = 1;
    /* New Test */
    ATL_LINE_ADD_OPCODE ( ATL_TPF_PT );
#if USE_ATU
    _atl_obstools_test_name ( 0 );
#else /* USE_ATU */
    _atl_obstools_test_name ( );
#endif /* USE_ATU */
    date = priv_date();
    if ( date != (char *)0 )
    {
      atl_line_add_char ( ' ' );
      atl_line_add_char ( RTRT_QUOT );
      atl_line_add ( date );
      atl_line_add_char ( RTRT_QUOT );
    }
    atl_line_dump ( );
  }
}

/*******************************************************************************
*  Service: _atp_dump_stack_info
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_stack_info
#if !RTRT_KR
( _AtlStackInfoP stack_info )
#else
( stack_info )
  _AtlStackInfoP stack_info;
#endif
{
  /* Print the PF line for a routine in the call stack */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PF );
  atl_line_inc ( unsigned_reverse_image_16 (
                   stack_info->Source->Key,
                   atl_line_cur ) );
  atl_line_add_char ( ' ' );
  atl_line_inc ( unsigned_reverse_image_16 (
                   stack_info->Source->CheckSum,
                   atl_line_cur ) );
  atl_line_add_char ( ' ' );
  atl_line_inc ( unsigned_reverse_image_16 (
                   stack_info->MessageTsf,
                   atl_line_cur ) );
  atl_line_add_char ( ' ' );
  atl_line_inc ( unsigned_reverse_image_16 (
                   stack_info->SourceLineOffset,
                   atl_line_cur ) );
  atl_line_dump ( );
}

/*******************************************************************************
*  Service: _atp_dump_call_stack_info
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_call_stack_info
#if !RTRT_KR
( _AtlCallStackInfoP stack_info )
#else
( stack_info )
  _AtlCallStackInfoP stack_info;
#endif
{
  int i;

  for ( i = 0;
        ( i < ATP_CALL_STACK_SIZE &&
          stack_info[i].Info.Source );
        i++ )
  {
	  /* Print the PF line for a routine in the call stack */
	  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PF );
#ifndef ATL_T_FAMILY
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].Info.Source->Key,
					   atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].Info.Source->CheckSum,
					   atl_line_cur ) );
#else
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].Key,
					   atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].CheckSum,
					   atl_line_cur ) );
#endif
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].Info.MessageTsf,
					   atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 (
					   stack_info[i].Info.SourceLineOffset,
					   atl_line_cur ) );
	  atl_line_dump ( );
  }
}
/*******************************************************************************
*  Service: _atp_dump_memory_block
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_memory_block
#if !RTRT_KR
( _AtpBlockP memory_block )
#else
( memory_block )
  _AtpBlockP memory_block;
#endif
{

  /* Dump the memory block size */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PB );
#if RTRT_64_BITS_PTR == 0
  atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)(memory_block->Size),
                                             atl_line_cur ) );
#else
  if(memory_block->Size>>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(memory_block->Size>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(memory_block->Size&0xFFFFFFFFUL),atl_line_cur ) );

#endif /* RTRT_64_BITS_PTR */
  atl_line_dump ( );

  _atp_dump_call_stack_info(memory_block->StackInfoTable);
}

/*******************************************************************************
*  Service: _atp_dump_err
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_err
#if !RTRT_KR
( char * err,RTRT_INT_SIZE_PTR v)
#else
( err,v )
  char * err;
  RTRT_INT_SIZE_PTR v;
#endif
{
  int i;
  _AtlStackLinkP stack_link;

  _atp_dump_start_test ( );
  /* Dump the COR event line */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  atl_line_add ( err );
#if RTRT_64_BITS_PTR ==0
  if(v) atl_line_inc ( unsigned_reverse_image_16 (v,atl_line_cur ) );
#else
  if(v>>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(v>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  if(v) atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(v&0xFFFFFFFFUL),atl_line_cur ) );

#endif /* RTRT_64_BITS_PTR */
  atl_line_dump ( );

  stack_link = _atl_get_stack_current ( );

  /* Dump the current call stack */
  for ( i = 0;
        i < ATP_CALL_STACK_SIZE &&
        stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        i++, stack_link = stack_link->Up )
  { /* Dump call stack */
    _atp_dump_stack_info ( &stack_link->Info );
  }
}


#if ATP_REPORTS_ABWL

/*******************************************************************************
*  Service: _atp_dump_ABWL (Late Detect Array Bounds Write)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_ABWL
#if !RTRT_KR
( _AtpBlockP block,
  char *ptr )
#else
( block, ptr )
  _AtpBlockP block;
  char *ptr;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("ABWL detected");
	  _att_print_change_instance ( 0 );
#endif
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_start_test ( );
  /* Dump the ABWL event line */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "ABWL " );
#if RTRT_64_BITS_PTR == 0
  atl_line_inc ( unsigned_reverse_image_16 (
                   (RTRT_U_INT32)block->Ptr,
                   atl_line_cur ) );
#else
  if((RTRT_U_INT64)(block->Ptr) >>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(block->Ptr)>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(block->Ptr)&0xFFFFFFFFUL),atl_line_cur ) );

#endif /* RTRT_64_BITS_PTR */

  atl_line_add_char ( ' ' );
#if RTRT_64_BITS_PTR == 0
  atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)ptr, atl_line_cur ) );
#else
  if((RTRT_U_INT64)ptr >>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(ptr)>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(ptr)&0xFFFFFFFFUL),atl_line_cur ) );

#endif /* RTRT_64_BITS_PTR */
  atl_line_dump ( );

  /* Dump the block information */
  _atp_dump_memory_block ( block );
  ATL_BUFFER_DUMP ( );
  CALL_ON_ATP_BLOCK_ERROR
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

#endif /* ATP_REPORTS_ABWL */

#if 0
/* Unused */
/*******************************************************************************
*  Service: _atp_dump_COR (Fatal Core Dump)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_COR
#if !RTRT_KR
( void )
#else
( )
#endif
{
  int i;
  _AtlStackLinkP stack_link;

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_err ( "COR",0 );
  ATL_BUFFER_DUMP ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif

#if ATP_REPORTS_FIU
#if RTRT_IO == RTRT_STD
/*******************************************************************************
*  Service: _atp_dump_FIU (File Descriptor in Use)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_FIU
#if !RTRT_KR
( void )
#else
( )
#endif
{
  int i;
  char status;

  /* Dump the FIU event line */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "FIU" );
  atl_line_dump ( );

  /* Parse file descriptors up to ATL_STD_IO_MAX_FD */
  for ( i = 3; i < ATL_STD_IO_MAX_FD; i++ )
  {
    if ( i == ATL_FILENO ( atl_out ) )
    {
      status = '0'; /* internal use */
    }
    else if ( priv_is_open ( i ) )
    {
      if ( _atp_previously_opened[i-3] )
      {
        continue; /* Yet opened (Parent process inherited) */
      }
      status = '1'; /* Locally opened and still not closed */
    }
    else
    {
      continue;
    }
    ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PO );
    atl_line_inc ( unsigned_reverse_image_16 ( i, atl_line_cur ) );
    atl_line_add_char ( ' ' );
    atl_line_add_char ( status );
    atl_line_dump ( );
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif /* RTRT_IO == RTRT_STD */
#endif /* ATP_REPORTS_FIU */

#if ATP_REPORTS_FFM_FMWL

/*******************************************************************************
*  Service: _atp_dump_FFM (Freeing Freed Memory)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_FFM
#if !RTRT_KR
( _AtpFreeQP freeq )
#else
( freeq )
  _AtpFreeQP freeq;
#endif
{
  _AtpBlockP block=freeq->Block; /* can  be used by the CALL_ macro */
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("FFM detected");
	  _att_print_change_instance ( 0 );
#endif

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  /* Dump the FFM event line */
  _atp_dump_err ( "FFM ",(RTRT_INT_SIZE_PTR)block->Ptr );

  /* Dump the block information */
  _atp_dump_memory_block ( block );
  ATL_BUFFER_DUMP ( );
  CALL_ON_ATP_ERROR
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

/*******************************************************************************
*  Service: _atp_dump_FMWL (Late Detect Free Memory Write)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_FMWL
#if !RTRT_KR
( _AtpFreeQP freeq,
  char *ptr )
#else
( freeq, ptr )
  _AtpFreeQP freeq;
  char *ptr;
#endif
{

  _AtpBlockP block=freeq->Block;
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("FMWL detected");
	  _att_print_change_instance ( 0 );
#endif

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_start_test ( );
  /* Dump the FMWL event line */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "FMWL " );
#if RTRT_64_BITS_PTR == 0
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(ptr - block->Ptr),
                                             atl_line_cur ) );
#else
  if((RTRT_U_INT64)(ptr - block->Ptr) >>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(ptr - block->Ptr)>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)((RTRT_U_INT64)(ptr - block->Ptr)&0xFFFFFFFFUL),atl_line_cur ) );

#endif /* RTRT_64_BITS_PTR */

  atl_line_dump ( );

  _atp_dump_call_stack_info(freeq->StackInfoTable);

  /* Dump the block information */
  _atp_dump_memory_block ( block );
  ATL_BUFFER_DUMP ( );
  CALL_ON_ATP_BLOCK_ERROR
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

#endif /* ATP_REPORTS_FFM_FMWL */
/*******************************************************************************
*  Service: _atp_dump_FIM (Freeing Invalid Memory) only for C++
*******************************************************************************/
#if ATP_REPORTS_FIM
RTRT_STATICFCT void _atp_dump_FIM
#if !RTRT_KR
( _AtpBlockP block )
#else
( memory_block )
  _AtpBlockP block;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("FIM detected");
	  _att_print_change_instance ( 0 );
#endif

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_err ( "FIM ",(RTRT_INT_SIZE_PTR)(block->Ptr));
  /* Dump the block information */
  _atp_dump_memory_block ( block );
  ATL_BUFFER_DUMP ( );
  CALL_ON_ATP_ERROR
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif
/*******************************************************************************
*  Service: _atp_dump_FUM (Freeing Unallocated Memory)
*******************************************************************************/
#if ATP_REPORTS_FUM
RTRT_STATICFCT void _atp_dump_FUM
#if !RTRT_KR
( char *ptr )
#else
( ptr )
  char *ptr;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("FUM detected");
	  _att_print_change_instance ( 0 );
#endif

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_err ( "FUM ",(RTRT_INT_SIZE_PTR)ptr);
  ATL_BUFFER_DUMP ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
  CALL_ON_ATP_ERROR
}
#endif /* ATP_REPORTS_FUM */
/*******************************************************************************
*  Service: _atp_dump_MAF (Memory Allocation Failed)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_MAF
#if !RTRT_KR
( RTRT_U_INT32 size )
#else
( size )
  RTRT_U_INT32 size;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("MAF detected");
	  _att_print_change_instance ( 0 );
#endif

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_err ( "MAF ",(RTRT_INT_SIZE_PTR)size);
  ATL_BUFFER_DUMP ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
  CALL_ON_ATP_ERROR
}

#if ATP_REPORTS_MIU
/*******************************************************************************
*  Service: _atp_dump_MIU (Memory In Use)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_MIU
#if !RTRT_KR
( _AtpBlockP memory_block )
#else
( memory_block )
  _AtpBlockP memory_block;
#endif
{
  /* Dump the MIU event line */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "MIU " );
  atl_line_inc ( unsigned_reverse_image_16 ( _ATP_GET_AREA_STATUS(memory_block->Status)+1, /* no 0 */
                                             atl_line_cur ) );
  atl_line_dump ( );

  /* Dump the unfreed memory block */
  _atp_dump_memory_block ( memory_block );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#endif /* ATP_REPORTS_MIU */

/*******************************************************************************
*  Service: _atp_dump_MLK (Memory Leak)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_MLK
#if !RTRT_KR
( _AtpBlockP memory_block )
#else
( memory_block )
  _AtpBlockP memory_block;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
  if(_atl_MLK_detected==0)
  {
	  _atl_MLK_detected=1;
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("MLK detected");
	  _att_print_change_instance ( 0 );
  }
#endif

  /* Dump the MLK event line */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "MLK" );
  atl_line_dump ( );

  /* Dump the leak memory block */
  _atp_dump_memory_block ( memory_block );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

/*******************************************************************************
*  Service: _atp_dump_MPK (Potential Memory Leak)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_MPK
#if !RTRT_KR
( _AtpBlockP memory_block )
#else
( memory_block )
  _AtpBlockP memory_block;
#endif
{
#if USE_ATT && ATT_ON_THE_FLY
  if(_atl_MPK_detected==0)
  {
	  _atl_MPK_detected=1;
	  _atl_tdf_line(_ATO_STYLE_CHECK_KO);
	  _atl_tdf_note_string("MPK detected");
	  _att_print_change_instance ( 0 );
  }
#endif

  /* Dump the MPK event line */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "MPK " );
  atl_line_inc ( unsigned_reverse_image_16 ( _ATP_GET_AREA_STATUS(memory_block->Status)+1, /* no 0 */
                                             atl_line_cur ) );
  atl_line_dump ( );

  /* Dump the leak memory block */
  _atp_dump_memory_block ( memory_block );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
extern "C" {
#endif

/*******************************************************************************
*  Service: atl_dump_CALLSTACK for #pragma attol atp_insert _ATL_DUMP_STACK("signalName")
*******************************************************************************/
_ATL_C_GLOBALFCT void atl_dump_CALLSTACK
#if !RTRT_KR
(char *name, _AtlStackLinkP stack_link)
#else
( name , stack_link )
  char * name;
  _AtlStackLinkP stack_link;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_start_test ( );
  stack_link = _atl_get_stack_current ( );
  /* Dump the SIG event line */
  ATL_LINE_ADD_OPCODE ( "PE SIG 0 \"" );
  if(name) atl_line_add      ( name );
  atl_line_add_char ( RTRT_QUOT );

#if ATL_MULTI_THREADS
  atl_line_add_char ( ' ' );
  atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)_atl_threads_table_used[stack_link->ThreadIndex],
                                             atl_line_cur ) );
#endif
  atl_line_dump ( );

  /* Dump the current call stack */
  for ( ;
  		stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        stack_link = stack_link->Up )
  { /* Dump call stack */
    _atp_dump_stack_info ( &stack_link->Info );
  }

#if ATL_MULTI_THREADS && defined(DUMP_STACK_ALL)
  for (i = 0; i < ATL_THREADS_MAX; i++) {
    stack_link = _atl_stack_table_current[i];
    /* Dump the call stack */
    if (RTRT_MT_THREAD_ISALIVE(_atl_threads_table_used[i])) {
      ATL_LINE_ADD_OPCODE ( "PE SIG 0 \"" );
      if(name) atl_line_add      ( name );
      atl_line_add_char ( RTRT_QUOT );
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)_atl_threads_table_used[stack_link->ThreadIndex],
                                             atl_line_cur ) );
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)i,
                                             atl_line_cur ) );
      atl_line_dump ( );
      for ( ;
	    stack_link
#if ATL_CHECK_STACK
	      && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
	      ;
	    stack_link = stack_link->Up )
	{ /* Dump call stack */
	  _atp_dump_stack_info ( &stack_link->Info );
	}
    }
  }

#endif
  ATL_BUFFER_DUMP ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
}
#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
} /* end of extern "C" */
#endif

#if ATP_REPORTS_SIG
#if RTRT_SIGNAL == RTRT_STD

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
extern "C" {
#endif

/*******************************************************************************
*  Service: _atp_dump_SIG (Signal handled)
*******************************************************************************/
RTRT_STATICFCT void _atp_dump_SIG
#if !RTRT_KR
( int signum )
#else
( signum )
  int signum;
#endif
{
  int i;
  char *str;
  _AtlStackLinkP stack_link;

  /* Restore previous handler */
  for ( i = 0;
        i < ATL_SIGNALS_CATCHED_CARD &&
        _atl_signal_datas[i].SigNum == signum;
        i++ );

  if ( i != ATL_SIGNALS_CATCHED_CARD )
  { /* Strange if not found but... */
    ATL_SIGNAL ( signum, _atl_signal_datas[i].Handler );
  }
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_start_test ( );
  /* Dump the SIG event line */
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PE );
  ATL_LINE_ADD_OPCODE ( "SIG " );
  atl_line_inc ( unsigned_reverse_image_16 ( (RTRT_U_INT32)signum,
                                             atl_line_cur ) );
  str = _atl_signal_string ( signum );
  if ( str != (char *)0 )
  {
    ATL_LINE_ADD_OPCODE ( " \"" );
    atl_line_add      ( str );
    atl_line_add_char ( RTRT_QUOT );
  }
  atl_line_dump ( );

  stack_link = _atl_get_stack_current ( );

  /* Dump the current call stack */
  for ( i = 0;
        i < ATP_CALL_STACK_SIZE &&
        stack_link
#if ATL_CHECK_STACK
        && ! _ATL_STACK_CHECK_MAGIC(stack_link)
#endif
        ;
        i++, stack_link = stack_link->Up )
  { /* Dump call stack */
    _atp_dump_stack_info ( &stack_link->Info );
  }
  ATL_BUFFER_DUMP ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

#ifdef RTRT_POST_SIG
  RTRT_POST_SIG
#endif /* RTRT_POST_SIG */

  /* and raise again */
  ATL_RAISE ( signum );
}

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
} /* end of extern "C" */
#endif

#endif /* RTRT_SIGNAL == RTRT_STD */
#endif /* ATP_REPORTS_SIG */

#if ATP_REPORTS_FFM_FMWL

/*******************************************************************************
*  Service: _atp_check_for_ffm
*******************************************************************************/
RTRT_STATICFCT short _atp_check_for_ffm
#if !RTRT_KR
( char *ptr )
#else
( ptr )
  char *ptr;
#endif
{
  _AtpFreeQP freeq;

  for ( freeq = _atp_first_freeq;
        freeq && freeq->Block->Ptr != ptr;
        freeq = freeq->Next );
  if ( freeq )
  { /* Yet in freeq: yet freed */
    _atp_dump_FFM ( freeq );
    return 1;
  }
  /* No problem with FFM here */
  return 0;
}

/*******************************************************************************
*  Service: _atp_check_for_fmwl
*******************************************************************************/
RTRT_STATICFCT int _atp_check_for_fmwl
#if !RTRT_KR
( _AtpFreeQP freeq )
#else
( freeq )
  _AtpFreeQP freeq;
#endif
{
  int err=0;
  if(! ( _ATP_IS_FMWL_STATUS(freeq->Block->Status) ))
    {
#if !ATP_KEEP_CONTENT
  char *ptr;
  int i;
	  for ( ptr = freeq->Block->Ptr, i = 0;
			ptr < freeq->Block->Ptr + freeq->Block->Size;
			ptr++, i = ( i + 1 ) & 0x03 )
	  {
		if ( *ptr != (char) _atp_dead_beef[i] )
		{
		  _atp_dump_FMWL ( freeq, ptr );
		  err |=1;
		  _ATP_SET_FMWL_STATUS(freeq->Block->Status);
		  break;
		}
	  }
#else
	  if (freeq->BlockCks != _atp_block_cks ( freeq->Block->Ptr, freeq->Block->Size ))
		{
		  _atp_dump_FMWL ( freeq, freeq->Block->Ptr );
		  err |=1;
		  _ATP_SET_FMWL_STATUS(freeq->Block->Status);
		}
#endif /* ATP_KEEP_CONTENT */
    }
  return err;
}

#endif /* ATP_REPORTS_FFM_FMWL */

#if ATP_REPORTS_ABWL

/*******************************************************************************
*  Service: _atp_check_for_abwl
*******************************************************************************/
RTRT_STATICFCT int _atp_check_for_abwl
#if !RTRT_KR
( _AtpBlockP block )
#else
( block )
  _AtpBlockP block;
#endif
{
  char *ptr;
  int i;
  int err=0;
  if (
#if ATP_REPORTS_FFM_FMWL
       ! ( _ATP_IS_FMWL_STATUS(block->Status) ) &&
#endif /* ATP_REPORTS_FFM_FMWL */
       ! ( _ATP_IS_ABWL_STATUS_UPSTREAM(block->Status) ) )
  {
    for ( ptr = block->Ptr - ATP_RED_ZONE_HEAD_SIZE, i = 0;
          ptr < block->Ptr;
          ptr++, i = ( i + 1 ) & 0x03 )
    {
      if ( *ptr != (char) _atp_dead_beef[i] )
      {
        _atp_dump_ABWL ( block, ptr );
        _ATP_SET_ABWL_STATUS_UPSTREAM(block->Status);
        err|=1;
        break;
      }
    }
  }
  if (
#if ATP_REPORTS_FFM_FMWL
       ! ( _ATP_IS_FMWL_STATUS(block->Status) ) &&
#endif /* ATP_REPORTS_FFM_FMWL */
       ! ( _ATP_IS_ABWL_STATUS_DOWNSTREAM(block->Status) ) )
  {
    for ( ptr = block->Ptr + block->Size, i = 0;
          ptr < block->Ptr + block->Size + ATP_RED_ZONE_TAIL_SIZE;
          ptr++, i = ( i + 1 ) & 0x03 )
    {
      if ( *ptr != (char) _atp_dead_beef[i] )
      {
        _atp_dump_ABWL ( block, ptr );
        _ATP_SET_ABWL_STATUS_DOWNSTREAM(block->Status);
        err|=1;
        break;
      }
    }
  }
return err;
}

#endif /* ATP_REPORTS_ABWL */
/*******************************************************************************
*  Service: _atp_insert_chk
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_insert_chk
#if !RTRT_KR
( void * ptr)
#else
( ptr )
  void * ptr;
#endif
{
#if ATP_CHECK_SELECTED_BLOCKS
 _AtpBlockP link;
 _AtpBlockP prev;

    /* Is it something ATP seen ? */
    link = _atp_find_link ( (char *)ptr, &prev );
    if ( link != (_AtpBlockP)0 )
    {
		if(_ATP_IS_BLOCK_STATUS_CHK(link->Status)==0)
		{
			RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)

			if(_atp_chk_list)
			{
				_atp_chk_list->ChkPrev = link;
			}
			link->ChkNext=_atp_chk_list;
			_atp_chk_list = link;
			RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
			_ATP_SET_BLOCK_STATUS_CHK(link->Status);
		}
	}
#endif /* ATP_CHECK_SELECTED_BLOCKS */
}
/*******************************************************************************
*  Service: _atp_chk_blocks
*  used to check a short list of blocks. Called on pragma
*******************************************************************************/
_ATL_C_GLOBALFCT int _atp_chk_blocks
#if !RTRT_KR
( void )
#else
( )
#endif
{
	int err;
#if ATP_CHECK_SELECTED_BLOCKS
	_AtpBlockP block;
#if ATP_REPORTS_FFM_FMWL
	_AtpFreeQ freeq;
	freeq.StackInfoTable[0].Info.Source = (_AtlStackSourceP)0;
	freeq.Next=(_AtpFreeQP)0;
#endif
	err=0;
	  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
	  for ( block = _atp_chk_list;
	        block ;
	        block = block->ChkNext )
	  {
#if ATP_REPORTS_ABWL
  		err |=_atp_check_for_abwl ( block );
#endif /* ATP_REPORTS_ABWL */
#if ATP_REPORTS_FFM_FMWL
	    if(_ATP_IS_BLOCK_STATUS_FREE(block->Status) && (_ATP_IS_FMWL_STATUS(block->Status)==0))
	    {
			freeq.Block = block;
			err |=_atp_check_for_fmwl(&freeq);
		}
#endif
	  }
	  if(err)
	  {
		  _atp_dump_err ( "CHK",0 );
		  ATL_BUFFER_DUMP ( );
	  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)
#else /* ATP_CHECK_SELECTED_BLOCKS */
err = _atp_dump(_ATCPQ_CHK_WL);
#endif /* ATP_CHECK_SELECTED_BLOCKS */
return (err);
}

#if ATP_CHECK_SELECTED_BLOCKS
/*******************************************************************************
*  Service: _atp_suppress_chk
*******************************************************************************/
RTRT_STATICFCT void _atp_suppress_chk
#if !RTRT_KR
( _AtpBlockP block )
#else
( block )
  _AtpBlockP block;
#endif
{
	  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)

	  if ( block->ChkPrev )
	  {
		block->ChkPrev->ChkNext = block->ChkNext;
	  }
	  else
	  {
		_atp_chk_list = block->ChkNext;
	  }
	  if(block->ChkNext)
	  {
		  block->ChkNext->ChkPrev = block->ChkPrev;
	  }
	  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
}
#endif /* ATP_CHECK_SELECTED_BLOCKS */

/*******************************************************************************
*  Service: _atp_link_parse_globals
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_link_parse_globals
#if !RTRT_KR
( _AtpGlobalsFuncTypeP thedata )
#else
( parse_func )
  _AtpGlobalsFuncTypeP thedata;
#endif
{
  thedata->Next = _atp_first_globals_func_data;
  _atp_first_globals_func_data = thedata;
}

#if 0
/*******************************************************************************
*  Service: _atp_link_parse_globals
*  return 1 if prev id the right block
*******************************************************************************/
RTRT_STATICFCT char _atp_test_and_set_mlk
#if !RTRT_KR
( char *pptr,
  _AtpBlockP prev,
  char isRefMPK,
  RTRT_U_INT16 area)
#else
( pptr, prev ,area)
  char *pptr;
  _AtpBlockP prev;
  char isRefMPK;
  RTRT_U_INT16 area;
#endif
{
	if(prev && ( (pptr) < prev->Ptr + prev->Size ))
	{/* this is the right block any way */
	   if ( _ATP_GET_BLOCK_STATUS(prev->Status) == _ATP_BLOCK_STATUS_MLK )
	   {
			_ATP_SET_BLOCK_AREA_STATUS(prev->Status,_ATP_BLOCK_STATUS_MPK,area);
			if(isRefMPK) _atp_add_to_dump_list(prev);
	   }
	   return 1;
	}
	return 0;
}
#endif
/*******************************************************************************
*  Service: _atp_add_to_dump_list
*******************************************************************************/
RTRT_STATICFCT void _atp_add_to_dump_list
#if !RTRT_KR
( _AtpBlockP prev)
#else
( prev )
  _AtpBlockP prev;
#endif
{
	prev->Dump = _atp_dump_list;
	_atp_dump_list=prev;
}
/*******************************************************************************
*  Service: _atp_check_leak
*  Goal: looking for memory references in a data
*  return: 0=no status changes, 1=MPK changes, 2=MIU changes, 3= both
*******************************************************************************/
RTRT_STATICFCT void _atp_check_leak
#if !RTRT_KR
( char *addr,
  RTRT_U_INT32 size,
  _AtpBlockStatus refStatus,
  RTRT_U_INT16 area)
#else
( addr, size ,refStatus ,area)
  char *addr;
  RTRT_U_INT32 size;
  _AtpBlockStatus refStatus;
  RTRT_U_INT16 area;
#endif
{
  char *ptr, *end;
  char isRefMPK;
  char changes=0;

  if ( size < sizeof ( ptr ) )
  { /* Too small to be a pointer */
    return ;
  }
  isRefMPK=(refStatus == _ATP_BLOCK_STATUS_MPK);
  end = addr + size - sizeof ( ptr );
  for ( ptr = addr;
#if RTRT_CARE_ABOUT_PTR_ALIGNMENT
        RTRT_PTR_TO_INT(ptr) & ( sizeof ( ptr ) - 1 );
        ptr++ );
  for ( ;
#endif /* RTRT_CARE_ABOUT_PTR_ALIGNMENT */
        ptr <= end;
        ptr +=
#ifdef ATP_BLOCK_PACKED
				1
#else
        sizeof ( ptr )
#endif
		)
  {
    char *pptr =(*(char **)ptr);
    if ( (pptr) >= _atp_minimum_addr &&
         (pptr) <= _atp_maximum_addr )
	{
	/* find ref from ptr (char **)ptr */
	  int i,j;
	  _AtpBlockP prev, unfreed;

      /* find_link */
      j=_ATP_HASH(pptr);
      unfreed=_atp_find_block(pptr,_atp_hash_table[j],&prev);

	  if(unfreed && ( unfreed->Ptr == (pptr) ))
	  {/* this is the right pointer */
	    if(_ATP_GET_BLOCK_STATUS(unfreed->Status) < refStatus)
	    {
			 _ATP_SET_BLOCK_AREA_STATUS(unfreed->Status,refStatus,(isRefMPK) ? _ATP_BLOCK_AREA_HEAP_MPK : area);
			 _atp_add_to_dump_list(unfreed);
		}
	  }
	  else
	  {
		if(prev && ( (pptr) < prev->Ptr + prev->Size ))
		{/* this is the right block any way */
		   if ( _ATP_GET_BLOCK_STATUS(prev->Status) == _ATP_BLOCK_STATUS_MLK )
		   {
				_ATP_SET_BLOCK_AREA_STATUS(prev->Status,_ATP_BLOCK_STATUS_MPK,area);
				if(isRefMPK) _atp_add_to_dump_list(prev);
		   }
		}
		else
	    {/* look into all other blocks */
		  for ( i = 0; i < _ATP_ENTRIES_CARD; i++ )
			if(i != j)
		    {
              	unfreed=_atp_find_block(pptr,_atp_hash_table[i],&prev);

				if(prev && ( (pptr) < prev->Ptr + prev->Size ))
				{/* we found the right block */
				   if ( _ATP_GET_BLOCK_STATUS(prev->Status) == _ATP_BLOCK_STATUS_MLK )
				   {
						_ATP_SET_BLOCK_AREA_STATUS(prev->Status,_ATP_BLOCK_STATUS_MPK,area);
						if(isRefMPK) _atp_add_to_dump_list(prev);
				   }
				   break;/* next pointer value */
				}
		    }
		}
	  }
	}
  }
}

/*******************************************************************************
*  Service: _atp_check_var called from instrumented code
*  Goal: looking for memory references in a global or static
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_check_var
#if !RTRT_KR
( char *addr,
  RTRT_U_INT32 size)
#else
( addr, size )
  char *addr;
  RTRT_U_INT32 size;
#endif
{
   _atp_check_leak(addr,size,_ATP_BLOCK_STATUS_MIU,_ATP_BLOCK_AREA_GLOBAL);
}

#ifndef RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN
#define RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN 0
#endif

/*******************************************************************************
*  Service: _atp_all_leaks
*  Goal: looking for memory references in all data
*******************************************************************************/
RTRT_STATICFCT void _atp_all_leaks
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ATL_MULTI_THREADS
  int thread_index= _atl_threads_get_index ();
#endif /* ATL_MULTI_THREADS */
  _atp_dump_list=(_AtpBlockP)0;

  /* Inspecting the (known) global data */
  {
    _AtpGlobalsFuncTypeP thedata;

    for ( thedata = _atp_first_globals_func_data;
          thedata;
          thedata = thedata->Next )
    {
      thedata->ParseGlobalsFunc();
    }
  }

  /* Inspecting the stack */

  RTRT_MT_CRITICAL_VAR_LOCK(atl_stack_critical)

#if USE_ATU || USE_ATO

  {
    if ( _atp_test_driver_top_stack )
    {
      /* Stack grows with pointer values or against */
      _atp_check_leak ( ( _atp_test_driver_top_stack < atl_stack_end ) ?
                          _atp_test_driver_top_stack : atl_stack_end,
                        ( _atp_test_driver_top_stack < atl_stack_end ) ?
                          atl_stack_end - _atp_test_driver_top_stack :
                          _atp_test_driver_top_stack - atl_stack_end ,
                          _ATP_BLOCK_STATUS_MIU, _ATP_BLOCK_AREA_STACK);
    }
  }

#endif /* USE_ATU || USE_ATO */

#if ( ! ATL_MULTI_THREADS ) || RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN

  {
    char *stack_start;

    if ( _atl_stack_current_link )
    {
      stack_start = (char *)_atl_stack_start_link;
      /* Stack grows with pointer values or against */
      _atp_check_leak ( ( stack_start < atl_stack_end ) ?
                          stack_start : atl_stack_end,
                        ( stack_start < atl_stack_end ) ?
                          atl_stack_end - stack_start : stack_start - atl_stack_end ,
                          _ATP_BLOCK_STATUS_MIU, _ATP_BLOCK_AREA_STACK);
    }
  }

#else /* ATL_MULTI_THREADS && ! RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN */

  {
    int i;
	char *stack_start, *stack_end;

    /* Inspecting all stacks */
    for ( i = 0;
          i <= ATL_THREADS_MAX;
          i++ )
    {
      if ( RTRT_MT_THREAD_ISALIVE(_atl_threads_table_used[i]) )
      {
		/*  looking for memory references in all stack data */
		  stack_start = (char *)_atl_stack_table_start[i];

		  if ( stack_start )
		  {
			/* Stack grows with pointer values or against */
			if ( i == thread_index )
			{
			  stack_end = atl_stack_end;
			}
			else
			{
			  stack_end = (char *)_atl_stack_table_current[i];
			}
			_atp_check_leak ( ( stack_start < stack_end ) ?
								stack_start : stack_end,
							  ( stack_start < stack_end ) ?
								stack_end - stack_start : stack_start - stack_end,
								_ATP_BLOCK_STATUS_MIU, _ATP_BLOCK_AREA_STACK);
		  }
      }
    }

  }

#endif /* ATL_MULTI_THREADS && ! RTRT_OTHER_THREADS_INSPECTION_FORBIDDEN */

  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_stack_critical)

  /* other function call for customization */
#ifdef ATP_OTHER_INSPECTION
  ATP_OTHER_INSPECTION
#endif

  /* Inspecting the heap for MIU */
  /* starting with MIU pointed by globals and local vars */
  while(_atp_dump_list)
  {
    _AtpBlockP heap_block;
	  heap_block =_atp_dump_list;
	  _atp_dump_list = _atp_dump_list->Dump;
	  /* check this block to enrich the _atp_dump_list with new MIU */
      _atp_check_leak ( heap_block->Ptr, heap_block->Size , _ATP_BLOCK_STATUS_MIU, _ATP_BLOCK_AREA_HEAP); /* this may change the top list _atp_dump_list */
  }

  /* Inspecting the heap for MPK */
  _atp_dump_list=(_AtpBlockP)0;
  {
    int i;
    _AtpBlockP heap_block;
    for ( i = 0;
          i < _ATP_ENTRIES_CARD;
          i++ )
    {
      for ( heap_block = _atp_hash_table[i];
            heap_block;
            heap_block = heap_block->Next )
      if(_ATP_GET_BLOCK_STATUS(heap_block->Status)==_ATP_BLOCK_STATUS_MPK) /* not yet reviewed and not MLK */
      {
		_atp_add_to_dump_list(heap_block);
      }
    }
	while(_atp_dump_list)
	{
	  heap_block =_atp_dump_list;
	  _atp_dump_list = _atp_dump_list->Dump;
	  /* check this block to enrich the _atp_dump_list with new MPK */
	  _atp_check_leak ( heap_block->Ptr, heap_block->Size , _ATP_BLOCK_STATUS_MPK, _ATP_BLOCK_AREA_HEAP); /* this may change the top list _atp_dump_list */
	}

  }
}

/*******************************************************************************
*  Service: _atp_dump
*******************************************************************************/
_ATL_C_GLOBALFCT int _atp_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
  int err=0;
  if((reset & (_ATCPQ_PRF|_ATCPQ_CHK_WL)) == 0) return(0);

  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  _atp_dump_start_test ( );

  if ( reset & _ATCPQ_PRF)
  {
#if _ATP_DEBUG_HASH
	  {
		int i;

		for ( i = 0; i < _ATP_ENTRIES_CARD; i++ )
		{
			if(_atp_hash_table_max[i])
			{
				ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PH );
				atl_line_inc ( unsigned_reverse_image_16 ( i,atl_line_cur ) );
				atl_line_inc ( unsigned_reverse_image_16 ( _atp_hash_table_max[i],atl_line_cur ) );
				atl_line_dump ( );
		    }
		}
	  }
#endif
	  /* Give memory usage summary (blocks) */
	  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PS );
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_total_blocks,
												 atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_unfreed_blocks,
												 atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_max_blocks,
												 atl_line_cur ) );
	  atl_line_dump ( );

	  /* Give memory usage summary (bytes) */
	  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PY );
#if RTRT_64_BITS_PTR == 0
	/* sizeof(RTRT_SIZE_T) <= 4 */
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_total_bytes,
												 atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_unfreed_bytes,
												 atl_line_cur ) );
	  atl_line_add_char ( ' ' );
	  atl_line_inc ( unsigned_reverse_image_16 ( _atp_max_bytes,
												 atl_line_cur ) );
#else
  if(_atp_total_bytes>>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_total_bytes>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_total_bytes&0xFFFFFFFFUL),atl_line_cur ) );
	  atl_line_add_char ( ' ' );

  if(_atp_unfreed_bytes>>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_unfreed_bytes>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_unfreed_bytes&0xFFFFFFFFUL),atl_line_cur ) );
	  atl_line_add_char ( ' ' );

  if(_atp_max_bytes>>32)
  {
	    atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_max_bytes>>32),atl_line_cur ) );
	    atl_line_add_char ( ',' );
  }
  atl_line_inc ( unsigned_reverse_image_16 ((RTRT_U_INT32)(_atp_max_bytes&0xFFFFFFFFUL),atl_line_cur ) );
#endif
	  atl_line_dump ( );
  }
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

  if ( _atp_unfreed_blocks )
  {
    int i;
    _AtpBlockP unfreed;

    /* Parse memory looking for reference to the memory blocks */
    if ( reset & _ATCPQ_PRF)
		_atp_all_leaks ( );

    /* Give unfreed blocks list: MLK, MPK or MIU */
    for ( i = 0;
          i < _ATP_ENTRIES_CARD;
          i++ )
    {
      for ( unfreed = _atp_hash_table[i];
            unfreed;
            unfreed = unfreed->Next )
      {
#if ATP_REPORTS_ABWL

        err |=_atp_check_for_abwl ( unfreed );

#endif /* ATP_REPORTS_ABWL */
        if ( reset & _ATCPQ_PRF)
		{
			switch ( _ATP_GET_BLOCK_STATUS(unfreed->Status) )
			{
			  case _ATP_BLOCK_STATUS_MLK :
				_atp_dump_MLK ( unfreed );
				break;
			  case _ATP_BLOCK_STATUS_MPK :
				_atp_dump_MPK ( unfreed );
				break;
			  case _ATP_BLOCK_STATUS_MIU :
#if ATP_REPORTS_MIU
				_atp_dump_MIU ( unfreed );
#endif /* ATP_REPORTS_MIU */
				break;
			}
		   _ATP_CLEAR_BLOCK_STATUS(unfreed->Status);/* reset RR  AREA bits as well */
	    }
      }
    }
#if USE_ATT && ATT_ON_THE_FLY
	_atl_MPK_detected=0;
	_atl_MLK_detected=0;
#endif
  }

#if ATP_REPORTS_FFM_FMWL
 if(reset & _ATCPQ_FREE_FRQ)
    _atp_suppress_from_freeq ( ATP_MAX_FREEQ_SIZE );
 else
    err |=_atp_check_freeq();
#endif /* ATP_REPORTS_FFM_FMWL */

  if ( reset & _ATCPQ_PRF)
  {
#if ATP_REPORTS_FIU
#if RTRT_IO == RTRT_STD
  	_atp_dump_FIU ( );
#endif /* RTRT_IO == RTRT_STD */
#endif /* ATP_REPORTS_FIU */

  /* Reset for next test */
  	_atp_test_started = 0;
  }
  else
  	  if(err)
  	  {
  		  _atp_dump_err ( "CHK DUMP",0 );
  	  }
ATL_BUFFER_DUMP ( );
return (err);
}

/*******************************************************************************
*  remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
void _atp_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
_AtpGlobalsFuncTypeP thedata;
_AtpGlobalsFuncTypeP dataPrev;

	RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
    thedata = _atp_first_globals_func_data;
    while((thedata) && (thedata->Family == family))
    {
		thedata = thedata->Next;
	}
	_atp_first_globals_func_data = thedata;
    dataPrev = thedata;
	while(thedata)
	{
       if(thedata->Family == family)
       {
	 	 dataPrev->Next = thedata->Next;
	   }
	   else
	   {
		   dataPrev = thedata;
	   }
	   thedata = thedata->Next;
	}
	RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
}
#endif


#if _ATP_ALLOC_API_NEEDED

/*******************************************************************************
* User Heap API Routines:
*******************************************************************************/

/*******************************************************************************
*  Service: _PurifyLTHeapAction
*******************************************************************************/
_ATL_C_GLOBALFCT void *_PurifyLTHeapAction
#if !RTRT_KR
( _PurifyLT_API_ACTION action,
  void *ptr,
  RTRT_U_INT32 size,
  RTRT_U_INT8 delayed_free )
#else
( action, ptr, size, delayed_free )
  _PurifyLT_API_ACTION action;
  void *ptr;
  RTRT_U_INT32 size;
  RTRT_U_INT8 delayed_free;
#endif
{
  _AtpBlockP link, prev;
  void *ret = (void *)0;
  char delayed=0;
#ifndef RTRT_NO_HOOK_GUARD
  THREADINDEX()
#endif
  if ( ! _ATL_INITIALIZED )
  {
	/* This branch is reached once in the process life */
	atl_early_init ( );
  }

#ifndef RTRT_NO_HOOK_GUARD

#if ATL_MULTI_THREADS
  thread_index= _atl_threads_get_index ();
#endif /* ATL_MULTI_THREADS */

  if ( _atp_hook_guard )
  { /* No recursive calls */
    return ptr;
  }

  _atp_hook_guard = 1;
#endif /* RTRT_NO_HOOK_GUARD */
  if ( action == _PurifyLT_API_ALLOC )
  {
#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
    if ( size == 0 )
    {
      /* It's a nop */
      ret = (void *)0;
    }
    else
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
    if ( ! ptr )
    {
      _atp_dump_MAF ( size );
      ret = (void *)0;
    }
    else
    {
      _atp_create_block ( _ATP_ACTUAL_USER_DATA_CHAR(ptr), size, 0 );
      ret = _ATP_ACTUAL_USER_DATA(ptr);
    }
  }
  else if ( action == _PurifyLT_API_BEFORE_REALLOC )
  {
    if ( ! ptr )
    {
      ret = (void *)0;
    }
    else
    {
      /* Is it something ATP seen ? */
      link = _atp_find_link ( (char *)ptr, &prev );
      if ( link == (_AtpBlockP)0 )
      {
#if ATP_REPORTS_FFM_FMWL
        if ( _atp_check_for_ffm ( (char *)ptr ) )
        {
		  /* avoid to free this ptr twice */
		  ptr = 0;
	    }
	    else
#endif /* ATP_REPORTS_FFM_FMWL */
        {
#if ATP_REPORTS_FUM
          _atp_dump_FUM ( (char *)ptr );
#endif /* ATP_REPORTS_FUM */
        }
        ret = ptr;
      }
      else
      {
#if ATP_REPORTS_FFM_FMWL
        if ( delayed_free && size == 0 )
        { /* It's a free */
          if(_atp_insert_into_freeq ( link ))
          		delayed=1;
        }
#endif /* ATP_REPORTS_FFM_FMWL */
        _atp_suppress_block ( link, prev, delayed);
#if ATP_REPORTS_FFM_FMWL
        if ( delayed )
        {
          ret = (void *)0;
        }
        else
#endif /* ATP_REPORTS_FFM_FMWL */
        ret = _ATP_ACTUAL_INTERNAL_DATA(ptr);
      }
    }
  }
  else if ( action == _PurifyLT_API_FREE )
  {
    /* Is it something ATP seen ? */
    link = _atp_find_link ( (char *)ptr, &prev );
    if ( link == (_AtpBlockP)0 )
    {
#if ATP_REPORTS_FFM_FMWL
      if ( ! delayed_free || ! _atp_check_for_ffm ( (char *)ptr ) )
#endif /* ATP_REPORTS_FFM_FMWL */
      {
#if ATP_REPORTS_FUM
        _atp_dump_FUM ( (char *)ptr );
#endif /* ATP_REPORTS_FUM */
      }
      ret = ptr;
    }
    else
    {
#if ATP_REPORTS_FFM_FMWL
      if ( delayed_free )
      {
          if(_atp_insert_into_freeq ( link ))
          		delayed=1;
      }
#endif /* ATP_REPORTS_FFM_FMWL */
      _atp_suppress_block ( link, prev ,delayed);
#if ATP_REPORTS_FFM_FMWL
      if(delayed)
         ret=(void *)0;
      else
#endif /* ATP_REPORTS_FFM_FMWL */
      {
         ret = _ATP_ACTUAL_INTERNAL_DATA(ptr);
         if ( delayed_free ==_PurifyLT_DO_FREE)
         {
		     RTRT_DO_FREE(ret);
	     }
      }
    }
  }
  /* else call is ignored */
#ifndef RTRT_NO_HOOK_GUARD
  _atp_hook_guard = 0;
#endif /* RTRT_NO_HOOK_GUARD */
  return ret;
}

#if RTRT_FREE_PARAM
/*******************************************************************************
*  Service: _PurifyLTHeapDelayedFree
*******************************************************************************/
_ATL_C_GLOBALFCT int _PurifyLTHeapDelayedFree
#if !RTRT_KR
(
  void *ptr,_freeParam *parameters
)
#else
( ptr, parameters )
  void *ptr;
  _freeParam *parameters;
#endif
{
  _AtpBlockP link, prev;
  _AtpFreeQP freeq;

    /* Is it something ATP seen ? */
    link = _atp_find_link ( (char *)ptr, &prev );
    if ( link == (_AtpBlockP)0 )
    {
#if ATP_REPORTS_FFM_FMWL
      if (! _atp_check_for_ffm ( (char *)ptr ) )
#endif /* ATP_REPORTS_FFM_FMWL */
      {
#if ATP_REPORTS_FUM
        _atp_dump_FUM ( (char *)ptr );
#endif /* ATP_REPORTS_FUM */
      }
	  return 1;
    }
    else
    {
#if ATP_REPORTS_FFM_FMWL
      freeq=_atp_insert_into_freeq ( link );
	if(freeq)
	{
		 priv_memcpy(&(freeq->freeParam),parameters,sizeof(_freeParam));
      	         _atp_suppress_block ( link, prev ,1);
	}
       else
#endif /* ATP_REPORTS_FFM_FMWL */
      {
      	         _atp_suppress_block ( link, prev ,0);
		 RTRT_DO_FREEM(_ATP_ACTUAL_INTERNAL_DATA(ptr),parameters,link);
      }
    }
    return 0;
}
#endif /* RTRT_FREE_PARAM */

/*******************************************************************************
*  Service: _PurifyLTHeapPtrSize
*******************************************************************************/
_ATL_C_GLOBALFCT RTRT_SIZE_T _PurifyLTHeapPtrSize
#if !RTRT_KR
( void *ptr )
#else
( ptr )
  void *ptr;
#endif
{
  _AtpBlockP link, prev;

  if ( ! ptr )
  {
    return (RTRT_SIZE_T)0;
  }
  /* Is it something ATP seen ? */
  link = _atp_find_link ( (char *)ptr, &prev );
  if ( link == (_AtpBlockP)0 )
  {
    return (RTRT_SIZE_T)0;
  }
  return (RTRT_SIZE_T)link->Size;
}

/*******************************************************************************
*  Service: _PurifyLTHeapActualSize
*******************************************************************************/
_ATL_C_GLOBALFCT RTRT_SIZE_T _PurifyLTHeapActualSize
#if !RTRT_KR
( RTRT_SIZE_T size )
#else
( size )
  RTRT_SIZE_T size;
#endif
{
#ifndef RTRT_NO_HOOK_GUARD
#if ATL_MULTI_THREADS
  int thread_index = _atl_threads_get_index ();
#endif /* ATL_MULTI_THREADS */
  if ( _atp_hook_guard )
  {
    return (RTRT_SIZE_T)size;
  }
#endif /* RTRT_NO_HOOK_GUARD */
#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
  if ( size == 0 )
  {
    return (RTRT_SIZE_T)size;
  }
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
  return _ATP_ACTUAL_SIZE(size);
}

/*******************************************************************************
*  Service: _PurifyLTHeapUserSize
*******************************************************************************/
_ATL_C_GLOBALFCT RTRT_SIZE_T _PurifyLTHeapUserSize
#if !RTRT_KR
( RTRT_SIZE_T size )
#else
( size )
  RTRT_SIZE_T size;
#endif
{
  return _ATP_USER_SIZE(size);
}

/*******************************************************************************
*  Service: _PurifyLTHeapActualPtr
*******************************************************************************/
_ATL_C_GLOBALFCT void * _PurifyLTHeapActualPtr
#if !RTRT_KR
( void * ptr )
#else
( ptr )
  void * ptr;
#endif
{
	  _AtpBlockP link, prev;

#ifndef RTRT_NO_HOOK_GUARD
#if ATL_MULTI_THREADS
  int thread_index = _atl_threads_get_index ();
#endif /* ATL_MULTI_THREADS */
  if ( _atp_hook_guard )
  {
    return ptr;
  }
#endif /* RTRT_NO_HOOK_GUARD */
  /* Is it something ATP seen ? */
  link = _atp_find_link ( (char *)ptr, &prev );
  if ( link == (_AtpBlockP)0 )
  {
    return ptr;
  }
  return (void *)(_ATP_ACTUAL_INTERNAL_DATA(ptr));
}

#ifdef NEED_BLOCK_DATA
/*******************************************************************************
*  Service: _PurifyLTHeapBlockData
*******************************************************************************/
_AtpBlockData * _PurifyLTHeapBlockData
#if !RTRT_KR
( void * ptr )
#else
( ptr )
  void * ptr;
#endif
{
	  _AtpBlockP link, prev;
  /* Is it something ATP seen ? */
  link = _atp_find_link ( (char *)ptr, &prev );
  if ( link == (_AtpBlockP)0 )
  {
    return (_AtpBlockData *)0;
  }
  return &(link->AtpBlockData);
}
#endif /* NEED_BLOCK_DATA */
#endif /* _ATP_ALLOC_API_NEEDED */


#if RTRT_HEAP == RTRT_STD

/*******************************************************************************
* Library Allocation Functions:
*******************************************************************************/

/*******************************************************************************
*  Service: _atp_alloc
*******************************************************************************/
_ATL_C_GLOBALFCT void *_atp_alloc
#if !RTRT_KR
( RTRT_SIZE_T size,
  RTRT_U_INT16 Mtype)
#else
( size, Mtype)
  RTRT_SIZE_T size;
  RTRT_U_INT16 Mtype;
#endif
{
  void *ret;

#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
  if ( size == 0 )
  {
    /* It's a nop */
    return (void *)0;
  }
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
  if ( atl_initialized != 2 )
  {
      if ( ! _ATL_INITIALIZED )
	  {
	    /* This branch is reached once in the process life */
	    atl_early_init ( );
	  }
	  else
	    return RTRT_DO_MALLOC ( size );
  }

  ret = RTRT_DO_MALLOC ( _ATP_ACTUAL_SIZE(size) );
  if ( ! ret )
  {
    _atp_dump_MAF ( size );
    ret = (void *)0;
  }
  else
  {
    _atp_create_block ( _ATP_ACTUAL_USER_DATA_CHAR(ret), size, Mtype );
    ret = _ATP_ACTUAL_USER_DATA(ret);
  }
  return ret;
}

/*******************************************************************************
*  Service: _atp_malloc
*******************************************************************************/
_ATL_C_GLOBALFCT void *_atp_malloc
#if !RTRT_KR
( RTRT_SIZE_T size ) RTRT_THROWING_EXCEPTION
#else
( size )
  RTRT_SIZE_T size;
#endif
{
	return _atp_alloc(size,(RTRT_U_INT16)0);
}

/*******************************************************************************
*  Service: _atp_calloc
*******************************************************************************/
_ATL_C_GLOBALFCT void *_atp_calloc
#if !RTRT_KR
( RTRT_SIZE_T nelem,
  RTRT_SIZE_T elsize ) RTRT_THROWING_EXCEPTION
#else
( nelem, elsize )
  RTRT_SIZE_T nelem;
  RTRT_SIZE_T elsize;
#endif
{
  void *ret;

#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
  if ( nelem == 0 || elsize == 0 )
  {
    /* It's a nop */
    return (void *)0;
  }
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
  if ( atl_initialized != 2 )
  {
      if ( ! _ATL_INITIALIZED )
	  {
	    /* This branch is reached once in the process life */
	    atl_early_init ( );
	  }
	  else
		return RTRT_DO_CALLOC ( nelem, elsize );
  }

  ret = RTRT_DO_CALLOC ( _ATP_ACTUAL_SIZE(nelem*elsize), 1 );
  if ( ! ret )
  {
    _atp_dump_MAF ( nelem * elsize );
    ret = (void *)0;
  }
  else
  {
    _atp_create_block ( _ATP_ACTUAL_USER_DATA_CHAR(ret), nelem * elsize, 0 );
    ret = _ATP_ACTUAL_USER_DATA(ret);
  }
  return ret;
}

/*******************************************************************************
*  Service: _atp_realloc
*******************************************************************************/
_ATL_C_GLOBALFCT void *_atp_realloc
#if !RTRT_KR
( void *ptr,
  RTRT_SIZE_T size ) RTRT_THROWING_EXCEPTION
#else
( ptr, size )
  void *ptr;
  RTRT_SIZE_T size;
#endif
{
  void *ret=0;
  char delayed=0;
#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
  if ( ptr == 0 && size == 0 )
  {
    /* It's a nop */
    return (void *)0;
  }
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
  if ( atl_initialized != 2 )
  {
      if ( ! _ATL_INITIALIZED )
	  {
	    /* This branch is reached once in the process life */
	    atl_early_init ( );
	  }
	  else
	    return RTRT_DO_REALLOC ( ptr, size );
  }

  if ( ptr != 0 )
  {
    _AtpBlockP link, prev;

    /* It's at least a free */
    /* Is it something ATP seen ? */
    link = _atp_find_link ( (char *)ptr, &prev );
    if ( link == (_AtpBlockP)0 )
    {
#if ATP_REPORTS_FFM_FMWL
      if ( _atp_check_for_ffm ( (char *)ptr ) )
      {
		  /* avoid to free this ptr twice */
		  ptr = 0;
	  }
	  else
#endif /* ATP_REPORTS_FFM_FMWL */
      {
#if ATP_REPORTS_FUM
        _atp_dump_FUM ( (char *)ptr );
#endif /* ATP_REPORTS_FUM */
      }
      ret = RTRT_DO_REALLOC ( (void *)ptr,
#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
                              size ? _ATP_ACTUAL_SIZE(size) : (RTRT_SIZE_T)0
#else /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
                              _ATP_ACTUAL_SIZE(size)
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
                                                                             );
    }
    else
    {
#if ATP_REPORTS_FIM
      if(_ATP_GET_ALLOC_STATUS(link->Status) !=0 )
		_atp_dump_FIM ( link );
#endif
      if ( size == 0 )
      { /* It's a free */
#if ATP_REPORTS_FFM_FMWL
          if(_atp_insert_into_freeq ( link ))
          		delayed=1;
#endif /* ATP_REPORTS_FFM_FMWL */
         _atp_suppress_block ( link, prev ,delayed);
          if(!delayed)
             ret = RTRT_DO_REALLOC ((void *)_ATP_ACTUAL_INTERNAL_DATA(ptr),
#ifdef RTRT_ZERO_SIZE_ALLOC_IS_NOP
                              (RTRT_SIZE_T)0
#else /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
                              _ATP_ACTUAL_SIZE(size)
#endif /* RTRT_ZERO_SIZE_ALLOC_IS_NOP */
				);
      }
	  else /* size !=0 */
	  {
#if ATP_REPORTS_FFM_FMWL && !defined(RTRT_NO_FMWL_ON_REALLOC)
		 if(link->Size <= ATP_MAX_FREEQ_BLOCK_SIZE)
		 {
            	   ret = RTRT_DO_REALLOC ((void *)0,
                   _ATP_ACTUAL_SIZE(size));
		    if(ret)
		    {
                	priv_memcpy(_ATP_ACTUAL_USER_DATA(ret), ptr, ((size < link->Size)?size:link->Size) );
               		_atp_insert_into_freeq ( link );
		        delayed=1;
		    }
	     }
#endif /* ATP_REPORTS_FFM_FMWL && !defined(RTRT_NO_FMWL_ON_REALLOC) */
         _atp_suppress_block ( link, prev,delayed);
#if ATP_REPORTS_FFM_FMWL && !defined(RTRT_NO_FMWL_ON_REALLOC)
	     if(!delayed)
#endif /* ATP_REPORTS_FFM_FMWL && !defined(RTRT_NO_FMWL_ON_REALLOC) */

            ret = RTRT_DO_REALLOC ((void *)_ATP_ACTUAL_INTERNAL_DATA(ptr),
                              _ATP_ACTUAL_SIZE(size));
      }
    }
  }
  else
  {
    /* It's a malloc */
    ret = RTRT_DO_REALLOC ( (void *)0, _ATP_ACTUAL_SIZE(size) );
  }
  if (size != 0  || ptr == 0 )
  { /* It's at least a malloc */
    if ( ! ret )
    {
      _atp_dump_MAF ( size );
    }
    else
    {
      _atp_create_block ( _ATP_ACTUAL_USER_DATA_CHAR(ret), size, 0 );
      ret = _ATP_ACTUAL_USER_DATA(ret);
    }
  }

  return ret;
}
/*******************************************************************************
*  Service: _atp_free
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_freeT
#if !RTRT_KR
( void *ptr,   RTRT_U_INT16 Mtype)
#else
( ptr )
  void *ptr;
  RTRT_U_INT16 Mtype;
#endif
{
  _AtpBlockP link, prev;
  char delayed=0;

  if ( atl_initialized != 2 )
  {
    /* Is it something ATP seen ? */
    link = _atp_find_link ( (char *)ptr, &prev );
    if ( link == (_AtpBlockP)0 )
    {
      RTRT_DO_FREE ( ptr );
    }
    else
    {
      RTRT_DO_FREE ( _ATP_ACTUAL_INTERNAL_DATA(ptr) );
    }
    return;
  }
  if ( ptr == (void *)0 )
  {
    /* It's a nop */
    return;
  }

  /* Is it something ATP seen ? */
  link = _atp_find_link ( (char *)ptr, &prev );
  if ( link == (_AtpBlockP)0 )
  {
#if ATP_REPORTS_FFM_FMWL
    if ( ! _atp_check_for_ffm ( (char *)ptr ) )
#endif /* ATP_REPORTS_FFM_FMWL */
    {
#if ATP_REPORTS_FUM
      _atp_dump_FUM ( (char *)ptr );
#endif /* ATP_REPORTS_FUM */
    }
    /* free() not called */
  }
  else
  {
#if ATP_REPORTS_FIM
	if(_ATP_GET_ALLOC_STATUS(link->Status) !=Mtype )
		_atp_dump_FIM ( link );
#endif
#if ATP_REPORTS_FFM_FMWL
    if(_atp_insert_into_freeq ( link ))
    	delayed=1;
#endif /* ATP_REPORTS_FFM_FMWL */
    _atp_suppress_block ( link, prev ,delayed);
#if ATP_REPORTS_FFM_FMWL
    if(!delayed)
#endif /* ATP_REPORTS_FFM_FMWL */
       RTRT_DO_FREE ( (void *)_ATP_ACTUAL_INTERNAL_DATA(ptr) );
  }
}
/*******************************************************************************
*  Service: _atp_free
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_free
#if !RTRT_KR
( void *ptr ) RTRT_THROWING_EXCEPTION
#else
( ptr )
  void *ptr;
#endif
{
	 _atp_freeT(ptr,(RTRT_U_INT16)0);
}

#endif /* RTRT_HEAP == RTRT_STD */

#if ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE )

#if RTRT_ATP_STRDUP
/*******************************************************************************
*  Service: _atp_strdup
*******************************************************************************/
_ATL_C_GLOBALFCT char *_atp_strdup
#if !RTRT_KR
( const char *s )
#else
( s )
  char *s;
#endif
{
  char *ret;

  ret = (char *) _atp_malloc ( atl_strlen ( (char *)s ) + 1 );
  atl_strcpy ( ret, (char *)s );
  return ret;
}
#endif /* RTRT_ATP_STRDUP */

#endif /* ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE ) */

/*******************************************************************************
* End of Library Allocations Functions
*******************************************************************************/


#if USE_ATU || USE_ATO
/*******************************************************************************
*  Service: _atp_start_test_driver
*  Goal: some things with test driver integration
*******************************************************************************/
_ATL_C_GLOBALFCT void _atp_start_test_driver
#if !RTRT_KR
( char *addr )
#else
( addr )
  char *addr;
#endif
{
  _atp_test_driver_top_stack = addr;
}
#endif /* USE_ATU || USE_ATO */

/*******************************************************************************
*  Service: atp_early_init
*  Goal: initializes global variables
*******************************************************************************/
_ATL_C_GLOBALFCT void atp_early_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ( ATP_REPORTS_FIU && (RTRT_IO == RTRT_STD) ) || ATL_MULTI_THREADS
  int i;
#endif
  ATP_HEAP_INIT
  /* Writes heading codes */
  RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
  ATL_LINE_ADD_OPCODE_SPACE ( ATL_TPF_PU );
  atl_line_add_char ( RTRT_QUOT );
  atl_buffer_add (ATL_NODE_NAME);
  atl_line_add_char ( RTRT_QUOT );
  atl_line_dump ( );
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_line_mutex)

  /* Initializes specific local variables */
  RTRT_MT_CRITICAL_VAR_INIT(atp_critical)

#if ATP_REPORTS_FFM_FMWL
  _atp_dead_beef[0] = (unsigned char)'\xde';
  _atp_dead_beef[1] = (unsigned char)'\xad';
  _atp_dead_beef[2] = (unsigned char)'\xbe';
  _atp_dead_beef[3] = (unsigned char)'\xef';
#endif /* ATP_REPORTS_FFM_FMWL */

#if ! RTRT_RAM_CLEAN

#if ATP_REPORTS_FFM_FMWL
  _atp_first_freeq = (_AtpFreeQP)0;
  _atp_last_freeq = (_AtpFreeQP)0;
  _atp_freeq_length = (RTRT_U_INT32)0;
  _atp_freeq_size = (RTRT_U_INT32)0;
#endif /* ATP_REPORTS_FFM_FMWL */

  _atp_hash_table = (_AtpBlockP *)0;
#if ATP_CHECK_SELECTED_BLOCKS
  _atp_chk_list = (_AtpBlockP)0;
#endif

  _atp_first_globals_func_data = (_AtpGlobalsFuncTypeP)0;

  _atp_test_started = 0;

#if USE_ATU || USE_ATO
  _atp_test_driver_top_stack = (char *)0;
#endif

#ifndef RTRT_NO_HOOK_GUARD
#if _ATP_ALLOC_API_NEEDED
#if ATL_MULTI_THREADS == ATL_YES
  for ( i = 0; i <= ATL_THREADS_MAX; i++ )
  {
    _atp_hook_guard_table[i] = 0;
  }
#else /* ATL_MULTI_THREADS == ATL_YES */
  _atp_hook_guard = 0;
#endif /* ATL_MULTI_THREADS == ATL_YES */
#endif /* _ATP_ALLOC_API_NEEDED */
#endif /* RTRT_NO_HOOK_GUARD */
  _atp_total_blocks = (RTRT_U_INT32)0;
  _atp_unfreed_blocks = (RTRT_U_INT32)0;
  _atp_max_blocks = (RTRT_U_INT32)0;
  _atp_total_bytes = (RTRT_U_INT32)0;
  _atp_unfreed_bytes = (RTRT_U_INT32)0;
  _atp_max_bytes = (RTRT_U_INT32)0;
  _atp_maximum_addr = (char *)0;
#if USE_ATT && ATT_ON_THE_FLY
  _atl_MPK_detected=0;
  _atl_MLK_detected=0;
#endif

#endif /* RTRT_RAM_CLEAN */
  _atp_minimum_addr = (char *)(-1);

#if ATP_REPORTS_FIU
#if RTRT_IO == RTRT_STD
  for ( i = 3; i < ATL_STD_IO_MAX_FD; i++ )
  {
    _atp_previously_opened[i-3] = (RTRT_U_INT8)priv_is_open(i);
  }
#endif /* RTRT_IO == RTRT_STD */
#endif /* ATP_REPORTS_FIU */

#if ATP_REPORTS_SIG
#if RTRT_SIGNAL == RTRT_STD
  _atl_signal_catch ( _atp_dump_SIG );
#endif /* RTRT_SIGNAL == RTRT_STD */
#endif /* ATP_REPORTS_SIG */
}

#ifdef _ATL_EXIT_PENDING
/*******************************************************************************
*  Service: atp_exit_pending
*  Goal: called when it is about time to exit
*******************************************************************************/
_ATL_C_GLOBALFCT void atp_exit_pending
#if !RTRT_KR
( void )
#else
( )
#endif
{
	int reset=_ATCPQ_ALL;
#if ! USE_ATU && ! USE_ATO
  atl_stack_end = (char *)&reset; /* memorise the top of the stack avoiding to check stack used by TP */
  _atp_dump ( reset );
#endif /* USE_ATU && USE_ATO */
}
#else
/*******************************************************************************
*  Service: atp_late_exit
*  Goal: called when it is time to quit
*******************************************************************************/
_ATL_C_GLOBALFCT void atp_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ATP_CLEAN
#if ATP_REPORTS_FFM_FMWL
	  _AtpFreeQP freeq,dataFree;
#endif
	  _AtpBlockP current, prev;
	  _AtpBlockP *table= _atp_hash_table;
#endif /* ATP_CLEAN */
	int reset=_ATCPQ_ALL;
#if ! USE_ATU && ! USE_ATO
  atl_stack_end = (char *)&reset; /* memorise the top of the stack avoiding to check stack used by TP */
  _atp_dump ( reset );
#endif /* USE_ATU && USE_ATO */
#if ATP_CLEAN

	 if(table)
     {
	int i;
	  RTRT_MT_CRITICAL_VAR_LOCK(atp_critical)
	    _atp_hash_table=(_AtpBlockP *)0;
		for ( i = 0;
			  i < _ATP_ENTRIES_CARD;
			  i++ )
		{
			current=table[i];
			table[i] = (_AtpBlockP)0;
			while(current)
			{
				prev= current;
				current = current->Next;
				RTRT_DO_FREEB( prev );
				/* don't free the use memory */
			}
		}
	    RTRT_DO_FREEH(table);
#if ATP_REPORTS_FFM_FMWL
		freeq = _atp_first_freeq;
		while(freeq)
		{
			dataFree= freeq;
			freeq = freeq->Next;
			RTRT_DO_FREEM( (void *)_ATP_ACTUAL_INTERNAL_DATA(dataFree->Block->Ptr),(&(dataFree->freeParam)),(dataFree->Block) );
 		    RTRT_DO_FREEB( (void *)dataFree->Block );
			RTRT_DO_FREEF( (void *)dataFree );
		}
#endif
	  RTRT_MT_CRITICAL_VAR_UNLOCK(atp_critical)
     }
#endif /* ATP_CLEAN */

  RTRT_MT_CRITICAL_VAR_DESTROY(atp_critical)
}
#endif /* _ATL_EXIT_PENDING */


