

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: atpapi.h,v 1.16.4.1 2008/03/18 10:55:50 build Exp $
* $Descr: PurifyLT API Header (included by custom.h or directly by user code) $
*******************************************************************************/


#ifndef _ATPAPI_INCLUDED
#define _ATPAPI_INCLUDED


#ifndef RTRT_HEAP
/* This is the 'directly by user code' way
 * Let's reload custom.h */
#include "custom.h"
#endif


#if ( RTRT_HEAP == RTRT_USR ) || ( RTRT_MEMLIB_EXTENSION == RTRT_USR )
#define _ATP_ALLOC_API_NEEDED 1
#endif

#ifndef _ATP_ALLOC_API_NEEDED
#define _ATP_ALLOC_API_NEEDED 0
#endif /* _ATP_ALLOC_API_NEEDED */

#if _ATP_ALLOC_API_NEEDED

/* First let's make the job of defaults.h */

#ifndef RTRT_CONCAT_MACRO
/*! Concatenates two macros into one */
#define RTRT_CONCAT_MACRO(a,b) a##b
#endif

#ifndef RTRT_U_INT8
/*! 8 bits unsigned integer */
#define RTRT_U_INT8 unsigned char
#endif

#ifndef RTRT_U_INT32
/*! 32 bits unsigned integer */
#define RTRT_U_INT32 unsigned long
#endif

#ifndef RTRT_SIZE_T
/*! Parameter type for malloc routine. Must be an integer base type, compatible
 *  *  in size and sign with size_t. */
#define RTRT_SIZE_T unsigned int
#endif

#ifndef RTRT_EXTERN
/*! Linkage directive for a function with a C compiler */
#define RTRT_EXTERN extern
#endif

#ifndef RTRT_EXTERN_C
/*! Linkage directive for a C-function with a C++ compiler */
#define RTRT_EXTERN_C extern "C"
#endif

#if RTRT_CPLUSPLUS_LANGUAGE
/*! Linkage directive for a C-function */
#define _ATP_EXTERN_C RTRT_EXTERN_C
#else
#define _ATP_EXTERN_C RTRT_EXTERN
#endif


/*******************************************************************************
* User Heap API Types and Routines:
*******************************************************************************/

typedef enum {
        _PurifyLT_API_ALLOC,
        _PurifyLT_API_BEFORE_REALLOC,
        _PurifyLT_API_FREE
} _PurifyLT_API_ACTION;

/* the block can be freed by the tool only if the free can be done by the usr_free or standard free */
/* the RTRT tool does not free the block because usr_free cannot be used to free the application memory block */
#define _PurifyLT_NO_DELAYED_FREE    0

/* and when usr_free can be used to free the application memory block
   the RTRT tool will free the block only if ATP_REPORTS_FFM_FMWL is set.
   The return value provide the pointer to be freed if the block free cannot be delayed
*/
#define _PurifyLT_DELAYED_FREE       1

/* the RTRT tool will free the block immediately or later if the block free can be delayed */
#define _PurifyLT_DO_FREE	     2

#if !RTRT_KR
_ATP_EXTERN_C void *      _PurifyLTHeapAction     ( _PurifyLT_API_ACTION,
                                                    void *,
                                                    RTRT_U_INT32,
                                                    RTRT_U_INT8 );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapPtrSize    ( void * );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapActualSize ( RTRT_SIZE_T );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapUserSize   ( RTRT_SIZE_T );
_ATP_EXTERN_C void *      _PurifyLTHeapActualPtr  ( void * );
#if RTRT_FREE_PARAM
_ATP_EXTERN_C int         _PurifyLTHeapDelayedFree( void *,_freeParam * );
/* _freeParam is typedef name for the parameter structure to be defined by the custom.h (TDP) */
#endif
#else /* RTRT_KR */
_ATP_EXTERN_C void *      _PurifyLTHeapAction     ( );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapPtrSize    ( );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapActualSize ( );
_ATP_EXTERN_C RTRT_SIZE_T _PurifyLTHeapUserSize ( );
_ATP_EXTERN_C void *      _PurifyLTHeapActualPtr  ( );
#if RTRT_FREE_PARAM
_ATP_EXTERN_C int         _PurifyLTHeapDelayedFree ( );
#endif
#endif /* RTRT_KR */


/* Some macros to save customization time: */

#define _PurifyLT_MALLOC_LIKE(func) \
void *RTRT_CONCAT_MACRO(RTRT_,func) ( RTRT_SIZE_T size ) \
{ \
  void *ret; \
  ret = func ( _PurifyLTHeapActualSize ( size ) ); \
  return _PurifyLTHeapAction ( _PurifyLT_API_ALLOC, ret, size, 0 ); \
}

#define _PurifyLT_CALLOC_LIKE(func) \
void *RTRT_CONCAT_MACRO(RTRT_,func) ( RTRT_SIZE_T nelem, RTRT_SIZE_T elsize ) \
{ \
  void *ret; \
  ret = func ( _PurifyLTHeapActualSize ( nelem * elsize ) ); \
  return _PurifyLTHeapAction ( _PurifyLT_API_ALLOC, ret, nelem * elsize, 0 ); \
}

#define _PurifyLT_REALLOC_LIKE(func,delayed_free) \
void *RTRT_CONCAT_MACRO(RTRT_,func) ( void *ptr, RTRT_SIZE_T size ) \
{ \
  void *ret; \
  ret = func ( _PurifyLTHeapAction ( _PurifyLT_API_BEFORE_REALLOC, \
                                     ptr, size, delayed_free ), \
               _PurifyLTHeapActualSize ( size ) ); \
  return _PurifyLTHeapAction ( _PurifyLT_API_ALLOC, ret, size, 0 ); \
}

#define _PurifyLT_FREE_LIKE(func) \
void RTRT_CONCAT_MACRO(RTRT_,func) ( void *ptr ) \
{ \
    _PurifyLTHeapAction ( _PurifyLT_API_FREE, ptr, 0, _PurifyLT_DO_FREE ); \
}

#endif /* _ATP_ALLOC_API_NEEDED */

#endif /* _ATPAPI_INCLUDED */
