

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: checks.h,v 1.32.4.1 2008/03/18 10:55:50 build Exp $
* $Descr: Configuration checking $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef CHECKS_H_INCLUDED
#define CHECKS_H_INCLUDED

/* This file checks for any incompatibility in the combination of environment
 * and user settings. */

#if USE_ATO && defined(RTRT_TDP_COMPILATION) && !RTRT_CPLUSPLUS_LANGUAGE
#error The C target deployment port does not support Object Testing feature -- use the C++ runtime library (TP.cpp) instead.
#endif

#if USE_ATK && defined(RTRT_TDP_COMPILATION) && !RTRT_CPLUSPLUS_LANGUAGE
#error The C target deployment port does not support Contract Check feature -- use the C++ runtime library (TP.cpp) instead.
#endif

#if USE_ATO && USE_ATT
#if (ATT_ITEMS_MAX>0)
#error Trace Item Buffer is incompatible with Object Testing.
#endif
#endif

#if USE_ATT && (ATT_ITEMS_MAX>0) && (!ATT_DUMP_FROM_BEGINNING)
#error Trace Item Buffer is incompatible with Trace Partial Message Dump.
#endif

#if USE_ATC && ! RTRT_VARIABLE_INIT_SUPPORTED && defined(RTRT_ATC_DECLARE_OVERLOAD)
#error Defining RTRT_ATC_DECLARE_OVERLOAD is forbidden when RTRT_VARIABLE_INIT_SUPPORTED is false, check your customization.
#endif /* RTRT_ATC_DECLARE_OVERLOAD */

#if USE_ATQ && ! RTRT_VARIABLE_INIT_SUPPORTED && defined(RTRT_ATQ_DECLARE_OVERLOAD)
#error Defining RTRT_ATQ_DECLARE_OVERLOAD is forbidden when RTRT_VARIABLE_INIT_SUPPORTED is false, check your customization.
#endif /* RTRT_ATQ_DECLARE_OVERLOAD */

#if USE_ATC && ! RTRT_VARIABLE_INIT_SUPPORTED && defined(RTRT_ATC_LINK_OVERLOAD)
#error Defining RTRT_ATC_LINK_OVERLOAD is forbidden when RTRT_VARIABLE_INIT_SUPPORTED is false, check your customization.
#endif

#if USE_ATP && ( RTRT_HEAP == RTRT_NONE )
#error Memory Profiling feature is not available since Heap Management is not available in the Target Deployment Port.
#endif

#if USE_ATP && ATP_REPORTS_ABWL && (ATP_RED_ZONE_SIZE < 1)
#error Memory Profiling feature cannot report ABWL errors if Red Zone Size setting is set to zero or less.
#endif

#if USE_ATS && defined(_ATS_RTTI_NEEDED) && ( RTRT_RTTI_SUPPORTED == 0 )
#error Compiler does not support RTTI, needed by VIRTUAL CALLBACK statement.
#endif

#if USE_ATO && !USE_ATT && ATL_MULTI_THREADS
#error Multi-threads support required to use trace.
#endif

#if (RTRT_THREADS == RTRT_NONE) && ATL_MULTI_THREADS
#error Multi-threads support is required in the settings, but is not implemented by the Target Deployment Port.
#endif

#if (RTRT_GETPID == RTRT_NONE) && ATL_MULTI_PROCESS
#error Multi-process support is required in the settings, but getpid is not implemented by the Target Deployment Port.
#endif

#if (USE_ATU || USE_ATS || USE_ATO || (ATT_ON_THE_FLY && USE_ATT)) && ATL_MULTI_PROCESS
#error Multi-process support is not compatible with Component Testing for C/C++ and System Testing for C or Trace on the fly.
#endif

#if (RTRT_CLOCK == RTRT_NONE) && USE_ATQ
#warning No clock implementation is provided by the Target Deployment Port. Quantify times will be null.
#endif

#if (RTRT_CLOCK == RTRT_NONE) && (USE_ATT && ATT_TIMESTAMP)
#warning No clock implementation is provided by the Target Deployment Port. Trace time stamps will be null.
#endif

#if (RTRT_MEMCPY == RTRT_NONE) && USE_ATS
#error System Testing feature is unavailable because no memcpy implementation is provided in the Target Deployment Port.
#endif

#if (RTRT_CLOCK == RTRT_NONE) && USE_ATS
#error System Testing feature is unavailable because no clock implementation is provided in the Target Deployment Port.
#endif

#if !RTRT_EXCEPTIONS && RTRT_GENERATE_EXCEPTIONS
#error RTRT_GENERATE_EXCEPTIONS may not be enabled when RTRT_EXCEPTIONS is disabled.
#endif

#if (RTRT_MEMCPY == RTRT_NONE) && USE_ATU
#error Component Testing feature for C is unavailable because no memcpy implementation is provided in the Target Deployment Port.
#endif

#endif
