

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: clock.c,v 1.17.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Time Management (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Optimizations
*******************************************************************************/

#if USE_ATS
/*! Needs priv_elapsed_time() */
#define NEEDS_priv_elapsed_time
#endif

#if USE_ATU || USE_ATC || USE_ATP || USE_ATQ
/*! Needs priv_date() */
#define NEEDS_priv_date
#endif

/******************************************************************************** Macros
*******************************************************************************/

#ifdef NEEDS_priv_clock

#if RTRT_CLOCK == RTRT_STD

#if _ATL_WIN32_API

#if ATL_CLOCK_KIND == ATL_OS_CLOCK

#ifdef RTRT_CLOCK_TICKS_NEEDED
#define RTRT_UNIT_TYPE_FORMAT "3"
#define RTRT_CLOCK_DIFF(r,c1,c2) ((r).QuadPart=(c1).QuadPart-(c2).QuadPart)
#define RTRT_CLOCK_TO_UNITS(c) ((c).QuadPart)
#define RTRT_UNIT_TYPE_NAME "ticks"
#define NEEDS_clock_init /* because resolution has to be pre-computed */
#define NEEDS_atl_clock_resolution
#define RTRT_CLOCK_TO_TIME(c) ((c).QuadPart * 1000L / \
			    atl_clock_resolution.QuadPart)
#define RTRT_TIME_TYPE_FORMAT "0"
#else /* RTRT_CLOCK_TICKS_NEEDED */
#define RTRT_UNIT_TYPE_FORMAT "1"
#define RTRT_CLOCK_DIFF(r,c1,c2) ((r).QuadPart=(c1).QuadPart-(c2).QuadPart)
#define RTRT_CLOCK_TO_UNITS(c) (((c).QuadPart)*1000000L/atl_clock_resolution.QuadPart)
#define RTRT_UNIT_TYPE_NAME "us"
#define NEEDS_clock_init /* because resolution has to be pre-computed */
#define NEEDS_atl_clock_resolution
#define RTRT_CLOCK_TO_TIME(c) ((c).QuadPart * 1000L / \
			    atl_clock_resolution.QuadPart)
#define RTRT_TIME_TYPE_FORMAT "0"
#endif /* RTRT_CLOCK_TICKS_NEEDED */

#elif ATL_CLOCK_KIND == ATL_PROCESS_CLOCK

#define RTRT_UNIT_TYPE_FORMAT "2"
#define RTRT_CLOCK_DIFF(r,c1,c2) (r=(c1)-(c2))
#define RTRT_CLOCK_TO_UNITS(c) (c)
#define RTRT_UNIT_TYPE_NAME "ticks"
#define RTRT_CLOCK_TO_TIME(c) ((c) * 1000 / CLOCKS_PER_SEC)
#define RTRT_TIME_TYPE_FORMAT "0"

#endif

#else /* _ATL_WIN32_API */

#define RTRT_UNIT_TYPE_FORMAT "1"
#define RTRT_CLOCK_DIFF(r,c1,c2) atl_clock_diff(&(r), &(c1), &(c2))
#define RTRT_CLOCK_TO_UNITS(c) \
  (((RTRT_UNIT_TYPE)(c).tv_sec*1000000L)+(c).tv_usec)
#define RTRT_UNIT_TYPE_NAME "us"
#define RTRT_CLOCK_TO_TIME(c) ((c).tv_sec * 1000 + (c).tv_usec / 1000)
#define RTRT_TIME_TYPE_FORMAT "0"

#ifdef NEEDS_priv_elapsed_time
RTRT_STATICFCT void atl_clock_diff(RTRT_CLOCK_TYPE *r,
                    RTRT_CLOCK_TYPE *c1, RTRT_CLOCK_TYPE *c2)
{
  r->tv_sec = c1->tv_sec - c2->tv_sec;
  if (c1->tv_usec < c2->tv_usec)
  {
    r->tv_sec--;
    r->tv_usec = 1000000L + c1->tv_usec - c2->tv_usec;
  }
  else
  {
    r->tv_usec = c1->tv_usec - c2->tv_usec;
  }
}
#endif /* NEEDS_priv_elapsed_time */

#endif /* _ATL_WIN32_API */

#elif RTRT_CLOCK == RTRT_NONE

#define RTRT_UNIT_TYPE_FORMAT "2"
#define RTRT_CLOCK_DIFF(r,c1,c2) ((r)=(c1)-(c2))
#define RTRT_CLOCK_TO_UNITS(c) (c)
#define RTRT_UNIT_TYPE_NAME "not implemented"
#define RTRT_CLOCK_TO_TIME(c) (c)
#define RTRT_TIME_TYPE_FORMAT "2"

#endif /* RTRT_CLOCK == RTRT_STD */

#if RTRT_CLOCK == RTRT_USR
#define NEEDS_clock_init
#endif /* RTRT_CLOCK == RTRT_USR */

#define _ATL_TIME_TO_SECONDS(t) \
   ((t) / (1000000L / RTRT_TIME_TYPE_STEP))
#define _ATL_TIME_TO_MSECONDS(t) \
  ((((t) % (1000000L / RTRT_TIME_TYPE_STEP)) * RTRT_TIME_TYPE_STEP) / 1000L)

#if defined(NEEDS_atl_clock_resolution)
RTRT_GLOBALVAR RTRT_CLOCK_TYPE atl_clock_resolution;
#endif

/*******************************************************************************
* Routines
*******************************************************************************/

#if RTRT_CLOCK == RTRT_STD
#if ATL_CLOCK_KIND == ATL_OS_CLOCK
#if _ATL_WIN32_API
#include <time.h>
#else
#include <unistd.h>
#endif /* _ATL_WIN32_API */
#elif ATL_CLOCK_KIND == ATL_PROCESS_CLOCK
#if _ATL_WIN32_API
#include <wtypes.h>
#include <winbase.h>
#else
#include <sys/resource.h>
#include <unistd.h>
#endif /* _ATL_WIN32_API */
#endif /* ATL_CLOCK_KIND */
#endif /* RTRT_CLOCK == RTRT_STD */

/*! Returns execution time */
_ATL_C_GLOBALFCT void priv_clock
#if !RTRT_KR
  (RTRT_CLOCK_TYPE *ret)
#else
  (ret)
  RTRT_CLOCK_TYPE *ret;
#endif
{
#if RTRT_CLOCK == RTRT_STD
#if ATL_CLOCK_KIND == ATL_OS_CLOCK
#if _ATL_WIN32_API
  QueryPerformanceCounter(ret);
#else
  static struct timezone tz;

  gettimeofday (ret, &tz);
#endif /* _ATL_WIN32_API */
#elif ATL_CLOCK_KIND == ATL_PROCESS_CLOCK
#if _ATL_WIN32_API
  *ret = clock();
#else
  static struct rusage usage;

  getrusage (RUSAGE_SELF, &usage);
  ret->tv_sec = usage.ru_utime.tv_sec + usage.ru_stime.tv_sec;
  ret->tv_usec = usage.ru_utime.tv_usec + usage.ru_stime.tv_usec;
#endif /* _ATL_WIN32_API */
#endif /* ATL_CLOCK_KIND */
#elif RTRT_CLOCK == RTRT_USR
  usr_clock (ret);
#elif RTRT_CLOCK == RTRT_NONE
  *ret = (RTRT_CLOCK_TYPE)0;
#endif
}

#endif /* NEEDS_priv_clock */

#ifdef NEEDS_priv_elapsed_time
/*! Compute elapsed time since from_date to to_date if use_to_date != 0.
 *  Compute elapsed time since from_date to now otherwise. */
_ATL_C_GLOBALFCT RTRT_TIME_TYPE priv_elapsed_time
#if !RTRT_KR
  (RTRT_CLOCK_TYPE *from_date,
   RTRT_CLOCK_TYPE *to_date,
   RTRT_U_INT8 use_to_date)
#else
  (from_date,to_date,use_to_date)
  RTRT_CLOCK_TYPE *from_date;
  RTRT_CLOCK_TYPE *to_date;
  RTRT_U_INT8 use_to_date;
#endif
{
  RTRT_CLOCK_TYPE diff;

  if (use_to_date)
  {
    RTRT_CLOCK_DIFF (diff, *to_date, *from_date);
  }
  else
  {
    RTRT_CLOCK_TYPE now;
    priv_clock (&now);
    RTRT_CLOCK_DIFF (diff, now, *from_date);
  }
  return RTRT_CLOCK_TO_TIME(diff);
}
#endif /* NEEDS_priv_elapsed_time */

#ifdef NEEDS_clock_init
RTRT_STATICFCT RTRT_GLOBALFCT void clock_init ()
{
#if RTRT_CLOCK == RTRT_STD
#ifdef NEEDS_atl_clock_resolution
#if _ATL_WIN32_API
  if(QueryPerformanceFrequency(&atl_clock_resolution)==0)
  {
    atl_clock_resolution.LowPart=0;
    atl_clock_resolution.HighPart=0;
  }
#endif
#endif /* NEEDS_atl_clock_resolution */
#elif RTRT_CLOCK == RTRT_USR
  usr_clock_init();
#endif /* RTRT_CLOCK == ... */
}
#endif /* NEEDS_clock_init */

#ifdef NEEDS_priv_date
RTRT_STATICFCT char * priv_date(); /* prototype for bad compiler */
/*! provide the date as string format for observation tools */
RTRT_STATICFCT char * priv_date()
{
#if RTRT_GET_DATE == RTRT_USR
 return(usr_date());
#else
	return (char *)0;
#endif
}
#endif /* NEEDS_priv_date */

