

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: image.c,v 1.40.2.4 2008/03/18 10:55:50 build Exp $
* $Descr: Common Presentation Functions (included by TP.c[pp]) $
*******************************************************************************/

/*******************************************************************************
* Optimizations
*******************************************************************************/

#if USE_ATO || USE_ATU
/*! Needs integer_image_10() */
#define NEEDS_integer_image_10
#endif

#if ( USE_ATT && ATT_COMPUTE_MAX_CALLING_LEVEL ) || USE_ATO || USE_ATU
/*! Needs unsigned_image_10() */
#define NEEDS_unsigned_image_10
#endif

#if USE_ATO || (ATC_DUMP_DRIVER ) || (ATQ_DUMP_DRIVER ) || ATL_MULTI_PROCESS
/*! Needs unsigned_image_16() */
#define NEEDS_unsigned_image_16
#endif

#if USE_ATO || USE_ATK || USE_ATC || USE_ATP || USE_ATQ || USE_ATT
/*! Needs unsigned_reverse_image_16() */
#define NEEDS_unsigned_reverse_image_16
#endif

#if (USE_ATO || USE_ATU) && RTRT_FLOAT
/*! Needs float_image() */
#define NEEDS_float_image
#endif

#if (USE_ATO || USE_ATU) && RTRT_FLOAT
/*! Needs double_image() */
#define NEEDS_double_image
#endif

#if (USE_ATO || USE_ATU) && RTRT_FLOAT && RTRT_LONG_DOUBLE
/*! Needs long_double_image() */
#define NEEDS_long_double_image
#endif

#if USE_ATU || USE_ATS || ( USE_ATP && ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE ) )
/*! Needs atl_strcpy() */
#define NEEDS_atl_strcpy
#endif

#if USE_ATP && ( RTRT_HEAP == RTRT_STD ) && ( RTRT_LOWLEVEL_MEMLIB == RTRT_NONE )
/*! Needs atl_strlen() */
#define NEEDS_atl_strlen
#endif

#if USE_ATS
/*! Needs atl_strcmp() */
#define NEEDS_atl_strcmp
#endif

#if USE_ATS
/*! Needs atl_strncmp() */
#define NEEDS_atl_strncmp
#endif

#if USE_ATS
/*! Needs atl_strcat() */
#define NEEDS_atl_strcat
#endif

#if USE_ATS
/*! Needs atl_strcat() */
#define NEEDS_atl_strstr
#endif

#if USE_ATS
/*! Needs atl_fltcmp() */
#define NEEDS_atl_fltcmp
#endif

#if USE_ATS
/*! Needs atl_dblcmp() */
#define NEEDS_atl_dblcmp
#endif

#if USE_ATS
/*! Needs atl_sprintf() */
#define NEEDS_atl_sprintf
#endif

#if defined(NEEDS_unsigned_image_16) || \
    defined(NEEDS_unsigned_reverse_image_16)
/*! Needs variable _atl_hexa_digits */
#define NEEDS_atl_hexa_digits
#endif

/*******************************************************************************
* Includes
*******************************************************************************/

#if RTRT_SPRINTF
#include <stdio.h>
#endif

/*******************************************************************************
* Macros
*******************************************************************************/

#define _ATL_MAKE_EPSILON2(e) RTRT_CONCAT_MACRO(1.0E-,e)
#define _ATL_MAKE_EPSILON1(e) _ATL_MAKE_EPSILON2(e)

#ifndef ATL_EPSILON_FLT
/*! Smallest float number */
#define ATL_EPSILON_FLT _ATL_MAKE_EPSILON1(RTRT_NBDIGITS_FLT)

/*! Smalled double number */
#define ATL_EPSILON_DBL _ATL_MAKE_EPSILON1(RTRT_NBDIGITS_DBL)

/*! Smallest long double number */
#define ATL_EPSILON_LDBL _ATL_MAKE_EPSILON1(RTRT_NBDIGITS_LDBL)
#endif

#define _ATL_MAKE_FMT2(NB) "%." #NB "E"
#define _ATL_MAKE_FMT1(NB) _ATL_MAKE_FMT2(NB)


#ifndef ATL_FLT_FMT
/*! String format (printf family) for float numbers */
#define ATL_FLT_FMT     _ATL_MAKE_FMT1(RTRT_NBDIGITS_FLT)

/*! String format (printf family) for double numbers */
#define ATL_DBL_FMT     _ATL_MAKE_FMT1(RTRT_NBDIGITS_DBL)

/*! String format (printf family) for long double numbers */
#define ATL_LDBL_FMT    _ATL_MAKE_FMT1(RTRT_NBDIGITS_LDBL)
#endif

#ifdef RTRT_INT64
/*! Largest representable signed integer */
#define _ATL_LARGEST_INT RTRT_INT64
#else
#define _ATL_LARGEST_INT RTRT_INT32
#endif

#ifdef RTRT_U_INT64
/*! Largest representable unsigned integer */
#define _ATL_LARGEST_U_INT RTRT_U_INT64
#else
#define _ATL_LARGEST_U_INT RTRT_U_INT32
#endif


/*******************************************************************************
* Variables
*******************************************************************************/

#ifdef NEEDS_atl_hexa_digits
#ifndef RTRT_CUSTOM_HEXA_DIGITS
RTRT_STATICVAR char _atl_hexa_digits[16];
/* some target cannot use pointer on const string */
RTRT_STATICFCT void _atl_ini_digits
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  int i;
  	for (i=0;i<10;i++) _atl_hexa_digits[i]='0'+i;
  	for (;i<=15;i++) _atl_hexa_digits[i]=i+('a'-10);
}
#define ATL_HEXA_DIGITS(x) _atl_hexa_digits[x]
#endif /* RTRT_CUSTOM_HEXA_DIGITS */
#endif

/*******************************************************************************
* Char image
*******************************************************************************/

#if USE_ATU
/*! Outputs character c in buffer res. Special characters are backslash
 *  protected. Returns the length of the generated string. */
RTRT_STATICFCT int char_image
#if !RTRT_KR
  (char c, char *res)
#else
  (c, res)
  char c;
  char *res;
#endif
{
  int i = 0;
  res[i++]= '\''; /* quot before */
  if ((c < ' ') || (c > 126))
  {
	res[i++] = '\\';
    switch (c)
    {
      case('\n'):
	res[i++] = 'n';
	break;

      case('\r'):
	res[i++] = 'r';
	break;

      case('\t'):
	res[i++] = 't';
	break;

      case('\f'):
	res[i++] = 'f';
	break;

      default:
	if (c & 0300) res[i++] = (((c) & 0300) >> 6) + '0';
	if (c & 0370) res[i++] = (((c) & 0070) >> 3) + '0';
	res[i++] = ((c) & 0007) + '0';
    }
  }
  else
  {
    if ((c == '\'') || (c == '\\'))
    {
      res[i++] = '\\';
    }
    res[i++] = (c);
  }
  res[i++]= '\''; /* quot after */
  res[i] = 0;
  return i;
}
/*******************************************************************************
* TRUE/FALSE image
*******************************************************************************/
RTRT_STATICFCT void atl_add_bool_image
#if !RTRT_KR
  (int value)
#else
  (value)
  int value;
#endif
{
  if (value)
	{
	  ATL_LINE_ADD_OPCODE("TRUE");
	}
  else
	{
	  ATL_LINE_ADD_OPCODE("FALSE");
	}
}
#endif

/*******************************************************************************
* String image
*******************************************************************************/

#if ATQ_DUMP_DRIVER
/*! Outputs string s in buffer res. Special characters are backslash
 *  protected. lres specify the maximum length of the generated string (it may
 *  be truncated if needed). Returns the length of the generated string. */
RTRT_STATICFCT int string_image
#if !RTRT_KR
  (const char *s, char *res, int lres)
#else
  (s, res, lres)
  const char *s;
  char *res;
  int lres;
#endif
{
  int i = 0;

  while (((*s) != '\0')&& (i < lres))
  {
    if ((*s < ' ') || (*s > 126))
    {
	  *res++ = '\\';
	  i+=2;
      switch (*s)
      {
	case('\n'):
	  *res++ = 'n';
	  break;

	case('\r'):
	  *res++ = 'r';
	  break;

	case('\t'):
	  *res++ = 't';
	  break;

	case('\f'):
	  *res++ = 'f';
	  break;

	default:
	  *res++ = (((*s) & 0300) >> 6) + '0';
	  *res++ = (((*s) & 0070) >> 3) + '0';
	  *res++ = ((*s) & 0007) + '0';
	  i+=2;
      }
    }
    else
    {
      if ((*s == RTRT_QUOT) || (*s == '\\'))
      {
		*res++ = '\\';
		i++;
      }
      *res++ = (*s);
    }
    ++s;
  }
  *res = '\0';
  return i;
}
#endif /* ATQ_DUMP_DRIVER */

#if (RTRT_STRING && USE_ATU) || USE_ATK
RTRT_STATICFCT void atl_buffer_string_image (const char *s)
{

	  while (*s)
	  {
		if ((*s < ' ') || (*s > 126))
		{
		  *atl_buffer_next++ = '\\';
		  switch (*s)
		  {
		case('\n'):
		  *atl_buffer_next++ = 'n';
		  break;

		case('\r'):
		  *atl_buffer_next++ = 'r';
		  break;

		case('\t'):
		  *atl_buffer_next++ = 't';
		  break;

		case('\f'):
		  *atl_buffer_next++ = 'f';
		  break;

		default:
		  *atl_buffer_next++ = (((*s) & 0300) >> 6) + '0';
		  *atl_buffer_next++ = (((*s) & 0070) >> 3) + '0';
		  *atl_buffer_next++ = ((*s) & 0007) + '0';
		  }
		}
		else
		{
		  if ((*s == RTRT_QUOT) || (*s == '\\'))
		  {
			*atl_buffer_next++ = '\\';
		  }
		  *atl_buffer_next++ = (*s);
		}
		++s;
		if(atl_buffer_next >= atl_buffer_slack)
			atl_buffer_dump ();
	  }

	  *atl_buffer_next = '\0';
}

#endif /* USE_ATU || USE_ATK */

#if USE_ATO || USE_ATK || USE_ATT
/* generate the string note in locale encoding */
RTRT_STATICFCT void atl_buffer_note (const char *s)
{
register char c;
	  while (*s)
	  {
		c=*s++;
		if ( c == '\n' )
		{
		   *atl_buffer_next++ = '\\';
		   *atl_buffer_next++ = '\\';
		   c = 'n';
		}
		else if ( c == '\r' )
		{
		   *atl_buffer_next++ = '\\';
		   *atl_buffer_next++ = '\\';
		   c = 'r';
		}
		if ((c == RTRT_QUOT) || (c == '\\'))
		{
			   *atl_buffer_next++ = '\\';
		}
		*atl_buffer_next++ = (c);

		if(atl_buffer_next >= atl_buffer_slack)
			atl_buffer_dump ();
	  }

	  *atl_buffer_next = '\0';
}
#endif


/*******************************************************************************
* Unsigned integer image base 10
*******************************************************************************/

#ifdef NEEDS_unsigned_image_10
/*! Outputs unsigned integer n in buffer res, represented in base 10.
 * Returns the length of the generated string. */
RTRT_STATICFCT int unsigned_image_10
#if !RTRT_KR
  (_ATL_LARGEST_U_INT n, char *res)
#else
  (n, res)
  _ATL_LARGEST_U_INT n;
  char *res;
#endif
{
#if !RTRT_SPRINTF
  _ATL_LARGEST_U_INT nint = 0;
  _ATL_LARGEST_U_INT n1 = 0;
  int l = 0;
  char* tmp = 0;

	  tmp=&res[21];/* maximum digit number */
	  *tmp--=0;
	  do
		{
		n1 = n / 10;
		nint = n -(n1*10);
		*tmp-- = '0' + (char)(nint);
		l++;
		n = n1 ;
		}
	  while (n != 0);

	/* move the string */
	  tmp++;
	  do {
		  *res++=*tmp++;
	  } while(*tmp);
  *res = 0;
  return l;
#else /* !RTRT_SPRINTF */
  return sprintf (res, RTRT_SPRINTF_UNSIGNED_FORMAT, n);
#endif /* !RTRT_SPRINTF */
}
#endif /* NEEDS_unsigned_image_10 */
/*******************************************************************************
* Signed integer image base 10
*******************************************************************************/

#ifdef NEEDS_integer_image_10
/*! Outputs signed integer n in buffer res, represented in base 10. Sign is put
 *  after the representation. Returns the length of the generated string. */
RTRT_STATICFCT int integer_image_10
#if !RTRT_KR
  (_ATL_LARGEST_INT n, char *res)
#else
  (n, res)
  _ATL_LARGEST_INT n;
  char *res;
#endif
{
#if !RTRT_SPRINTF
  int i;
  i = 0;
  if  (n < 0) {
    n = -n;
    *res++ = '-';
    i=1;
  }

  return (unsigned_image_10(n,res)+i);
#else /* !RTRT_SPRINTF */
  return sprintf(res, RTRT_SPRINTF_INTEGER_FORMAT, n);
#endif /* !RTRT_SPRINTF */
}
#endif /* NEEDS_integer_image */


/*******************************************************************************
* Unsigned integer image base 16
*******************************************************************************/
#ifdef NEEDS_unsigned_image_16
/*! Outputs unsigned integer n in buffer res, in regular hexadecimal format.
 *  No prefix or suffix is generated! */
RTRT_STATICFCT int unsigned_image_16
#if !RTRT_KR
  (RTRT_U_INT32 n, char *res)
#else
  (n, res)
  RTRT_U_INT32 n;
  char *res;
#endif
{
  char *p=res;
  short i=32;
  short write=0;

  do{
	i -= 4;
    *p = ATL_HEXA_DIGITS((RTRT_U_INT8)(n>>i)&0x0F);
    if ( write ) p++;
    else if ( *p > '0' || i==0) { p++; write = 1; }
  } while(i);

  *p = (char)0;
  return (int)(p - res);
}
#endif /* NEEDS_unsigned_image_16 */

/*******************************************************************************
* Unsigned integer image base 16 in reverse order
*******************************************************************************/
#ifdef NEEDS_unsigned_reverse_image_16
/*! Outputs unsigned integer n in buffer res, in reverse hexadecimal format.
 *  No prefix or suffix is generated! Note: this is the most efficient way
 *  to generate a hexadecimal unsigned integer. */
RTRT_STATICFCT int unsigned_reverse_image_16
#if !RTRT_KR
  (RTRT_U_INT32 n, char *res)
#else
  (n, res)
  RTRT_U_INT32 n;
  char *res;
#endif
{
  char *p;

  p = &res[0];
  do
  {
    *p++ = ATL_HEXA_DIGITS(n&0x0000000F);
    n >>= 4;
  } while (n);
  *p = (char)0;
  return (int)(p - res);
}
#endif /* NEEDS_unsigned_reverse_image_16 */


/*******************************************************************************
* Floating-point numbers image
*******************************************************************************/

#if !RTRT_SPRINTF && !RTRT_FLOAT_DUMP && \
    (defined(NEEDS_float_image) || defined(NEEDS_double_image))
/*! Outputs floating number f into buffer res. Returns the length of the
 *  generated string. This routine is used internally in image.inc, not to be
 *  called by other modules. */
RTRT_STATICFCT int real_image
#if !RTRT_KR
  (double f, char *res, double epsilon, int precision)
#else
  (f, res, epsilon, precision)
  double f;
  char *res;
  double epsilon;
  int precision;
#endif
{
  int posit = 0;
  int point_set = 0;
  long n = 0;
  int m = precision;
  int expo = 0;
  double pow = 0.1;
  double l;
  double r;

  if (f == 0.0)
  {
    res[posit++] = '0';
  }
  else
  {
    /* Sign */
    if (f < 0.0)
    {
      res[posit++] = '-';
      f = (-f);
    }

    l= f;
	r=(0.1 + epsilon);
	while (l < r)
	{
		--expo;
		l *= 10.0;
		pow /= 10.0;
	}
	r=10.0 *(1.0 - epsilon);
	l = f / pow ;
	while (l > r)
	{
		expo++;
		pow *= 10.0;
		l = f / pow ;
	}
	r=epsilon;
	pow=(1.0 - r);
    do
    {
      n = (long)l;
      l = l - (double)n;
      if (l > pow) {n++; l= 0.0;}
      else if (l < r) {l=0.0;}
      /* stip off error and round up */
      else {l *= 10.0; r *= 10.0; pow=(1.0 - r/2.0);}
      /* now write the number and dot */
      res[posit++] = n + '0';
      if ( ! point_set )
      { /* write the dot at the beginning if les than 1.0 or if power > precision */
        if ((--expo <= 0) || (expo > m))
		{
          res[posit++] = '.';
          point_set = 1;
          /* we are writting the last digit so add one 0 after the dots */
          if(m==0){res[posit++] = '0';}
		}
      }
	  else
	  {
          if(l == 0.0) {m=0;} /* no need to continue to write more 0 after the dot */
	  }
    }
    while (m--); /* NB+1 digits to display as the sprintf does */
  }
  if ( ! point_set )
  {
	  res[posit++] = '.';
	  res[posit++] = '0';
  }
  if (expo != 0)
  {
	  res[posit++] = 'e';
	  if (expo > 0) res[posit++] = '+';
	  posit += integer_image_10(expo, &(res[posit]));
  }
  else
  {
	  res[posit] = '\0';
  }
  return posit;
}
#endif

#ifdef NEEDS_float_image
/*! Outputs floating number f into buffer res. Returns the length of the
 *  generated string. */
RTRT_STATICFCT int float_image
#if !RTRT_KR
  (float f, char *res)
#else
  (f, res)
  float f;
  char *res;
#endif
{
#if !RTRT_SPRINTF
#if RTRT_FLOAT_DUMP
  union X {
     float f;
     RTRT_DUMP_OBJ N[(sizeof(f)+sizeof(RTRT_DUMP_OBJ)-1)/sizeof(RTRT_DUMP_OBJ)]; } conv;
  RTRT_DUMP_OBJ i;
  char * b;
  int l;
  char c;
  b=res;

  conv.f=f;

  (*b)='!';
  b++;
  for (l=0;l<(sizeof(conv.N)/sizeof(conv.N[0]));++l)
  {
    for ( i=1;i;conv.N[l]>>=8,i<<=8)
    {
      c=(conv.N[l]&0x0f0)>>4;
      if (c<10)
      {
	(*b)=c+'0';
      }
      else
      {
	(*b)=c+('A'-10);
      }
      b++;
      c=(conv.N[l]&0x0f);
      if (c<10)
      {
	(*b)=c+'0';
      }
      else
      {
	(*b)=c+('A'-10);
      }
      b++;
    }
  }
  (*b)='!';
  b++;
  (*b)='\0';

  return (b - res);
#else /* RTRT_FLOAT_DUMP */
  return real_image((double)f, res, ATL_EPSILON_FLT, RTRT_NBDIGITS_FLT);
#endif /* RTRT_FLOAT_DUMP */
#else /* !RTRT_SPRINTF */
  return sprintf (res, ATL_FLT_FMT, f);
#endif /* !RTRT_SPRINTF */
}
#endif /* NEEDS_float_image */

#ifdef NEEDS_double_image
/*! Outputs floating number f into buffer res. Returns the length of the
 *  generated string. */
RTRT_STATICFCT int double_image
#if !RTRT_KR
  (double f, char *res)
#else
  (f, res)
  double f;
  char *res;
#endif
{
#if !RTRT_SPRINTF
#if RTRT_FLOAT_DUMP
  union X {
     double f;
     RTRT_DUMP_OBJ N[(sizeof(f)+sizeof(RTRT_DUMP_OBJ)-1)/sizeof(RTRT_DUMP_OBJ)]; } conv;
  RTRT_DUMP_OBJ i;
  char * b;
  int l;
  char c;
  b=res;

  conv.f=f;
  (*b)='!';
  b++;
  for (l=0;l<(sizeof(conv.N)/sizeof(conv.N[0]));++l)
  {
  for ( i=1;i;conv.N[l]>>=8,i<<=8)
    {
      c=(conv.N[l]&0x0f0)>>4;
      if (c<10)
        {
          (*b)=c+'0';
        }
      else
        {
          (*b)=c+('A'-10);
        }
      b++;
      c=(conv.N[l]&0x0f);
      if (c<10)
        {
          (*b)=c+'0';
        }
      else
        {
          (*b)=c+('A'-10);
        }
      b++;
    }
  }
  (*b)='!';
  b++;
  (*b)='\0';

  return (b - res);
#else /* RTRT_FLOAT_DUMP */
  return real_image(f, res, ATL_EPSILON_DBL, RTRT_NBDIGITS_DBL);
#endif /* RTRT_FLOAT_DUMP */
#else /* !RTRT_SPRINTF */
  return sprintf (res, ATL_DBL_FMT, f);
#endif /* !RTRT_SPRINTF */
  }
#endif /* NEEDS_double_image */

#ifdef NEEDS_long_double_image
/*! Outputs floating number f into buffer res. Returns the length of the
 *  generated string. */
RTRT_STATICFCT int long_double_image
#if !RTRT_KR
  (long double f, char *res)
#else
  (f, res)
  long double f;
  char *res;
#endif
{
#if!RTRT_SPRINTF
#if RTRT_FLOAT_DUMP
  union X {
    long double f;
    RTRT_DUMP_OBJ N[(sizeof(f)+sizeof(RTRT_DUMP_OBJ)-1)/sizeof(RTRT_DUMP_OBJ)]; } conv;
  RTRT_DUMP_OBJ i;
  char * b;
  int l;
  char c;
  b=res;

  conv.f=f;
  (*b)='!';
  b++;
  for (l=0;l<(sizeof(conv.N)/sizeof(conv.N[0]));++l)
  {
    for ( i=1;i;conv.N[l]>>=8,i<<=8)
    {
      c=(conv.N[l]&0x0f0)>>4;
      if (c<10)
      {
	(*b)=c+'0';
      }
      else
      {
	(*b)=c+('A'-10);
      }
      b++;
      c=(conv.N[l]&0x0f);
      if (c<10)
      {
	(*b)=c+'0';
      }
      else
      {
	(*b)=c+('A'-10);
      }
      b++;
    }
  }
  (*b)='!';
  b++;
  (*b)='\0';

  return (b - res);
#else /* RTRT_FLOAT_DUMP */
  return real_image((long double)f, res, ATL_EPSILON_LDBL, RTRT_NBDIGITS_LDBL);
#endif /* RTRT_FLOAT_DUMP */
#else /* !RTRT_SPRINTF */
  return sprintf (res, ATL_LDBL_FMT, (double)f);
#endif /* !RTRT_SPRINTF */
  }
#endif /* NEEDS_long_double_image */

/*******************************************************************************
* Other routines
*******************************************************************************/

#ifdef NEEDS_atl_strcpy
/*! Copy the null-terminated string o to d */
_ATL_C_GLOBALFCT char *atl_strcpy
#if !RTRT_KR
( char *d,
  char *o )
#else
( d, o )
  char *d;
  char *o;
#endif
{
  char *p, *q;

  for ( p = o, q = d; *p; p++, q++ )
    *q = *p;
  *q = (char)0;
  return d;
}
#endif /* NEEDS_atl_strcpy */

#ifdef NEEDS_atl_strlen
/*! give the string length of o */
_ATL_C_GLOBALFCT int atl_strlen
#if !RTRT_KR
( char *o )
#else
( o )
  char *o;
#endif
{
  char *p;

  for ( p = o; *p; p++ );
  return (int)(p - o);
}
#endif /* NEEDS_atl_strlen */

#ifdef NEEDS_atl_strcmp
_ATL_C_GLOBALFCT int atl_strcmp
#if !RTRT_KR
( const char *str1,
  const char *str2)
#else
( str1, str2)
const char *str1;
const char *str2;
#endif
{
  return (strcmp (str1, str2));
}
#endif /* NEEDS_atl_strcmp */

#ifdef NEEDS_atl_strncmp
_ATL_C_GLOBALFCT int atl_strncmp
#if !RTRT_KR
  (const char *str1, const char *str2, unsigned int index)
#else
  (str1, str2, index)
  const char *str1;
  const char *str2;
  const unsigned int index;
#endif
{
  return (strncmp (str1, str2, index));
}
#endif /* NEEDS_atl_strncmp */

#ifdef NEEDS_atl_strcat
#include <string.h>
_ATL_C_GLOBALFCT char *atl_strcat
#if !RTRT_KR
  (char *str1, const char *str2)
#else
  (str1, str2)
  char *str1;
  const char *str2;
#endif
{
  return (strcat (str1, str2));
}
#endif /* NEEDS_atl_strcat */

#ifdef NEEDS_atl_strstr
#include <string.h>
_ATL_C_GLOBALFCT char *atl_strstr
#if !RTRT_KR
  (char *str1, char *str2)
#else
  (str1, str2)
  char *str1;
  char *str2;
#endif
{
  return (strstr (str1, str2));
}
#endif /* NEEDS_atl_strstr */

#define _ATL_IMAGE_ABS(x) (((x)<0)?-(x):(x))

#ifdef NEEDS_atl_fltcmp
_ATL_C_GLOBALFCT int atl_fltcmp
#if !RTRT_KR
  (float x, float y)
#else
  (x, y)
  float x;
  float y;
#endif
{
  if (_ATL_IMAGE_ABS (x - y) < RTRT_PRECISION)
    return (0);
  else
    return (1);
}
#endif /* NEEDS_atl_fltcmp */

#ifdef NEEDS_atl_dblcmp
_ATL_C_GLOBALFCT int atl_dblcmp
#if !RTRT_KR
  (double x, double y)
#else
  (x, y)
  double x;
  double y;
#endif
{
  if (_ATL_IMAGE_ABS (x - y) < RTRT_PRECISION)
    return (0);
  else
    return (1);
}
#endif /* NEEDS_atl_dblcmp */

#ifdef NEEDS_atl_sprintf
_ATL_C_GLOBALFCT int atl_sprintf
#if !RTRT_KR
  (char *str, const char *format,...)
#else
  (str, format, va_alist)
  char *str;
  char *format;
  va_dcl
#endif
{
  int result;
  va_list   va;

#if !RTRT_KR
  va_start (va, format);
#else
  va_start (va);
#endif

  result = vsprintf (str, format, va);
  va_end (va);
  if (result == EOF)
  {
    ATL_ERROR ('F',"FATAL ERROR on sprintf Please contact your administrative.\n");
    ATL_ERROR ('A',format);
    priv_exit (-1);
  }
  return (result);
}
#endif /* NEEDS_atl_sprintf */
