

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: initexit.c,v 1.40.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Init & Exit Code (included by TP.c[pp]) $
*******************************************************************************/


#ifdef NEEDS_atl_out
#if ATL_MULTI_PROCESS
RTRT_STATICVAR char _atl_out_traces_file_name[ATL_TRACES_FILE_LEN+1+6];
/* +1 for the nul +6 for the PID and dot */
#endif /* ATL_MULTI_PROCESS */

/******************************************************************************** Routines
*******************************************************************************/

/*! Build a distinct name for the traces file, depending on getpid(). */
#if RTRT_IO == RTRT_NONE
#define _atl_out_init() (void)priv_init((char*)0)
#define atl_out_open()
#else
RTRT_STATICFCT char *atl_out_traces_file
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#if defined(RTRT_FILENAME_NOT_USED)
  return (char *)0;
#else
#if USE_ATU && defined(PTU_PROVIDE_RIO_NAME)
  char *filename;
  filename = atu_traces_file ();
  if(filename) return filename;
  return (char *) ATL_TRACES_FILE;
#else
#if ATL_MULTI_PROCESS
  char *p, *q, *d;

  _atl_current_pid = priv_getpid();
  for (p = (char *)ATL_TRACES_FILE,
       d = p + ATL_TRACES_FILE_LEN;
       d >= p && *d != (char)'.';
       d--);
  if (d < p)
  {
    for (q = _atl_out_traces_file_name;
         *p;
         p++, q++)
      *q = *p;
  }
  else
  {
    for (q = _atl_out_traces_file_name;
         p < d;
         p++, q++)
      *q = *p;
  }
  *q = '.'; q++;
  q += unsigned_image_16(_atl_current_pid, q);
  for (;
       *p;
       p++, q++)
  {
    *q = *p;
  }
  *q = (char)0;
  return _atl_out_traces_file_name;
#else /* ATL_MULTI_PROCESS */
  return (char *) ATL_TRACES_FILE;
#endif /* ATL_MULTI_PROCESS */
#endif /* USE_ATU */
#endif /* RTRT_FILENAME_NOT_USED defined in the TDP in user mode */

}

/*! Opens the traces output at init if possible
 *  This routine is to be called by atl_early_init only. */
RTRT_STATICFCT void _atl_out_init
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  atl_out = priv_init (atl_out_traces_file());
}

/*! Opens the traces output if not done at init yet
 *  This routine is called by atl_buffer_dump. */
RTRT_STATICFCT void atl_out_open
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  atl_out = priv_open (atl_out_traces_file());
  if (atl_out == (RTRT_FILE_TYPE)0)
  {
#if _ATL_ERROR_IMPLEMENTED
    ATL_ERROR ('O',"Cannot open traces file");
#endif
    priv_exit (-1);
    return;
  }
}
#endif /* RTRT_IO == RTRT_NONE */
#endif /* NEEDS_atl_out */

#ifdef _ATL_EXIT_PENDING

/*! Signals each component that the tested program is switching from
 *  buffered mode to safe mode. The execution is not far from end, but
 *  is likely to continue. Modules should not call it directly. */
RTRT_STATICFCT void atl_exit_pending
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#ifdef NEEDS_atl_initialized
  if ( atl_initialized != 2 )
  {
    return;
  }
  atl_initialized = 3;
#endif
#if USE_ATU
  atu_exit_pending ();
#elif USE_ATS
  ats_exit_pending ();
#elif USE_ATO
  ato_exit_pending ();
#endif
#if USE_ATK
  atk_exit_pending ();
#endif
#if USE_ATC
  atc_exit_pending ();
#endif
#if USE_ATT || USE_ATP || USE_ATQ
  atl_stack_exit_pending ();
#endif
#if USE_ATT
  att_exit_pending ();
#endif
#if USE_ATP
  atp_exit_pending ();
#endif
#if USE_ATQ
  atq_exit_pending ();
#endif
  /* mark the end of the test */
  ATL_LINE_ADD_OPCODE("tdp end");
  atl_line_dump();
}

#else /* _ATL_EXIT_PENDING */

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
extern "C" {
#endif

/*! Called once at the very end of the execution. This function will be not
 *  called if no atexit is provided with an observation tool enabled, or if a
 *  non-instrumented exit point is executed. */
RTRT_STATICFCT void atl_late_exit
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#ifdef NEEDS_atl_initialized
  if ( atl_initialized != 2 )
  {
    return;
  }
  atl_initialized = 3;
#endif
#if USE_ATU
  atu_late_exit ();
#elif USE_ATS
  ats_late_exit ();
#elif USE_ATO
  ato_late_exit ();
#endif
#if USE_ATK
  atk_late_exit ();
#endif
#if USE_ATC
  atc_late_exit ();
#endif
#if USE_ATT || USE_ATP || USE_ATQ
  atl_stack_clean_up ();
#endif
#if USE_ATQ
  atq_late_exit ();
#endif
#if USE_ATT
  att_late_exit ();
#endif
#if USE_ATP
  atp_late_exit ();
#endif
#if ATL_MULTI_THREADS && (USE_ATK || USE_ATT || USE_ATP || USE_ATQ)
  _atl_threads_late_exit ();
#endif
#if USE_ATT || USE_ATP || USE_ATQ
  atl_stack_late_exit ();
#endif
#ifdef RTRT_LATE_EXIT
  RTRT_LATE_EXIT
#endif
#if !USE_ATS
  /* mark the end of the test */
  ATL_LINE_ADD_OPCODE("tdp end");
  atl_line_dump();
#endif
#ifdef NEEDS_atl_buffer
  atl_buffer_terminate ();
#endif
#ifdef NEEDS_atl_initialized
  atl_initialized = 4;
#endif
#ifdef NEEDS_atl_out
/* when RTRT_IO=NONE brkPt are set to priv_close and priv_writeln */
/* atl_initialized can be checked to know which brkPt is reached */
#ifdef NEED_atl_stop
  atl_stop=1;
#endif  /* NEEDS_atl_stop */
#if RTRT_IO == RTRT_NONE
  priv_close (0);
#else
  if(atl_out)
  {
    priv_close (atl_out);
    atl_out = (RTRT_FILE_TYPE)0;
  }
#endif /* RTRT_IO == RTRT_NONE */
#endif /* NEEDS_atl_out */
}

#if RTRT_CPLUSPLUS_LANGUAGE && !RTRT_STATIC_AND_EXTERN_C_FORBIDDEN
} /* end of extern "C" */
#endif

#endif /* _ATL_EXIT_PENDING */

/*! Called only once at an exit point. */
_ATL_C_GLOBALFCT void atl_exit
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#ifdef _ATL_EXIT_PENDING
  if (!_atl_exit_pending)
  {
    atl_exit_pending ();
    _atl_exit_pending = 1;
#ifdef NEEDS_atl_buffer
    atl_buffer_dump ();
#endif
  }
#elif !defined(ATL_R_EXIT)
  atl_late_exit ();
#endif
}

/*! Early initialization. This routine opens the output and do early-init for
 *  every module. This routine is threads-safe : only the first call is
 * effective, further calls do nothing. */
RTRT_STATICFCT void atl_early_init
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  RTRT_MT_CRITICAL_DECL

#if ATL_MULTI_THREADS && USE_ATP && ( RTRT_LOWLEVEL_MEMLIB == RTRT_USR )
  if((atl_initialized == 1) && (RTRT_MT_THREAD_SELF == _atl_threads_current)) return;
#endif /* ATL_MULTI_THREADS && USE_ATP*/

  RTRT_MT_CRITICAL_LOCK
#ifdef NEEDS_atl_initialized
  if ((atl_initialized == 0) || (atl_initialized>4))
  {
    atl_initialized = 1;
#endif
/* re-organized for multi-threading _atl_out_init may require to use ATP (linux)*/
#if ATL_MULTI_THREADS && (USE_ATK || USE_ATT || USE_ATP || USE_ATQ)
    _atl_threads_init ();
#if USE_ATP && ( RTRT_LOWLEVEL_MEMLIB == RTRT_USR )
    _atl_threads_current = RTRT_MT_THREAD_SELF;
#endif /* USE_ATP */
#endif /* ATL_MULTI_THREADS &&  ... */
#ifdef NEEDS_atl_hexa_digits
    _atl_ini_digits();
#endif
#if (USE_ATT || USE_ATP || USE_ATQ)
    atl_stack_early_init ();
#endif
#ifdef NEEDS_atl_buffer
    atl_buffer_init ();
#endif
#if USE_ATP
    atp_early_init ();
#endif
#ifdef NEEDS_atl_out
    _atl_out_init ();
#endif
#ifdef ATL_R_EXIT
    ATL_R_EXIT (atl_late_exit);
#endif
#if (USE_ATO || USE_ATK || USE_ATT)
    _atl_tdf_init ();
#endif
#if USE_ATT
    att_early_init ();
#endif
#if USE_ATQ
    atq_early_init ();
#endif
#if USE_ATC
    atc_early_init ();
#endif
#ifdef _ATL_EXIT_PENDING
    _ATL_EXIT_PENDING = 0;
#endif
#ifdef NEEDS_clock_init
    clock_init ();
#endif

#if USE_ATU
    atu_early_init ();
#elif USE_ATS
    ats_early_init ();
#elif USE_ATO
    ato_early_init ();
#endif
#if USE_ATK
    atk_early_init ();
#endif
#if ATL_MULTI_THREADS && USE_ATP && ( RTRT_LOWLEVEL_MEMLIB == RTRT_USR )
    _atl_threads_current = 0;
#endif /* ATL_MULTI_THREADS && USE_ATP*/
#ifdef NEEDS_atl_initialized
    atl_initialized = 2;
#endif
#ifdef NEEDS_atl_initialized
  }
#endif
  RTRT_MT_CRITICAL_UNLOCK
}
