

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: initexit.h,v 1.32.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Init & Exit Code header (included by TP.c[pp]) $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef INITEXIT_H_INCLUDED
#define INITEXIT_H_INCLUDED

/*******************************************************************************
* Computed settings
*******************************************************************************/

#if USE_ATU || USE_ATO || USE_ATK || USE_ATT || USE_ATC || USE_ATP || USE_ATQ
/*! Needs atl_out variable */
#define NEEDS_atl_out
#endif

/* Needs I/O buffer */
#if USE_ATU || USE_ATO || USE_ATK || USE_ATT || USE_ATC || USE_ATP || USE_ATQ
#define NEEDS_atl_buffer
#endif

/* Needs _atl_initialized variable */
#if _ATL_OBSERVATION_TOOL
#define NEEDS_atl_initialized
#endif

/*! 1 if traces may be generated after exit or main return */
#if _ATL_OBSERVATION_TOOL && RTRT_CPLUSPLUS_LANGUAGE
#define _ATL_TRACE_MAY_OCCUR_AFTER_EXIT 1
#else
#define _ATL_TRACE_MAY_OCCUR_AFTER_EXIT 0
#endif

/*******************************************************************************
* Routines declarations
*******************************************************************************/

#ifdef RTRT_TDP_COMPILATION
RTRT_STATICFCT void atl_early_init
#if !RTRT_KR
  (void)
#else
  ()
#endif
  ;
#if RTRT_IO != RTRT_NONE
RTRT_STATICFCT void atl_out_open
#if !RTRT_KR
  (void)
#else
  ()
#endif
  ;
#endif /* RTRT_IO != RTRT_NONE */
#endif /* RTRT_TDP_COMPILATION */

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void atl_exit
#if !RTRT_KR
  (void)
#else
  ()
#endif
  ;

/*******************************************************************************
* Macros / variables
*******************************************************************************/

#ifdef RTRT_TDP_COMPILATION

#if RTRT_ATEXIT == RTRT_STD
#define ATL_R_EXIT(f) atexit(f)
#elif RTRT_ATEXIT == RTRT_USR
#define ATL_R_EXIT(f) usr_atexit(f)
#elif RTRT_ATEXIT == RTRT_NONE
/* ATL_R_EXIT(f) must be undefined if no implementation is provided */
#endif /* RTRT_ATEXIT */

#ifdef NEEDS_atl_initialized

/*! This variable is 0 if the TDP has not been initialized (i.e. atl_early_init
 *  has not been called yet),
 *  1 during the atl_early_init call,
 *  2 after atl_early_init returns,
 *  3 during the atl_late_exit call,
 *  4 after the atl_late_exit_call. */
RTRT_GLOBALVAR RTRT_U_INT8 atl_initialized
#if RTRT_VARIABLE_INIT_SUPPORTED
				           = 0 /* GLOBAL INITIALIZATION */
#endif
					      ;

/*! This macro returns 1 when TDP has already been intialized (i.e.
 *  _atl_early_init() has been called successfully), 0 otherwise. */
#if RTRT_VARIABLE_INIT_SUPPORTED && !ATL_MULTI_THREADS
#define _ATL_INITIALIZED (atl_initialized > 0)
#else /* RTRT_VARIABLE_INIT_SUPPORTED */
#define _ATL_INITIALIZED ((atl_initialized&(RTRT_U_INT8)0xFE)==2) /* ==2 || ==3 */
#endif /* RTRT_VARIABLE_INIT_SUPPORTED */
#endif /* NEEDS_atl_initialized */

#if ((!defined(ATL_R_EXIT)) && (!defined(RTRT_NEED_CLOSE_CALL))) && \
    (RTRT_UNCONTROLED_EXIT || _ATL_TRACE_MAY_OCCUR_AFTER_EXIT)
/*! When this variable has a non-null value, exit may occur at any time, without
 *  exit functions being called (so bufferization is forbidden). */
RTRT_STATICVAR RTRT_U_INT8 _atl_exit_pending;
/*! This macro is defined when variable _atl_exit_pending is defined. */
#define _ATL_EXIT_PENDING _atl_exit_pending
#endif

#endif /* RTRT_TDP_COMPILATION */

/*! This macro is placed by the C & C++ instrumentors just before:
 *   - calls to function listed in #pragma attol exit_instr (C language)
 *   - calls to one of these C++ special calls: exit, abort and terminate
 *   - return statements inside main() function
 *  This should insure that the atl_exit_pending mechanism will be called once.
 */
#ifndef _ATL_END_EXEC
#define _ATL_END_EXEC atl_exit()
#endif

#ifndef _ATL_END_EXEC_EXPR
#define _ATL_END_EXEC_EXPR (atl_exit(),0)
#endif

#endif
