

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: obstools.c,v 1.26.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Observation Tools common functions Source $
*******************************************************************************/


/*******************************************************************************
* Global variables
*******************************************************************************/

#if USE_ATU && USE_ATC
RTRT_GLOBALVAR int _atc_atu_num_last_op;
#endif /* USE_ATU && USE_ATC */

#if USE_ATP
RTRT_STATICVAR char * atl_stack_end;
#endif /* USE_ATP */


#if !RTRT_KR
_ATL_EXTERN_C void _atl_obstools_finished ( int );
#else
_ATL_EXTERN_C void _atl_obstools_finished ( );
#endif

/*******************************************************************************
*  Service: _atl_obstools_finished
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_finished
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
#ifdef ATL_OBSTOOLS_FINISHED
  ATL_OBSTOOLS_FINISHED
#endif
  if(reset & _ATCPQ_CLOSE)
  {
#if RTRT_IO == RTRT_NONE
	  priv_close(0);
#else
	  priv_close(atl_out);
	  atl_out=0;
#endif
  }
}

#if USE_ATC || USE_ATP || USE_ATQ || USE_ATT

#if USE_ATC
#if !RTRT_KR
_ATL_EXTERN_C void _atc_dump ( int );
#else
_ATL_EXTERN_C void _atc_dump ( );
#endif
#endif /* USE_ATC */

#if USE_ATP
#if !RTRT_KR
_ATL_EXTERN_C int _atp_dump ( int );
#else
_ATL_EXTERN_C int _atp_dump ( );
#endif
#endif /* USE_ATP */

#if USE_ATQ
#if !RTRT_KR
_ATL_EXTERN_C void _atq_dump ( int );
#else
_ATL_EXTERN_C void _atq_dump ( );
#endif
#endif /* USE_ATQ */

#ifdef ATL_T_FAMILY
#if USE_ATC
#if !RTRT_KR
_ATL_EXTERN_C void _atc_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atc_remove_family ( );
#endif
#endif /* USE_ATC */

#if USE_ATP
#if !RTRT_KR
_ATL_EXTERN_C void _atp_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atp_remove_family ( );
#endif
#endif /* USE_ATP */

#if USE_ATQ
#if !RTRT_KR
_ATL_EXTERN_C void _atq_remove_family ( ATL_T_FAMILY );
#else
_ATL_EXTERN_C void _atq_remove_family ( );
#endif
#endif /* USE_ATQ */

#endif /* ATL_T_FAMILY */

/*******************************************************************************
*  Service: _atl_obstools_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_dump
#if !RTRT_KR
( int reset )
#else
( reset )
  int reset;
#endif
{
#if USE_ATC || USE_ATP || USE_ATQ
  /* reset value of 0 is deprecated: 1 is assumed in any subsequent calls */
  if ( ! reset )
  {
    RTRT_MT_CRITICAL_VAR_LOCK(atl_line_mutex)
    /* Emit a warning to be read in the .rej file */
    ATL_ERROR ('d', "TestRT Obstools (Code Coverage, Memory and Performance Profiling) warning: while dumping, reset value of 0 is deprecated: 1 is assumed" );
	reset = _ATCPQ_ALL;
  }
  if(reset <= _ATCPQ_CLOSE ) reset |= _ATCPQ_ALL;
#endif /* USE_ATC || USE_ATP || USE_ATQ */
  atl_early_init(); /* init if it was not */
#if USE_ATC
  _atc_dump ( reset );
#endif /* USE_ATC */
#if USE_ATP
  atl_stack_end = (char *)&reset; /* memorise the top of the stack avoiding to check stack used by TP */
  (void)_atp_dump ( reset );
#endif /* USE_ATP */
#if USE_ATQ
  _atq_dump ( reset );
#endif /* USE_ATQ */
  if(reset & _ATCPQ_CLOSE )
  {
	  /* mark the end of the test */
	  ATL_LINE_ADD_OPCODE("tdp end");
	  atl_line_dump();
  }
  /* Dump the last lines remaining in the multi-line buffer */
  atl_buffer_dump ( );
  /* And say it's done */
  _atl_obstools_finished ( reset );
}


/*******************************************************************************
*  Service: _atl_remove_family remove data for a source Family
*******************************************************************************/
#ifdef ATL_T_FAMILY
_ATL_C_GLOBALFCT void _atl_remove_family
#if !RTRT_KR
(ATL_T_FAMILY family)
#else
(family)
ATL_T_FAMILY family;
#endif
{
#if USE_ATC
  _atc_remove_family ( family );
#endif /* USE_ATC */
#if USE_ATP
  _atp_remove_family ( family );
#endif /* USE_ATP */
#if USE_ATQ
  _atq_remove_family ( family );
#endif /* USE_ATQ */
#ifdef OTHER_REMOVE
OTHER_REMOVE( family );
#endif
}
#endif /* ATL_T_FAMILY */
#endif /* USE_ATC || USE_ATP || USE_ATQ || USE_ATT */

#if ! ( ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) )

/*******************************************************************************
*  Service: _atl_obstools_test_name
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_test_name
#if USE_ATU
#if !RTRT_KR
( int suffix )
#else
( suffix )
  int suffix;
#endif
#else /* USE_ATU */
#if !RTRT_KR
( void )
#else
( )
#endif
#endif /* USE_ATU */
{
#if USE_ATU

  if ( _atu_name_current_op != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( _atu_name_current_op );
    if ( _atu_name_current_test == (char *)0 )
    {
      atl_line_add_char ( RTRT_QUOT );
    }
  }

  if ( _atu_name_current_test != (char *)0 )
  {
    if ( _atu_name_current_op == (char *)0 )
    {
      atl_line_add_char ( ' ' );
      atl_line_add_char ( RTRT_QUOT );
    }
    else
    {
      atl_line_add_char ( '/' );
    }
    atl_buffer_add ( _atu_name_current_test );
    atl_line_add_char ( RTRT_QUOT );
  }

#if USE_ATC
  if ( suffix )
  {
    if ( _atu_num_current_test != -1 )
    {
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( _atu_num_current_test,
                                                 atl_line_cur ) );
    }

    if ( _atu_num_current_op != _atc_atu_num_last_op )
    {
      atl_line_add_char ( ' ' );
      atl_line_inc ( unsigned_reverse_image_16 ( _atu_num_current_op,
                                                 atl_line_cur ) );
      _atc_atu_num_last_op = _atu_num_current_op;
    }
  }
#endif

#elif USE_ATO

#if USE_ATC || USE_ATP || USE_ATQ
  if ( _ato_obstools_block_name != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( _ato_obstools_block_name );
    atl_line_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATC || USE_ATP || USE_ATQ */

#else /* Standalone */

#ifdef ATL_GETENV
  char *current_test_name;

  if ( ( current_test_name = ATL_GETENV ( "ATL_TEST_NAME" ) )
       != (char *)0 )
  {
    atl_line_add_char ( ' ' );
    atl_line_add_char ( RTRT_QUOT );
    atl_buffer_add ( current_test_name );
    atl_line_add_char ( RTRT_QUOT );
  }
#endif /* ATL_GETENV */

#endif /* USE_ATU */
}

#else /* ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) */

RTRT_GLOBALVAR char *_atl_obstools_str_cur;

/* Advised size for the user buffer: 128 */
#define _atl_obstools_str_init(str) \
  { _atl_obstools_str_cur = str; \
    *_atl_obstools_str_cur = (char)0; }

/*******************************************************************************
*  Service: _atl_obstools_str_add
*******************************************************************************/
RTRT_STATICFCT void _atl_obstools_str_add
#if !RTRT_KR
( const char *str )
#else
( str )
  char *str;
#endif
{
  while (*str)
  {
    *_atl_obstools_str_cur++ = *str++;
  }
  *_atl_obstools_str_cur = (char)0;
}

#define _atl_obstools_str_add_char(c) \
  ( ( *_atl_obstools_str_cur++ = (c) ), \
    ( *_atl_obstools_str_cur = (char)0 ) )

#define _atl_obstools_str_inc(len) \
  ( _atl_obstools_str_cur += (len) )

/*******************************************************************************
*  Service: _atl_obstools_test_name
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_obstools_test_name
#if USE_ATU
#if !RTRT_KR
( int suffix )
#else
( suffix )
  int suffix;
#endif
#else /* USE_ATU */
#if !RTRT_KR
( void )
#else
( )
#endif
#endif /* USE_ATU */
{
#if USE_ATU

  if ( _atu_name_current_op != (char *)0 )
  {
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_inc ( string_image ( _atu_name_current_op,
                                          _atl_obstools_str_cur,
                                          48 ) );
    if ( _atu_name_current_test == (char *)0 )
    {
      _atl_obstools_str_add_char ( RTRT_QUOT );
    }
  }

  if ( _atu_name_current_test != (char *)0 )
  {
    if ( _atu_name_current_op == (char *)0 )
    {
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_add_char ( RTRT_QUOT );
    }
    else
    {
      _atl_obstools_str_add_char ( '/' );
    }
    _atl_obstools_str_inc ( string_image ( _atu_name_current_test,
                                          _atl_obstools_str_cur,
                                          48 ) );
    _atl_obstools_str_add_char ( RTRT_QUOT );
  }

  if ( suffix )
  {
    if ( _atu_num_current_test != -1 )
    {
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_image_16 (
                               _atu_num_current_test,
                               atl_line_cur ) );
    }

    if ( _atu_num_current_op != _atc_atu_num_last_op )
    {
      _atl_obstools_str_add_char ( ' ' );
      _atl_obstools_str_inc ( unsigned_image_16 (
                               _atu_num_current_op,
                               atl_line_cur ) );
      _atc_atu_num_last_op = _atu_num_current_op;
    }
  }

#elif USE_ATO

#if USE_ATC || USE_ATP || USE_ATQ
  if ( _ato_obstools_block_name != (char *)0 )
  {
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_inc ( string_image ( _ato_obstools_block_name,
                                           _atl_obstools_str_cur,
                                           48 ) );
    _atl_obstools_str_add_char ( RTRT_QUOT );
  }
#endif /* USE_ATC || USE_ATP || USE_ATQ */

#else /* Standalone */

#ifdef ATL_GETENV
  char *current_test_name;

  if ( ( current_test_name = ATL_GETENV ( "ATL_TEST_NAME" ) )
       != (char *)0 )
  {
    _atl_obstools_str_add_char ( ' ' );
    _atl_obstools_str_add_char ( RTRT_QUOT );
    _atl_obstools_str_inc ( string_image ( current_test_name,
                                          _atl_obstools_str_cur,
                                          96 ) );
    _atl_obstools_str_add_char ( RTRT_QUOT );
  }
#endif /* ATL_GETENV */

#endif /* USE_ATU */
}

#endif /* ( USE_ATC && ATC_DUMP_DRIVER ) || ( USE_ATQ && ATQ_DUMP_DRIVER ) */

