

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: priv.c,v 1.38.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: I/O and Systems Calls (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Optimizations
*******************************************************************************/

#if USE_ATS
/*! Needs priv_get_hostid() */
#define NEEDS_priv_get_hostid
#endif


#if USE_ATU || USE_ATO || USE_ATK || USE_ATT || USE_ATC || USE_ATQ || USE_ATP
/*! Needs atl_out variable */
#define NEEDS_atl_out
#endif

#if _ATL_WIN32_API && defined(_DLL)
#define ATL_LIBC_PROTO_PREFIX __declspec(dllimport)
#else
#define ATL_LIBC_PROTO_PREFIX
#endif

/*******************************************************************************
* priv_... types and functions
*******************************************************************************/

#ifdef NEEDS_atl_out
/*! Variable handled by I/O routines */
RTRT_GLOBALVAR RTRT_FILE_TYPE atl_out;
#endif /* NEEDS_atl_out */

/*! Variable handled by I/O routines: compiler bug workaround */

#ifdef NEEDS_atl_stop
/* define to be set in the TDP to get var declaration */
/*! Variable handled by I/O routines: 1 at priv_close() and exitCode after priv_exit() */
RTRT_GLOBALVAR unsigned int atl_stop;
#endif

#if RTRT_ERROR == RTRT_STD
#include <stdio.h>
#include <errno.h>
RTRT_STATICFCT void priv_error
#if !RTRT_KR
  (const char *msg)
#else
  (msg)
  const char *msg;
#endif
{
  fprintf (stderr, "TestRT TDP: ");
  if (errno != 0)
  {
    perror (msg);
  }
  else
  {
    fprintf (stderr, "%s\n", msg);
  }
}
#define ATL_ERROR(c,s) priv_error(s)
#define _ATL_ERROR_IMPLEMENTED 1

#elif RTRT_ERROR == RTRT_USR

#define ATL_ERROR(c,s) usr_error(s)
#define _ATL_ERROR_IMPLEMENTED 1

#elif RTRT_ERROR == RTRT_NONE
/* in this case if the TDP defines ATL_ERROR(c,s) to nothing
   all messages are suppressed from the TP. to take less space
*/

#ifdef NEEDS_atl_buffer
#ifndef ATL_ERROR
#define ATL_ERROR(c,s) _atl_error_dump(c)
#endif /* ATL_ERROR */
#endif /* NEEDS_atl_buffer */

#ifndef ATL_ERROR
#define ATL_ERROR(c,s)
#endif /* ATL_ERROR */

#ifndef _ATL_ERROR_IMPLEMENTED
#define _ATL_ERROR_IMPLEMENTED 0
#endif

#endif /* RTRT_ERROR */

#if RTRT_EXIT == RTRT_STD
#include <stdlib.h>
#endif
/*! Exit with error code ec */
_ATL_C_GLOBALFCT void priv_exit
#if !RTRT_KR
  (RTRT_INT32 ec)
#else
  (ec)
  RTRT_INT32 ec;
#endif
{
#ifdef NEEDS_atl_stop
  atl_stop=ec;
#endif

#if RTRT_EXIT == RTRT_NONE
  ATL_ERROR('e',"exit function not implemented");
#endif
#ifdef NEEDS_atl_out
  if(atl_out)
  {
	  priv_close (atl_out);
	  atl_out=0;
  }
#endif
#if RTRT_EXIT == RTRT_STD
  exit (ec);
#elif RTRT_EXIT == RTRT_USR
  usr_exit (ec);
#endif
}

#ifndef RTRT_INIT_MODE
#define RTRT_INIT_MODE "w"
#endif
#ifndef RTRT_OPEN_MODE
#define RTRT_OPEN_MODE "a"
#endif
/*! Opens the I/O channel */
_ATL_C_GLOBALFCT RTRT_FILE_TYPE priv_init
#if !RTRT_KR
  (char * name)
#else
  (name)
  char * name;
#endif
{
#ifdef NEEDS_atl_stop
  atl_stop=0;
#endif

#if (RTRT_IO == RTRT_NONE) || defined(RTRT_OPEN_LATE)
  return((RTRT_FILE_TYPE)(0));
#elif RTRT_IO == RTRT_STD
  return((FILE *) fopen(name,RTRT_INIT_MODE));
#elif RTRT_IO == RTRT_USR
  return((RTRT_FILE_TYPE)(usr_init(name)));
#endif /* RTRT_IO */
}
/*! Opens the I/O channel before write \
 * if open not done at init or if closed by the TDP */
_ATL_C_GLOBALFCT RTRT_FILE_TYPE priv_open
#if !RTRT_KR
  (char * name)
#else
  (name)
  char * name;
#endif
{
#ifdef NEEDS_atl_stop
  atl_stop=0;
#endif

#if RTRT_IO == RTRT_STD
  return((FILE *) fopen(name,RTRT_OPEN_MODE));
#elif RTRT_IO == RTRT_USR
  return((RTRT_FILE_TYPE)(usr_open(name)));
#elif RTRT_IO == RTRT_NONE
  return((RTRT_FILE_TYPE)(1));
#endif /* RTRT_IO */
}

/*! Closes the I/O channel */
_ATL_C_GLOBALFCT void priv_close
#if !RTRT_KR
  (RTRT_FILE_TYPE file)
#else
  (file)
  RTRT_FILE_TYPE file;
#endif
{
#if RTRT_IO == RTRT_STD
  fclose(file);
#elif RTRT_IO == RTRT_USR
  usr_close((RTRT_FILE_TYPE)file);
#endif /* RTRT_IO */
}

/*! Flush the I/O channel */
#ifdef NEEDS_priv_flush
_ATL_C_GLOBALFCT void priv_flush
#if !RTRT_KR
  (RTRT_FILE_TYPE file)
#else
  (file)
  RTRT_FILE_TYPE file;
#endif
{
#if RTRT_IO == RTRT_STD
  fflush (file);
#endif
}
#endif /* NEEDS_priv_flush */

/*! Writes a line to the optimization buffer or directly to the I/O channel */
_ATL_C_GLOBALFCT void priv_writeln
#if !RTRT_KR
  (RTRT_FILE_TYPE file, char * str)
#else
  (file, str)
  RTRT_FILE_TYPE file;
  char * str;
#endif
{
#if RTRT_IO == RTRT_STD
  fprintf(file,"%s",str);
  fflush (file);
#elif RTRT_IO == RTRT_USR
  usr_writeln((RTRT_FILE_TYPE)file,str);
#endif /* RTRT_IO */
}

_ATL_C_GLOBALFCT void priv_check_failed
#if !RTRT_KR
  ()
#else
  (void)
#endif
{
  /* Set a breakpoint here */
}

/*******************************************************************************
* System functions
*******************************************************************************/

#if RTRT_GETENV == RTRT_STD
#define ATL_GETENV(e) getenv(e)
#elif RTRT_GETENV == RTRT_USR
#define ATL_GETENV(e) usr_getenv(e)
#elif RTRT_GETENV == RTRT_NONE
/* ATL_GETENV(e) must be undefined if no implementation is provided */
#endif

#ifdef NEEDS_priv_get_hostid
#if RTRT_GETHOSTNAME == RTRT_STD
#if _ATL_WIN32_API
#include <winsock.h>
#else
#include <unistd.h>
#endif
#endif
_ATL_C_GLOBALFCT void priv_get_hostid
#if !RTRT_KR
  (char *id)
#else
  (id)
  char *id;
#endif
{
#if RTRT_GETHOSTNAME == RTRT_STD
#if _ATL_WIN32_API
   WSADATA   WSAData;
   WSAStartup (MAKEWORD (1, 1), &WSAData);
   gethostname (id, 50);
   WSACleanup ();
#else
  gethostname (id, 50);
#endif /* _ATL_WIN32_API */
#elif RTRT_GETHOSTNAME == RTRT_USR
  usr_gethostname (id);
#elif RTRT_GETHOSTNAME == RTRT_NONE
  ATL_ERROR ('h',"warning: gethostname not implemented");
  *id = (char) 0;
#endif
}
#endif /* NEEDS_priv_get_hostid */

#ifdef NEEDS_priv_rdv
#if RTRT_RDV == RTRT_STD
#if _ATL_WIN32_API
#include <windows.h>
#include <winsock.h>
#include <io.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#ifdef _AIX
#include <strings.h>
#endif
#endif /* _ATL_WIN32_API */
#endif
_ATL_C_GLOBALFCT int priv_rdv
#if !RTRT_KR
  (char *token_name)
#else
  (token_name)
  char *token_name;
#endif
{
#if RTRT_RDV == RTRT_STD
  int       socket_rdv;
  char      buffer[80];
  struct sockaddr_in addr_socket;
#if _ATL_WIN32_API
  char      portnumber[80];
#else
  char     *portnumber;
#endif
#if _ATL_WIN32_API
  WSADATA   WSAData;
#endif

#if _ATL_WIN32_API
  if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
    return (-1);
  GetEnvironmentVariable ("ATS_PORT_RDV", (LPTSTR) portnumber, 80);
#else
  portnumber = ATL_GETENV ("ATS_PORT_RDV");
#endif

  if (portnumber == NULL)
    {
      ATL_ERROR('p',"getenv ATS_PORT_RDV");
      return (-1);
    }

  socket_rdv = socket (AF_INET, SOCK_STREAM, 0);
  if (socket_rdv < 0)
    {
      ATL_ERROR('s',"socket");
      return (-1);
    }

  memset ((void *) &addr_socket, 0, sizeof (struct sockaddr_in));

#if _ATL_WIN32_API
  addr_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
#else
  addr_socket.sin_addr.s_addr = INADDR_ANY;
#endif
  addr_socket.sin_family = AF_INET;
#if _ATL_WIN32_API
  addr_socket.sin_port = htons ((short)atoi (portnumber));
#else
  addr_socket.sin_port = htons (atoi (portnumber));
#endif

  if (connect (socket_rdv, (struct sockaddr *) &addr_socket,
               sizeof (addr_socket)) == -1)
    {
#if _ATL_WIN32_API
      int       ret = WSAGetLastError ();
      closesocket (socket_rdv);
      WSACleanup ();
#else
      close (socket_rdv);
#endif
      ATL_ERROR('c',"RDV connect");
      return (-1);
    }

  if ((send (socket_rdv, token_name, strlen (token_name), 0) == strlen (token_name))
      &&
      recv (socket_rdv, buffer, sizeof (buffer), 0) > 0)
    {
#if _ATL_WIN32_API
      closesocket (socket_rdv);
      WSACleanup ();
#else
      close (socket_rdv);
#endif
      if ((buffer[0] == 'O') && (buffer[1] == 'k'))
        {
          return (0);
        }
      else
        {
          return (-1);
        }
    }
  else
    {
      ATL_ERROR('w',"read write");
#if _ATL_WIN32_API
      closesocket (socket_rdv);
      WSACleanup ();
#else
      close (socket_rdv);
#endif
      return (-1);
    }
#elif RTRT_RDV == RTRT_USR
    return usr_rdv (token_name);
#elif RTRT_RDV == RTRT_NONE
    ATL_ERROR ('v',"warning: rendez-vous not implemented");
    return -1;
#endif /* RTRT_RDV */
}
#endif /* NEEDS_priv_rdv */


#ifdef NEEDS_tagtlk
#if RTRT_DYNAMIC_TRACKING == 1 && RTRT_RDV != RTRT_STD
#if _ATL_WIN32_API
#include <windows.h>
#include <winsock.h>
#include <io.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#ifdef _AIX
#include <strings.h>
#endif
#endif /* _ATL_WIN32_API */
#endif /* RTRT_DYNAMIC_TRACKING == 1 && RTRT_RDV != RTRT_STD */

_ATL_C_GLOBALFCT int priv_boot_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *tagtsocket,
   char *instance_name,
   char *testernaming,
   int *recved_tagt_timeout,
   int *recved_studio_link_avail)
#else
  (tagtsocket, instance_name, testernaming, recved_tagt_timeout, recved_studio_link_avail)
  ATL_TSTAGT_COMMTYPE *tagtsocket;
  char *instance_name;
  char *testernaming;
  int *recved_tagt_timeout;
  int *recved_studio_link_avail;
#endif
{
#if RTRT_DYNAMIC_TRACKING
  int       bootchk;
  struct sockaddr_in addr_socket;

#if _ATL_WIN32_API
  char      portnumber[80];
#else
  char     *portnumber;
#endif
#if _ATL_WIN32_API
  WSADATA   WSAData;
#endif

#if _ATL_WIN32_API
  if (WSAStartup (MAKEWORD (1, 1), &WSAData) != 0)
    return (-1);
  GetEnvironmentVariable ("ATS_PORT_TSTAGT", (LPTSTR) portnumber, 80);
#else
  portnumber = ATL_GETENV ("ATS_PORT_TSTAGT");
#endif

  if (portnumber == NULL)
    {
      ATL_ERROR('a',"getenv ATS_PORT_TSTAGT");
      return (-1);
    }

  *tagtsocket = socket (AF_INET, SOCK_STREAM, 0);
  if (*tagtsocket < 0)
    {
      ATL_ERROR('s',"socket");
      return (-1);
    }

  memset ((void *) &addr_socket, 0, sizeof (struct sockaddr_in));

#if _ATL_WIN32_API
  addr_socket.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
#else
  addr_socket.sin_addr.s_addr = INADDR_ANY;
#endif
  addr_socket.sin_family = AF_INET;
#if _ATL_WIN32_API
  addr_socket.sin_port = htons ((short)atoi (portnumber));
#else
  addr_socket.sin_port = htons (atoi (portnumber));
#endif

  if (connect (*tagtsocket, (struct sockaddr *) &addr_socket,
               sizeof (addr_socket)) == -1)
    {
#if _ATL_WIN32_API
      int       ret = WSAGetLastError ();
      closesocket (*tagtsocket);
      WSACleanup ();
#else
      close (*tagtsocket);
#endif
      ATL_ERROR('c',"TSTAGT connect");
      return (-1);
    }

  if ((bootchk = priv_send_chk (tagtsocket, testernaming)) == -1)
    return (-1);

  *recved_studio_link_avail = 1;
  *recved_tagt_timeout = ATS_INTERRECV_TIMEOUT;

  return (0);
#else /* RTRT_DYNAMIC_TRACKING */
    ATL_ERROR ('t',"warning: tester-agent link (RTRT_DYNAMIC_TRACKING) not implemented");
    return -1;
#endif /* RTRT_DYNAMIC_TRACKING */
}

_ATL_C_GLOBALFCT void priv_close_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *tagtsocket)
#else
  (tagtsocket)
  ATL_TSTAGT_COMMTYPE *tagtsocket;
#endif
{
#if RTRT_DYNAMIC_TRACKING
  if (tagtsocket)
    {
#if _ATL_WIN32_API
      closesocket (*tagtsocket);
      WSACleanup ();
#else
      close (*tagtsocket);
#endif
    }
#else /* RTRT_DYNAMIC_TRACKING */
    ATL_ERROR ('t',"warning: tester-agent link (RTRT_DYNAMIC_TRACKING) not implemented");
#endif /* RTRT_DYNAMIC_TRACKING */
}

_ATL_C_GLOBALFCT int priv_raiseline_tagtlk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *tagtsocket, char *rioline)
#else
  (tagtsocket, rioline)
  ATL_TSTAGT_COMMTYPE *tagtsocket;
  char *rioline;
#endif
{
#if RTRT_DYNAMIC_TRACKING
  if (tagtsocket && (*tagtsocket>0))
    {
      if (send (*tagtsocket, rioline, strlen (rioline), 0) == strlen (rioline))
	{
	  /* rio line sent to the agent ! */
	}
      else
	{
	  ATL_ERROR ('w',"write");
#if _ATL_WIN32_API
	  closesocket (*tagtsocket);
	  WSACleanup ();
#else
	  close (*tagtsocket);
#endif
	  return (-1);
	}
    }
  else
    {
      ATL_ERROR ('b',"bad socket");
      return (-1);
    }
  return 0;
#else /* RTRT_DYNAMIC_TRACKING */
    ATL_ERROR ('t',"warning: tester-agent link (RTRT_DYNAMIC_TRACKING) not implemented");
    return -1;
#endif /* RTRT_DYNAMIC_TRACKING */
}

_ATL_C_GLOBALFCT int priv_interrecv
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *tagtsocket,
   int  tagttimeout,
   char *recved_msg, int len, char **pt_tster, char **pt_info)
#else
  (tagtsocket, tagttimeout, recved_msg, len, pt_tster, pt_info)
  ATL_TSTAGT_COMMTYPE *tagtsocket;
  int  tagttimeout;
  char *recved_msg;
  int len;
  char **pt_tster;
  char **pt_info;
#endif
{
#if RTRT_DYNAMIC_TRACKING
  fd_set          readfd;
  struct timeval  timeout;
  char            *p_buffer;
  int             nfound;

  p_buffer = recved_msg;

  timeout.tv_sec = tagttimeout;
  timeout.tv_usec = 0;
  FD_ZERO (&readfd);
  FD_SET  (*tagtsocket, &readfd);

  /** check socket validity **/
  if (! (tagtsocket && (*tagtsocket>0)))
    {
      ATL_ERROR ('b',"bad socket");
      return (-1);
    }

  if ((nfound = select(FD_SETSIZE, &readfd, NULL, NULL, &timeout)) == -1)
    {
      ATL_ERROR('i',"interrecv select");
      return (-1);
    }

  if (nfound==0)
    {
      /** timeout has expired **/
      return 0;
    }

  /** something was received : let's read it **/
  if (recv (*tagtsocket, p_buffer, len * sizeof (char), 0) > 0)
    {
      char *p_end, *p_sep;

      p_end = strstr(p_buffer,"\r\n");

      if (p_end)
	{
	  p_sep = strstr(p_buffer,"\"\"\"");
	  p_sep[0] = '\0';

	  /** setting direct accessors for fields **/
	  *pt_info  = p_buffer;
	  *pt_tster = p_sep + 3;

	  p_end[0] = '\0';
	  return 0;
	}
      else
	{
	  return (-1);
	}
    }
  else
    {
      ATL_ERROR ('r',"socket recv");
#if _ATL_WIN32_API
      closesocket (*tagtsocket);
      WSACleanup ();
#else
      close (*tagtsocket);
#endif
      return (-1);
    }
  return 0;
#else /* RTRT_DYNAMIC_TRACKING */
    ATL_ERROR ('t',"warning: tester-agent link (RTRT_DYNAMIC_TRACKING) not implemented");
    return -1;
#endif /* RTRT_DYNAMIC_TRACKING */
}

_ATL_C_GLOBALFCT int priv_send_chk
#if !RTRT_KR
  (ATL_TSTAGT_COMMTYPE *tagtsocket, char *msg)
#else
  (tagtsocket, msg)
  ATL_TSTAGT_COMMTYPE *tagtsocket;
  char *msg;
#endif
{
#if RTRT_DYNAMIC_TRACKING
  char buffer[10];

  if ((send (*tagtsocket, msg, strlen(msg), 0) == strlen (msg)) &&
      (recv (*tagtsocket, buffer, sizeof (buffer), 0) > 0))
    {
      if ((buffer[0] == 'O') && (buffer[1] == 'k'))
	{
	  return (0);
	}
      else
	{
	  /* ATL_ERROR('z',"no ack recved from agent"); */
	  return (-1);
	}
    }
  else
    {
      ATL_ERROR('x',"send_with_checking");
#if _ATL_WIN32_API
      closesocket (*tagtsocket);
      WSACleanup ();
#else
      close (*tagtsocket);
#endif
      return (-1);
    }
#else /* RTRT_DYNAMIC_TRACKING */
    ATL_ERROR ('t',"warning: tester-agent link (RTRT_DYNAMIC_TRACKING) not implemented");
    return -1;
#endif /* RTRT_DYNAMIC_TRACKING */
}

#endif /* NEEDS_tagtlk */

#ifdef NEEDS_priv_sleep
#if RTRT_CLOCK == RTRT_STD
#if _ATL_WIN32_API
#include <winbase.h>
#else
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif /* _ATL_WIN32_API */
#endif /* RTRT_CLOCK == RTRT_STD */
_ATL_C_GLOBALFCT void priv_sleep
#if !RTRT_KR
  (RTRT_TIME_TYPE duration)
#else
  (duration)
  RTRT_TIME_TYPE duration;
#endif
{
#if RTRT_CLOCK == RTRT_STD
#if _ATL_WIN32_API
  Sleep (duration * RTRT_TIME_TYPE_STEP / 1000);
#else
  struct timeval delta;

  delta.tv_sec = (duration * RTRT_TIME_TYPE_STEP) / 1000000L;
  delta.tv_usec = (duration * RTRT_TIME_TYPE_STEP) % 1000000L;
  select (0, NULL, NULL, NULL, &delta);
#endif /* _ATL_WIN32_API */
#elif RTRT_CLOCK == RTRT_USR && RTRT_SLEEP == RTRT_USR
  usr_sleep (duration);
#elif RTRT_CLOCK == RTRT_NONE || RTRT_SLEEP == RTRT_NONE
  ATL_ERROR('o',"warning: sleep function not implemented");
#endif
}
#endif /* NEEDS_priv_sleep */


/*******************************************************************************
* Dynamic memory management
*******************************************************************************/

#if defined(NEEDS_priv_malloc) || defined(NEEDS_priv_free)
#if RTRT_HEAP == RTRT_STD
#include <stdlib.h>
#if _ATL_WIN32_API
#include <malloc.h>
#endif
#endif
#endif

#ifdef NEEDS_priv_malloc
_ATL_C_GLOBALFCT void *priv_malloc
#if !RTRT_KR
  (RTRT_SIZE_T size)
#else
  (size)
  RTRT_SIZE_T size;
#endif
{
#if RTRT_HEAP == RTRT_NONE
  ATL_ERROR('m',"malloc not implemented");
  return ((void*)0);
#else
void* m=(void*)0;
#if RTRT_HEAP == RTRT_STD
  m = ((void*) malloc (size));
#elif RTRT_HEAP == RTRT_USR
  m = ((void*) usr_malloc(size));
#else
  m = (void*)0;
#endif
  if( m == (void*)0)
  {
     ATL_ERROR('m',"No memory left");
     priv_exit (-1);
  }
  return (m);
#endif /*RTRT_HEAP == RTRT_NONE*/
}
#endif /* NEEDS_priv_malloc */

#ifdef NEEDS_priv_free
_ATL_C_GLOBALFCT void priv_free
#if !RTRT_KR
  (void *p)
#else
  (p)
  void *p;
#endif
{
#if RTRT_HEAP == RTRT_STD
  free (p);
#elif RTRT_HEAP == RTRT_USR
  usr_free (p);
#else
  ATL_ERROR('f',"free not implemented");
#endif
}
#endif /* NEEDS_priv_free */

#ifdef NEEDS_priv_realloc
_ATL_C_GLOBALFCT void *priv_realloc
#if !RTRT_KR
  (void *p,
   RTRT_SIZE_T new_size)
#else
  (p, new_size)
  RTRT_SIZE_T size;
#endif
{
#if RTRT_HEAP == RTRT_STD
  return ((void*) realloc (p, new_size));
#elif RTRT_HEAP == RTRT_USR
  /* Not supported in release 2002A. */
  ATL_ERROR('n',"realloc not user-definable in this release");
  return (void*)0;
#else
  ATL_ERROR('l',"realloc not implemented");
  return (void*)0;
#endif
}
#endif /* NEEDS_priv_malloc */


/*******************************************************************************
* File descriptor query
*******************************************************************************/
#if USE_ATP && ATP_REPORTS_FIU
#if RTRT_IO == RTRT_STD

#include <errno.h>
#include <sys/stat.h>

#ifndef ATL_STD_IO_MAX_FD
#define ATL_STD_IO_MAX_FD 40
#endif /* ATL_STD_IO_MAX_FD */

#ifndef ATL_FILENO
#ifndef fileno
/* Some compilers defines a macro */
_ATL_EXTERN_LIBC ATL_LIBC_PROTO_PREFIX int fileno ( FILE * ) RTRT_THROWING_EXCEPTION;
#endif
#define ATL_FILENO(f) fileno(f)
#endif /* ATL_FILENO */

RTRT_STATICFCT int priv_is_open
#if !RTRT_KR
  (int fd)
#else
  (fd)
  int fd;
#endif
{
  struct stat stat;

  errno = 0;
  fstat ( fd, &stat );
  if ( errno == EBADF )
  { /* File descriptor not used */
    return 0;
  }
  return 1;
}
#endif /* RTRT_IO == RTRT_STD */
#endif /* USE_ATP && ATP_REPORTS_FIU */

#ifdef NEEDS_priv_memcpy
#if RTRT_MEMCPY == RTRT_STD
#include <string.h>
#endif
/*! Copy a memory zone from s to d */
_ATL_C_GLOBALFCT void *priv_memcpy
#if !RTRT_KR
( void *d,
  void *s,
  unsigned int size)
#else
( d, s, size)
void *d;
void *s;
unsigned int size;
#endif
{
#if RTRT_MEMCPY == RTRT_STD
  return (memcpy (d, s, size));
#elif RTRT_MEMCPY == RTRT_USR
  return usr_memcpy (d, s, size);
#elif RTRT_MEMCPY == RTRT_NONE
  ATL_ERROR('g',"memcpy not implemented in this TDP");
  return (void*)0;
#endif
}
#endif /* NEEDS_priv_memcpy */

#ifdef NEEDS_priv_getpid
#if RTRT_GETPID == RTRT_STD
#ifdef _MSC_VER
#include <process.h>
#else /* _MSC_VER */
#include <sys/types.h>
#include <unistd.h>
#endif /* _MSC_VER */
#endif
/*! Get the running process id */
_ATL_C_GLOBALFCT RTRT_U_INT32 priv_getpid
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if RTRT_GETPID == RTRT_STD
#ifdef _MSC_VER
  return (RTRT_U_INT32)_getpid();
#else /* _MSC_VER */
  return (RTRT_U_INT32)getpid();
#endif /* _MSC_VER */
#elif RTRT_GETPID == RTRT_USR
  return usr_getpid();
#elif RTRT_GETPID == RTRT_NONE
  ATL_ERROR('k',"getpid not implemented in this TDP");
  return 0;
#endif
}
#endif /* NEEDS_priv_getpid */

/*******************************************************************************
* Var-args support
*******************************************************************************/

#if (USE_ATU && RTRT_STR_CONCAT_SUPPORTED) || USE_ATS
#if !RTRT_KR
#include <stdarg.h>
#else
#include <varargs.h>
#endif
#endif
