

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: rtbuffer.c,v 1.38.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Common buffer library (included by TP.c[pp]) $
*******************************************************************************/


/*******************************************************************************
* Static variables
*******************************************************************************/

/*! Buffer where lines are built */
RTRT_GLOBALVAR char atl_buffer[ATL_BUFFER_SIZE];

#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
/*! Pointer to the first useful character in buffer */
RTRT_STATICVAR char *atl_buffer_top;
#else
#define atl_buffer_top atl_buffer
#endif

/*! Pointer to the next character in buffer */
RTRT_STATICVAR char *atl_buffer_next;

/*! Pointer to the first character in buffer slack */
RTRT_STATICVAR char *atl_buffer_slack;

/*! Mutex variable controlling access to atl_buffer */
RTRT_MT_CRITICAL_VAR_DECL(atl_buffer_mutex)

#if ATL_MULTI_PROCESS
RTRT_STATICVAR RTRT_U_INT32 _atl_current_pid;
#endif /* ATL_MULTI_PROCESS */

/*! Macro to get remaining space in buffer */
#define _ATL_BUFFER_REMAINING (atl_buffer_slack-atl_buffer_next)
/*! Macro allowing newline char modification by the TDP */
#ifndef RTRT_NEWLINE_CHAR
#define RTRT_NEWLINE_CHAR '\n'
#endif /* RTRT_NEWLINE_CHAR */

/* normaly sizeof(RTRT_LINE_SPLIT_MARKER_END)+max line size */
#if (ATL_BUFFER_SIZE <256) || ((USE_ATO || USE_ATU) && RTRT_FLOAT)
#define ATL_CHECK_SLACK
#define _ATL_BUFFER_SLACK_SIZE 25
#else
#define _ATL_BUFFER_SLACK_SIZE 45
#endif

#if RTRT_IO == RTRT_NONE
/* do not break the execution for few lines */
#define  ATL_BUFFER_DUMP()
#else /* */
#define  ATL_BUFFER_DUMP() atl_buffer_dump()
#endif

RTRT_STATICFCT void atl_buffer_add
#if !RTRT_KR
  (const char *str)
#else
   ()
#endif
;
/*******************************************************************************
* Static functions (or macros)
*******************************************************************************/

/*! Dump the buffer into the output */
RTRT_STATICFCT void atl_buffer_dump
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
#ifdef RTRT_LINE_SPLIT_MARKER_END
  char *buffer_slack_sav;
#endif
  if (atl_buffer_next == atl_buffer_top)
  {
    return;
  }
#ifdef RTRT_LINE_SPLIT_MARKER_END
  buffer_slack_sav=atl_buffer_slack;
  atl_buffer_slack=atl_buffer + ATL_BUFFER_SIZE-1;
  if(atl_buffer_next < atl_buffer_slack)
  {
    atl_buffer_add(RTRT_LINE_SPLIT_MARKER_END);
  }
  atl_buffer_slack = buffer_slack_sav;
#endif
  *atl_buffer_next = '\0';
#ifdef NEEDS_atl_initialized
  if (atl_initialized > 3)
  {
#if _ATL_ERROR_IMPLEMENTED /* avoid reentrance */
    ATL_ERROR ('Z',"attempt to use Target Deployment Port resources after exit handler has been called (level 4)");
#endif
    atl_buffer_next = atl_buffer;
    return;
  }
#endif /* NEEDS_atl_initialized */
#if RTRT_IO == RTRT_NONE
  priv_writeln (0, 0);
#else /* RTRT_IO == RTRT_NONE */
  if (atl_out == (RTRT_FILE_TYPE)0
#if ATL_MULTI_PROCESS
     || _atl_current_pid != priv_getpid()
#endif /* ATL_MULTI_PROCESS */
     )
  {
    atl_out_open();
  }
  priv_writeln (atl_out, atl_buffer);
#endif /* RTRT_IO == RTRT_NONE */
  atl_buffer_next = atl_buffer;
#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
  atl_buffer_add(RTRT_LINE_SPLIT_MARKER_BEGIN);
  atl_buffer_top = atl_buffer_next;
#endif
}

/*! Append string str to the buffer */
/*! Append string str to the buffer */
RTRT_STATICFCT void atl_buffer_add
#if !RTRT_KR
  (const char *str)
#else
   (str)
  const char *str;
#endif
{
  while (*str)
  {
    *atl_buffer_next++ = *str++;
    if (atl_buffer_next >= atl_buffer_slack)
    {
      atl_buffer_dump ();
    }
  }
}

#ifdef ATL_CHECK_SLACK
#define atl_line_add(s) atl_buffer_add(s)
#else
RTRT_STATICFCT void atl_line_add
#if !RTRT_KR
  (const char *str)
#else
   (str)
  const char *str;
#endif
{
  while (*str)
  {
    *atl_buffer_next++ = *str++;
  }
}
#endif

#define atl_line_add_char(c) *atl_buffer_next++ = c
/*! Append character c to the buffer *
RTRT_STATICFCT void atl_buffer_add_char
#if !RTRT_KR
  (char c)
#else
  (c)
  char c;
#endif
{
  *atl_buffer_next++ = c;
  if (atl_buffer_next >= atl_buffer_slack)
  {
    atl_buffer_dump ();
  }
}
atl_buffer_add_char end*/

#ifndef ATL_CHECK_SLACK
#define atl_line_inc(l) atl_buffer_next += (l)
#else
#define atl_line_inc(l) atl_buffer_inc(l)

/*! Increment the buffer pointer of l characters */
RTRT_STATICFCT void atl_buffer_inc
#if !RTRT_KR
  (int l)
#else
  (l)
  int l;
#endif
{
  atl_buffer_next += l;
  if (atl_buffer_next >= atl_buffer_slack)
  {
    atl_buffer_dump ();
  }
}
#endif

/*! End current line of the buffer */
RTRT_STATICFCT void atl_buffer_endline
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  *atl_buffer_next++ = RTRT_NEWLINE_CHAR;
#ifdef _ATL_EXIT_PENDING
  if (_ATL_EXIT_PENDING)
  {
    atl_buffer_dump ();
  }
  else
#endif
  if (atl_buffer_next >= atl_buffer_slack)
  {
    atl_buffer_dump ();
  }
}

/*! Initialize the buffer */
RTRT_STATICFCT void atl_buffer_init
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  atl_buffer_next = atl_buffer;
  atl_buffer_slack = atl_buffer + ATL_BUFFER_SIZE - _ATL_BUFFER_SLACK_SIZE - 2;
  RTRT_MT_CRITICAL_VAR_INIT(atl_buffer_mutex)
#ifdef RTRT_LINE_SPLIT_MARKER_BEGIN
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  atl_buffer_add(RTRT_LINE_SPLIT_MARKER_BEGIN);
  atl_buffer_top = atl_buffer_next;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
#endif
#if ATL_MULTI_PROCESS
  _atl_current_pid = priv_getpid();
#endif /* ATL_MULTI_PROCESS */
}

/*! Dump the remaining buffer and destroy allocated resources. */
RTRT_STATICFCT void atl_buffer_terminate
#if !RTRT_KR
  (void)
#else
  ()
#endif
{
  if (atl_buffer_next > atl_buffer_top)
  {
    atl_buffer_dump ();
  }
  RTRT_MT_CRITICAL_VAR_DESTROY(atl_buffer_mutex)
}
#if USE_ATU
/*******************************************************************************
*  Service: _atl_add_error used by autostubing to generate error
*		the code is like "W2 " msg is the text
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_add_error
#if !RTRT_KR
( const char *errcode,const char *msg )
#else
( errcode , msg )
  const char *errcode;
  const char *msg;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  atl_line_add ( errcode );
  atl_line_add ( msg  );
  atl_buffer_endline(); /* Add a \n */
  atl_buffer_dump (); /* dump */
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
}
#endif
/*******************************************************************************
*  Service: _atl_error_dump
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_error_dump
#if !RTRT_KR
( const char icode )
#else
( error )
  const char icode;
#endif
{
  RTRT_MT_CRITICAL_VAR_LOCK(atl_buffer_mutex)
  ATL_LINE_ADD_OPCODE ( "WA " );
/*  atl_line_inc ( unsigned_reverse_image_16 ( icode, atl_buffer_next ) ); */
  atl_line_add_char(icode);
  atl_buffer_endline(); /* Add a \n */
  atl_buffer_dump (); /* dump */
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_buffer_mutex)
}


#define atl_line_dump atl_buffer_endline
#define atl_line_cur atl_buffer_next
#define atl_line_mutex atl_buffer_mutex
#define _ATL_LINE_REMAINING _ATL_BUFFER_REMAINING
