

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: rtsignal.c,v 1.12.4.1 2008/03/18 10:55:50 build Exp $
* $Descr: Standard POSIX Signals (included by TP.c[pp]) $
*******************************************************************************/

#if RTRT_SIGNAL == RTRT_STD

#if USE_ATP
#define NEEDS_atl_signal_string
#endif

#if USE_ATP
#define NEEDS_atl_signal_catch
#endif

#include <signal.h>
#define ATL_SIGNAL(s,f) signal(s,f)
#define ATL_RAISE(s) raise(s)

#ifndef ATL_STRSIGNAL
#if defined(__linux) || defined(__sun)
#ifdef _GNU_SOURCE
#include <string.h>
#else
#if !RTRT_KR
_ATL_EXTERN_C char *strsignal ( int );
#else
_ATL_EXTERN_C char *strsignal ( );
#endif
#endif /* _GNU_SOURCE */
#define ATL_STRSIGNAL(s) strsignal(s)
#endif /* __linux || __sun */
#endif /* ATL_STRSIGNAL */

#elif RTRT_SIGNAL == RTRT_USR

#define ATL_SIGNAL(s,f) usr_signal(s,f)
#define ATL_RAISE(s) /* not supported so far in user-defined mode */
#else
/* ATL_SIGNAL and ATL_RAISE must be undefined if no implementation is
   provided */

#endif /* RTRT_SIGNAL */

#ifdef NEEDS_atl_signal_string
_ATL_C_GLOBALFCT char *_atl_signal_string
#if !RTRT_KR
  (int signum)
#else
  (signum)
  int signum;
#endif
{
#ifdef ATL_STRSIGNAL
  return ATL_STRSIGNAL ( signum );
#elif defined(__hpux) || defined(_AIX)
  switch ( signum )
  {
    case SIGINT :
      return (char*) "Interrupt";
    case SIGQUIT :
      return (char*) "Quit";
    case SIGILL :
      return (char*) "Illegal instruction";
    case SIGABRT :
      return (char*) "ABORT instruction";
    case SIGFPE :
      return (char*) "Floating point exception";
    case SIGBUS :
      return (char*) "Bus error";
    case SIGSEGV :
      return (char*) "Segmentation fault";
    case SIGSYS :
      return (char*) "Bad system call";
    case SIGTERM :
      return (char*) "Terminated";
    default :
      return (char *)0;
  }
#elif _ATL_WIN32_API
  switch ( signum )
  {
    case SIGINT :
      return (char*) "Interrupt";
    case SIGILL :
      return (char*) "Illegal instruction";
    case SIGABRT :
      return (char*) "ABORT instruction";
    case SIGFPE :
      return (char*) "Floating point exception";
    case SIGSEGV :
      return (char*) "Segmentation fault";
#ifndef __GNUC__
    case SIGBREAK :
      return (char*) "Ctrl-Break sequence";
#endif
    case SIGTERM :
      return (char*) "Terminated";
    default :
      return (char *)0;
  }
#else
  return (char *)0;
#endif
}
#endif /* NEEDS_atl_signal_string */

#ifdef NEEDS_atl_signal_catch

#if defined(__linux) || defined(__sun)
#define ATL_SIGNALS_CATCHED_CARD 8
#elif defined(__hpux) || defined(__CYGWIN32__) || defined(_AIX)
#define ATL_SIGNALS_CATCHED_CARD 9
#elif _ATL_WIN32_API
#define ATL_SIGNALS_CATCHED_CARD 7
#else
#define ATL_SIGNALS_CATCHED_CARD 8
#endif

#if RTRT_CPLUSPLUS_LANGUAGE
extern "C" {
#endif

typedef struct {
        int        SigNum;
#if !RTRT_KR
        void     (*Handler) ( int );
#else
        void     (*Handler) ( );
#endif
} _AtlSignalDataType;

#if RTRT_CPLUSPLUS_LANGUAGE
} /* end of extern "C" */
#endif

_AtlSignalDataType _atl_signal_datas[ATL_SIGNALS_CATCHED_CARD];

_ATL_C_GLOBALFCT void _atl_signal_catch
#if !RTRT_KR
  ( void (*handler) ( int ) )
#else
  ( handler )
  void (*handler) ( );
#endif
{
  int index;

  index = 0;
  _atl_signal_datas[index].SigNum = SIGINT;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGINT, handler );
#if !_ATL_WIN32_API
  _atl_signal_datas[index].SigNum = SIGQUIT;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGQUIT, handler );
#endif
  _atl_signal_datas[index].SigNum = SIGILL;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGILL, handler );
  _atl_signal_datas[index].SigNum = SIGABRT;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGABRT, handler );
  _atl_signal_datas[index].SigNum = SIGFPE;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGFPE, handler );
#if !_ATL_WIN32_API
  _atl_signal_datas[index].SigNum = SIGBUS;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGBUS, handler );
#endif
  _atl_signal_datas[index].SigNum = SIGSEGV;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGSEGV, handler );
#if defined(__hpux) || defined(__CYGWIN32__)
  _atl_signal_datas[index].SigNum = SIGSYS;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGSYS, handler );
#elif _ATL_WIN32_API && !defined(__GNUC__)
  _atl_signal_datas[index].SigNum = SIGBREAK;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGBREAK, handler );
#endif
  _atl_signal_datas[index].SigNum = SIGTERM;
  _atl_signal_datas[index++].Handler = ATL_SIGNAL ( SIGTERM, handler );
}

#endif /* NEEDS_atl_signal_catch */

