

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: rtstack.h,v 1.32.4.1 2008/03/18 10:55:50 build Exp $
* $Descr: Call Stack Header (included across TP.h) $
*******************************************************************************/


/* Multiple inclusions guard: */
#ifndef ATL_STACK_H_INCLUDED
#define ATL_STACK_H_INCLUDED


#ifndef USE_ATT
#define _ATT_CHECK_CONST 0
#else
#define _ATT_CHECK_CONST (USE_ATT<<2)
#endif

#ifndef USE_ATP
#define _ATP_CHECK_CONST 0
#else
#define _ATP_CHECK_CONST (USE_ATP<<1)
#endif

#ifndef USE_ATQ
#define _ATQ_CHECK_CONST 0
#else
#define _ATQ_CHECK_CONST (USE_ATQ)
#endif

#ifdef ATL_T_FAMILY
#define _ATL_CONST_FAMILY   (1<<3)
#else
#define _ATL_CONST_FAMILY   0
#endif

#if ATL_MULTI_THREADS
#define _ATL_CONST_MULTI_THREADS (1<<4)
#else
#define _ATL_CONST_MULTI_THREADS 0
#endif

#define _ATL_STACK_CONST (_ATT_CHECK_CONST|_ATP_CHECK_CONST|_ATQ_CHECK_CONST|_ATL_CONST_FAMILY|_ATL_CONST_MULTI_THREADS)

/*******************************************************************************
*  Type to get accurate time (*must* be compatible with unsigned int)
*******************************************************************************/

#if USE_ATQ || ( USE_ATT && ATT_TIMESTAMP )
typedef RTRT_UNIT_TYPE _atl_T_time;
#endif

/*******************************************************************************
*  C/C++ Instrumentors
*******************************************************************************/

typedef struct {
#if USE_ATT
        RTRT_U_INT32              SourceNumber;
#define _ATL_STACK_INIT_SN(i) \
  _ATL_STACK_SOURCE(i).SourceNumber = (RTRT_U_INT32)0;
#define _ATL_STACK_SN_ZERO (RTRT_U_INT32)0,
#else
#define _ATL_STACK_INIT_SN(i)
#define _ATL_STACK_SN_ZERO
#endif
        RTRT_U_INT32              Key;
        RTRT_U_INT32              CheckSum;
#ifdef ATL_T_FAMILY
        ATL_T_FAMILY		      Family;
#endif
        RTRT_U_INT8               UseCheck;
} _AtlStackSource, *_AtlStackSourceP;

typedef struct {
        _AtlStackSourceP          Source;
        RTRT_U_INT32              MessageTsf;
#if USE_ATP
        RTRT_U_INT32              SourceLineOffset;
#define _ATL_STACK_INIT_ATP(s) s->Info.SourceLineOffset= (RTRT_U_INT32)0;
#else
#define _ATL_STACK_INIT_ATP(s)
#endif /* USE_ATP */
} _AtlStackInfo, *_AtlStackInfoP;

typedef struct {
        _AtlStackInfo             Info;
#ifdef ATL_T_FAMILY
        RTRT_U_INT32              Key;
        RTRT_U_INT32              CheckSum;
#endif
} _AtlCallStackInfo, *_AtlCallStackInfoP;

typedef struct _AtlStackLink_ {
#if ATL_CHECK_STACK
        RTRT_U_INT32              Magic1;
#endif /* ATL_CHECK_STACK */
        struct _AtlStackLink_    *Up;
        struct _AtlStackLink_    *Down;
        _AtlStackInfo             Info;
#if ATL_MULTI_THREADS
  		RTRT_U_INT32			  ThreadIndex;
#endif /* ATL_MULTI_THREADS */
#if USE_ATT
        RTRT_U_INT32              InstanceId;
#endif /* USE_ATT */
#if USE_ATQ
        void                     *AtqData;
        _atl_T_time               PeriodStart;
        _atl_T_time               LocalAmount;
        _atl_T_time               TotalAmount;
#define _ATL_STACK_INIT_ATQ(s) s->AtqData = (void *)0;
#else /* USE_ATQ */
#define _ATL_STACK_INIT_ATQ(s)
#endif /* USE_ATQ */
#if ATL_CHECK_STACK
        RTRT_U_INT32              Magic2;
#endif /* ATL_CHECK_STACK */
} _AtlStackLink, *_AtlStackLinkP;

#if ATL_CHECK_STACK
#define _ATL_STACK_SET_MAGIC(m) m = 0xdeadbeef;
#define _ATL_STACK_CHECK_MAGIC(s) ( s->Magic1 != 0xdeadbeef || s->Magic2 != 0xdeadbeef )
#else /* ATL_CHECK_STACK */
#define _ATL_STACK_SET_MAGIC(m)
#endif /* ATL_CHECK_STACK */

#define _ATL_STACK_DECL \
  _AtlStackLink _atl_stack_link;

#define _ATL_STACK_INIT(s,m) \
  _atl_stack_link.Info.Source = s; \
  _atl_stack_link.Info.MessageTsf = m;

/* i is source number */
#define _ATL_STACK_SOURCE(i) RTRT_CONCAT_MACRO(_atl_source_struct,i)
#define _ATL_STACK_SOURCE_ADR(i) &(_ATL_STACK_SOURCE(i))

#if RTRT_VARIABLE_INIT_SUPPORTED

#define _ATL_CALL_STACK_SOURCE_INIT(i)

#ifndef ATL_T_FAMILY
#define _ATL_STACK_FAMILY_INIT
#else
#ifndef ATL_FAMILY_INIT_VALUE
#define ATL_FAMILY_INIT_VALUE (ATL_T_FAMILY)0
#endif /*ATL_FAMILY_INIT_VALUE */
#define _ATL_STACK_FAMILY_INIT    ATL_FAMILY_INIT_VALUE,
#endif /* _ATL_T_FAMILY */

/* i is source number, k is key and c is checksum */
#define _ATL_STACK_SOURCE_DECLARE(i,k,c) \
static _AtlStackSource _ATL_STACK_SOURCE(i) = \
        { _ATL_STACK_SN_ZERO k, c, \
        _ATL_STACK_FAMILY_INIT \
        _ATL_STACK_CONST };

#else /* RTRT_VARIABLE_INIT_SUPPORTED */

#define _ATL_STACK_SOURCE_INIT(i) RTRT_CONCAT_MACRO(_att_struct_init,i)
#define _ATL_STACK_SOURCE_INIT_DONE(i) RTRT_CONCAT_MACRO(_att_struct_init_done,i)

#ifndef ATL_T_FAMILY
#define _ATL_STACK_FAMILY_INIT(i)
#else
#ifndef ATL_FAMILY_INIT_VALUE
#define ATL_FAMILY_INIT_VALUE (ATL_T_FAMILY)0
#endif /*ATL_FAMILY_INIT_VALUE */
#define _ATL_STACK_FAMILY_INIT(i)    _ATL_STACK_SOURCE(i).Family=ATL_FAMILY_INIT_VALUE;
#endif /* _ATL_T_FAMILY */

#define _ATL_STACK_SOURCE_DECLARE(i,k,c) \
static _AtlStackSource _ATL_STACK_SOURCE(i); \
static int _ATL_STACK_SOURCE_INIT_DONE(i) ATL_OBSTOOLS_INITIALIZE_TO_ZERO; \
static void _ATL_STACK_SOURCE_INIT(i) ( ) \
{ \
  if ( RTRT_IS_NOT_INITIALIZED(_ATL_STACK_SOURCE_INIT_DONE(i))) \
  { \
    _ATL_STACK_SOURCE_INIT_DONE(i) = RTRT_VARIABLE_INITIALIZED; \
    _ATL_STACK_INIT_SN(i) \
    _ATL_STACK_SOURCE(i).Key = k; \
    _ATL_STACK_SOURCE(i).CheckSum = c; \
    _ATL_STACK_FAMILY_INIT(i) \
    _ATL_STACK_SOURCE(i).UseCheck = _ATL_STACK_CONST; \
  } \
}
#define _ATL_CALL_STACK_SOURCE_INIT(i) _ATL_STACK_SOURCE_INIT(i)();

#endif /* RTRT_VARIABLE_INIT_SUPPORTED */


/*******************************************************************************
*  C Instrumentor
*******************************************************************************/
#if ! RTRT_CPLUSPLUS_LANGUAGE

#define _ATL_STACK_IN(i,m) \
  _ATL_STACK_DECL \
  _ATL_CALL_STACK_SOURCE_INIT(i) \
  _ATL_STACK_INIT(_ATL_STACK_SOURCE_ADR(i),m) \
  _atl_stack_link_push(&_atl_stack_link);

#define _ATL_STACK_OUT \
  _atl_stack_link_pop(&_atl_stack_link);

#define _ATL_STACK_OUT_EXPR_END(r) \
  +(_atl_stack_link_pop_return(&_atl_stack_link,r))

#define _ATL_STACK_OUT_VOID_EXPR_START(r) \
  _atl_stack_link_pop_return(&_atl_stack_link,r)

#define _ATL_STACK_OUT_EXPR_TYPE_START(t) \
  t _atl_return_value =
#define _ATL_STACK_OUT_EXPR_TYPE_END(r) \
  ; \
  _atl_stack_link_pop_return(&_atl_stack_link,r); \
  return _atl_return_value

#define _ATL_STACK_OUT_EXPR_FTYPE_START(t) \
  t (*_atl_return_value)() =
#define _ATL_STACK_OUT_EXPR_FTYPE_END(r) \
  ; \
  _atl_stack_link_pop_return(&_atl_stack_link,r); \
  return _atl_return_value

/* i is source number, k is key and c is checksum */
#define _ATL_STACK_DECLARE(i,k,c) \
_ATL_STACK_SOURCE_DECLARE(i,k,c)

#endif /* C Instrumentor */



/*******************************************************************************
*  C++ Instrumentor
*******************************************************************************/
#if RTRT_CPLUSPLUS_LANGUAGE
#if ATL_MULTI_THREADS
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int _atl_threads_get_index       ( );
#endif

class _att_StackLink {  public : _AtlStackLinkP StackLink;};

#if USE_ATT
class _att_Field_StackLink : public _att_StackLink {
	_AtlStackLink localStackLink;
	public:
		_att_Field_StackLink(unsigned long instanceId) {
			localStackLink.InstanceId = instanceId;
#if ATL_MULTI_THREADS
			localStackLink.ThreadIndex = _atl_threads_get_index();
#endif
			StackLink = &localStackLink;
		}
};
#endif

#ifndef ATL_SUPPRESS_UNUSED_WARNINGS
#define ATL_SUPPRESS_UNUSED_WARNINGS 1
#endif

#if ATL_SUPPRESS_UNUSED_WARNINGS
#define _ATL_ANOTHER_METHOD public : void _atl_avoid_unused_warning() { }
#define _ATL_ANOTHER_METHOD_CALL(o) o._atl_avoid_unused_warning();
#else
#define _ATL_ANOTHER_METHOD
#define _ATL_ANOTHER_METHOD_CALL(o)
#endif

#define _ATL_STACK_DEFINE_CLASS_IN(i) \
class _ATL_STACK_CLASS_IN(i) \
{ \
  public : \
    _ATL_STACK_DECL \
    _ATL_STACK_CLASS_IN(i) ( RTRT_U_INT32 message_tsf ) \
    { \
      _ATL_STACK_INIT(_ATL_STACK_SOURCE_ADR(i),message_tsf) \
      _atl_stack_link_push(&_atl_stack_link); \
    } \
    ~_ATL_STACK_CLASS_IN(i) ( ) \
    { \
      _atl_stack_link_pop(&_atl_stack_link); \
    } \
    _ATL_ANOTHER_METHOD \
};

#define _ATL_STACK_IN(i,m) \
  _ATL_CALL_STACK_SOURCE_INIT(i) \
  _ATL_STACK_CLASS_IN(i) _atl_stack_object(m); \
  _ATL_ANOTHER_METHOD_CALL(_atl_stack_object)

/* i is source number, k is key and c is checksum */
#define _ATL_STACK_DECLARE(i,k,c) \
_ATL_STACK_SOURCE_DECLARE(i,k,c) \
_ATL_STACK_DEFINE_CLASS_IN(i)

#endif /* C++ Instrumentor */



/*******************************************************************************
*  Function Prototypes
*******************************************************************************/
#if !RTRT_KR

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _atl_stack_link_push       ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _atl_stack_link_pop        ( _AtlStackLinkP );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _atl_stack_link_pop_return ( _AtlStackLinkP, int );
#ifdef _ATL_EXIT_PENDING
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void atl_stack_exit_pending     ( void );
#else /* _ATL_EXIT_PENDING */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void atl_stack_late_exit        ( void );
#endif /* _ATL_EXIT_PENDING */

#else /* RTRT_KR */

_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _atl_stack_link_push       ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void _atl_stack_link_pop        ( );
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int  _atl_stack_link_pop_result ( );
#ifdef _ATL_EXIT_PENDING
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void atl_stack_exit_pending     ( );
#else /* _ATL_EXIT_PENDING */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     void atl_stack_late_exit        ( );
#endif /* _ATL_EXIT_PENDING */

#endif /* RTRT_KR */


#endif /* ATL_STACK_H_INCLUDED */
