

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: tasking.h,v 1.32.4.1 2008/03/18 10:55:50 build Exp $
* $Descr: Synchronization/multi-task protection objects header $
*******************************************************************************/
/* Multiple inclusions guard: */
#ifndef TASKING_H_INCLUDED
#define TASKING_H_INCLUDED

#if USE_ATS
/*! Needs priv_thread_exit() */
#define NEEDS_priv_thread_exit
#endif

#if defined(RTRT_TDP_COMPILATION)
#define NEEDS_MT_CRITICAL_SECTION
#endif

#if defined(NEEDS_MT_REC_MUTEX) || defined(RTRT_TDP_COMPILATION)
#define NEEDS_MT_CRITICAL_VAR
#endif

#if USE_ATK || USE_ATT || USE_ATP || USE_ATQ || USE_ATC || USE_ATT
#define NEEDS_MT_THREAD_SELF
#endif

#if defined(RTRT_TDP_COMPILATION) && \
    (USE_ATK || USE_ATT || USE_ATP || USE_ATQ || USE_ATC || USE_ATT)
#define NEEDS_THREAD_SPECIFIC_DATA
#endif

#if ATL_MULTI_THREADS && (RTRT_THREADS != RTRT_NONE)

#if RTRT_THREADS == RTRT_STD /* POSIX, Win32, or OSF1 */

#ifdef _MSC_VER

/*******************************************************************************
* Win32 threads
*******************************************************************************/

#ifndef _REENTRANT
#define _REENTRANT
#endif
#if defined(RTRT_TDP_COMPILATION) || !defined( NO_SYS_INCLUDE )
#include <wtypes.h>
#include <winbase.h>
#endif /* NO_SYS_INCLUDE */

#ifdef NEEDS_MT_CRITICAL_SECTION
#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void atl_mutex_init
#if !RTRT_KR
  (CRITICAL_SECTION *var, int *init)
#else
  (var, init)
  CRITICAL_SECTION *var;
  int *init;
#endif
{
  if (*init == 0)
  {
    InitializeCriticalSection(var);
    *init = 1;
  }
}
#else /* RTRT_TDP_COMPILATION */
_ATL_EXTERN_C void atl_mutex_init
#if !RTRT_KR
  (CRITICAL_SECTION *var, int *init)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

/* These 3 macros are not required, but this implementation is slightly faster
   than the default one. */
#define RTRT_MT_CRITICAL_DECL       static CRITICAL_SECTION _atl_critical_var; \
                                    static int _atl_critical_init = 0; \
                                    atl_mutex_init ( &_atl_critical_var, \
                                                    &_atl_critical_init );
#define RTRT_MT_CRITICAL_LOCK       EnterCriticalSection(&_atl_critical_var);
#define RTRT_MT_CRITICAL_UNLOCK     LeaveCriticalSection(&_atl_critical_var);
#endif /* NEEDS_MT_CRITICAL_SECTION */

#ifdef NEEDS_MT_CRITICAL_VAR
#define RTRT_MT_CRITICAL_VAR_DECL(v)      CRITICAL_SECTION v;
#define RTRT_MT_CRITICAL_VAR_INIT(v)      InitializeCriticalSection(&v);
#define RTRT_MT_CRITICAL_VAR_LOCK(v)      EnterCriticalSection(&v);
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)    LeaveCriticalSection(&v);
#define RTRT_MT_CRITICAL_VAR_DESTROY(v)   DeleteCriticalSection(&v);
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 1
#endif /* NEEDS_MT_CRITICAL_VAR */

#ifdef NEEDS_MT_THREAD_SELF
#define RTRT_MT_THREAD_SELF_T            DWORD
#define RTRT_MT_THREAD_SELF              GetCurrentThreadId ()
#define RTRT_MT_THREAD_EQUAL(t1,t2)     ((t1)==(t2))
#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif
#endif /* NEEDS_MT_THREAD_SELF */

#ifdef NEEDS_THREAD_SPECIFIC_DATA
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE DWORD _atl_thread_data;
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE _atl_thread_data = TlsAlloc();
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE TlsFree(_atl_thread_data);
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v) \
				        v = (int) TlsGetValue(_atl_thread_data);
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v) \
					 TlsSetValue(_atl_thread_data,(void*)v);
#endif /* NEEDS_THREAD_SPECIFIC_DATA */

#define RTRT_MT_THREAD_EXIT(v)            ExitThread((DWORD)(v));

#else /* _MSC_VER */

#ifndef _REENTRANT
#define _REENTRANT
#endif
#define _MIT_POSIX_THREADS 1
#ifdef RTRT_TDP_COMPILATION
#include <pthread.h>
#endif

#ifdef PTHREAD_MUTEX_INITIALIZER

/*******************************************************************************
* POSIX threads
*******************************************************************************/

#ifdef NEEDS_MT_CRITICAL_SECTION
/* These 3 macros are not required, but this implementation is slightly faster
   than the default one. */
#define RTRT_MT_CRITICAL_DECL       static pthread_mutex_t _atl_critical_var \
                                     = PTHREAD_MUTEX_INITIALIZER;
#define RTRT_MT_CRITICAL_LOCK       pthread_mutex_lock(&_atl_critical_var);
#define RTRT_MT_CRITICAL_UNLOCK     pthread_mutex_unlock(&_atl_critical_var);
#endif /* NEEDS_MT_CRITICAL_SECTION */

#ifdef NEEDS_MT_CRITICAL_VAR
#define RTRT_MT_CRITICAL_VAR_DECL(v)    pthread_mutex_t v;
#define RTRT_MT_CRITICAL_VAR_INIT(v)    pthread_mutex_init(&v, NULL);
#if 1
#define RTRT_MT_CRITICAL_VAR_LOCK(v)    pthread_mutex_lock(&v);
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)  pthread_mutex_unlock(&v);
#else
#define RTRT_MT_CRITICAL_VAR_LOCK(v)    {fprintf(stderr,"Locking %lu on " #v " [" __FILE__ ":%lu]\n",(unsigned long)pthread_self(),__LINE__); pthread_mutex_lock(&v); fprintf(stderr,"Locked  %lu on " #v " [" __FILE__ ":%lu]\n",(unsigned long)pthread_self(),__LINE__);}
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)  {fprintf(stderr,"Unlocking %lu on " #v " [" __FILE__ ":%lu]\n",(unsigned long)pthread_self(),__LINE__); pthread_mutex_unlock(&v); fprintf(stderr,"Unlocked %lu on " #v " [" __FILE__ ":%lu]\n",(unsigned long)pthread_self(),__LINE__);}
#endif
#define RTRT_MT_CRITICAL_VAR_DESTROY(v) pthread_mutex_destroy(&v);
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 0
#endif /* NEEDS_MT_CRITICAL_VAR */

#ifdef NEEDS_MT_THREAD_SELF
#define RTRT_MT_THREAD_SELF_T        pthread_t
#define RTRT_MT_THREAD_SELF          pthread_self()
#define RTRT_MT_THREAD_EQUAL(t1,t2) pthread_equal(t1,t2)
#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif
#endif /* NEEDS_MT_THREAD_SELF */

#ifdef NEEDS_THREAD_SPECIFIC_DATA
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE pthread_key_t _atl_thread_data;
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE \
	       pthread_key_create(&_atl_thread_data,_atl_threads_release_index);
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE \
				           pthread_key_delete(_atl_thread_data);
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v) \
				v = RTRT_PTR_TO_INT( pthread_getspecific(_atl_thread_data) );
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v) \
				 pthread_setspecific(_atl_thread_data,(void*)v);
#endif /* NEEDS_THREAD_SPECIFIC_DATA */

#define RTRT_MT_THREAD_EXIT(v)            pthread_exit((void*)(&(v)));

#else /* PTHREAD_MUTEX_INITIALIZER */   /* DEC/OSF1 Threads */

/*******************************************************************************
* DEC/OSF1 Threads
*******************************************************************************/

#ifdef NEEDS_MT_CRITICAL_SECTION
#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void atl_mutex_init
#if !RTRT_KR
  (pthread_mutex_t *var, int *init)
#else
  (var, init)
  pthread_mutex_t *var;
  int *init;
#endif
{
  if ( *init == 0 )
  {
    pthread_mutex_init ( var, pthread_mutexattr_default );
    *init = 1;
  }
}
#else
_ATL_EXTERN_C void atl_mutex_init
#if !RTRT_KRKR
  (void *, int *)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

/* These 3 macros are not required, but this implementation is slightly faster
   than the default one. */
#define RTRT_MT_CRITICAL_DECL       static pthread_mutex_t _atl_critical_var; \
                                    static int _atl_critical_init = 0; \
                                    atl_mutex_init ( &_atl_critical_var, \
                                                    &_atl_critical_init );
#define RTRT_MT_CRITICAL_LOCK       pthread_mutex_lock(&_atl_critical_var);
#define RTRT_MT_CRITICAL_UNLOCK     pthread_mutex_unlock(&_atl_critical_var);
#endif /* NEEDS_MT_CRITICAL_SECTION */

#ifdef NEEDS_MT_CRITICAL_VAR
#define RTRT_MT_CRITICAL_VAR_DECL(v)     pthread_mutex_t v;
#define RTRT_MT_CRITICAL_VAR_INIT(v)     pthread_mutex_init(&v, \
                                                     pthread_mutexattr_default);
#define RTRT_MT_CRITICAL_VAR_LOCK(v)     pthread_mutex_lock(&v);
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)   pthread_mutex_unlock(&v);
#define RTRT_MT_CRITICAL_VAR_DESTROY(v)  pthread_mutex_destroy(&v);
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 0
#endif /* NEEDS_MT_CRITICAL_VAR */

#ifdef NEEDS_MT_THREAD_SELF
#define RTRT_MT_THREAD_SELF_T         pthread_t
#define RTRT_MT_THREAD_SELF           pthread_self()
#define RTRT_MT_THREAD_EQUAL(t1,t2)  pthread_equal(t1,t2)
#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif
#endif /* NEEDS_MT_THREAD_SELF */

#ifdef NEEDS_THREAD_SPECIFIC_DATA
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE pthread_key_t _atl_thread_data;
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE \
		pthread_keycreate(&_atl_thread_data,_atl_threads_release_index);
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE /* Nothing */
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v) \
			      pthread_getspecific(_atl_thread_data,&((void*)v));
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v) \
			         pthread_setspecific(_atl_thread_data,(void*)v);
#endif /* NEEDS_THREAD_SPECIFIC_DATA */

#define RTRT_MT_THREAD_EXIT(v)            pthread_exit((pthread_addr_t)(&(v)));

#endif /* PTHREAD_MUTEX_INITIALIZER */

#endif /* MSC_VER... */

#elif RTRT_THREADS == SOLARIS_THREADS

/*******************************************************************************
* Solaris threads
*******************************************************************************/

#ifndef _REENTRANT
#define _REENTRANT
#endif
#if defined(RTRT_TDP_COMPILATION) || !defined( NO_SYS_INCLUDE )
#include <thread.h>
#endif /* NO_SYS_INCLUDE */

#ifdef NEEDS_MT_CRITICAL_SECTION
/* These 3 macros are not required, but this implementation is slightly faster
   than the default one. */
#define RTRT_MT_CRITICAL_DECL       static mutex_t _atl_critical_var \
                                     = DEFAULTMUTEX;
#define RTRT_MT_CRITICAL_LOCK       mutex_lock(&_atl_critical_var);
#define RTRT_MT_CRITICAL_UNLOCK     mutex_unlock(&_atl_critical_var);
#endif /* NEEDS_MT_CRITICAL_SECTION */

#ifdef NEEDS_MT_CRITICAL_VAR
#define RTRT_MT_CRITICAL_VAR_DECL(v)    mutex_t v;
#define RTRT_MT_CRITICAL_VAR_INIT(v)    mutex_init(&v, USYNC_THREAD, NULL);
#define RTRT_MT_CRITICAL_VAR_LOCK(v)    mutex_lock(&v);
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)  mutex_unlock(&v);
#define RTRT_MT_CRITICAL_VAR_DESTROY(v) mutex_destroy(&v);
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 0
#endif /* NEEDS_MT_CRITICAL_VAR */

#ifdef NEEDS_MT_THREAD_SELF
#define RTRT_MT_THREAD_SELF_T         thread_t
#define RTRT_MT_THREAD_SELF           thr_self()
#define RTRT_MT_THREAD_EQUAL(t1,t2)  ((t1)==(t2))
#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif
#endif /* NEEDS_MT_THREAD_SELF */

#ifdef NEEDS_THREAD_SPECIFIC_DATA
extern int thr_keycreate(thread_key_t *, void(*)(void *));
extern int thr_setspecific(thread_key_t, void *);
extern int thr_getspecific(thread_key_t, void **);
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE thread_key_t _atl_thread_data;
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE \
		    thr_keycreate(&_atl_thread_data,_atl_threads_release_index);
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE /* Nothing ! */
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v) \
				  thr_getspecific(_atl_thread_data,&((void*)v));
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v) \
				     thr_setspecific(_atl_thread_data,(void*)v);
#endif

#define RTRT_MT_THREAD_EXIT(v)            thr_exit((void*)(&(v)));

#endif /* RTRT_THREADS == ... */

#if RTRT_THREADS != RTRT_NONE

/*******************************************************************************
* General routines
*******************************************************************************/

#ifdef NEEDS_MT_REC_MUTEX
#if RTRT_MT_CRITICAL_VAR_RECURSIVE
/* The system mutex is already recursive, so recursive mutex routines are
 * identical to simple mutex routines. */

#define RTRT_MT_REC_MUTEX_DECL(v) RTRT_MT_CRITICAL_VAR_DECL(v)
#define RTRT_MT_REC_MUTEX_INIT(v) RTRT_MT_CRITICAL_VAR_INIT(v)
#define RTRT_MT_REC_MUTEX_LOCK(v) RTRT_MT_CRITICAL_VAR_LOCK(v)
#define RTRT_MT_REC_MUTEX_UNLOCK(v) RTRT_MT_CRITICAL_VAR_UNLOCK(v)
#define RTRT_MT_REC_MUTEX_DESTROY(v) RTRT_MT_CRITICAL_VAR_DESTROY(v)

#elif !defined(RTRT_MT_REC_MUTEX_DECL)
/* No system recursive mutex is provided: this section provides one, based on
 * non-recursive mutex. */

struct _atl_rec_mutex_t
#ifdef RTRT_TDP_COMPILATION
{
  RTRT_MT_THREAD_SELF_T owner;
  int counter;
  RTRT_MT_CRITICAL_VAR_DECL(mutex)
}
#endif
;

#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void _atl_rec_mutex_init
#if !RTRT_KR
  (struct _atl_rec_mutex_t **m)
#else
  (m)
  struct _atl_rec_mutex_t **m;
#endif
{
  *m = (_atl_rec_mutex_t*) priv_malloc(sizeof(_atl_rec_mutex_t));
  (*m)->owner = (RTRT_MT_THREAD_SELF_T) 0;
  (*m)->counter = 0;
  RTRT_MT_CRITICAL_VAR_INIT((*m)->mutex)
}
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_rec_mutex_init
#if !RTRT_KR
  (struct _atl_rec_mutex_t **)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void _atl_rec_mutex_lock
#if !RTRT_KR
  (struct _atl_rec_mutex_t *m)
#else
  (m)
  struct _atl_rec_mutex_t *m;
#endif
{
  if (RTRT_MT_THREAD_EQUAL(RTRT_MT_THREAD_SELF, m->owner))
  {
    m->counter++;
  }
  else
  {
    RTRT_MT_CRITICAL_VAR_LOCK(m->mutex)
    m->owner = RTRT_MT_THREAD_SELF;
    m->counter = 1;
  }
}
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_rec_mutex_lock
#if !RTRT_KR
  (struct _atl_rec_mutex_t *)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void _atl_rec_mutex_unlock
#if !RTRT_KR
  (struct _atl_rec_mutex_t *m)
#else
  (m)
  struct _atl_rec_mutex_t *m;
#endif
{
  m->counter--;
  if (m->counter == 0)
  {
    m->owner = (RTRT_MT_THREAD_SELF_T) 0;
    RTRT_MT_CRITICAL_VAR_UNLOCK(m->mutex)
  }
}
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_rec_mutex_unlock
#if !RTRT_KR
  (struct _atl_rec_mutex_t *)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void _atl_rec_mutex_destroy
#if !RTRT_KR
  (struct _atl_rec_mutex_t *m)
#else
  (m)
  struct _atl_rec_mutex_t *m;
#endif
{
  RTRT_MT_CRITICAL_VAR_DESTROY(m->mutex)
  priv_free(m);
}
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_rec_mutex_destroy
#if !RTRT_KR
  (struct _atl_rec_mutex_t *)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */

#define RTRT_MT_REC_MUTEX_DECL(v)    _atl_rec_mutex_t *v;
#define RTRT_MT_REC_MUTEX_INIT(v)    _atl_rec_mutex_init (&v);
#define RTRT_MT_REC_MUTEX_LOCK(v)    _atl_rec_mutex_lock(v);
#define RTRT_MT_REC_MUTEX_UNLOCK(v)  _atl_rec_mutex_unlock(v);
#define RTRT_MT_REC_MUTEX_DESTROY(v) _atl_rec_mutex_destroy(v);

#endif /* RTRT_MT_CRITICAL_VAR_RECURSIVE */
#endif /* NEEDS_MT_REC_MUTEX */

#ifdef NEEDS_MT_CRITICAL_SECTION
#ifndef RTRT_MT_CRITICAL_DECL
/* The customization does not provide these macros, so implement them based
   on customization for critical sections. */

#define RTRT_MT_CRITICAL_DECL \
   static RTRT_MT_CRITICAL_VAR_DECL(_atl_critical_var) \
   static int _atl_critical_init = 0; \
   if (!_atl_critical_init) { \
      _atl_critical_init = 1; \
      RTRT_MT_CRITICAL_VAR_INIT(_atl_critical_var) \
   }
#define RTRT_MT_CRITICAL_LOCK RTRT_MT_CRITICAL_VAR_LOCK(_atl_critical_var)
#define RTRT_MT_CRITICAL_UNLOCK RTRT_MT_CRITICAL_VAR_UNLOCK(_atl_critical_var)

#endif /* RTRT_MT_CRITICAL_DECL */
#endif /* NEEDS_MT_CRITICAL_SECTION */

#endif /* RTRT_THREADS != RTRT_NONE */

#else /* ATL_MULTI_THREADS && RTRT_THREADS != RTRT_NONE */

#ifdef NEEDS_MT_CRITICAL_SECTION
/*! Declaration and initialization of a critical section function-static
    variable */
#define RTRT_MT_CRITICAL_DECL
/*! Start of a function-static critical section */
#define RTRT_MT_CRITICAL_LOCK
/*! End of a function-static critical section */
#define RTRT_MT_CRITICAL_UNLOCK
#endif /* NEEDS_MT_CRITICAL_SECTION */

#ifdef NEEDS_MT_CRITICAL_VAR
/*! Declaration of a critical section variable */
#define RTRT_MT_CRITICAL_VAR_DECL(v)
/*! Initialization of a critical section variable */
#define RTRT_MT_CRITICAL_VAR_INIT(v)
/*! Start of a critical section v */
#define RTRT_MT_CRITICAL_VAR_LOCK(v)
/*! End of a critical section v */
#define RTRT_MT_CRITICAL_VAR_UNLOCK(v)
/*! Destroy critical section v */
#define RTRT_MT_CRITICAL_VAR_DESTROY(v)
/*! Define to 1 if critical section is recursive (i.e. a thread can enter
 *  a critical section more than once without deadblocking). */
#define RTRT_MT_CRITICAL_VAR_RECURSIVE 0
#endif /* NEEDS_MT_CRITICAL_VAR */

#ifdef NEEDS_MT_THREAD_SELF
/*! Type to identify a thread */
#define RTRT_MT_THREAD_SELF_T
/*! Function to get the current thread id */
#define RTRT_MT_THREAD_SELF
#ifndef RTRT_MT_THREAD_ISALIVE
/*! says if the thread id is alive or not (=0). */
#define RTRT_MT_THREAD_ISALIVE(id) (id)
#endif
#endif /* NEEDS_MT_THREAD_SELF */

#ifdef NEEDS_THREAD_SPECIFIC_DATA
/*! Type for thread specific data keys */
#define RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE
/*! Key creation */
#define RTRT_MT_THREAD_SPECIFIC_DATA_CREATE
/*! Key deletion */
#define RTRT_MT_THREAD_SPECIFIC_DATA_DELETE
/*! Getting thread specific data */
#define RTRT_MT_THREAD_SPECIFIC_DATA_GET(v)
/*! Setting thread specific data */
#define RTRT_MT_THREAD_SPECIFIC_DATA_SET(v)
/*! Exits the current thread with v as status value. In some configurations,
 *  address of v must be taken, and v must not be a local variable. */
#endif /* NEEDS_THREAD_SPECIFIC_DATA */

#define RTRT_MT_THREAD_EXIT(v)

#ifdef NEEDS_MT_REC_MUTEX
/* These macros are optional. If they are not provided, a default runtime
 * implementation will be used. But if one of them is defined, they must
 * all be defined. */
/*! Declaration of a recursive mutex */
#define RTRT_MT_REC_MUTEX_DECL(v)
/*! Initialization of a recursive mutex */
#define RTRT_MT_REC_MUTEX_INIT(v)
/*! Lock of a recursive mutex */
#define RTRT_MT_REC_MUTEX_LOCK(v)
/*! Unlock of a recursive mutex */
#define RTRT_MT_REC_MUTEX_UNLOCK(v)
/*! Destroy a recursive mutex */
#define RTRT_MT_REC_MUTEX_DESTROY(v)
#endif /* NEEDS_MT_REC_MUTEX */

#endif /* ATL_MULTI_THREADS && RTRT_THREADS != RTRT_NONE */

#ifdef NEEDS_priv_thread_exit
#ifdef RTRT_TDP_COMPILATION
_ATL_C_GLOBALFCT void priv_thread_exit
#if !RTRT_KR
  (int flag)
#else
  (flag)
  int flag;
#endif
{
  RTRT_MT_THREAD_EXIT(flag);
}
#else
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void priv_thread_exit
#if !RTRT_KR
  (int)
#else
  ()
#endif
  ;
#endif /* RTRT_TDP_COMPILATION */
#endif /* NEEDS_priv_thread_exit */

#ifndef RTRT_TDP_COMPILATION
#if USE_ATQ || USE_ATP || USE_ATT
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX void _atl_threads_dying
#if !RTRT_KR
  (void)
#else
  ()
#endif
  ;
#else /* USE_ATQ || USE_ATP || USE_ATT */
#define _atl_threads_dying() 0
#endif /* USE_ATQ || USE_ATP || USE_ATT */
#endif /* #ifndef RTRT_TDP_COMPILATION */

#endif
