

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: threads.c,v 1.24.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: Threads Management (included by TP.c[pp]) $
*******************************************************************************/


#if ATL_MULTI_THREADS
/* proto declared in the rtstack.h included later in TP.c[pp] */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     int _atl_threads_get_index       ( );

enum {
        _ATL_THREADS_CREATED,
        _ATL_THREADS_CONTINUE,
        _ATL_THREADS_SCHEDULED
};

#define THREADINDEX() int thread_index;

/*******************************************************************************
* Global variables
*******************************************************************************/

RTRT_GLOBALVAR RTRT_MT_THREAD_SELF_T _atl_threads_table_used[ATL_THREADS_MAX+1];

RTRT_GLOBALVAR RTRT_MT_THREAD_SELF_T _atl_threads_current;
RTRT_GLOBALVAR RTRT_U_INT32 _atl_threads_index;
RTRT_GLOBALVAR RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE

RTRT_MT_CRITICAL_VAR_DECL(atl_index_critical)

#if USE_ATP || USE_ATQ || USE_ATT
RTRT_STATICFCT void _atl_stack_finish ( int );
#endif /* USE_ATP || USE_ATQ || USE_ATT */

/*******************************************************************************
*  Service: _atl_threads_release_index
*  Goal: called automatically by the OS when a thread terminates
*         (if implemented)
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_release_index
#if !RTRT_KR
( void *thedata )
#else
( thedata )
  void *thedata;
#endif
{
 int i;
 i=(int)RTRT_PTR_TO_INT(thedata);
#if USE_ATP || USE_ATQ || USE_ATT
  if ( i != 0 )
  {
    _atl_stack_finish (i);
  }
#endif /* USE_ATP || USE_ATQ || USE_ATT */
  RTRT_MT_THREAD_SPECIFIC_DATA_SET ( 0 );
  _atl_threads_table_used[i] = 0;
}

#if USE_ATP || USE_ATQ || USE_ATT
/*******************************************************************************
*  Service: _atl_threads_dying
*  Goal: called automatically without parameter by the OS when a thread terminates
*         (if implemented)
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_dying
#if !RTRT_KR
( void )
#else
( )
#endif
{
	int thedata;
	RTRT_MT_THREAD_SPECIFIC_DATA_GET( thedata );
	_atl_threads_release_index((void*)thedata);
}
/*******************************************************************************
*  Service: _atl_threads_kill
*  Goal: called with self as parameter when a thread is killed
*         (if implemented)
*******************************************************************************/
#ifdef NEEDS_threads_kill
_ATL_C_GLOBALFCT void _atl_threads_kill
#if !RTRT_KR
( RTRT_MT_THREAD_SELF_T self )
#else
( self )
 RTRT_MT_THREAD_SELF_T self;
#endif
{
	/* find the index of this thread */
        int thedata;
	for(thedata=0;
            thedata <= ATL_THREADS_MAX && _atl_threads_table_used[thedata] != self;
            thedata++ );
        if ( thedata > ATL_THREADS_MAX )
        {
           ATL_ERROR ('K', "try to kill unknown thread" );
        }
        else
        {
            _atl_stack_finish (thedata );
	    _atl_threads_table_used[thedata] = 0;
        }
}
#endif /* NEEDS_threads_kill */
#endif /* USE_ATP || USE_ATQ || USE_ATT */

/*******************************************************************************
*  Service: _atl_threads_get_index
*******************************************************************************/
_ATL_C_GLOBALFCT int _atl_threads_get_index
#if !RTRT_KR
( void )
#else
( )
#endif
{
  RTRT_MT_THREAD_SELF_T self,cur;
  int thedata;

  self = RTRT_MT_THREAD_SELF;
  RTRT_MT_CRITICAL_VAR_LOCK(atl_index_critical)
  thedata= (int)_atl_threads_index;
  cur =_atl_threads_current;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
  if ( self != cur )
  {
	  RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );

	  if ( _atl_threads_table_used[thedata] != self )
	  {
			ATL_ERROR('M',"_atl_threads_get_index Unknow thread");
	  }
  }
  return thedata;
}
/*******************************************************************************
*  Service: _atl_threads_get_status
*    returns _ATL_THREADS_CREATED if thread is new,
*            _ATL_THREADS_SCHEDULED if it has just been scheduled,
*            _ATL_THREADS_CONTINUE else
*******************************************************************************/
_ATL_C_GLOBALFCT int _atl_threads_get_status
#if !RTRT_KR
( int *current )
#else
( current)
int *current;
#endif
{
  RTRT_MT_THREAD_SELF_T self;
  RTRT_U_INT32 i;
  int thedata;

  if(*current)
  {
	  if(*current==_atl_threads_index)
	  {
		  return _ATL_THREADS_CONTINUE;
	  }
	  thedata=*current;
  }

  self = RTRT_MT_THREAD_SELF;
  RTRT_MT_CRITICAL_VAR_LOCK(atl_index_critical)

  if ( self == (RTRT_MT_THREAD_SELF_T)0 )
  { /* This could be the main process: first item of the table */
    _atl_threads_current = 0;
    _atl_threads_index = 0;
    *current=0;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_CONTINUE;
  }
  if ( self == _atl_threads_current )
  { /* No scheduling has been done between threads */
    *current=_atl_threads_index;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_CONTINUE;
  }

  if(*current==0)
  {
  	RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );
  }
  _atl_threads_index = thedata;
  _atl_threads_current = self;

  if ( _atl_threads_table_used[thedata] == self )
  { /* This thread is woken up */
    *current=thedata;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_SCHEDULED;
  }

  /* This thread is just created: let's give him a data space */
  for ( i = 1;
        i <= ATL_THREADS_MAX && _atl_threads_table_used[i];
        i++ );
  if ( i > ATL_THREADS_MAX )
  { /* Error : no space enough in table ...
     * let's use last one, just can't stop */
    ATL_ERROR ('O', "Maximum number of threads reached" );
    i = ATL_THREADS_MAX;
  }
  _atl_threads_index = i;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
  thedata = i;
  _atl_threads_table_used[i] = self;
  RTRT_MT_THREAD_SPECIFIC_DATA_SET ( thedata )
  if(*current)
  {
	  ATL_ERROR('X',"_atl_threads_get_status CREATION WARNING");
  }
  *current=thedata;
  return _ATL_THREADS_CREATED;
}

/*******************************************************************************
*  Service: _atl_threads_init
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! RTRT_RAM_CLEAN

  RTRT_U_INT32 i;

  /* global variables initialization */
  _atl_threads_current = 0;
  _atl_threads_index = 0;
  /* _atl_threads_key creation */

  /* global usage table initialization */
  for ( i = 0;
        i <= ATL_THREADS_MAX;
        i++ )
  {
    _atl_threads_table_used[i] = 0;
  }
#endif
  RTRT_MT_CRITICAL_VAR_INIT(atl_index_critical) /* mutex protecting the thread_index manipulations */

  RTRT_MT_THREAD_SPECIFIC_DATA_CREATE
}

/*******************************************************************************
*  Service: _atl_threads_init
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
  RTRT_MT_THREAD_SPECIFIC_DATA_DELETE
}
#else /* ATL_MULTI_THREADS */

#define THREADINDEX()

#endif /* ATL_MULTI_THREADS */

