
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: CoverageCompact.java,v 1.8.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Compact Mode Coverage Library Source $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public class CoverageCompact extends CoverageData {

  private byte branches[][];
  private int nb_branches[];

  public CoverageCompact ( int _key,
                           int _checksum,
                           int nb_proc, 
                           int nb_block ) {
    super ( _key, _checksum );
    createBranches ( nb_proc, nb_block );
  }

  protected void createBranches ( int nb_proc, int nb_block ) {
    nb_branches = new int[NB_FAMILIES];
    nb_branches[PROC] = nb_proc;
    nb_branches[BLOCK] = nb_block;
    branches = new byte[NB_FAMILIES][];
    if ( nb_proc > 0 ) {
      branches[PROC] = new byte[(nb_proc>>3)+(((nb_proc&0x07)>0)?1:0)];
    }
    if ( nb_block > 0 ) {
      branches[BLOCK] = new byte[(nb_block>>3)+(((nb_block&0x07)>0)?1:0)];
    }
  }

  private void resetTable ( byte family ) {
    if ( branches[family] == null ) {
      return;
    }
    for ( int i = 0;
          i < branches[family].length;
          i++ ) {
      branches[family][i] = 0;
    }
  }

  protected void resetTables ( ) {
    resetTable ( PROC );
    resetTable ( BLOCK );
  }

  protected boolean isCountCompatible ( ) {
    return false;
  }

  protected void dumpBranches ( byte family,
                                byte type,
                                boolean count_still_allowed ) {
    if ( branches[family] == null ) {
      return;
    }
    for ( int i = 0;
          i < nb_branches[family];
          i++ ) {
      if ( getBranch ( family, i ) ) {
	  if (com.rational.testrealtime.Buffer.is_open())
	      {
		  Buffer.pushOpCode ( (byte)'T', type ); // TP or TB
		  Buffer.pushRUnsigned16 ( i );
		  Buffer.pushLine ( );
	      }
      }
    }
  }

  public boolean getBranch ( byte family,
                             int branch_index ) {
    if ( (branches[family][branch_index>>3] & (0x01<<(branch_index&0x07)))
         != 0 ) {
      return true;
    } else {
      return false;
    }
  }

  public void setBranch ( byte family,
                          int branch_index ) {
    branches[family][branch_index>>3] |= (0x01<<(branch_index&0x07));
  }

  public void unsetBranch ( byte family,
                            int branch_index,
                            boolean local_copy ) {
    synchronized ( branches[family] ) {
      if ( ! local_copy &&
           getBranch ( family, branch_index ) ) {
        branches[family][branch_index>>3] &= ~(0x01<<(branch_index&0x07));
      }
    }
  }

  public boolean setBranchTernary ( int true_index,
                                    int false_index,
                                    boolean expr ) {
    if ( expr ) {
      setBranch ( BLOCK, true_index );
    } else {
      setBranch ( BLOCK, false_index );
    }
    return expr;
  }

}

