
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: CoverageCount.java,v 1.9.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Count Mode Coverage Library Source $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public class CoverageCount extends CoverageData {

  private int branches[][];
  private static final int MAX_COUNT_HIT = 0x7FFFFFFF;

  public CoverageCount ( int _key,
                         int _checksum,
                         int nb_proc, 
                         int nb_block ) {
    super ( _key, _checksum );
    createBranches ( nb_proc, nb_block );
  }

  protected void createBranches ( int nb_proc, int nb_block ) {
    if ( nb_proc > 0 || nb_block > 0 ) {
      branches = new int[NB_FAMILIES][];
      if ( nb_proc > 0 ) {
        branches[PROC] = new int[nb_proc];
      }
      if ( nb_block > 0 ) {
        branches[BLOCK] = new int[nb_block];
      }
    }
  }

  private void resetTable ( byte family ) {
    if ( branches[family] == null ) {
      return;
    }
    for ( int i = 0;
          i < branches[family].length;
          i++ ) {
      branches[family][i] = 0;
    }
  }

  protected void resetTables ( ) {
    resetTable ( PROC );
    resetTable ( BLOCK );
  }

  protected boolean isCountCompatible ( ) {
    return true;
  }

  protected void dumpBranches ( byte family,
                                byte type,
                                boolean count_still_allowed ) {
    if ( branches[family] == null ) {
      return;
    }
    for ( int i = 0;
          i < branches[family].length;
          i++ ) {
      if ( branches[family][i] > 0 ) {
	  if (com.rational.testrealtime.Buffer.is_open())
	      {
		  Buffer.pushOpCode ( (byte)'T', type ); // TP or TB
		  Buffer.pushRUnsigned16 ( i );
		  if ( count_still_allowed ) {
		      Buffer.pushSpace ( );
		      Buffer.pushRUnsigned16 ( branches[family][i] );
		  }
		  Buffer.pushLine ( );
	      }
      }
    }
  }

  public int getBranch ( byte family,
                         int branch_index ) {
    synchronized ( branches[family] ) {
      return branches[family][branch_index];
    }
  }

  public void setBranch ( byte family,
                          int branch_index ) {
    synchronized ( branches[family] ) {
      if ( branches[family][branch_index] < MAX_COUNT_HIT ) {
        branches[family][branch_index]++;
      }
    }
  }

  public void unsetBranch ( byte family,
                            int branch_index,
                            int local_copy ) {
    synchronized ( branches[family] ) {
      if ( branches[family][branch_index] > local_copy ) {
        branches[family][branch_index] = local_copy;
      }
    }
  }

  public boolean setBranchTernary ( int true_index,
                                    int false_index,
                                    boolean expr ) {
    if ( expr ) {
      setBranch ( BLOCK, true_index );
    } else {
      setBranch ( BLOCK, false_index );
    }
    return expr;
  }

}

