
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: CoverageData.java,v 1.18.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Coverage Library Source $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public abstract class CoverageData implements Dumpable {

  public static final byte PROC = 0, BLOCK = 1, NB_FAMILIES = 2;

  abstract protected void resetTables ( );
  abstract protected void dumpBranches ( byte family,
                                         byte type,
                                         boolean count );
  abstract protected boolean isCountCompatible ( );

  public CoverageData ( int _key,
                        int _checksum ) {
    key = _key;
    checksum = _checksum;
    linked = false;
  }

  public void link ( ) {
    if ( ! linked ) {
      synchronized ( Buffer.buffer ) {
        if ( first_data != null ) {
          next = first_data;
        } else {
          TestRT.dump_1 = this;
        }
        first_data = this;
        if ( still_is_count_compatible &&
             ! isCountCompatible ( ) ) {
          still_is_count_compatible = false;
        }
      }
      linked = true;
    }
  }

  public static void resetDatas ( ) {
    CoverageData current, _next;

    for ( current = first_data;
          current != null;
          ) {
      _next = current.next;
      current.next = null;
      current.linked = false;
      current = _next;
    }
  }

  protected int key, checksum;
  protected boolean linked;
  protected CoverageData next = null;

  private static CoverageData first_data = null;
  private static boolean still_is_count_compatible = true;

  protected static void pushModeLine ( ) {
    if ( still_is_count_compatible ) {
	if (com.rational.testrealtime.Buffer.is_open())
	    {
		Buffer.pushOpCodeSingle ( (byte)'C', (byte)'O' ); // CO
		Buffer.pushLine ( );
	    }
    }
  }

  public void dumpData ( ) {

      if (com.rational.testrealtime.Buffer.is_open())
	  {
	      Buffer.pushOpCode ( (byte)'F', (byte)'T' ); // FT
	      Buffer.pushRUnsigned16 ( key );
	      Buffer.pushLine ( );
	      Buffer.pushOpCode ( (byte)'D', (byte)'C' ); // DC
	      Buffer.pushRUnsigned16 ( checksum );
	      Buffer.pushLine ( );
	  }
      dumpBranches ( PROC, (byte)'P', still_is_count_compatible ); // TP
      dumpBranches ( BLOCK, (byte)'B', still_is_count_compatible ); // TB
  }

  public static void dump ( boolean link_reset, boolean branch_reset ) {
    CoverageData current, _next;
    if (com.rational.testrealtime.Buffer.is_open())
	{
	    if ( TestNode.nametest != null ) {
	      Buffer.pushOpCode ( (byte)'N', (byte)'T' ); // NT
              Buffer.pushLiteral ( TestNode.nametest );
            } else {
	      Buffer.pushOpCodeSingle ( (byte)'N', (byte)'T' ); // NT
            }
	    Buffer.pushLine ( );
	    Buffer.pushOpCode ( (byte)'D', (byte)'T' ); // DT
	    Buffer.pushRUnsigned16 ( 0 );
	    Buffer.pushLine ( );
	}
    pushModeLine ( );
    for ( current = first_data;
          current != null;
          ) {
      current.dumpData ( );
      if ( branch_reset ) {
        current.resetTables ( );
      }
      _next = current.next;
      if ( link_reset ) {
        current.next = null;
        current.linked = false;
      }
      current = _next;
    }
    if ( link_reset ) {
      first_data = null;
      TestRT.dump_1 = null;
    }
  }

  // Dumpable implementation

  public void dump ( ) {
    dump ( true, true );
  }

}

