
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: QuantifyLT.java,v 1.17.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: QuantifyLT Library Source $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public class QuantifyLT implements Dumpable, StackLinkListener {

    public QuantifyLT ( SourceId _source_id,
                        int nb_proc ) {
      source_id = _source_id;
      calls_table = new int[nb_proc];
      local_table = new long[nb_proc];
      total_table = new long[nb_proc];
    }

    private SourceId source_id;
    private int calls_table[];
    private long local_table[], total_table[];
    private boolean linked;
    private QuantifyLT next = null;

    private static QuantifyLT first = null;

    public void link ( ) {
      if ( ! linked ) {
        if ( first != null ) {
          next = first;
        } else {
          TestRT.dump_2 = this;
        }
        first = this;
        linked = true;
      }
    }

    public void resetTables ( ) {
      for ( int i = 0;
            i < calls_table.length;
            i++ ) {
        calls_table[i] = 0;
        local_table[i] = 0;
        total_table[i] = 0;
      }
    }

    public static void resetDatas ( ) {
      QuantifyLT current, _next;

      for ( current = first;
            current != null;
            ) { 
        _next = current.next;
        current.next = null;
        current.linked = false;
        current = _next;
      }
    }

    public void dumpData ( ) {

	  if (com.rational.testrealtime.Buffer.is_open())
	      {
		  Buffer.pushOpCode ( (byte)'Q', (byte)'S' ); // QS
		  Buffer.pushRUnsigned16 ( source_id.tsf_id1 );
		  Buffer.pushSpace ( );
		  Buffer.pushRUnsigned16 ( source_id.tsf_id2 );
		  Buffer.pushLine ( );
	      }
	  for ( int i = 0;
		i < calls_table.length;
		i++ ) {
        if ( calls_table[i] > 0 ) {

	    if (com.rational.testrealtime.Buffer.is_open())
		{
		    Buffer.pushOpCode ( (byte)'Q', (byte)'F' ); // QF
		    // message_id have been divided by 2
		    Buffer.pushRUnsigned16 ( (i<<1) + 1 );
		    Buffer.pushSpace ( );
		    Buffer.pushRUnsigned16 ( calls_table[i] );
		    Buffer.pushSpace ( );
		    if ( ( local_table[i] >> 32 ) != 0 ) {
			Buffer.pushRUnsigned16 ( (int)(local_table[i] >> 32) );
			Buffer.pushComma ( );
		    }
		    Buffer.pushRUnsigned16 ( (int)(local_table[i] & 0xFFFFFFFF) );
		    Buffer.pushSpace ( );
		    if ( ( total_table[i] >> 32 ) != 0 ) {
			Buffer.pushRUnsigned16 ( (int)(total_table[i] >> 32) );
			Buffer.pushComma ( );
		    }
		    Buffer.pushRUnsigned16 ( (int)(total_table[i] & 0xFFFFFFFF) );
		    Buffer.pushLine ( );
		}
        }
      }
    }

    public static void dump ( boolean link_reset, boolean branch_reset ) {
      QuantifyLT current, _next;

      if (com.rational.testrealtime.Buffer.is_open()) {
	Buffer.pushOpCode ( (byte)'Q', (byte)'U' ); // QU
	Buffer.pushLiteral ( Custom.getClockUnit ( ) );
	Buffer.pushSpace ( );
	Buffer.pushLiteral ( Products.node_name.getBytes ( ) );
	Buffer.pushLine ( );
	      
	if ( TestNode.nametest != null ) {
	  Buffer.pushOpCode ( (byte)'Q', (byte)'T' ); // QT
          Buffer.pushLiteral ( TestNode.nametest );
        } else {
	  Buffer.pushOpCodeSingle ( (byte)'Q', (byte)'T' ); // QT
        }
	Buffer.pushLine ( );
	Buffer.pushOpCode ( (byte)'Q', (byte)'D' ); // QD
	Buffer.pushRUnsigned16 ( 0 );
	Buffer.pushLine ( );
      }
      for ( current = first; current != null; ) {
        current.dumpData ( );
	if ( branch_reset ) {
	  current.resetTables ( );
	}
        _next = current.next;
        if ( link_reset ) {
          current.next = null;
          current.linked = false;
        }
        current = _next;
      }
      if ( link_reset ) {
        first = null;
        TestRT.dump_1 = null;
      }
    }

    // StackLinkListener implementation

    public void onPush ( StackLink stack_link ) {
      long now = Custom.getClock ( );
      // Let's link it !
      link ( );
      // message_id are to be divided by 2
      calls_table[stack_link.message_id>>1]++;
      stack_link.local_amount = 0;
      stack_link.total_amount = 0;
      if ( stack_link.up != null ) {
        stack_link.up.local_amount += now - stack_link.up.period_start;
        stack_link.up.total_amount += now - stack_link.up.period_start;
      }
      stack_link.period_start = Custom.getClock ( );
    }

    public void onPop ( StackLink stack_link ) {
      long now = Custom.getClock ( );
      stack_link.local_amount += now - stack_link.period_start;
      stack_link.total_amount += now - stack_link.period_start;
      // message_id are to be divided by 2
      local_table[stack_link.message_id>>1] += stack_link.local_amount;
      StackLink up;
      /* Is it a recursive call ? */
      for ( up = stack_link.up;
            up != null &&
            ( up.source_id != stack_link.source_id ||
              up.message_id != stack_link.message_id );
            up = up.up );
      if ( up == null ) {
        total_table[stack_link.message_id>>1] += stack_link.total_amount;
      }
      if ( stack_link.up != null ) {
        stack_link.up.total_amount += stack_link.total_amount;
        stack_link.up.period_start = Custom.getClock ( );
      }
    }

    // Dumpable implementation

    public void dump ( ) {
      dump ( true, true );
    }

}

