
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: StackLink.java,v 1.12.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Call Stack Management $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public class StackLink {

  public StackLink ( SourceId _source_id,
                     int _message_id,
                     int _class_id,
                     int _instance_id,
                     boolean _in_ctor ) {

    source_id = _source_id;
    up = null;
    down = null;
    message_id = _message_id;
    class_id = _class_id;
    instance_id = _instance_id;
    in_ctor = _in_ctor;
    in_finalize = false;

    synchronized ( ThreadInfo.threads_table ) {

      boolean is_new_thread = ThreadInfo.checkThread ( source_id );
      thread_index = ThreadInfo.current_index;

      if ( is_new_thread ) {
        ThreadInfo.threads_table[thread_index].first = this;
        ThreadInfo.threads_table[thread_index].current = this;
      } else {
        up = ThreadInfo.threads_table[thread_index].current;
        if ( up != null ) {
          up.down = this;
        }
        ThreadInfo.threads_table[thread_index].current = this;
      }

      if ( source_id.thread_listener != null ) {
        source_id.thread_listener.onPush ( this );
      }

      if ( source_id.listener_plt != null ) {
        source_id.listener_plt.onPush ( this );
      }

      if ( source_id.listener_qlt != null ) {
        source_id.listener_qlt.onPush ( this );
      }
    }
  }

  public void pop ( boolean _in_finalize ) {

    synchronized ( ThreadInfo.threads_table ) {

      ThreadInfo.checkThread ( source_id );

      in_finalize = _in_finalize;

      if ( source_id.thread_listener != null ) {
        source_id.thread_listener.onPop ( this );
      }

      if ( source_id.listener_plt != null ) {
        source_id.listener_plt.onPop ( this );
      }
    
      if ( source_id.listener_qlt != null ) {
        source_id.listener_qlt.onPop ( this );
      }
      ThreadInfo.threads_table[thread_index].current = up;
      if ( up == null ) {
        ThreadInfo.threads_table[thread_index].first = null;
      } else {
        up.down = null;
      }
    }
  }

  public SourceId source_id;
  public int message_id, class_id, instance_id;
  public boolean in_ctor, in_finalize;
  public long period_start = 0, local_amount = 0, total_amount = 0;

  public int thread_index;

  public StackLink up, down;

}

