
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: ThreadInfo.java,v 1.17.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Threads Management $
////////////////////////////////////////////////////////////////////////////////


package com.rational.testrealtime;

public class ThreadInfo {

  public static final int NEW = 0, SCHEDULE = 1, SAME = 2;
  private static Thread current_thread = null;
  public static int current_index = -1;
  public static int current_unique_id = 1;
  public static int max_call_stack = 0;

  public static ThreadInfo threads_table[];
  static {
    threads_table = new ThreadInfo[Products.max_threads];
    for ( int i = 0; i < Products.max_threads; i++ ) {
      threads_table[i] = null;
    }
  }

  public Thread thread_ref;
  public int unique_id;

  // Here are any information that must be private to the thread
  public StackLink first, current;
  public int current_instance_id;
  public int local_exception_id, local_exception_tsf;
  public int current_call_stack;
  public boolean trace_is_on;
  public SourceId throw_source_id;

  private ThreadInfo ( Thread thread ) {
    init ( thread );
  }

  private void init ( Thread thread ) {
    thread_ref = thread;
    first = null;
    current = null;
    local_exception_id = 0;
    local_exception_tsf = 0;
    throw_source_id = null;
    current_call_stack = 0;
    unique_id = current_unique_id++;
    trace_is_on = false;
  }

  private static boolean findIndex ( SourceId source_id ) {

    if ( source_id != null &&
         source_id.thread_listener != null &&
         current_thread != null ) {
      if ( current_thread.isAlive ( ) ) {
        source_id.thread_listener.onFallAsleep ( current_index );
      }
      else {
        source_id.thread_listener.onDead ( current_index );
        threads_table[current_index] = null;
      }
    }

    current_thread = Thread.currentThread();
    int i;
    for ( i = 0;
          i < Products.max_threads;
          i++ ) {
      if ( threads_table[i] != null &&
           threads_table[i].thread_ref == current_thread ) {
        current_index = i;
        if ( source_id != null &&
             source_id.thread_listener != null ) {
          source_id.thread_listener.onActivate ( current_index );
        }
        return false;
      }
    }

    for ( i = 0;
          i < Products.max_threads;
          i++ ) {
      if ( threads_table[i] == null ) {
        threads_table[i] = new ThreadInfo ( current_thread );
        current_index = i;
        if ( source_id != null &&
             source_id.thread_listener != null ) {
          source_id.thread_listener.onNew ( source_id, current_index );
        }
        return true;
      } else if ( ! threads_table[i].thread_ref.isAlive ( ) ) {
        if ( source_id != null &&
             source_id.thread_listener != null ) {
          source_id.thread_listener.onDead ( i );
        }
        threads_table[i].init ( current_thread );
        current_index = i;
        if ( source_id != null &&
             source_id.thread_listener != null ) {
          source_id.thread_listener.onNew ( source_id, current_index );
        }
        return true;
      }
    }

    // Products.max_threads settings too small
    Custom.pushError ( "Maximum number of threads is too small (currently: "
                       + Products.max_threads + ")" );
    // Re-use the last one as an error recovery try
    threads_table[i-1].init ( current_thread );
    current_index = i - 1;
    if ( source_id != null &&
         source_id.thread_listener != null ) {
      source_id.thread_listener.onNew ( source_id, current_index );
    }
    return true;
  }

  public static synchronized boolean checkThread ( SourceId source_id ) {
    boolean is_new_thread = false;
    if ( Thread.currentThread() != current_thread ) {
      is_new_thread = findIndex ( source_id );
    }
    return is_new_thread;
  }

}

