
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

////////////////////////////////////////////////////////////////////////////////
// $Id: Trace.java,v 1.31.4.1 2008/03/18 13:31:17 build Exp $
// $Descr: Trace Library Source $
////////////////////////////////////////////////////////////////////////////////

package com.rational.testrealtime;

public class Trace implements ThreadListener, ExitListener {

    private static int last_file_id = 0;
    private static int current_source_id = 0;
    private static long clock_first = 0;
    private static long heap_size = -1;

    private static void pushLine ( ) {

	if (com.rational.testrealtime.Buffer.is_open()) {
	    Buffer.pushLine ( );
	}	

	if ( Products.trace_on_the_fly ) {
	    Buffer.flush ( );
	}
	
    }
    
    public static void newSource ( SourceId source_id ) {

	if (com.rational.testrealtime.Buffer.is_open()) {
	
	    if ( source_id.trace_dynamic_id == 0 ) {
		source_id.trace_dynamic_id = ++last_file_id;
		current_source_id = source_id.trace_dynamic_id;
		Buffer.pushOpCode ( (byte)'S', (byte)'F' ); // SF
		Buffer.pushUnsigned16h ( current_source_id );
		Buffer.pushSpace ( );
		Buffer.pushUnsigned16h ( source_id.tsf_id1 );
		Buffer.pushSpace ( );
		Buffer.pushUnsigned16h ( source_id.tsf_id2 );
		pushLine ( );
	    } else {
		changeSource ( source_id.trace_dynamic_id );
	    }

	}
    }
    
    public static void changeSource ( int trace_dynamic_id ) {
	if (com.rational.testrealtime.Buffer.is_open()) {
	    if ( current_source_id != trace_dynamic_id ) {
		current_source_id = trace_dynamic_id;
		Buffer.pushOpCode ( (byte)'C', (byte)'S' ); // CS
		Buffer.pushUnsigned16h ( current_source_id );
		pushLine ( );
	    }
	}
    }

    private static int last_instance_id = 0;


    public static int getInstanceId() {
      return ( ++last_instance_id );
    }

    public static void setCurrentInstance ( int instance_id ) {
      ThreadInfo.threads_table[ThreadInfo.current_index].current_instance_id =
        instance_id;
    }

    public static int getCurrentInstance ( ) {
      return
        ThreadInfo.threads_table[ThreadInfo.current_index].current_instance_id;
    }

    public static int newInstance ( StackLink stack_link ) {
      if ( stack_link.instance_id == 0 ) {
        stack_link.instance_id = getInstanceId();
      }
      enterMethod ( stack_link );

      if (com.rational.testrealtime.Buffer.is_open()) {
        Buffer.pushOpCode ( (byte)'N', (byte)'I' ); // NI
        Buffer.pushUnsigned16h ( stack_link.class_id );
        Buffer.pushSpace ( );
        Buffer.pushUnsigned16h ( stack_link.instance_id );
        pushLine ( );
      }
      setCurrentInstance ( stack_link.instance_id );
      return stack_link.instance_id;
    }

    public static void deleteInstance ( StackLink stack_link ) {
      exitMethod ( stack_link );
      if (com.rational.testrealtime.Buffer.is_open()) {
        Buffer.pushOpCode ( (byte)'D', (byte)'I' ); // DI
        Buffer.pushUnsigned16h ( stack_link.instance_id );
        Buffer.pushSpace ( );
        Buffer.pushUnsigned16h ( stack_link.message_id );
        pushLine ( );
      }
    }

    public static void changeInstance ( int instance_id ) {
      setCurrentInstance ( instance_id );
      if (com.rational.testrealtime.Buffer.is_open()) {
        Buffer.pushOpCode ( (byte)'C', (byte)'I' ); // CI
        Buffer.pushUnsigned16h ( instance_id );
        pushLine ( );
      }
    }

    public static void enterMethod ( StackLink stack_link ) {
      if ( Products.trace_partial_message_dump &&
           ! traceIsOn ( ) ) {
        return;
      }
      if (com.rational.testrealtime.Buffer.is_open()) {
        printTimeStamp ( );
        printHeapSize ( );
        Buffer.pushOpCode ( (byte)'M', (byte)'E' ); // ME
        Buffer.pushUnsigned16h ( stack_link.message_id );
        Buffer.pushSpace ( );
        Buffer.pushUnsigned16h ( stack_link.instance_id );
        pushLine ( );
      }
      setCurrentInstance ( stack_link.instance_id );
    }

    public static void exitMethod ( StackLink stack_link ) {
      if ( Products.trace_partial_message_dump &&
           ! traceIsOn ( ) ) {
        return;
      }
      if (com.rational.testrealtime.Buffer.is_open()) {
        printTimeStamp ( );
        printHeapSize ( );
        changeSource ( stack_link.source_id.trace_dynamic_id );
        Buffer.pushOpCode ( (byte)'M', (byte)'R' ); // MR
        Buffer.pushUnsigned16h ( stack_link.message_id );
        pushLine ( );
      }
      if ( stack_link.up != null ) {
        setCurrentInstance ( stack_link.up.instance_id );
      } else {
        setCurrentInstance ( 0 );
      }
    }

    private static int last_exception_id = 0;

    public static void throwException ( SourceId source_id,
                                        int message_id,
                                        int instance_id ) {
      synchronized ( ThreadInfo.threads_table ) {
        if (com.rational.testrealtime.Buffer.is_open()) {
          printTimeStamp ( );
          printHeapSize ( );
        }
        ThreadInfo.checkThread ( source_id );
        changeSource ( source_id.trace_dynamic_id );
        ThreadInfo.threads_table[ThreadInfo.current_index].
          throw_source_id = source_id;
        ThreadInfo.threads_table[ThreadInfo.current_index].
          local_exception_id = ++last_exception_id;
        ThreadInfo.threads_table[ThreadInfo.current_index].
          local_exception_tsf = message_id;
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'M', (byte)'C' ); // MC
          Buffer.pushUnsigned16h ( ThreadInfo.
                                   threads_table[ThreadInfo.current_index].
                                   local_exception_id );
          Buffer.pushSpace ( );
          Buffer.pushUnsigned16h ( message_id );
          Buffer.pushSpace ( );
          Buffer.pushUnsigned16h ( instance_id );
          pushLine ( );
        }
      }
    }

    public static void catchException ( SourceId source_id,
                                        int text_id,
                                        int instance_id,
                                        int exception_id ) {
      synchronized ( ThreadInfo.threads_table ) {
        printTimeStamp ( );
        printHeapSize ( );
        ThreadInfo.checkThread ( source_id );
        if ( ThreadInfo.threads_table[ThreadInfo.current_index].
             local_exception_id == 0 ) {
          // The corresponding throw has NOT been traced
          changeInstance ( 0 );
          if (com.rational.testrealtime.Buffer.is_open()) {
            Buffer.pushOpCode ( (byte)'M', (byte)'E' ); // ME
            Buffer.pushUnsigned16h ( exception_id );
            Buffer.pushSpace ( );
            Buffer.pushUnsigned16h ( instance_id );
            pushLine ( );
          }
        } else {
          // The corresponding throw has been traced
          changeSource ( ThreadInfo.
                         threads_table[ThreadInfo.current_index].
                         throw_source_id.trace_dynamic_id );

          if (com.rational.testrealtime.Buffer.is_open()) {
            Buffer.pushOpCode ( (byte)'M', (byte)'E' ); // ME
            Buffer.pushUnsigned16h ( ThreadInfo.
                                     threads_table[ThreadInfo.current_index].
                                     local_exception_tsf );
            Buffer.pushSpace ( );
            Buffer.pushUnsigned16h ( instance_id );
            Buffer.pushSpace ( );
            Buffer.pushUnsigned16h ( ThreadInfo.
                                     threads_table[ThreadInfo.current_index].
                                     local_exception_id );
            pushLine ( );
          }
          ThreadInfo.threads_table[ThreadInfo.current_index].
            throw_source_id = null;
          ThreadInfo.threads_table[ThreadInfo.current_index].
            local_exception_id = 0;
          ThreadInfo.threads_table[ThreadInfo.current_index].
            local_exception_tsf = 0;
        }
      }
      printNoteId ( source_id, text_id, instance_id );
    }

    public static void printNoteId ( SourceId source_id,
                                     int text_id,
                                     int instance_id ) {
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( source_id );
        if ( Products.trace_partial_message_dump &&
             ! traceIsOn ( ) ) {
          return;
        }

        if (com.rational.testrealtime.Buffer.is_open()) {
          printTimeStamp ( );
          printHeapSize ( );
        }
        changeSource ( source_id.trace_dynamic_id );
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'N', (byte)'O' ); // NO
          Buffer.pushUnsigned16h ( text_id );
          pushLine ( );
        }
        changeInstance ( instance_id );
      }
    }

    public static void printNoteId ( SourceId source_id,
                                     int text_id,
                                     int instance_id,
				     byte text[] ) {
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( source_id );
        if ( Products.trace_partial_message_dump &&
             ! traceIsOn ( ) ) {
          return;
        }

        if (com.rational.testrealtime.Buffer.is_open()) {
          printTimeStamp ( );
          printHeapSize ( );
        }
        changeSource ( source_id.trace_dynamic_id );
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'N', (byte)'O' ); // NO
          Buffer.pushUnsigned16h ( text_id );
          Buffer.pushSpace ( );
	  Buffer.pushLiteral ( text );
          pushLine ( );
        }
        changeInstance ( instance_id );
      }
    }

    public static void printGlyphId ( SourceId source_id,
                                     int glyph_id,
                                     int text_id,
                                     int instance_id ) {
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( source_id );
        if ( Products.trace_partial_message_dump &&
             ! traceIsOn ( ) ) {
          return;
        }

        if (com.rational.testrealtime.Buffer.is_open()) {
          printTimeStamp ( );
          printHeapSize ( );
        }
        changeSource ( source_id.trace_dynamic_id );
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'G', (byte)'L' ); // NO
          Buffer.pushUnsigned16h ( glyph_id );
          Buffer.pushSpace ( );
          Buffer.pushUnsigned16h ( text_id );
          pushLine ( );
        }
        changeInstance ( instance_id );
      }
    }

    public static void printGlyphId ( SourceId source_id,
                                     int glyph_id,
                                     int text_id,
                                     int instance_id,
				     byte text[] ) {
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( source_id );
        if ( Products.trace_partial_message_dump &&
             ! traceIsOn ( ) ) {
          return;
        }

        if (com.rational.testrealtime.Buffer.is_open()) {
          printTimeStamp ( );
          printHeapSize ( );
        }
        changeSource ( source_id.trace_dynamic_id );
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'G', (byte)'L' ); // NO
          Buffer.pushUnsigned16h ( glyph_id );
          Buffer.pushSpace ( );
          Buffer.pushUnsigned16h ( text_id );
          Buffer.pushSpace ( );
	  Buffer.pushLiteral ( text );
          pushLine ( );
        }
        changeInstance ( instance_id );
      }
    }


    public static void printNoteText ( byte text[],
                                       int instance_id ) {
      if (com.rational.testrealtime.Buffer.is_open()) {
        printTimeStamp ( );
        printHeapSize ( );
        Buffer.pushOpCode ( (byte)'N', (byte)'O' ); // NO
        Buffer.pushLiteral ( text );
        pushLine ( );
      }
      changeInstance ( instance_id );
    }

    // Partial Message Dump:

    public static boolean traceIsOn ( ) {
      return ThreadInfo.threads_table[ThreadInfo.current_index].trace_is_on;
    }

    public static void printCallStack ( ) {
      for ( StackLink current = ThreadInfo.
                                  threads_table[ThreadInfo.current_index].first;
            current != null;
            current = current.down ) {
        enterMethod ( current );
      }
    }

    private static String messageDumpOnString = null,
                          messageDumpOffString = null;

    public static void actuallySetMessageDump ( boolean mode ) {
      ThreadInfo.threads_table[ThreadInfo.current_index].trace_is_on = mode;
      if ( mode ) {
        if ( messageDumpOnString == null ) {
          messageDumpOnString = new String ( "Message dump starts here with call stack:" );
        }
        printNoteText ( messageDumpOnString.getBytes ( ), 0 );
        printCallStack ( );
      } else {
        if ( messageDumpOffString == null ) {
          messageDumpOffString = new String ( "Message dump stops here" );
        }
        printNoteText ( messageDumpOffString.getBytes ( ), 0 );
      }
    }

    public static void setMessageDump ( boolean mode ) {
      if ( ! Products.trace_partial_message_dump ) {
        return;
      }
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( null );
        actuallySetMessageDump ( mode );
      }
    }

    public static void toggleMessageDump ( ) {
      if ( ! Products.trace_partial_message_dump ) {
        return;
      }
      synchronized ( ThreadInfo.threads_table ) {
        ThreadInfo.checkThread ( null );
        if ( ThreadInfo.threads_table[ThreadInfo.current_index].trace_is_on ) {
          actuallySetMessageDump ( false );
        } else {
          actuallySetMessageDump ( true );
        }
      }
    }

    // Style:

    public static void printStyle  ( int _style )
    {
      if (com.rational.testrealtime.Buffer.is_open()) {
        Buffer.pushOpCode ( (byte)'S', (byte)'T' ); // ST
        Buffer.pushUnsigned16h ( _style );
        Buffer.pushLine ( );
      }
    }



    // Time Stamp:

    public static void initTimeStamp ( ) {
      if ( Products.trace_time_stamp ) {
        clock_first = Custom.getClock ( );
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'T', (byte)'U' ); // TU
          Buffer.pushLiteral ( Custom.getClockUnit ( ) );
          Buffer.pushLine ( );
        }
      }
    }

    public static void printTimeStamp ( ) {
      if ( Products.trace_time_stamp ) {
        long now = Custom.getClock ( );
        now -= clock_first;
        if (com.rational.testrealtime.Buffer.is_open()) {
          Buffer.pushOpCode ( (byte)'T', (byte)'M' ); // TM
          if ( ( now >> 32 ) != 0 ) {
            Buffer.pushUnsigned16h ( (int)(now >> 32) );
            Buffer.pushSpace ( );
          }
          Buffer.pushUnsigned16h ( (int)(now & 0xFFFFFFFF) );
          pushLine ( );
        }
      }
    }

    // Heap Size:

    public static void printHeapSize ( ) {
      if ( Products.trace_heap_size ) {
        long current_heap_size =
          java.lang.Runtime.getRuntime().totalMemory ( ) -
          java.lang.Runtime.getRuntime().freeMemory ( );
        if ( current_heap_size != heap_size ) {
          heap_size = current_heap_size;
          if (com.rational.testrealtime.Buffer.is_open()) {
            Buffer.pushOpCode ( (byte)'H', (byte)'S' ); // TM
            if ( ( heap_size >> 32 ) != 0 ) {
              Buffer.pushUnsigned16h ( (int)(heap_size >> 32) );
              Buffer.pushSpace ( );
            }
            Buffer.pushUnsigned16h ( (int)(heap_size & 0xFFFFFFFF) );
            pushLine ( );
          }
        }
      }
    }

    // StackLinkListener implementation:

    public void onPush ( StackLink stack_link ) {
      newSource ( stack_link.source_id );
      if ( stack_link.in_ctor ) {
        stack_link.instance_id = newInstance ( stack_link );
      } else {
        enterMethod ( stack_link );
      }
      if ( Products.trace_max_call_stack ) {
        ThreadInfo.threads_table[ThreadInfo.current_index].
          current_call_stack++;
        if ( ThreadInfo.threads_table[ThreadInfo.current_index].
               current_call_stack >
             ThreadInfo.max_call_stack ) {
          ThreadInfo.max_call_stack =
            ThreadInfo.threads_table[ThreadInfo.current_index].
              current_call_stack;
        }
      }
    }

    public void onPop ( StackLink stack_link ) {
      if ( stack_link.in_finalize ) {
        deleteInstance ( stack_link );
      } else {
        exitMethod ( stack_link );
      }
      if ( Products.trace_max_call_stack ) {
        ThreadInfo.threads_table[ThreadInfo.current_index].
          current_call_stack--;
      }
    }

    // ThreadListener implementation:

    public void printStartThreadInfo ( int index,
                                       byte status ) {
      if (com.rational.testrealtime.Buffer.is_open()) {
        Buffer.pushOpCode ( (byte)'T', (byte)'I' ); // TI
        Buffer.pushUnsigned16h ( ThreadInfo.threads_table[index].unique_id );
        Buffer.pushOpCode ( (byte)' ', status );
        Buffer.pushUnsigned16h ( ThreadInfo.threads_table[index].thread_ref.
                                   getPriority ( ) );
      }
    }

    public void onNew ( SourceId source_id, int index ) {
      // Declare the new thread with the active state and maybe its name
      newSource ( source_id );
      if ( Products.trace_thread_info ) {
        printStartThreadInfo ( index, (byte)'1' /* ACTIVE */ );
        byte[] name = Custom.threadName ( );
        if ( name != null ) {
	    if (com.rational.testrealtime.Buffer.is_open()) {
		Buffer.pushSpace ( );
		Buffer.pushLiteral ( name );
	    }
        }
	if (com.rational.testrealtime.Buffer.is_open()) {
	    pushLine ( );
	}
      }
      changeInstance ( 0 );
    }

    public void onActivate ( int index ) {
      if ( Products.trace_thread_info ) {
        // Set the active state to the thread
	  if (com.rational.testrealtime.Buffer.is_open()) {
	      printStartThreadInfo ( index, (byte)'1' /* ACTIVE */ );
	      pushLine ( );
	  }
      }
      changeInstance ( ThreadInfo.threads_table[index].current_instance_id );
    }

    public void onFallAsleep ( int index ) {
      if ( Products.trace_thread_info ) {
        // Set the sleeping state to the thread
	  if (com.rational.testrealtime.Buffer.is_open()) {
	      printStartThreadInfo ( index, (byte)'2' /* SLEEPING */ );
	      pushLine ( );
	  }
      }
    }

    public void onDead ( int index ) {
      if ( Products.trace_thread_info ) {
        // Set the terminated state to the thread
	  if (com.rational.testrealtime.Buffer.is_open()) {
	      printStartThreadInfo ( index, (byte)'3' /* TERMINATED */ );
	      pushLine ( );
	  }
      }
    }

    // ExitListener implementation:

    public void onExit ( ) {
      if ( Products.trace_max_call_stack ) {
        String smsg = Integer.toString(ThreadInfo.max_call_stack) +
                      " (Max Calling Level reached)";
        byte msg[];
        try {
          msg = smsg.getBytes ( "UTF-8" );
        } catch ( java.io.UnsupportedEncodingException e ) {
          msg = smsg.getBytes ( );
        }
        synchronized ( ThreadInfo.threads_table ) {
          // push the SD Note on the World Actor
          printNoteText ( msg, 0 );
        }
      }
    }

    // On class load:

    public static Trace listener;

    static {
      listener = new Trace ( );
      TestRT.exit_listener = listener;
      initTimeStamp ( );
    }

}

