
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package j2meunit.framework;

/**
 * A set of assert methods.
 */

public class Assert {
public Assert() {
}
/**
 * Asserts that a condition is true. If it isn't it throws
 * an AssertionFailedError with the given message.
 */
public void assert(String message, boolean condition) {
	onAssertion();
	if (!condition)
		fail(message);
}
/**
 * Asserts that a condition is true. If it isn't it throws
 * an AssertionFailedError.
 */
public void assert(boolean condition) {
	assert(null, condition);
}
/**
 * Asserts that two longs are equal.
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertEquals(long expected, long actual) {
	assertEquals(null, expected, actual);
}
/**
 * Asserts that two objects are equal. If they are not
 * an AssertionFailedError is thrown.
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertEquals(Object expected, Object actual) {
	assertEquals(null, expected, actual);
}
/**
 * Asserts that two longs are equal.
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertEquals(String message, long expected, long actual) {
	assertEquals(message, new Long(expected), new Long(actual));
}
/**
 * Asserts that two objects are equal. If they are not
 * an AssertionFailedError is thrown.
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertEquals(String message, Object expected, Object actual) {
	onAssertion();
	if (expected == null && actual == null)
		return;
	if (expected != null && expected.equals(actual))
		return;
	failNotEquals(message, expected, actual);
}
/**
 * Asserts that an object isn't null.
 */
public void assertNotNull(Object object) {
	assertNotNull(null, object);
}
/**
 * Asserts that an object isn't null.
 */
public void assertNotNull(String message, Object object) {
	assert(message, object != null);
}
/**
 * Asserts that an object is null.
 */
public void assertNull(Object object) {
	assertNull(null, object);
}
/**
 * Asserts that an object is null.
 */
public void assertNull(String message, Object object) {
	assert(message, object == null);
}
/**
 * Asserts that two objects refer to the same object. If they are not
 * the same an AssertionFailedError is thrown.
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertSame(Object expected, Object actual) {
	assertSame(null, expected, actual);
}
/**
 * Asserts that two objects refer to the same object. If they are not
 * an AssertionFailedError is thrown.
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 */
public void assertSame(String message, Object expected, Object actual) {
	onAssertion();
	if (expected == actual)
		return;
	failNotSame(message, expected, actual);
}
/**
 * Fails a test with no message.
 */
public void fail() {
	fail(null);
}
/**
 * Fails a test with the given message.
 */
public void fail(String message) {
	throw new AssertionFailedError(message);
}
protected void failNotEquals(String message, Object expected, Object actual) {
	String formatted = "";
	if (message != null)
		formatted = message + " ";
	fail(formatted + "expected:<" + expected + "> but was:<" + actual + ">");
}
protected void failNotSame(String message, Object expected, Object actual) {
	String formatted = "";
	if (message != null)
		formatted = message + " ";
	fail(formatted + "expected same");
}
/**
 * This method is called every time an assertion is made (this does not include direct fails).
 */
protected void onAssertion()
{
}
}
