
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package junit.framework;

import junit.framework.*;
import java.lang.reflect.*;
import java.util.*;
import com.rational.testrealtime.*;
public class TestSuite extends TestSuitePrim{
/**
 * Dynamical id for the tsf id.
 * Used during the generation of the block.
 */
static int idin = 15;
/**
 * Tsf id of the class.
 * This members are set during the tsf generation.
 */
protected int tsfid = 11;
public void settsfid(int id) { tsfid = id;}
public int gettsfid() { return tsfid;}
/**
 * Dynamical id for the block which can
 * be generated.
 */
static protected int idtsf1 = 1;
static protected int idtsf2 = 2;
/**
 * TestSuiteRT can be a frame,
 * it enable this class to generate tsf blocks.
 */
protected boolean isframe = true;
public final void setAsFrame() { isframe = true;}
public final boolean isFrame() { return isframe;}

static
  {
    if (com.rational.testrealtime.Buffer.JTestThread == null)
      com.rational.testrealtime.Buffer.setJtestThread(Thread.currentThread());
  }

/**
 * Source id associated with the class.
 * This source id can be changed according to
 * the generated tsf block.
 */
protected com.rational.testrealtime.SourceId __trtt_cs_id = new com.rational.testrealtime.SourceId( 1, 2, com.rational.testrealtime.Trace.listener, null );
public final void setSourceId(int id1, int id2) {__trtt_cs_id = new com.rational.testrealtime.SourceId( id1, id2, com.rational.testrealtime.Trace.listener, null );};
/**
 * Style to generate a note for the run of the suite.
 */
public final static int style_name_suite = 181; //Test suite id for the styles.
/*******************************************************************************
 * Class TestSuite will presume that the member:
 * _trtt_getThis() will return the dynamical id for the class.
 *
 * Then the ouput of a note is:
 * Change to the source id.
 * Write the style.
 * Write the note.
 */
public int _trtt_getThis() { return _trtt_this; }
public void _trtt_setThis(int _this) { _trtt_this=_this; }
private int _trtt_this;
/**
 * Write note for the name of this instance.
 */
static private String StrUTF_8 = new String("<UTF-8 unsupported>");
private synchronized void writeNoteName()
{
    String message = "";
    if (toString() != null)
	{
	    message = message + toString();
	}
    else
	{
	    message = message + "null";
	}
    com.rational.testrealtime.Trace.printStyle( style_name_suite );
     try{
	com.rational.testrealtime.Trace.printNoteText(message.getBytes( "UTF-8" ), _trtt_getThis());
    }
    catch( java.io.UnsupportedEncodingException e )
	{
	    com.rational.testrealtime.Trace.printNoteText(StrUTF_8.getBytes(), _trtt_getThis());
	}
}
/**
 * Constructors.
 * During this process the constructor ask for an instance id
 * to the Trace manager.
 */
public TestSuite()
{
    super("null");
    _trtt_setThis( com.rational.testrealtime.Trace.getInstanceId());
}
/**
 *
 */
public TestSuite(String name)
{
    super(name);
    _trtt_setThis( com.rational.testrealtime.Trace.getInstanceId());
}
/**
 * Private methods to insert only TestRT compliant classes.
 */
private Constructor getConstructor(Class theClass) throws NoSuchMethodException
{
    Class[] args= { String.class };
    return theClass.getConstructor(args);
}
/**
 *
 */
private boolean isPublicTestMethod(Method m)
{
    return isTestMethod(m) && Modifier.isPublic(m.getModifiers());
}
/**
 *
 */
private boolean isTestMethod(Method m)
{
    String name= m.getName();
    Class[] parameters= m.getParameterTypes();
    Class returnType= m.getReturnType();
    return parameters.length == 0 && name.startsWith("test") && returnType.equals(Void.TYPE);
}
/**
 *
 */
private void addTestMethod(Method m, Vector names, Constructor constructor)
{
    String name= m.getName();
    if (names.contains(name))
	return;
    if (isPublicTestMethod(m)) {
	names.addElement(name);
	Object[] args= new Object[]{name};
	try {
	    addTest((TestCase)constructor.newInstance(args));
	} catch (InstantiationException e) {
	    com.rational.testrealtime.Custom.pushError("Can not instanciate test case " + e.getMessage());
	} catch (InvocationTargetException e) {
	    com.rational.testrealtime.Custom.pushError("Exception during construction" + e.getMessage());
	} catch (IllegalAccessException e) {
	    com.rational.testrealtime.Custom.pushError("Cannot access test case" + e.getMessage());
	}
    } else { // almost a test method
  	if (isTestMethod(m))
	    com.rational.testrealtime.Custom.pushError("Test method is not public");
    }
}
/**
 *
 */
public TestSuite(Class theClass)
{
    _trtt_setThis( com.rational.testrealtime.Trace.getInstanceId());
    if (theClass != null)
	{
	    setName(theClass.getName());
	    Constructor constructor= null;
	    try
		{
		    constructor= getConstructor(theClass);
		}
	    catch (Throwable e) {
		com.rational.testrealtime.Custom.pushError(e.getMessage());
		return;
	    }

	    if (!Modifier.isPublic(theClass.getModifiers()))
		{
		    com.rational.testrealtime.Custom.pushError("No public constructor " + theClass.getName());
		    return;
		}

	    Class superClass= theClass;
	    Vector names= new Vector();
	    while ( (TestCase.class.isAssignableFrom(superClass))) {
		Method[] methods= superClass.getDeclaredMethods();
		for (int i= 0; i < methods.length; i++) {
		    addTestMethod(methods[i], names, constructor);
		}
		superClass= superClass.getSuperclass();
	    }
	    Enumeration e = tests();
	    if (!e.hasMoreElements())
		com.rational.testrealtime.Custom.pushError("No test in the suite" + theClass.getName());
	}
    else
	{
	    setName("null");
	}
}
/**
 *
 */
public final void addTest(Test theTest)
{
    if (theTest != null)
	{
	    Class testclass = theTest.getClass();
	    try
		{
		    Class testrtclass = Class.forName("junit.framework.TestCase");
		    Class testrtsuiteclass = Class.forName("junit.framework.TestSuite");

		    //The check compliance.
		    if ( testrtclass.isAssignableFrom(testclass) || testrtsuiteclass.isAssignableFrom(testclass)  )
			{
			    super.addTest(theTest);
			}
		    else
			{
			    com.rational.testrealtime.Custom.pushError("AddTest:Class test is not compliant with test rt");
			}

		}
	    catch(Throwable e)
		{
		    com.rational.testrealtime.Custom.pushError("Cannot access the class");
		}
	}
}
/**
 *
 */
public final void addTestSuite(final Class thesuite)
{
    //Same purpose.
    addTest( new TestSuite(thesuite));
}
/**
 *
 */
protected final synchronized int generatetsf(TestSuite ref,int current_class_id)
{
    //Generate a tsf with the following information:
    int cid = current_class_id;
    try
	{

	    Class testcaseclass = Class.forName("junit.framework.TestCase");
	    Class testsuiteclass = Class.forName("junit.framework.TestSuite");

	    for (Enumeration e= ref.tests(); e.hasMoreElements(); ) {
		Test test = (Test)e.nextElement();
		Class tryclass;
		tryclass = test.getClass();

		if (testcaseclass.isAssignableFrom(tryclass))
		    {
			TestCase current = (TestCase)test;
			String name2 = current.getName();
			TsfMng.writeSI(cid);
			try {
                              String full_name = current.getClass().toString();
                              int position_class = full_name.lastIndexOf("class");
                              position_class += 6;
                              full_name = full_name.substring(position_class);
                              full_name += "." + name2;

        		    TsfMng.writeIN(TsfMng.testcase,++idin,full_name.getBytes( "UTF-8" ));
			}
			catch ( java.io.UnsupportedEncodingException exc ) {
			    TsfMng.writeIN(TsfMng.testcase,++idin,StrUTF_8.getBytes());

			    }
			current.settsfid(idin);
			current.isframe = false;
			current.setSourceId(idtsf1,idtsf2);
		    }
		else if (testsuiteclass.isAssignableFrom(tryclass))
		    {
			TestSuite current = (TestSuite)test;
			String name2 = current.toString();
			TsfMng.writeSI(cid);
			try
			    {
                              String full_name = current.getClass().toString();
                              int position_class = full_name.lastIndexOf("class");
                              position_class += 6;
                              full_name = full_name.substring(position_class);
                              full_name += "." + name2;
                              TsfMng.writeIN(TsfMng.testsuite,++idin,full_name.getBytes( "UTF-8" ));
			    }
			catch( java.io.UnsupportedEncodingException exc ) {
			    TsfMng.writeIN(TsfMng.testsuite,++idin,StrUTF_8.getBytes());
			    }
			current_class_id++;
			current.settsfid(idin);
			current.isframe = false;
			current.setSourceId(idtsf1,idtsf2);
			generatetsf(current,idin);

		    }
	    }
	    return (cid);
	}
    catch(Throwable e)
	{
	    String errormsg = "Can not build class object";
	    com.rational.testrealtime.Custom.pushError( errormsg );
	    return(0);
	}
}
/**
 * End of a report generation for All the test suite RT.
 */
static public final void lateExit()
  {
    com.rational.testrealtime.TestRT.lateExit();
  }
/**
 * Run of a test suite RT with the declaration of the instances.
 */
private final synchronized void CloseBuffer()
{
    if ( !TestCase.is_test_on )
    {
      com.rational.testrealtime.Buffer.closebufferflag();
    }
}
public final synchronized void run(TestResult result) {

    //In order to have a correct display:
    int iddisplay = InstanceStack.pushInstanceSuite( _trtt_getThis() );
    _trtt_setThis(iddisplay);

    //Open the buffer for the trace output.
      com.rational.testrealtime.Buffer.openbufferflag();
    if ( isFrame() == true )
	{
	    int id_file;
	    String name2 =    "ObjecTest-Tracer";
	    String File =     "File";
	    String Filekind = "report" + new Integer(idtsf1).toString();

	    try {
		TsfMng.writeDB(name2.getBytes( "UTF-8" ));
		TsfMng.writeIN(File.getBytes( "UTF-8" ),++idin,Filekind.getBytes( "UTF-8" ));
		TsfMng.writeSI(idin);
		id_file = idin;
		name2 = toString();
		TsfMng.writeIN( TsfMng.testsuite ,++idin,name2.getBytes( "UTF-8" ));
	    }
	    catch( java.io.UnsupportedEncodingException e )
		{
		    TsfMng.writeDB(StrUTF_8.getBytes());
		    TsfMng.writeIN(StrUTF_8.getBytes(),++idin,StrUTF_8.getBytes());
		    TsfMng.writeSI(idin);
		    id_file = idin;
		    name2 = toString();
		    TsfMng.writeIN( TsfMng.testsuite ,++idin,StrUTF_8.getBytes());
		}

	    setSourceId(idtsf1,idtsf2);
	    settsfid(idin);
	    generatetsf(this,idin);
	    TsfMng.writeSI(id_file);
	    TsfMng.writeIN(TsfMng.testcase,TsfMng.id_testcase,TsfMng.testcase);
	    TsfMng.writeSI(id_file);
	    TsfMng.writeIN(TsfMng.testsuite,TsfMng.id_testsuite,TsfMng.testsuite);
	    TsfMng.writeMS(TsfMng.runtestsuite,TsfMng.id_runtestsuite);
	    TsfMng.writeMS(TsfMng.runtestcase,TsfMng.id_runtestcase);
	    TsfMng.writeID(idtsf1,idtsf2);
	    idtsf1++;
	    TsfMng.writeED();
	}

    com.rational.testrealtime.TimeMng.writeTime();
    com.rational.testrealtime.StackLink __trtt_stack_link2 = new com.rational.testrealtime.StackLink( __trtt_cs_id,TsfMng.id_runtestsuite, gettsfid(), _trtt_getThis(), true );

    writeNoteName();
    CloseBuffer();
    try {

	for (Enumeration e= tests(); e.hasMoreElements(); ) {
	    if (result.shouldStop() )
		break;
            com.rational.testrealtime.Buffer.openbufferflag();
	    com.rational.testrealtime.Trace.changeInstance(_trtt_getThis());
            CloseBuffer();
	    Test test= (Test)e.nextElement();
	    runTest(test,result);
	}

    } finally {

        com.rational.testrealtime.Buffer.openbufferflag();
	InstanceStack.popInstanceSuite();
	__trtt_stack_link2.pop( true ); __trtt_stack_link2=null;
        CloseBuffer();
    }
}

}
























































