
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package j2meunit.framework;

import java.util.*;
public class TestSuitePrim implements Test{
	public Vector fTests= new Vector(10);
	private String fName;

/**
 * TestSuite constructor comment.
 */
public TestSuitePrim() {

}

public TestSuitePrim(Class theClass, String[] testNames) {
	this(theClass.getName());
	for (int i = 0; i < testNames.length; i++) {
		TestCase testCase = null;
		try {
			testCase = (TestCase)theClass.newInstance();
		} catch (Throwable e) {
			System.out.println("Need to have public default constructor in " + theClass.getName());
			throw new RuntimeException("Need to have public default constructor in " + theClass.getName());
		}
		testCase.setTestMethodName(testNames[i]);
		addTest(testCase);
	}
}

public TestSuitePrim(String name) {
	fName = name;
}
/**
 * Adds a test to the suite.
 */
public void addTest(Test test) {
	fTests.addElement(test);
}
/**
 * Counts the number of test cases that will be run by this test.
 */
public int countTestCases() {
	int count = 0;
	for (Enumeration e = tests(); e.hasMoreElements();) {
		Test test = (Test) e.nextElement();
		count = count + test.countTestCases();
	}
	return count;
}
/**
 * Runs the tests and collects their result in a TestResult.
 */
public void run(TestResult result) {
	while (fTests.size() > 0) {
		if (result.shouldStop())
			break;
		((Test) fTests.elementAt(0)).run(result);
		fTests.removeElementAt(0);
	}
}
/**
 * Returns the test at the given index
 */
public Test testAt(int index) {
	return (Test) fTests.elementAt(index);
}
/**
 * Returns the number of tests in this suite
 */
public int testCount() {
	return fTests.size();
}
/**
 * Returns the tests as an enumeration
 */
public Enumeration tests() {
	return fTests.elements();
}

public String toString() {
	if (fName != null)
		return fName;
	return super.toString();
}
/**
 * Returns a test which will fail and log a warning message.
 */
private Test warning(final String message) {
	return new TestCase("warning") {
		protected void runTest() {
			fail(message);
		}
		public Test suite() {
			return null;
		}
	};
}
}
