
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package j2meunit.textui;

import java.util.*;
import java.io.PrintStream;
import com.rational.testrealtime.*;
import j2meunit.framework.*;
import j2meunit.util.*;

/**
 * A command line based tool to run tests.
 * <pre>
 * java test.textui.TestRunner TestCaseClass
 * </pre>
 * TestRunner expects the name of a TestCase class as argument.
 * If this class defines a static <code>suite</code> method it
 * will be invoked and the returned test is run. Otherwise all
 * the methods starting with "test" having no arguments are run.
 * <p>
 * When the wait command line argument is given TestRunner
 * waits until the users types RETURN.
 * <p>
 * TestRunner prints a trace as the tests are executed followed by a
 * summary at the end.
 */
public final class TestRunner implements TestListener, VerifyListener {
	protected PrintStream fWriter;
/**
 * This method was created in VisualAge.
 * @param writer java.io.PrintStream
 */
public TestRunner() {
}
/**
 * This method was created in VisualAge.
 * @param writer java.io.PrintStream
 */
public TestRunner(PrintStream writer) {
	this();
	fWriter= writer;
}
	public synchronized void addError(Test test, Throwable t) {
		writer().print("E");
	}
public synchronized void addFailure(Test test, Throwable t) {
	writer().print("F");
}
public synchronized void addVerifyFailure(Test test) {
  writer().print("V");
}
/**
 * Creates the TestResult to be used for the test run.
 */
protected TestResult createTestResult() {
	return new TestResult();
}
protected void doRun(Test suite) {
	TestResult result = createTestResult();
	result.addListener(this);
	result.addVerifyListener(this);
	long startTime = System.currentTimeMillis();
	suite.run(result);
	long endTime = System.currentTimeMillis();
	long runTime = endTime - startTime;
	writer().println();
	writer().println("Time: " + StringUtil.elapsedTimeAsString(runTime));
	print(result);

	writer().println();

	//if (!result.wasSuccessful())

}
	public void endTest(Test test) {
	}
	private String extractClassName(String className) {
		if(className.startsWith("Default package for"))
			return className.substring(className.lastIndexOf('.')+1);
		return className;
	}
/**
 * main entry point.
 */
public static void main(String args[]) {
	TestRunner aTestRunner = new TestRunner();
	aTestRunner.start(args);
        com.rational.testrealtime.TestRT.lateExit();
}
	/**
	 * Prints failures to the standard output
	 */
	public synchronized void print(TestResult result) {
	    printHeader(result);
       printVerifys(result);
       printErrors(result);
	    printFailures(result);
	}
	/**
	 * Prints the errors to the standard output
	 */
	public void printErrors(TestResult result) {
	    if (result.errorCount() != 0) {
	        if (result.errorCount() == 1)
		        writer().println("There was "+result.errorCount()+" error:");
	        else
		        writer().println("There were "+result.errorCount()+" errors:");

			int i= 1;
			for (Enumeration e= result.errors(); e.hasMoreElements(); i++) {
			    TestFailure failure= (TestFailure)e.nextElement();
				writer().println(i+") "+failure.failedTest());
				writer().println(failure.thrownException().getMessage());
		    }
		}
	}
           /**
         * Prints Verify Failure
         */
	public void printVerifys(TestResult result) {
	    if (result.verifyCount() != 0) {
	        if (result.verifyCount() == 1)
		        writer().println("There was "+result.verifyCount()+" verify failed");
	        else
		        writer().println("There were "+result.verifyCount()+" verify failed");
		}
	}

	/**
	 * Prints failures to the standard output
	 */
	public void printFailures(TestResult result) {
		if (result.failureCount() != 0) {
			if (result.failureCount() == 1)
				writer().println("There was " + result.failureCount() + " failure:");
			else
				writer().println("There were " + result.failureCount() + " failures:");
			int i = 1;
			for (Enumeration e= result.failures(); e.hasMoreElements(); i++) {
				TestFailure failure= (TestFailure) e.nextElement();
				writer().print(i + ") " + failure.failedTest());
				Throwable t= failure.thrownException();
				if (t.getMessage() != null)
					writer().println(" \"" + StringUtil.truncate(t.getMessage(), 80) + "\"");
				else {
					writer().println();
					writer().println(failure.thrownException().getMessage());
				}
			}
		}
	}
	/**
	 * Prints the header of the report
	 */
	public void printHeader(TestResult result) {
		if (result.wasSuccessful()) {
			writer().println();
			writer().print("OK");
			writer().println (" (" + result.runCount() + " tests)");

		} else {
			writer().println();
			writer().println("FAILURES!!!");
	        writer().println("Test Results:");
			writer().println("Run: "+result.runCount()+
  				         ",  Verify failed: "+result.verifyCount()+
				         " Failures: "+result.failureCount()+
				         " Errors: "+result.errorCount());
		}
	}
/**
 * Starts a test run. Analyzes the command line arguments
 * and runs the given test suite.
 */
protected String processArguments(String args[]) {
	String testCase = "";

	for (int i = 0; i < args.length; i++) {
		if (args[i].equals("-c"))
			testCase = StringUtil.extractClassName(args[++i]);
		else if (args[i].equals("-v"))
			System.out.println("J2ME Unit " + Version.id() );
		else
			testCase = args[i];
	}

	if (testCase.equals("")) {
		com.rational.testrealtime.Custom.pushError("Usage: TestRunner testCaseName, where name is the name of the TestCase class");
		return null;
	}

	return testCase;
}
/**
 * Runs a single test and collects its results.
 * This method can be used to start a test run
 * from your program.
 * <pre>
 * public static void main (String[] args) {
 *     test.textui.TestRunner.run(suite());
 * }
 * </pre>
 */
static public void run(Test suite) {
	TestRunner aTestRunner = new TestRunner();
	aTestRunner.doRun(suite);
}
/**
 * Starts a test run. Analyzes the command line arguments
 * and runs the given test suite.
 */
protected void start(String args[]) {
	String testCaseName = processArguments(args);

	try {
		Class testClass = Class.forName(testCaseName);
		TestCase testCase = (TestCase) testClass.newInstance();
		Test suite = testCase.suite();
		doRun(suite);
	}
	catch (Exception e) {
		com.rational.testrealtime.Custom.pushError("Could not create and run test suite");
		return;
	}
}
public synchronized void startTest(Test test) {
	System.out.print(".");
}
private PrintStream writer() {
	if (fWriter == null)
		return System.out;
	return fWriter;
}
}
