

/* 
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*


// Product version: 7.0.5.1


//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2001-2003 Rational Software Corporation. All rights reserved.//
//                                                                            //
//                Product: Rational(R) Test RealTime                          //
//                                                                            //
//      Product Component: All                                                //
//                                                                            //
//               Language: java                                               //
//                                                                            //
//        Product version: 2001A.04.00                                        //
//                                                                            //
//                                                                            //
// Rational, the Rational logo are registered trademarks of                   //
// Rational Software Corporation.                                             //
//                                                                            //
// The placing of the copyright notices on Rational source code is not        //
// intended to mean that they are published works, but are included in the    //
// event of inadvertent publication. The source code of Rational contains     //
// trade secrets and other valuable confidential information of Rational      //
// Software Corporation. Rational Software Corporation and its licensors      //
// retain ownership to Rational Products in both source and object code       //
// format, and related documentation and all rights therein including         //
// copyrights, patents and trade secrets.                                     //
//                                                                            //
//                                                                            //
// U.S. Government Rights                                                     //
//                                                                            //
// Use, duplication, or disclosure by the U.S. Government is subject to       //
// restrictions set forth in the applicable Rational License Agreement        //
// and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS                 //
// 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,       //
// or FAR 52.227-14, as applicable.                                           //
//                                                                            //
// Unless otherwise agreed to in writing, use of Rational products is         //
// governed by the Rational Product License Agreement provided with           //
// the product.                                                               //
//////////////////////////////////////////////////////////////////////////////
*/
/*************************************************************************
*
*
* Module designed to handle and dump the events of the JVM.
*
*
**************************************************************************/
#include "pagent.h"   /*Structure, definition*/
#include "jcustom.h"
/*************************************************************************
 *
 * Storage part.
 * Class identification on load.
 */
/*64 entries*/
#define CARD_ID_1 64
#define MASK_ID_1 0x3F
/*128 entries*/
#define CARD_ID_2 128
#define MASK_ID_2 0x7F
/*256*/
#define CARD_ID_3 256
#define MASK_ID_3 0xFF
/*512*/
#define CARD_ID_4 512
#define MASK_ID_4 0x1FF
/*1024*/
#define CARD_ID_5 1024
#define MASK_ID_5  0x3FC
/*2048*/
#define CARD_ID_6 2048
#define MASK_ID_6 0x7FF
/*4096*/
#define CARD_ID_7 4096
#define MASK_ID_7 0xFFF

static int CARD_CLASS_ID_TABLE = 256;
static int MASK_CLASS_ID_TABLE = 0xFF;
static ClassIdTypeP* ClassIdTable;
/*
 * Storage part.
 * Object identification on load.
 */
static int CARD_OBJECT_ID_TABLE = 1024;
static int MASK_OBJECT_ID_TABLE = 0x3FC;
static AllocObjectsP* ObjectIdTable;

void JVM_exit_on_mem();
void AllocHastables()
{
  ClassIdTable = (ClassIdTypeP*) JNEWARRAY( ClassIdTypeP , CARD_CLASS_ID_TABLE);
  if (ClassIdTable == (ClassIdTypeP*) 0)
    JVM_exit_on_mem();
  ObjectIdTable = (AllocObjectsP*) JNEWARRAY( AllocObjectsP , CARD_OBJECT_ID_TABLE);
  if (ObjectIdTable == (AllocObjectsP*)0)
    JVM_exit_on_mem();
  return;
} 
/*************************************************************************
 *
 * Garbage part.
 *
 * This part is designed to control the call of run gc from the thread
 * which does the export.
 *
 */
static bool should_dump_be_done_after_gc = false;   /*Enable a dump after a gc.             */
static bool dump_at_the_end = false;   /*Enable a dump at the shut down of the JVM.        */
static bool remove_objects_to_be_done   = false;
static bool gc_running = false;
static bool can_do_call_gc = false;
/*Do we have to clean all the deleted objects after a GC start/stop*/
/*At this point all the objects are taged and should be removed    */
/*The remove is then done in the next event.                       */
/*************************************************************************
 *
 * Time part.
 *
 */
static jlong ms_from_begin =0;
/*************************************************************************
 *
 * Dump File.
 *
 */

#ifdef _PAGENT_DEBUG
#define DUMP(str) fprintf(stderr,"%s\n",str);
#define DUMPGC(str) fprintf(stderr,"%s",str);
#endif

#ifndef _PAGENT_DEBUG
#define DUMP(str)
#define DUMPGC(str)
#endif

#define CUSTOM_OUT
#ifdef CUSTOM_OUT
/*Buffer for the outputs*/
static JByteBuffer * jwOut = (JByteBuffer*)0;
#endif
/*************************************************************************
 *
 * Dump Part, to associate information to the methods.
 *
 */
#define _DUMP_METHOD_ENTRY (char)1
#define _DUMP_METHOD_EXIT (char)2
#define _DUMP_WITH_STACK (char)4
#define _JUNIT_TEST (char)8
#define _JUNIT_RUNTEST (char)16
#define _FILTER_METHOD (char)32
#define METHOD_WITH_DUMP_ENTRY(val) (val & _DUMP_METHOD_ENTRY)
#define METHOD_WITH_DUMP_EXIT(val) (val & _DUMP_METHOD_EXIT)
#define METHOD_WITH_STACK(val) (val & _DUMP_WITH_STACK)
#define METHOD_WITH_TEST_JUNIT(val) (val & _JUNIT_TEST)
#define METHOD_WITH_RUNTEST_JUNIT(val) (val & _JUNIT_RUNTEST)
#define METHOD_TO_FILTER(val) (val & _FILTER_METHOD)
/*************************************************************************
 *
 * Snapshot part for the tests.
 *
 * If the mode is not unitesting, only one dump for the PT.
 * This dump will be for the main.
 *
 * If the mode is unitesting,
 * If there is no test loaded the dump will be main + inc.
 * until another test will be loaded.
 * If there is a test loaded, the dump will be class_id, method num, + inc.
 * until another test or the end.
 *
 */
static bool main_has_been_dumped = false; /*Value for the first dump in not unitest mode*/
static bool test_has_been_dumped = false; /*The test has already been dumped            */
static bool first_dump = true;            /*Enable the PU instruction to be dumped      */
static int count_test = 0;                /*Static value to inc the test name, if the same is runned*/
/*
 * Has the test to be declared before the dump of the snapshot.
 */
bool has_test_to_be_declared()
{
  if ( is_mode_unitest )
    {
      return ( !test_has_been_dumped );
    }
  else
    {
      return ( !main_has_been_dumped );
    }
}
/*************************************************************************
 *
 * Test part.
 * Number used for the test.
 */
static jobjectID class_id_for_the_current_test = (jobjectID)0;
static ClassIdType * class_id_arrow_for_the_current_test = (ClassIdType*)0;
static MethodIdType *  method_id_for_the_current_test = (MethodIdType*)0;
/*************************************************************************
 *
 * Mode filter.
 *
 */
static bool mode_filter_on = false;
/*
 * Check in a stack that the test method is the first one.
 */
bool Is_other_test_method(StackTypeP stack)
{
  bool other_method = false;
  bool under_runbare = false;
  if (stack != (StackTypeP)0)
    {
      /*Search in the stack if there is another test method under a runBare.*/
      StackTypeP current = stack->up;
      /*Search if there is another test method*/
      while (current != (StackTypeP)0)
	{
	  if ( METHOD_WITH_TEST_JUNIT(current->mid->dump)!=0 )
	    {
	      other_method = true;
	      break;
	    }
	  current = current->up;
	}
      /*And if it is under a runBare*/
      while (current != (StackTypeP)0)
	{
	  if ( METHOD_WITH_RUNTEST_JUNIT(current->mid->dump)!=0 )
	    {
	      under_runbare = true;
	      break;
	    }
	  current = current->up;
	}
    }
  else
    {
      return (false);
    }

  return ( (other_method==true)&&(under_runbare==true) );

}
/*************************************************************************
 *
 * Command line part.
 *
 */
MethodToDump * CmdCreateMtoD()
{
  MethodToDump * current;
  current = JNEW(MethodToDump);
  if (current != (MethodToDump*)0)
    {
      current->name_class = (char*)0;
      current->name_method = (char*)0;
      current->on_entry = false;
      current->is_for_dump = false;
      current->only_to_observe = false;
      current->package_to_observe = false;
      current->with_object_stack_observation = false;
      current->next = (MethodToDump*)0;
    }
  return(current);
}
/*
 * Add A Struct.
 */
void CmdAddStructMTD(MethodToDump * method)
{
  MethodToDump * current;
  if ( DumpSet == (MethodToDump*)0)
    {
      DumpSet = method;
    }
  else
    {
      current = DumpSet;
      while ((current != ((MethodToDump*)0)) &&(current->next != ((MethodToDump*)0)) )
	{
	  current = current->next;
	}
      /*We are on the last elt.*/
      current->next = method;
    }

  return;
}
/*
 * Delete the set after use.
 * And all the variables associated with the command line.
 */
void CmdDeleteSet()
{
  MethodToDump * current, * prev;
  prev = (MethodToDump*)0;
  current = DumpSet;

  if (port_sock !=  ( (char*) 0) )
    {
      JDELETE(port_sock);
    }

  if (ATLOUT !=  ( (char*) 0) )
    {
      JDELETE(ATLOUT);
    }


  if (host !=  ( (char*) 0) )
    {
      JDELETE(host);
    }


  if (name_user !=  ( (char*) 0) )
    {
      JDELETE(name_user);
    }

  if (node_used !=  ( (char*) 0) )
    {
      JDELETE(node_used);
    }

  if (date_run !=  ( (char*) 0) )
    {
      JDELETE(date_run);
    }

  while(current != (MethodToDump*)0)
    {
      prev = current;
      current = current->next;
      if (prev->name_method != (char*)0)
	JDELETE(prev->name_method);
      if (prev->name_class != (char*)0)
	JDELETE(prev->name_class);
      JDELETE(prev);
    }
  DumpSet = (MethodToDump*)0;
  return;
}
/*
 * CMD LINE IS:
 * -JVM           Start with the JVM and Native JNI information.
 *                (Before the JVM INIT)
 *
 * -GLOBAL        All the information are stored, whatever the thread it comes from.
 * -THREAD        All the information are stored by thread env.
 *
 * -DELETE_OBJECTS_WITH_SNAPSHOT  The objects still referenced are not reseted with an entry call, but with a snapshot.
 * -DELETE_OBJECTS_NEVER          The objects still referenced are never reseted.
 *
 * -TRY_GC_BEFORE_SNAPSHOT         Call the gc before a dump.
 *
 * -UNITESTING_MODE               Find the test method specific calls.
 *
 * -DUMPMETHODS:[method_name,,class_name,0/1 dump on exit or entry],[...],[...]
 * All these methods will be used as dump call.
 *
 * -OBSERVE_METHODS_WITH_STACK:[method_name],class_name,[...],[...]
 * All these methods will have there objects with a stack call.
 *
 * -NAME_USER:"name_user"
 *
 * -DATE_START:"date_start"
 *
 * ...
 */
#define WITH_JVM (char*)"-JVM"
#define WITH_GLOBAL_INFO (char*)"-GLOBAL"
#define WITH_THREAD_INFO (char*)"-THREAD"
#define WITH_DELETE_WITH_SNAPSHOT   (char*)"-D_O_W_S"
#define WITH_DELETE_OBJECTS_NEVER   (char*)"-D_O_N"
#define WITH_TRY_GC_BEFORE_SNAPSHOT (char*)"-T_GC"
#define WITH_UNITEST_MODE (char*)"-U_T"
#define WITH_METHOD_TO_DUMP (char*)"-D_M"
#define WITH_METHOD_TO_FILTER (char*)"-F_M"
#define WITH_PACKAGE_TO_FILTER (char*)"-P_M"
#define WITH_METHOD_TO_OBSERVE (char*)"-O_M"
#define WITH_NAME_USER (char*)"-U_S"
#define WITH_DATE_USED (char*)"-D_U"
#define WITH_NODE_USED (char*)"-N_U"
#define WITHOUT_OBJ (char*)"-N_O"
#define WITH_PORT (char*)"-P_T"
#define WITH_OUT_FILE (char*)"-OUT"
#define WITH_DUMP_AFTER_GC (char*)"-D_GC"
#define WITH_DUMP_PARTIAL (char*) "-D_PGC"
#define WITH_HOST_NAME (char*)"-H_N"
#define WITH_DUMP_END (char*)"-D_E"

#define WITH_HASH_CLASS_1 (char*)"-H_C1"
#define WITH_HASH_CLASS_3 (char*)"-H_C3"
#define WITH_HASH_CLASS_5 (char*)"-H_C5"
#define WITH_HASH_CLASS_7 (char*)"-H_C7"

#define WITH_HASH_OBJECT_1 (char*)"-H_O1"
#define WITH_HASH_OBJECT_3 (char*)"-H_O3"
#define WITH_HASH_OBJECT_5 (char*)"-H_O5"
#define WITH_HASH_OBJECT_7 (char*)"-H_O7"
/*
 *
 */
char * Cmd_ExtractStr(char * cmd_line, char* what)
{
  char * ret = (char*)0;
  char * start  =  _STRSTR(cmd_line,what);
  if (start != (char*)0)
    {
      start = _STRSTR(cmd_line,"[");
      if (start != (char*)0)
	{
	  start++;
	  char * end = _STRSTR(start,"]");
	  if (end != (char*)0)
	    {
	      *end = '\0';
	      ret = (char*) _STRDUP(start);
	    }
	}
    }
  return (ret);
}
/*
 *
 */
char * Cmd_ExtractStrName(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);
  char * start = _STRSTR(cp,WITH_NAME_USER);
  ret = Cmd_ExtractStr(start,WITH_NAME_USER);
  JDELETE(cp);
  return (ret);
}
/*
 *
 */
char * Cmd_ExtractStrPort(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);
  char * start = _STRSTR(cp,WITH_PORT);
  ret = Cmd_ExtractStr(start,WITH_PORT);
  JDELETE(cp);
  return (ret);
}
/*
 *
 */
char * Cmd_ExtractStrHostName(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);
  char * start = _STRSTR(cp,WITH_HOST_NAME);
  ret = Cmd_ExtractStr(start,WITH_HOST_NAME);
  JDELETE(cp);
  return (ret);
}

/*
 *
 */
char * Cmd_ExtractStrDate(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);
  char * start = _STRSTR(cp,WITH_DATE_USED);
  ret = Cmd_ExtractStr(start,WITH_DATE_USED);
  JDELETE(cp);
  return (ret);
}
/*
 *
 */
char * Cmd_ExtractStrNodeUsed(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);

  char * start = _STRSTR(cp,WITH_NODE_USED);
  ret = Cmd_ExtractStr(start,WITH_NODE_USED);
  JDELETE(cp);
  return (ret);
}
/*
 *
 */
char * Cmd_ExtractStrOutFile(char * cmd_line)
{
  char * ret = (char*)0;
  char * cp = (char*)_STRDUP(cmd_line);
  if (cp == (char*)0)
    return ((char*)0);

  char * start = _STRSTR(cp,WITH_OUT_FILE);
  ret = Cmd_ExtractStr(start,WITH_OUT_FILE);
  JDELETE(cp);
  return (ret);
}
/*
 *
 */
void CmdInitHastable(char * cmd_line)
{

  if (_STRSTR( cmd_line, WITH_HASH_CLASS_1 ) != (char*)0)
    {
      CARD_CLASS_ID_TABLE = CARD_ID_1;
      MASK_CLASS_ID_TABLE = MASK_ID_1;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_CLASS_3 ) != (char*)0)
    {
      CARD_CLASS_ID_TABLE = CARD_ID_3;
      MASK_CLASS_ID_TABLE = MASK_ID_3;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_CLASS_5 ) != (char*)0)
    {
      CARD_CLASS_ID_TABLE = CARD_ID_5;
      MASK_CLASS_ID_TABLE = MASK_ID_5;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_CLASS_7 ) != (char*)0)
    {
      CARD_CLASS_ID_TABLE = CARD_ID_7;
      MASK_CLASS_ID_TABLE = MASK_ID_7;
    }

  if (_STRSTR( cmd_line, WITH_HASH_OBJECT_1 ) != (char*)0)
    {
      CARD_OBJECT_ID_TABLE = CARD_ID_1;
      MASK_OBJECT_ID_TABLE = MASK_ID_1;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_OBJECT_3 ) != (char*)0)
    {
      CARD_OBJECT_ID_TABLE = CARD_ID_3;
      MASK_OBJECT_ID_TABLE = MASK_ID_3;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_OBJECT_5 ) != (char*)0)
    {
      CARD_OBJECT_ID_TABLE = CARD_ID_5;
      MASK_OBJECT_ID_TABLE = MASK_ID_5;
    }
  else if (_STRSTR( cmd_line, WITH_HASH_OBJECT_7 ) != (char*)0)
    {
      CARD_OBJECT_ID_TABLE = CARD_ID_7;
      MASK_OBJECT_ID_TABLE = MASK_ID_7;
    }

  return;
 
}
/*
 *
 */
void CmdUpdateVariables(char* cmd_line)
{

  CmdInitHastable(cmd_line);

  DUMPGC(cmd_line);


  if (_STRSTR( cmd_line, WITH_DUMP_PARTIAL ) != (char*)0)
    {
      set_partial_dump();
    }

  if (_STRSTR( cmd_line, WITHOUT_OBJ ) != (char*)0)
    {
      set_no_obj_allocation();
    }

  if (_STRSTR( cmd_line, WITH_JVM ) != (char*)0)
    {
      set_obs_from_begin();
    }

  if (_STRSTR( cmd_line, WITH_GLOBAL_INFO ) != (char*)0)
    {
      set_global_datas();
    }

  if (_STRSTR( cmd_line, WITH_DUMP_END ) != (char*)0)
    {
      dump_at_the_end = true;
    }


  if (_STRSTR( cmd_line, WITH_THREAD_INFO ) != (char*)0)
    {
      /*env_global_datas = false;*/
      /* Usual configuration */
    }

  if (_STRSTR( cmd_line, WITH_DELETE_WITH_SNAPSHOT ) != (char*)0)
    {
      set_snap_delete(); 
      set_objects_stay();
    }

  if (_STRSTR( cmd_line, WITH_DELETE_OBJECTS_NEVER ) != (char*)0)
    {
      /*should_snapshot_delete_objects = false;*/
      /*Basic configuration*/
      set_objects_stay(); 
    }

  if (_STRSTR( cmd_line, WITH_TRY_GC_BEFORE_SNAPSHOT ) != (char*)0)
    {
      set_gc();
    }

  if (_STRSTR( cmd_line, WITH_DUMP_AFTER_GC ) != (char*)0)
    {
      set_dump_after_gc();
    }

  if (_STRSTR( cmd_line, WITH_OUT_FILE ) != (char*)0)
    {
      set_out_file( Cmd_ExtractStrOutFile(cmd_line) );
    }

  if (_STRSTR( cmd_line, WITH_PORT ) != (char*)0)
    {
      set_port( Cmd_ExtractStrPort(cmd_line) );
    }

  if (_STRSTR( cmd_line, WITH_HOST_NAME ) != (char*)0)
    {
      set_hostname( Cmd_ExtractStrHostName(cmd_line) );
    }
 
  if (_STRSTR( cmd_line, WITH_UNITEST_MODE ) != (char*)0)
    {
      set_unitest();
    }

  if (_STRSTR( cmd_line, WITH_NAME_USER ) != (char*)0)
    {
      set_name_user( Cmd_ExtractStrName(cmd_line)   );
    }

  if (_STRSTR( cmd_line, WITH_DATE_USED ) != (char*)0)
    {
      set_date_start( Cmd_ExtractStrDate(cmd_line) );
    }

  if (_STRSTR( cmd_line, WITH_NODE_USED ) != (char*)0)
    {
      set_node_used( Cmd_ExtractStrNodeUsed(cmd_line) );
    }
  return;
}
/*
 *
 */
MethodToDump * CmdExtractStruct( char * cmd_line)
{
  /*Create A struct and run ...*/
  if (cmd_line != (char*)0 )
  {
    MethodToDump * current = CmdCreateMtoD();
    if (current == (MethodToDump*)0)
      return (current);

    /*Name of the method*/
    char * name = (char*)0;
    char * start = cmd_line;
    char * end = _STRSTR(cmd_line,",");
    if (end != (char*)0)
      {
	*end = '\0';
	name  =(char*)_STRDUP( start );
	start = ++end;
      }
    else
      {
	return ((MethodToDump*)0);
      }
    /*Class of the method*/
    char * class_name = (char*)0;
    end = _STRSTR(start,",");
    if (end != (char*)0)
      {
	*end = '\0';
	class_name  = (char*)_STRDUP( start );
	start = ++end;
      }
    else
      {
	return ((MethodToDump*)0);
      }
    /*on entry*/
    char * on_entry = (char*)0;
    bool entry = false;
    on_entry  =(char*)_STRDUP( start );
    if (on_entry != (char*)0)
      {
	entry = (*on_entry == '1');
	JDELETE(on_entry);
      }
    else
      {
	return ((MethodToDump*)0);
      }

    current->name_method = name;
    current->name_class = class_name;

    current->is_for_dump = true;
    current->on_entry = entry;
    return(current);
    /*Create and upload*/
  }
  return ((MethodToDump*)0);
}
/*
 * Method for observation.
 */
MethodToDump * CmdExtractStructObservation( char * cmd_line)
{
  /*Create A struct and run ...*/
  if (cmd_line != (char*)0)
  {
    MethodToDump * current = CmdCreateMtoD();
    if (current == (MethodToDump*)0)
      return (current);

    /*Name of the method*/
    char * name = (char*)0;
    char * start = cmd_line;
    char * end = _STRSTR(cmd_line,",");
    if (end != (char*)0)
      {
	*end = '\0';
	name  =(char*)_STRDUP( start );
	start = ++end;
      }
    else
      {
	return ((MethodToDump*)0);
      }

    /*Class of the method*/
    char * class_name = (char*)0;
    class_name  =(char*)_STRDUP( start );

    current->name_method = name;
    current->name_class = class_name;
    current->with_object_stack_observation = true;
    return(current);
    /*Create and upload*/
  }
  return ((MethodToDump*)0);
}
/*
 *
 */
MethodToDump * CmdExtractStructFilter( char * cmd_line)
{
  /*Create A struct and run ...*/
  if (cmd_line)
  {
    MethodToDump * current = CmdCreateMtoD();
    if (current == (MethodToDump*)0)
      return (current);

    /*Name of the method*/
    char * name = (char*)0;
    char * start = cmd_line;
    char * end = _STRSTR(cmd_line,",");
    if (end != (char*)0)
      {
	*end = '\0';
	if (end == start)
	  {
	    name  =(char*)_STRDUP((char*)"any");

	  }
	  else
	    {
	      name  =(char*)_STRDUP( start );
	    }
	start = ++end;
      }
    else
      {
	return ((MethodToDump*)0);
      }

    /*Class of the method*/
    char * class_name = (char*)0;
    class_name  =(char*)_STRDUP( start );

    current->name_method = name;
    current->name_class = class_name;

    current->only_to_observe = true;
    /*Mode filter on is confirmed*/
    mode_filter_on = true;
    return(current);
    /*Create and upload*/
  }
  return ((MethodToDump*)0);
}
/*
 *
 */
MethodToDump * CmdExtractStructFilterPackage( char * cmd_line)
{
  /*Create A struct and run ...*/
  if (cmd_line)
  {
    MethodToDump * current = CmdCreateMtoD();
    if (current == (MethodToDump*)0)
      return (current);

    /*Name of the method*/
    char * name = (char*)0;
    char * start = cmd_line;
    char * end = _STRSTR(cmd_line,",");
    if (end != (char*)0)
      {
	*end = '\0';
	if (end == start)
	  {
	    name  =(char*)_STRDUP((char*)"any");

	  }
	  else
	    {
	      name  =(char*)_STRDUP( start );
	    }
	start = ++end;
      }
    else
      {
	return ((MethodToDump*)0);
      }

    /*Class of the method*/
    char * class_name = (char*)0;
    class_name  =(char*)_STRDUP( start );

    current->name_method = name;
    current->name_class = class_name;

    current->package_to_observe = true;
    current->only_to_observe = true;
    /*Mode filter on is confirmed*/
    mode_filter_on = true;



    return(current);
    /*Create and upload*/
  }
  return ((MethodToDump*)0);
}
/*
 *
 */
#define _KIND_OBSERVATION (int)1
#define _KIND_DUMP (int)2
#define _KIND_FILTER (int)3
#define _KIND_PACKAGE (int)4
void CmdLoadStructFor(char * cmd_line,int kind)
{
  /*Char* is at the beginning*/
  /*Search the [ and later ] */
  /*While not ]]             */

  /*Search the first value*/

  char * start = _STRSTR(cmd_line,"[");
  bool bend = false;
  if (start != (char*)0)
    {
      while ( ! bend )
	{
	  /*Get the value after [ and , */
	  /*With the value after , and ]*/
	  /*End is ]] at the end*/
	  start = _STRSTR(start,"[");
	  if (start == (char*)0)
	    break;

	  start++;

	  char * end = _STRSTR(start,"]");
	  if (end == (char*)0)
	    break;

	  *end = '\0';

	  if (end++ == '\0')
	    bend = true;

	  /*Load the struct vals*/
	  MethodToDump * new_method = (MethodToDump*)0;;
	  switch(kind)
	    {
	    case _KIND_PACKAGE:
	      new_method = CmdExtractStructFilterPackage(start);
	      break;
	    case _KIND_OBSERVATION:
	      new_method = CmdExtractStructObservation(start);
	      break;
	    case _KIND_DUMP:
	      new_method = CmdExtractStruct(start);
	      break;
	    case _KIND_FILTER:
	      new_method = CmdExtractStructFilter(start);
	      break;
	    }
	  if (new_method != (MethodToDump*)0)
	    {
	      CmdAddStructMTD(new_method);
	    }

	  start = end;
	}
    }
  return;
}
/*
 *
 */
int CmdLoadStructInfo(char * cmd_line,char * start_val, int kind_extract)
{
  char * cp = (char*)_STRDUP(cmd_line);
  char *start = _STRSTR(cp,start_val);
  if (start != (char*)0){
    start = _STRSTR(start,"[[");
    if (start!=(char*)0)
      {start++;}
    else
      {
	if (cp!= (char*)0)
	  JDELETE(cp);
	return(0);
      }
    /*Start to extract the values*/
    char * end = _STRSTR(start,"]]");
    if (end != (char*) 0)
      {*(++end) = '\0';}
    else
      {
	if (cp!= (char*)0)
	  JDELETE(cp);
	return(0);
      }

    CmdLoadStructFor( start, kind_extract );
    if (cp!= (char*)0)
      JDELETE(cp);
    return(1);
  }
  else
    {
      if (cp!= (char*)0)
	JDELETE(cp);
      return(1);
    }
}
/*
 *
 */
int CmdTreatCmdLine(char * cmd_line)
{
  if (cmd_line != (char*)0)
    {
      /*Global values*/
      CmdUpdateVariables(cmd_line);
      if (CmdLoadStructInfo(cmd_line,WITH_METHOD_TO_DUMP,_KIND_DUMP)==0)
	return(0);
      if (CmdLoadStructInfo(cmd_line,WITH_METHOD_TO_OBSERVE,_KIND_OBSERVATION)==0)
	return(0);
      if (CmdLoadStructInfo(cmd_line,WITH_METHOD_TO_FILTER,_KIND_FILTER)==0)
	return(0);
      if (CmdLoadStructInfo(cmd_line,WITH_PACKAGE_TO_FILTER,_KIND_PACKAGE)==0)
	return(0);
    }
  else
    {
      /*Set the usual values*/
    }
  return (1);
}
/*
 * Function designed to tag all the classes observed.
 */

/*
 * Part for J9 representation of data.
 */
char * updatename(char * name)
{
  char * cur;
  if (name)
    {
      for (cur = name;*cur;cur++)
	{
	  if (*cur == '/')
	    *cur = '.';
	}
      
      return name;
    }
  else
  {
    return ((char*)0);
  }
}
#define _TRY_METHOD_TO_DUMP 1
#define _TRY_METHOD_TO_OBSERVE 2
#define _TRY_METHOD_TO_FILTER 3
MethodToDump * Cmd_is_method_for(const char* name_c, const char* name_m, int kind)
{

  if ( (name_c == (char*)0) || (name_m == (char*)0) )
    {
      return ((MethodToDump*)0);
    }

  char * name_class = _STRDUP((char*)name_c);
  char * name_method = _STRDUP((char*)name_m);
  
  if ( (name_class == (char*)0) || (name_method == (char*)0) )
    {
      return ((MethodToDump*)0);
    }

  name_class = updatename(name_class);
  name_method = updatename(name_method);

  MethodToDump * current = DumpSet;

  bool go_on = false;

  while ( current != ((MethodToDump*)0))
    {
      if (current->name_class != (char*)0)
	{
	  if (kind == _TRY_METHOD_TO_FILTER)
	    {
	      if (current->package_to_observe)
		{
		  if ( _STRSTR( name_class, current->name_class ) != 0 )

		    {
		      go_on = true;
		    }
		}
	      else
		{
		  
		  if ( _STRCMP( name_class, current->name_class ) == 0 )
		    {
		      go_on = true;
		    }
		}
	    }
	  else
	    {
	      if ( _STRCMP( name_class, current->name_class ) == 0 )
		{
		  go_on = true;
		}
	    }
	  
	  if ( go_on &&  (current->name_method != (char*)0))
	    {
	      switch(kind)
		{
		case _TRY_METHOD_TO_DUMP:
		  if ( _STRCMP( name_method, current->name_method ) == 0 )
		    {
		      if ( current->is_for_dump)
			{
			  if (name_class != (char*)0)
			    JDELETE(name_class);
			  if (name_method != (char*)0)
			    JDELETE(name_method);
			  
			  return (current);
			}
		    }
		  break;
		case _TRY_METHOD_TO_OBSERVE: 
		  if ( _STRCMP( name_method, current->name_method ) == 0 )
		    {
		      if ( current->with_object_stack_observation)
			{
			  if (name_class != (char*)0)
			    JDELETE(name_class);
			  if (name_method != (char*)0)
			    JDELETE(name_method);
			  
			  return (current);
			  
			}
		    }
		  break;
		case _TRY_METHOD_TO_FILTER: 
		  if ( ( _STRCMP("any",current->name_method)==0) || (_STRCMP( name_method, current->name_method ) == 0) )
		    if ( current->only_to_observe)
		      {
			if (name_class != (char*)0)
			  JDELETE(name_class);
			if (name_method != (char*)0)
			  JDELETE(name_method);
			
			return (current);
		      }
		  break;
		}
	    }
	}

      
      current = current->next;
      go_on = false;
    }

  if (name_class != (char*)0)
    JDELETE(name_class);
  if (name_method != (char*)0)
    JDELETE(name_method);

  return ( (MethodToDump*)0);
}
/*
 * Will tag the method.
 * The condition for having a Junit test will still be:
 * Be in Unitest mode.
 * Have a runTest in the stack.
 * Have the call of this kind of method.
 * The name of the test will be the name of the test method.
 */
bool Cmd_is_test_junit(const char * name_method)
{
  return (_STRNCMP(name_method,"test",4) == 0);
}
/*
 *
 */
bool Cmd_is_runtest_junit(const char * name_method)
{
  return ( _STRNCMP(name_method,"runBare",7) == 0 );
}
/*************************************************************************
 *
 * External and global information (Settings).
 *
 */
static bool env_is_on = false;         /*Is the profiler on */
/*************************************************************************
 * JVMPI INTERFACE.
 */
JVMPI_Interface *JvmpiInterface;
/*
 *
 */
void JVM_exit_on_mem()
{
  _JERROR((char*)"ERROR: Not enought memory");
  JvmpiInterface->ProfilerExit( JNI_ERR );
  return;
}
/*
 * Lock on the Raw Monitor.
 */
static JVMPI_RawMonitor GC_Call;   /*Lock on data access*/
/*************************************************************************
 * Call back thread.
 */
void dumpInformation(JNIEnv * env_id);
static bool dump_required_from_pagent = false;
#ifndef JVMPI_SOCKET_JVMPI_NONE
static JVMPI_SOCKET_ID sock_fd = 0;
static char received[LENGTH_SOCKET_BUF];
void RemoveObjForAllClasses();

static void pagent_callback(void *p)
{
  int received_ = 0;

  _STRCPY(received,(char*)"TRY");

  if (port_sock == (char*) 0)
    {
      DUMPGC("EXIT AGENT CALLBACK");
      return;
    }

  DUMPGC("TRY CONNECT");

  sock_fd = CONNECT_TO_SERVER(host,port_sock);
  if (sock_fd == 0)
    {
      /*Error is already dumped*/
      JvmpiInterface->ProfilerExit( JNI_ERR );
      return;
    }

  DUMPGC("CONNECTED");


  while (env_is_on)
  {
    if (sock_fd != 0)
      {
	if ( ! gc_running )
	  {
	    
	    bool go = false;

	    while (!go)	    
	      {
		/*Wait for a garbage running*/
		if (!gc_running && (env_is_on))
		  {
		    /*Blocking socket*/
		    received_ = RECEIVE_FROM_SERVER(sock_fd,received,LENGTH_SOCKET_BUF/*,0*/);
		    
		    if (!env_is_on)
		      {
			DUMPGC("CLOSE SOCK");
			CLOSESOCK(sock_fd);
			sock_fd = 0;
			return;
		      }
		    
		    
		    if ((received_ == -1)&&(sock_fd==0))
		      {
			return;
		      }
		    
		    if ((sock_fd==0))
		      {
			return;
		      }
		    
		    if (received_ != -1)
		      {
			
			DUMPGC("TEST");
			if ( _STRNCMP( received, (char*) "DUMP",4 ) == 0 )
			  {
			    DUMPGC("OK");
			    go = true;
			    dump_required_from_pagent = true;
			  }
		      }
		  }
		
		if ((received_>0) && (!gc_running))
		  {
		    /*Warning on lock*/
		    /*Now lock the threads*/
		    /*But not the GC*/
		    if (
			(do_the_gc == true)&&
			(can_do_call_gc == true)
			 )/*Gc is required before the dump*/
			JvmpiInterface->RunGC();

		    JvmpiInterface->RawMonitorEnter( GC_Call );/*Lock the threads*/

		    /*If it has not already been done by another thread*/
		    if (remove_objects_to_be_done)
		      {
			/*Will remove all the deleted objects after the GC start/stop.*/
			/*Has also to be done here in case of loop with a call        */
			RemoveObjForAllClasses();
		      }
		    
		    
		    if (is_dump_partial_on_gc)
		      {
			if (should_dump_be_done_after_gc)
			  {
			    should_dump_be_done_after_gc = false;/*Already done*/
			    force_dump_after_gc = false;
			    dumpInformation( (JNIEnv *) 0 );
			    DUMPGC("DUMP FROM THREAD");
			  }
			else
			  {
			    if (dump_required_from_pagent)
			      {
				force_dump_after_gc = true;
				DUMPGC("FORCE");
			      }
			  }
		      }
		    else
		      {
			/*If there is no thread to do the job   */
			/*Between the RECV and the enter of Lock*/
			if (dump_required_from_pagent)
			  {
			    should_dump_be_done_after_gc = false;/*Already done*/
			    dumpInformation( (JNIEnv *) 0 );
			    DUMPGC("DUMP IMMEDIATLY");
			  }
		      }
		    
		    if (sock_fd!=0)
		      SEND(sock_fd,(char*)"GCE.......",10);
		    
		    dump_required_from_pagent = false;
		    JvmpiInterface->RawMonitorExit( GC_Call );
		  }
	      }
	  }
      }
  }
  
  return;
}
#endif
/*
 * VM STATUS control.
 */
VMStatusType VMStatus;
/*
 * Defines to return before the initialization of the JVM.
 */
#define CHECK_VMSTATUS_METHOD if ((!is_obs_from_begin) && ( VMStatus != VMStatus_OK )) return;
#define CHECK_END if (( VMStatus == VMStatus_AFTER_SHUTDOWN )) return;
/*************************************************************************
 *
 * OBJECT ID.
 *
 */
void EnterObjId ( AllocObjectsP cid )
{
  if (cid == (AllocObjectsP)0 )
    return;
  
  AllocObjectsP cur, prev;
  int i = (((unsigned long)(cid->obj_id))>>3) & MASK_OBJECT_ID_TABLE;
  
  cid->hash_next = (AllocObjectsP)0;

  for ( cur = ObjectIdTable[i], prev = (AllocObjectsP)0;
        (cur && (cur->obj_id < cid->obj_id));
        prev = cur, cur = cur->hash_next );
  
  if ( ! cur )
    {
      if ( ! prev )
	{
	  ObjectIdTable[i] = cid;
	}
      else
	{
	  prev->hash_next = cid;
	}
    }
  else
    {

      cid->hash_next = cur;

      if ( prev )
	{
	  prev->hash_next = cid;
	}
      else
	{
	  ObjectIdTable[i] = cid;
	}
    }
  return;
}
/*
 * Delete Associated.
 */
void RemoveObjId( AllocObjectsP cid )
{
  if (cid == (AllocObjectsP) 0)
    return;
  /*Find it back*/
  /*And reset the connections*/
  AllocObjectsP cur, prev;
  int i = (((unsigned long)(cid->obj_id))>>3) & MASK_OBJECT_ID_TABLE;
  prev = (AllocObjectsP)0;
  
  for ( cur = ObjectIdTable[i];
        cur && (cur->obj_id < cid->obj_id);
        prev = cur, cur = cur->hash_next );
  
  if ( (! cur) || (cur->obj_id != cid->obj_id) )
    {
      return;
    }
  else
    {
      /*Do the remove operation.*/
      if (prev == (AllocObjectsP)0)
   	{
	  /*First item*/
	  ObjectIdTable[i] = cur->hash_next;
	}
      else
	{
	  prev->hash_next = cur->hash_next;
	  /*Delete is done by the methods*/
	}
    }
  return;
}
/*
 *
 */
AllocObjectsP FindObjId( jobjectID obj_id )
{
  AllocObjectsP cur =0;
  int i = (((unsigned long)(obj_id))>>3) & MASK_OBJECT_ID_TABLE;

  for ( cur = ObjectIdTable[i];
        cur && (cur->obj_id < obj_id);
        cur = cur->hash_next );

  if ( (! cur) || (cur->obj_id != obj_id) )
    {
      /*Obj id are not found in they have already been deleted*/
      return ((AllocObjectsP)0) ;
    }
  return cur;
}
/*
 * Do not do that for the classes.
 * Some objects can still referenced this class.
 *
 */
void FreeObjId( jobjectID obj_id )
{
  AllocObjectsP cur =0;
  cur = FindObjId( obj_id );
  /*If there is such an object in the table*/
  if (cur != (AllocObjectsP)0 )
    {
      cur->is_deleted = true;
      RemoveObjId( cur );
    }
  return;
}
/*
 * Reset in case of class unload.
 *
 */
void ResetObjectClassLink( ClassIdTypeP class_id )
{
  AllocObjects * cur, *prev;
  for (int i = 0; i< CARD_OBJECT_ID_TABLE; i++)
    {
      prev = (AllocObjects *)0;
      cur = ObjectIdTable[i];
      while (cur)
	{
	  if (cur->class_id == class_id)
	    {
	      cur->class_id = ((ClassIdTypeP) 0);/*Set as deleted*/
	    }

	  cur = cur->hash_next;
	  
	}
    }
  return;
}
/*
 * Arena part.
 */
void FreeObjectForArena( jint arenaval )
{
  /*Scan all the object to set them as deleted*/
  AllocObjects * cur, *prev;
  int i;
  for ( i = 0; i< CARD_OBJECT_ID_TABLE; i++)
    {
      prev = (AllocObjects *)0;
      cur = ObjectIdTable[i];
      while (cur)
	{
	  if (cur->arena_id == arenaval)
	    {
	      cur->is_deleted = true;/*Set as deleted*/
	    }
	  cur = cur->hash_next;
	}
    }
  /*And remove the objects*/
  for ( i = 0; i< CARD_OBJECT_ID_TABLE; i++)
    {
      prev = (AllocObjects *)0;
      cur = ObjectIdTable[i];
      while (cur)
	{
	  if (cur->arena_id == arenaval)
	    {
	      
	      /*Do the remove operation.*/
	      if (prev == (AllocObjectsP)0)
		{
		  /*First item*/
		  ObjectIdTable[i] = cur->hash_next;
		}
	      else
		{
		  /*update teh last not deleted*/
		  prev->hash_next = cur->hash_next;
		  /*Delete is done by the methods or the call after the GC FINISH*/
		  /*Not now*/
		}
	      cur = cur->hash_next;
	    }
	  else
	    {
	      prev = cur;
	      cur = cur->hash_next;
	    }
	}
    }
  return;
}
/*
 * RemoveAllDeletedObjects.
 * Scan all the repository for the mid involved.
 */
void RemoveDeletedObjectInformation(AllocSum * for_mid)
{

  if (for_mid == (AllocSum*)0)
    return;

  AllocObjects * current, *prev;
  AllocObjects * head, *curhead;

  head = (AllocObjects*)0;
  curhead = (AllocObjects*)0;

  bool head_found = false;
  
  if (for_mid->start != (AllocObjects*)0)
    {
      current = for_mid->start;
      prev = for_mid->start;

      while (current != (AllocObjects*)0)
	{
	  prev = current;
	  current = current->next;

	  if ( ! prev->is_deleted )
	    { 
	      if ( head_found)
		{
		   curhead->next = prev;
		   curhead = prev;
		} 
	      else
		{
		   head_found = true;
		   head = prev;
		    curhead = prev;
		 }
	    }
	  else
	    { 
	      /*And free the object*/
	      if (prev->info_stack) JDELETE(prev->info_stack);
	      JDELETE(prev);
	   } 
	}
      /*Update the informations*/
      if (curhead)
	curhead->next = (AllocObjects*)0;
      for_mid->start = head;
      for_mid->last = curhead;
    }
  return;
}
/*
 *
 */
void RemoveAllDeletedObjForAllocSum(AllocSum * for_mid)
{
  AllocSum * current;
  current = for_mid;
  while(current != (AllocSum*)0)
    {
      RemoveDeletedObjectInformation(current);
      current = current->next;
    }  
  return;
}
/*
 *
 */
void RemoveDeletedObjectsForAllTheMid(struct MethodIdType* methods, jint num_methods)
{
  struct MethodIdType * current = methods;
  for (int i = 0; i < num_methods; i++)
    {
      if (current != (struct MethodIdType*) 0)
	{
	  RemoveAllDeletedObjForAllocSum(&(current->alloc));
	  current++;
	}
    }
  return;
}
/*
 * Scan all the repository for all the classes.
 */
void RemoveObjForAllClasses()
{
  ClassIdTypeP cur =0;
  
  for ( int i = 0; i< CARD_CLASS_ID_TABLE; i++)
    {
      cur = ClassIdTable[i];
      while (cur)
	{
	  RemoveDeletedObjectsForAllTheMid(cur->methods,cur->num_methods);
	  cur = cur->next;
	}
    }
  remove_objects_to_be_done = false;
  return;
}
/*
 *
 *
 */
void MoveObjId( jobjectID obj_id_before , jobjectID obj_id_after, jint arena_id_after )
{
  AllocObjectsP cur = 0;
  AllocObjectsP prev = 0;
  AllocObjectsP next = 0;
  //Current_index

  int i = (((unsigned long)(obj_id_before))>>3) & MASK_OBJECT_ID_TABLE;
  for ( cur = ObjectIdTable[i];
        cur && (cur->obj_id < obj_id_before);
        prev = cur, cur = cur->hash_next );

  if ( (! cur) || (cur->obj_id != obj_id_before) )
    {
      return;
    }
  /*Else we have the current class id.*/
  /*Do the move.*/
  next = cur->hash_next;
  /*Remove from this set.*/

  if (prev != 0)
    {
      prev->hash_next = next;
    }
  else
    {
      /*First element.*/
      /*So the head will be the next.*/
      ObjectIdTable[i] = next;
    }
  /*And insert at the new position.*/
  /*Update the Id.*/

  cur->obj_id = obj_id_after;
  cur->arena_id = arena_id_after;

  EnterObjId(cur);
  return;
}
/*************************************************************************
 *
 * Class representation  //  Allocation managment.
 *
 */
void LoadInfoStack( StackInfo * info, StackTypeP env_stack)
{
  /*Load all the stack information*/
  if ( (info !=(StackInfo*)0) && (env_stack != (StackTypeP)0))
    {
      StackTypeP current_stack = env_stack;
      int i = 0;
      while(current_stack && i<10)
	{
	  /*Load the stack information*/
	  info->methods[i] = current_stack->mid;
	  current_stack  =  current_stack->up;
	  i++;
	}
    }
  return;
}
/*
 *
 */
StackInfo * CreateInfoStack(StackTypeP env_stack)
{
  StackInfo * info = JNEW(StackInfo);

  if (info == (StackInfo*)0 )
    return ((StackInfo*)0 );

  info->num = 10;
  info->methods = JNEWARRAY( MethodIdTypeP , info->num);

  if (info->methods == ((MethodIdTypeP*)0))
    {
      JVM_exit_on_mem();
      return ((StackInfo*)0);
    }

  for (int i = 0; i< info->num; i++)
    {
      info->methods[i] = (MethodIdTypeP)0;
    }

  LoadInfoStack(info,env_stack);
  return info;
}
/*
 *
 */


/*
 * For the creation the env_stack can be loaded.
 */
ClassIdTypeP FindClassId ( jobjectID class_id );
AllocObjects* CreateAllocObjects(StackTypeP env_stack,jint arena_id,jobjectID class_id,jobjectID obj_id, jlong size, jint is_array)
{
  AllocObjects * new_obj = JNEW(AllocObjects);

  if (new_obj == (AllocObjects*)0 )
    {
      JVM_exit_on_mem();
      return (AllocObjects*)0;
    }
  new_obj->info_stack = ( ( METHOD_WITH_STACK(env_stack->mid->dump) )  ? CreateInfoStack(env_stack):(StackInfo*)0);

  new_obj->arena_id = arena_id;
  new_obj->obj_id = obj_id;
  new_obj->class_id = (ClassIdType*)0;
  new_obj->next = (AllocObjects*)0;
  new_obj->hash_next = (AllocObjects*)0;
  new_obj->is_deleted = false;
  new_obj->is_array = is_array;
  new_obj->size = size;
  new_obj->merged = false;
  ClassIdTypeP cid;
  switch(new_obj->is_array)
    {
    case JVMPI_NORMAL_OBJECT:
      cid = FindClassId ( class_id  );
      new_obj->class_id = cid;/*Might be null*/
      break;
    case JVMPI_CLASS        :
      cid = FindClassId ( class_id  );
      new_obj->class_id = cid;/*Might be null*/
      break;
    }
  /*Secure*/
  if (FindObjId(   new_obj->obj_id = obj_id ))
    FreeObjId(new_obj->obj_id = obj_id);
  return (new_obj);
}
/*
 *
 */
AllocSum * CreateAllocSum(JNIEnv * env_id)
{
  AllocSum * new_sum = JNEW( AllocSum );
  if ( new_sum == (AllocSum*)0 )
    {
      JVM_exit_on_mem();
      return (AllocSum*)0;
    }
  new_sum->start = (AllocObjects*)0;
  new_sum->last = (AllocObjects*)0;
  new_sum->env_id = env_id;
  new_sum->next = (AllocSum*)0;
  new_sum->calls = 0;
  new_sum->total_bytes = 0;
  new_sum->total_objects = 0;
  new_sum->total_and_d_objects = 0;
  new_sum->total_and_d_bytes = 0;
  return new_sum ;
}
/*
 * Store one allocation.
 *
 */
AllocObjects * AddAllocObjects(AllocSum* subject,AllocObjects * obj)
{
  if (subject == (AllocSum*)0)
    return (AllocObjects*)0;

  if (obj == (AllocObjects*)0)
    return (AllocObjects*)0;
  
  if (subject->start != (AllocObjects*)0)
    {
      subject->last->next = obj;
      subject->last = obj;
    }

  else

    {
      subject->start = obj;
      subject->last = obj;
    }

  return obj;
}
/*
 *
 */
AllocSum * AddAllocSumTo(AllocSum* subject, JNIEnv * env_id)
{
  AllocSum * new_sum, *last;

  if (subject == (AllocSum*)0)
    return ((AllocSum*)0);
  
  new_sum = CreateAllocSum(env_id);
  last = subject;

  while ( last->next != (AllocSum*)0 )
    {
      last =last->next;
    }

  last->next = new_sum;
  return new_sum;
}
/*
 * Search the Alloc env for the thread involved.
 *
 */
AllocSum * GetForThisEnv(AllocSum* base, JNIEnv *env_id)
{
  AllocSum * cur;

  /*Search the required env.*/
  if ( env_global_datas )
    {
      /*All is done in the global without specific env*/
      return (base);
    }

  if (base->env_id == env_id)
    return (base);

  cur = base;
  while ( cur->next )
    {
      if (cur->next->env_id == env_id)
	return( cur->next);

      cur = cur->next;
    }
  return (( AllocSum* )0);
}
/*
 * Remove one Allocation env.
 */
void DeleteAllObjectInformation(AllocObjects * base);
void DeleteForThisEnv(AllocSum * base, JNIEnv *env_id)
{
  AllocSum * good,*cur,*prev,*next;

  if (base == ((AllocSum*)0))
    return;
  /*
   * First case not treated,
   * The absolute values are always loaded.
   */

  good = (AllocSum*)0;
  cur = base/*->next*/;
  prev = base;

  while ( cur )
    {
      if (cur->env_id == env_id)
	{
	  good = cur;
	  break;
	}
      prev = cur;
      cur = cur->next;
    }

  if (good != (AllocSum*)0)
    {
      next = good->next;
      prev->next = next;
      DeleteAllObjectInformation(good->start);
      if ( env_id != (JNIEnv *) 0)
	JDELETE(good);
    }

  return;
}
/*
 * Remove methods Allocation for the thread env.
 */
void DeleteMethodsForThisEnv(JNIEnv *env_id,struct MethodIdType * methods,int num)
{
  int j = 0;
  for (j = 0; j<num;j++)
    {
      DeleteForThisEnv(&(methods->alloc), env_id);
      methods++;
    }
  return;
}
/*
 *  Clean all the allocations informations for the thread env.
 */
void DeleteClassIdsForThisEnv(JNIEnv *env_id)
{
  for ( int i = 0; i < CARD_CLASS_ID_TABLE; i++ )
    {
      ClassIdTypeP cur, next;
      for ( cur = ClassIdTable[i]; cur; )
    	{
	  if (cur != 0 )
	    {
	      next = cur->next;
	      DeleteMethodsForThisEnv(env_id,cur->methods,cur->num_methods);
	    }
	  cur = next;
	}
    }
  return;
}
/*
 * Delete all the objects informations.
 */
void DeleteAllObjectInformation(AllocObjects * base)
{
  AllocObjects * current, *prev;
  
  if (base != (AllocObjects*)0)
    {
      current = base;
      prev = base;
      while (current != (AllocObjects*)0)
	{
	  prev = current;
	  current = current->next;
	  /*Remove from the hash part.*/
	  RemoveObjId( prev );
	  /*And free the object*/
	  if (prev->info_stack) JDELETE(prev->info_stack);
	  JDELETE(prev);
	}
    }
  return;
}
/*
 * Delete all the methods id information.
 */
void DeleteAllAllocInformation(AllocSum * base)
{
  AllocSum * current, *prev;
  
  if (base != (AllocSum*)0)
    {
      current = base;
      prev = base;
      while (current != (AllocSum*)0)
	{
	  prev = current;
	  /*If there are some objects informations*/
	  DeleteAllObjectInformation(current->start);
	  current = current->next;
	  JDELETE(prev);
	}
    }
  return;
}
/*
 *
 */
void DeleteMethodsIds(struct MethodIdType * methods,int num)
{
  MethodIdType * current;
  if (methods != (MethodIdType*)0)
    {
      for (int i=0; i < num; i++)
	{
	  current = methods;
	  

	  if (current != (MethodIdType*)0)
	    {
#ifdef _WITH_MAX_OUT
	      if (current->signature != (char*)0)
         	JDELETE(current->signature);

	      if (current->name != (char*)0)
         	JDELETE(current->name);
#endif
	      DeleteAllAllocInformation(current->alloc.next);

	      methods++;
	    }
	  else
	    {
	      break;
	    }

	}
    }
  return;
}
/*
 * Class id delete (all the information).
 */
void DeleteClassIds(ClassIdType * to_delete)
{
  if (to_delete == (ClassIdType*)0)
    return;

#ifdef _WITH_MAX_OUT
  if (to_delete->class_name != (char*)0 )
    JDELETE(to_delete->class_name);
 
  if (to_delete->source_name != (char*)0 )
    JDELETE(to_delete->source_name);
#endif
  
  DeleteMethodsIds(to_delete->methods,to_delete->num_methods);
  /*And delete the last*/
  if (to_delete->num_methods >0)
    JDELETE(to_delete->methods);

  JDELETE(to_delete);
  return;
}
/*
 * Class id managment.
 * Reset the methods Allocation values for one thread env.
 */
void Resetmethods(JNIEnv *env_id,struct MethodIdType * methods,int num)
{
  int j = 0;
  for (j = 0; j<num;j++)
    {
      AllocSum * for_env;
      for_env  = GetForThisEnv(&(methods->alloc), env_id);
      
      if (for_env !=  (AllocSum*)0)
	{
	  for_env->calls = 0;
	  for_env->total_objects = 0;
	  for_env->total_and_d_objects = 0;
	  for_env->total_bytes = 0;
	  for_env->total_and_d_bytes = 0;
	}
      methods++;
    }
  return;
}
/*
 * Reset the class ids.
 */
void ResetClassIds(JNIEnv * env_id)
{
  for ( int i = 0; i < CARD_CLASS_ID_TABLE; i++ )
    {
      ClassIdTypeP cur, next;
      for ( cur = ClassIdTable[i]; cur; )
	{
	  if (cur != 0 )
	    {
	      
	      next = cur->next;
	      
	      if ( cur->num_methods )
		{
		  Resetmethods(env_id,cur->methods,cur->num_methods);
		}

	      cur = next;
	    }
	}
    }
  return;
}
/*
 * Insert one elt.
 */
void EnterClassId ( ClassIdTypeP cid )
{
  ClassIdTypeP cur, prev;

  int i = (((unsigned long)(cid->class_id))>>3) & MASK_CLASS_ID_TABLE;

  cid->next = (ClassIdTypeP)0;

  for ( cur = ClassIdTable[i], prev = (ClassIdTypeP)0;
        (cur && (cur->class_id < cid->class_id));
        prev = cur, cur = cur->next );

  if ( ! cur )
    {
      if ( ! prev )
	{
	  ClassIdTable[i] = cid;
	}
      else
	{
	  prev->next = cid;
	}
    }
  else
    {
      if ( cur->class_id == cid->class_id )
	{
	  _JERROR((char*)"WARNING: Two classes with the same jobjectID.");
	}

      cid->next = cur;

      if ( prev )
	{
	  prev->next = cid;
	}
      else
	{
	  ClassIdTable[i] = cid;
	}
    }
  return;
}
/*
 * UnloadClassId
 */
void UnloadClassId( jobjectID class_id_before )
{
  /*
   * Delete this class in the global set.
   *
   */
  DUMPGC("UL");

  ClassIdTypeP cur = 0;
  ClassIdTypeP prev = 0;
  ClassIdTypeP next = 0;

  //Current_index
  int i = (((unsigned long)(class_id_before))>>3) & MASK_CLASS_ID_TABLE;

  for ( cur = ClassIdTable[i];
        cur && cur->class_id < class_id_before;
        prev = cur, cur = cur->next );

  if ( (! cur) || (cur->class_id != class_id_before) )
    {
      return;
    }

  /*Else we have the current class id.*/
  /*Do the move.*/
  next = cur->next;
  /*Remove from this set.*/

  if (prev !=0)
    {
      prev->next = next;
    }
  else
    {
      /*First element.*/
      /*So the head will be the next.*/
      ClassIdTable[i] = next;
    }
  /*Reset object which still ref this class*/
  ResetObjectClassLink(cur);
  /*And delete the element */
  DeleteClassIds(cur);

  return;
}
/*
 * Find from an id.
 */
ClassIdTypeP FindClassId ( jobjectID class_id )
{
  ClassIdTypeP cur =0;
  if (class_id ==0) return 0;
  int i = (((unsigned long)(class_id))>>3) & MASK_CLASS_ID_TABLE;
  for ( cur = ClassIdTable[i];
        cur && (cur->class_id < class_id);
        cur = cur->next );
  if ( (! cur) || (cur->class_id != class_id) )
    {
      return ((ClassIdTypeP)0) ;
    }
  return cur;
}
/*
 * Move id (cf move event).
 * Since move will update the Object class_id,
 * it has to be done for all the objects.
 */
void MoveClassId( jobjectID class_id_before, jobjectID class_id_after )
{
  /*
   * Move is used during the move operation.
   * Move will extract the current_class_id, save the value.
   * Insert the new one,  and update the class_id.
   */
  ClassIdTypeP cur = 0;
  ClassIdTypeP prev = 0;
  ClassIdTypeP next = 0;

  //Current_index
  int i = (((unsigned long)(class_id_before))>>3) & MASK_CLASS_ID_TABLE;


  for ( cur = ClassIdTable[i];
        cur && (cur->class_id != class_id_before);
        prev = cur, cur = cur->next );

  if ( (! cur) || (cur->class_id != class_id_before) )
    {
      if (! cur) _JERROR((char*)"Error On move class");
      else _JERROR((char*)"Error on id class");
      return;
    }

  /*Else we have the current class id.*/
  /*Do the move.*/
  next = cur->next;
  /*Remove from this set.*/

  if (prev !=0)
    {
      prev->next = next;
    }
  else
    {
      /*First element.*/
      /*So the head will be the next.*/
      ClassIdTable[i] = next;
    }
  /*And insert at the new position.*/
  /*Update the Id.*/


  cur->class_id = class_id_after;

  EnterClassId(cur);

  return;
}
/*************************************************************************
 *
 * Dump functions.
 *
 * Dump is done with:
 * PU: Name of the test node or application.
 * PV: Test done in the node ( Tsf id1 id2, num message, inc value).
 * PP: Snapshot done (user name, date, inc value for the thread time).
 * PC: Method called, (Tsf id 1 / 2 with num method), with
 *     calls, allocated, total, total+D, global, global+D.
 * PD: Object data (Tsf id1 and / 2) for the class associated, array informations, size.
 *     Tsf id1 and 2 might be null in case of array of primitiv type.
 * PG: stack informations for the object ( Tsf id1, id2 ) and num message.
 *
 */
#ifdef CUSTOM_OUT
void dumpInfoStack(StackInfo* info_stack)
{
  if (info_stack == (StackInfo*)0)
    return;

  for (int i = 0; i< 10; i++)
    {
      if (info_stack->methods[i] != (MethodIdTypeP)0)
	{
	  jbbufString(jwOut,"PG ");
	  /*Followed by the tsf id*/
	  /*cid and class_id*/
	  jbbufRUnsigned16( jwOut,(uint32)info_stack->methods[i]->class_id);
	  jbbufString(jwOut," ");
	  jbbufRUnsigned16( jwOut,(uint32)info_stack->methods[i]->class_id->first_id);
	  jbbufString(jwOut," ");
	  /*And the method number for the link*/
	  jbbufRUnsigned16( jwOut,(uint32)info_stack->methods[i]->number_message);
	  jbbufString(jwOut,"\n");
	  jbbufSend(jwOut);
	}
      else
	{
	  return;
	};

    }
  return;
}
/*
 *
 */
void resetallocmerge(struct AllocObjects * start)
{
  AllocObjects * arrow;
  
  if (start != (AllocObjects*) 0)
    {
      arrow = start;
      while (arrow != (AllocObjects*)0)
	{
	  arrow->merged = false;
	  arrow = arrow->next;
	}
    }
  return;
}
/*
 *
 */
bool aresamestack(StackInfo* info_stack_ref,StackInfo* info_stack)
{
  if ( (info_stack_ref != 0) && (info_stack!=0))
    {
      for (int i = 0; i< 10; i++)
	{
	  if (info_stack->methods[i] != info_stack_ref->methods[i])
	    {
	      return false;
	    }
	}
      return true;
    }
  else
    {
      if ( (info_stack_ref == 0) && (info_stack==0))
	return true;
      return false;
    }
}
/*
 *
 */
bool aresameobject(struct AllocObjects * one, 
		   struct AllocObjects * two)
{
  if ((one!=0) && (two!=0))
    {
      if ( one->class_id != two->class_id )
	return false;
      
      if ( one->is_array != two->is_array )
	return false;

      if (!(aresamestack(one->info_stack,two->info_stack)))
	return false;
      
      return true;
    }
  else
    {
      return false;
    }
} 
/*
 *
 */
void getinfoforthisobject(jlong *size, jint* amount,struct AllocObjects * start)
{
  if (start!=0)
    {
      
      start->merged = true;

      *size += start->size;
      (*amount)++;
 
      AllocObjects * arrow;

      arrow = start->next;
      while (arrow != (AllocObjects*)0)
	{
	  if ( ! arrow->merged)
	    {
	      if ( aresameobject(start,arrow))
		{
		  *size += arrow->size;
		  (*amount)++;
		  arrow->merged = true;
		}
	    }
	  arrow = arrow->next;

	}	      
    }
}
/*
 *
 */
void dumpalloc(struct AllocObjects * start)
{
  AllocObjects * arrow;
  
  if (start != (AllocObjects*) 0)
    {
      
      resetallocmerge(start);

      arrow = start;

      while (arrow != (AllocObjects*)0)
	{

	  if ((arrow->class_id != 0)&&(arrow->class_id->is_tdp))
	    {
	    }
	  else
	    {

	      if (! arrow->merged)
		{
		  jlong size = 0;
		  jint amount = 0;
		  
		  getinfoforthisobject(&size,&amount,arrow);
		  
		  jbbufString(jwOut,"PD ");
		  /*Followed by the tsf id of this class if exist*/
		  /*cid and class_id*/
		  if ( arrow->class_id != (ClassIdTypeP) 0 )
		    {
		      jbbufRUnsigned16( jwOut,(uint32)arrow->class_id);
		      jbbufString(jwOut," ");
		      jbbufRUnsigned16( jwOut,(uint32)arrow->class_id->first_id);
		      jbbufString(jwOut," ");
		    }
		  else
		    {
		      jbbufRUnsigned16( jwOut,(uint32)0);
		      jbbufString(jwOut," ");
		      jbbufRUnsigned16( jwOut,(uint32)0);
		      jbbufString(jwOut," ");
		    }
		  /*Then the buffer type*/
		  switch(arrow->is_array)
		    {
		    case JVMPI_NORMAL_OBJECT:
		      jbbufRUnsigned16( jwOut,(uint32)1);
		      break;
		    case JVMPI_CLASS        :
		      jbbufRUnsigned16( jwOut,(uint32)2);
		      break;
		    case JVMPI_BOOLEAN      :
		      jbbufRUnsigned16( jwOut,(uint32)3);
		      break;
		    case JVMPI_BYTE         :
		      jbbufRUnsigned16( jwOut,(uint32)4);
		      break;
		    case JVMPI_CHAR         :
		      jbbufRUnsigned16( jwOut,(uint32)5);
		      break;
		    case JVMPI_SHORT        :
		      jbbufRUnsigned16( jwOut,(uint32)6);
		      break;
		    case JVMPI_INT          :
		      jbbufRUnsigned16( jwOut,(uint32)7);
		      break;
		    case JVMPI_LONG         :
		      jbbufRUnsigned16( jwOut,(uint32)8);
		      break;
		    case JVMPI_FLOAT        :
		      jbbufRUnsigned16( jwOut,(uint32)9);
		      break;
		    case JVMPI_DOUBLE       :
		      jbbufRUnsigned16( jwOut,(uint32)10);
		      break;
		    }
		  jbbufString(jwOut," ");
		  /*Now the size*/
		  jbbufRUnsigned16( jwOut,(uint32)size);
		  jbbufString(jwOut," ");
		  /*Now the amount*/
		  jbbufRUnsigned16( jwOut,(uint32)amount);
		  jbbufString(jwOut,"\n");
		  jbbufSend(jwOut);
		  dumpInfoStack(arrow->info_stack);
		  
		}

	    }
	  arrow = arrow->next;
	}
    }
  
  return;
}
/*
 *
 */
void dumpmethods(JNIEnv * env_id,ClassIdTypeP current_class ,struct MethodIdType * methods,int num)
{
  int j = 0;
  AllocSum * for_env;

  for (j = 0; j<num;j++)
    {
      for_env  = GetForThisEnv(&(methods->alloc), env_id);
      if (for_env != (AllocSum*)0)
	{
	  /*Type of method to dump*/
	  if (mode_filter_on && ((METHOD_TO_FILTER(methods->dump)==0)))
	    {
	    }
	  else
	    {
	      if ( (for_env->total_objects != 0)/*||(for_env->start != 0)*/ )
		{
		  jbbufString(jwOut,"PC ");
		  /*Followed by the tsf id*/
		  /*cid and class_id*/
		  jbbufRUnsigned16( jwOut,(uint32)current_class);
		  jbbufString(jwOut," ");
		  jbbufRUnsigned16( jwOut,(uint32)current_class->first_id);
		  jbbufString(jwOut," ");
		  /*Method number */
		  jbbufRUnsigned16( jwOut,(uint32) (j+5) );
		  jbbufString(jwOut," ");
		  /*Calls*/
		  jbbufRUnsigned16( jwOut,(uint32)(for_env->calls));
		  jbbufString(jwOut," ");
		  /*Allocated*/
		  jbbufRUnsigned16( jwOut,(uint32)(for_env->total_objects));
		  jbbufString(jwOut," ");
		  /*Total bytes*/
		  jbbufRUnsigned32( jwOut,(uint32)(for_env->total_bytes));
		  jbbufString(jwOut," ");
		  /*Allocated*/
		  jbbufRUnsigned16( jwOut,(uint32)(for_env->total_and_d_objects));
		  jbbufString(jwOut," ");
		  /*total_and_d_bytes*/
		  jbbufRUnsigned32( jwOut,(uint32)(for_env->total_and_d_bytes));
		  jbbufString(jwOut,"\n");
		  jbbufSend(jwOut);
		  dumpalloc(for_env->start);
		}
	    }
	}
      methods++;
    }
  return;
}
/*
 *
 */
void dumpClassIds(JNIEnv * env_id)
{
  ClassIdTypeP cur, next;
  for ( int i = 0; i < CARD_CLASS_ID_TABLE; i++ )
    {
      for ( cur = ClassIdTable[i]; cur; )
	{
	  if (cur != 0 )
	    {
	      next = cur->next;
	      if ( cur->num_methods )
		{
		  if ( ! cur->is_tdp )
		    dumpmethods(env_id,cur,cur->methods,cur->num_methods);
		}
	      cur = next;
	    }
	}
    }
  return;
}
/*
 *
 */
void dumpSnapshot()
{
  jbbufString(jwOut,"PP \"");
  jbbufString(jwOut,( (name_user != (char*)0) ? name_user : "<unknown user>" ));
  jbbufString(jwOut,"\" \"");
  jbbufString(jwOut,( (date_run != (char*)0) ? date_run : "<undated>" ));
  jbbufString(jwOut,"\" ");
  jbbufRUnsigned16( jwOut,(uint32)ms_from_begin);
  jbbufString(jwOut,"\n");
  jbbufSend(jwOut);
  return;
}
/*
 *Dump the test plan for the report.
 */
void dumpTestPlanEmpty()
{
  jbbufString(jwOut,"PV ");
  jbbufRUnsigned16( jwOut,(uint32)0);
  jbbufString(jwOut," ");
  jbbufRUnsigned16( jwOut,(uint32)0);
  jbbufString(jwOut," ");
  jbbufRUnsigned16( jwOut,(uint32)0);
  jbbufString(jwOut," ");
  jbbufRUnsigned16( jwOut,(uint32)count_test);
  jbbufString(jwOut,"\n");
  jbbufSend(jwOut);
  return;
}
/*
 *
 */
void dumpTestPlan()
{
  if (is_mode_unitest)
    {
      if (class_id_for_the_current_test != ((jobjectID)0) )
	{
	  jbbufString(jwOut,"PV ");
	  /*TSF*/
	  jbbufRUnsigned16( jwOut,(uint32)class_id_arrow_for_the_current_test);
	  jbbufString(jwOut," ");
	  jbbufRUnsigned16( jwOut,(uint32)class_id_for_the_current_test);
	  jbbufString(jwOut," ");
	  /*MS*/
	  jbbufRUnsigned16( jwOut,(uint32)method_id_for_the_current_test->number_message);
	  jbbufString(jwOut," ");
	  /*Modification*/
	  jbbufRUnsigned16( jwOut,(uint32)count_test);
	  jbbufString(jwOut,"\n");
	  jbbufSend(jwOut);
	}
      else
	{
	  count_test++;
	  dumpTestPlanEmpty();
	}
      test_has_been_dumped = true;
    }
  else
    {
      dumpTestPlanEmpty();
      main_has_been_dumped = true;
    }
  return;
}
/*
 *
 */
void dumpInformation(JNIEnv * env_id)
{
  
  ms_from_begin = _GETTIME();


  if (first_dump)
    {
      /*In this case dump the TU instruction*/
      /*TU*/
      jbbufString(jwOut,"PU \"-xJxAxVxAx-\" \"");
      jbbufString(jwOut, ((node_used != (char*)0) ? node_used : "No Node"));
      jbbufString(jwOut,"\"\n");
      jbbufSend(jwOut);
      first_dump = false;
    }

  if (has_test_to_be_declared())
    dumpTestPlan();

  dumpSnapshot();
  dumpClassIds(env_id);
  jbbufString(jwOut,"XX");
  jbbufString(jwOut,"\n");



  ResetClassIds(env_id);

  /*Send*/
  
  return;
}
#endif /*CUSTOM OUT*/
/*
 * Reset all the class ids.
 */
void freeClassIds()
{
  for ( int i = 0; i < CARD_CLASS_ID_TABLE; i++ )
    {
      ClassIdTypeP cur, next;
      for ( cur = ClassIdTable[i]; cur; )
	{
	  if (cur != 0)
	    {

	      next = cur->next;
	      DeleteClassIds(cur);
	      cur = next;
	    }
	}
      ClassIdTable[i] = (ClassIdTypeP)0;
    }
}
/*
 * Find the methods id.
 */
MethodIdTypeP FindMethodId ( ClassIdTypeP cid, jmethodID method_id, jint *mi )
{
  for ( *mi = 0;
        *mi < cid->num_methods && method_id != cid->methods[*mi].method_id;
        (*mi)++ );
  if ( *mi == cid->num_methods )
    {
      *mi = -1;
      _JERROR((char*)"WARNING: No method id");
      return (MethodIdTypeP)0;
    }
  return &cid->methods[*mi];
}
/*
 * Init the class ids.
 */
void initClassIds()
{
  for ( int i = 0; i < CARD_CLASS_ID_TABLE; i++ )
    {
      ClassIdTable[i] = (ClassIdTypeP)0;
    }
  return;
}
/*
 * Init the Object ids.
 */
void initObjectIds()
{
  for ( int i = 0; i < CARD_OBJECT_ID_TABLE; i++ )
    {
      ObjectIdTable[i] = (AllocObjectsP)0;
    }
  return;
}
/*************************************************************************
 * CALL STACK.
 */
StackTypeP PushStack ( EnvTypeP env_to_use, MethodIdTypeP mid )
{
  
  EnvTypeP env = env_to_use;
  
  StackTypeP stack = JNEW( StackType );

  if (stack == ((StackTypeP)0) )
    {
      JVM_exit_on_mem();
      return (StackTypeP) 0 ;
    }
  
  stack->mid = mid;
  
  stack->alloc.start = (AllocObjects*)0;		/*It will always be for the current thread.*/
  stack->alloc.next = (AllocSum*)0;		      /*It will always be for the current thread.*/
  stack->alloc.last = (AllocObjects*)0;		/*It will always be for the current thread.*/
  stack->alloc.env_id = 0;           	      /*In this context the env_id is not required.*/
  stack->alloc.calls = 0;
  stack->alloc.total_bytes = 0;
  stack->alloc.total_objects = 0;
  stack->alloc.total_and_d_objects = 0;
  stack->alloc.total_and_d_bytes = 0;
  stack->ignore = mid->class_id->is_tdp;
  stack->up = env->current;

  stack->down = (StackTypeP)0;

  if ( env->top == (StackTypeP)0 )
    {
      env->top = stack;
    }
  else
    {
      env->current->down = stack;
    }
  env->current = stack;
  env->stack_level++;
  return stack;
}
/*
 * Pop functions.
 */
StackTypeP PopStack ( EnvTypeP to_use, JVMPI_Event *event )
{

  EnvTypeP env = to_use;
  StackTypeP stack = env->current;
  if (stack == (StackTypeP) 0)
    return 0;

  if (stack->mid == 0)
    return 0;

  if ( env->top == stack )
    {
      env->top = (StackTypeP)0;
      env->current = (StackTypeP)0;
    }
  else
    {
      if (env->current != 0)
	{
	  env->current = env->current->up;
	  env->current->down = (StackTypeP)0;
	}
    }

  env->stack_level--;
  return (stack);
}
/************************************************************************
 * TSF SOURCE FILE.
 */
bool ClassIsTdp(JVMPI_Event *event)
{

  bool result = false;

  if (event->u.class_load.class_name == (char*)0)
    return false;
  
  result = (_STRNCMP( event->u.class_load.class_name,
		     "com.rational.testrealtime.", 26 )==0);

  /*J9*/
  if (!result)
    result = (_STRNCMP( event->u.class_load.class_name,
			"com/rational/testrealtime/", 26 )==0);

  if (result)
    return (result);

  if (is_mode_unitest)
    {
      result = 
	((_STRNCMP( event->u.class_load.class_name,
		   "junit.", 6 )==0) ||
	(_STRNCMP( event->u.class_load.class_name,
		   "j2meunit.", 9 )==0));

      if (!result)
	result = 
	  ((_STRNCMP( event->u.class_load.class_name,
		      "junit/", 6 )==0) ||
	   (_STRNCMP( event->u.class_load.class_name,
		      "j2meunit/", 9 )==0));
      
    }

  return ( result );


}
#define TCF_NAME "\"ObjecTest-Tracer\""
void TsfDefineBlock(JVMPI_Event *event)
{

  if (event == 0)
    return;
 
#define ACCESS(f) event->u.class_load.f
  ClassIdTypeP cid = JNEW( ClassIdType );
  if (cid == ((ClassIdType*)0) )
    {
      JVM_exit_on_mem();
      return;
    }

  cid->class_id = ACCESS(class_id);
  cid->first_id = cid->class_id;/*First id which will be used for the dump and link with the tsf*/
  cid->is_tdp = false;
  cid->num_methods = ACCESS(num_methods);
  cid->methods = (MethodIdTypeP)0;
  if (cid->num_methods>0)
    {
      cid->methods = JNEWARRAY( MethodIdType, cid->num_methods );
      
      if (cid->methods == ((MethodIdTypeP)0))
	{
	  JVM_exit_on_mem();
	  return;
	}
    }
#ifdef _WITH_MAX_OUT
  const char *c = event->u.class_load.class_name ;
  cid->class_name= (char*)0;
  if ( c != (char*)0 )
    {
      cid->class_name = JNEWARRAY(  char, _STRLEN(c)+1 );
      _STRCPY( cid->class_name, c );
    }
#endif
#ifdef _WITH_MAX_OUT
  c = event->u.class_load.source_name ;
  cid->source_name = (char*)0;
  if ( c != (char*)0 )
    {
      cid->source_name = JNEWARRAY( char, _STRLEN(c)+1 );
      _STRCPY( cid->source_name, c );
    }
#endif
  if (ClassIsTdp(event))
    {
      cid->is_tdp = true ;
    }
#ifdef CUSTOM_OUT
  /*Out in the custom part*/
  jbbufSend(jwOut);
  /*DB*/
  jbbufString(jwOut,"DB ");
  jbbufString(jwOut,TCF_NAME);
  jbbufString(jwOut,"\n");
  /*SO*/
  jbbufString(jwOut,"SO \"");
  jbbufString(jwOut,((ACCESS(source_name)!=(char*)0)?ACCESS(source_name):"no_source"));
  jbbufString(jwOut,"\" 0 \"txt\"\n");
  /*LK*/
  jbbufString(jwOut,"LK 1\nIN \"File\" 1 \"");
  jbbufString(jwOut,((ACCESS(source_name)!=(char*)0)?ACCESS(source_name):"no_source"));
  jbbufString(jwOut,"\"\n");
  /*SI*/
  jbbufString(jwOut,"SI 1\nLK 1\nIN \"Class\" 2 \"");
  jbbufString(jwOut,((ACCESS(class_name)!=(char*)0)?ACCESS(class_name):"no_class"));
  jbbufString(jwOut,"\"\n");

  /*Specific message for the class*/
  /*LK*/
  jbbufString(jwOut,"LK ");
  jbbufUnsigned16h( jwOut,(uint32)1);
  jbbufString(jwOut,"\n");
  /*MS*/
  jbbufString(jwOut,"MS ");
  jbbufUnsigned16h( jwOut,(uint32) (4));
  jbbufString(jwOut," \"");
  jbbufString(jwOut,((ACCESS(class_name)!=(char*)0)?ACCESS(class_name):"no_class"));
  jbbufString(jwOut,"\"\n");
  jbbufSend(jwOut);
#endif

  for ( int i = 0; i < ACCESS(num_methods); i++ )
    {
#define ACCESSM(f) ACCESS(methods)[i].f
      cid->methods[i].method_id = ACCESSM(method_id);

      cid->methods[i].alloc.start = (AllocObjects*)0;  //First creation.
      cid->methods[i].alloc.last = (AllocObjects*)0;  //First creation.
      cid->methods[i].alloc.next = (AllocSum*)0;  //First creation.
      cid->methods[i].alloc.env_id = 0;           //In this context the env_id is not required.
      //The results will be for all threads.
      cid->methods[i].alloc.calls = 0;
      cid->methods[i].alloc.total_bytes = 0;
      cid->methods[i].alloc.total_and_d_bytes = 0;
      cid->methods[i].alloc.total_objects = 0;
      cid->methods[i].alloc.total_and_d_objects = 0;
      cid->methods[i].class_id = cid;
      cid->methods[i].dump = (char)0;

#ifdef _WITH_MAX_OUT
      cid->methods[i].name = JNEWARRAY( char, _STRLEN(event->u.class_load.methods[i].method_name)+1 );
      _STRCPY( cid->methods[i].name,
	      event->u.class_load.methods[i].method_name );
#endif

      /*Informations about methods */
      MethodToDump * current;

      if ((current = Cmd_is_method_for( 
					    event->u.class_load.class_name,
					    event->u.class_load.methods[i].method_name,
					    _TRY_METHOD_TO_DUMP
					    ))!=(MethodToDump*)0 )
      {
	cid->methods[i].dump = (current->on_entry ? _DUMP_METHOD_ENTRY : _DUMP_METHOD_EXIT);
      }

      if ( (current = Cmd_is_method_for( 
					       event->u.class_load.class_name, 
					       event->u.class_load.methods[i].method_name,
					       _TRY_METHOD_TO_OBSERVE
					       ) )!= ((MethodToDump*)0))

      {
	cid->methods[i].dump |= _DUMP_WITH_STACK;
      }

      if ( (current = Cmd_is_method_for(
					       event->u.class_load.class_name,
					       event->u.class_load.methods[i].method_name,
					       _TRY_METHOD_TO_FILTER
					       ) )!= ((MethodToDump*)0))

      {
	cid->methods[i].dump |= _FILTER_METHOD;
      }

      if ( Cmd_is_test_junit(
			     event->u.class_load.methods[i].method_name
			     ) )
	{
	  DUMPGC("JUNIT TEST");
	  cid->methods[i].dump |= _JUNIT_TEST;
	}


      if ( Cmd_is_runtest_junit(
			     event->u.class_load.methods[i].method_name
			     ) )
	{
	  DUMPGC("JUNIT RUNTEST");
	  cid->methods[i].dump |= _JUNIT_RUNTEST;
	}
#ifdef _WITH_MAX_OUT
      cid->methods[i].signature = JNEWARRAY( char, _STRLEN(event->u.class_load.methods[i].method_signature)+1 );
      _STRCPY( cid->methods[i].signature,
	      event->u.class_load.methods[i].method_signature );
#endif
      /*i + 4 due to the above informations added*/
      cid->methods[i].number_message = (i+5);
#ifdef CUSTOM_OUT
  /*Out in the custom part*/
  jbbufSend(jwOut);
  /*LK*/
  jbbufString(jwOut,"LK ");
  jbbufUnsigned16h( jwOut,(uint32)ACCESSM(start_lineno));
  jbbufString(jwOut,"\n");
  /*MS*/
  jbbufString(jwOut,"MS ");
  jbbufUnsigned16h( jwOut,(uint32) (i+5));
  jbbufString(jwOut," \"");
  jbbufString(jwOut,((ACCESS(class_name)!=(char*)0)?ACCESS(class_name):"no_class"));
  jbbufString(jwOut,".");
  jbbufString(jwOut,((ACCESSM(method_name)!=(char*)0)?ACCESSM(method_name):"no_method"));
  jbbufString(jwOut,".");
  jbbufString(jwOut,((ACCESSM(method_signature)!=(char*)0)?ACCESSM(method_signature):"no_sign"));
  jbbufString(jwOut,"\"\n");
  jbbufSend(jwOut);

#endif

#undef ACCESSM
    }
  EnterClassId ( cid );

#ifdef CUSTOM_OUT
  /*Out in the custom part*/
  jbbufSend(jwOut);
  /*ID*/
  jbbufString(jwOut,"ID ");
  jbbufUnsigned16h( jwOut,(uint32)cid);
  jbbufString(jwOut," ");
  jbbufUnsigned16h( jwOut,(uint32)cid->class_id);
  jbbufString(jwOut,"\nED\n");
  jbbufSend(jwOut);
#endif

#undef ACCESS
  return;
}
/*************************************************************************
 *
 * Initialize threads env with one call.
 *
 */
void simulateStartThread( JNIEnv *env_id );
EnvTypeP InitializeEnv( JNIEnv *env_id )
{
  EnvTypeP env;
  if (env_id == 0)
  	return 0;
  env = JNEW(EnvType);
  if (env == ((EnvTypeP)0))
    {
      JVM_exit_on_mem();
      return ((EnvTypeP) 0);
    }
  env->top = (StackTypeP)0;
  env->current = (StackTypeP)0;
  env->stack_level=0;
  env->last_time_used = 0;

  ClassIdTypeP cid;
  jobjectID class_id  = (jobjectID)2;
  cid = FindClassId ( class_id );
  if (cid == 0)
   {
      simulateStartThread( env_id );
      cid = FindClassId ( class_id );
      if (cid == 0)
      {
      	return 0;
      }
   }
  jint mi = 0;
  jmethodID m_id  = (jmethodID)2;
  MethodIdTypeP mid = FindMethodId ( cid, m_id , &mi );
  AllocSum * for_env;
  for_env = AddAllocSumTo(&(mid->alloc), env_id);
  /*Update the allocation for the mid*/
  /*Update the local env calls       */
  for_env->calls = 1;
  for_env->total_objects =0;
  for_env->total_and_d_objects =0;
  for_env->total_and_d_bytes =0;
  /*Push it*/
  PushStack ( env , mid );
  return (env);
}
/********************************************************************
 * Events functions.
 */
void on_JVMPI_EVENT_JVM_INIT_DONE( JVMPI_Event *event )
{
  DUMPGC("JVMPI_EVENT_JVM_INIT_DONE\n");
  env_is_on = true;

  /*Create listener to perform the RunGC*/
#ifndef JVMPI_SOCKET_JVMPI_NONE
  if (JvmpiInterface->CreateSystemThread((char*)"pagent listener",
  					 JVMPI_MINIMUM_PRIORITY,
					 pagent_callback) == JNI_ERR)

   {
    _JERROR((char*)"ERROR: Create Pagent Listener");
  }


#endif

  if (!is_obs_from_begin)
    {
      ResetClassIds(event->env_id);
    }

  dumpInformation(event->env_id);
  
  /*But reset the start thread call already done*/
  ClassIdTypeP cid;
  jobjectID class_id  = (jobjectID)2;
  cid = FindClassId ( class_id );
  jint mi = 0;
  jmethodID m_id  = (jmethodID)2;
  MethodIdTypeP mid = FindMethodId ( cid, m_id , &mi );
  AllocSum * alloc_linked = GetForThisEnv(&(mid->alloc),event->env_id);
  if (alloc_linked != ((AllocSum*)0))
    alloc_linked->calls = 1;

  VMStatus = VMStatus_OK;
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_JVM_SHUT_DOWN(JVMPI_Event * event)
{
  DUMPGC("SHUT DOWN");
  gc_running = true;
  env_is_on = false;
  VMStatus = VMStatus_AFTER_SHUTDOWN;

#ifndef JVMPI_SOCKET_JVMPI_NONE
  if (sock_fd!=0)
    {
      DUMPGC("END");
      SEND(sock_fd,(char*)"END.......",10);
    }
#endif

  if (!is_mode_unitest)
    dumpInformation( (JNIEnv *) 0 );
  
  jbbufString(jwOut,"YY");
  jbbufString(jwOut,"\n");

  freeClassIds ( );
  /*Delete the command line*/
  CmdDeleteSet();

  /*And the hash tables*/
  JDELETE(ClassIdTable);
  JDELETE(ObjectIdTable);

#ifdef CUSTOM_OUT
  jbbufSend(jwOut);
  JDELETE(jwOut);
#endif
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_CLASS_LOAD(JVMPI_Event *event)
{
  DUMPGC("CL");
  CHECK_END
  ClassIdTypeP cid;
  cid = FindClassId ( event->u.class_load.class_id );
  if ( cid == (ClassIdType*)0)
  {
    TsfDefineBlock ( event );
  }
  /*Load all the classes, most of them are still used after the start*/
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_CLASS_UNLOAD(JVMPI_Event * event)
{
  DUMPGC("UL");
  CHECK_END
  UnloadClassId( event->u.class_unload.class_id );
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_GC_START( JVMPI_Event * /*_event*/ )
{
  /*Get the lock for the free ... */
  gc_running = true;

  DUMPGC("[GC START]");
#ifndef JVMPI_SOCKET_JVMPI_NONE
  if (VMStatus == VMStatus_OK)
    {
      if (sock_fd!=0)
	{
	  DUMPGC("SEND GCS");
	  SEND(sock_fd,(char*)"GCS.......",10);
	}
    }
#endif
  DUMPGC("[return from GC START]");
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_GC_FINISH( JVMPI_Event * /*_event*/ )
{
  DUMPGC("[GC STOP]");
  /*remove to be done to the next event*/
  remove_objects_to_be_done = true;
  should_dump_be_done_after_gc = true;/*If needed*/
  gc_running = false;
  /*Reset the lock after GC*/
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_THREAD_START(JVMPI_Event *event)
{
  DUMPGC("THS");
  CHECK_END
  EnvTypeP env;
  if ( (env =(EnvTypeP)JvmpiInterface->GetThreadLocalStorage(event->env_id)) == (EnvTypeP)0)
    {
      env = InitializeEnv(event->env_id);
      if (env)
	JvmpiInterface->SetThreadLocalStorage(event->env_id, (void *)env);
      /*Update the time used by the thread*/
      
    }
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_THREAD_END(JVMPI_Event *event)
{
  DUMPGC("THE");
  CHECK_END
  /*All the thread information are deleted and removed */
  /*from the object hash table here                    */
  DeleteClassIdsForThisEnv(event->env_id);
  /*And then delete the ctx*/
  EnvTypeP env = (EnvTypeP)JvmpiInterface->GetThreadLocalStorage(event->env_id);
  if (env != (EnvTypeP)0)
    {
      StackTypeP prev = env->top;
      StackTypeP current = env->top;
      while (current != (StackTypeP)0)
	{
	  prev = current;
	  current = current->down;
	  JDELETE(prev);
	}
      JDELETE(env);
    }
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_METHOD_ENTRY(JVMPI_Event *event)
{
  CHECK_VMSTATUS_METHOD
  CHECK_END
  DUMP("EN")

  if (remove_objects_to_be_done)
    {
#ifndef JVMPI_SOCKET_JVMPI_NONE
      if (sock_fd!=0)
	{
	  DUMPGC((char*)"SEND GCE FROM ENTRY");
	  SEND(sock_fd,(char*)"GCE.......",10);
	}
#endif
      /*Will remove all the deleted objects after the GC start/stop.*/
      RemoveObjForAllClasses();


      if ((dump_after_gc && should_dump_be_done_after_gc && env_is_on)
	  ||(force_dump_after_gc && should_dump_be_done_after_gc && env_is_on)
	  ||(dump_required_from_pagent && should_dump_be_done_after_gc && env_is_on))
	{
	  DUMPGC("DUMP2");  
	  force_dump_after_gc = false;
	  dump_required_from_pagent = false;
	  dumpInformation( (JNIEnv *) 0 );
	}
      should_dump_be_done_after_gc = false;
    }

  EnvTypeP env;
  if ( (env =(EnvTypeP)JvmpiInterface->GetThreadLocalStorage(event->env_id)) == (EnvTypeP)0)
    {
      env = InitializeEnv(event->env_id);
      if (env == 0)
	{
	  return;
	}
      JvmpiInterface->SetThreadLocalStorage(event->env_id, (void *)env);
    }

  jmethodID method_id = event->u.method.method_id;
  jobjectID class_id  = (jobjectID)1;

  class_id = JvmpiInterface->GetMethodClass ( method_id );

  ClassIdTypeP cid;
  if ( (cid = FindClassId ( class_id )) == 0)
    {
      JvmpiInterface->RequestEvent( JVMPI_EVENT_CLASS_LOAD,class_id);
      cid = FindClassId ( class_id  );
      if (cid == 0)
	{
	  if((VMStatus != VMStatus_AFTER_SHUTDOWN))
	    _JERROR((char*)"ERROR: entry with class");

	  return;
	}
    }
  
  jint mi;
  MethodIdTypeP mid = FindMethodId ( cid, method_id, &mi );
  AllocSum * for_env;

  if ( (for_env  = GetForThisEnv(&(mid->alloc), event->env_id))  == (AllocSum*)0)
    for_env = AddAllocSumTo(&(mid->alloc), event->env_id);
  /*Update the allocation for the mid*/
  /*Clean also the vals                                         */
  /*All the objects loaded in the current stack of this fonction*/
  /*Are deleted here.                                           */
  /*Delete means: Remove from the hash table and free.          */
  /*DeleteAllObjectInformation depends on the flag              */
  /*And set the ref to the last thread for the global mode*/

  if ( ! should_the_objects_stay )
    {
      DeleteAllObjectInformation(for_env->start);
      for_env->start = (AllocObjectsP)0;
      for_env->last = (AllocObjectsP)0;
    }
  else
    {
      /*We keep for the dump all the objects informations*/
    }

  /*Update the local env calls*/
  for_env->calls++;
  /*Push it*/

  StackTypeP stack = PushStack ( env , mid );
  stack->ignore = cid->is_tdp ;

  /*UNITEST FLAG*/
  /*Do this test before, it will be faster for all the others methods*/
  if (mid->dump != (char)0)
    {
      if (is_mode_unitest)
	{
	  if ( METHOD_WITH_RUNTEST_JUNIT(mid->dump)!=0 )
	    {
	      /*Set the flag for a junit test*/
	      is_run_test_load = true;
	    }

	  if (is_run_test_load && (!(is_run_method_test_load)))
	    {
	      if ( METHOD_WITH_TEST_JUNIT(mid->dump)!=0 )
		{
		  DUMPGC("VALUE SET FOR JUNIT");
		  /*Set the name of the method as class_id / method_id */
		  count_test++;/*Inc the count for the junit test      */
		  /*To be used for the dump part                       */
		  class_id_for_the_current_test = cid->first_id;
		  class_id_arrow_for_the_current_test = cid;
		  method_id_for_the_current_test = mid;
		  /*To avoid another load of test*/
		  is_run_method_test_load = true;
		  /*To declare this test in the dump function*/
		  test_has_been_dumped = false;
		}
	    }
	}

      if ((METHOD_WITH_DUMP_ENTRY(mid->dump)!=0))/*Dump is required*/
	{
	  /*Gc is required before the dump*/
	  /*Remove this call, under rs6000 system the GC */
	  /*is called in another thread.                 */
	  /*
	    if (do_the_gc == true )
	    {
	    JvmpiInterface->RunGC();
	    }
	  */

 	  if (is_dump_partial_on_gc)
	    {
	      if (should_dump_be_done_after_gc)
		{
		  should_dump_be_done_after_gc = false;/*Already done*/
		  DUMPGC("DUMP");  
		  dumpInformation(event->env_id);
		}
	      else
		{
		  force_dump_after_gc = true;
		}
	    }
	  else
	    {
	      DUMPGC("DUMP");  
	      dumpInformation(event->env_id);
	    }
	}
    }

  return;
}
/*
 * The declaration of this class is used to have
 * a method call after each thread start.
 * This way the allocation done before any call can
 * be trapped.
 *
 */
void simulateStartThread( JNIEnv *env_id )
{
  /*Simulate the load of the VM */
  /*For all tge allocations done*/
  /*Before any load class evt   */
  JVMPI_Event evt;
  evt.env_id = env_id ;

  JVMPI_Method vm_method;
  vm_method.method_name      = (char*)"thread.()V";
  vm_method.method_signature = (char*)"";
  vm_method.start_lineno    = 0;
  vm_method.end_lineno       = 0;
  vm_method.method_id        = (jmethodID)2;

  evt.event_type = JVMPI_EVENT_CLASS_LOAD;
  evt.u.class_load.class_name          = (char*)"Start";
  evt.u.class_load.source_name         = (char*)"Start";
  evt.u.class_load.num_interfaces      = 0;
  evt.u.class_load.num_methods         = 1;
  evt.u.class_load.methods             = &vm_method ;
  evt.u.class_load.num_static_fields   = 0;
  evt.u.class_load.statics             = 0;
  evt.u.class_load.num_instance_fields = 0;
  evt.u.class_load.instances           = 0;
  evt.u.class_load.class_id            = (jobjectID)2;

  TsfDefineBlock( &evt );

  return;
}
/*
 *
 */
void on_JVMPI_EVENT_METHOD_EXIT(JVMPI_Event *event)
{
  CHECK_VMSTATUS_METHOD
  CHECK_END
  DUMP("EX");


  EnvTypeP env_stack;
  StackTypeP stack;
  env_stack = (EnvTypeP)JvmpiInterface->GetThreadLocalStorage(event->env_id);
  if (env_stack == ((EnvTypeP)0))
    {
      return;
    }

  stack = PopStack ( env_stack, event );
  /*Unknown method allocation have been caught by the simulate class*/
  if (stack == ((StackTypeP)0))
    {
      return;
    }

  /*For the end of the function*/
  bool dump_required = false;
  /*Do this test before, it will be faster for all the others methods*/
  if (stack->mid->dump != (char)0)
    {
      dump_required = (METHOD_WITH_DUMP_EXIT(stack->mid->dump)!=0);
      /*UNITEST FLAG*/
      if (is_mode_unitest && is_run_test_load && (METHOD_WITH_TEST_JUNIT(stack->mid->dump)!=0) )
	{
	  if ( !Is_other_test_method(stack))
	    {
	      /*If it is the single test method in the stack         */
	      /*ReSet the name of the method as class_id / method_id */
	      /*If it is the same as the loaded one                  */
	      if ( stack->mid == method_id_for_the_current_test)
		{
		  if ( class_id_for_the_current_test == stack->mid->class_id->first_id )
		    {
		      
		      /*
			class_id_for_the_current_test = (jobjectID)0;
			method_id_for_the_current_test = (MethodIdType*)0;
		      */
		      
		      is_run_method_test_load = false;
		      /*And same for the runBare*/
		      is_run_test_load = false;
		      /*To declare that another kind of dump might happen*/
		      test_has_been_dumped = false;
		    }
		}
	    }
	}
    }

  if ( !stack->ignore )
    {
      AllocSum * for_env;
      /*Update the global alloc information*/
      for_env  = GetForThisEnv(&(stack->mid->alloc), event->env_id);
      if (for_env != ((AllocSum*)0))
	{
	  for_env->total_and_d_bytes += stack->alloc.total_and_d_bytes;
	  for_env->total_and_d_objects += stack->alloc.total_and_d_objects;
	}
      StackTypeP up = stack->up;
      if (up!= ((StackTypeP)0) )
   	{
	  stack->up->alloc.total_and_d_bytes += stack->alloc.total_and_d_bytes;
	  stack->up->alloc.total_and_d_objects += stack->alloc.total_and_d_objects;
	}
    }

  JDELETE(stack);

  if (dump_required)/*Dump is required*/
    {
      /*Gc is required before the dump*/
      /*Remove this call, under rs6000 system the GC */
      /*is called in another thread.                 */
      /*
	if (do_the_gc == true )
	{
	JvmpiInterface->RunGC();
	}
      */

      if (is_dump_partial_on_gc)
	{
	  if (should_dump_be_done_after_gc)
	    {
	      should_dump_be_done_after_gc = false;/*Already done*/
	      DUMPGC("DUMPEXITOK");  
	      dumpInformation(event->env_id);
	    }
	  else
	    {
	      DUMPGC("FORCE DUMP");  
	      force_dump_after_gc = true;
	    }
	}
      else
	{
	  DUMPGC("DUMPEXIT");  
	  dumpInformation(event->env_id);
	}
    }
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_ARENA_DELETE( JVMPI_Event *event )
{
  DUMPGC("ARENA");
  CHECK_END
  FreeObjectForArena( event->u.delete_arena.arena_id );
  return;
}
/*
 *
 */
void on_JVMPI_EVENT_OBJECT_ALLOC( JVMPI_Event *event )
{
  CHECK_VMSTATUS_METHOD
  CHECK_END
  DUMP("alloc");

  AllocObjects * new_obj;
  EnvTypeP env;
  /*Has to be done here for the JNI threads*/
  if ( (env =(EnvTypeP)JvmpiInterface->GetThreadLocalStorage(event->env_id)) == (EnvTypeP)0)
    {
      env = InitializeEnv(event->env_id);
      if (env==0)
      	return;
      JvmpiInterface->SetThreadLocalStorage(event->env_id, (void *)env);
    }

  StackTypeP stack = env->current;
  if( stack != ((StackTypeP)0))
    {
      if (remove_objects_to_be_done)
	{
	  /*Will remove all the deleted objects after the GC start/stop.*/
	  /*Has also to be done here in case of loop with a call        */
#ifndef JVMPI_SOCKET_JVMPI_NONE
	  if (sock_fd!=0)
	    {
	      DUMPGC("SEND GCE FROM ALLOC");
	      SEND(sock_fd,(char*)"GCE.......",10);
	    }
#endif

	  RemoveObjForAllClasses();

	  if ((dump_after_gc && should_dump_be_done_after_gc && env_is_on)
	      ||(force_dump_after_gc && should_dump_be_done_after_gc && env_is_on)
	      ||(dump_required_from_pagent && should_dump_be_done_after_gc && env_is_on))

	    {
	      DUMPGC("DUMP2");  
	      dumpInformation( (JNIEnv *) 0 );
	      force_dump_after_gc = false;
	      dump_required_from_pagent = false;
	    }
	  should_dump_be_done_after_gc = false;
	}

      if ( !stack->ignore )
	{

	  stack->alloc.total_bytes += event->u.obj_alloc.size  ;
	  stack->alloc.total_and_d_bytes += event->u.obj_alloc.size  ;
	  stack->alloc.total_objects++;
	  stack->alloc.total_and_d_objects++;
	}

      if (mode_filter_on && (!(METHOD_TO_FILTER(stack->mid->dump)!=0)))
	{
	  AllocSum * for_env;
	  for_env  = GetForThisEnv(&(stack->mid->alloc), event->env_id);
	  if (for_env != (AllocSum*) 0)
	    {
	      for_env->total_bytes += event->u.obj_alloc.size;
	      for_env->total_objects ++;
	    }
	  return;
	}

      /*Find the class and update for the mid*/
      AllocSum * for_env;
      for_env  = GetForThisEnv(&(stack->mid->alloc), event->env_id);

      if (for_env != (AllocSum*) 0)
	{
	  for_env->total_bytes += event->u.obj_alloc.size;
	  for_env->total_objects ++;
	  /*for_env->total_and_d_objects ++;*/
	  
	  if ( !with_no_obj_allocation )
	    {
	      new_obj = CreateAllocObjects(
					   stack,
					   event->u.obj_alloc.arena_id ,
					   event->u.obj_alloc.class_id ,
					   event->u.obj_alloc.obj_id ,
					   event->u.obj_alloc.size,
					   event->u.obj_alloc.is_array );
	      AddAllocObjects(for_env,new_obj);
	      EnterObjId(new_obj);
	    }
	  
	}
    }
  return;
}
/*
 * FREE OPERATION.
 */
void on_JVMPI_EVENT_OBJECT_FREE( JVMPI_Event *event )
{
  /*Set the obj loaded as deleted                         */
  /*These obj are still in the table, they will be removed*/
  /*During the free done in ENTRY methods or Thread end.  */
  CHECK_END
  FreeObjId( event->u.obj_free.obj_id);
  return;
}
/*
 * MOVE OPERATION.
 */
void on_JVMPI_EVENT_OBJECT_MOVE( JVMPI_Event *event )
{
  /*Change the Class id and object id*/
  DUMP(".");
  CHECK_END
  /*This test is cheaper*/
  if (FindClassId(event->u.obj_move.obj_id) != ((ClassIdType*) 0))
    {
      DUMP("C");
      MoveClassId( event->u.obj_move.obj_id, event->u.obj_move.new_obj_id);
    }
  else
    {
      AllocObjectsP cur =0;
      cur = FindObjId( event->u.obj_move.obj_id );
      if (cur)
	{
	  cur = FindObjId( event->u.obj_move.new_obj_id );
	  if (cur)
	    {
	      /*Sometimes the move is made for nothing (same adresses)*/
	      /*Secure*/
	      if ((cur->obj_id) != (event->u.obj_move.new_obj_id))
		FreeObjId( event->u.obj_move.new_obj_id);
	    }
	  
	  MoveObjId(
		    event->u.obj_move.obj_id ,
		    event->u.obj_move.new_obj_id,
		    event->u.obj_move.new_arena_id );
	}
    }
  
  return;
}
/*
 * JVMPI EVENT HANDLER.
 */
typedef void (*EventHandlerFct)(JVMPI_Event *);
typedef struct
{
  EventHandlerFct handler_fct;  //!< handler fct for event
  bool            can_lock;     //!< this event support monitor/lock
} EventHandler ;
EventHandler event_handlers[JVMPI_MAX_EVENT_TYPE_VAL+1];
/*
 * Notify Event function.
 *
 */
void closeEventHandler();
static void NotifyEvent(JVMPI_Event *event)
{
  switch(event->event_type) {

  case JVMPI_EVENT_JVM_INIT_DONE:
    JvmpiInterface->RawMonitorEnter(GC_Call);
    on_JVMPI_EVENT_JVM_INIT_DONE(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_JVM_SHUT_DOWN:
    JvmpiInterface->RawMonitorEnter(GC_Call);
    on_JVMPI_EVENT_JVM_SHUT_DOWN(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    /*Might be run in another thread*/
    //closeEventHandler();
    return;

  case JVMPI_EVENT_CLASS_LOAD:
    /*GC suspended*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = false;
    on_JVMPI_EVENT_CLASS_LOAD(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_CLASS_LOAD | JVMPI_REQUESTED_EVENT:
    /*GC suspended*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = false;
    on_JVMPI_EVENT_CLASS_LOAD(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_CLASS_UNLOAD:
    /*GC suspended*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = false;
    on_JVMPI_EVENT_CLASS_UNLOAD(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_OBJECT_DUMP | JVMPI_REQUESTED_EVENT :
    return;

  case JVMPI_EVENT_OBJECT_ALLOC:
    /*GC suspended*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = true;
    on_JVMPI_EVENT_OBJECT_ALLOC(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

    /*Do not grabb lock here*/
  case JVMPI_EVENT_OBJECT_MOVE:
    /*Thread suspended*/
    on_JVMPI_EVENT_OBJECT_MOVE(event);
    return;

    /*Do not grabb lock here*/
  case JVMPI_EVENT_ARENA_DELETE:
    /*Thread suspended*/
    on_JVMPI_EVENT_ARENA_DELETE(event);
    return;

    /*Do not grabb lock here*/
  case JVMPI_EVENT_OBJECT_FREE:
    /*Thread suspended*/
    on_JVMPI_EVENT_OBJECT_FREE(event);
    return;

  case JVMPI_EVENT_THREAD_START:
    /*GC is disabled*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = false;
    on_JVMPI_EVENT_THREAD_START(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_THREAD_END:
    /*GC is not disabled*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = true;
    on_JVMPI_EVENT_THREAD_END(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_METHOD_ENTRY:
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = true;
    on_JVMPI_EVENT_METHOD_ENTRY(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

  case JVMPI_EVENT_METHOD_EXIT:
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = true;
    on_JVMPI_EVENT_METHOD_EXIT(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;

    /*No more lock to avoid dead lock*/
  case JVMPI_EVENT_GC_START:
    /*Thread suspended mode start*/
    JvmpiInterface->RawMonitorEnter(GC_Call);
    can_do_call_gc = false;
    on_JVMPI_EVENT_GC_START(event);
    return;

  case JVMPI_EVENT_GC_FINISH:
    /*Thread suspended mode stop*/
    on_JVMPI_EVENT_GC_FINISH(event);
    JvmpiInterface->RawMonitorExit(GC_Call);
    return;
  }

}
/*
 * Set the events.
 */
void initEventHandler()
{
  JvmpiInterface->NotifyEvent = NotifyEvent;

  for ( int i = 0; i < JVMPI_MAX_EVENT_TYPE_VAL; i++ )
    {
      event_handlers[i].handler_fct = (EventHandlerFct)0;
      event_handlers[i].can_lock    = false;
    }
#define TRACKEVENT( __evt__, __can_lock__ ) \
 { JvmpiInterface->EnableEvent( __evt__, 0 ); \
   event_handlers[__evt__].handler_fct = on_##__evt__; \
	event_handlers[__evt__].can_lock    = __can_lock__; }
  TRACKEVENT( JVMPI_EVENT_JVM_INIT_DONE, true );
  TRACKEVENT( JVMPI_EVENT_JVM_SHUT_DOWN, true );
  TRACKEVENT( JVMPI_EVENT_CLASS_LOAD,    true );
  TRACKEVENT( JVMPI_EVENT_CLASS_UNLOAD,  true );
  TRACKEVENT( JVMPI_EVENT_METHOD_ENTRY,  true );
  TRACKEVENT( JVMPI_EVENT_METHOD_EXIT,   true );
  TRACKEVENT( JVMPI_EVENT_OBJECT_ALLOC,  true );
  TRACKEVENT( JVMPI_EVENT_OBJECT_FREE,   false );
  TRACKEVENT( JVMPI_EVENT_OBJECT_MOVE,   false );
  TRACKEVENT( JVMPI_EVENT_THREAD_START,  true  );
  TRACKEVENT( JVMPI_EVENT_THREAD_END,    true  );
  TRACKEVENT( JVMPI_EVENT_ARENA_DELETE,  false );
  TRACKEVENT( JVMPI_EVENT_GC_START,      true );
  TRACKEVENT( JVMPI_EVENT_GC_FINISH,     true );
#undef TRACKEVENT

  return;
}
/*
 * Close the event handler.
 */
void closeEventHandler()
{
  DUMPGC("UNTRACK");
#define UNTRACKEVENT( __evt__ ) \
   JvmpiInterface->DisableEvent ( __evt__, 0 )
    UNTRACKEVENT( JVMPI_EVENT_JVM_INIT_DONE );
    UNTRACKEVENT( JVMPI_EVENT_JVM_SHUT_DOWN );
    UNTRACKEVENT( JVMPI_EVENT_CLASS_LOAD    );
    UNTRACKEVENT( JVMPI_EVENT_CLASS_UNLOAD  );
    UNTRACKEVENT( JVMPI_EVENT_METHOD_ENTRY  );
    UNTRACKEVENT( JVMPI_EVENT_METHOD_EXIT   );
    UNTRACKEVENT( JVMPI_EVENT_OBJECT_ALLOC  );
    UNTRACKEVENT( JVMPI_EVENT_OBJECT_FREE   );
    UNTRACKEVENT( JVMPI_EVENT_OBJECT_MOVE   );
    UNTRACKEVENT( JVMPI_EVENT_THREAD_START  );
    UNTRACKEVENT( JVMPI_EVENT_THREAD_END    );
    UNTRACKEVENT( JVMPI_EVENT_ARENA_DELETE  );
    UNTRACKEVENT( JVMPI_EVENT_GC_START      );
    UNTRACKEVENT( JVMPI_EVENT_GC_FINISH     );
#undef UNTRACKEVENT
    /*Exit for the Lock is already done    */
    return;
}
/*
 * End Event Function.
 ********************************************************************/

/*
 * Main part.
 */

extern "C"
JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *jvm,
                                  char * options,
                                  void * /*reserved*/)
{
  /*VM is not initialized*/
  VMStatus = VMStatus_BEFORE_INIT;
  

  /*Load the command line options*/
 if ( CmdTreatCmdLine(options) != 1 )
    {
      _JERROR((char*)"ERROR: Command line.");
      return ( JNI_ERR );
    }

  /*Create Hash*/
  AllocHastables();
  /*Init the Ids for all the objects*/
  initObjectIds();
  /*Init the Ids for all the classes*/
  initClassIds();

#ifdef CUSTOM_OUT
  /*In case of Custom part */
  /*Create the buffer      */
  jwOut = jbbufCreate();
  if (jwOut)
    {
      if (jwOut->jw_buffer)
	{
	  if (jwOut->jw_buffer->failed())
	    {
	      return JNI_ERR;
	    }
	}
      else
	{
	  return JNI_ERR;
	}
    }
  else
    {
      return JNI_ERR;
    }

#endif
  /*Add a class for all the allocation */
  /*without a call.                    */
  simulateStartThread( (JNIEnv *) 0 );
  int res = jvm->GetEnv((void **)&JvmpiInterface, JVMPI_VERSION_1);
  if (res !=  0) {
    _JERROR((char*)"ERROR: Cannot start profiler");
    return JNI_ERR;
  }

  /*Set the events to be caught*/
  GC_Call = JvmpiInterface->RawMonitorCreate((char*)"_GC_Call");

  /*Respect the order*/
  JvmpiInterface->RawMonitorEnter(GC_Call);
  initEventHandler();
  JvmpiInterface->RawMonitorExit(GC_Call);
  return JNI_OK;
}
/*
  ************************************************************************/








