
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 

package j2meunit.framework;

import j2meunit.framework.*;

import java.util.Vector;
import java.util.Date;
import java.util.Enumeration;

public abstract class TestCase extends TestCasePrim implements StubListener, VerifyListener{
/**
 * The check results are a part of the assert.
 * A check is a message and a result for a Test.
 */
static private Check fCheckdone;
private Vector fVerifyListener;

/**
 * The "timer" which can be used are contained in
 * this vector.
 */
protected Vector fDate;
/**
 *
 */
  public TestCase() {
  super("null");
  fDate = new Vector();
  resetExceptionWatch();
  fVerifyListener = new Vector();

  }

  public TestCase(String testMethodName) {
  super(testMethodName);
  fDate = new Vector();
  resetExceptionWatch();
  fVerifyListener = new Vector();

  }
/**
 *
 */
public final void addVerifyListener(VerifyListener Listener)
{
  fVerifyListener.addElement(Listener);
}
/**
 * Get the last check done.
 *
 */
static private Check getcheck()
{
  return fCheckdone;
}
/**
 *
 */
public final synchronized void run(TestResult result)
{
  this.result = result;
  addVerifyListener(result);
  result.run(this);
}
/**
 *
 */
public final synchronized void runBare() throws Throwable {

  listentoStub();
  //To enable some checks.
  try
  {
    super.runBare();
  }

  catch (AssertionFailedError e)
  {
        //To throw the result to the result object.
     	throw e;
  }

  catch (Throwable e)
    {
        if ( exceptionwaited == true)
        {
          //To return it as an error.
          if (! isWaitedException(e) )
	  {
            updateVerifyListener(false);
	    throw e;
	  }
       }
       else
       {
          throw e;//No exception waited, it is an error.
       }
   }

   finally
   {
    resetExceptionWatch();
    endListentoStub();
   }
}
/**
 *
 */
final protected void listentoStub()
 {
    TestSynchroStub.referenceTest(this);
    //And listen.
    TestSynchroStub.addListener(this);
 }
 /**
  *
  */
final protected void endListentoStub()
 {
    TestSynchroStub.unloadTest(this);
    TestSynchroStub.removeListener(this);
 }
/**
 *
 */
final public void enterStub(StubInfo stub)
 {
 }
/**
 *
 */
final public void exitStub(StubInfo stub)
 {
 }
/**
 *
 */
final public void failStub(StubInfo stub)
 {
 }
/**
 *
 */

 /*******************************************************************************
 * CHECKS ON EXCEPTION
 *
 */
protected Throwable tocatch;
protected boolean exceptionwaited;
final public void verify(Throwable exceptocheck)
{
  if (exceptocheck != null)
    tocatch = exceptocheck;
    exceptionwaited = true;
}
final protected void resetExceptionWatch()
{
  tocatch = null;
  exceptionwaited = false;
}
final protected boolean isWaitedException(Throwable launched)
{

  boolean result = false;
  if (launched != null)
  {
    //Check that the class is the same or compatible.
    if (tocatch!= null)
    {
      Class fromlaunched = launched.getClass();
      Class fromwaited = tocatch.getClass();

      if ( fromwaited.isAssignableFrom(fromlaunched))
      {
        result = true;
      }
      else
      {
        result = false;
      }
    }
    else
    {
      result = false;
    }
  }
  else
  {
    result = true;
  }
  return result;

}

 /*******************************************************************************
 * CHECKS ON TIME
 *
 */
final public int createTimer(String logoncreation)
{
  //And log the message.
  verifyLogMessage(logoncreation);
  //The date will be logged with a Command Start later.
  Date currentdate = new Date();
  fDate.addElement(currentdate);
  return( fDate.size() );//This is the index required to use this date.
}
/**
 *
 */
final protected boolean isvalidtimerid(int timerid)
{
  if ( (timerid > fDate.size()) || (timerid <= 0))
  {
    //No such a created timer.
    //Will be changed to a note in the tdp part.
    verifyLogMessage("no such a timer id");
    return false;
  }
  return true;
}
/**
 *
 */
final public void timerStart(int timerid, String startmessage)
{
  if (isvalidtimerid(timerid))
  {
    //Get current time, and log a message with this time.
    Date current_time = new Date();

    fDate.removeElementAt((timerid-1));
    fDate.insertElementAt(current_time,(timerid-1));
    Integer value = new Integer(timerid);
    String message = "Timer Start " + value.toString() + " " + current_time.toString() + " ";

    if (startmessage != null) message += startmessage;

    //And log the message.
    verifyLogMessage(message);
  }
}
/**
 *
 */
final protected long timerElapsedTime(int timerid)
{
    return timerEllapsedTime(timerid);

}
final protected long timerEllapsedTime(int timerid)
{
    Date current_time = new Date();
    long ellapsed_time;
    try
    {
      Date logged = ((Date)fDate.elementAt((timerid-1)));
      ellapsed_time = (current_time.getTime() - logged.getTime());
    }
    catch( Throwable e)
    {
      return(0);
    }
    return (ellapsed_time);
}
/**
 *
 */
final public void timerReportElapsedTime(int timerid, String ellapsedtimemessage)
{
    timerReportEllapsedTime(timerid,ellapsedtimemessage);
}
final public void timerReportEllapsedTime(int timerid, String ellapsedtimemessage)
{
  if (isvalidtimerid(timerid))
  {
    //Get current time, and log a message with this time.
    long ellapsed_time = timerEllapsedTime(timerid);
    Integer value = new Integer(timerid);
    Long valueellapsed = new Long(ellapsed_time);
    String message = "Timer " + value.toString() + " ellapsed time" + valueellapsed.toString() + " ";
    if (ellapsedtimemessage != null) message += ellapsedtimemessage;
    //And log the message.
    verifyLogMessage(message);
  }
}
/**
 *
 */
final public void verifyElapsedTime(int timerid, long maxtime, String checkellapsedmessage)
{
    verifyEllapsedTime(timerid, maxtime,checkellapsedmessage);
}
final public void verifyEllapsedTime(int timerid, long maxtime, String checkellapsedmessage)
{
  if (isvalidtimerid(timerid))
  {
    long ellapsedtime = timerEllapsedTime(timerid);
    String message = "Check Ellapsed Time ";
    if (checkellapsedmessage != null)
    {
      message += checkellapsedmessage;
    }
    verifyLowerThanEquals(message,maxtime,ellapsedtime);
  }
}

/*******************************************************************************
 * CHECK LOG FAIL
 */
 final public void verifyLogfail(String message)
{
 updateverifyFail(message);
 onCheck();
}
/*******************************************************************************
 * CHECK LOG MESSAGE
 */
 final public void verifyLogMessage(String message)
{
  updateverifyLog(message);
  onCheck();
}
/******************************************************************************
 * CHECK CONDITION
 */
 final public void verifyTrue(boolean condition)
{
  updateverify(null,condition);
  onCheck();
}
/**
 *
 */
 final public void verifyTrue(String message,boolean condition)
{
  updateverify(message,condition);
  onCheck();
}
/*******************************************************************************
 * CHECK EQUALS
 */
 final public void verifyEquals(String message,Object expected, Object actual)
{
  if ((expected==null)&&(actual==null))
  {
    updateverifyEquals(message,expected,actual,true);
  }
  else
  if ((expected!=null)&&(expected.equals(actual)))
  {
    updateverifyEquals(message,expected,actual,true);
  }
  else
  {
    updateverifyEquals(message,expected,actual,false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyEquals(Object expected, Object actual)
{
  verifyEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyEquals(long expected, long actual)
{
  verifyEquals(null,new Long(expected),new Long(actual));
}
/**
 * This method will compare the length for the two vector.
 * And later compare the content.
 */
 final public void verifyEquals(String message, Vector expected, Vector actual)
 {
   if ((expected == null) || (actual==null))
   {
      updateverifyEquals(message,expected,actual,false);
   }

   if ((expected != null) && (actual !=null))
   {
      if (expected.size() != actual.size())
      {
        updateverifyEquals(message,expected,actual,false);
      }
      else
      {
        int result = 1;
        Enumeration enumexpected = expected.elements();
        Enumeration enumactual = actual.elements();
        for ( ; (enumexpected.hasMoreElements()&&enumactual.hasMoreElements());)
        {
          //Get the elements and compare.
          Object in_actual, in_expected;
          in_actual = enumactual.nextElement();
          in_expected = enumexpected.nextElement();

          if ((in_actual!=null)&&(in_expected!=null) )
          {
            if (!(in_actual.equals(in_expected)))
            {
              result = 0;
              break;
            }
          }
          else
          {
            if ((in_actual == null)&&(in_expected==null))
            {
              //These test is good, same objects.
            }
            else
            {
              result=0;
              break;
            }
          }
        }//end for.
        if (result == 1)
        {
          updateverifyEquals(message,expected,actual,true);
        }
        else
        {
          updateverifyEquals(message,expected,actual,false);
        }
      }
   }
   else
   {
    //One null vector.
    updateverifyEquals(message,expected,actual,false);
   }
   onCheck();
}
/**
 *
 */
 final public void verifyEquals(Vector expected, Vector actual)
{
  verifyEquals(null, expected, actual);
}
/**
 *
 */
 final public void verifyEquals(String message,long expected, long actual)
{
  verifyEquals(message,new Long(expected),new Long(actual));
}
/**
 *
 */
 final public void verifyEquals(String message,int expected, int actual)
{
  verifyEquals(message,new Integer(expected),new Integer(actual));
}
/**
 *
 */
 final public void verifyEquals(String message,byte expected, byte actual)
{
  verifyEquals(message,new Byte(expected),new Byte(actual));
}
/**
 *
 */
 final public void verifyEquals(byte expected, byte actual)
{
  verifyEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyEquals(String message,boolean expected, boolean actual)
{
  verifyEquals(message,new Boolean(expected),new Boolean(actual));
}
/**
 *
 */
 final public void verifyEquals(boolean expected, boolean actual)
{
  verifyEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyEquals(int expected, int actual)
{
  verifyEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyEquals(String message,char expected, char actual)
{
  verifyEquals(message,new Character(expected),new Character(actual));
}
/**
 *
 */
 final public void verifyEquals(short expected, short actual)
{
  verifyEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyEquals(String message,short expected, short actual)
{
  verifyEquals(message,new Short(expected),new Short(actual));
}
/**
 *
 */
 final public void verifyEquals(char expected, char actual)
{
  verifyEquals(null,expected,actual);
}
/*******************************************************************************
 * CHECK EQUALS WITH DELTA
 */
final public void verifyApproxEquals(String message, short expected, short actual, short delta)
{
    verifyAproxEquals(message,expected,actual,delta);
}
final public void verifyAproxEquals(String message, short expected, short actual, short delta)
{
   if ((Math.abs(expected-actual) <= delta))
   {
     updateverifyApproxEquals(message,new Short(delta),new Short(actual), new Short(expected), true);
    }
    else
    {
     updateverifyApproxEquals(message,new Short(delta),new Short(actual), new Short(expected), false);
    }
  onCheck();
}
/**
 *
 */
final public void verifyApproxEquals(short expected, short actual, short delta)
{
    verifyAproxEquals(expected,actual,delta);
}
final public void verifyAproxEquals(short expected, short actual, short delta)
{
  verifyAproxEquals(null,expected,actual,delta);
}
/**
 *
 */
final public void verifyApproxEquals(String message, int expected, int actual, int delta)
{
    verifyAproxEquals(message,expected,actual,delta);
}
final public void verifyAproxEquals(String message, int expected, int actual, int delta)
{

   if ((Math.abs(expected-actual) <= delta))
   {
     updateverifyApproxEquals(message,new Integer(delta), new Integer(expected), new Integer(actual),true);
    }
    else
    {
     updateverifyApproxEquals(message,new Integer(delta), new Integer(expected), new Integer(actual),false);
    }
  onCheck();
}
/**
 *
 */
final public void verifyApproxEquals(int expected, int actual, int delta)
{
    verifyAproxEquals(expected,actual,delta);
}
final public void verifyAproxEquals(int expected, int actual, int delta)
{
  verifyAproxEquals(null,expected,actual,delta);
}
/**
 *
 */
final public void verifyApproxEquals(String message, long expected, long actual, long delta)
{
    verifyAproxEquals( message, expected, actual, delta);
}
final public void verifyAproxEquals(String message, long expected, long actual, long delta)
{
   if ((Math.abs(expected-actual) <= delta))
   {
     updateverifyApproxEquals(message,new Long(delta), new Long(expected), new Long(actual),true);
    }
    else
    {
     updateverifyApproxEquals(message,new Long(delta), new Long(expected), new Long(actual),false);
    }
  onCheck();
}
/**
 *
 */
final public void verifyApproxEquals(long expected, long actual, long delta)
{
    verifyAproxEquals( expected,actual,delta);
}
final public void verifyAproxEquals(long expected, long actual, long delta)
{
  verifyAproxEquals(null,expected,actual,delta);
}
/*******************************************************************************
 * CHECK GREATER THAN
 */
 final public void verifyGreaterThan(String message,long expected, long actual)
{
  if (actual > expected)
  {
    updateverifyGreaterThan(message,new Long(expected),new Long(actual),true);
  }
  else
  {
    updateverifyGreaterThan(message,new Long(expected),new Long(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyGreaterThan(String message,short expected, short actual)
{
  if (actual > expected)
  {
    updateverifyGreaterThan(message,new Short(expected),new Short(actual),true);
  }
  else
  {
    updateverifyGreaterThan(message,new Short(expected),new Short(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyGreaterThan(long expected, long actual)
{
  verifyGreaterThan(null,expected,actual);
}
/**
 *
 */
 final public void verifyGreaterThan(String message,int expected, int actual)
{
   if (actual > expected)
    {
      updateverifyGreaterThan(message,new Integer(expected),new Integer(actual),true);
    }
    else
    {
      updateverifyGreaterThan(message,new Integer(expected),new Integer(actual),false);
    }
  onCheck();
}
 /**
  *
  */
 final public void verifyGreaterThan(int expected,int actual)
{
  verifyGreaterThan(null,expected,actual);
}
/**
 *
 */
 final public void verifyGreaterThan(short expected,short actual)
{
  verifyGreaterThan(null,expected,actual);
}
/*******************************************************************************
 * CHECK GREATER THAN EQUALS
 */
 final public void verifyGreaterThanEquals(String message,long expected, long actual)
{
  if (actual >= expected)
  {
    updateverifyGreaterThanEquals(message,new Long(expected),new Long(actual),true);
  }
  else
  {
    updateverifyGreaterThanEquals(message,new Long(expected),new Long(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyGreaterThanEquals(long expected, long actual)
{
  verifyGreaterThanEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyGreaterThanEquals(String message,short expected,short actual)
{
  if (actual >= expected)
  {
    updateverifyGreaterThanEquals(message,new Short(expected),new Short(actual),true);
  }
  else
  {
    updateverifyGreaterThanEquals(message,new Short(expected),new Short(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyGreaterThanEquals(int expected, int actual)
{
  verifyGreaterThanEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyGreaterThanEquals(String message,int expected,int actual)
{
  if (actual  >= expected)
  {
    updateverifyGreaterThanEquals(message,new Integer(expected),new Integer(actual),true);
  }
  else
  {
    updateverifyGreaterThanEquals(message,new Integer(expected),new Integer(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyGreaterThanEquals(short expected, short actual)
{
  verifyGreaterThanEquals(null,expected,actual);
}
/*******************************************************************************
 * CHECK LOWER THAN
 */
 final public void verifyLowerThan(String message,long expected, long actual)
{
  if (expected > actual)
  {
    updateverifyLowerThan(message,new Long(expected),new Long(actual),true);
  }
  else
  {
    updateverifyLowerThan(message,new Long(expected),new Long(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThan(long expected, long actual)
{
  verifyLowerThan(null,expected,actual);
}
/**
 *
 */
 final public void verifyLowerThan(String message,short expected, short actual)
{
  if (expected > actual)
  {
    updateverifyLowerThan(message,new Short(expected),new Short(actual),true);
  }
  else
  {
    updateverifyLowerThan(message,new Short(expected),new Short(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThan(short expected, short actual)
{
  verifyLowerThan(null,expected,actual);
}
/**
 *
 */
 final public void verifyLowerThan(String message,int expected, int actual)
{
  if (expected > actual)
  {
    updateverifyLowerThan(message,new Integer(expected),new Integer(actual),true);
  }
  else
  {
    updateverifyLowerThan(message,new Integer(expected),new Integer(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThan(int expected, int actual)
{
  verifyLowerThan(null,expected,actual);
}
/*******************************************************************************
 * CHECK LOWER THAN EQUALS
 */
 final public void verifyLowerThanEquals(String message,long expected, long actual)
{
  if (expected >= actual)
  {
    updateverifyLowerThanEquals(message,new Long(expected),new Long(actual),true);
  }
  else
  {
    updateverifyLowerThanEquals(message,new Long(expected),new Long(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThanEquals(long expected, long actual)
{
  verifyLowerThanEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyLowerThanEquals(String message,short expected, short actual)
{
  if (expected >= actual)
  {
    updateverifyLowerThanEquals(message,new Short(expected),new Short(actual),true);
  }
  else
  {
    updateverifyLowerThanEquals(message,new Short(expected),new Short(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThanEquals(short expected, short actual)
{
  verifyLowerThanEquals(null,expected,actual);
}
/**
 *
 */
 final public void verifyLowerThanEquals(String message,int expected, int actual)
{
  if (expected >= actual)
  {
    updateverifyLowerThanEquals(message,new Integer(expected),new Integer(actual),true);
  }
  else
  {
    updateverifyLowerThanEquals(message,new Integer(expected),new Integer(actual),false);
  }
  onCheck();
}
/**
 *
 */
 final public void verifyLowerThanEquals(int expected, int actual)
{
  verifyLowerThanEquals(null,expected,actual);
}
/*******************************************************************************
 * CHECK NOT NULL
 */
 final public void verifyNotNull(Object object)
{
  verifyTrue(null, (object!=null));
}
/**
 *
 */
 final public void verifyNotNull(String message, Object object)
{
  verifyTrue(message, (object != null));
}
/*******************************************************************************
 * CHECK NULL
 */
 final public void verifyNull(Object object)
{
  verifyTrue(null,(object==null));
}
/**
 *
 */
 final public void verifyNull(String message,Object object)
{
  verifyTrue(message,(object==null));
}
/*******************************************************************************
 * CHECK SAME
 */
 final public void verifySame(String message, Object expected, Object actual)
{
  updateverifySame(message,expected,actual,(expected==actual));
  onCheck();
}
 /**
  *
  */
 final public void verifySame(Object expected, Object actual)
{
  verifySame(null,expected,actual);
}
/**
 * This method is called every time a verify is made.
 */
final  private synchronized void onCheck()
 {
 }
/**
 *
 */

   final protected void updateVerifyListener(boolean result)
  {

    if (!result)
      addVerifyFailure(this);

  }
/**
 * Update the verify done.
 *
 */
 final protected void updateverify(String message,boolean result)
{
  String formatted = updateresultStr(message,result);
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted,result);
}
/**
 *
 */
 final protected String updateresultStr(String message, boolean result)
{
  String formatted = "";
  if ( message != null)
    formatted = message + " ";
  if (result==true)
    formatted = message + " [true] ";
  else
    formatted = message + " [false] ";

  return formatted;
}
/**
 *
 */
 final protected void updateverifyEquals(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected equals:<" + expected_str + "> and:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyGreaterThan(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected greater:<" + expected_str + "> than:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyGreaterThanEquals(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected greater:<" + expected_str + "> than or equals:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyLowerThan(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected lower:<" + expected_str + "> than:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyLowerThanEquals(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected lower:<" + expected_str + "> than or equals:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyGreaterLowerThanEquals(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected lower:<" + expected_str + "> than or equals:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyApproxEquals(String message,Object delta,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  String delta_str ="null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
if (delta != null) delta_str = delta.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected approx equals:<" + expected_str + "> and:<" + actual_str +"> delta:<" +delta_str + "> ", result );
}
/**
 *
 */
 final protected void updateverifySame(String message,Object expected, Object actual,boolean result)
{
  String formatted = updateresultStr(message,result);
  String expected_str = "null";
  String actual_str = "null";
  if (expected != null) expected_str = expected.toString();
  if (actual != null) actual_str = actual.toString();
  updateVerifyListener(result);
  fCheckdone = new Check(null,formatted + "expected same:<" + expected_str + "> and:<" + actual_str +"> ", result );
}
/**
 *
 */
 final protected void updateverifyFail(String message)
{
  String formatted = "null";
  if (message != null) formatted = "fail:<"+message+">";
  fCheckdone = new Check(null,formatted,false);
  updateVerifyListener(false);
}
/**
 *
 */
 final  protected void updateverifyLog(String message)
{
  String formatted = "null";
  if (message != null) formatted = "log:<"+message+">";
  fCheckdone = new Check(null,formatted,true);
}
 /**
  *
  */
 public final void addVerifyFailure(Test test)
 {
 		for (Enumeration e= cloneListeners().elements(); e.hasMoreElements(); ) {
			((VerifyListener)e.nextElement()).addVerifyFailure(this);
		}
}
	private synchronized Vector cloneListeners() {
	Vector cloneListeners = new Vector();
	Enumeration iterator = fVerifyListener.elements();
	while (iterator.hasMoreElements())
		cloneListeners.addElement(iterator.nextElement());
	return cloneListeners;
	}

}





