
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 

package j2meunit.framework;

public abstract class TestCasePrim extends Assert implements Test{
	protected String testMethodName;
	protected TestResult result;

public TestCasePrim() {

}

public TestCasePrim(String testMethodName) {
	this.testMethodName = testMethodName;
}
/**
 * Counts the number of test cases executed by run(TestResult result).
 */
public int countTestCases() {
	return 1;
}
/**
 * Creates a default TestResult object
 *
 * @see TestResult
 */
protected TestResult createResult() {
	return new TestResult();
}
public String getTestMethodName() {
	return testMethodName;
}
protected void onAssertion()
{
	result.assertionMade();
}
/**
 * A convenience method to run this test, collecting the results with a
 * default TestResult object.
 *
 * @see TestResult
 */
public TestResult run() {
	TestResult result = createResult();
	run(result);
	return result;
}
/**
 * Runs the bare test sequence.
 * @exception Throwable if any exception is thrown
 */
public void runBare() throws Throwable {
	setUp();
	try {
		runTest();
	}
	finally {
		tearDown();
	}
}
/**
 * Override to run the test : Since there is
 * no reflection runTest must test the testMethodName
 * and explicitly call the necessary method.
 * @exception Throwable if any exception is thrown
 */

protected abstract void runTest() throws Throwable;
/**
 * Gets the name of the test case.
 */
public void setTestMethodName(String aTestMethodName) {
	this.testMethodName = aTestMethodName;
}
/**
 * Sets up the fixture, for example, open a network connection.
 * This method is called before a test is executed.
 */
protected void setUp() throws Exception {
}
//
// This method must be overridden and it must return at least one TestSuite.
// This TestSuite should be constructed as below:
// new TestSuite(this.class, new String[] {"...test method name...", "...test method name..."});
//
// For multiple suites do the following:
// TestSuite suite = new TestSuite();
// suite.addTest( new TestSuite(this.class, new String[] {"...test method name...", "...test method name..."});
// suite.addTest(...
// return suite;
// If the suite is for a runall class then it should be constructed
// by suite.addTest(new TestClassName().suite());
//
//
public abstract Test suite();
/**
 * Tears down the fixture, for example, close a network connection.
 * This method is called after a test is executed.
 */
protected void tearDown() throws Exception {
}
/**
 * Returns a string representation of the test case
 */
public String toString() {
	return getTestMethodName() + "(" + getClass().getName() + ")";
}
}
