
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package junit.extensions;

import junit.framework.*;

/**
 * A TestCase that expects an Exception of class fExpected to be thrown.
 * The other way to check that an expected exception is thrown is:
 * <pre>
 * try {
 *   shouldThrow();
 * }
 * catch (SpecialException e) {
 *   return;
 * }
 * fail("Expected SpecialException");
 * </pre>
 *
 * To use ExceptionTestCase, create a TestCase like:
 * <pre>
 * new ExceptionTestCase("testShouldThrow", SpecialException.class);
 * </pre>
 */
public class ExceptionTestCase extends TestCase {
	Class fExpected;

	public ExceptionTestCase(String name, Class exception) {
		super(name);
		fExpected= exception;
	}
	/**
	 * Execute the test method expecting that an Exception of
	 * class fExpected or one of its subclasses will be thrown
	 */
	protected void runTest() throws Throwable {
		try {
			super.runTest();
		}
		catch (Exception e) {
			if (fExpected.isAssignableFrom(e.getClass()))
				return;
			else
				throw e;
		}
		fail("Expected exception " + fExpected);
	}
}