
/////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Licensed Materials - Property of IBM
//  Restricted Material of IBM
//  IBM Rational Test RealTime
//  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
//
//  US Governement Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////////////////////////////

// Product version: 7.0.5.1

/////////////////////////////////////////////////////////////////////// 
//JUnit is a regression testing framework written by Erich           // 
//Gamma and Kent Beck. It is used by the developer who               // 
//implements unit tests in Java.                                     // 
//JUnit is Open Source http://www.opensource.org/ Software,          // 
//released under the IBM's Common Public License Version 0.5         // 
//http://oss.software.ibm.com/developerworks/oss/license-cpl.htm     // 
//l and hosted on SourceForge http://sourceforge.net/projects/junit/.// 
/////////////////////////////////////////////////////////////////////// 
package junit.framework;
import junit.framework.*;
import java.lang.reflect.*;
import java.util.*;

public class TestSuite extends TestSuitePrim {




  public TestSuite() {
  super("null");
  }

  public TestSuite(String name){
    super(name);
  }

  public final synchronized void run(TestResult result)
  {
    super.run(result);
  }
/**
 * Private methods to insert only TestRT compliant classes.
 */
private Constructor getConstructor(Class theClass) throws NoSuchMethodException
{
    Class[] args= { String.class };
    return theClass.getConstructor(args);
}
/**
 *
 */
private boolean isPublicTestMethod(Method m)
{
    return isTestMethod(m) && Modifier.isPublic(m.getModifiers());
}
/**
 *
 */
private boolean isTestMethod(Method m)
{
    String name= m.getName();
    Class[] parameters= m.getParameterTypes();
    Class returnType= m.getReturnType();
    return parameters.length == 0 && name.startsWith("test") && returnType.equals(Void.TYPE);
}
/**
 *
 */
private void addTestMethod(Method m, Vector names, Constructor constructor)
{
    String name= m.getName();
    if (names.contains(name))
	return;
    if (isPublicTestMethod(m)) {
	names.addElement(name);
	Object[] args= new Object[]{name};
	try {
	    addTest((TestCase)constructor.newInstance(args));
	} catch (InstantiationException e) {
	    System.out.println("Can not instanciate test case " + e.getMessage());
	} catch (InvocationTargetException e) {
	    System.out.println("Exception during construction" + e.getMessage());
	} catch (IllegalAccessException e) {
	    System.out.println("Cannot access test case" + e.getMessage());
	}
    } else { // almost a test method
  	if (isTestMethod(m))
	    System.out.println("Test method is not public");
    }
}
/**
 *
 */
public TestSuite(Class theClass)
{
    if (theClass != null)
	{
	    setName(theClass.getName());
	    Constructor constructor= null;
	    try
		{
		    constructor= getConstructor(theClass);
		}
	    catch (Throwable e) {
		System.out.println(e.getMessage());
		return;
	    }

	    if (!Modifier.isPublic(theClass.getModifiers()))
		{
		    System.out.println("No public constructor " + theClass.getName());
		    return;
		}

	    Class superClass= theClass;
	    Vector names= new Vector();
	    while ( (TestCase.class.isAssignableFrom(superClass))) {
		Method[] methods= superClass.getDeclaredMethods();
		for (int i= 0; i < methods.length; i++) {
		    addTestMethod(methods[i], names, constructor);
		}
		superClass= superClass.getSuperclass();
	    }
	    Enumeration e = tests();
	    if (!e.hasMoreElements())
		System.out.println("No test in the suite" + theClass.getName());
	}
    else
	{
	    setName("null");
	}
}
/**
 *
 */
public final void addTest(Test theTest)
{
    if (theTest != null)
	{
	    Class testclass = theTest.getClass();
	    try
		{
		    Class testrtclass = Class.forName("junit.framework.TestCase");
		    Class testrtsuiteclass = Class.forName("junit.framework.TestSuite");

		    //The check compliance.
		    if ( testrtclass.isAssignableFrom(testclass) || testrtsuiteclass.isAssignableFrom(testclass)  )
			{
			    super.addTest(theTest);
			}
		    else
			{
			    System.out.println("AddTest:Class test is not compliant with test rt");
			}

		}
	    catch(Throwable e)
		{
		    System.out.println("Cannot access the class");
		}
	}
}
/**
 *
 */
public final void addTestSuite(final Class thesuite)
{
    //Same purpose.
    addTest( new TestSuite(thesuite));
}

public final void setAsFrame() { }

static public final void lateExit() { }


}







