
/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

/* Product version: 7.0.5.1 */

/*******************************************************************************
* $Id: ats.c,v 1.16.2.1 2008/03/18 10:55:50 build Exp $
* $Descr: System Testing Library Source (included by TP.c[pp]) $
*******************************************************************************/

/*******************************************************************************
* Macros
*******************************************************************************/

/* These macros were formerly customizable */

/* first part of the date in seconds */
#define ATL_DATE_ONE(a) _ATL_TIME_TO_SECONDS(RTRT_CLOCK_TO_TIME(a))

/* second part of the date in another unit */
#define ATL_DATE_TWO(a) _ATL_TIME_TO_MSECONDS(RTRT_CLOCK_TO_TIME(a))

/* conversion between first part and second part of the date */
#define ATL_DATE_ONE_TWO 1000L

/* conversion between second part of the date and micro secondes */
#define ATL_DATE_TWO_MICRO_SEC 1000L

/******************************************************************************/
/* Common part from body_c.c and body_cpp.cc                                  */

#if RTRT_CPLUSPLUS_LANGUAGE == 0

/******************************************************************************/
/* C specifics                                                                */

#define ATS_CTXA ctx,
#define ATS_CTXA1 ctx
#define CTX(a) RTRT_CONCAT_MACRO(ctx->atl_ctx,a)
#define TCTX(a) RTRT_CONCAT_MACRO(ctx->atl_ctxtracectx.atl_ctx,a)
#define STCTX(a,i) TCTX(CYTctx_stack)[i].a
#define ATS_CLP
#define ATS_CLP_ADR(a) a
#define ATS_STATIC RTRT_STATICFCT

#else /* RTRT_CPLUSPLUS_LANGUAGE */

/******************************************************************************/
/* C++ specifics                                                              */

#define ATS_CTXA
#define ATS_CTXA1
#define CTX(a) RTRT_CONCAT_MACRO(atl_ctx,a)
#define TCTX(a) CTX(a)
#define STCTX(a,i) RTRT_CONCAT_MACRO(atl_ctxCYTctx_stack_,a) [i]
#define ATS_CLP ATL_C_RUNTIME::
#define ATS_CLP_ADR(a) &ATS_CLP a
#define ATS_STATIC

#endif /* RTRT_CPLUSPLUS_LANGUAGE */

#define ATL_M_out_sep atl_p_out_str (ATS_CTXA (char*)", ")
#define ATL_M_out_empty_field atl_p_out_str (ATS_CTXA (char*)"?")
#define ATL_K_EMPTY_FIELD (char*)"?"

#define atl_car_isprint(car) ((car >= ' ') && (car <= '~'))

#define ATL_LINE_SIZE(p)         (((*(p+1)) << 2 ) + 2)
#define ATL_END_BUFFERTRACE      (TCTX(buffertrace) + TCTX(buffersize))
#define ATL_K_MAX_RIOLINE_SIZE   ((2 << 8*sizeof(ATL_T_CAR)) - 6)

enum {
  ATL_CYT_MONOLINE,
  ATL_CYT_SPECIAL_MONOLINE,
  ATL_CYT_BEGIN_VIRTUAL_BLOCK,
  ATL_CYT_END_VIRTUAL_BLOCK,
  ATL_CYT_BEGIN_BLOCK,
  ATL_CYT_END_OK_BLOCK,
  ATL_CYT_END_KO_BLOCK,
  ATL_CYT_BEGIN_MULTILINE,
  ATL_CYT_NESTED,
  ATL_CYT_END_OK_MULTILINE,
  ATL_CYT_END_KO_MULTILINE
};

/* New RTRT TDP routines */

static void ats_writeln (RTRT_FILE_TYPE file, ATL_PT_CAR buf)
{
   priv_writeln (file, buf);
   priv_writeln (file, (char*)"\n");
}

#ifdef _ATL_EXIT_PENDING

static void ats_exit_pending ()
{
}

#else

static void ats_late_exit ()
{
}

#endif /* _ATL_EXIT_PENDING */

static void ats_early_init ()
{
}

/* Common routines from body_c.c and body_cpp.cc */

#if RTRT_CPLUSPLUS_LANGUAGE
#define GENERIC_FLUSH_LINE (this->*atl_ctxgeneric_flush_line) ()
#else
#define GENERIC_FLUSH_LINE (*TCTX(generic_flush_line)) (ctx)
#endif

#define ATS_CYT_FLUSH(p) \
  if (TCTX(tracepriority) == K_TRACECTX_CIRCULAR_PRIORITY) \
    TCTX(CYTstatus) = p; \
  GENERIC_FLUSH_LINE

#if ATS_RT_BUFFER

/** implementation of a proprietary hexa format  **/
/** (prefixing the data by associated format)    **/
/** inverted print order: lsq first :87654321    **/
/** Encoding example of 12345678 => ihgfedcb     **/

#define ATL_K_FIRST_HEXA        0x61
#define ATL_K_BASE_CODE          0x1
#define ATL_K_CODE_SHIFTING      '@'
#define ATL_K_BUFRT_FLOAT_CODE    20
#define ATL_K_BUFRT_DOUBLE_CODE   21

#define ATL_DUMP_OBJ       ATL_T_ULONG
#define ATL_K_RATIO_REAL   ((sizeof(ATL_T_REAL)+sizeof(ATL_DUMP_OBJ)-1)/sizeof(ATL_DUMP_OBJ))
#define ATL_K_RATIO_DOUBLE ((sizeof(ATL_T_DOUBLE)+sizeof(ATL_DUMP_OBJ)-1)/sizeof(ATL_DUMP_OBJ))

typedef union {
  ATL_T_DOUBLE d;
  struct {
    ATL_T_REAL f2;
    ATL_T_REAL f1;
  } f;
  struct {
    ATL_T_LONG l2;
    ATL_T_LONG l1;
  } s;
} ATL_BUFRT_CODING_TYPE;

typedef union {
  ATL_T_REAL    f;
  ATL_DUMP_OBJ  N[ATL_K_RATIO_REAL];
} ATL_BUFRT_CONV_FLOAT;

typedef union {
  ATL_T_DOUBLE  d;
  ATL_DUMP_OBJ  N[ATL_K_RATIO_DOUBLE];
} ATL_BUFRT_CONV_DOUBLE;

#define TEST_SMALL_VALUE_CASE(s,idx,val) {   \
 if ((base==10) && (val>=0) && (val<10))     \
  {                                          \
    s[idx++] = '0'+(ATL_T_UCAR)val;          \
    s[idx] = '\0';                           \
    return (s);                              \
  }                                          \
}

#define ENCODE_BASE(s,idx,b) {            \
  s[idx++] = ATL_K_BASE_CODE;             \
  s[idx++] = ATL_K_CODE_SHIFTING + b ;    \
}

#define ENCODE_TWO_QUARTETS(s,idx,val) {                            \
  s[idx++] = (ATL_T_UCAR)(ATL_K_FIRST_HEXA+((val) & 0x0F));         \
  s[idx++] = (ATL_T_UCAR)(ATL_K_FIRST_HEXA+(((val) >> 4) & 0x0F));  \
}

#define FINISH_STRING(s,i) { \
 s[i] = '\0';                \
}

/** function aimed at decoding ONE "rio" parameter (pseudo-hexa encoded) **/
/** area to be decoded : pointed by (p)                                  **/
/** recipient area     : pointed by (p2)                                 **/
/** Updated pointers in both areas : (adr_succ_p) and (adr_succ_p2)      **/
ATS_STATIC void ATS_CLP atl_p_decode
#if !RTRT_KR
  (ATL_PT_CAR p,
   ATL_PT_CAR p2,
   ATL_T_CAR **adr_succ_p,
   ATL_T_CAR **adr_succ_p2)
#else
  (p,p2,adr_succ_p,adr_succ_p2)
  ATL_PT_CAR p ATL_SP;
  ATL_PT_CAR p2 ATL_SP;
  ATL_T_CAR **adr_succ_p ATL_SP;
  ATL_T_CAR **adr_succ_p2 ATL_EP;
#endif
{
  ATL_PT_CAR pp;
  ATL_T_INT idx,len,base;
  ATL_BUFRT_CODING_TYPE value;

  len = 0;
  value.s.l1=0;
  value.s.l2=0;
  p++;
  base=(ATL_T_INT)((*p) - ATL_K_CODE_SHIFTING);
  p++;

  switch (base)
    {
    case ATL_K_BUFRT_FLOAT_CODE:
      {
        ATL_DUMP_OBJ i_type;
	ATL_T_INT k;
	ATL_BUFRT_CONV_FLOAT conv_float;

	for (k=0;k<ATL_K_RATIO_REAL;k++)
	  {
	    conv_float.N[k] =0;
	    for (i_type=1;(i_type!=0); i_type<<=4)
	      {
		conv_float.N[k] |= (i_type * ((*p)- ATL_K_FIRST_HEXA));
                p++;
	      }
	  }
	value.f.f1 = conv_float.f;
      }
      break;

    case ATL_K_BUFRT_DOUBLE_CODE:
      {
	ATL_T_INT k;
        ATL_DUMP_OBJ i_type;
	ATL_BUFRT_CONV_DOUBLE conv_double;

	for (k=0;k<ATL_K_RATIO_DOUBLE;k++)
	  {
	    conv_double.N[k] =0;
	    for (i_type=1;(i_type!=0); i_type<<=4)
	      {
		conv_double.N[k] |= (i_type * ((*p)- ATL_K_FIRST_HEXA));
                p++;
	      }
      	  }
	value.d = conv_double.d;
      }
      break;

    default:
      {
	while(((*p)>=ATL_K_FIRST_HEXA) && ((*p)<=(ATL_K_FIRST_HEXA+16)))
	  {
	    if (len >= 8*sizeof(ATL_T_LONG))
	      {
		value.s.l2|=( ((ATL_T_LONG)((*p) - ATL_K_FIRST_HEXA)) << (len-8*sizeof(ATL_T_LONG)));
	      }
	    else
	      {
		value.s.l1|=( ((ATL_T_LONG)((*p) - ATL_K_FIRST_HEXA)) <<len);
	      }
	    p++;
	    len+=4;
	  }
      }
    } /* end switch (base) */

  switch (base)
    {
    case 2:
      pp = p2;
      p2 += len;
      idx = 0;
      while(idx<len)
	{
	  if(value.s.l1 & (((ATL_T_LONG) 1) << (len - idx - 1)))
	    pp[idx++] = '1';
	  else
	    pp[idx++] = '0';
	}
      pp[idx] = '\0';
      break;
    case 8:
      p2+=atl_sprintf (p2, (char*)"0%o", value.s.l1);
      break;
    case 10:
      p2+=atl_sprintf (p2, (char*)"%d", value.s.l1);
      break;
    case 16:
      p2+=atl_sprintf (p2, (char*)"0x%X", value.s.l1);
      break;
    case ATL_K_BUFRT_FLOAT_CODE:
      p2+=atl_sprintf (p2, (char*)"%E", value.f.f1);
      break;
    case ATL_K_BUFRT_DOUBLE_CODE:
      p2+=atl_sprintf (p2, (char*)"%E", value.d);
      break;
    default:
      priv_exit (-11);;
    }
  *adr_succ_p  = p;
  *adr_succ_p2 = p2;
}

/** function aimed at decoding a complete pseudo-hexa encoded RIO line **/
ATS_STATIC void ATS_CLP atl_p_toplvl_decode
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR begin)
#else
  (ctx, begin)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR begin;
#endif
{
  ATL_T_CAR *p,*p2;
  ATL_T_CAR locbuf[ATS_BUFFER_SIZE];

  p  = begin;
  p2 = locbuf;
  while (*p)
    {
      if(*p == ATL_K_BASE_CODE)
	{
	  atl_p_decode(p,p2,&p,&p2);
	}
      else
	{
	  *p2++=*p++;
	}
    }
  *p2 = '\0';
  ats_writeln (CTX(fdrio), locbuf);
}
#endif /* #if ATS_RT_BUFFER */


ATS_STATIC void ATS_CLP atl_bufindex_error
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  CTX(bufindex) = 0;
  atl_p_begin_line (ATS_CTXA (char*)"x", 0);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"output buffer overflow");

  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);

  atl_flush_circular_buffer (ATS_CTXA K_TRACECTX_MAX_PRIORITY);
  priv_close (CTX(fdrio));
  priv_exit (-2);;
}

ATS_STATIC void ATS_CLP atl_bin2ascii
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_CAR car,
   ATL_PT_CAR dest)
#else
  (ctx, car, dest)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_CAR car;
  ATL_PT_CAR dest;
#endif
{
  ATL_T_INT i;

  i = 0;
  dest[i++] = '\\';
  switch ((ATL_T_INT) car)
    {
    case ('\n'):
      {
        dest[i++] = 'n';
        break;
      }
    case ('\r'):
      {
        dest[i++] = 'r';
        break;
      }
    case ('\t'):
      {
        dest[i++] = 't';
        break;
      }
    case ('\f'):
      {
        dest[i++] = 'f';
        break;
      }
    default:
      {
        dest[i++] = (((car) & 0300) >> 6) + '0';
        dest[i++] = (((car) & 0070) >> 3) + '0';
        dest[i++] = ((car) & 0007) + '0';
      }
    }
  dest[i] = '\0';
}

ATS_STATIC void ATS_CLP atl_p_inc_bufindex
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  if (CTX(bufindex) >= (ATS_BUFFER_SIZE - 1))
  {
    atl_bufindex_error (ATS_CTXA1);
  }
  else
  {
    CTX(bufindex)++;
  }
}


ATS_STATIC void ATS_CLP atl_p_out_bin
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_CAR car)
#else
  (ctx, car)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_CAR car;
#endif
{
  if (CTX(bufindex) >= (ATS_BUFFER_SIZE - 4))
  {
    atl_bufindex_error (ATS_CTXA1);
  }
  CTX(bufout)[CTX(bufindex)++] = '\\';
  switch ((ATL_T_INT) car)
  {
    case ('\n'):
      {
        CTX(bufout)[CTX(bufindex)++] = 'n';
        break;
      }
    case ('\r'):
      {
        CTX(bufout)[CTX(bufindex)++] = 'r';
        break;
      }
    case ('\t'):
      {
        CTX(bufout)[CTX(bufindex)++] = 't';
        break;
      }
    case ('\f'):
      {
        CTX(bufout)[CTX(bufindex)++] = 'f';
        break;
      }
    default:
      {
        CTX(bufout)[CTX(bufindex)++] = (((car) & 0300) >> 6) + '0';
        CTX(bufout)[CTX(bufindex)++] = (((car) & 0070) >> 3) + '0';
        CTX(bufout)[CTX(bufindex)++] = ((car) & 0007) + '0';
      }
  }
}

ATS_STATIC void ATS_CLP atl_p_out_car
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_CAR car)
#else
  (ctx, car)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_CAR car;
#endif
{
  if (CTX(bufindex) >= (ATS_BUFFER_SIZE - 2))
  {
    atl_bufindex_error (ATS_CTXA1);
  }
  if (atl_car_isprint (car))
  {
    if (car == '"')
    {
      CTX(bufout)[CTX(bufindex)++] = car;
    }
    CTX(bufout)[CTX(bufindex)++] = car;
  }
  else
  {
    atl_p_out_bin (ATS_CTXA car);
  }
  CTX(bufout)[CTX(bufindex)] = '\0';
}

ATS_STATIC void ATS_CLP atl_p_out_str
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR str)
#else
  (ctx, str)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR str;
#endif
{
  while (*str)
  {
    CTX(bufout)[CTX(bufindex)] = (*str++);
    atl_p_inc_bufindex (ATS_CTXA1);
  }
  CTX(bufout)[CTX(bufindex)] = '\0';
}

ATS_STATIC void ATS_CLP atl_p_out_quot
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR str)
#else
  (ctx, str)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR str;
#endif
{
  CTX(bufout)[CTX(bufindex)] = '"';
  atl_p_inc_bufindex (ATS_CTXA1);
  while (*str)
  {
    if (*str == '"')
    {
      CTX(bufout)[CTX(bufindex)] = '"';
      atl_p_inc_bufindex (ATS_CTXA1);
    }
    CTX(bufout)[CTX(bufindex)] = (*str++);
    atl_p_inc_bufindex (ATS_CTXA1);
  }
  CTX(bufout)[CTX(bufindex)] = '"';
  atl_p_inc_bufindex (ATS_CTXA1);
  CTX(bufout)[CTX(bufindex)] = '\0';
}

ATS_STATIC void ATS_CLP atl_p_flush_line
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  if (CTX(rio_onoff))
  {
#if ATS_RT_BUFFER
    atl_p_toplvl_decode(ATS_CTXA CTX(bufout));
#else
    ats_writeln (CTX(fdrio), CTX(bufout));
#endif
  }
  CTX(bufout)[0] = '\0';
  CTX(bufindex) = 0;
}


#if ATS_DYNFLW
ATS_STATIC void ATS_CLP atl_p_raise_dynfline
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR suppl_info,
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT tsffileindex)
#else
  (ctx, suppl_info, tsfindex, tsffileindex)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR suppl_info;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT tsffileindex;
#endif
{
  char buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      if (suppl_info==NULL)
	{
	  atl_sprintf(buffer, (char*)"%s, \"%d\", \"%d\", NONE\r\n", CTX(bufout), tsfindex, tsffileindex);
	}
      else
	{
	  atl_sprintf(buffer, (char*)"%s, \"%d\", \"%d\", %s\r\n",
		      CTX(bufout), tsfindex, tsffileindex, suppl_info);
	}
      priv_raiseline_tagtlk (&(CTX(tagtsocket)), buffer);
    }
}
#endif


ATS_STATIC void ATS_CLP atl_p_stackblock
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR dest,
   ATL_CONST ATL_T_INT raz)
#else
  (ctx, dest, raz)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR dest;
  ATL_CONST ATL_T_INT raz;
#endif
{
  atl_strcpy (dest, CTX(bufout));
  if (raz)
  {
    CTX(bufout)[0] = '\0';
    CTX(bufindex) = 0;
  }
}


ATS_STATIC ATL_PT_CAR ATS_CLP atl_car_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR str,
   ATL_PT_CAR dest)
#else
  (ctx, str, dest)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR str;
  ATL_PT_CAR dest;
#endif
{
  ATL_T_CAR ascii[5];
  ATL_PT_CAR pascii;
  ATL_PT_CAR pdest;

  pdest = dest;
  pascii = ascii;
  (*pdest++) = '\'';
  pascii = ascii;
  if (atl_car_isprint (*str))
  {
    (*pdest++) = *str;
  }
  else
  {
    atl_bin2ascii (ATS_CTXA *str, ascii);
    while (*pascii)
    {
      (*pdest++) = (*pascii++);
    }
  }
  (*pdest++) = '\'';
  *pdest = '\0';
  return (dest);
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_string_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR str,
   ATL_PT_CAR dest,
   ATL_T_INT maxlen)
#else
  (ctx, str, dest)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR str;
  ATL_PT_CAR dest;
  ATL_T_INT maxlen;
#endif
{
  ATL_PT_CAR src;
  ATL_T_CAR ascii[5];
  ATL_PT_CAR pascii;
  ATL_PT_CAR pdest;
  ATL_T_INT len;

  src = str;
  if (src)
    {
      pdest = dest;
      (*pdest++) = '"';
	  len=maxlen-10; /* keep space for security */
      while ((ATL_T_INT) (*src) && (len>0))
        {
          if (atl_car_isprint (*src))
            {
              (*pdest++) = *src;
              len--;
            }
          else
            {
              atl_bin2ascii (ATS_CTXA *src, ascii);
              pascii = ascii;
              while (*pascii)
                {
                  (*pdest++) = (*pascii++);
                  len--;
                }
            }
          ++src;
        }
      (*pdest++) = '"';
      *pdest = '\0';
    }
  else
    {
      pdest = atl_pointer_image (ATS_CTXA (ATL_PT_VOID) 0, dest, 16);
    }
  return (dest);
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_pointer_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_VOID ptr,
   ATL_PT_CAR str,
   ATL_CONST ATL_T_INT base)
#else
  (ctx, ptr, str, base)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_VOID ptr;
  ATL_PT_CAR str;
  ATL_CONST ATL_T_INT base;
#endif
{
#if ATS_RT_BUFFER /** Pointer : encoding version **/
  ATL_T_INT i,index;
  index=0;
  TEST_SMALL_VALUE_CASE(str,index,(ATL_T_LONG)ptr)
  ENCODE_BASE(str,index,base)
  i=0;
  do {
    ENCODE_TWO_QUARTETS(str,index, ((ATL_T_LONG)ptr) >> (i*8))
    i++;
  } while( (i<sizeof(ATL_PT_VOID)) && (((ATL_T_LONG)ptr >> (i*8))!=0) );
  FINISH_STRING(str,index)
  return (str);
#else
  ATL_T_INT idx;
  ATL_PT_CAR format;

  if (base==2)
    {
      ATL_T_INT i,byte,printing;
      ATL_T_UCAR oneByte;

      idx = 0;
      printing = 0;

      for (byte = 0; byte < sizeof (ATL_PT_VOID); byte++)
	{
	  oneByte = (ATL_T_UCAR)( (ATL_T_ULONG)ptr >> (8*(sizeof(ATL_PT_VOID)-1-byte))) & 0xFF;

	  if ( (oneByte!=0) || ((! printing) && (byte == (sizeof (ATL_PT_VOID) -1) )))
	    printing=1;

	  if (printing)
	    {
	      for (i=7;i>=0;i--)
		{
		  str[idx+(7-i)] = '0'+ ((oneByte >> i) & 1);
		}
	      idx += 8;
	    }
	}
      str[idx] = '\0';
    }
  else
    {
      switch (base)
	{
	case 10:
	  {
	    format = (char *)"%ld";
	    break;
	  }
	case 16:
	  {
	    format = (char *)"0x%X";
	    break;
	  }
	case 8:
	  {
	    format = (char *)"0%o";
	    break;
	  }
	}
      atl_sprintf (str, format, ptr);
    }
  return (str);
#endif
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_long_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_LONG value,
   ATL_PT_CAR str,
   ATL_CONST ATL_T_INT base)
#else
  (ctx, value, str, base)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_LONG value;
  ATL_PT_CAR str;
  ATL_CONST ATL_T_INT base;
#endif
{
#if ATS_RT_BUFFER /** Long : encoding version **/
  ATL_T_INT i,index;
  index=0;
  TEST_SMALL_VALUE_CASE(str,index,value)
  ENCODE_BASE(str,index,base)
  i=0;
  do {
    ENCODE_TWO_QUARTETS(str,index, (value >> (i*8)))
    i++;
  } while( (i<sizeof(ATL_T_LONG)) && ((value >> (i*8))!=0) );
  FINISH_STRING(str,index)
  return (str);
#else
  ATL_PT_CAR format;
  ATL_T_INT idx;

  if (base==2)
    {
      ATL_T_INT i,byte,printing;
      ATL_T_UCAR oneByte;

      idx = 0;
      printing = 0;

      for (byte = 0; byte < sizeof (ATL_T_LONG); byte++)
	{
	  oneByte = (ATL_T_UCAR)( (ATL_T_ULONG)value >> (8*(sizeof(ATL_T_LONG)-1-byte))) & 0xFF;

	  if ( (oneByte!=0) || ((! printing) && (byte == (sizeof (ATL_T_LONG) -1) )))
	    printing=1;

	  if (printing)
	    {
	      for (i=7;i>=0;i--)
		{
		  str[idx+(7-i)] = '0'+ ((oneByte >> i) & 1);
		}
	      idx += 8;
	    }
	}
      str[idx] = '\0';
    }
  else
    {
      switch (base)
        {
        case 10:
          {
            format = (char *)"%ld";
            break;
          }
        case 16:
          {
            format = (char *)"0x%X";
            break;
          }
        case 8:
          {
            format = (char *)"0%o";
            break;
          }
        }
      atl_sprintf (str, format, value);
    }
  return (str);
#endif
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_byte_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_UCAR value,
   ATL_PT_CAR str,
   ATL_CONST ATL_T_INT base)
#else
  (ctx, value, str, base)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_UCAR value;
  ATL_PT_CAR str;
  ATL_CONST ATL_T_INT base;
#endif
{
#if ATS_RT_BUFFER  /** Byte : encoding version **/
  ATL_T_INT index=0;
  ENCODE_BASE(str,index,base)
  ENCODE_TWO_QUARTETS(str,index,value)
  FINISH_STRING(str,index)
  return (str);
#else
  ATL_PT_CAR format;
  ATL_T_LONG tmp;
  ATL_T_INT idx;

  if (base == 2)
    {
      for (idx = 0, tmp = value; idx < 8 * sizeof (ATL_T_UCAR); idx++, tmp = tmp << 1)
        {
          if (tmp & (1 << (8 * sizeof (ATL_T_UCAR) - 1)))
            str[idx] = '1';
          else
            str[idx] = '0';
        }
      str[idx] = '\0';
    }
  else
    {
      switch (base)
        {
        case 10:
          {
            format = (char *)"%d";
            break;
          }
        case 16:
          {
            format = (char *)"0x%X";
            break;
          }
        case 8:
          {
            format = (char *)"0%o";
            break;
          }
        }
      atl_sprintf (str, format, value);
    }
  return (str);
#endif
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_int_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT value,
   ATL_PT_CAR str,
   ATL_CONST ATL_T_INT base)
#else
  (ctx, value, str, base)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT value;
  ATL_PT_CAR str;
  ATL_CONST ATL_T_INT base;
#endif
{
#if ATS_RT_BUFFER /** Int : encoding version **/
  ATL_T_INT i,index;
  index=0;
  TEST_SMALL_VALUE_CASE(str,index,value)
  ENCODE_BASE(str,index,base)
  i=0;
  do {
    ENCODE_TWO_QUARTETS(str,index, (value>>(i*8)))
    i++;
  } while( (i<sizeof(ATL_T_INT)) && ((value>>(i*8))!=0) );
  FINISH_STRING(str,index)
  return (str);
#else
  ATL_PT_CAR format;
  ATL_T_INT idx;

  if (base==2)
    {
      ATL_T_INT i,byte,printing;
      ATL_T_UCAR oneByte;

      idx = 0;
      printing = 0;

      for (byte = 0; byte < sizeof (ATL_T_INT); byte++)
	{
	  oneByte = (ATL_T_UCAR)( (ATL_T_ULONG)value >> (8*(sizeof(ATL_T_INT)-1-byte))) & 0xFF;

	  if ( (oneByte!=0) || ((! printing) && (byte == (sizeof (ATL_T_INT) -1) )))
	    printing=1;

	  if (printing)
	    {
	      for (i=7;i>=0;i--)
		{
		  str[idx+(7-i)] = '0'+ ((oneByte >> i) & 1);
		}
	      idx += 8;
	    }
	}
      str[idx] = '\0';
    }
  else
    {
      switch (base)
        {
        case 10:
          {
            format = (char *)"%d"; /* "%ld" RATLC00515543 */
            break;
          }
        case 16:
          {
            format = (char *)"0x%X";
            break;
          }
        case 8:
          {
            format = (char *)"0%o";
            break;
          }
        }
      atl_sprintf (str, format, value);
    }
  return (str);
#endif
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_float_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_REAL f_value,
   ATL_PT_CAR str)
#else
  (ctx, f_value, str)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_REAL f_value;
  ATL_PT_CAR str;
#endif
{
#if ATS_RT_BUFFER /** Float: encoding version **/
  ATL_T_INT k,index;
  ATL_DUMP_OBJ i_type;
  ATL_BUFRT_CONV_FLOAT conv;
  index = 0;

  ENCODE_BASE(str,index,ATL_K_BUFRT_FLOAT_CODE)

  conv.N[0]=0;
  conv.N[ATL_K_RATIO_REAL-1]=0;
  conv.f = f_value;

  for (k=0;k<ATL_K_RATIO_REAL;k++)
    {
      for (i_type=1;(i_type!=0); conv.N[k]>>=4,i_type<<=4)
	{
          str[index]=(ATL_T_UCAR)(ATL_K_FIRST_HEXA+ (conv.N[k]&0x0f));
          index++;
	}
    }
  FINISH_STRING(str,index)
  return (str);
#else
  atl_sprintf (str, (char*)"%E", f_value);
  return (str);
#endif
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_double_image
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_DOUBLE d_value,
   ATL_PT_CAR str)
#else
  (ctx, d_value, str)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_DOUBLE d_value;
  ATL_PT_CAR str;
#endif
{
#if ATS_RT_BUFFER /** Double: encoding version **/
  ATL_DUMP_OBJ i_type;
  ATL_T_INT k,index;
  ATL_BUFRT_CONV_DOUBLE conv;

  index = 0;
  ENCODE_BASE(str,index,ATL_K_BUFRT_DOUBLE_CODE)

  conv.N[0]=0;
  conv.N[ATL_K_RATIO_DOUBLE-1]=0;
  conv.d = d_value;

  for (k=0;k<ATL_K_RATIO_DOUBLE;k++)
    {
      for (i_type=1;(i_type!=0); conv.N[k]>>=4,i_type<<=4)
	{
          str[index]=(ATL_T_UCAR)(ATL_K_FIRST_HEXA+ (conv.N[k]&0x0f));
          index++;
	}
    }
  FINISH_STRING(str,index)
  return (str);
#else
  atl_sprintf (str, (char*)"%E", d_value);
  return (str);
#endif
}

ATS_STATIC void ATS_CLP atl_p_begin_line
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR pcar,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, pcar, index)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR pcar;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_out_str (ATS_CTXA pcar);
  atl_p_out_str (ATS_CTXA atl_int_image (ATS_CTXA index, str, 10));
}

ATS_STATIC void ATS_CLP atl_hexa_image
#if !RTRT_KR
  (ATL_CONST ATL_T_UCAR value,
   ATL_PT_CAR hex1,
   ATL_PT_CAR hex2)
#else
  (value, hex1, hex2)
  ATL_CONST ATL_T_UCAR value;
  ATL_PT_CAR hex1;
  ATL_PT_CAR hex2;
#endif
{
   static char i2a[]="0123456789ABCDEF";
   *hex1 =i2a[(value>>4)&0x0f];
   *hex2 =i2a[(value )&0x0f];
}

#if !RTRT_CPLUSPLUS_LANGUAGE
static void atl_p_erasecleanup
#if !RTRT_KR
  (ATL_PT_RUNCTX ctx)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  if (!CTX(lock))
  {
    while (CTX(indexcleanup))
    {
      --CTX(indexcleanup);
    }
  }
}
#endif

ATS_STATIC void ATS_CLP atl_p_range
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT min,
   ATL_CONST ATL_T_INT max,
   ATL_CONST ATL_T_INT state)
#else
  (ctx, min, max, state)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT min;
  ATL_CONST ATL_T_INT max;
  ATL_CONST ATL_T_INT state;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  if (state)
    {
      ATL_M_out_empty_field;
      ATL_M_out_sep;
      ATL_M_out_empty_field;
      ATL_M_out_sep;
    }
  else
    {
      atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA min, str, 10));
      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA max, str, 10));
      ATL_M_out_sep;
    }
}

#if RTRT_CPLUSPLUS_LANGUAGE
void ATS_CLP atl_var_catch
  (const ATL_T_INT index)
{
  atl_p_begin_line ((char*)"i", index);
  atl_p_out_car (' ');
  atl_p_out_quot ((char*)"1");
  atl_ctxflagrt = 1;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  atl_p_out_quot ((char*)"exception");

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_catch
  ()
{
  atl_ctxflagrt = 1;
}

#endif

ATS_STATIC void ATS_CLP atl_p_var_elem
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_PT_CAR v_rec,
   ATL_PT_CAR v_expect,
   ATL_CONST ATL_T_INT idxcur,
   ATL_CONST ATL_T_INT idxmax,
   ATL_CONST ATL_T_INT passed)
#else
  (ctx, index, v_rec, v_expect, idxcur, idxmax, passed)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_PT_CAR v_rec;
  ATL_PT_CAR v_expect;
  ATL_CONST ATL_T_INT idxcur;
  ATL_CONST ATL_T_INT idxmax;
  ATL_CONST ATL_T_INT passed;
#endif
{
  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    atl_strcpy (CTX(varrec), v_rec);
    atl_strcpy (CTX(varexpect), v_expect);
    CTX(idxmin) = idxcur;
    CTX(lastcmp) = 2;
  }
  if (
       (atl_strcmp (CTX(varexpect), v_expect) ||
        atl_strcmp (v_rec, CTX(varrec)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", index);
    atl_p_out_car (ATS_CTXA ' ');
    if (CTX(lastcmp))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxcur - 1, 0);
    atl_p_out_quot (ATS_CTXA CTX(varexpect));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA CTX(varrec));
    ATS_CYT_FLUSH(ATL_CYT_NESTED);
    atl_strcpy (CTX(varrec), v_rec);
    atl_strcpy (CTX(varexpect), v_expect);
    CTX(idxmin) = idxcur;
  }
  CTX(lastcmp) = passed;
  if ((idxmax == -1) || (idxcur == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", index);
    atl_p_out_car (ATS_CTXA ' ');
    if (passed)
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    atl_p_out_quot (ATS_CTXA v_expect);
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA v_rec);
    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

ATS_STATIC void ATS_CLP atl_p_cmp_elem
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_PT_CAR v_rec,
   ATL_CONST ATL_PT_CAR v_expect,
   ATL_CONST ATL_T_INT idxcur,
   ATL_CONST ATL_T_INT idxmax,
   ATL_CONST ATL_T_INT passed)
#else
  (ctx, index, v_rec, v_expect, idxcur, idxmax, passed)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_PT_CAR v_rec;
  ATL_CONST ATL_PT_CAR v_expect;
  ATL_CONST ATL_T_INT idxcur;
  ATL_CONST ATL_T_INT idxmax;
  ATL_CONST ATL_T_INT passed;
#endif
{
  if ((CTX(lastcmp) == 2) || (idxmax == -1))
    {
      atl_strcpy (CTX(varrec), v_rec);
      atl_strcpy (CTX(varexpect), v_expect);
      CTX(idxmin) = idxcur;
      CTX(lastcmp) = 2;
    }
  if (
       (atl_strcmp (CTX(varexpect), v_expect) ||
        atl_strcmp (v_rec, CTX(varrec)))
       &&
       (CTX(lastcmp) != 2)
    )
    {
      if (!CTX(lastcmp))
        {
          CTX(flagrt) = 1;
        }
      atl_strcpy (CTX(varrec), v_rec);
      atl_strcpy (CTX(varexpect), v_expect);
      CTX(idxmin) = idxcur;
    }
  CTX(lastcmp) = passed;
  if ((idxmax == -1) || (idxcur == idxmax))
    {
      if (!passed)
        {
          CTX(flagrt) = 1;
        }
    }
}

ATS_STATIC void ATS_CLP atl_p_dmp_elem
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_PT_CAR v_rec,
   ATL_CONST ATL_T_INT idxcur,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, index, v_rec, idxcur, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_PT_CAR v_rec;
  ATL_CONST ATL_T_INT idxcur;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  if ((idxmax == -1) || (CTX(lastcmp) == 2))
  {
    atl_strcpy (CTX(varrec), v_rec);
    CTX(idxmin) = idxcur;
    CTX(lastcmp) = 0;
  }
  else
  {
    if (atl_strcmp (v_rec, CTX(varrec)))
    {
      atl_p_begin_line (ATS_CTXA (char*)"i", index);
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"0");
      ATL_M_out_sep;
      atl_p_range (ATS_CTXA CTX(idxmin), idxcur - 1, 0);
      ATL_M_out_empty_field;
      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA CTX(varrec));
      ATS_CYT_FLUSH(ATL_CYT_NESTED);
      atl_strcpy (CTX(varrec), v_rec);
      CTX(idxmin) = idxcur;
    }
  }
  if ((idxmax == -1) || (idxcur == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", index);
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"0");
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    ATL_M_out_empty_field;
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA v_rec);
    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

void ATS_CLP atl_rio_on
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  CTX(rio_onoff) = 1;
}

void ATS_CLP atl_rio_off
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  CTX(rio_onoff) = 0;
}

void ATS_CLP atl_trace_on
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_ULONG dtron;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  if (CTX(trace_onoff)==0)
    {
      atl_p_begin_line (ATS_CTXA (char*)"tn", index);
      atl_p_out_car (ATS_CTXA ' ');
      dtron = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	      - CTX(duration)[CTX(level)];
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA dtron, str, 10));

      ATS_CYT_FLUSH(ATL_CYT_SPECIAL_MONOLINE);
      CTX(trace_onoff) = 1;
    }
}

void ATS_CLP atl_trace_off
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_ULONG dtrof;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  if (CTX(trace_onoff) == 1)
    {
      atl_p_begin_line (ATS_CTXA (char*)"tf", index);
      atl_p_out_car (ATS_CTXA ' ');
      dtrof = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	      - CTX(duration)[CTX(level)];
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA dtrof, str, 10));

      ATS_CYT_FLUSH(ATL_CYT_SPECIAL_MONOLINE);
      CTX(trace_onoff) = 0;
    }
}

void ATS_CLP atl_flush_trace
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_ULONG dftrace;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  /** force the printing of the 'ft' lines **/
  TCTX(currentcodepriority) = 0;

  atl_p_begin_line (ATS_CTXA (char*)"ft", index);
  atl_p_out_car (ATS_CTXA ' ');
  dftrace = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	    - CTX(duration)[CTX(level)];
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA dftrace, str, 10));

  ATS_CYT_FLUSH(ATL_CYT_SPECIAL_MONOLINE);

  /** remember that a FT occured **/
  if (TCTX(CYTftoccured) < 2)
    TCTX(CYTftoccured)++;

  TCTX(CYTft_status) = 1;

  /** store current stack for eventual & artificial closing of the current block **/
  TCTX(CYTlasttracelevel_forclosure)[1] = CTX(tracelevel);
  atl_p_CYTcopystack (ATS_CTXA1);
}

#ifdef ATL_SIGNAL
RTRT_GLOBALFCT void atl_handler
#if !RTRT_KR
  (ATL_T_INT foo)
#else
  (foo)
  ATL_T_INT foo;
#endif
{
  char str[50];
  atl_sprintf (str, (char*)"signal %d", foo);
  ATL_ERROR('Y',str);
  priv_exit (-3);
}
#endif

void ATS_CLP atl_numofid
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"o", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"6");
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

#if RTRT_CPLUSPLUS_LANGUAGE

void ATS_CLP atl_set_instancevalid
  (ATL_T_INT status)
{
  atl_instancevalid = status;
}

void ATS_CLP atl_set_instancerequired
  (ATL_T_INT status)
{
  atl_instancerequired = status;
}

void ATS_CLP atl_set_instancename
  (ATL_T_INT argc,
   ATL_PT_CAR* argv,
   ATL_PT_CAR f_instance)
{
  if (f_instance == (ATL_PT_CAR) 0)
  {
    int i;
    for (i=1; i<argc; i++)
    {
      if ((atl_strncmp(argv[i], (char*)"-INSTANCE=", 10) == 0) ||
          (atl_strncmp(argv[i], (char*)"-instance=", 10) == 0))
      {
	atl_strcpy (atl_instancename,  &(argv[i][10]));
	break;
      }
    }
  }
  else
  {
    atl_strcpy (atl_instancename,  f_instance);
  }
}

void ATS_CLP atl_set_occid
  (ATL_T_INT argc,
   ATL_PT_CAR* argv,
   ATL_T_INT defaultOccidValue)
{
  int i;

  CTX(occid) = defaultOccidValue;

  for (i=1; i<argc; i++)
  {
    if ((atl_strncmp(argv[i], (char*)"-OCCID=", 7) == 0) ||
	(atl_strncmp(argv[i], (char*)"-occid=", 7) == 0))
    {
      CTX(occid) = argv[i][7];
      break;
    }
  }
}

void ATS_CLP atl_set_testernaming ()
{
  if (atl_instancename[0]!='\0')
   {
    atl_sprintf (CTX(testernaming), (char*)"%s_%d", atl_instancename, CTX(occid));
   }
  else
   {
    atl_sprintf (CTX(testernaming), (char*)"%s_%d", atl_riofilename, CTX(occid));
   }
}


void ATS_CLP atl_set_riofilename
  (ATL_T_INT argc,
   ATL_PT_CAR* argv,
   ATL_PT_CAR defaultRioFileName)
{
  atl_strcpy (atl_riofilename, defaultRioFileName);
  int i;

  for (i=1; i<argc; i++)
  {
    if ((atl_strncmp(argv[i], (char*)"-RIO=", 5) == 0) ||
	(atl_strncmp(argv[i], (char*)"-rio=", 5) == 0))
    {
      atl_strcpy (atl_riofilename,  &(argv[i][5]));
      break;
    }
  }
  if (atl_strstr (atl_riofilename, (char*)".rio") == NULL)
  {
    atl_strcat (atl_riofilename, (char*)".rio");
  }
}


void ATS_CLP atl_set_lstflt
  (ATL_T_INT argc,
   ATL_T_CAR** argv,
   ATL_PT_CAR defaultLstFlt)
{
  atl_lstflt = defaultLstFlt;
  int i;
  for (i=1; i<argc; i++)
  {
    if ((atl_strncmp(argv[i], (char*)"-RIO=", 5) != 0) &&
	(atl_strncmp(argv[i], (char*)"-rio=", 5) != 0) &&
	(atl_strncmp(argv[i], (char*)"-INSTANCE=", 10) != 0) &&
	(atl_strncmp(argv[i], (char*)"-instance=", 10) != 0) &&
	(atl_strncmp(argv[i], (char*)"-OCCID=", 7) != 0) &&
	(atl_strncmp(argv[i], (char*)"-occid=", 7) != 0))
    {
      atl_lstflt = argv[i];
      break;
    }
  }
}

#endif /* RTRT_CPLUSPLUS_LANGUAGE */


void ATS_CLP atl_initrio
#if !RTRT_KR
  (ATS_CTXP
#if RTRT_CPLUSPLUS_LANGUAGE == 0
   ATL_CONST ATL_PT_CAR atl_riofilename,
   ATL_CONST ATL_PT_CAR atl_lstflt,
   ATL_CONST ATL_T_INT  atl_instancevalid,
   ATL_CONST ATL_T_INT  atl_instancerequired,
   ATL_CONST ATL_PT_CAR instance_name,
   ATL_CONST ATL_T_INT  occid,
#endif /* RTRT_CPLUSPLUS_LANGUAGE == 0 */
   ATL_CONST ATL_PT_CAR timestamp,
   ATL_CONST ATL_PT_CAR debug)
#else
  (ctx, atl_riofilename, atl_lstflt, atl_instancevalid, atl_instancerequired, instance_name,
   occid, timestamp, debug)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR atl_riofilename;
  ATL_CONST ATL_PT_CAR atl_lstflt;
  ATL_CONST ATL_T_INT  atl_instancevalid;
  ATL_CONST ATL_T_INT  atl_instancerequired;
  ATL_CONST ATL_PT_CAR instance_name;
  ATL_CONST ATL_T_INT  occid;
  ATL_CONST ATL_PT_CAR timestamp;
  ATL_CONST ATL_PT_CAR debug;
#endif
{
  ATL_PT_CAR currentptr;
  ATL_T_INT i;
  ATL_T_CAR tmpstr[50];
  ATL_T_LONG date_one, date_two;

  atl_early_init ();

#ifdef ATL_SIGNAL
  ATL_SIGNAL (SIGABRT, atl_handler);
  ATL_SIGNAL (SIGILL, atl_handler);
  ATL_SIGNAL (SIGINT, atl_handler);
  ATL_SIGNAL (SIGTERM, atl_handler);
  ATL_SIGNAL (SIGSEGV, atl_handler);
#if !_ATL_WIN32_API
  ATL_SIGNAL (SIGPIPE, atl_handler);
  ATL_SIGNAL (SIGQUIT, atl_handler);
  ATL_SIGNAL (SIGBUS, atl_handler);
#endif
#endif /* ATL_SIGNAL */

  CTX(rio_onoff) = 1;
  CTX(trace_onoff) = 1;
  CTX(tagtcommactivated) = 1;

  TCTX(CYTctx_stackindex) = 0;
  TCTX(CYToverpassindex) = 0;
  TCTX(CYTbufferoverpassed) = 0;
  TCTX(CYTfboccured) = 0;
  TCTX(CYTftoccured) = 0;
  TCTX(CYTft_status) = 0;
  TCTX(CYTlasttracelevel_forclosure)[0] = -1;
  TCTX(CYTlasttracelevel_forclosure)[1] = -1;

  atl_strcpy (CTX(bufout), (char*)"");
  CTX(bufindex) = (ATL_T_INT) 0;
  CTX(lock) = 0;
  atl_strcpy (CTX(strwaittil), (char*)"");

  CTX(dsend) = (ATL_T_LONG) 0;
  CTX(lbcf) = (ATL_T_LONG) 0;
  CTX(dwait) = (ATL_T_LONG) 0;
  CTX(disend) = (ATL_T_LONG) 0;
  CTX(direcv) = (ATL_T_LONG) 0;

  CTX(idxmin) = (ATL_T_INT) 0;
  CTX(lastcmp) = (ATL_T_INT) 0;
  atl_strcpy (CTX(varrec), (char*)"");
  atl_strcpy (CTX(varexpect), (char*)"");
  CTX(varfrec) = (ATL_T_REAL) 0;
  CTX(varfexpect) = (ATL_T_REAL) 0;
  CTX(vardrec) = (ATL_T_DOUBLE) 0;
  CTX(vardexpect) = (ATL_T_DOUBLE) 0;
  CTX(prev) = (ATL_PT_VOID) 0;

#if !RTRT_CPLUSPLUS_LANGUAGE
  for (i = 0; i < ATS_CLEANUP_STACK_SIZE; ++i)
  {
    CTX(gestcleanup)[i] = (ATL_T_CLEANUP) 0;
  }
  CTX(indexcleanup) = (ATL_T_INT) 0;
#endif

  CTX(level) = (ATL_T_INT) 0;
  CTX(flagrt) = (ATL_T_INT) 0;
  for (i = 0; i < ATS_FILTERS_MAX; ++i)
  {
    CTX(filter)[i] = (ATL_T_INT) 0;
  }

  CTX(wtime) = (ATL_T_LONG) 0;

  CTX(fdrio) = priv_init (atl_riofilename);
  if(CTX(fdrio)==0) CTX(fdrio) = priv_open (atl_riofilename);

  atl_p_out_str (ATS_CTXA (char*)"ATTOL SystemTest ");
  atl_p_out_quot (ATS_CTXA (char*)"v2.6a");
  ATL_M_out_sep;
#if RTRT_CPLUSPLUS_LANGUAGE
  atl_p_out_quot (ATS_CTXA (char*)"ATS_C++");
#else
  atl_p_out_quot (ATS_CTXA (char*)"ATS_C");
#endif
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA (char*)"RIO file");
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA timestamp);
  atl_p_flush_line (ATS_CTXA1);

  atl_p_begin_line (ATS_CTXA (char*)"g", 0);
  atl_p_out_car (ATS_CTXA ' ');
  priv_get_hostid (tmpstr);
  atl_p_out_quot (ATS_CTXA tmpstr);

  ++CTX(level);

  if (CTX(flagprobe))
    priv_memcpy(&CTX(stdate),&CTX(usrdate),sizeof(RTRT_CLOCK_TYPE));
  else
    priv_clock (&CTX(stdate));

  ATL_M_out_sep;
  date_one = ATL_DATE_ONE(CTX(stdate));
  date_two = ATL_DATE_TWO(CTX(stdate));
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA date_one, tmpstr, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA date_two, tmpstr, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA
                  atl_long_image (ATS_CTXA ATL_DATE_ONE_TWO, tmpstr, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA
                  atl_long_image (ATS_CTXA ATL_DATE_TWO_MICRO_SEC, tmpstr, 10));
  atl_p_flush_line (ATS_CTXA1);

  atl_p_begin_line (ATS_CTXA (char*)"h", 1);
  if (tmpstr[0])
  {
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }

  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA (char*)RTRT_TIME_TYPE_FORMAT);
  atl_p_flush_line (ATS_CTXA1);

#if RTRT_CPLUSPLUS_LANGUAGE
#define instance_name atl_instancename
#endif

  atl_p_begin_line (ATS_CTXA (char*)"f", 2);
  if (atl_lstflt != (ATL_PT_CAR) 0)
  {
    currentptr = atl_lstflt;
    while (*currentptr)
    {
      if ((*currentptr >= 'A') && (*currentptr <= 'Z'))
      {
	CTX(filter)[(ATL_T_INT) (*currentptr - 'A')] = 1;
	++currentptr;
      }
      else
      {
	atl_p_out_car (ATS_CTXA ' ');
	atl_p_out_quot (ATS_CTXA (char*)"!!! invalid filters !!!");
	atl_p_flush_line (ATS_CTXA1);

	/** fill last general info line **/
	atl_p_begin_line (ATS_CTXA (char*)"ie", 3);
	atl_p_out_car (ATS_CTXA ' ');
	atl_p_out_quot (ATS_CTXA (char*)"");

	ATL_M_out_sep;
	atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(occid), tmpstr, 10));

	ATL_M_out_sep;
	atl_p_out_quot (ATS_CTXA debug);

	atl_p_flush_line (ATS_CTXA1);

	priv_close (CTX(fdrio));
	priv_exit (-4);
      }
    }
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA atl_lstflt);
    atl_p_flush_line (ATS_CTXA1);
  }
  else
  {
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"");
    atl_p_flush_line (ATS_CTXA1);
  }

  atl_p_begin_line (ATS_CTXA (char*)"ie", 3);

  if (instance_name != (ATL_PT_CAR) 0)
  {
    if (!atl_instancevalid)
    {
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"!!! invalid instance specified !!!");

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(occid), tmpstr, 10));

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA debug);
      atl_p_flush_line (ATS_CTXA1);

      priv_close (CTX(fdrio));
      priv_exit (-5);
    }
    else
    {
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA instance_name);

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(occid), tmpstr, 10));

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA debug);
      atl_p_flush_line (ATS_CTXA1);
    }
  }
  else
  {
    if (atl_instancerequired)
    {
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"!!! instance must be specified !!!");

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(occid), tmpstr, 10));

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA debug);

      atl_p_flush_line (ATS_CTXA1);

      priv_close (CTX(fdrio));
      priv_exit (-6);
    }
    else
    {
      /** scenario has nothing to do with instances : print it anyway **/
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"");

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(occid), tmpstr, 10));

      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA debug);
      atl_p_flush_line (ATS_CTXA1);
    }
  }
#if RTRT_CPLUSPLUS_LANGUAGE
#undef instance_name
#endif
}

void ATS_CLP atl_send
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_PT_CAR msg_name)
#else
  (ctx, index, msg_name)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_PT_CAR msg_name;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"sd", index);
  atl_p_out_car (ATS_CTXA ' ');
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  CTX(dsend) = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	       - CTX(duration)[CTX(level)];
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(dsend), str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA msg_name, 0, 0);
#endif

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);
}

void ATS_CLP atl_isend
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT statusKO,
   ATL_CONST ATL_PT_CAR msg,
   ATL_CONST ATL_PT_CAR receip_tester)
#else
  (ctx, index, statusKO, msg, receip_tester)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT statusKO;
  ATL_CONST ATL_PT_CAR msg;
  ATL_CONST ATL_PT_CAR receip_tester;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_CAR locbuffer[ATS_TAGTBUF_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"id", index);
  atl_p_out_car (ATS_CTXA ' ');

  if (statusKO)
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
    }
  else
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
  ATL_M_out_sep;
  CTX(disend) = priv_elapsed_time (&CTX(stdate),&CTX(usrdate),CTX(flagprobe)) - CTX(duration)[CTX(level)];
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA CTX(disend), str, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA msg, locbuffer,ATS_TAGTBUF_SIZE));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA receip_tester, locbuffer,ATS_TAGTBUF_SIZE));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA NULL, 0, 0);
#endif

  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}


void ATS_CLP atl_end_send
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT statusKO,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, statusKO, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT statusKO;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_ULONG sf;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"sd", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (statusKO)
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
  {
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }
  ATL_M_out_sep;
  sf = (priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
       - CTX(duration)[CTX(level)]) - CTX(dsend);
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA sf, str, 10));

  ATS_CYT_FLUSH(statusKO ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);
}


void ATS_CLP atl_bcf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"a", index);
  atl_p_out_car (ATS_CTXA ' ');
  CTX(lbcf) = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	      - CTX(duration)[CTX(level)];
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(lbcf), str, 10));
  ATL_M_out_sep;
  ATL_M_out_empty_field;

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);
}

ATL_T_INT ATS_CLP atl_cf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT localint;
  ATL_T_ULONG cf;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"a", index);
  atl_p_out_car (ATS_CTXA ' ');
  localint = CTX(flagrt);
  if (CTX(flagrt))
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
  {
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }
  /* SLY: correction RATLC 00866779 */
  CTX(flagrt) = 0;

  cf = (priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
       - CTX(duration)[CTX(level)]) - CTX(lbcf);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA cf, str, 10));

  ATS_CYT_FLUSH(CTX(flagrt) ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);

  return (localint);
}

void ATS_CLP atl_brdv
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_PT_CAR rdv_name)
#else
  (ctx, index, rdv_name)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_PT_CAR rdv_name;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"n", index);
  atl_p_out_car (ATS_CTXA ' ');
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  CTX(lbrdvf) = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	        - CTX(duration)[CTX(level)];
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(lbrdvf), str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA rdv_name, 0, 0);
#endif

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);
}

void ATS_CLP atl_ardv
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT result,
   ATL_CONST ATL_PT_CAR rdv_name)
#else
  (ctx, index, result, rdv_name)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT result;
  ATL_CONST ATL_PT_CAR rdv_name;
#endif
{
  ATL_T_ULONG rdv;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"n", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (result)
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
  {
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }
  rdv = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	- CTX(duration)[CTX(level)] - CTX(lbrdvf);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA rdv, str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA rdv_name, 0, 0);
#endif

  ATS_CYT_FLUSH(result ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);
}

void ATS_CLP atl_binit
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  CTX(tracelevel) = scn_level;
  ++CTX(level);
  CTX(duration)[CTX(level)] = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),
						CTX(flagprobe));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(duration)[CTX(level)], str, 10));

  switch(TCTX(tracepriority))
    {
    case 5:
      atl_p_stackblock (ATS_CTXA TCTX(binitstack)[scn_level], 1);
      break;

    case K_TRACECTX_CIRCULAR_PRIORITY:
      atl_p_stackblock (ATS_CTXA TCTX(binitstack)[scn_level], 0);
      TCTX(CYTstatus) = ATL_CYT_BEGIN_BLOCK;
      GENERIC_FLUSH_LINE;
      break;

    default:
      GENERIC_FLUSH_LINE;
    }
}

void ATS_CLP atl_ainit
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  CTX(tracelevel) = scn_level;
  ATL_M_out_sep;
  ATL_M_out_empty_field;

  ATS_CYT_FLUSH(state ? ATL_CYT_END_KO_BLOCK : ATL_CYT_END_OK_BLOCK);

  TCTX(binitstack)[scn_level][0] = '\0';
  CTX(level)--;

  /** upgrading status concerning former FT instruction. Now, it's sure **/
  /** the (eventual) forthcoming 'fb' will occur in an other block      **/
  if (TCTX(CYTft_status) == 1)
    TCTX(CYTft_status) = 2;
}

void ATS_CLP atl_bterm
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  ATL_M_out_sep;
  CTX(tracelevel) = scn_level;
  ++CTX(level);
  CTX(duration)[CTX(level)] = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),
						CTX(flagprobe));
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(duration)[CTX(level)], str, 10));

  switch(TCTX(tracepriority))
    {
    case 5:
      atl_p_stackblock (ATS_CTXA TCTX(btermstack)[scn_level], 1);
      break;

    case K_TRACECTX_CIRCULAR_PRIORITY:
      atl_p_stackblock (ATS_CTXA TCTX(btermstack)[scn_level], 0);
      TCTX(CYTstatus) = ATL_CYT_BEGIN_BLOCK;
      GENERIC_FLUSH_LINE;
      break;

    default:
      GENERIC_FLUSH_LINE;
    }
}

void ATS_CLP atl_aterm
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  CTX(tracelevel) = scn_level;
  ATL_M_out_sep;
  ATL_M_out_empty_field;

  ATS_CYT_FLUSH(state ? ATL_CYT_END_KO_BLOCK : ATL_CYT_END_OK_BLOCK);

  TCTX(btermstack)[scn_level][0] = '\0';
  CTX(level)--;

  /** upgrading status concerning the former FT instruction. Now, we're **/
  /** sure the (eventual) forthcoming 'fb' will occur in another block **/
  if (TCTX(CYTft_status) == 1)
    TCTX(CYTft_status) = 2;
}

void ATS_CLP atl_bexcep
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  ATL_M_out_sep;
  CTX(tracelevel) = scn_level;
  ++CTX(level);
  CTX(duration)[CTX(level)] = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),
						CTX(flagprobe));
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(duration)[CTX(level)], str, 10));

  switch(TCTX(tracepriority))
    {
    case 5:
      atl_p_stackblock (ATS_CTXA TCTX(bexceptstack)[scn_level], 1);
      break;

    case K_TRACECTX_CIRCULAR_PRIORITY:
      atl_p_stackblock (ATS_CTXA TCTX(bexceptstack)[scn_level], 0);
      TCTX(CYTstatus) = ATL_CYT_BEGIN_BLOCK;
      GENERIC_FLUSH_LINE;
      break;

    default:
      GENERIC_FLUSH_LINE;
    }
}

void ATS_CLP atl_aexcep
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop)
#else
  (ctx, state, index, scn_level, scn_loop)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"z", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  CTX(tracelevel) = scn_level;
  ATL_M_out_sep;
  ATL_M_out_empty_field;

  ATS_CYT_FLUSH(state ? ATL_CYT_END_KO_BLOCK : ATL_CYT_END_OK_BLOCK);

  TCTX(bexceptstack)[scn_level][0] = '\0';
  CTX(level)--;

  /** upgrading status concerning the former FT instruction. Now, it's sure **/
  /** the (eventual) forthcoming 'fb' will occur in an other block **/
  if (TCTX(CYTft_status) == 1)
    TCTX(CYTft_status) = 2;
}

void ATS_CLP atl_startchronomarkloop
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT endidx,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop,
   ATL_CONST ATL_PT_CAR scn_name,
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT tsffileindex)
#else
  (ctx, endidx, index, scn_level, scn_loop, scn_name,tsfindex,tsffileindex)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT endidx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
  ATL_CONST ATL_PT_CAR scn_name;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT tsffileindex;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_SCN_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"s", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA endidx, str, 10));
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA scn_loop, str, 10));
  CTX(tracelevel) = scn_level;
  ATL_M_out_sep;
  ++CTX(level);
  CTX(duration)[CTX(level)] = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),
						CTX(flagprobe));
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(duration)[CTX(level)], str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA scn_name, tsfindex, tsffileindex);
#endif

  switch(TCTX(tracepriority))
    {
    case 5:
      atl_p_stackblock (ATS_CTXA TCTX(bscnstack)[scn_level], 1);
      break;

    case K_TRACECTX_CIRCULAR_PRIORITY:
      atl_p_stackblock (ATS_CTXA TCTX(bscnstack)[scn_level], 0);

      if (scn_level==0)
	TCTX(CYTstatus) = ATL_CYT_BEGIN_VIRTUAL_BLOCK;
      else
	TCTX(CYTstatus) = ATL_CYT_BEGIN_BLOCK;

      GENERIC_FLUSH_LINE;
      break;

    default:
      GENERIC_FLUSH_LINE;
    }
}

void ATS_CLP atl_stopchronomarkloop
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT scn_level,
   ATL_CONST ATL_T_INT scn_loop,
   ATL_CONST ATL_PT_CAR scn_name,
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT tsffileindex)
#else
  (ctx, state, index, scn_level, scn_loop, scn_name, tsfindex, tsffileindex)
  ATL_PT_RUNCTX ctx;
  ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT scn_level;
  ATL_CONST ATL_T_INT scn_loop;
  ATL_CONST ATL_PT_CAR scn_name;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT tsffileindex;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_CAR strtime[ATS_IMAGE_SIZE];
  RTRT_TIME_TYPE loctime;

  loctime = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	    - CTX(duration)[CTX(level)];
  atl_long_image (ATS_CTXA loctime, strtime, 10);
  TCTX(currentcodepriority) = K_TRACECTX_SCN_PRIORITY;
#if !RTRT_CPLUSPLUS_LANGUAGE
  if (state && (state < 3))
  {
    atl_p_execcleanup (ATS_CTXA1);
  }
  atl_p_erasecleanup (ATS_CTXA1);
#endif
  atl_p_begin_line (ATS_CTXA (char*)"s", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (state >= 3)
  {
    state -= 2;
  }
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA strtime);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA scn_loop, str, 10));
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  CTX(tracelevel) = scn_level;

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA scn_name, tsfindex, tsffileindex);
#endif

  ATS_CYT_FLUSH(scn_level==0 ? ATL_CYT_END_VIRTUAL_BLOCK : (state ? ATL_CYT_END_KO_BLOCK : ATL_CYT_END_OK_BLOCK));
  --CTX(level);

  /** upgrading status concerning the former FT instruction. Now, it's sure **/
  /** the (eventual) forthcoming 'fb' will occur in an other block          **/
  if (TCTX(CYTft_status) == 1)
    TCTX(CYTft_status) = 2;
}

void ATS_CLP atl_string_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_PT_CAR vinit,
   ATL_CONST ATL_PT_CAR vexpect,
   ATL_CONST ATL_PT_CAR vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_PT_CAR vinit;
  ATL_CONST ATL_PT_CAR vexpect;
  ATL_CONST ATL_PT_CAR vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_BUFFER_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect && vrec)
        {
          if (atl_strcmp (vexpect, vrec))
            {
              compare = (char *)"1";
              CTX(flagrt) = 1;
            }
        }
      else
        {
          if (vrec || vexpect)
            {
              compare = (char *)"1";
              CTX(flagrt) = 1;
            }
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA vinit, str,ATS_BUFFER_SIZE));
  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA vexpect, str,ATS_BUFFER_SIZE));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA vrec, str,ATS_BUFFER_SIZE));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_pointer_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_PT_VOID vinit,
   ATL_CONST ATL_PT_VOID vexpect,
   ATL_CONST ATL_PT_VOID vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_PT_VOID vinit;
  ATL_CONST ATL_PT_VOID vexpect;
  ATL_CONST ATL_PT_VOID vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect != vrec)
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA vinit, str, 16));

  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA vexpect, str, 16));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA vrec, str, 16));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_nonil_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_PT_VOID vinit,
   ATL_CONST ATL_PT_VOID vexpect,
   ATL_CONST ATL_PT_VOID vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_PT_VOID vinit;
  ATL_CONST ATL_PT_VOID vexpect;
  ATL_CONST ATL_PT_VOID vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect == NIL)
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA vinit, str, 16));

  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA (char*)"NONIL");
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA vrec, str, 16));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_T_LONG vinit,
   ATL_CONST ATL_T_LONG vexpect,
   ATL_CONST ATL_T_LONG vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_T_LONG vinit;
  ATL_CONST ATL_T_LONG vexpect;
  ATL_CONST ATL_T_LONG vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect != vrec)
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA vinit, str, 10));
  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA vexpect, str, 10));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA vrec, str, 10));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_T_INT vinit,
   ATL_CONST ATL_T_INT vexpect,
   ATL_CONST ATL_T_INT vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_T_INT vinit;
  ATL_CONST ATL_T_INT vexpect;
  ATL_CONST ATL_T_INT vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect != vrec)
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA vinit, str, 10));
  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA vexpect, str, 10));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA vrec, str, 10));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_char_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_T_CAR vinit,
   ATL_CONST ATL_T_CAR vexpect,
   ATL_CONST ATL_T_CAR vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_T_CAR vinit;
  ATL_CONST ATL_T_CAR vexpect;
  ATL_CONST ATL_T_CAR vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str1[2];
  ATL_T_CAR str2[10];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (vexpect != vrec)
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  str1[0] = vinit;
  str1[1] = '\0';
  atl_p_out_quot (ATS_CTXA atl_car_image (ATS_CTXA str1, str2));
  ATL_M_out_sep;
  if (dotest)
    {
      str1[0] = vexpect;
      str1[1] = '\0';
      atl_p_out_quot (ATS_CTXA atl_car_image (ATS_CTXA str1, str2));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  str1[0] = vrec;
  str1[1] = '\0';
  atl_p_out_quot (ATS_CTXA atl_car_image (ATS_CTXA str1, str2));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_T_REAL vinit,
   ATL_CONST ATL_T_REAL vexpect,
   ATL_CONST ATL_T_REAL vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_T_REAL vinit;
  ATL_CONST ATL_T_REAL vexpect;
  ATL_CONST ATL_T_REAL vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (atl_fltcmp (vexpect, vrec))
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA vinit, str));
  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA vexpect, str));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA vrec, str));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT dotest,
   ATL_CONST ATL_T_DOUBLE vinit,
   ATL_CONST ATL_T_DOUBLE vexpect,
   ATL_CONST ATL_T_DOUBLE vrec,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, dotest, vinit, vexpect, vrec, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT dotest;
  ATL_CONST ATL_T_DOUBLE vinit;
  ATL_CONST ATL_T_DOUBLE vexpect;
  ATL_CONST ATL_T_DOUBLE vrec;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_PT_CAR compare;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  compare = (char *)"0";
  atl_p_begin_line (ATS_CTXA (char*)"r", index);
  atl_p_out_car (ATS_CTXA ' ');
  if (dotest)
    {
      if (atl_dblcmp (vexpect, vrec))
        {
          compare = (char *)"1";
          CTX(flagrt) = 1;
        }
    }
  atl_p_out_quot (ATS_CTXA compare);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA vinit, str));
  ATL_M_out_sep;
  if (dotest)
    {
      atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA vexpect, str));
    }
  else
    {
      atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
    }
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA vrec, str));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

#if !RTRT_CPLUSPLUS_LANGUAGE
void ATS_CLP atl_enable
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_INT (*proc) (void))
#else
  (ctx, proc)
  ATL_PT_RUNCTX ctx;
  ATL_T_INT (*proc);
#endif
{
  if (!CTX(lock))
  {
    if (CTX(indexcleanup) == (ATS_CLEANUP_STACK_SIZE - 1))
    {
      atl_p_begin_line (ATS_CTXA (char*)"x", 0);
      atl_p_out_quot (ATS_CTXA (char*)"cleanup table overflow");
      ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
      priv_exit (-7);;
    }
    else
    {
#if !RTRT_KR
      CTX(gestcleanup)[CTX(indexcleanup)++] = (int (*)(ATL_PT_RUNCTX)) proc;
#else
      CTX(gestcleanup)[CTX(indexcleanup)++] = proc;
#endif
    }
  }
}

void ATS_CLP atl_disable
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_INT (*proc) (void))
#else
  (ctx, proc)
  ATL_PT_RUNCTX ctx;
  ATL_T_INT (*proc);
#endif
{
  ATL_T_INT i;

  i = CTX(indexcleanup) - 1;
#if !RTRT_KR
  while ((i >= 0) && (CTX(gestcleanup)[i] != (int (*)(ATL_PT_RUNCTX)) proc))
#else
  while ((i >= 0) && (CTX(gestcleanup)[i] != proc))
#endif
  {
    --i;
  }
  if (i != -1)
  {
#if !RTRT_KR
    CTX(gestcleanup)[i] = (int (*)(ATL_PT_RUNCTX)) 0;
#else
    CTX(gestcleanup)[i] = (int (*)()) 0;
#endif
  }
}
#endif

#if !RTRT_CPLUSPLUS_LANGUAGE
static void atl_p_execcleanup
#if !RTRT_KR
  (ATL_PT_RUNCTX ctx)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  if (!CTX(lock))
  {
    CTX(lock) = 1;
    while (CTX(indexcleanup))
    {
      if (CTX(gestcleanup)[CTX(indexcleanup) - 1])
      {
	(*CTX(gestcleanup)[CTX(indexcleanup) - 1]) (ATS_CTXA1);
      }
      --CTX(indexcleanup);
    }
    CTX(lock) = 0;
  }
}
#endif

void ATS_CLP atl_endwaittil
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT state,
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_PT_CAR msg_name)
#else
  (ctx, state, index, msg_name)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_PT_CAR msg_name;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_ULONG wait;

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_p_begin_line (ATS_CTXA (char*)"w", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state, str, 10));
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA CTX(strwaittil));
  ATL_M_out_sep;

  wait = atl_getevtdate (ATS_CTXA1);
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA wait, str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA msg_name, 0, 0);
#endif

  ATS_CYT_FLUSH(state ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);
}

void ATS_CLP atl_error
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"o", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"1");
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_nomsg
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"o", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"2");
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_msgdate
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  CTX(logdate) = atl_getevtdate (ATS_CTXA1);
  atl_p_begin_line (ATS_CTXA (char*)"o", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"3");
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_nodefmsg
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"w", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"4");
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_noaddid
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"w", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"5");
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_rioexit
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"q", index);
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

#if RTRT_CPLUSPLUS_LANGUAGE
void ATS_CLP atl_userexit
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR str)
#else
  (ctx, str)
  ATL_PT_RUNCTX ctx ATL_SP
  ATL_PT_CAR str ATL_EP
#endif
{
#if !RTRT_CPLUSPLUS_LANGUAGE
  atl_p_execcleanup (ATS_CTXA1);
  atl_p_erasecleanup (ATS_CTXA1);
#endif
  atl_p_begin_line (ATS_CTXA (char*)"x", 0);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA str);
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
  priv_close (CTX(fdrio));
  priv_exit (-1);;
}
#endif /* RTRT_CPLUSPLUS_LANGUAGE */

void ATS_CLP atl_setfilter
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR filter,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, filter, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR filter;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT i;
  ATL_T_INT j;

  ATL_T_CAR atl_filtset[2 * ATS_FILTERS_MAX + 1];

  i = 0;
  while (filter[i])
    {
      CTX(filter)[(ATL_T_INT) (filter[i++] - 'A')] = 1;
    }
  atl_filtset[0] = '\0';

  j = 0;
  for (i = 0; i < ATS_FILTERS_MAX; ++i)
    {
      if (CTX(filter)[i])
        {
          atl_filtset[j++] = 'A' + i;
          atl_filtset[j++] = ',';
        }
    }
  atl_filtset[j - 1] = '\0';
  atl_p_begin_line (ATS_CTXA (char*)"j", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_filtset);
  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_unsetfilter
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR filter,
   ATL_CONST ATL_T_INT index)
#else
  (ctx, filter, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR filter;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT i;
  ATL_T_INT j;

  ATL_T_CAR atl_filtset[2 * ATS_FILTERS_MAX + 1];

  i = 0;
  while (filter[i])
    {
      CTX(filter)[(ATL_T_INT) (filter[i++] - 'A')] = 0;
    }
  atl_filtset[0] = '\0';

  j = 0;
  for (i = 0; i < ATS_FILTERS_MAX; ++i)
    {
      if (CTX(filter)[i])
        {
          atl_filtset[j++] = 'A' + i;
          atl_filtset[j++] = ',';
        }
    }
  atl_filtset[j - 1] = '\0';
  atl_p_begin_line (ATS_CTXA (char*)"u", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_filtset);

  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_comment
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"c", index);

  ATS_CYT_FLUSH(ATL_CYT_MONOLINE);
}

void ATS_CLP atl_evt
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT idx_ch,
   ATL_CONST ATL_T_INT displaylastfield,
   ATL_CONST ATL_PT_CAR type)
#else
  (ctx, index, idx_ch, displaylastfield, type)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT idx_ch;
  ATL_CONST ATL_T_INT displaylastfield;
  ATL_CONST ATL_PT_CAR type;
#endif
{
  ATL_T_CAR reald[ATS_IMAGE_SIZE],
            logd[ATS_IMAGE_SIZE], idx_chd[ATS_IMAGE_SIZE], lastfield[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  atl_long_image (ATS_CTXA CTX(logdate), reald, 10);
  atl_long_image (ATS_CTXA (CTX(logdate) / ATL_WTIME_TO_TIME), logd, 10);
  atl_int_image (ATS_CTXA idx_ch, idx_chd, 10);
  atl_int_image (ATS_CTXA displaylastfield, lastfield, 10);
  atl_p_begin_line (ATS_CTXA (char*)"t", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA reald);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA logd);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA idx_chd);
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA lastfield);
  ATL_M_out_sep;
  atl_p_out_quot(ATS_CTXA type);
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_endevt
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"t", index);
  atl_p_out_car (ATS_CTXA ' ');
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

ATL_T_ULONG ATS_CLP atl_getevtdate
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  return priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	 - CTX(dwait);
}

ATL_T_ULONG ATS_CLP atl_getwtime
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  return (priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	  - CTX(dwait)) / ATL_WTIME_TO_TIME;
}

void ATS_CLP atl_state
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT state,
  ATL_CONST ATL_T_INT index)
#else
  (ctx, state, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT state;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"y", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA (1 - state), str, 10));

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_def_msg
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"v", index);
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}


void ATS_CLP atl_irecv
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  CTX(direcv) = priv_elapsed_time (&CTX(stdate),&CTX(usrdate),CTX(flagprobe));

  atl_p_begin_line (ATS_CTXA (char*)"ic", index);
  atl_p_out_car (ATS_CTXA ' ');

  ATL_M_out_empty_field;
  ATL_M_out_sep;
  ATL_M_out_empty_field;
  ATL_M_out_sep;

  ATL_M_out_empty_field;
  ATL_M_out_sep;

  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA CTX(direcv) - CTX(duration)[CTX(level)], str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA NULL, 0, 0);
#endif

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);
}

void ATS_CLP atl_end_irecv
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT state_timeout,
   ATL_CONST ATL_PT_CAR emit_tster,
   ATL_CONST ATL_PT_CAR info_msg)
#else
  (ctx, index, state_timeout, emit_tster, info_msg)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT state_timeout;
  ATL_CONST ATL_PT_CAR emit_tster;
  ATL_CONST ATL_PT_CAR info_msg;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_CAR locbuffer[ATS_TAGTBUF_SIZE];
  ATL_T_ULONG recvwait;

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"ic", index);
  atl_p_out_car (ATS_CTXA ' ');

  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA state_timeout, str, 10));
  ATL_M_out_sep;

  if (emit_tster)
    {
      /* dump received message */
      atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA info_msg, locbuffer,ATS_TAGTBUF_SIZE));
      ATL_M_out_sep;
      /* dump emitting tester */
      atl_p_out_quot (ATS_CTXA atl_string_image (ATS_CTXA emit_tster, locbuffer,ATS_TAGTBUF_SIZE));
    }
  else
    {
      ATL_M_out_empty_field;
      ATL_M_out_sep;
      ATL_M_out_empty_field;
    }

  ATL_M_out_sep;
  recvwait = priv_elapsed_time (&CTX(stdate),&CTX(usrdate),CTX(flagprobe)) - CTX(direcv); /* interrecv duration */
  atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA recvwait, str, 10));

#if ATS_DYNFLW
  atl_p_raise_dynfline(ATS_CTXA NULL, 0, 0);
#endif

  ATS_CYT_FLUSH((state_timeout<2) ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);
}


ATL_T_INT ATS_CLP atl_send_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT file_index)
#else
  (ctx,tsfindex,file_index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_send_message %d %d %d\r\n",
		  CTX(numinstance), tsfindex, file_index);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}

ATL_T_INT ATS_CLP atl_waittil_tsf
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_PT_CAR msgtype,
  ATL_CONST ATL_T_INT tsfindex,
  ATL_CONST ATL_T_INT file_index)
#else
  (ctx,msgtype,tsfindex,file_index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR msgtype;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_waittil_message %s %d %d %d\r\n",
		  msgtype, CTX(numinstance), tsfindex, file_index);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}

ATL_T_INT ATS_CLP atl_begin_waittil_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT file_index)
#else
  (ctx,tsfindex,file_index)
   ATL_PT_RUNCTX ctx;
   ATL_CONST ATL_T_INT tsfindex;
   ATL_CONST ATL_T_INT file_index;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_begin_waittil %d %d %d\r\n",
		  CTX(numinstance), tsfindex, file_index);

      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}


ATL_T_INT ATS_CLP atl_end_waittil_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT file_index,
   ATL_CONST ATL_T_INT waittil_status)
#else
  (ctx,tsfindex,file_index,waittil_status)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
  ATL_CONST ATL_T_INT waittil_status;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_end_waittil %d %d %d %d\r\n",
		  CTX(numinstance), tsfindex, file_index, waittil_status);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}

ATL_T_INT ATS_CLP atl_intersend_tsf
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT tsfindex,
  ATL_CONST ATL_T_INT file_index,
  ATL_CONST ATL_PT_CAR fullidentifier)
#else
  (ctx,tsfindex,file_index,fullidentifier)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
  ATL_CONST ATL_PT_CAR fullidentifier;
#endif
{
  ATL_T_INT status;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_intersend %d %d %d %s\r\n",
		  CTX(numinstance), tsfindex, file_index, fullidentifier);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}

ATL_T_INT ATS_CLP atl_interrecv_tsf
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT tsfindex,
  ATL_CONST ATL_T_INT file_index,
  ATL_CONST ATL_PT_CAR recved_msg,
  ATL_CONST ATL_PT_CAR emitter)
#else
  (ctx,tsfindex,file_index,recved_msg,emitter)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
  ATL_CONST ATL_PT_CAR recved_msg;
  ATL_CONST ATL_PT_CAR emitter;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_interrecv %d %d %d %s \"\"\"%s\"\"\"\r\n",
		  CTX(numinstance), tsfindex, file_index, emitter, recved_msg);

      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}



ATL_T_INT ATS_CLP atl_notify_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT locnumfic,
   ATL_CONST ATL_T_INT id1,
   ATL_CONST ATL_T_INT id2)
#else
  (ctx,locnumfic,id1,id2)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT locnumfic;
  ATL_CONST ATL_T_INT id1;
  ATL_CONST ATL_T_INT id2;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_notify %d %ld %ld\r\n",
		  locnumfic, id1, id2);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}



ATL_T_INT ATS_CLP atl_declare_ni_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT file_index)
#else
  (ctx,file_index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT file_index;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_declare_ni %s %d %d\r\n",
		  CTX(testernaming), CTX(numinstance), file_index);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}


ATL_T_INT ATS_CLP atl_rdv_reached_tsf
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT statusOK,
   ATL_CONST ATL_T_INT tsfindex,
   ATL_CONST ATL_T_INT file_index)
#else
  (ctx,statusOK,tsfindex,file_index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT statusOK;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_synchro %d %d %d %d\r\n",
		  CTX(numinstance), statusOK, tsfindex, file_index);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}

ATL_T_INT ATS_CLP atl_special_block_tsf
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT tsfindex,
  ATL_CONST ATL_T_INT file_index,
  ATL_CONST ATL_T_INT block_type,
  ATL_CONST ATL_T_INT is_beginning)
#else
  (ctx,tsfindex,file_index,block_type,is_beginning)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT tsfindex;
  ATL_CONST ATL_T_INT file_index;
  ATL_CONST ATL_T_INT block_type;
  ATL_CONST ATL_T_INT is_beginning;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  if (CTX(tagtcommactivated))
    {
      atl_sprintf(buffer, (char*)"TSF_inittermexcept %d %d %d\r\n",
		  CTX(numinstance), tsfindex, file_index, block_type);
      status = priv_send_chk (&CTX(tagtsocket), buffer);
    }
  return (status);
}


ATL_T_INT ATS_CLP atl_endoftester_tagtlk
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  atl_sprintf(buffer, (char*)"End_of \"%s\"\r\n", CTX(testernaming));
  status = priv_send_chk (&CTX(tagtsocket), buffer);
  return (status);
}


#if ATS_DYNFLW
ATL_T_INT ATS_CLP atl_intersend
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_PT_CAR line,
   ATL_CONST ATL_PT_CAR dest_tester)
#else
  (ctx,line,dest_tester)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_PT_CAR line;
  ATL_CONST ATL_PT_CAR dest_tester;
#endif
{
  ATL_T_INT status = 0;
  ATL_T_CAR buffer[ATS_TAGTBUF_SIZE];

  atl_sprintf(buffer,"INTERSEND \"\"\"%s\"\"\", \"\"\"%s\"\"\"\r\n",
	      line, dest_tester);
  status = priv_send_chk (&CTX(tagtsocket), buffer);

  return (status);
}
#endif

void ATS_CLP atl_waittil
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;
  CTX(dwait) = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe));
  atl_long_image (ATS_CTXA CTX(dwait) - CTX(duration)[CTX(level)], CTX(strwaittil), 10);
  atl_p_begin_line (ATS_CTXA (char*)"w", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
  ATL_M_out_sep;
  atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);
  ATL_M_out_sep;
  atl_p_out_str (ATS_CTXA ATL_K_EMPTY_FIELD);

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);
}

ATL_T_INT ATS_CLP atl_var_va
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT locint;

  atl_p_begin_line (ATS_CTXA (char*)"k", index);
  atl_p_out_car (ATS_CTXA ' ');
  locint = CTX(flagrt);
  CTX(flagrt) = 0;
  if (locint)
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
  {
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_MULTILINE);

  return (locint);
}

ATL_T_INT ATS_CLP atl_end_var_va
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT locint;

  atl_p_begin_line (ATS_CTXA (char*)"k", index);
  atl_p_out_car (ATS_CTXA ' ');
  locint = CTX(flagrt);
  CTX(flagrt) = 0;
  if (locint)
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }

  ATS_CYT_FLUSH(locint ? ATL_CYT_END_KO_MULTILINE : ATL_CYT_END_OK_MULTILINE);

  return (locint);
}


ATL_T_INT ATS_CLP atl_var
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  ATL_T_INT locint;

  atl_p_begin_line (ATS_CTXA (char*)"k", index);
  atl_p_out_car (ATS_CTXA ' ');
  locint = CTX(flagrt);
  CTX(flagrt) = 0;
  if (locint)
  {
    atl_p_out_quot (ATS_CTXA (char*)"1");
  }
  else
  {
    atl_p_out_quot (ATS_CTXA (char*)"0");
  }

  ATS_CYT_FLUSH(ATL_CYT_NESTED);

  return (locint);
}

void ATS_CLP atl_vartab
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"b", index);
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_varstruct
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
#endif
{
  atl_p_begin_line (ATS_CTXA (char*)"d", index);
  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_varplage
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_INT idxmin,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, index, idxmin, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_INT idxmin;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_begin_line (ATS_CTXA (char*)"l", index);
  if (idxmin == -1)
  {
    atl_p_out_car (ATS_CTXA ' ');
    ATL_M_out_empty_field;
    ATL_M_out_sep;
    ATL_M_out_empty_field;
  }
  else
  {
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA idxmin, str, 10));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_int_image (ATS_CTXA idxmax, str, 10));
  }

  ATS_CYT_FLUSH(ATL_CYT_NESTED);
}

void ATS_CLP atl_varreset
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  CTX(lastcmp) = 2;
  CTX(varbyte)[0] = '\0';
}

void ATS_CLP atl_setflagrt
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  CTX(flagrt) = 1;
}

ATL_T_INT ATS_CLP atl_cmp
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT locint;

  locint = CTX(flagrt);
  CTX(flagrt) = 0;
  CTX(lastcmp) = 2;
  CTX(varbyte)[0] = '\0';
  return (!locint);
}

/*******************************************************************************
* atl_dmp_*_c                                                                  *
*******************************************************************************/

void ATS_CLP atl_dmp_range_byte_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_UCAR v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_UCAR v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_UCAR v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_UCAR v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    atl_byte_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    atl_byte_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_p_dmp_elem (ATS_CTXA idx, str_range, curidx, idxmax);

}

void ATS_CLP atl_dmp_range_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_INT v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_INT v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_INT v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_INT v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    atl_int_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    atl_int_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_p_dmp_elem (ATS_CTXA idx, str_range, curidx, idxmax);

}

void ATS_CLP atl_dmp_range_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_LONG v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_LONG v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_LONG v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_LONG v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    atl_long_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    atl_long_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_p_dmp_elem (ATS_CTXA idx, str_range, curidx, idxmax);
}

void ATS_CLP atl_dmp_range_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_REAL v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_REAL v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_REAL v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_REAL v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    atl_float_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    atl_float_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_p_dmp_elem (ATS_CTXA idx, str_range, curidx, idxmax);
}

void ATS_CLP atl_dmp_range_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_DOUBLE v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_DOUBLE v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_DOUBLE v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_DOUBLE v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
   {
    atl_double_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
   }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
   {
    atl_double_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
   }
  atl_strcat(str_range,(char*)"]");

  atl_p_dmp_elem (ATS_CTXA idx, str_range, curidx, idxmax);
}


/*******************************************************************************
* atl_cmp_*_c                                                                  *
*******************************************************************************/

void ATS_CLP atl_cmp_range_byte_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_UCAR v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_UCAR v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_UCAR v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_UCAR v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_byte_image (ATS_CTXA v_min, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_byte_image (ATS_CTXA v_max, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_byte_image (ATS_CTXA v_rec, str_rec, 10);

  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_range_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_INT v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_INT v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_INT v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_INT v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_int_image (ATS_CTXA v_min, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_int_image (ATS_CTXA v_max, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_int_image (ATS_CTXA v_rec, str_rec, 10);

  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_range_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_LONG v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_LONG v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_LONG v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_LONG v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_LONG v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_LONG v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_long_image (ATS_CTXA v_min, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_long_image (ATS_CTXA v_max, tmp, 10);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_long_image (ATS_CTXA v_rec, str_rec, 10);

  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_range_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_REAL v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_REAL v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_REAL v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_REAL v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_REAL v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_REAL v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_float_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_float_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_float_image (ATS_CTXA v_rec, str_rec);

  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_range_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_DOUBLE v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_DOUBLE v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_DOUBLE v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_DOUBLE v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_DOUBLE v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_DOUBLE v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_double_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_double_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_double_image (ATS_CTXA v_rec, str_rec);

  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

/*******************************************************************************
* atl_var_*_c                                                                  *
*******************************************************************************/

void ATS_CLP atl_var_pointer_c
#if !RTRT_JR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_VOID v_rec,
   ATL_CONST ATL_PT_VOID v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_VOID v_rec;
  ATL_CONST ATL_PT_VOID v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_pointer_image (ATS_CTXA v_rec, str_rec, 16),
  atl_pointer_image (ATS_CTXA v_expect, str_exp, 16),
  passed = (v_rec==v_expect);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_var_nonil_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_PT_VOID v_rec,
   ATL_CONST ATL_PT_VOID v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_PT_VOID v_rec;
  ATL_CONST ATL_PT_VOID v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    CTX(prev) = v_rec;
    CTX(idxmin) = curidx;
    CTX(lastcmp) = 2;
  }
  equal = (v_rec != NIL);

  if (
       ((v_rec != CTX(prev)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (CTX(lastcmp))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), curidx - 1, 0);
    atl_p_out_quot (ATS_CTXA (char*)"NONIL");
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA CTX(prev), str, 16));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
    CTX(prev) = v_rec;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (v_rec != NIL)
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    atl_p_out_quot (ATS_CTXA (char*)"NONIL");
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_pointer_image (ATS_CTXA v_rec, str, 16));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

void ATS_CLP atl_var_long_c
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT idx,
  ATL_CONST ATL_T_LONG v_rec,
  ATL_CONST ATL_T_LONG v_expect,
  ATL_CONST ATL_T_INT base,
  ATL_CONST ATL_T_LONG curidx,
  ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_LONG v_rec;
  ATL_CONST ATL_T_LONG v_expect;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_LONG curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_long_image (ATS_CTXA v_rec, str_rec, base),
  atl_long_image (ATS_CTXA v_expect, str_exp, base),
  passed = (v_rec==v_expect);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_var_int_c
#if !RTRT_KR
  (ATS_CTXP
  ATL_CONST ATL_T_INT idx,
  ATL_CONST ATL_T_INT v_rec,
  ATL_CONST ATL_T_INT v_expect,
  ATL_CONST ATL_T_INT base,
  ATL_CONST ATL_T_INT curidx,
  ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_rec;
  ATL_CONST ATL_T_INT v_expect;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_int_image (ATS_CTXA v_rec, str_rec, base),
  atl_int_image (ATS_CTXA v_expect, str_exp, base),
  passed = (v_rec==v_expect);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_var_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_REAL v_rec,
   ATL_CONST ATL_T_REAL v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_REAL v_rec;
  ATL_CONST ATL_T_REAL v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    CTX(varfrec) = v_rec;
    CTX(varfexpect) = v_expect;
    CTX(idxmin) = curidx;
    CTX(lastcmp) = 2;
  }
  equal = !atl_fltcmp (v_rec, v_expect);

  if (
       (atl_fltcmp (CTX(varfexpect), v_expect) ||
        atl_fltcmp (v_rec, CTX(varfrec)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (CTX(lastcmp))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), curidx - 1, 0);
    atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA (ATL_T_REAL) CTX(varfexpect), str));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA (ATL_T_REAL) CTX(varfrec), str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
    CTX(varfrec) = v_rec;
    CTX(varfexpect) = v_expect;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (!atl_fltcmp (v_rec, v_expect))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA (ATL_T_REAL) v_expect, str));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA (ATL_T_REAL) v_rec, str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

void ATS_CLP atl_var_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_DOUBLE v_rec,
   ATL_CONST ATL_T_DOUBLE v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_DOUBLE v_rec;
  ATL_CONST ATL_T_DOUBLE v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    CTX(vardrec) = v_rec;
    CTX(vardexpect) = v_expect;
    CTX(idxmin) = curidx;
    CTX(lastcmp) = 2;
  }
  equal = !atl_dblcmp (v_rec, v_expect);

  if (
       (atl_dblcmp (CTX(vardexpect), v_expect) ||
        atl_dblcmp (v_rec, CTX(vardrec)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (CTX(lastcmp))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), curidx - 1, 0);
    atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA CTX(vardexpect), str));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA CTX(vardrec), str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
    CTX(vardrec) = v_rec;
    CTX(vardexpect) = v_expect;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (!atl_dblcmp (v_rec, v_expect))
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA v_expect, str));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA v_rec, str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}


void ATS_CLP atl_var_range_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_INT v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_INT v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx ATL_SP
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_INT v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_INT v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_int_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_int_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_int_image (ATS_CTXA v_rec, str_rec, base);

  atl_p_var_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);

}

void ATS_CLP atl_var_range_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_LONG v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_LONG v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_LONG v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_LONG v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_LONG v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_LONG v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_long_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_long_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_long_image (ATS_CTXA v_rec, str_rec, base);

  atl_p_var_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}


void ATS_CLP atl_var_range_byte_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_CAR v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_CAR v_max,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_CAR v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_CAR v_max;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec >= v_min);
    atl_byte_image (ATS_CTXA v_min, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec <= v_max);
    atl_byte_image (ATS_CTXA v_max, tmp, base);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_byte_image (ATS_CTXA v_rec, str_rec, base);

  atl_p_var_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_var_range_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_REAL v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_REAL v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_REAL v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_REAL v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_REAL v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_REAL v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec > v_min);
    atl_float_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec < v_max);
    atl_float_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_float_image (ATS_CTXA v_rec, str_rec);

  atl_p_var_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_var_range_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_DOUBLE v_rec,
   ATL_CONST ATL_T_INT v_min_opt,
   ATL_CONST ATL_T_DOUBLE v_min,
   ATL_CONST ATL_T_INT v_max_opt,
   ATL_CONST ATL_T_DOUBLE v_max,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_min_opt, v_min, v_max_opt, v_max, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_DOUBLE v_rec;
  ATL_CONST ATL_T_INT v_min_opt;
  ATL_CONST ATL_T_DOUBLE v_min;
  ATL_CONST ATL_T_INT v_max_opt;
  ATL_CONST ATL_T_DOUBLE v_max;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_range[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR tmp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  passed =  1;

  atl_strcpy(str_range,(char*)"[");
  if (v_min_opt)
  {
    passed &= (v_rec > v_min);
    atl_double_image (ATS_CTXA v_min, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"..");
  if (v_max_opt)
  {
    passed &= (v_rec < v_max);
    atl_double_image (ATS_CTXA v_max, tmp);
    atl_strcat(str_range,tmp);
  }
  atl_strcat(str_range,(char*)"]");

  atl_double_image (ATS_CTXA v_rec, str_rec);

  atl_p_var_elem (ATS_CTXA idx, str_rec, str_range, curidx, idxmax, passed);
}

void ATS_CLP atl_var_bytetab_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_CAR v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_CAR v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR hex1,hex2;
  ATL_T_INT i;

  if (CTX(lastcmp) || (idxmax == -1))
  {
    CTX(idxmin) = curidx;
    CTX(varrec)[0] = '\0';
    CTX(varexpect)[0] = '\0';
    CTX(lastcmp) = 0;
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varrec)[i] = ' ';
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varexpect)[i] = ' ';
  }

  atl_hexa_image ((ATL_T_UCAR) v_rec, &hex1, &hex2 );
  CTX(varrec)[(curidx - CTX(idxmin)) * 2] = hex1;
  CTX(varrec)[(curidx - CTX(idxmin)) * 2 + 1] = hex2;

  if (atl_car_isprint (v_rec))
    CTX(varrec)[curidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = v_rec;
  else
    CTX(varrec)[curidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = '.';

  atl_hexa_image ((ATL_T_UCAR) v_expect, &hex1, &hex2 );
  CTX(varexpect)[(curidx - CTX(idxmin)) * 2] = hex1;
  CTX(varexpect)[(curidx - CTX(idxmin)) * 2 + 1] = hex2;

  if (atl_car_isprint (v_expect))
    CTX(varexpect)[curidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = v_expect;
  else
    CTX(varexpect)[curidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = '.';

  if ((curidx == idxmax) ||
      ((curidx - CTX(idxmin)) == (RTST_HEX_LG - 1)) ||
      (idxmax == -1))
  {
    CTX(varexpect)[curidx - CTX(idxmin) + 4 + RTST_HEX_LG * 2] = '\0';
    CTX(varrec)[curidx - CTX(idxmin) + 4 + RTST_HEX_LG * 2] = '\0';
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    if (atl_strcmp (CTX(varrec), CTX(varexpect)))
    {
      atl_p_out_quot (ATS_CTXA (char*)"1");
      CTX(flagrt) = 1;
    }
    else
    {
      atl_p_out_quot (ATS_CTXA (char*)"0");
    }
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), curidx, (idxmax == -1));
    atl_p_out_quot (ATS_CTXA CTX(varexpect));
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA CTX(varrec));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);

    CTX(varrec)[0] = '\0';
    CTX(varexpect)[0] = '\0';
    CTX(idxmin) = curidx + 1;
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varrec)[i] = ' ';
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varexpect)[i] = ' ';
  }
}

void ATS_CLP atl_var_byte_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_CAR v_expect,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_CAR v_expect;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_byte_image (ATS_CTXA v_rec, str_rec, base),
  atl_byte_image (ATS_CTXA v_expect, str_exp, base),
  passed = (v_rec==v_expect);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_var_char_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_CAR v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_CAR v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_att_car[2];
  ATL_T_CAR str_rec_car[2];
  ATL_T_INT passed;

  str_rec_car[0] = v_rec;
  str_rec_car[1] = '\0';
  str_att_car[0] = v_expect;
  str_att_car[1] = '\0';
  atl_car_image (ATS_CTXA str_rec_car, str_rec);
  atl_car_image (ATS_CTXA str_att_car, str_exp);
  passed = (v_rec==v_expect);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_var_string_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_CAR v_rec,
   ATL_CONST ATL_PT_CAR v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_CAR v_rec;
  ATL_CONST ATL_PT_CAR v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_BUFFER_SIZE];
  ATL_T_CAR str_exp[ATS_BUFFER_SIZE];
  ATL_T_INT passed;

  atl_string_image (ATS_CTXA v_rec, str_rec,ATS_BUFFER_SIZE);
  atl_string_image (ATS_CTXA v_expect, str_exp,ATS_BUFFER_SIZE);
  passed = !strcmp(str_rec,str_exp);
  atl_p_var_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed );
}

/*******************************************************************************
* atl_cmp_*_c                                                                  *
*******************************************************************************/

void ATS_CLP atl_cmp_pointer_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_VOID v_rec,
   ATL_CONST ATL_PT_VOID v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_VOID v_rec;
  ATL_CONST ATL_PT_VOID v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_att[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_pointer_image (ATS_CTXA v_rec, str_rec, 16);
  atl_pointer_image (ATS_CTXA v_expect, str_att, 16);
  passed = (v_rec==v_expect);
  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_att, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_nonil_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_PT_VOID v_rec,
   ATL_CONST ATL_PT_VOID v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_PT_VOID v_rec;
  ATL_CONST ATL_PT_VOID v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
    {
      CTX(prev) = v_rec;
      CTX(idxmin) = curidx;
      CTX(lastcmp) = 2;
    }
  equal = (v_rec != NIL);

  if (
       ((v_rec != CTX(prev)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    if (!CTX(lastcmp))
    {
      CTX(flagrt) = 1;
    }
    CTX(prev) = v_rec;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    if (v_rec == NIL)
    {
      CTX(flagrt) = 1;
    }
  }
}

void ATS_CLP atl_cmp_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_LONG v_rec,
   ATL_CONST ATL_T_LONG v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_LONG v_rec;
  ATL_CONST ATL_T_LONG v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_long_image (ATS_CTXA v_rec, str_rec, 10);
  atl_long_image (ATS_CTXA v_expect, str_exp, 10);
  passed = (v_rec==v_expect);
  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax,passed);
}

void ATS_CLP atl_cmp_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_rec,
   ATL_CONST ATL_T_INT v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_rec;
  ATL_CONST ATL_T_INT v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_exp[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_INT passed;

  atl_int_image (ATS_CTXA v_rec, str_rec, 10);
  atl_int_image (ATS_CTXA v_expect, str_exp, 10);
  passed = (v_rec==v_expect);
  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_exp, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_REAL v_rec,
   ATL_CONST ATL_T_REAL v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_REAL v_rec;
  ATL_CONST ATL_T_REAL v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    CTX(varfrec) = v_rec;
    CTX(varfexpect) = v_expect;
    CTX(idxmin) = curidx;
    CTX(lastcmp) = 2;
  }
  equal = !atl_fltcmp (v_rec, v_expect);

  if (
       (atl_fltcmp (CTX(varfexpect), v_expect) ||
        atl_fltcmp (v_rec, CTX(varfrec)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    if (!CTX(lastcmp))
    {
      CTX(flagrt) = 1;
    }
    CTX(varfrec) = v_rec;
    CTX(varfexpect) = v_expect;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    if (atl_fltcmp (v_rec, v_expect))
    {
      CTX(flagrt) = 1;
    }
  }
}

void ATS_CLP atl_cmp_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_DOUBLE v_rec,
   ATL_CONST ATL_T_DOUBLE v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_DOUBLE v_rec;
  ATL_CONST ATL_T_DOUBLE v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_INT equal;

  if ((CTX(lastcmp) == 2) || (idxmax == -1))
  {
    CTX(vardrec) = v_rec;
    CTX(vardexpect) = v_expect;
    CTX(idxmin) = curidx;
    CTX(lastcmp) = 2;
  }
  equal = !atl_dblcmp (v_rec, v_expect);

  if (
       (atl_dblcmp (CTX(vardexpect), v_expect) ||
        atl_dblcmp (v_rec, CTX(vardrec)))
       &&
       (CTX(lastcmp) != 2)
    )
  {
    if (!CTX(lastcmp))
    {
      CTX(flagrt) = 1;
    }
    CTX(vardrec) = v_rec;
    CTX(vardexpect) = v_expect;
    CTX(idxmin) = curidx;
  }
  CTX(lastcmp) = equal;
  if ((idxmax == -1) || (curidx == idxmax))
  {
    if (atl_dblcmp (v_rec, v_expect))
    {
      CTX(flagrt) = 1;
    }
  }
}

void ATS_CLP atl_cmp_char_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_CAR v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_CAR v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_att[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_att_car[2];
  ATL_T_CAR str_rec_car[2];
  ATL_T_INT passed;

  str_rec_car[0] = v_rec;
  str_rec_car[1] = '\0';
  str_att_car[0] = v_expect;
  str_att_car[1] = '\0';

  atl_car_image (ATS_CTXA str_rec_car, str_rec),
  atl_car_image (ATS_CTXA str_att_car, str_att),
  passed = (v_rec==v_expect);
  atl_p_cmp_elem (ATS_CTXA idx, str_rec, str_att, curidx, idxmax, passed);
}

void ATS_CLP atl_cmp_string_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_CAR v_rec,
   ATL_CONST ATL_PT_CAR v_expect,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, v_expect, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_CAR v_rec;
  ATL_CONST ATL_PT_CAR v_expect;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str1[ATS_BUFFER_SIZE];
  ATL_T_CAR str2[ATS_BUFFER_SIZE];
  ATL_T_INT passed;

  atl_string_image (ATS_CTXA v_rec, str1,ATS_BUFFER_SIZE);
  atl_string_image (ATS_CTXA v_expect, str2,ATS_BUFFER_SIZE);
  passed = !atl_strcmp (str1, str2);
  atl_p_cmp_elem (ATS_CTXA idx, str1, str2, curidx, idxmax, passed);
}

/*******************************************************************************
* Dump routines                                                                *
*******************************************************************************/

void ATS_CLP atl_dmp_pointer_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_VOID v_rec,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_VOID v_rec;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_pointer_image (ATS_CTXA v_rec, str_rec, 16),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_nonil_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_PT_VOID v_rec,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_PT_VOID v_rec;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_pointer_image (ATS_CTXA v_rec, str_rec, 16),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_int_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_INT v_rec,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_INT v_rec;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_int_image (ATS_CTXA v_rec, str_rec, base),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_long_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_LONG v_rec,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_LONG v_rec;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_long_image (ATS_CTXA v_rec, str_rec, base),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_float_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_REAL v_rec,
   ATL_CONST ATL_T_INT idxcur,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, index, v_rec, idxcur, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_REAL v_rec;
  ATL_CONST ATL_T_INT idxcur;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  if ((idxmax == -1) || (CTX(lastcmp) == 2))
  {
    CTX(varfrec) = v_rec;
    CTX(idxmin) = idxcur;
    CTX(lastcmp) = 0;
  }
  else
  {
    if (atl_fltcmp (v_rec, CTX(varfrec)))
    {
      atl_p_begin_line (ATS_CTXA (char*)"i", index);
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"0");
      ATL_M_out_sep;
      atl_p_range (ATS_CTXA CTX(idxmin), idxcur - 1, 0);
      ATL_M_out_empty_field;
      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA
		     atl_float_image (ATS_CTXA (ATL_T_REAL) CTX(varfrec), str));

      ATS_CYT_FLUSH(ATL_CYT_NESTED);

      CTX(varfrec) = v_rec;
      CTX(idxmin) = idxcur;
    }
  }
  if ((idxmax == -1) || (idxcur == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", index);
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"0");
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    ATL_M_out_empty_field;
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_float_image (ATS_CTXA (ATL_T_REAL) v_rec, str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

void ATS_CLP atl_dmp_double_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_DOUBLE v_rec,
   ATL_CONST ATL_T_INT idxcur,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, index, v_rec, idxcur, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_DOUBLE v_rec;
  ATL_CONST ATL_T_INT idxcur;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  if ((idxmax == -1) || (CTX(lastcmp) == 2))
  {
    CTX(vardrec) = v_rec;
    CTX(idxmin) = idxcur;
    CTX(lastcmp) = 0;
  }
  else
  {
    if (atl_dblcmp (v_rec, CTX(varfrec)))
    {
      atl_p_begin_line (ATS_CTXA (char*)"i", index);
      atl_p_out_car (ATS_CTXA ' ');
      atl_p_out_quot (ATS_CTXA (char*)"0");
      ATL_M_out_sep;
      atl_p_range (ATS_CTXA CTX(idxmin), idxcur - 1, 0);
      ATL_M_out_empty_field;
      ATL_M_out_sep;
      atl_p_out_quot (ATS_CTXA
		   atl_double_image (ATS_CTXA CTX(vardrec), str));

      ATS_CYT_FLUSH(ATL_CYT_NESTED);

      CTX(vardrec) = v_rec;
      CTX(idxmin) = idxcur;
    }
  }
  if ((idxmax == -1) || (idxcur == idxmax))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", index);
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"0");
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), idxmax, (idxmax == -1));
    ATL_M_out_empty_field;
    ATL_M_out_sep;
    atl_p_out_quot (ATS_CTXA atl_double_image (ATS_CTXA v_rec, str));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);
  }
}

void ATS_CLP atl_dmp_byte_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_INT base,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, base, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_INT base;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_byte_image (ATS_CTXA v_rec, str, base),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_bytetab_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR hex1,hex2;
  ATL_T_INT i,locidx;

  if (idxmax == -1) locidx=0; else locidx=curidx;

  if (CTX(varbyte)[0] == '\0')
  {
    CTX(idxmin) = locidx;
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varbyte)[i] = ' ';
  }

  atl_hexa_image ((ATL_T_UCAR) v_rec, &hex1, &hex2 );
  CTX(varbyte)[(locidx - CTX(idxmin)) * 2] = hex1;
  CTX(varbyte)[(locidx - CTX(idxmin)) * 2 + 1] = hex2;

  if (atl_car_isprint (v_rec))
    CTX(varbyte)[locidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = v_rec;
  else
    CTX(varbyte)[locidx - CTX(idxmin) + RTST_HEX_LG * 2 + 3] = '.';

  if ((locidx == idxmax) || ((locidx - CTX(idxmin)) == (RTST_HEX_LG - 1)) || (idxmax == -1))
  {
    atl_p_begin_line (ATS_CTXA (char*)"i", idx);
    atl_p_out_car (ATS_CTXA ' ');
    atl_p_out_quot (ATS_CTXA (char*)"0");
    ATL_M_out_sep;
    atl_p_range (ATS_CTXA CTX(idxmin), locidx, (idxmax == -1));
    ATL_M_out_empty_field;
    ATL_M_out_sep;
    CTX(varbyte)[locidx - CTX(idxmin) + 4 + RTST_HEX_LG * 2] = '\0';
    atl_p_out_quot (ATS_CTXA CTX(varbyte));

    ATS_CYT_FLUSH(ATL_CYT_NESTED);

    CTX(varbyte)[0] = '\0';
    CTX(idxmin) = locidx + 1;
    for (i = 1; i < (RTST_HEX_LG * 3 + 3); i++)
      CTX(varbyte)[i] = ' ';
  }
}

void ATS_CLP atl_dmp_char_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_T_CAR v_rec,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_T_CAR v_rec;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str_rec[ATS_IMAGE_SIZE];
  ATL_T_CAR str_rec_car[2];

  str_rec_car[0] = v_rec;
  str_rec_car[1] = '\0';

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_car_image (ATS_CTXA str_rec_car, str_rec),
                  curidx,
                  idxmax);
}

void ATS_CLP atl_dmp_string_c
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT idx,
   ATL_CONST ATL_PT_CAR v_rec,
   ATL_CONST ATL_T_INT curidx,
   ATL_CONST ATL_T_INT idxmax)
#else
  (ctx, idx, v_rec, curidx, idxmax)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT idx;
  ATL_CONST ATL_PT_CAR v_rec;
  ATL_CONST ATL_T_INT curidx;
  ATL_CONST ATL_T_INT idxmax;
#endif
{
  ATL_T_CAR str1[ATS_BUFFER_SIZE];

  atl_p_dmp_elem (ATS_CTXA idx,
                  atl_string_image (ATS_CTXA v_rec, str1,ATS_BUFFER_SIZE),
                  curidx,
                  idxmax);
}

/*******************************************************************************
* Miscellaneous routines                                                       *
*******************************************************************************/

void ATS_CLP atl_end
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  priv_close (CTX(fdrio));
}

#if RTRT_CPLUSPLUS_LANGUAGE
void ATS_CLP atl_v1_3
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
}
#endif /* RTRT_CPLUSPLUS_LANGUAGE */

ATL_T_ULONG ATS_CLP atl_time
#if !RTRT_KR
  (RTRT_CLOCK_TYPE *timer)
#else
  (timer)
  RTRT_CLOCK_TYPE *timer;
#endif
{
  RTRT_TIME_TYPE time;

  time = priv_elapsed_time (timer,(RTRT_CLOCK_TYPE *) 0,0);
  return ((ATL_T_ULONG) (time / ATL_WTIME_TO_TIME));
}

void ATS_CLP atl_pause
#if !RTRT_KR
  (ATL_CONST ATL_T_ULONG pause)
#else
  (pause)
  ATL_CONST ATL_T_ULONG pause;
#endif
{
  priv_sleep (pause * ATL_PAUSE_TIME_TO_TIME);
}

void ATS_CLP atl_print
#if !RTRT_KR
  (ATS_CTXP
   ATL_CONST ATL_T_INT index,
   ATL_CONST ATL_T_ULONG value)
#else
  (ctx, index, value)
  ATL_PT_RUNCTX ctx;
  ATL_CONST ATL_T_INT index;
  ATL_CONST ATL_T_ULONG value;
#endif
{
  ATL_T_CAR tmp[ATS_BUFFER_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"m", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA value, tmp, 10));

  ATS_CYT_FLUSH(ATL_CYT_SPECIAL_MONOLINE);
}

void ATS_CLP atl_bproc
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  atl_p_begin_line (ATS_CTXA (char*)"p", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"b");
  ATL_M_out_sep;
  ++CTX(level);
  CTX(duration)[CTX(level)] = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),
						CTX(flagprobe));
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA CTX(duration)[CTX(level)], str, 10));

  ATS_CYT_FLUSH(ATL_CYT_BEGIN_BLOCK);
}

void ATS_CLP atl_aproc
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_INT index)
#else
  (ctx, index)
  ATL_PT_RUNCTX ctx;
  ATL_T_INT index;
#endif
{
  ATL_T_CAR str[ATS_IMAGE_SIZE];
  RTRT_TIME_TYPE loctime;

  TCTX(currentcodepriority) = K_TRACECTX_TIME_PRIORITY;

  loctime = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	    - CTX(duration)[CTX(level)];
  atl_p_begin_line (ATS_CTXA (char*)"p", index);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA (char*)"a");
  ATL_M_out_sep;
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA loctime, str, 10));

  /** a PROC has no return status => forcing it to "OK" **/
  ATS_CYT_FLUSH(ATL_CYT_END_OK_BLOCK);
  CTX(level)--;

  /** upgrading status concerning the former FT instruction. Now, it's sure **/
  /** the (eventual) forthcoming 'fb' will occur in an other block          **/
  if (TCTX(CYTft_status) == 1)
    TCTX(CYTft_status) = 2;
}

void ATS_CLP atl_set_trace_functions
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  switch(TCTX(tracepriority))
  {
    case 0: /** no -TRACE option activated **/
      TCTX(generic_flush_line) = ATS_CLP_ADR(atl_p_flush_line);
      break;

    case K_TRACECTX_CIRCULAR_PRIORITY: /** -TRACE=CIRCULAR option activated **/
      TCTX(generic_flush_line) = ATS_CLP_ADR(atl_p_CYTtoplvl_flushbufoutline);
      break;

    default: /** -TRACE={ERROR,SCN,TIME} option activated **/
      TCTX(generic_flush_line) = ATS_CLP_ADR(atl_p_circular_buffer_flush_bufout_line);
  }
}

/*******************************************************************************
* Buffer routines                                                              *
*******************************************************************************/

void ATS_CLP atl_flush_circular_buffer
#if !RTRT_KR
  (ATS_CTXP
   ATL_T_UCAR priority)
#else
  (ctx, priority)
  ATL_PT_RUNCTX ctx;
  ATL_T_UCAR priority;
#endif
{
  ATL_T_UCAR saved_priority = TCTX(tracepriority);

  if (TCTX(tracepriority) == 5 && priority == K_TRACECTX_MAX_PRIORITY)
  {
    atl_p_flushstack (ATS_CTXA1);
  }

  if (TCTX(tracepriority) == K_TRACECTX_CIRCULAR_PRIORITY &&
      TCTX(CYTbufferoverpassed) == 1)
  {
    atl_p_CYToverpass_warning(ATS_CTXA1);
  }

  TCTX(tracepriority) = priority;

  atl_p_circular_buffer_flush_line (ATS_CTXA
				    TCTX(linetoerase),
				    TCTX(lastline));
  atl_p_circular_buffer_flush_line (ATS_CTXA
				    TCTX(buffertrace),
				    TCTX(currentline));

  TCTX(tracepriority) = saved_priority;

  TCTX(buffersizeleft) = TCTX(buffersize);
  TCTX(currentline) = TCTX(buffertrace);
  TCTX(linetoerase) = TCTX(buffertrace);
  TCTX(lastline) = TCTX(buffertrace);

  /** forcing disk flush in -TRACE=CIRCULAR mode **/
  if (TCTX(tracepriority) == K_TRACECTX_CIRCULAR_PRIORITY)
  {
    priv_flush(CTX(fdrio));
  }
}

ATS_STATIC void ATS_CLP atl_p_CYTflush_bufout_line
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_LONG line_size;

  if (CTX(rio_onoff))
  {
    if (CTX(bufindex) > ATL_K_MAX_RIOLINE_SIZE)
    {
      /** rio line size is too big to be encoded. Exiting  **/
      atl_bufindex_error(ATS_CTXA1);
    }

    CTX(bufindex) += 4;
    CTX(bufindex) >>= 2;
    line_size = (CTX(bufindex) << 2) + 2;

    if (line_size < TCTX(buffersizeleft))
    {
      /** buffer can contain the whole line => straight copy **/
      *TCTX(currentline)++ = TCTX(currentcodepriority);
      *TCTX(currentline)++ = CTX(bufindex);
      atl_strcpy (TCTX(currentline), CTX(bufout));
      TCTX(currentline) += (line_size - 2);
      TCTX(buffersizeleft) -= line_size;
    }
    else
    {
      /** not enough space => freeing some place / updating 'erase' ptr **/
      ATL_T_INT size_of_freed_lines = 0;
      ATL_PT_CAR old_linetoerase = TCTX(linetoerase);

      TCTX(CYTbufferoverpassed) = 1;

      while ((TCTX(buffersizeleft) + size_of_freed_lines < line_size) &&
	     (TCTX(linetoerase) < ATL_END_BUFFERTRACE))
      {
	if (TCTX(linetoerase) < TCTX(lastline))
	{
	  size_of_freed_lines += ATL_LINE_SIZE(TCTX(linetoerase));
	  TCTX(linetoerase) += ATL_LINE_SIZE(TCTX(linetoerase));
	}
	else
	{ /** special case "erase==lastline" (strict > impossible by construction) **/
	  size_of_freed_lines += line_size;
	  TCTX(linetoerase) += line_size;
	  TCTX(lastline) = TCTX(linetoerase);
	}
      }
      TCTX(buffersizeleft) += size_of_freed_lines;


      if ( /** Very beginning configuration **/
	  (TCTX(currentline) > old_linetoerase)
	|| /** Nominal configuration **/
	  ((TCTX(currentline) <= old_linetoerase) &&
	   (TCTX(linetoerase) >= ATL_END_BUFFERTRACE)))
      {
	/** let's abandon this not large enough part of the buffer **/
	TCTX(lastline) = TCTX(currentline);
	TCTX(buffersizeleft) -= size_of_freed_lines;

	/** candidate line will appear at the beginning of the buffer **/
	TCTX(currentline) = TCTX(buffertrace);

	/** restarting the freeing process from the start of the buffer **/
	TCTX(CYToverpassindex)++;
	TCTX(linetoerase) = TCTX(buffertrace);
	size_of_freed_lines = 0;

	while ( (size_of_freed_lines < line_size) &&
		(TCTX(linetoerase) < ATL_END_BUFFERTRACE) )
	{
	  size_of_freed_lines += ATL_LINE_SIZE(TCTX(linetoerase));
	  TCTX(linetoerase) += ATL_LINE_SIZE(TCTX(linetoerase));
	}

	/** updating 'buffersizeleft' value **/
	TCTX(buffersizeleft) = TCTX(linetoerase) - TCTX(buffertrace);

	if (TCTX(linetoerase) >= ATL_END_BUFFERTRACE)
	{
	  /** means that one rio line overpass the buffer capacity ! **/
	  /** (circular buffer size must be increased)               **/
	  atl_bufindex_error(ATS_CTXA1);
	  priv_exit (-8);;
	}
      }

      *TCTX(currentline)++ = TCTX(currentcodepriority);
      *TCTX(currentline)++ = CTX(bufindex);
      atl_strcpy (TCTX(currentline), CTX(bufout));
      TCTX(currentline) += (line_size - 2);
      TCTX(buffersizeleft) -= line_size;
    }
  }
  TCTX(currentcodepriority) = K_TRACECTX_ERROR_PRIORITY;

  CTX(bufout)[0] = '\0';
  CTX(bufindex) = 0;
}


ATS_STATIC void ATS_CLP atl_p_circular_buffer_flush_bufout_line
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_LONG line_size;

  if (CTX(rio_onoff))
  {
    CTX(bufindex) += 4;
    CTX(bufindex) >>= 2;
    line_size = (CTX(bufindex) << 2) + 2;

    if (line_size > TCTX(buffersizeleft))
    {
      atl_p_circular_buffer_eval_flush_line (
	ATS_CTXA TCTX(linetoerase), TCTX(lastline));

      TCTX(lastline) = TCTX(currentline);
      TCTX(currentline) = TCTX(buffertrace);
      TCTX(linetoerase) = TCTX(buffertrace);
      TCTX(buffersizeleft) = TCTX(buffersize);
    }

    if (line_size > (TCTX(linetoerase) - TCTX(currentline)))
    {
      TCTX(linetoerase) = atl_p_circular_buffer_eval_flush_line (ATS_CTXA
					    TCTX(linetoerase),
			     (TCTX(currentline) + line_size));
    }

    *TCTX(currentline)++ = TCTX(currentcodepriority);
    *TCTX(currentline)++ = CTX(bufindex);
    atl_strcpy (TCTX(currentline), CTX(bufout));
    TCTX(currentline) += (line_size - 2);
    TCTX(buffersizeleft) -= line_size;
  }
  TCTX(currentcodepriority) = K_TRACECTX_ERROR_PRIORITY;

  CTX(bufout)[0] = '\0';
  CTX(bufindex) = 0;
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_p_circular_buffer_eval_flush_line
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR begin,
   ATL_PT_CAR end)
#else
  (ctx, begin, end)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR begin;
  ATL_PT_CAR end;
#endif
{
  if (TCTX(lastline) <= begin)
  {
    return (TCTX(buffertrace) + TCTX(buffersize));
  }

  if (end > TCTX(lastline))
  {
    atl_p_circular_buffer_flush_line (ATS_CTXA begin, TCTX(lastline));
    return (TCTX(buffertrace) + TCTX(buffersize));
  }

  return (atl_p_circular_buffer_flush_line (ATS_CTXA begin, end));
}

ATS_STATIC ATL_PT_CAR ATS_CLP atl_p_circular_buffer_flush_line
#if !RTRT_KR
  (ATS_CTXP
   ATL_PT_CAR begin,
   ATL_PT_CAR end)
#else
  (ctx, begin, end)
  ATL_PT_RUNCTX ctx;
  ATL_PT_CAR begin;
  ATL_PT_CAR end;
#endif
{
  ATL_T_UCAR line_priority;
  ATL_T_INT line_size;

  while (begin < end)
  {
    line_priority = *begin;
    *begin++ = K_TRACECTX_MAX_PRIORITY;
    line_size = *begin++;
    line_size <<= 2;

    if (line_priority <= TCTX(tracepriority))
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA begin);
#else
      ats_writeln (CTX(fdrio), begin);
#endif
    }
    begin += line_size;
  }

  TCTX(CYTbufferoverpassed) = 0;

  return (begin);
}

ATS_STATIC void ATS_CLP atl_p_CYTflushclosingstack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT i, before_last;
  ATL_T_INT cond;

  if ( (TCTX(CYTlasttracelevel_forclosure)[0] == -1) &&
       (TCTX(CYTlasttracelevel_forclosure)[1] != -1) )
  {
    /** case when 1) one 'ft' has occured 2) tester ends before another 'ft' **/
    before_last = 1;
  }
  else
  {
    /** nominal case **/
    before_last = 0;
  }

  cond = ( (TCTX(CYTstoredinit)[before_last][0] != '\0') ||
	   (TCTX(CYTstoredterm)[before_last][0] != '\0') ||
	   (TCTX(CYTstoredexcept)[before_last][0] != '\0')
	   );

  if (TCTX(CYTstoredinit)[before_last][0] != '\0')
  {
#if ATS_RT_BUFFER
    atl_p_toplvl_decode(ATS_CTXA TCTX(CYTstoredinit)[before_last]);
#else
    ats_writeln (CTX(fdrio), TCTX(CYTstoredinit)[before_last]);
#endif
    TCTX(CYTstoredinit)[before_last][0] = '\0';
  }

  /** don't close a scn when closing an associated block **/
  if (! cond)
  {
    /* for(i=1;i<=TCTX(CYTlasttracelevel_forclosure)[0];i++) */
    i = TCTX(CYTlasttracelevel_forclosure)[before_last];
    {
      if (TCTX(CYTbscnstackclosure)[before_last][i][0] != '\0')
      {
#if ATS_RT_BUFFER
	atl_p_toplvl_decode(ATS_CTXA TCTX(CYTbscnstackclosure)[before_last][i]);
#else
	ats_writeln (CTX(fdrio), TCTX(CYTbscnstackclosure)[before_last][i]);
#endif
	TCTX(CYTbscnstackclosure)[before_last][i][0] = '\0';
      }
    }
  }
  if (TCTX(CYTstoredterm)[before_last][0] != '\0')
  {
#if ATS_RT_BUFFER
    atl_p_toplvl_decode(ATS_CTXA TCTX(CYTstoredterm)[before_last]);
#else
    ats_writeln (CTX(fdrio), TCTX(CYTstoredterm)[before_last]);
#endif
    TCTX(CYTstoredterm)[before_last][0] = '\0';
  }
  if (TCTX(CYTstoredexcept)[before_last][0] != '\0')
  {
#if ATS_RT_BUFFER
    atl_p_toplvl_decode(ATS_CTXA TCTX(CYTstoredexcept)[before_last]);
#else
    ats_writeln (CTX(fdrio), TCTX(CYTstoredexcept)[before_last]);
#endif
    TCTX(CYTstoredexcept)[before_last][0] = '\0';
  }

  /** Now, let's translate values associated to last 'ft' as values associated **/
  /** to the before last 'ft'                                                  **/
  if (before_last==0)
  {
    atl_strcpy(TCTX(CYTstoredinit)[0], TCTX(CYTstoredinit)[1]);

    atl_strcpy(TCTX(CYTstoredterm)[0], TCTX(CYTstoredterm)[1]);

    atl_strcpy(TCTX(CYTstoredexcept)[0], TCTX(CYTstoredexcept)[1]);

    for(i=1; i<= TCTX(CYTlasttracelevel_forclosure)[1]; i++)
    {
      atl_strcpy(TCTX(CYTbscnstackclosure)[0][i],
		 TCTX(CYTbscnstackclosure)[1][i]);
    }

    TCTX(CYTlasttracelevel_forclosure)[0] =
					  TCTX(CYTlasttracelevel_forclosure)[1];
    TCTX(CYTlasttracelevel_forclosure)[1] = -1;
  }
}

ATS_STATIC void ATS_CLP atl_p_CYTbegintoendblock_conversion
#if !RTRT_KR
  (ATL_PT_CAR dest,
   ATL_PT_CAR source,
   ATL_PT_CAR added_params)
#else
  (dest, source, added_params)
  ATL_PT_CAR dest;
  ATL_PT_CAR source;
  ATL_PT_CAR added_params;
#endif
{
  ATL_T_INT i,j;

  i = 0;
  j = 1;
  /** copy token ('s' or 'z' or 'p') */
  dest[0] = source[0];
  i++;
  while ((source[j]!='\"') && (source[j]!='\0'))
    j++;
  /** change begin index for end index **/
  j++;
  while ((source[j]!='\"') && (source[j]!='\0'))
  {
    dest[i++] = source[j];
    j++;
  }
  /** concatenate the "to-be-added" params **/
  atl_strcpy(&(dest[i]),added_params);
}


ATS_STATIC void ATS_CLP atl_p_CYTcopystack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT i;

  if (TCTX(binitstack)[CTX(tracelevel)][0] != '\0')
  {
    atl_p_CYTbegintoendblock_conversion(TCTX(CYTstoredinit)[1],
					TCTX(binitstack)[CTX(tracelevel)],
					(char*)" \"0\", ?");
  }

  for(i = CTX(tracelevel); i>=0; i--)
  {
    if (TCTX(bscnstack)[i][0] != '\0')
    {
      atl_p_CYTbegintoendblock_conversion(TCTX(CYTbscnstackclosure)[1][i],
					  TCTX(bscnstack)[i],
					  (char*)" \"0\", \"0\", \"0\", ?");
    }
  }

  if (TCTX(btermstack)[CTX(tracelevel)][0] != '\0')
  {
    atl_p_CYTbegintoendblock_conversion(TCTX(CYTstoredterm)[1],
					TCTX(btermstack)[CTX(tracelevel)],
					(char*)" \"0\", ?");
  }

  if (TCTX(bexceptstack)[CTX(tracelevel)][0] != '\0')
  {
    atl_p_CYTbegintoendblock_conversion(TCTX(CYTstoredexcept)[1],
					TCTX(bexceptstack)[CTX(tracelevel)],
					(char*)" \"0\", ?");
  }
}

ATS_STATIC void ATS_CLP atl_p_flushstack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT i;

  for (i = 0; i <= CTX(tracelevel); i++)
  {
    if (TCTX(binitstack)[i][0] != '\0')
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA TCTX(binitstack)[i]);
#else
      ats_writeln (CTX(fdrio), TCTX(binitstack)[i]);
#endif
      TCTX(binitstack)[i][0] = '\0';
    }
    if (TCTX(bscnstack)[i][0] != '\0')
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA TCTX(bscnstack)[i]);
#else
      ats_writeln (CTX(fdrio), TCTX(bscnstack)[i]);
#endif
      TCTX(bscnstack)[i][0] = '\0';
    }
    if (TCTX(btermstack)[i][0] != '\0')
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA TCTX(btermstack)[i]);
#else
      ats_writeln (CTX(fdrio), TCTX(btermstack)[i]);
#endif
      TCTX(btermstack)[i][0] = '\0';
    }
    if (TCTX(bexceptstack)[i][0] != '\0')
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA TCTX(bexceptstack)[i]);
#else
      ats_writeln (CTX(fdrio), TCTX(bexceptstack)[i]);
#endif
      TCTX(bexceptstack)[i][0] = '\0';
    }
    if (TCTX(bprocstack)[i][0] != '\0')
    {
#if ATS_RT_BUFFER
      atl_p_toplvl_decode(ATS_CTXA TCTX(bprocstack)[i]);
#else
      ats_writeln (CTX(fdrio), TCTX(bprocstack)[i]);
#endif
      TCTX(bprocstack)[i][0] = '\0';
    }
  }
}

ATS_STATIC void ATS_CLP atl_p_CYTupdatestack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  TCTX(CYTctx_stackindex) = 0;
}


ATS_STATIC void ATS_CLP atl_p_CYTpushstack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  STCTX(lastline,TCTX(CYTctx_stackindex)) = TCTX(lastline);
  STCTX(currentline,TCTX(CYTctx_stackindex)) = TCTX(currentline);
  STCTX(linetoerase,TCTX(CYTctx_stackindex)) = TCTX(linetoerase);
  STCTX(buffersizeleft,TCTX(CYTctx_stackindex)) = TCTX(buffersizeleft);
  STCTX(overpassindex,TCTX(CYTctx_stackindex)) = TCTX(CYToverpassindex);
  TCTX(CYTctx_stackindex)++;
}


ATS_STATIC void ATS_CLP atl_p_CYTupdate_circularbuffer_sizeleft
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  if (TCTX(linetoerase) > TCTX(currentline))
  {
    TCTX(buffersizeleft) = (ATL_T_INT)(TCTX(linetoerase) - TCTX(currentline));
  }
  else
  {
    TCTX(buffersizeleft) = ATL_END_BUFFERTRACE - TCTX(currentline);
  }
}

ATS_STATIC void ATS_CLP atl_p_CYTpopstack
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_LONG diffoverpass;

  if (TCTX(CYTctx_stackindex) > 0)
  {
    diffoverpass = TCTX(CYToverpassindex) - STCTX(overpassindex,TCTX(CYTctx_stackindex));

    /** controling overpass indexes **/
    if (diffoverpass == 0 ||
	(diffoverpass == 1 &&
	 TCTX(currentline) > STCTX(currentline,TCTX(CYTctx_stackindex)-1)))
    {
      /** optimal valid poping **/
      TCTX(CYTctx_stackindex)--;
      TCTX(lastline) = STCTX(lastline,TCTX(CYTctx_stackindex));
      TCTX(currentline) = STCTX(currentline,TCTX(CYTctx_stackindex));
      /** non changing value : atl_ctxlinetoerase **/
      atl_p_CYTupdate_circularbuffer_sizeleft (ATS_CTXA1);

      TCTX(CYToverpassindex) = STCTX(overpassindex,TCTX(CYTctx_stackindex));
    }
    else
    {
      if (diffoverpass == 1 &&
	  TCTX(currentline) < STCTX(currentline,TCTX(CYTctx_stackindex)-1) &&
	  TCTX(linetoerase) < STCTX(currentline,TCTX(CYTctx_stackindex)-1))
      {
	/** former (linetoerase-->last) is completely lost but a part of the
	    former (buffertrace-->current) area is still popable **/
	ATL_T_INT popsize;

	popsize = (ATL_T_INT) (STCTX(currentline,TCTX(CYTctx_stackindex)-1)
							   - TCTX(linetoerase));
	priv_memcpy(TCTX(buffertrace), TCTX(linetoerase),
	           popsize*sizeof(ATL_T_CAR));

	TCTX(currentline) = TCTX(buffertrace) + popsize;
	TCTX(linetoerase) = TCTX(buffertrace);
	TCTX(lastline) = TCTX(buffertrace);
	atl_p_CYTupdate_circularbuffer_sizeleft (ATS_CTXA1);

	TCTX(CYToverpassindex) = 0;
      }
      else
	{
	  /** non valid popping (buffer overpass makes it unrelevant) **/
	  TCTX(CYToverpassindex) = 0;

	  /** restarting buffer from scratch **/
	  TCTX(buffersizeleft) = TCTX(buffersize);
	  TCTX(currentline) = TCTX(buffertrace);
	  TCTX(linetoerase) = TCTX(buffertrace);
	  TCTX(lastline) = TCTX(buffertrace);
	}
    }
  }
  else
    {
      priv_exit (-9);;
    }
}


ATS_STATIC void ATS_CLP atl_p_CYTtoplvl_flushbufoutline
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_INT instrType = TCTX(CYTstatus);

  if (! CTX(rio_onoff))
  {
    /** current instruction is included in a PROCSEND/CALLBACK **/
    /** => must not reach the circular buffer **/
    CTX(bufout)[0] = '\0';
    CTX(bufindex) = 0;
  }
  else
  {
    switch(instrType)
    {
      case ATL_CYT_MONOLINE:
	{
	  if (CTX(trace_onoff))
	  {
	    atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  }
	  else
	  {
	    /** instruction is filtered **/
	    CTX(bufout)[0] = '\0';
	    CTX(bufindex) = 0;
	  }
	  break;
	}

      case ATL_CYT_SPECIAL_MONOLINE:
	{
	  /** print/trace on/trace off/flush_trace **/
	  atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  break;
	}

      case ATL_CYT_BEGIN_BLOCK:
      case ATL_CYT_END_OK_BLOCK:
      case ATL_CYT_END_KO_BLOCK:
      case ATL_CYT_BEGIN_VIRTUAL_BLOCK:
      case ATL_CYT_END_VIRTUAL_BLOCK:
	{
	  atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  break;
	}

      case ATL_CYT_BEGIN_MULTILINE:
	{
	  atl_p_CYTpushstack(ATS_CTXA1);
	  atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  break;
	}

      case ATL_CYT_NESTED:
	{
	  atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  break;
	}

      case ATL_CYT_END_OK_MULTILINE:
	{
	  if (! CTX(trace_onoff))
	    {
	      /** no error and TRACE_OFF mode => poping **/
	      CTX(bufout)[0] = '\0';
	      CTX(bufindex) = 0;
	      atl_p_CYTpopstack (ATS_CTXA1);
	    }
	  else
	    {
	      /** no error and TRACE ON => line must be traced **/
	      atl_p_CYTflush_bufout_line (ATS_CTXA1);
	      /** let's consider this portion of the buffer as definitive **/
	      atl_p_CYTupdatestack (ATS_CTXA1);
	    }
	  break;
	}

      case ATL_CYT_END_KO_MULTILINE:
	{
	  /** error => flush in all cases **/
	  atl_p_CYTflush_bufout_line (ATS_CTXA1);
	  /** let's consider this portion of the buffer as definitive **/
	  atl_p_CYTupdatestack (ATS_CTXA1);
	  break;
	}
      default:
	 priv_exit (-10);;
    }
  }
}


ATS_STATIC void ATS_CLP atl_p_CYToverpass_warning
#if !RTRT_KR
  (ATS_CTXP1)
#else
  (ctx)
  ATL_PT_RUNCTX ctx;
#endif
{
  ATL_T_ULONG dwarning;
  ATL_T_CAR str[ATS_IMAGE_SIZE];

  dwarning = priv_elapsed_time(&CTX(stdate),&CTX(usrdate),CTX(flagprobe))
	     - CTX(duration)[CTX(level)];

  TCTX(CYTstatus) = ATL_CYT_SPECIAL_MONOLINE;

  /** If a FT has occured inside an older block than **/
  /** previous one, then close it artificially       **/
  if ( (TCTX(CYTft_status) > 0) && (TCTX(CYTftoccured) == 2) /* ==2 */ )
  {
    atl_p_CYTflushclosingstack (ATS_CTXA1);
    TCTX(CYTft_status) = 0;
  }

  /** line 6 of TDC file = virtual warning (cf. tdcgen.c) **/
  atl_p_begin_line (ATS_CTXA (char*)"fb", 6);
  atl_p_out_car (ATS_CTXA ' ');
  atl_p_out_quot (ATS_CTXA atl_long_image (ATS_CTXA dwarning, str, 10));
  atl_p_flush_line (ATS_CTXA1);


  if (TCTX(CYTftoccured) < 2)
  {
    atl_p_CYTflushclosingstack (ATS_CTXA1);
    TCTX(CYTft_status) = 0;
  }

  TCTX(CYTfboccured) = 1;
}
